Package: pillar
Title: Coloured Formatting for Columns
Version: 1.11.1
Authors@R: 
    c(person(given = "Kirill",
             family = "M\u00fcller",
             role = c("aut", "cre"),
             email = "kirill@cynkra.com",
             comment = c(ORCID = "0000-0002-1416-3412")),
      person(given = "Hadley",
             family = "Wickham",
             role = "aut"),
      person(given = "RStudio",
             role = "cph"))
Description: Provides 'pillar' and 'colonnade' generics designed
    for formatting columns of data using the full range of colours
    provided by modern terminals.
License: MIT + file LICENSE
URL: https://pillar.r-lib.org/, https://github.com/r-lib/pillar
BugReports: https://github.com/r-lib/pillar/issues
Imports: cli (>= 2.3.0), glue, lifecycle, rlang (>= 1.0.2), utf8 (>=
        1.1.0), utils, vctrs (>= 0.5.0)
Suggests: bit64, DBI, debugme, DiagrammeR, dplyr, formattable, ggplot2,
        knitr, lubridate, nanotime, nycflights13, palmerpenguins,
        rmarkdown, scales, stringi, survival, testthat (>= 3.1.1),
        tibble, units (>= 0.7.2), vdiffr, withr
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.3.9000
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/testthat/start-first: format_multi_fuzz, format_multi_fuzz_2,
        format_multi, ctl_colonnade, ctl_colonnade_1, ctl_colonnade_2
Config/autostyle/scope: line_breaks
Config/autostyle/strict: true
Config/gha/extra-packages: units=?ignore-before-r=4.3.0
Config/Needs/website: tidyverse/tidytemplate
NeedsCompilation: no
Packaged: 2025-09-17 03:59:12 UTC; kirill
Author: Kirill Müller [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1416-3412>),
  Hadley Wickham [aut],
  RStudio [cph]
Maintainer: Kirill Müller <kirill@cynkra.com>
Repository: CRAN
Date/Publication: 2025-09-17 11:20:02 UTC
Built: R 4.6.0; ; 2026-04-25 11:14:26 UTC; unix

Package: pillar
Title: Coloured Formatting for Columns
Version: 1.8.0
Authors@R: 
    c(person(given = "Kirill",
             family = "M\u00fcller",
             role = c("aut", "cre"),
             email = "krlmlr+r@mailbox.org"),
      person(given = "Hadley",
             family = "Wickham",
             role = "aut"),
      person(given = "RStudio",
             role = "cph"))
Description: Provides 'pillar' and 'colonnade' generics designed
    for formatting columns of data using the full range of colours
    provided by modern terminals.
License: MIT + file LICENSE
URL: https://pillar.r-lib.org/, https://github.com/r-lib/pillar
BugReports: https://github.com/r-lib/pillar/issues
Imports: cli (>= 2.3.0), fansi, glue, lifecycle, rlang (>= 1.0.2), utf8
        (>= 1.1.0), utils, vctrs (>= 0.3.8)
Suggests: bit64, debugme, DiagrammeR, dplyr, formattable, ggplot2,
        knitr, lubridate, nanotime, nycflights13, palmerpenguins,
        rmarkdown, scales, stringi, survival, testthat (>= 3.1.1),
        tibble, units (>= 0.7.2), vdiffr, withr
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.2.0
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/testthat/start-first: format_multi_fuzz, format_multi_fuzz_2,
        format_multi, ctl_colonnade, ctl_colonnade_1, ctl_colonnade_2
Config/autostyle/scope: line_breaks
Config/autostyle/strict: true
Config/gha/extra-packages: DiagrammeR=?ignore-before-r=3.5.0
NeedsCompilation: no
Packaged: 2022-07-09 04:42:15 UTC; kirill
Author: Kirill Müller [aut, cre],
  Hadley Wickham [aut],
  RStudio [cph]
Maintainer: Kirill Müller <krlmlr+r@mailbox.org>
Repository: CRAN
Date/Publication: 2022-07-18 08:50:06 UTC

# Generated by roxygen2: do not edit by hand

S3method(as_tbl,data.frame)
S3method(ctl_new_pillar,tbl)
S3method(ctl_new_pillar_list,tbl)
S3method(ctl_new_rowid_pillar,tbl)
S3method(extra_cols,pillar_squeezed_colonnade)
S3method(format,pillar)
S3method(format,pillar_1e)
S3method(format,pillar_capital)
S3method(format,pillar_char)
S3method(format,pillar_char_attr)
S3method(format,pillar_colonnade)
S3method(format,pillar_continuation_shaft)
S3method(format,pillar_num)
S3method(format,pillar_num_attr)
S3method(format,pillar_ornament)
S3method(format,pillar_rif_shaft)
S3method(format,pillar_rif_title)
S3method(format,pillar_rif_type)
S3method(format,pillar_shaft)
S3method(format,pillar_shaft_decimal)
S3method(format,pillar_shaft_simple)
S3method(format,pillar_squeezed_colonnade)
S3method(format,pillar_tbl_format_setup)
S3method(format,pillar_title)
S3method(format,pillar_type)
S3method(format,tbl)
S3method(format_glimpse,character)
S3method(format_glimpse,default)
S3method(format_glimpse,factor)
S3method(format_glimpse,list)
S3method(format_type_sum,AsIs)
S3method(format_type_sum,default)
S3method(glimpse,data.frame)
S3method(glimpse,default)
S3method(glimpse,tbl)
S3method(obj_print_data,pillar_vctr)
S3method(obj_print_footer,pillar_vctr)
S3method(obj_print_header,pillar_vctr)
S3method(obj_sum,AsIs)
S3method(obj_sum,default)
S3method(pillar_shaft,AsIs)
S3method(pillar_shaft,Date)
S3method(pillar_shaft,POSIXt)
S3method(pillar_shaft,Surv)
S3method(pillar_shaft,Surv2)
S3method(pillar_shaft,character)
S3method(pillar_shaft,default)
S3method(pillar_shaft,factor)
S3method(pillar_shaft,glue)
S3method(pillar_shaft,integer64)
S3method(pillar_shaft,list)
S3method(pillar_shaft,logical)
S3method(pillar_shaft,numeric)
S3method(pillar_shaft,pillar_empty_col)
S3method(pillar_shaft,pillar_vctr)
S3method(pillar_shaft,vctrs_list_of)
S3method(pillar_shaft,vctrs_unspecified)
S3method(pillar_shaft,vctrs_vctr)
S3method(print,pillar)
S3method(print,pillar_1e)
S3method(print,pillar_colonnade)
S3method(print,pillar_ornament)
S3method(print,pillar_shaft)
S3method(print,pillar_squeezed_colonnade)
S3method(print,pillar_tbl_format_setup)
S3method(print,pillar_vctr)
S3method(print,pillar_vctr_attr)
S3method(print,tbl)
S3method(size_sum,default)
S3method(tbl_format_body,pillar_tbl_format_setup)
S3method(tbl_format_body,tbl)
S3method(tbl_format_footer,pillar_tbl_format_setup)
S3method(tbl_format_footer,tbl)
S3method(tbl_format_header,pillar_tbl_format_setup)
S3method(tbl_format_header,tbl)
S3method(tbl_format_setup,tbl)
S3method(tbl_nrow,data.frame)
S3method(tbl_nrow,tbl)
S3method(tbl_sum,default)
S3method(tbl_sum,tbl)
S3method(type_sum,Surv)
S3method(type_sum,Surv2)
S3method(type_sum,default)
S3method(type_sum,factor)
S3method(type_sum,ordered)
S3method(type_sum,vctrs_vctr)
S3method(vec_arith,pillar_num)
S3method(vec_arith.numeric,pillar_num)
S3method(vec_arith.pillar_num,MISSING)
S3method(vec_arith.pillar_num,default)
S3method(vec_cast,character.pillar_char)
S3method(vec_cast,double.pillar_num)
S3method(vec_cast,pillar_char.character)
S3method(vec_cast,pillar_char.pillar_char)
S3method(vec_cast,pillar_num.double)
S3method(vec_cast,pillar_num.integer)
S3method(vec_cast,pillar_num.pillar_num)
S3method(vec_math,pillar_num)
S3method(vec_proxy_compare,pillar_vctr)
S3method(vec_proxy_order,pillar_vctr)
S3method(vec_ptype2,character.pillar_char)
S3method(vec_ptype2,double.pillar_num)
S3method(vec_ptype2,integer.pillar_num)
S3method(vec_ptype2,pillar_char.character)
S3method(vec_ptype2,pillar_char.pillar_char)
S3method(vec_ptype2,pillar_num.double)
S3method(vec_ptype2,pillar_num.integer)
S3method(vec_ptype2,pillar_num.pillar_num)
S3method(vec_ptype_abbr,pillar_char)
S3method(vec_ptype_abbr,pillar_empty_col)
S3method(vec_ptype_abbr,pillar_num)
S3method(vec_ptype_full,pillar_char)
S3method(vec_ptype_full,pillar_num)
S3method(vec_ptype_full,pillar_vctr)
export(align)
export(char)
export(colonnade)
export(ctl_new_pillar)
export(ctl_new_pillar_list)
export(ctl_new_rowid_pillar)
export(dim_desc)
export(expect_known_display)
export(extra_cols)
export(format_glimpse)
export(format_type_sum)
export(get_extent)
export(get_max_extent)
export(glimpse)
export(is_vector_s3)
export(new_ornament)
export(new_pillar)
export(new_pillar_component)
export(new_pillar_shaft)
export(new_pillar_shaft_simple)
export(new_pillar_title)
export(new_pillar_type)
export(num)
export(obj_sum)
export(pillar)
export(pillar_component)
export(pillar_shaft)
export(scale_x_num)
export(scale_y_num)
export(set_char_opts)
export(set_num_opts)
export(size_sum)
export(squeeze)
export(style_bold)
export(style_na)
export(style_neg)
export(style_num)
export(style_subtle)
export(style_subtle_num)
export(tbl_format_body)
export(tbl_format_footer)
export(tbl_format_header)
export(tbl_format_setup)
export(tbl_nrow)
export(tbl_sum)
export(type_sum)
import(lifecycle)
import(rlang)
importFrom(cli,ansi_strip)
importFrom(cli,ansi_strwrap)
importFrom(cli,ansi_substr)
importFrom(cli,symbol)
importFrom(cli,utf8_nchar)
importFrom(glue,as_glue)
importFrom(utils,head)
importFrom(utils,str)
importFrom(vctrs,data_frame)
importFrom(vctrs,new_data_frame)
importFrom(vctrs,new_list_of)
importFrom(vctrs,new_vctr)
importFrom(vctrs,obj_print_data)
importFrom(vctrs,obj_print_footer)
importFrom(vctrs,obj_print_header)
importFrom(vctrs,unspecified)
importFrom(vctrs,vec_arith)
importFrom(vctrs,vec_arith.numeric)
importFrom(vctrs,vec_arith_base)
importFrom(vctrs,vec_assert)
importFrom(vctrs,vec_c)
importFrom(vctrs,vec_cast)
importFrom(vctrs,vec_cast.double)
importFrom(vctrs,vec_cast.integer)
importFrom(vctrs,vec_data)
importFrom(vctrs,vec_is)
importFrom(vctrs,vec_math)
importFrom(vctrs,vec_math_base)
importFrom(vctrs,vec_proxy)
importFrom(vctrs,vec_proxy_compare)
importFrom(vctrs,vec_proxy_order)
importFrom(vctrs,vec_ptype2)
importFrom(vctrs,vec_ptype_abbr)
importFrom(vctrs,vec_ptype_full)
importFrom(vctrs,vec_rbind)
importFrom(vctrs,vec_restore)
importFrom(vctrs,vec_size)
importFrom(vctrs,vec_slice)

<!-- README.md and index.md are generated from README.Rmd. Please edit that file. -->

# pillar

<!-- badges: start -->

[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![R build
status](https://github.com/r-lib/pillar/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/r-lib/pillar/actions)
[![Coverage
status](https://codecov.io/gh/r-lib/pillar/branch/main/graph/badge.svg)](https://app.codecov.io/gh/r-lib/pillar)
[![CRAN
status](https://www.r-pkg.org/badges/version/pillar)](https://cran.r-project.org/package=pillar)
<!-- badges: end -->

pillar provides tools for styling columns of data, artfully using colour
and unicode characters to guide the eye.

## Installation

``` r
# pillar is installed if you install the tidyverse package:
install.packages("tidyverse")

# Alternatively, install just pillar:
install.packages("pillar")
```

## Usage

pillar is a developer-facing package that is not designed for end-users.
It powers the `print()` and `format()` methods for
[tibble](https://tibble.tidyverse.org/)s. It also and defines generics
and helpers that are useful for package authors who create custom vector
classes (see <https://github.com/krlmlr/awesome-vctrs#readme> for
examples) or custom table classes (like
[dbplyr](https://dbplyr.tidyverse.org/) or
[sf](https://r-spatial.github.io/sf/)).

``` r
library(pillar)

x <- 123456789 * (10^c(-3, -5, NA, -8, -10))
pillar(x)
#> <pillar>
#>       <dbl>
#> 123457.    
#>   1235.    
#>     NA     
#>      1.23  
#>      0.0123

tbl_format_setup(tibble::tibble(x))
#> <pillar_tbl_format_setup>
#> <tbl_format_header(setup)>
#> # A tibble: 5 × 1
#> <tbl_format_body(setup)>
#>             x
#>         <dbl>
#> 1 123457.    
#> 2   1235.    
#> 3     NA     
#> 4      1.23  
#> 5      0.0123
#> <tbl_format_footer(setup)>
```

## Custom vector classes

The primary user of this package is
[tibble](https://github.com/tidyverse/tibble), which lets pillar do all
the formatting work. Packages that implement a data type to be used in a
tibble column can customize the display by implementing a
`pillar_shaft()` method.

``` r
library(pillar)

percent <- vctrs::new_vctr(9:11 * 0.01, class = "percent")

pillar_shaft.percent <- function(x, ...) {
  fmt <- format(vctrs::vec_data(x) * 100)
  new_pillar_shaft_simple(paste0(fmt, " ", style_subtle("%")), align = "right")
}

pillar(percent)
#> <pillar>
#> <percent>
#>       9 %
#>      10 %
#>      11 %
```

See `vignette("pillar", package = "vctrs")` for details.

## Custom table classes

pillar provides various extension points for customizing how a
tibble-like class is printed.

``` r
tbl <- vctrs::new_data_frame(list(a = 1:3), class = c("my_tbl", "tbl"))

tbl_sum.my_tbl <- function(x, ...) {
  c("Hello" = "world!")
}

tbl
#> # Hello: world!
#>       a
#>   <int>
#> 1     1
#> 2     2
#> 3     3
```

See `vignette("extending", package = "pillar")` for a walkthrough of the
options.

------------------------------------------------------------------------

## Code of Conduct

Please note that the pillar project is released with a [Contributor Code
of Conduct](https://pillar.r-lib.org/CODE_OF_CONDUCT.html). By
contributing to this project, you agree to abide by its terms.

<!-- README.md is generated from README.Rmd on GitHub Actions: do not edit by hand -->

<!-- .github/README.md is copied manually from README.md and adapted -->

# pillar

<!-- badges: start -->

[![Lifecycle: stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable) [![R build status](https://github.com/r-lib/pillar/workflows/R-CMD-check/badge.svg)](https://github.com/r-lib/pillar/actions) [![Coverage status](https://codecov.io/gh/r-lib/pillar/branch/main/graph/badge.svg)](https://codecov.io/github/r-lib/pillar?branch=main) [![CRAN status](https://www.r-pkg.org/badges/version/pillar)](https://cran.r-project.org/package=pillar)

<!-- badges: end -->

pillar provides tools for styling columns of data, artfully using colour and unicode characters to guide the eye.

## Installation

<pre class='chroma'>
<span class='c'># pillar is installed if you install the tidyverse package:</span>
<span class='nf'><a href='https://rdrr.io/r/utils/install.packages.html'>install.packages</a></span><span class='o'>(</span><span class='s'>"tidyverse"</span><span class='o'>)</span>

<span class='c'># Alternatively, install just pillar:</span>
<span class='nf'><a href='https://rdrr.io/r/utils/install.packages.html'>install.packages</a></span><span class='o'>(</span><span class='s'>"pillar"</span><span class='o'>)</span></pre>

## Usage

pillar is a developer-facing package that is not designed for end-users. It powers the [`print()`](https://rdrr.io/r/base/print.html) and [`format()`](https://rdrr.io/r/base/format.html) methods for [tibble](https://tibble.tidyverse.org/)s. It also and defines generics and helpers that are useful for package authors who create custom vector classes (see <https://github.com/krlmlr/awesome-vctrs#readme> for examples) or custom table classes (like [dbplyr](https://dbplyr.tidyverse.org/) or [sf](https://r-spatial.github.io/sf/)).

<pre class='chroma'>
<span class='kr'><a href='https://rdrr.io/r/base/library.html'>library</a></span><span class='o'>(</span><span class='nv'><a href='https://pillar.r-lib.org/'>pillar</a></span><span class='o'>)</span>

<span class='nv'>x</span> <span class='o'>&lt;-</span> <span class='m'>123456789</span> <span class='o'>*</span> <span class='o'>(</span><span class='m'>10</span> <span class='o'>^</span> <span class='nf'><a href='https://rdrr.io/r/base/c.html'>c</a></span><span class='o'>(</span><span class='o'>-</span><span class='m'>3</span>, <span class='o'>-</span><span class='m'>5</span>, <span class='kc'>NA</span>, <span class='o'>-</span><span class='m'>8</span>, <span class='o'>-</span><span class='m'>10</span><span class='o'>)</span><span class='o'>)</span>
<span class='nf'><a href='https://pillar.r-lib.org/reference/pillar.html'>pillar</a></span><span class='o'>(</span><span class='nv'>x</span><span class='o'>)</span>
<span class='c'>#&gt; <span style='font-weight: bold;'>&lt;pillar&gt;</span></span>
<span class='c'>#&gt;       <span style='color: #555555;font-style: italic;'>&lt;dbl&gt;</span></span>
<span class='c'>#&gt; <span style='text-decoration: underline;'>123</span><span>457.    </span></span>
<span class='c'>#&gt;   <span style='text-decoration: underline;'>1</span><span>235.    </span></span>
<span class='c'>#&gt;     <span style='color: #BB0000;'>NA</span><span>     </span></span>
<span class='c'>#&gt;      1.23  </span>
<span class='c'>#&gt;      0.012<span style='text-decoration: underline;'>3</span></span>

<span class='nf'><a href='https://pillar.r-lib.org/reference/tbl_format_setup.html'>tbl_format_setup</a></span><span class='o'>(</span><span class='nf'>tibble</span><span class='nf'>::</span><span class='nf'><a href='https://tibble.tidyverse.org/reference/tibble.html'>tibble</a></span><span class='o'>(</span><span class='nv'>x</span><span class='o'>)</span><span class='o'>)</span>
<span class='c'>#&gt; <span style='font-weight: bold;'>&lt;pillar_tbl_format_setup&gt;</span></span>
<span class='c'>#&gt; <span style='font-weight: bold;'>&lt;tbl_format_header(setup)&gt;</span></span>
<span class='c'>#&gt; <span style='color: #555555;'># A data frame: 5 × 1</span></span>
<span class='c'>#&gt; <span style='font-weight: bold;'>&lt;tbl_format_body(setup)&gt;</span></span>
<span class='c'>#&gt;             <span style='font-weight: bold;'>x</span></span>
<span class='c'>#&gt;         <span style='color: #555555;font-style: italic;'>&lt;dbl&gt;</span></span>
<span class='c'>#&gt; <span style='color: #555555;'>1</span><span> </span><span style='text-decoration: underline;'>123</span><span>457.    </span></span>
<span class='c'>#&gt; <span style='color: #555555;'>2</span><span>   </span><span style='text-decoration: underline;'>1</span><span>235.    </span></span>
<span class='c'>#&gt; <span style='color: #555555;'>3</span><span>     </span><span style='color: #BB0000;'>NA</span><span>     </span></span>
<span class='c'>#&gt; <span style='color: #555555;'>4</span><span>      1.23  </span></span>
<span class='c'>#&gt; <span style='color: #555555;'>5</span><span>      0.012</span><span style='text-decoration: underline;'>3</span></span>
<span class='c'>#&gt; <span style='font-weight: bold;'>&lt;tbl_format_footer(setup)&gt;</span></span></pre>

## Custom vector classes

The primary user of this package is [tibble](https://github.com/tidyverse/tibble), which lets pillar do all the formatting work. Packages that implement a data type to be used in a tibble column can customize the display by implementing a [`pillar_shaft()`](https://pillar.r-lib.org/reference/pillar_shaft.html) method.

<pre class='chroma'>
<span class='kr'><a href='https://rdrr.io/r/base/library.html'>library</a></span><span class='o'>(</span><span class='nv'><a href='https://pillar.r-lib.org/'>pillar</a></span><span class='o'>)</span>

<span class='nv'>percent</span> <span class='o'>&lt;-</span> <span class='nf'>vctrs</span><span class='nf'>::</span><span class='nf'><a href='https://vctrs.r-lib.org/reference/new_vctr.html'>new_vctr</a></span><span class='o'>(</span><span class='m'>9</span><span class='o'>:</span><span class='m'>11</span> <span class='o'>*</span> <span class='m'>0.01</span>, class <span class='o'>=</span> <span class='s'>"percent"</span><span class='o'>)</span>

<span class='nv'>pillar_shaft.percent</span> <span class='o'>&lt;-</span> <span class='kr'>function</span><span class='o'>(</span><span class='nv'>x</span>, <span class='nv'>...</span><span class='o'>)</span> <span class='o'>{</span>
  <span class='nv'>fmt</span> <span class='o'>&lt;-</span> <span class='nf'><a href='https://rdrr.io/r/base/format.html'>format</a></span><span class='o'>(</span><span class='nf'>vctrs</span><span class='nf'>::</span><span class='nf'><a href='https://vctrs.r-lib.org/reference/vec_data.html'>vec_data</a></span><span class='o'>(</span><span class='nv'>x</span><span class='o'>)</span> <span class='o'>*</span> <span class='m'>100</span><span class='o'>)</span>
  <span class='nf'><a href='https://pillar.r-lib.org/reference/new_pillar_shaft.html'>new_pillar_shaft_simple</a></span><span class='o'>(</span><span class='nf'><a href='https://rdrr.io/r/base/paste.html'>paste0</a></span><span class='o'>(</span><span class='nv'>fmt</span>, <span class='s'>" "</span>, <span class='nf'><a href='https://pillar.r-lib.org/reference/style_subtle.html'>style_subtle</a></span><span class='o'>(</span><span class='s'>"%"</span><span class='o'>)</span><span class='o'>)</span>, align <span class='o'>=</span> <span class='s'>"right"</span><span class='o'>)</span>
<span class='o'>}</span>

<span class='nf'><a href='https://pillar.r-lib.org/reference/pillar.html'>pillar</a></span><span class='o'>(</span><span class='nv'>percent</span><span class='o'>)</span>
<span class='c'>#&gt; <span style='font-weight: bold;'>&lt;pillar&gt;</span></span>
<span class='c'>#&gt; <span style='color: #555555;font-style: italic;'>&lt;percent&gt;</span></span>
<span class='c'>#&gt;       9 <span style='color: #555555;'>%</span></span>
<span class='c'>#&gt;      10 <span style='color: #555555;'>%</span></span>
<span class='c'>#&gt;      11 <span style='color: #555555;'>%</span></span></pre>

See [`vignette("pillar", package = "vctrs")`](https://vctrs.r-lib.org/articles/pillar.html) for details.

## Custom table classes

pillar provides various extension points for customizing how a tibble-like class is printed.

<pre class='chroma'>
<span class='nv'>tbl</span> <span class='o'>&lt;-</span> <span class='nf'>vctrs</span><span class='nf'>::</span><span class='nf'><a href='https://vctrs.r-lib.org/reference/new_data_frame.html'>new_data_frame</a></span><span class='o'>(</span><span class='nf'><a href='https://rdrr.io/r/base/list.html'>list</a></span><span class='o'>(</span>a <span class='o'>=</span> <span class='m'>1</span><span class='o'>:</span><span class='m'>3</span><span class='o'>)</span>, class <span class='o'>=</span> <span class='nf'><a href='https://rdrr.io/r/base/c.html'>c</a></span><span class='o'>(</span><span class='s'>"my_tbl"</span>, <span class='s'>"tbl"</span><span class='o'>)</span><span class='o'>)</span>

<span class='nv'>tbl_sum.my_tbl</span> <span class='o'>&lt;-</span> <span class='kr'>function</span><span class='o'>(</span><span class='nv'>x</span>, <span class='nv'>...</span><span class='o'>)</span> <span class='o'>{</span>
  <span class='nf'><a href='https://rdrr.io/r/base/c.html'>c</a></span><span class='o'>(</span><span class='s'>"Hello"</span> <span class='o'>=</span> <span class='s'>"world!"</span><span class='o'>)</span>
<span class='o'>}</span>

<span class='nv'>tbl</span>
<span class='c'>#&gt; <span style='color: #555555;'># Hello: world!</span></span>
<span class='c'>#&gt;       <span style='font-weight: bold;'>a</span></span>
<span class='c'>#&gt;   <span style='color: #555555;font-style: italic;'>&lt;int&gt;</span></span>
<span class='c'>#&gt; <span style='color: #555555;'>1</span><span>     1</span></span>
<span class='c'>#&gt; <span style='color: #555555;'>2</span><span>     2</span></span>
<span class='c'>#&gt; <span style='color: #555555;'>3</span><span>     3</span></span></pre>

See [`vignette("extending", package = "pillar")`](https://pillar.r-lib.org/articles/extending.html) for a walkthrough of the options.

------------------------------------------------------------------------

## Code of Conduct

Please note that the pillar project is released with a [Contributor Code of Conduct](https://pillar.r-lib.org/CODE_OF_CONDUCT.html). By contributing to this project, you agree to abide by its terms.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  eval = rlang::is_installed("tibble"),
  comment = "#>"
)

pillar:::set_show_source_hooks()

## ----setup--------------------------------------------------------------------
library(pillar)

## ----echo = FALSE, error = TRUE-----------------------------------------------
try({
text <- paste(
  readLines("format.mmd"),
  collapse = "\n"
)
DiagrammeR::mermaid(text)
})

## -----------------------------------------------------------------------------
tbl <- tibble::tibble(a = 1:3, b = tibble::tibble(c = 4:6, d = 7:9), e = 10:12)
print(tbl, width = 23)
str(tbl)

## ----show_source = TRUE-------------------------------------------------------
# print.tbl <- function (x, width = NULL, ..., n = NULL, max_extra_cols = NULL, 
#     max_footer_lines = NULL) 
# {
#     print_tbl(x, width, ..., n = n, max_extra_cols = max_extra_cols, 
#         max_footer_lines = max_footer_lines)
# }

## ----show_source = TRUE-------------------------------------------------------
# format.tbl <- function (x, width = NULL, ..., n = NULL, max_extra_cols = NULL, 
#     max_footer_lines = NULL) 
# {
#     format_tbl(x, width, ..., n = n, max_extra_cols = max_extra_cols, 
#         max_footer_lines = max_footer_lines)
# }

## -----------------------------------------------------------------------------
setup <- tbl_format_setup(tbl, width = 24)
setup

## ----show_source = TRUE-------------------------------------------------------
# tbl_format_setup <- function (x, width = NULL, ..., setup = list(tbl_sum = tbl_sum(x)), 
#     n = NULL, max_extra_cols = NULL, max_footer_lines = NULL, 
#     focus = NULL) 
# {
#     "!!!!DEBUG tbl_format_setup()"
#     width <- get_width_print(width)
#     n <- get_n_print(n, tbl_nrow(x))
#     max_extra_cols <- get_max_extra_cols(max_extra_cols)
#     max_footer_lines <- get_max_footer_lines(max_footer_lines)
#     out <- tbl_format_setup_dispatch(x, width, ..., setup = setup, 
#         n = n, max_extra_cols = max_extra_cols, max_footer_lines = max_footer_lines, 
#         focus = focus)
#     return(out)
#     UseMethod("tbl_format_setup")
# }

## ----show_source = TRUE-------------------------------------------------------
# tbl_format_setup.tbl <- function (x, width, ..., setup, n, max_extra_cols, max_footer_lines, 
#     focus) 
# {
#     "!!!!DEBUG tbl_format_setup.tbl()"
#     if (is.null(setup)) {
#         tbl_sum <- tbl_sum(x)
#         return(new_tbl_format_setup(width, tbl_sum, rows_total = NA))
#     }
#     else {
#         tbl_sum <- setup$tbl_sum
#     }
#     rows <- tbl_nrow(x)
#     lazy <- is.na(rows)
#     if (lazy) {
#         max <- attr(n, "max") %||% n
#         df <- as.data.frame(head(x, max + 1))
#         if (nrow(df) <= max) {
#             rows <- nrow(df)
#             n <- rows
#         }
#         else {
#             df <- vec_head(df, n)
#         }
#     }
#     else {
#         df <- df_head(x, n)
#     }
#     if (is.na(rows)) {
#         needs_dots <- (nrow(df) >= n)
#     }
#     else {
#         needs_dots <- (rows > n)
#     }
#     if (needs_dots) {
#         rows_missing <- rows - n
#     }
#     else {
#         rows_missing <- 0
#     }
#     rownames(df) <- NULL
#     colonnade <- ctl_colonnade(df, has_row_id = if (!lazy && 
#         .row_names_info(x) > 0) 
#         "*"
#     else TRUE, width = width, controller = x, focus = focus)
#     body <- colonnade$body
#     extra_cols <- colonnade$extra_cols
#     extra_cols_total <- length(extra_cols)
#     if (extra_cols_total > max_extra_cols) {
#         length(extra_cols) <- max_extra_cols
#     }
#     abbrev_cols <- colonnade$abbrev_cols
#     new_tbl_format_setup(x = x, df = df, width = width, tbl_sum = tbl_sum, 
#         body = body, rows_missing = rows_missing, rows_total = rows, 
#         extra_cols = extra_cols, extra_cols_total = extra_cols_total, 
#         max_footer_lines = max_footer_lines, abbrev_cols = abbrev_cols)
# }

## -----------------------------------------------------------------------------
tbl_format_header(tbl, setup)
tbl_format_body(tbl, setup)
tbl_format_footer(tbl, setup)

## -----------------------------------------------------------------------------
class(tbl_format_body(tbl, setup))
typeof(tbl_format_body(tbl, setup))

## ----show_source = TRUE-------------------------------------------------------
# tbl_format_header.tbl <- function (x, setup, ...) 
# {
#     named_header <- setup$tbl_sum
#     focus <- attr(x, "pillar_focus")
#     if (!is.null(focus)) {
#         named_header <- c(named_header, `Focus columns` = collapse(tick_if_needed(focus)))
#     }
#     if (all(names2(named_header) == "")) {
#         header <- named_header
#     }
#     else {
#         header <- paste0(align(paste0(names2(named_header), ":"), 
#             space = NBSP), " ", named_header)
#     }
#     style_subtle(format_comment(header, width = setup$width))
# }

## ----show_source = TRUE-------------------------------------------------------
# tbl_format_body.tbl <- function (x, setup, ...) 
# {
#     force(setup)
#     setup$body
# }

## ----show_source = TRUE-------------------------------------------------------
# tbl_format_footer.tbl <- function (x, setup, ...) 
# {
#     footer <- format_footer(x, setup)
#     footer_comment <- wrap_footer_bullet(footer, setup)
#     footer_advice <- format_footer_advice(x, setup)
#     footer_advice_comment <- wrap_footer_bullet(footer_advice, 
#         setup, lines = 1, ellipsis = FALSE, bullet = symbol$info)
#     style_subtle(c(footer_comment, footer_advice_comment))
# }

## -----------------------------------------------------------------------------
ctl_new_pillar_list(tbl, tbl$a, width = 20)
ctl_new_pillar_list(tbl, tbl$b, width = 20)

## ----show_source = TRUE-------------------------------------------------------
# ctl_new_pillar_list.tbl <- function (controller, x, width, ..., title = NULL, first_pillar = NULL) 
# {
#     "!!!!DEBUG ctl_new_pillar_list.tbl(`v(width)`, `v(title)`)"
#     if (is.data.frame(x)) {
#         new_data_frame_pillar_list(x, controller, width, title = title, 
#             first_pillar = first_pillar)
#     }
#     else if (is.matrix(x) && !inherits(x, c("Surv", "Surv2"))) {
#         new_matrix_pillar_list(x, controller, width, title = title, 
#             first_pillar = first_pillar)
#     }
#     else if (is.array(x) && length(dim(x)) > 2) {
#         new_array_pillar_list(x, controller, width, title = title, 
#             first_pillar = first_pillar)
#     }
#     else {
#         if (is.null(first_pillar)) {
#             first_pillar <- ctl_new_pillar(controller, x, width, 
#                 ..., title = prepare_title(title))
#         }
#         new_single_pillar_list(first_pillar, width)
#     }
# }

## -----------------------------------------------------------------------------
ctl_new_pillar(tbl, tbl$a, width = 20)

## ----show_source = TRUE-------------------------------------------------------
# ctl_new_pillar.tbl <- function (controller, x, width, ..., title = NULL) 
# {
#     "!!!!DEBUG ctl_new_pillar.tbl(`v(width)`, `v(title)`)"
#     pillar(x, title, if (!is.null(width)) 
#         max0(width))
# }

## ----show_source = TRUE-------------------------------------------------------
# pillar <- function (x, title = NULL, width = NULL, ...) 
# {
#     "!!!!DEBUG pillar(`v(class(x))`, `v(title)`, `v(width)`)"
#     pillar_from_shaft(new_pillar_title(title), new_pillar_type(x), 
#         pillar_shaft(x, ...), width)
# }

## ----show_source = TRUE-------------------------------------------------------
# new_pillar <- function (components, ..., width = NULL, class = NULL, extra = deprecated()) 
# {
#     "!!!!DEBUG new_pillar(`v(width)`, `v(class)`)"
#     if (is_present(extra)) {
#         deprecate_warn("1.7.0", "pillar::new_pillar(extra = )")
#     }
#     check_dots_empty()
#     if (length(components) > 0 && !is_named(components)) {
#         abort("All components must have names.")
#     }
#     structure(components, width = width, class = c(class, "pillar"))
# }

## ----show_source = TRUE-------------------------------------------------------
# format.pillar <- function (x, width = NULL, ...) 
# {
#     if (is.null(width)) {
#         width <- get_width(x)
#     }
#     if (is.null(width)) {
#         width <- pillar_get_width(x)
#     }
#     as_glue(pillar_format_parts_2(x, width)$aligned)
# }

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(pillar)

## -----------------------------------------------------------------------------
example_tbl <- function(class = NULL) {
  vctrs::new_data_frame(
    list(
      a = letters[1:3],
      b = data.frame(c = 1:3, d = 4:6 + 0.5)
    ),
    class = c(class, "tbl")
  )
}

## -----------------------------------------------------------------------------
example_tbl()

## -----------------------------------------------------------------------------
tbl_sum.default_header_extend <- function(x, ...) {
  default_header <- NextMethod()
  c(default_header, "New" = "A new header")
}

example_tbl("default_header_extend")

tbl_sum.default_header_replace <- function(x, ...) {
  c("Override" = "Replace all headers")
}

example_tbl("default_header_replace")

## -----------------------------------------------------------------------------
tbl_format_header.custom_header_replace <- function(x, setup, ...) {
  cli::style_italic(names(setup$tbl_sum), " = ", setup$tbl_sum)
}

example_tbl("custom_header_replace")

## -----------------------------------------------------------------------------
tbl_format_footer.custom_footer_extend <- function(x, setup, ...) {
  default_footer <- NextMethod()

  extra_info <- "and with extra info in the footer"
  extra_footer <- style_subtle(paste0("# ", cli::symbol$ellipsis, " ", extra_info))

  c(default_footer, extra_footer)
}

print(example_tbl("custom_footer_extend"), n = 2)

tbl_format_footer.custom_footer_replace <- function(x, setup, ...) {
  paste0("The table has ", setup$rows_total, " rows in total.")
}

print(example_tbl("custom_footer_replace"), n = 2)

## -----------------------------------------------------------------------------
tbl_format_setup.extra_info <- function(x, width, ...) {
  setup <- NextMethod()
  cells <- prod(dim(x))
  setup$cells <- cells
  setup$tbl_sum <- c(setup$tbl_sum, "Cells" = as.character(cells))
  setup
}

tbl_format_footer.extra_info <- function(x, setup, ...) {
  paste0("The table has ", setup$cells, " cells in total.")
}

example_tbl("extra_info")

## -----------------------------------------------------------------------------
ctl_new_rowid_pillar.pillar_roman <- function(controller, x, width, ...) {
  out <- NextMethod()
  rowid <- utils::as.roman(seq_len(nrow(x)))
  width <- max(nchar(as.character(rowid)))
  new_pillar(
    list(
      title = out$title,
      type = out$type,
      data = pillar_component(
        new_pillar_shaft(list(row_ids = rowid),
          width = width,
          class = "pillar_rif_shaft"
        )
      )
    ),
    width = width
  )
}

example_tbl("pillar_roman")

## -----------------------------------------------------------------------------
ctl_new_pillar.pillar_rule <- function(controller, x, width, ..., title = NULL) {
  out <- NextMethod()
  new_pillar(list(
    top_rule = new_pillar_component(list("========"), width = 8),
    title = out$title,
    type = out$type,
    mid_rule = new_pillar_component(list("--------"), width = 8),
    data = out$data,
    bottom_rule = new_pillar_component(list("========"), width = 8)
  ))
}

example_tbl("pillar_rule")

## -----------------------------------------------------------------------------
rule <- function(char = "-") {
  stopifnot(nchar(char) == 1)
  structure(char, class = "rule")
}

format.rule <- function(x, width, ...) {
  paste(rep(x, width), collapse = "")
}

ctl_new_pillar.pillar_rule_adaptive <- function(controller, x, width, ..., title = NULL) {
  out <- NextMethod()
  if (is.null(out)) {
    return(NULL)
  }

  new_pillar(list(
    top_rule = new_pillar_component(list(rule("=")), width = 1),
    title = out$title,
    type = out$type,
    mid_rule = new_pillar_component(list(rule("-")), width = 1),
    data = out$data,
    bottom_rule = new_pillar_component(list(rule("=")), width = 1)
  ))
}

example_tbl("pillar_rule_adaptive")

## -----------------------------------------------------------------------------
ctl_new_pillar_list.hide_df <- function(controller, x, width, ..., title = NULL) {
  if (!is.data.frame(x)) {
    return(NextMethod())
  }

  if (width < 8) {
    return(NULL)
  }

  list(new_pillar(
    list(
      title = pillar_component(new_pillar_title(title)),
      type = new_pillar_component(list("<hidden>"), width = 8),
      data = new_pillar_component(list(""), width = 1)
    ),
    width = 8
  ))
}

example_tbl("hide_df")

## -----------------------------------------------------------------------------
tbl_format_body.oldskool <- function(x, setup, ...) {
  capture.output(print.data.frame(setup$df))
}

print(example_tbl("oldskool"), n = 2)

## 
## ## Integrating with other classes
## 
## ### gt
## 
## - Consumption via new `fmt_auto()`?
## 
## FIXME
## 
## ### units
## 
## xxx{r numbers-17}
## library(units)
## 
## set_units.pillar_num <- function(x, ...) {
##   unclassed <- x
##   class(unclassed) <- NULL
##   set_units(unclassed, ...)
## }
## 
## m <- set_units(1:3, m)
## km <- set_units(1:3, km)
## 
## tibble(
##   sci_int = set_num_opts(m + km, notation = "sci"),
##   digits_int = set_num_opts(km + m, digits = 4),
##   sci_ext = set_units(num(1:3 + 0.001, notation = "sci"), km)
## )
## 
## tibble(
##   sci_int = set_num_opts(m, notation = "sci") + km,
##   digits_int = set_num_opts(km, digits = 4) + m,
##   sci_ext = set_units(num(1:3, notation = "sci"), m) + km
## )
## xxx
## 
## ### formattable
## 
## xxx{r numbers-18, error = TRUE}
## library(formattable)
## 
## pillar_shaft.formattable <- function(x, ...) {
##   pillar::new_pillar_shaft_simple(format(x), align = "right")
## }
## 
## pillar_shaft.formattable_currency <- function(x, ...) {
##   formattable <- attr(x, "formattable", exact = TRUE)
## 
##   pillar_shaft(num(unclass(x), digits = formattable$digits))
## }
## 
## pillar_shaft.formattable_percent <- function(x, ...) {
##   formattable <- attr(x, "formattable", exact = TRUE)
## 
##   pillar_shaft(num(unclass(x), digits = formattable$digits, label = "%", scale = 100))
## }
## 
## pillar_shaft.formattable_scientific <- function(x, ...) {
##   pillar_shaft(num(unclass(x), notation = "sci"))
## }
## 
## type_sum.formattable <- function(x) {
##   formattable <- attr(x, "formattable", exact = TRUE)
## 
##   if (inherits(x, "formattable_currency")) {
##     I(sub("^formattable_", "", class(x)[[1]]))
##   } else if (inherits(x, "formattable_percent")) {
##     I("%")
##   } else {
##     abbreviate(sub("^formattable_", "", class(x)[[1]]), 4)
##   }
## }
## 
## num_currency(1:3 * 100 + 0.1)
## num_percent(1:3 * 0.1 + 0.001)
## num_scientific(1:3 * 0.1 + 0.001)
## 
## tibble(
##   currency = num_currency(1:3 * 100 + 0.1),
##   percent = num_percent(1:3 * 0.1 + 0.001),
##   scientific = num_scientific(1:3 * 0.1 + 0.001)
## )
## xxx
## 
## ### scales
## 
## xxx{r numbers-scales, error = TRUE}
## library(scales)
## 
## x <- num(1:10 / 100, label = "%", scale = 100)
## 
## scales::squish(x)
## 
## x < 0
## x < 0L
## 
## scales::cscale(x, scales::rescale_pal())
## xxx
## 
## ### ggplot2
## 
## xxx{r numbers-19}
## library(ggplot2)
## 
## scale_type.pillar_num <- function(x, ...) {
##   "continuous"
## }
## 
## data.frame(x = x, y = 1:10) %>%
##   ggplot(aes(x = x, y = y)) %>%
##   + geom_point()
## xxx
## 
## ## Rule-based decoration
## 
## 
## 
## xxx{r}
## library(dplyr)
## 
## data_units <-
##   palmerpenguins::penguins %>%
##   mutate(across(ends_with("_mm"), set_units, "mm")) %>%
##   mutate(across(ends_with("_g"), set_units, "g"))
## 
## data_units %>%
##   mutate(bill_area = bill_length_mm * bill_depth_mm, .after = island)
## xxx
## 
## xxx{r eval = FALSE}
## data_decor <-
##   data_units %>%
##   decorate(year, digits = 0) %>%
##   decorate(where(is.numeric), digits = 3)
## xxx
## 
## xxx{r eval = FALSE}
## data_decor %>%
##   mutate(bill_area = bill_length_mm * bill_depth_mm, .after = island)
## xxx
## 
## xxx{r echo = FALSE}
## data_units %>%
##   mutate(bill_area = bill_length_mm * bill_depth_mm, .after = island) %>%
##   mutate(across(year, set_num_opts, digits = 0)) %>%
##   mutate(across(where(is.numeric), set_num_opts, digits = 3))
## xxx
## 

#' Provide a succinct summary of an object
#'
#' `tbl_sum()` gives a brief textual description of a table-like object,
#' which should include the dimensions and the data source in the first element,
#' and additional information in the other elements (such as grouping for \pkg{dplyr}).
#' The default implementation forwards to [pillar::obj_sum()].
#'
#' This generic will be moved to \pkg{pillar}, and reexported from there
#' as soon as it becomes available.
#'
#' @return A named character vector, describing the dimensions in the first element
#'   and the data source in the name of the first element.
#'
#' @seealso [pillar::type_sum()]
#' @param x Object to summarise.
#' @export
tbl_sum <- function(x) {
  UseMethod("tbl_sum", x)
}

#' @export
tbl_sum.default <- function(x) {
  c("Description" = obj_sum(x))
}

#' @export
tbl_sum.tbl <- function(x) {
  c("A data frame" = dim_desc(x))
}

#' Format a character vector in a tibble
#'
#' These functions are reexported as [tibble::char()] and [tibble::set_char_opts()].
#'
#' @keywords internal
#' @export
char <- function(x, ..., min_chars = NULL,
                 shorten = c("back", "front", "mid", "abbreviate")) {
  stopifnot(is.character(x))
  check_dots_empty()

  if (missing(shorten)) {
    shorten <- NULL
  }

  out <- set_char_opts(
    x,
    min_chars = min_chars,
    shorten = shorten
  )

  new_class <- c("pillar_char", "pillar_vctr", "vctrs_vctr", "character")
  class(out) <- new_class

  out
}

#' @export
vec_ptype_full.pillar_char <- function(x, ...) {
  # No longer inherited in vctrs > 0.4.1,
  # https://github.com/r-lib/vctrs/pull/1571
  format(attr(x, "pillar", exact = TRUE))
}

#' @export
vec_ptype_abbr.pillar_char <- function(x, ...) {
  pillar_attr <- attr(x, "pillar", exact = TRUE)

  out <- "char"

  shorten <- pillar_attr$shorten
  if (!is.null(shorten) && shorten == "abbr") {
    out <- "abbr"
  }

  out
}

#' @export
format.pillar_char <- function(x, trim = FALSE, ...) {
  "!!!!DEBUG format.pillar_char()"

  shaft <- pillar_shaft(x)
  out <- format(shaft, width = get_width(shaft))
  if (trim) {
    attributes(out) <- NULL
  } else {
    out <- format(out, align = "left")
  }
  out
}

#' @export
#' @rdname char
set_char_opts <- function(x, ..., min_chars = NULL,
                          shorten = c("back", "front", "mid", "abbreviate")) {
  check_dots_empty()

  if (missing(shorten)) {
    shorten <- NULL
  } else if (!is.null(shorten)) {
    shorten <- arg_match(shorten)
  }

  pillar_attr <- structure(
    list(
      min_chars = min_chars,
      shorten = shorten
    ),
    class = c("pillar_char_attr", "pillar_vctr_attr", "tibble_vec_attr")
  )
  attr(x, "pillar") <- pillar_attr
  x
}

#' @export
format.pillar_char_attr <- function(x, ...) {
  out <- "pillar_char"

  min_chars <- x$min_chars
  if (!is.null(min_chars)) {
    out <- paste0(out, "(", min_chars, ")")
  }

  shorten <- x$shorten
  if (!is.null(shorten)) {
    extra <- switch(shorten,
      back = ">",
      front = "<",
      mid = "|",
      never = "=",
      abbr = "&"
    )

    out <- paste0(out, extra)
  }

  out
}

#' @export
vec_ptype2.pillar_char.pillar_char <- function(x, y, ...) {
  x
}
#' @export
vec_ptype2.pillar_char.character <- function(x, y, ...) {
  x
}
#' @export
vec_ptype2.character.pillar_char <- function(x, y, ...) {
  y
}

#' @export
vec_cast.pillar_char.pillar_char <- function(x, to, ...) {
  pillar_x <- attr(x, "pillar", exact = TRUE)
  pillar_to <- attr(to, "pillar", exact = TRUE)

  pillar_x_shorten <- pillar_x$shorten
  pillar_to_shorten <- pillar_to$shorten

  if (!is.null(pillar_x_shorten) && !is.null(pillar_to_shorten)) {
    if (!identical(pillar_x$shorten, pillar_to$shorten)) {
      abort("Only `pillar_char` objects with the same shortening setting can be combined.")
    }
  }

  pillar_x_min_chars <- pillar_x$min_chars
  pillar_to_min_chars <- pillar_to$min_chars

  if (!is.null(pillar_x_min_chars) && !is.null(pillar_to_min_chars)) {
    pillar_to$min_chars <- max(pillar_x_min_chars, pillar_to_min_chars)
  }

  attr(x, "pillar") <- pillar_to
  x
}
#' @export
vec_cast.character.pillar_char <- function(x, to, ...) {
  vec_data(x)
}
#' @export
vec_cast.pillar_char.character <- function(x, to, ...) {
  out <- char(x)
  attr(out, "pillar") <- attr(to, "pillar")
  out
}

test_that("max argument", {
  expect_snapshot({
    print(num(1:10), max = 5)
    print(char(letters), max = 10)
  })
})

test_that("max.print option", {
  local_options(max.print = 3)

  expect_snapshot({
    num(1:10)
    char(letters)
  })
})

rowidformat <- function(n, has_title_row = FALSE, has_star = FALSE, ...) {
  capital <- rif_capital(has_title_row, has_star, ...)
  shaft <- rif_shaft(n, ...)
  new_pillar_1e(capital, shaft)
}

new_pillar_1e <- function(capital, shaft, width = NULL) {
  ret <- structure(
    list(capital = capital, shaft = shaft),
    class = "pillar_1e"
  )
  ret <- set_width(ret, width)
  ret
}

#' @export
format.pillar_1e <- function(x, width = NULL, ...) {
  width <- pillar_get_width_1e(x, width)
  out <- pillar_format_parts(x, width)

  as_glue(unlist(unname(out)))
}

#' @export
print.pillar_1e <- function(x, ...) {
  writeLines(style_bold("<pillar>"))
  print(format(x, ...))
}

pillar_get_width_1e <- function(x, width) {
  if (is.null(width)) {
    width <- get_width(x)
  }

  if (is.null(width)) {
    widths <- get_widths(x)
    width <- max(widths)
  }

  min_widths <- max(get_min_widths(x))
  if (width < min_widths) width <- min_widths

  width
}

pillar_format_parts <- function(x, width, ...) {
  capital_format <- format(x$capital, width = width, ...)
  shaft_format <- format(x$shaft, width = width, ...)
  align <- attr(shaft_format, "align", exact = TRUE)

  capital_format <- align(capital_format, width = width, align = align)
  shaft_format <- align(shaft_format, width = width, align = align)

  list(
    capital_format = capital_format,
    shaft_format = shaft_format
  )
}

format_abbrev <- function(x, title = NULL, space = " ") {
  type_format <- ansi_strip(format_full_pillar_type(x))
  if (is.null(title)) {
    type_format
  } else {
    title_format <- format_full_pillar_title(title)
    paste0(title_format, space, type_format)
  }
}

format_full_pillar_title <- function(title) {
  # Backticks already added here
  title <- format_title(title, Inf)
  style_title(title)
}

context("format_rowid")

test_that("output test", {
  expect_pillar_output(
    xf = rowidformat(3),
    filename = "rowid-3.txt"
  )
  expect_pillar_output(
    xf = rowidformat(12, has_title_row = TRUE, has_star = TRUE),
    filename = "rowid-star-title-12.txt"
  )
})

test_that("can format new_pillar_title()", {
  expect_equal(
    format(new_pillar_title(tick_if_needed(c("a", "b c")))),
    format_full_pillar_title(tick_if_needed(c("a", "b c")))
  )
  expect_equal(
    format(new_pillar_title(names(mtcars))),
    format_full_pillar_title(names(mtcars))
  )
})

test_that("titles are left-adjusted", {
  expect_equal(
    get_extent(format(new_pillar_title(c("ab", "c")))),
    c(2, 2)
  )
  expect_equal(
    format(new_pillar_title(c("ab", "c"))),
    style_title(c("ab", "c "))
  )
})

rif_title <- function(has_title, ...) {
  ret <- structure(
    list(
      has_title = has_title
    ),
    class = "pillar_rif_title"
  )

  ret <- set_width(ret, 0L)
  ret
}

#' @export
format.pillar_rif_title <- function(x, width, ...) {
  if (!x$has_title) {
    character()
  } else {
    ""
  }
}

#' Provide a succinct summary of an object
#'
#' `tbl_sum()` gives a brief textual description of a table-like object,
#' which should include the dimensions and the data source in the first element,
#' and additional information in the other elements (such as grouping for \pkg{dplyr}).
#' The default implementation forwards to [pillar::obj_sum()].
#'
#' @return A named character vector, describing the dimensions in the first element
#'   and the data source in the name of the first element.
#'
#' @examples
#' tbl_sum(1:10)
#' tbl_sum(matrix(1:10))
#' tbl_sum(data.frame(a = 1))
#' tbl_sum(Sys.Date())
#' tbl_sum(Sys.time())
#' tbl_sum(mean)
#'
#' @seealso [pillar::type_sum()]
#' @param x Object to summarise.
#' @export
tbl_sum <- function(x) {
  UseMethod("tbl_sum", x)
}

#' @export
tbl_sum.default <- function(x) {
  c("Description" = obj_sum(x))
}

#' @export
tbl_sum.tbl <- function(x) {
  c("A data frame" = dim_desc(x))
}

#' @export
pillar_shaft.vctrs_unspecified <- function(x, ...) {
  new_pillar_shaft_simple(rep(".", vec_size(x)), align = "right")
}

#' @export
pillar_shaft.vctrs_vctr <- function(x, ...) {
  align <- if (is_character(x)) "left" else "right"
  new_pillar_shaft_simple(format(x), align = align)
}

#' @export
type_sum.vctrs_vctr <- function(x) {
  pillar_attr <- attr(x, "pillar", exact = TRUE)

  label <- pillar_attr$label
  if (!is.null(label)) {
    return(I(label))
  }

  vec_ptype_abbr(x)
}

#' @export
pillar_shaft.vctrs_list_of <- function(x, ...) {
  out <- style_subtle(map_chr(x, size_sum))

  new_pillar_shaft_simple(out, align = "right")
}

#' Format a character vector in a tibble
#'
#' These functions are reexported as [tibble::char()] and [tibble::set_char_opts()].
#'
#' @keywords internal
#' @export
char <- function(x, ..., min_chars = NULL,
                 shorten = c("back", "front", "mid", "abbreviate")) {
  stopifnot(is.character(x))
  check_dots_empty()

  if (missing(shorten)) {
    shorten <- NULL
  }

  out <- set_char_opts(
    x,
    min_chars = min_chars,
    shorten = shorten
  )

  new_class <- c("pillar_char", "pillar_vctr", "vctrs_vctr", "character")
  class(out) <- new_class

  out
}

#' @export
vec_ptype_abbr.pillar_char <- function(x, ...) {
  pillar_attr <- attr(x, "pillar", exact = TRUE)

  out <- "char"

  shorten <- pillar_attr$shorten
  if (!is.null(shorten) && shorten == "abbr") {
    out <- "abbr"
  }

  out
}

#' @export
format.pillar_char <- function(x, trim = FALSE, ...) {
  "!!!!DEBUG format.pillar_char()"

  shaft <- pillar_shaft(x)
  out <- format(shaft, width = get_width(shaft))
  if (trim) {
    attributes(out) <- NULL
  } else {
    out <- format(out, align = "left")
  }
  out
}

#' @export
#' @rdname char
set_char_opts <- function(x, ..., min_chars = NULL,
                          shorten = c("back", "front", "mid", "abbreviate")) {
  check_dots_empty()

  if (missing(shorten)) {
    shorten <- NULL
  } else if (!is.null(shorten)) {
    shorten <- arg_match(shorten)
  }

  pillar_attr <- structure(
    list(
      min_chars = min_chars,
      shorten = shorten
    ),
    class = c("pillar_char_attr", "pillar_vctr_attr", "tibble_vec_attr")
  )
  attr(x, "pillar") <- pillar_attr
  x
}

#' @export
format.pillar_char_attr <- function(x, ...) {
  out <- "pillar_char"

  min_chars <- x$min_chars
  if (!is.null(min_chars)) {
    out <- paste0(out, "(", min_chars, ")")
  }

  shorten <- x$shorten
  if (!is.null(shorten)) {
    extra <- switch(shorten,
      back = ">",
      front = "<",
      mid = "|",
      never = "=",
      abbr = "&"
    )

    out <- paste0(out, extra)
  }

  out
}

#' @export
vec_ptype2.pillar_char.pillar_char <- function(x, y, ...) {
  x
}
#' @export
vec_ptype2.pillar_char.character <- function(x, y, ...) {
  x
}
#' @export
vec_ptype2.character.pillar_char <- function(x, y, ...) {
  y
}

#' @export
vec_cast.pillar_char.pillar_char <- function(x, to, ...) {
  pillar_x <- attr(x, "pillar", exact = TRUE)
  pillar_to <- attr(to, "pillar", exact = TRUE)

  pillar_x_shorten <- pillar_x$shorten
  pillar_to_shorten <- pillar_to$shorten

  if (!is.null(pillar_x_shorten) && !is.null(pillar_to_shorten)) {
    if (!identical(pillar_x$shorten, pillar_to$shorten)) {
      abort("Only `pillar_char` objects with the same shortening setting can be combined.")
    }
  }

  pillar_x_min_chars <- pillar_x$min_chars
  pillar_to_min_chars <- pillar_to$min_chars

  if (!is.null(pillar_x_min_chars) && !is.null(pillar_to_min_chars)) {
    pillar_to$min_chars <- max(pillar_x_min_chars, pillar_to_min_chars)
  }

  attr(x, "pillar") <- pillar_to
  x
}
#' @export
vec_cast.character.pillar_char <- function(x, to, ...) {
  vec_data(x)
}
#' @export
vec_cast.pillar_char.character <- function(x, to, ...) {
  vec_restore(vec_proxy(x), to)
}

#' @export
type_sum.vctrs_unspecified <- function(x) {
  # FIXME: Remove when vec_ptype_abbr.vctrs_unspecified() becomes available
  "???"
}

#' @export
pillar_shaft.vctrs_vctr <- function(x, ...) {
  align <- if (is_character(x)) "left" else "right"
  new_pillar_shaft_simple(format(x), align = align)
}

#' @export
type_sum.vctrs_vctr <- function(x) {
  vctrs::vec_ptype_abbr(x)
}

#' @export
pillar_shaft.vctrs_list_of <- function(x, ...) {
  out <- style_subtle(paste0("[", map_chr(x, pillar::dim_desc), "]"))

  new_pillar_shaft_simple(out, align = "right")
}

#' Package options
#'
#' Options that affect display of tibble-like output.
#'
#' These options can be set via [options()] and queried via [getOption()].
#'
#' @usage NULL
#' @format NULL
#'
#' @examplesIf rlang::is_installed("tibble")
#' df <- tibble::tibble(x = c(1.234567, NA, 5:10))
#' df
#'
#' # Change for the duration of the session:
#' old <- options(
#'   pillar.sigfig = 6,
#'   pillar.print_max = 5,
#'   pillar.print_min = 5,
#'   pillar.advice = FALSE
#' )
#' df
#'
#' # Change back to the original value:
#' options(old)
#' df
#'
#' @section Options for the pillar package:
#'
#' - `width`: The width option controls the output width.
#'   Setting `options(pillar.width = )` to a larger value
#'   will lead to printing in multiple tiers (stacks).
#'
pillar_options <- list2(
  #' - `pillar.print_max`: Maximum number of rows printed, default: `20`.
  #'   Set to \code{Inf} to always print all rows.
  #'   For compatibility reasons, `getOption("tibble.print_max")` and
  #'   `getOption("dplyr.print_max")` are also consulted,
  #'   this will be soft-deprecated in pillar v2.0.0.
  print_max = make_option_impl(
    getOption("pillar.print_max", default = tibble_opt("print_max", 20L))
  ),
  #' - `pillar.print_min`: Number of rows printed if the table has more than
  #'   `print_max` rows, default: `10`.
  #'   For compatibility reasons, `getOption("tibble.print_min")` and
  #'   `getOption("dplyr.print_min")` are also consulted,
  #'   this will be soft-deprecated in pillar v2.0.0.
  print_min = make_option_impl(
    getOption("pillar.print_min", default = tibble_opt("print_min", 10L))
  ),
  #' - `pillar.width`: Output width. Default: `NULL`
  #'   (use `getOption("width")`).
  #'   This can be larger than `getOption("width")`, in this case the output
  #'   of the table's body is distributed over multiple tiers for wide tibbles.
  #'   For compatibility reasons, `getOption("tibble.width")` and
  #'   `getOption("dplyr.width")` are also consulted,
  #'   this will be soft-deprecated in pillar v2.0.0.
  width = make_option_impl(
    getOption("pillar.width", default = tibble_opt("width", getOption("width")))
  ),
  #' - `pillar.max_footer_lines`: The maximum number of lines in the footer,
  #'     default: `7`. Set to `Inf` to turn off truncation of footer lines.
  #'     The `max_extra_cols` option still limits
  #'     the number of columns printed.
  max_footer_lines = make_option_impl(
    getOption("pillar.max_footer_lines", default = 7L)
  ),
  #' - `pillar.max_extra_cols`: The maximum number of columns printed in the footer,
  #'     default: `100`. Set to `Inf` to show all columns.
  #'     Set the more predictable `max_footer_lines` to control the number
  #'     of footer lines instead.
  max_extra_cols = make_option_impl(
    getOption("pillar.max_extra_cols", default = tibble_opt("max_extra_cols", 100L))
  ),
  #' - `pillar.bold`: Use bold font, e.g. for column headers? This currently
  #'     defaults to `FALSE`, because many terminal fonts have poor support for
  #'     bold fonts.
  bold = make_option_impl(
    getOption("pillar.bold", default = FALSE)
  ),
  #' - `pillar.subtle`: Use subtle style, e.g. for row numbers and data types?
  #'     Default: `TRUE`.
  subtle = make_option_impl(
    getOption("pillar.subtle", default = TRUE)
  ),
  #' - `pillar.subtle_num`: Use subtle style for insignificant digits? Default:
  #'     `FALSE`, is also affected by the `subtle` option.
  subtle_num = make_option_impl(
    getOption("pillar.subtle_num", default = FALSE)
  ),
  #' - `pillar.neg`: Highlight negative numbers? Default: `TRUE`.
  neg = make_option_impl(
    getOption("pillar.neg", default = TRUE)
  ),
  #' - `pillar.sigfig`: The number of significant digits that will be printed and
  #'     highlighted, default: `3`. Set the `subtle` option to `FALSE` to
  #'     turn off highlighting of significant digits.
  sigfig = make_option_impl(option_name = "pillar.sigfig", {
    sigfig <- getOption("pillar.sigfig", default = 3L)
    if (!is.numeric(sigfig) || length(sigfig) != 1 || sigfig < 1L) {
      inform("Option pillar.sigfig must be a positive number greater or equal 1. Resetting to 3.")
      sigfig <- 3L
      set_pillar_option_sigfig(sigfig)
    }
    sigfig
  }),
  #' - `pillar.min_title_chars`: The minimum number of characters for the column
  #'     title, default: `20`.  Column titles may be truncated up to that width to
  #'     save horizontal space. Set to `Inf` to turn off truncation of column
  #'     titles.
  min_title_chars = make_option_impl(
    getOption("pillar.min_title_chars", default = 20L)
  ),
  #' - `pillar.min_chars`: The minimum number of characters wide to
  #'     display character columns, default: `3`.  Character columns may be
  #'     truncated up to that width to save horizontal space. Set to `Inf` to
  #'     turn off truncation of character columns.
  min_chars = make_option_impl(option_name = "pillar.min_chars", {
    min_chars <- getOption("pillar.min_chars", default = 3L)
    if (!is.numeric(min_chars) || length(min_chars) != 1 || min_chars < 3L) {
      inform("Option pillar.min_chars must be a nonnegative number greater or equal 3. Resetting to 3.")
      min_chars <- 3L
      set_pillar_option_min_chars(min_chars)
    }
    min_chars
  }),
  #' - `pillar.max_dec_width`: The maximum allowed width for decimal notation,
  #'     default: `13`.
  max_dec_width = make_option_impl(
    getOption("pillar.max_dec_width", default = 13L)
  ),
  #' - `pillar.bidi`: Set to `TRUE` for experimental support for bidirectional scripts.
  #'     Default: `FALSE`. When this option is set, "left right override"
  #'     and "first strong isolate"
  #'     [Unicode controls](https://www.w3.org/International/questions/qa-bidi-unicode-controls)
  #'     are inserted to ensure that text appears in its intended direction
  #'     and that the column headings correspond to the correct columns.
  bidi = make_option_impl(
    getOption("pillar.bidi", default = FALSE)
  ),
  #' - `pillar.superdigit_sep`: The string inserted between superscript digits
  #'   and column names in the footnote. Defaults to a `"\u200b"`, a zero-width
  #'   space, on UTF-8 platforms, and to `": "` on non-UTF-8 platforms.
  superdigit_sep = make_option_impl(
    getOption("pillar.superdigit_sep", default = superdigit_sep_default())
  ),
  #' - `pillar.advice`: Should advice be displayed in the footer when columns or rows
  #'   are missing from the output? Defaults to `TRUE` for interactive sessions,
  #'   and to `FALSE` otherwise.
  advice = make_option_impl(
    getOption("pillar.advice", default = is_interactive())
  ),
)

tibble_opt <- function(x, default) {
  x_tibble <- paste0("tibble.", x)
  res <- getOption(x_tibble)
  if (!is.null(res)) {
    return(res)
  }

  x_dplyr <- paste0("dplyr.", x)
  res <- getOption(x_dplyr)
  if (!is.null(res)) {
    return(res)
  }

  default
}

get_width_print <- function(width) {
  if (!is.null(width)) {
    return(width)
  }

  get_pillar_option_width()
}

get_width_glimpse <- function(width) {
  width <- get_width_print(width)

  if (is.finite(width)) {
    width
  } else {
    getOption("width")
  }
}

get_n_print <- function(n, rows) {
  if (!is.null(n) && n >= 0) {
    return(n)
  }

  if (is.na(rows) || rows > get_pillar_option_print_max()) {
    get_pillar_option_print_min()
  } else {
    rows
  }
}

get_max_extra_cols <- function(max_extra_cols) {
  if (!is.null(max_extra_cols) && max_extra_cols >= 0) {
    return(max_extra_cols)
  }

  get_pillar_option_max_extra_cols()
}

get_max_footer_lines <- function(max_footer_lines) {
  if (!is.null(max_footer_lines) && max_footer_lines >= 0) {
    return(max_footer_lines)
  }

  get_pillar_option_max_footer_lines()
}

test_that("print() returns output invisibly", {
  expect_output(ret <- withVisible(print(as_tbl(trees))))
  expect_false(ret$visible)
  expect_identical(ret$value, as_tbl(trees))
})

test_that("format() reacts on cli.num_colors option", {
  local_colors()

  expect_snapshot({
    format(as_tbl(trees))
    options(cli.num_colors = 1)
    format(as_tbl(trees))
  })
})

test_that("print() output", {
  expect_snapshot({
    as_tbl(mtcars)

    print(as_tbl(mtcars), n = 8, width = 30)

    print(as_tbl(mtcars), n = 30)

    print(as_tbl(mtcars), n = 100)

    print(as_tbl(mtcars), width = 40, max_extra_cols = 1)

    print(as_tbl(mtcars), width = 30, max_footer_lines = 3)

    rlang::with_options(
      tibble.print_min = 5,
      as_tbl(mtcars)
    )

    rlang::with_options(
      tibble.print_max = 50,
      as_tbl(mtcars)
    )

    rlang::with_options(
      tibble.width = 30,
      as_tbl(mtcars)
    )
  })

  expect_snapshot({
    print(as_tbl(mtcars), width = 40, max_extra_cols = 1)

    print(tbl_format_setup(new_tbl(trees, pillar_focus = "Volume"), width = 30))
  })
})

test_that("get_n_print()", {
  local_options(tibble.print_min = 3, tibble.print_max = 6)

  expect_equal(get_n_print(5, 2), 5)
  expect_equal(get_n_print(5, 3), 5)
  expect_equal(get_n_print(5, 5), 5)
  expect_equal(get_n_print(5, 6), 5)
  expect_equal(get_n_print(5, 7), 5)

  expect_equal(get_n_print(Inf, 7), Inf)

  expect_equal(get_n_print(NULL, 2), 2)
  expect_equal(get_n_print(NULL, 3), 3)
  expect_equal(get_n_print(NULL, 5), 5)
  expect_equal(get_n_print(NULL, 6), 6)
  expect_equal(get_n_print(NULL, 7), 3)
})

test_that("get_width_print()", {
  local_options(tibble.width = 120)

  expect_equal(get_width_print(NULL), 120)
  expect_equal(get_width_print(40), 40)
  expect_equal(get_width_print(80), 80)
  expect_equal(get_width_print(140), 140)
})

test_that("print() returns output invisibly", {
  expect_output(ret <- withVisible(print(as_tbl(trees))))
  expect_false(ret$visible)
  expect_identical(ret$value, as_tbl(trees))
})

test_that("format() reacts on cli.num_colors option", {
  local_colors()

  expect_snapshot({
    format(as_tbl(trees))
    options(cli.num_colors = 1)
    format(as_tbl(trees))
  })
})

test_that("print() output", {
  expect_snapshot({
    as_tbl(mtcars)

    print(as_tbl(mtcars), n = 8, width = 30)

    print(as_tbl(mtcars), n = 30)

    print(as_tbl(mtcars), n = 100)

    print(as_tbl(mtcars), width = 40, max_extra_cols = 1)

    print(as_tbl(mtcars), width = 30, max_footer_lines = 3)

    rlang::with_options(
      tibble.print_min = 5,
      as_tbl(mtcars)
    )

    rlang::with_options(
      tibble.print_max = 50,
      as_tbl(mtcars)
    )

    rlang::with_options(
      tibble.width = 30,
      as_tbl(mtcars)
    )
  })

  expect_snapshot({
    print(as_tbl(mtcars), width = 40, n_extra = 1)

    print(tbl_format_setup(new_tbl(trees, pillar_focus = "Volume"), width = 30))
  })
})

test_that("get_n_print()", {
  local_options(tibble.print_min = 3, tibble.print_max = 6)

  expect_equal(get_n_print(5, 2), 5)
  expect_equal(get_n_print(5, 3), 5)
  expect_equal(get_n_print(5, 5), 5)
  expect_equal(get_n_print(5, 6), 5)
  expect_equal(get_n_print(5, 7), 5)

  expect_equal(get_n_print(Inf, 7), Inf)

  expect_equal(get_n_print(NULL, 2), 2)
  expect_equal(get_n_print(NULL, 3), 3)
  expect_equal(get_n_print(NULL, 5), 5)
  expect_equal(get_n_print(NULL, 6), 6)
  expect_equal(get_n_print(NULL, 7), 3)
})

test_that("get_width_print()", {
  local_options(tibble.width = 120)

  expect_equal(get_width_print(NULL), 120)
  expect_equal(get_width_print(40), 40)
  expect_equal(get_width_print(80), 80)
  expect_equal(get_width_print(140), 140)
})

rowidformat <- function(n, has_title_row = FALSE, has_star = FALSE, ...) {
  capital <- rif_capital(has_title_row, has_star, ...)
  shaft <- rif_shaft(n, ...)
  new_pillar_1e(capital, shaft)
}

new_pillar_1e <- function(capital, shaft, width = NULL) {
  ret <- structure(
    list(capital = capital, shaft = shaft),
    class = "pillar_1e"
  )
  ret <- set_width(ret, width)
  ret
}

#' @export
format.pillar_1e <- function(x, width = NULL, ...) {
  width <- pillar_get_width_1e(x, width)
  out <- pillar_format_parts(x, width)

  as_glue(unlist(unname(out)))
}

#' @export
print.pillar_1e <- function(x, ...) {
  writeLines(style_bold("<pillar>"))
  print(format(x, ...))
}

pillar_get_width_1e <- function(x, width) {
  if (is.null(width)) {
    width <- get_width(x)
  }

  if (is.null(width)) {
    widths <- get_widths(x)
    width <- max(widths)
  }

  min_widths <- max(get_min_widths(x))
  if (width < min_widths) width <- min_widths

  width
}

pillar_format_parts <- function(x, width, ...) {
  capital_format <- format(x$capital, width = width, ...)
  shaft_format <- format(x$shaft, width = width, ...)
  align <- attr(shaft_format, "align", exact = TRUE)

  capital_format <- align(capital_format, width = width, align = align)
  shaft_format <- align(shaft_format, width = width, align = align)

  list(
    capital_format = capital_format,
    shaft_format = shaft_format
  )
}

format_abbrev <- function(x, title = NULL, space = " ") {
  type_format <- fansi::strip_ctl(format_full_pillar_type(x))
  if (is.null(title)) {
    type_format
  } else {
    title_format <- format_full_pillar_title(title)
    paste0(title_format, space, type_format)
  }
}

format_full_pillar_title <- function(title) {
  # Backticks already added here
  title <- format_title(title, Inf)
  style_title(title)
}

# nocov start - zzz.R

#' @import rlang
#' @import lifecycle
#' @importFrom glue as_glue
#' @importFrom vctrs data_frame
#' @importFrom vctrs new_data_frame
#' @importFrom vctrs new_list_of
#' @importFrom vctrs new_vctr
#' @importFrom vctrs obj_print_footer
#' @importFrom vctrs obj_print_header
#' @importFrom vctrs obj_print_data
#' @importFrom vctrs unspecified
#' @importFrom vctrs vec_arith
#' @importFrom vctrs vec_arith.numeric
#' @importFrom vctrs vec_arith_base
#' @importFrom vctrs vec_assert
#' @importFrom vctrs vec_c
#' @importFrom vctrs vec_cast
#' @importFrom vctrs vec_cast.double
#' @importFrom vctrs vec_cast.integer
#' @importFrom vctrs vec_data
#' @importFrom vctrs vec_is
#' @importFrom vctrs vec_math
#' @importFrom vctrs vec_math_base
#' @importFrom vctrs vec_proxy
#' @importFrom vctrs vec_proxy_compare
#' @importFrom vctrs vec_proxy_order
#' @importFrom vctrs vec_ptype_abbr
#' @importFrom vctrs vec_ptype_full
#' @importFrom vctrs vec_ptype2
#' @importFrom vctrs vec_rbind
#' @importFrom vctrs vec_restore
#' @importFrom vctrs vec_size
#' @importFrom vctrs vec_slice
#' @importFrom cli symbol ansi_strwrap
NULL

# nolint start
.onLoad <- function(libname, pkgname) {
  # nolint end
  # Can't use vctrs::s3_register() here with vctrs 0.1.0
  # https://github.com/r-lib/vctrs/pull/314
  s3_register("scales::rescale", "pillar_num")
  s3_register("ggplot2::scale_type", "pillar_num")
  s3_register("knitr::knit_print", "pillar_squeezed_colonnade")

  assign_crayon_styles()

  if (getRversion() < "3.3.0") {
    strrep <<- strrep_compat
  } else {
    rm("strrep", inherits = TRUE)
  }

  # Necessary to re-parse environment variable
  if (Sys.getenv("DEBUGME") != "" && rlang::is_installed("debugme")) {
    # activate_debugme()
    debugme::debugme()
    debug_info()
  }

  invisible()
}

activate_debugme <- function(level = 2) {
  old_debugme <- remove_from_logging(get_debugme())
  old_debugme <- gsub("(.)$", "\\1,", old_debugme)

  my_debugme <- paste0(strrep("!", level), get_pkgname())

  set_debugme(paste0(old_debugme, my_debugme))
}

deactivate_debugme <- function() {
  new_debugme <- remove_from_logging(get_debugme())
  set_debugme(new_debugme)
}

get_debugme <- function() {
  Sys.getenv("DEBUGME")
}

set_debugme <- function(debugme) {
  Sys.setenv("DEBUGME" = debugme)
  message("DEBUGME=", debugme)
}

remove_from_logging <- function(spec) {
  spec <- gsub(paste0("!*", get_pkgname(), ""), "", spec)
  spec <- gsub(",,+", ",", spec)
  spec
}

debug_info <- function(pkgname) {
  "!DEBUG Loaded"
  "!!DEBUG Level 2"
  "!!!DEBUG Level 3"
  "!!!!DEBUG Level 4"
  "!!!!!DEBUG Level 5"
  "!!!!!!DEBUG Level 6"
}

get_pkgname <- function() {
  environmentName(topenv(environment()))
}
# nocov end

#' Helper to define the contents of a pillar
#'
#' This function is useful if your data renders differently depending on the
#' available width. In this case, implement the [pillar_shaft()] method for your
#' class to return a subclass of "pillar_shaft" and have the [format()] method
#' for this subclass call `new_ornament()`. See the implementation of
#' `pillar_shaft.numeric()` and `format.pillar_shaft_decimal()` for an example.
#'
#' @param x A character vector with formatting,
#'   can use ANYI styles e.g provided by the \pkg{cli} package.
#'
#' @param width An optional width of the resulting pillar, computed from `x` if
#'   missing
#' @param align Alignment, one of `"left"` or `"right"`
#'
#' @export
#' @examples
#' new_ornament(c("abc", "de"), align = "right")
new_ornament <- function(x, width = NULL, align = NULL) {
  if (is.null(width)) {
    width <- get_max_extent(x)
  }

  ret <- structure(
    x,
    align = align,
    class = "pillar_ornament"
  )
  ret <- set_width(ret, width)
  ret
}

#' @export
print.pillar_ornament <- function(x, ...) {
  writeLines(style_bold("<pillar_ornament>"))
  writeLines(format(x, ...))
  invisible(x)
}

#' @export
format.pillar_ornament <- function(x, width = NULL, ...) {
  align <- attr(x, "align", exact = TRUE)
  align(x, width = width %||% get_width(x), align = align)
}

test_that("ornament", {
  expect_snapshot({
    new_ornament(c("abc", "de"))
    new_ornament(c("abc", "de"), align = "right")
    new_ornament(c("abc", "de"), width = 5, align = "right")
    format(new_ornament(c("abc", "de")))
    format(new_ornament(c("abc", "de")), width = 5)
  })
})

test_that("vertical", {
  expect_snapshot({
    as_glue(c("abc", "de"))
  })
})

#' Helper to define the contents of a pillar
#'
#' This function is useful if your data renders differently depending on the
#' available width. In this case, implement the [pillar_shaft()] method for your
#' class to return a subclass of "pillar_shaft" and have the [format()] method
#' for this subclass call `new_ornament()`. See the implementation of
#' `pillar_shaft.numeric()` and `format.pillar_shaft_decimal()` for an example.
#'
#' @param x A character vector with formatting, see [crayon]
#' @param width An optional width of the resulting pillar, computed from `x` if
#'   missing
#' @param align Alignment, one of `"left"` or `"right"`
#'
#' @export
#' @examples
#' new_ornament(c("abc", "de"), align = "right")
new_ornament <- function(x, width = NULL, align = NULL) {
  if (is.null(width)) width <- get_max_extent(x)
  ret <- structure(
    x,
    align = align,
    class = "pillar_ornament"
  )
  ret <- set_width(ret, width)
  ret
}

#' @export
print.pillar_ornament <- function(x, ...) {
  if (length(x) > 0) {
    cat_line(paste(
      align(x, width = get_width(x), align = attr(x, "align")),
      collapse = "\n"
    ))
  }
  invisible(x)
}

new_vertical <- function(x, ..., class = NULL) {
  ret <- structure(
    x,
    ...,
    class = c(class, "pillar_vertical")
  )
  ret
}

#' @export
print.pillar_vertical <- function(x, ...) {
  if (length(x) > 0) {
    cat_line(paste(x, collapse = "\n"))
  }
  invisible(x)
}

style_title <- style_bold

#' Prepare a column title for formatting
#'
#' Call [format()] on the result to render column titles.
#'
#' @param x A character vector of column titles.
#' @inheritParams rlang::args_dots_empty
#' @export
#' @examples
#' format(new_pillar_title(names(trees)))
new_pillar_title <- function(x, ...) {
  "!!!!DEBUG new_pillar_title(`v(x)`)"
  if (!missing(...)) {
    check_dots_empty(action = warn)
  }

  if (is.null(x)) {
    width <- 0L
  } else if (all(is.na(x) | x == "")) {
    width <- 0L
    x <- NULL
  } else {
    width <- get_max_extent(format_title(x, width = Inf))
    stopifnot(!is.na(width))
  }

  ret <- structure(list(x), class = "pillar_title")

  ret <- set_width(ret, width)
  ret <- set_min_width(ret, min(width, get_min_title_width(width)))
  ret
}

get_min_title_width <- function(width) {
  title_chars <- get_pillar_option_min_title_chars()
  if (!is.numeric(title_chars) || length(title_chars) != 1 || title_chars < 0) {
    stop("Option pillar.min_title_chars must be a nonnegative number", call. = FALSE)
  }

  if (is.infinite(title_chars)) {
    return(width)
  }

  # We don't use the ellipsis if we don't truncate, a solution with min()
  # is difficult to make work in all corner cases (and slower too)
  if (width <= title_chars) {
    return(width)
  }
  title_chars + 2L
}

#' @export
format.pillar_title <- function(x, width = NULL, ..., footnote_idx = 0L) {
  title <- x[[1]]
  if (is.null(title)) {
    return(character())
  }

  if (is.null(width)) {
    width <- get_width(x)
  }

  footnote <- (footnote_idx > 0L)
  title <- format_title(title, width, footnote)
  if (footnote) {
    title <- gsub("\u02df$", superdigit(footnote_idx), title)
  }
  style_title(title)
}

#' Customize the appearance of simple pillars in your tibble subclass
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Gain full control over the appearance of the pillars of your tibble subclass
#' in its body.
#' This method is intended for implementers of subclasses of the `"tbl"` class.
#' Users will rarely need them.
#'
#' @details
#' `ctl_new_pillar()` is called to construct pillars for regular (one-dimensional)
#' vectors.
#' The default implementation returns an object constructed with [pillar()].
#' Extend this method to modify the pillar components returned from the default
#' implementation.
#' Override this method to completely change the appearance of the pillars.
#' Components are created with [new_pillar_component()] or [pillar_component()].
#' In order to customize printing of row IDs, a method can be supplied for the
#' `ctl_new_rowid_pillar()` generic.
#'
#' All components must be of the same height.
#' This restriction may be levied in the future.
#'
#' Implementations should return `NULL` if none of the data
#' fits the available width.
#'
#' @inheritParams rlang::args_dots_empty
#' @param controller The object of class `"tbl"` currently printed.
#' @param x A simple (one-dimensional) vector.
#' @param width The available width, can be a vector for multiple tiers.
#' @param title The title, derived from the name of the column in the data.
#'
#' @seealso
#' See [ctl_new_pillar_list()] for creating pillar objects for compound columns:
#' packed data frames, matrices, or arrays.
#'
#' @export
#' @examplesIf rlang::is_installed(c("palmerpenguins", "tibble"))
#' # Create pillar objects
#' ctl_new_pillar(
#'   palmerpenguins::penguins,
#'   palmerpenguins::penguins$species[1:3],
#'   width = 60
#' )
#'
#' ctl_new_pillar(
#'   palmerpenguins::penguins,
#'   palmerpenguins::penguins$bill_length_mm[1:3],
#'   width = 60
#' )
#' @examples
#'
#' # Customize output
#' lines <- function(char = "-") {
#'   stopifnot(nchar(char) == 1)
#'   structure(char, class = "lines")
#' }
#'
#' format.lines <- function(x, width, ...) {
#'   paste(rep(x, width), collapse = "")
#' }
#'
#' ctl_new_pillar.line_tbl <- function(controller, x, width, ...) {
#'   out <- NextMethod()
#'   new_pillar(list(
#'     title = out$title,
#'     type = out$type,
#'     lines = new_pillar_component(list(lines("=")), width = 1),
#'     data = out$data
#'   ))
#' }
#'
#' ctl_new_rowid_pillar.line_tbl <- function(controller, x, width, ...) {
#'   out <- NextMethod()
#'   new_pillar(
#'     list(
#'       title = out$title,
#'       type = out$type,
#'       lines = new_pillar_component(list(lines("=")), width = 1),
#'       data = out$data
#'     ),
#'     width = as.integer(floor(log10(max(nrow(x), 1))) + 1)
#'   )
#' }
#'
#' vctrs::new_data_frame(
#'   list(a = 1:3, b = letters[1:3]),
#'   class = c("line_tbl", "tbl")
#' )
#'
#' ctl_new_rowid_pillar.roman_tbl <- function(controller, x, width, ...) {
#'   out <- NextMethod()
#'   rowid <- utils::as.roman(seq_len(nrow(x)))
#'   width <- max(nchar(as.character(rowid)))
#'   new_pillar(
#'     list(
#'       title = out$title,
#'       type = out$type,
#'       data = pillar_component(
#'         new_pillar_shaft(list(row_ids = rowid),
#'           width = width,
#'           class = "pillar_rif_shaft"
#'         )
#'       )
#'     ),
#'     width = width
#'   )
#' }
#'
#' vctrs::new_data_frame(
#'   list(a = 1:3, b = letters[1:3]),
#'   class = c("roman_tbl", "tbl")
#' )
#'
ctl_new_pillar <- function(controller, x, width, ..., title = NULL) {
  "!!!!DEBUG ctl_new_pillar(`v(width)`, `v(title)`)"

  check_dots_empty()

  UseMethod("ctl_new_pillar")
}

#' @export
ctl_new_pillar.tbl <- function(controller, x, width, ..., title = NULL) {
  "!!!!DEBUG ctl_new_pillar.tbl(`v(width)`, `v(title)`)"

  pillar(x, title, if (!is.null(width)) max0(width))
}

#' @param type String for specifying a row ID type. Current values in use are
#' `NULL` and `"*"`.
#' @rdname ctl_new_pillar
#' @export
ctl_new_rowid_pillar <- function(controller, x, width, ..., title = NULL, type = NULL) {
  "!!!!DEBUG ctl_new_rowid_pillar(`v(width)`, `v(title)`)"

  check_dots_empty()

  if (length(width) == 0) {
    return(NULL)
  }

  UseMethod("ctl_new_rowid_pillar")
}

#' @export
ctl_new_rowid_pillar.tbl <- function(controller, x, width, ..., title = NULL, type = NULL) {
  "!!!!DEBUG ctl_new_rowid_pillar.tbl(`v(width)`, `v(title)`)"

  template <- names(ctl_new_pillar(controller, vector(), width, title = title))

  if (!length(template)) {
    return(NULL)
  }

  data <- rif_shaft(nrow(x))

  out <- map(set_names(template), function(.x) "")

  if ("type" %in% template) {
    out$type <- pillar_component(rif_type(identical(type, "*")))
  }

  if ("data" %in% template) {
    out$data <- pillar_component(data)
  }

  new_pillar(out, width = get_width(data))
}

max0 <- function(x) {
  if (length(x) > 0) {
    max(x)
  } else {
    0L
  }
}

test_that("ctl_new_pillar()", {
  expect_snapshot({
    ctl_new_pillar(new_tbl(), 1:3, width = 20, title = "a")
  })
})

test_that("ctl_new_pillar_list()", {
  expect_snapshot({
    ctl_new_pillar_list(new_tbl(), 1:3, width = 20, title = "a")
    ctl_new_pillar_list(new_tbl(), trees[1:3, ], width = 8, title = "a")
    ctl_new_pillar_list(new_tbl(), trees[1:3, ], width = 20, title = "a")
    ctl_new_pillar_list(new_tbl(), as.matrix(trees[1:3, ]), width = 8, title = "a")
    ctl_new_pillar_list(new_tbl(), as.matrix(trees[1:3, ]), width = 20, title = "a")
    ctl_new_pillar_list(new_tbl(), matrix(1:6, ncol = 2), width = 8, title = "a")
    ctl_new_pillar_list(new_tbl(), matrix(1:6, ncol = 2), width = 20, title = "a")
    ctl_new_pillar_list(new_tbl(), matrix(1:6, ncol = 3), width = 10, title = "a")
  })
})

test_that("ctl_new_pillar_list() for tables", {
  skip_if_not_installed("vctrs", "0.3.6.9000")
  expect_snapshot({
    ctl_new_pillar_list(new_tbl(), Titanic, width = 20, title = "a")
  })
})

pillar_capital <- function(x, title = NULL) {
  title <- new_pillar_title(title)
  type <- new_pillar_type(x)
  new_pillar_capital(title, type)
}

new_pillar_capital <- function(title, type) {
  ret <- structure(
    list(
      title = title,
      type = type
    ),
    class = "pillar_capital"
  )

  ret <- set_width(ret, max(get_width(title), get_width(type)))
  ret <- set_min_width(ret, max(get_min_width(title), get_min_width(type)))
  ret
}

#' @export
format.pillar_capital <- function(x, width, ...) {
  c(
    format(x$title, width = width, ...),
    format(x$type, width = width, ...)
  )
}

# Draw a sparkline bar graph with unicode block characters
#
# Rendered using [block elements](https://en.wikipedia.org/wiki/Block_Elements).
# In most common fixed width fonts these are rendered wider than regular
# characters which means they are not suitable if you need precise alignment.
#
# @param x A numeric vector between 0 and 1
# @param safe Nominally there are 8 block elements from 1/8 height to full
#   height (8/8). However, the half-height and full-height blocks appear
#   to be rendered inconsistently (possibly due to font substitution).
# @examples
# \dontrun{
# x <- seq(0, 1, length = 6)
# spark_bar(x)
# spark_bar(sample(x))
#
# # This might work if you're lucky
# spark_bar(seq(0, 1, length = 8), safe = FALSE)
#
# spark_bar(c(0, NA, 0.5, NA, 1))
# }
spark_bar <- function(x, safe = TRUE) {
  stopifnot(is.numeric(x))

  bars <- vapply(0x2581:0x2588, intToUtf8, character(1))
  if (safe) {
    bars <- bars[-c(4, 8)]
  }

  factor <- cut(
    x,
    breaks = seq(0, 1, length = length(bars) + 1),
    labels = bars,
    include.lowest = TRUE
  )
  chars <- as.character(factor)
  chars[is.na(chars)] <- style_spark_na(bars[length(bars)])

  new_vertical(paste0(chars, collapse = ""), class = c("pillar_spark_bar", "pillar_spark"))
}

#' Package options
#'
#' Options that affect display of tibble-like output.
#'
#' These options can be set via [options()] and queried via [getOption()].
#' For this, add a `pillar.` prefix (the package name and a dot) to the option name.
#' Example: for an option `foo`, use `options(pillar.foo = value)` to set it
#' and `getOption("pillar.foo")` to retrieve the current value.
#' An option value of `NULL` means that the default is used.
#'
#' @format NULL
#'
#' @examples
#' # Default setting:
#' getOption("pillar.sigfig")
#' pillar(1.234567)
#'
#' # Change for the duration of the session:
#' old <- options(pillar.sigfig = 6)
#' pillar(1.234567)
#'
#' # Change back to the original value:
#' options(old)
#' pillar(1.234567)
#'
#' # Local scope:
#' local({
#'   rlang::local_options(pillar.sigfig = 6)
#'   pillar(1.234567)
#' })
#' pillar(1.234567)
#' @section Options for the pillar package:
pillar_options <- list2(
  #' - `print_max`: Maximum number of rows printed, default: `20`.
  #'   Set to \code{Inf} to always print all rows.
  #'   For compatibility reasons, `getOption("tibble.print_max")` and
  #'   `getOption("dplyr.print_max")` are also consulted,
  #'   this will be soft-deprecated in pillar v2.0.0.
  print_max = make_option_impl(
    getOption("pillar.print_max", default = tibble_opt("print_max", 20L))
  ),
  #' - `print_min`: Number of rows printed if the table has more than
  #'   `print_max` rows, default: `10`.
  #'   For compatibility reasons, `getOption("tibble.print_min")` and
  #'   `getOption("dplyr.print_min")` are also consulted,
  #'   this will be soft-deprecated in pillar v2.0.0.
  print_min = make_option_impl(
    getOption("pillar.print_min", default = tibble_opt("print_min", 10L))
  ),
  #' - `width`: Output width. Default: `NULL`
  #'   (use `getOption("width")`).
  #'   This can be larger than `getOption("width")`, in this case the output
  #'   of the table's body is distributed over multiple tiers for wide tibbles.
  #'   For compatibility reasons, `getOption("tibble.width")` and
  #'   `getOption("dplyr.width")` are also consulted,
  #'   this will be soft-deprecated in pillar v2.0.0.
  width = make_option_impl(
    getOption("pillar.width", default = tibble_opt("width", getOption("width")))
  ),
  #' - `max_footer_lines`: The maximum number of lines in the footer,
  #'     default: `7`. Set to `Inf` to turn off truncation of footer lines.
  #'     The `max_extra_cols` option still limits
  #'     the number of columns printed.
  max_footer_lines = make_option_impl(
    getOption("pillar.max_footer_lines", default = 7L)
  ),
  #' - `max_extra_cols`: The maximum number of columns printed in the footer,
  #'     default: `100`. Set to `Inf` to show all columns.
  #'     Set the more predictable `max_footer_lines` to control the number
  #'     of footer lines instead.
  max_extra_cols = make_option_impl(
    getOption("pillar.max_extra_cols", default = tibble_opt("max_extra_cols", 100L))
  ),
  #' - `bold`: Use bold font, e.g. for column headers? This currently
  #'     defaults to `FALSE`, because many terminal fonts have poor support for
  #'     bold fonts.
  bold = make_option_impl(
    getOption("pillar.bold", default = FALSE)
  ),
  #' - `subtle`: Use subtle style, e.g. for row numbers and data types?
  #'     Default: `TRUE`.
  subtle = make_option_impl(
    getOption("pillar.subtle", default = TRUE)
  ),
  #' - `subtle_num`: Use subtle style for insignificant digits? Default:
  #'     `FALSE`, is also affected by the `subtle` option.
  subtle_num = make_option_impl(
    getOption("pillar.subtle_num", default = FALSE)
  ),
  #' - `neg`: Highlight negative numbers? Default: `TRUE`.
  neg = make_option_impl(
    getOption("pillar.neg", default = TRUE)
  ),
  #' - `sigfig`: The number of significant digits that will be printed and
  #'     highlighted, default: `3`. Set the `subtle` option to `FALSE` to
  #'     turn off highlighting of significant digits.
  sigfig = make_option_impl(option_name = "pillar.sigfig", {
    sigfig <- getOption("pillar.sigfig", default = 3L)
    if (!is.numeric(sigfig) || length(sigfig) != 1 || sigfig < 1L) {
      inform("Option pillar.sigfig must be a positive number greater or equal 1. Resetting to 3.")
      sigfig <- 3L
      set_pillar_option_sigfig(sigfig)
    }
    sigfig
  }),
  #' - `min_title_chars`: The minimum number of characters for the column
  #'     title, default: `3`.  Column titles may be truncated up to that width to
  #'     save horizontal space. Set to `Inf` to turn off truncation of column
  #'     titles.
  min_title_chars = make_option_impl(
    getOption("pillar.min_title_chars", default = 5L)
  ),
  #' - `min_chars`: The minimum number of characters wide to
  #'     display character columns, default: `5`.  Character columns may be
  #'     truncated up to that width to save horizontal space. Set to `Inf` to
  #'     turn off truncation of character columns.
  min_chars = make_option_impl(option_name = "pillar.min_chars", {
    min_chars <- getOption("pillar.min_chars", default = 3L)
    if (!is.numeric(min_chars) || length(min_chars) != 1 || min_chars < 3L) {
      inform("Option pillar.min_chars must be a nonnegative number greater or equal 3. Resetting to 3.")
      min_chars <- 3L
      set_pillar_option_min_chars(min_chars)
    }
    min_chars
  }),
  #' - `max_dec_width`: The maximum allowed width for decimal notation,
  #'     default: `13`.
  max_dec_width = make_option_impl(
    getOption("pillar.max_dec_width", default = 13L)
  ),
  #' - `bidi`: Set to `TRUE` for experimental support for bidirectional scripts.
  #'     Default: `FALSE`. When this option is set, "left right override"
  #'     and "first strong isolate"
  #'     [Unicode controls](https://www.w3.org/International/questions/qa-bidi-unicode-controls)
  #'     are inserted to ensure that text appears in its intended direction
  #'     and that the column headings correspond to the correct columns.
  bidi = make_option_impl(
    getOption("pillar.bidi", default = FALSE)
  ),
  #' - `superdigit_sep`: The string inserted between superscript digits
  #'   and column names in the footnote. Defaults to a `"\u200b"`, a zero-width
  #'   space, on UTF-8 platforms, and to `": "` on non-UTF-8 platforms.
  superdigit_sep = make_option_impl(
    getOption("pillar.superdigit_sep", default = superdigit_sep_default())
  ),
)

tibble_opt <- function(x, default) {
  x_tibble <- paste0("tibble.", x)
  res <- getOption(x_tibble)
  if (!is.null(res)) {
    return(res)
  }

  x_dplyr <- paste0("dplyr.", x)
  res <- getOption(x_dplyr)
  if (!is.null(res)) {
    return(res)
  }

  default
}

get_width_print <- function(width) {
  if (!is.null(width)) {
    return(width)
  }

  get_pillar_option_width()
}

get_width_glimpse <- function(width) {
  width <- get_width_print(width)

  if (is.finite(width)) {
    width
  } else {
    getOption("width")
  }
}

get_n_print <- function(n, rows) {
  if (!is.null(n) && n >= 0) {
    return(n)
  }

  if (is.na(rows) || rows > get_pillar_option_print_max()) {
    get_pillar_option_print_min()
  } else {
    rows
  }
}

get_max_extra_cols <- function(max_extra_cols) {
  if (!is.null(max_extra_cols) && max_extra_cols >= 0) {
    return(max_extra_cols)
  }

  get_pillar_option_max_extra_cols()
}

get_max_footer_lines <- function(max_footer_lines) {
  if (!is.null(max_footer_lines) && max_footer_lines >= 0) {
    return(max_footer_lines)
  }

  get_pillar_option_max_footer_lines()
}

# nocov start - zzz.R

#' @import rlang
#' @import lifecycle
#' @importFrom glue as_glue
#' @importFrom vctrs data_frame
#' @importFrom vctrs new_data_frame
#' @importFrom vctrs new_list_of
#' @importFrom vctrs new_vctr
#' @importFrom vctrs obj_print_footer
#' @importFrom vctrs obj_print_header
#' @importFrom vctrs obj_print_data
#' @importFrom vctrs unspecified
#' @importFrom vctrs vec_arith
#' @importFrom vctrs vec_arith.numeric
#' @importFrom vctrs vec_arith_base
#' @importFrom vctrs vec_assert
#' @importFrom vctrs vec_c
#' @importFrom vctrs vec_cast
#' @importFrom vctrs vec_cast.double
#' @importFrom vctrs vec_cast.integer
#' @importFrom vctrs vec_data
#' @importFrom vctrs vec_is
#' @importFrom vctrs vec_math
#' @importFrom vctrs vec_math_base
#' @importFrom vctrs vec_proxy
#' @importFrom vctrs vec_proxy_compare
#' @importFrom vctrs vec_proxy_order
#' @importFrom vctrs vec_ptype_abbr
#' @importFrom vctrs vec_ptype_full
#' @importFrom vctrs vec_ptype2
#' @importFrom vctrs vec_rbind
#' @importFrom vctrs vec_restore
#' @importFrom vctrs vec_size
#' @importFrom vctrs vec_slice
#' @importFrom cli symbol
NULL

# nolint start
.onLoad <- function(libname, pkgname) {
  # nolint end
  # Can't use vctrs::s3_register() here with vctrs 0.1.0
  # https://github.com/r-lib/vctrs/pull/314
  s3_register("scales::rescale", "pillar_num")
  s3_register("ggplot2::scale_type", "pillar_num")
  s3_register("knitr::knit_print", "pillar_squeezed_colonnade")

  assign_crayon_styles()

  if (getRversion() < "3.3.0") {
    strrep <<- strrep_compat
  } else {
    rm("strrep", inherits = TRUE)
  }

  # Necessary to re-parse environment variable
  if (Sys.getenv("DEBUGME") != "" && rlang::is_installed("debugme")) {
    # activate_debugme()
    debugme::debugme()
    debug_info()
  }

  # https://github.com/r-lib/pkgdown/issues/1540
  if (Sys.getenv("IN_PKGDOWN") != "") {
    s3_register("pillar::type_sum", "accel")
  }

  invisible()
}

activate_debugme <- function(level = 2) {
  old_debugme <- remove_from_logging(get_debugme())
  old_debugme <- gsub("(.)$", "\\1,", old_debugme)

  my_debugme <- paste0(strrep("!", level), get_pkgname())

  set_debugme(paste0(old_debugme, my_debugme))
}

deactivate_debugme <- function() {
  new_debugme <- remove_from_logging(get_debugme())
  set_debugme(new_debugme)
}

get_debugme <- function() {
  Sys.getenv("DEBUGME")
}

set_debugme <- function(debugme) {
  Sys.setenv("DEBUGME" = debugme)
  message("DEBUGME=", debugme)
}

remove_from_logging <- function(spec) {
  spec <- gsub(paste0("!*", get_pkgname(), ""), "", spec)
  spec <- gsub(",,+", ",", spec)
  spec
}

debug_info <- function(pkgname) {
  "!DEBUG Loaded"
  "!!DEBUG Level 2"
  "!!!DEBUG Level 3"
  "!!!!DEBUG Level 4"
  "!!!!!DEBUG Level 5"
  "!!!!!!DEBUG Level 6"
}

get_pkgname <- function() {
  environmentName(topenv(environment()))
}
# nocov end

#' Customize the appearance of compound pillars in your tibble subclass
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Gain full control over the appearance of the pillars of your tibble subclass
#' in its body.
#' This method is intended for implementers of subclasses of the `"tbl"` class.
#' Users will rarely need them, and we also expect the default implementation
#' to be sufficient for the vast majority of cases.
#'
#' @details
#' `ctl_new_pillar_list()` is called to construct a list of pillars.
#' If `x` is a regular (one-dimensional) vector, the list contains one pillar
#' constructed by [ctl_new_pillar()].
#' This method also works for compound columns: columns that are data frames,
#' matrices or arrays, with the following behavior:
#'
#' - If `width` is `NULL`, the method always returns a list of length one
#'   containing one pillar object that represents the first sub-column in this
#'   compound column.
#' - Otherwise, the returned list contains one pillar object for all sub-columns
#'   that can be fit in the available horizontal space.
#'   These pillar objects are obtained by calling `ctl_new_pillar_list()`
#'   with `width = NULL` on each sub-column until the available width is
#'   exhausted.
#'
#' This method is called to initiate the construction of all pillars
#' in the tibble to be printed.
#' To ensure that all packed columns that fit the available space are printed,
#' `ctl_new_pillar_list()` may be called twice on the same input:
#' once with `width = NULL`, and
#' once with `width` corresponding to the then known available space
#' and with `first_pillar` set to the pillar object constructed in the
#' first call.
#'
#' @inheritParams ctl_new_pillar
#' @param x A vector, can also be a data frame, matrix, or array.
#' @param width The available width, can be a vector for multiple tiers.
#'   If `NULL`, only the first pillar is instantiated.
#' @param first_pillar Can be passed to this method if the first pillar
#'   for a compound pillar (or the pillar itself for a simple pillar)
#'   has been constructed already.
#' @export
#' @examplesIf rlang::is_installed(c("palmerpenguins", "tibble")) && requireNamespace("tibble")
#' # Simple column
#' ctl_new_pillar_list(
#'   tibble::tibble(),
#'   palmerpenguins::penguins$weight[1:3],
#'   width = 10
#' )
#'
#' # Packed data frame: unknown width
#' ctl_new_pillar_list(
#'   tibble::tibble(),
#'   palmerpenguins::penguins[1:3, ],
#'   width = NULL
#' )
#'
#' # Packed data frame: known width
#' ctl_new_pillar_list(
#'   tibble::tibble(),
#'   palmerpenguins::penguins,
#'   width = 60
#' )
#'
#' # Deeply packed data frame with known width:
#' # showing only the first sub-column even if the width is sufficient
#' ctl_new_pillar_list(
#'   tibble::tibble(),
#'   tibble::tibble(x = tibble::tibble(b = 1, c = 2), y = 3),
#'   width = 60
#' )
#'
#' # Packed matrix: unknown width
#' ctl_new_pillar_list(tibble::tibble(), matrix(1:6, ncol = 2), width = NULL)
#'
#' # Packed matrix: known width
#' ctl_new_pillar_list(tibble::tibble(), matrix(1:6, ncol = 2), width = 60)
#'
#' # Packed array
#' ctl_new_pillar_list(tibble::tibble(), Titanic, width = 60)
ctl_new_pillar_list <- function(controller, x, width, ..., title = NULL, first_pillar = NULL) {
  "!!!!DEBUG ctl_new_pillar_list(`v(width)`, `v(title)`)"

  check_dots_empty()

  UseMethod("ctl_new_pillar_list")
}

#' @export
ctl_new_pillar_list.tbl <- function(controller, x, width, ..., title = NULL, first_pillar = NULL) {
  "!!!!DEBUG ctl_new_pillar_list.tbl(`v(width)`, `v(title)`)"

  if (is.data.frame(x)) {
    new_data_frame_pillar_list(x, controller, width, title = title, first_pillar = first_pillar)
  } else if (is.matrix(x) && !inherits(x, c("Surv", "Surv2"))) {
    new_matrix_pillar_list(x, controller, width, title = title, first_pillar = first_pillar)
  } else if (is.array(x) && length(dim(x)) > 2) {
    new_array_pillar_list(x, controller, width, title = title, first_pillar = first_pillar)
  } else {
    if (is.null(first_pillar)) {
      first_pillar <- ctl_new_pillar(controller, x, width, ..., title = prepare_title(title))
    }
    new_single_pillar_list(first_pillar, width)
  }
}

new_data_frame_pillar_list <- function(x, controller, width, title, first_pillar = NULL) {
  "!!!!!DEBUG new_data_frame_pillar_list(`v(width)`, `v(title)`)"

  if (ncol(x) == 0) {
    pillar <- pillar_from_shaft(
      new_pillar_title(prepare_title(title)),
      new_pillar_type(x),
      new_empty_shaft(nrow(x)),
      width
    )
    return(new_single_pillar_list(pillar, width))
  }

  max_n_pillars <- sum(width %/% 2)
  pillars <- vector("list", max_n_pillars)

  ticked_names <- tick_if_needed(names(x))

  for (i in seq_along(x)) {
    "!!!!!DEBUG i = `i`, width = `width`"

    if (i == 1 && !is.null(first_pillar)) {
      pillar <- first_pillar
    } else {
      # Call ctl_new_pillar_list(), return only the first sub-pillar
      # thanks to width = NULL
      new_pillars <- ctl_new_pillar_list(
        controller, x[[i]],
        width = NULL,
        title = c(title, ticked_names[[i]])
      )

      # Safety check:
      if (length(new_pillars) == 0) {
        # NULL return: doesn't fit
        break
      }

      pillar <- new_pillars[[1]]
    }

    if (is.null(pillar)) {
      # NULL return: doesn't fit
      break
    }

    if (is.null(width)) {
      return(list(pillar))
    }

    # Compute remaining width
    new_width <- deduct_width(width, pillar_get_min_width(pillar))
    if (is.null(new_width)) {
      # NULL return: current pillar doesn't fit
      break
    }

    pillars[[i]] <- pillar
    width <- new_width

    if (!is.null(title)) {
      title[] <- ""
    }
  }

  pillars <- compact(pillars)

  new_pillar_list(
    pillars,
    extra = names(x)[seq2(length(pillars) + 1, length(x))],
    remaining_width = width
  )
}

new_matrix_pillar_list <- function(x, controller, width, title, first_pillar = NULL) {
  if (ncol(x) == 0) {
    pillar <- pillar_from_shaft(
      new_pillar_title(prepare_title(title)),
      new_pillar_type(x),
      new_empty_shaft(nrow(x)),
      width
    )
    return(new_single_pillar_list(pillar, width))
  }

  max_n_pillars <- sum(width %/% 2)
  pillars <- vector("list", max_n_pillars)

  idx <- colnames(x)
  if (is.null(idx)) {
    idx <- seq_len(ncol(x))
  } else {
    idx <- encodeString(idx, quote = '"')
  }

  ticked_names <- paste0("[,", idx, "]")

  for (i in seq_along(ticked_names)) {
    if (i == 1 && !is.null(first_pillar)) {
      pillar <- first_pillar
    } else {
      # FIXME
      # sub_title <- c(title, ticked_names[[i]])
      sub_title <- ticked_names[[i]]
      if (!is.null(title)) {
        if (i > 1) {
          title[[length(title)]] <- ""
        }
        sub_title <- c(title, sub_title)
      }

      # Call ctl_new_pillar() only for objects that can fit
      pillar <- ctl_new_pillar(
        controller, x[, i],
        width = NULL,
        title = prepare_title(sub_title)
      )
    }

    if (is.null(pillar)) {
      # NULL return: doesn't fit
      break
    }

    if (is.null(width)) {
      return(list(pillar))
    }

    # Compute remaining width
    new_width <- deduct_width(width, pillar_get_min_width(pillar))
    if (is.null(new_width)) {
      # NULL return: current pillar doesn't fit
      break
    }

    pillars[[i]] <- pillar
    width <- new_width
  }

  pillars <- compact(pillars)
  new_pillar_list(
    pillars,
    extra = seq2(length(pillars) + 1, ncol(x)),
    remaining_width = width
  )
}

new_array_pillar_list <- function(x, controller, width, title, first_pillar = NULL) {
  if (!is.null(first_pillar)) {
    return(list(first_pillar))
  }

  first_slice <- head(as.vector(x), vec_size(x))

  body <- pillar_shaft(first_slice)

  pillar <- pillar_from_shaft(
    new_pillar_title(title),
    new_pillar_type(x),
    new_continuation_shaft(body),
    width
  )
  new_single_pillar_list(pillar, width)
}

# Can be rewritten with a repeat loop
deduct_width <- function(width, consumed_widths) {
  "!!!!!DEBUG deduct_width(`v(width)`, `v(consumed_widths)`)"

  if (length(consumed_widths) == 0) {
    # All sub-pillars distributed
    width
  } else if (length(width) == 0) {
    # Doesn't fit
    NULL
  } else if (width[[1]] >= consumed_widths[[1]]) {
    # Fits first tier
    deduct_width(
      c(max(width[[1]] - consumed_widths[[1]] - 1L, 0), width[-1]),
      consumed_widths[-1]
    )
  } else {
    # Doesn't fit first tier: try next tier
    deduct_width(width[-1], consumed_widths)
  }
}

new_single_pillar_list <- function(pillar, width) {
  pillar_width <- pillar_get_min_width(pillar)
  if (is.null(width)) {
    new_width <- NULL
  } else {
    new_width <- deduct_width(width, pillar_width)
    stopifnot(!is.null(new_width))
  }
  new_pillar_list(
    list(pillar),
    extra = NULL,
    remaining_width = new_width,
    simple = TRUE
  )
}

new_pillar_list <- function(pillar_list, extra, remaining_width, simple = FALSE) {
  structure(
    pillar_list,
    extra = extra,
    remaining_width = remaining_width,
    simple = simple
  )
}

prepare_title <- function(title) {
  n_title <- length(title)
  if (n_title == 0) {
    title
  } else if (grepl("^[[]", title[[n_title]])) {
    paste0(paste(title[-n_title], collapse = "$"), title[[n_title]])
  } else {
    paste(title, collapse = "$")
  }
}

#' Object for formatting a vector suitable for tabular display
#'
#' `pillar()` creates an object that formats a vector.
#' The output uses one row for a title (if given), one row for the type,
#' and `vec_size(x)` rows for the data.
#'
#' A pillar consists of arbitrary components.
#' The `pillar()` constructor uses `title`, `type`, and `data`.
#'
#' - `title` via [new_pillar_title()]
#' - `type` via [new_pillar_type()], which calls [type_sum()]
#'   internally
#' - `data` via [pillar_shaft()]
#'
#' All components are formatted via [format()] when displaying the pillar.
#' A `width` argument is passed to each `format()` call.
#'
#' As of pillar 1.5.0, `pillar()` returns `NULL` if the width is insufficient
#' to display the data.
#'
#' @param x A vector to format.
#' @param title An optional title for the column. The title will be
#'   used "as is", no quoting will be applied.
#' @param width Default width, optional.
#' @param ... Passed on to [pillar_shaft()].
#' @export
#' @examples
#' x <- 123456789 * (10^c(-1, -3, -5, NA, -8, -10))
#' pillar(x)
#' pillar(-x)
#' pillar(runif(10))
#' pillar(rcauchy(20))
#'
#' # Special values are highlighted
#' pillar(c(runif(5), NA, NaN, Inf, -Inf))
#'
#' # Very wide ranges will be displayed in scientific format
#' pillar(c(1e10, 1e-10), width = 20)
#' pillar(c(1e10, 1e-10))
#'
#' x <- c(FALSE, NA, FALSE, FALSE, TRUE, FALSE, FALSE, TRUE, FALSE, TRUE)
#' pillar(x)
#'
#' x <- c("This is string is rather long", NA, "?", "Short")
#' pillar(x)
#' pillar(x, width = 30)
#' pillar(x, width = 5)
#'
#' date <- as.Date("2017-05-15")
#' pillar(date + c(1, NA, 3:5))
#' pillar(as.POSIXct(date) + c(30, NA, 600, 3600, 86400))
pillar <- function(x, title = NULL, width = NULL, ...) {
  "!!!!DEBUG pillar(`v(class(x))`, `v(title)`, `v(width)`)"

  pillar_from_shaft(
    new_pillar_title(title),
    new_pillar_type(x),
    pillar_shaft(x, ...),
    width
  )
}

pillar_from_shaft <- function(title, type, data, width) {
  "!!!!!DEBUG pillar_from_shaft(`v(width)`)"

  if (is.null(width)) {
    my_width <- Inf
  } else {
    my_width <- width
  }

  if (get_min_width(title) > my_width) {
    "!!!!!DEBUG Early exit, (`v(get_min_width(title))`) > (`v(my_width)`)"
    return(NULL)
  }

  data_min_width <- get_min_width(data)
  if (data_min_width > my_width) {
    "!!!!!DEBUG Early exit, (`v(get_min_width(data))`) > (`v(my_width)`)"
    return(NULL)
  }
  data_width <- get_width(data)

  data_component <- new_pillar_component(list(data), width = data_width, min_width = data_min_width)

  # The type is taken either from the shaft (e.g. in the case of num() with
  # common exponents) or from the type provided by the caller
  shaft_type <- attr(data, "type", exact = TRUE)
  if (!is.null(shaft_type)) {
    type <- shaft_type
  } else {
    force(type)
  }

  # Delay querying width of type until it is decided if we take the type_sum
  # from the data or from the argument provided
  if (get_min_width(type) > my_width) {
    "!!!!!DEBUG Early exit, (`v(get_min_width(type))`) > (`v(my_width)`)"
    return(NULL)
  }

  new_pillar(
    list(
      title = pillar_component(title),
      type = pillar_component(type),
      data = data_component
    ),
    width = width
  )
}

#' Construct a custom pillar object
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' `new_pillar()` is the low-level constructor for pillar objects.
#' It supports arbitrary components.
#' See [pillar()] for the high-level constructor with default components.
#'
#' @details
#' Arbitrary components are supported.
#' If your tibble subclass needs more or different components in its pillars,
#' override or extend [ctl_new_pillar()] and perhaps [ctl_new_pillar_list()].
#'
#' @inheritParams rlang::args_dots_empty
#' @inheritParams pillar
#' @param components A named list of components constructed with [pillar_component()].
#' @param class Name of subclass.
#' @param extra Deprecated.
#'
#' @export
#' @examples
#' lines <- function(char = "-") {
#'   stopifnot(nchar(char) == 1)
#'   structure(char, class = "lines")
#' }
#'
#' format.lines <- function(x, width, ...) {
#'   paste(rep(x, width), collapse = "")
#' }
#'
#' new_pillar(list(
#'   title = pillar_component(new_ornament(c("abc", "de"), align = "right")),
#'   lines = new_pillar_component(list(lines("=")), width = 1)
#' ))
new_pillar <- function(components, ..., width = NULL, class = NULL,
                       extra = deprecated()) {
  "!!!!DEBUG new_pillar(`v(width)`, `v(class)`)"

  if (is_present(extra)) {
    # Signal the deprecation to the user
    deprecate_warn("1.7.0", "pillar::new_pillar(extra = )")
  }

  check_dots_empty()
  if (length(components) > 0 && !is_named(components)) {
    abort("All components must have names.")
  }

  structure(
    components,
    width = width,
    class = c(class, "pillar")
  )
}

#' @export
format.pillar <- function(x, width = NULL, ...) {
  if (is.null(width)) {
    width <- get_width(x)
  }

  if (is.null(width)) {
    width <- pillar_get_width(x)
  }

  as_glue(pillar_format_parts_2(x, width)$aligned)
}

#' @export
print.pillar <- function(x, ...) {
  writeLines(style_bold("<pillar>"))
  print(format(x, ...))
}

test_that("survival output", {
  skip_if_not_installed("survival", "3.2.9")

  x <- head(survival::Surv(survival::lung$time, survival::lung$status))
  expect_snapshot({
    pillar(x, width = 20)
    new_tbl(list(x = x))
  })

  skip_if_not(exists("Surv2", asNamespace("survival"), mode = "function"))

  x <- head(survival::Surv2(survival::lung$time, survival::lung$status))
  expect_snapshot({
    pillar(x, width = 20)
    new_tbl(list(x = x))
  })
})

test_that("with and without title", {
  expect_snapshot({
    pillar(add_special(10^(1:6)))
    pillar(add_special(10^(1:6)), title = "crayon")
    pillar(add_special(10^(1:6)), title = "short")
    pillar(add_special(10^(1:6)), title = "somewhat_wider")
    pillar(add_special(10^(1:6)), title = "exactly_fifteen")
    pillar(add_special(10^(1:6)), title = "absolutely_breaking_all_sensible_boundaries", width = 18)
  })
})

test_that("ellipsis has width 1", {
  expect_equal(get_extent(get_ellipsis()), 1)
})

test_that("integer output will use scientific if necessary", {
  x <- 10000000L + 1:3
  expect_snapshot({
    pillar(add_special(x), width = 6)
    pillar(add_special(x), width = 7)
    pillar(add_special(x), width = 8)
    pillar(add_special(x), width = 9)
  })
})

test_that("S4 character class (tidyverse/tibble#1367)", {
  skip_if_not_installed("DBI")
  expect_snapshot({
    pillar(DBI::SQL("x"))
  })
})

test_that("Infinite times (#645)", {
  skip_if(getRversion() < "4.3")
  expect_snapshot({
    pillar(.POSIXct(Inf))
  })
})

chartype_frame <- function() {
  chars <- character()
  desc <- character()

  chars[1] <- "\u0001\u001f"
  desc[1] <- "C0 control code"

  chars[2] <- "\a\b\f\n\r\t"
  desc[2] <- "Named control code"

  chars[3] <- "abcdefuvwxyz"
  desc[3] <- "ASCII"

  chars[4] <- "\u0080\u009f"
  desc[4] <- "C1 control code"

  chars[5] <- paste0(
    "\u00a0\u00a1\u00a2\u00a3\u00a4\u00a5",
    "\u00fa\u00fb\u00fc\u00fd\u00fe\u00ff"
  )
  desc[5] <- "Latin-1"

  chars[6] <- paste0(
    "\u0100\u0101\u0102\u0103\u0104\u0105",
    "\u0106\u0107\u0108\u0109\u010a\u010b"
  )
  desc[6] <- "Unicode"

  chars[7] <- "\uff01\uff02\uff03\uff04\uff05\uff06"
  desc[7] <- "Unicode wide"

  chars[8] <- "\ue00\u2029"
  desc[8] <- "Unicode control"

  chars[9] <- paste0(
    "x\u00adx\u200bx\u200cx\u200dx\u200ex\u200f",
    "x\u034fx\ufeffx", intToUtf8(0xE0001), "x",
    intToUtf8(0xE0020), "x", intToUtf8(0xE01EF), "x"
  )
  desc[9] <- "Unicode ignorable"

  chars[10] <- paste0(
    "a\u0300a\u0301a\u0302a\u0303a\u0304a\u0305",
    "a\u0306a\u0307a\u0308a\u0309a\u030aa\u030b"
  )
  desc[10] <- "Unicode mark"

  chars[11] <- paste0(
    intToUtf8(0x1F600), intToUtf8(0x1F601),
    intToUtf8(0x1F602), intToUtf8(0x1F603),
    intToUtf8(0x1F604), intToUtf8(0x1F483)
  )
  desc[11] <- "Emoji"

  chars[12] <- paste0("x", intToUtf8(0x10ffff), "x")
  desc[12] <- "Unassigned"

  chars[13] <- "\xfd\xfe\xff"
  desc[13] <- "Invalid"

  chars[14] <- "\\"
  desc[14] <- "Backslash"

  chars[15] <- '"'
  desc[15] <- "Quote"

  Encoding(chars) <- "UTF-8"

  data.frame(chars, desc, stringsAsFactors = FALSE)
}

test_that("output test", {
  expect_snapshot({
    pillar(add_special(letters[1:5]))
    pillar(add_special(paste(letters, collapse = "")))
    pillar(add_special(paste(letters, collapse = "")), width = 10)
    pillar(add_special(paste(letters, collapse = "")), width = 3)
    pillar(add_special(c("")), width = 5)
    pillar(add_special(c(" ")), width = 5)
    pillar(add_special(c(" a")), width = 5)
    pillar(add_special(c("a ")), width = 5)
    pillar(add_special(c("a b")), width = 5)
    pillar(add_special(c("\t")), width = 10)
    pillar(add_special(c("a\nb")), width = 10)
    pillar(add_special(c("a\001b")), width = 10)
  })
})

test_that("output test (not on Windows)", {
  skip_on_os("windows")

  if (getRversion() == "4.0.4") {
    # This seems to show differently on the Mac:
    # writeLines(evaluate::evaluate('c("\\u6210", "\\u6210")')[[1]]$src)
    skip_on_os("mac")
  }

  expect_snapshot({
    pillar(add_special("\u6210\u4ea4\u65e5"), title = "\u6210\u4ea4")
    pillar(add_special("\u6210\u4ea4"), title = "\u6210\u4ea4\u65e5")
    pillar(add_special(1L), title = "\u6210\u4ea4\u65e5")
  })

  # Spurious warnings on Windows
  skip_on_os("solaris")
  suppressWarnings(
    expect_snapshot({
      ctl_colonnade(chartype_frame(), width = 50)
    })
  )
})

test_that("output test", {
  expect_snapshot({
    pillar(add_special(as.POSIXct("2017-07-28 18:04:35 +0200")))
    pillar(add_special(as.POSIXlt("2017-07-28 18:04:35 +0200")))
  })
  withr::with_options(
    list(digits.secs = 4),
    expect_snapshot({
      pillar(add_special(as.POSIXlt("2017-07-28 18:04:35 +0200")))
    })
  )

  testthat::skip_if(getRversion() < "3.3")
  expect_snapshot({
    pillar(add_special(as.difftime(8:11, units = "secs")))
  })
})

#' @description
#' `new_pillar_shaft_simple()` provides an implementation of the `pillar_shaft`
#' class suitable for output that has a fixed formatting, which will be
#' truncated with a continuation character (ellipsis or `~`) if it doesn't fit
#' the available width.
#' By default, the required width is computed from the natural width of the
#' `formatted` argument.
#'
#' @details
#' The `formatted` argument may also contain ANSI escapes to change color
#' or other attributes of the text, see [crayon].
#'
#' @param formatted An object coercible to [character].
#' @param align Alignment of the column.
#' @param na String to use as `NA` value, defaults to `"NA"` styled with
#'   [style_na()] with fallback if color is not available.
#' @param na_indent Indentation of `NA` values.
#' @export
#' @rdname new_pillar_shaft
new_pillar_shaft_simple <- function(formatted, ..., width = NULL, align = "left",
                                    min_width = NULL, na = NULL, na_indent = 0L) {
  if (is.null(width)) {
    width <- get_max_extent(as.character(formatted))
  }

  if (is.null(na)) {
    na <- pillar_na()
  }

  new_pillar_shaft(
    list(formatted),
    ...,
    width = width,
    min_width = min_width,
    align = align,
    na = na,
    na_indent = na_indent,
    class = "pillar_shaft_simple"
  )
}

#' @export
format.pillar_shaft_simple <- function(x, width, ...) {
  align <- attr(x, "align")
  desired_width <- get_width(x)
  shaft <- as.character(x[[1]])
  if (width < desired_width) {
    shaft <- str_trunc(shaft, width)
  }
  shaft[is.na(shaft)] <- paste0(
    strrep(" ", attr(x, "na_indent")),
    # Fallback for compatibility, remove pillar_na() call in pillar 1.5.0
    attr(x, "na") %||% pillar_na()
  )

  new_ornament(shaft, width = width, align = align)
}

## 

## ## Integrating with other classes

## 

## ### gt

## 

## - Consumption via new `fmt_auto()`?

## 

## FIXME

## 

## ### units

## 

## xxx{r numbers-17}

## library(units)

## 

## set_units.pillar_num <- function(x, ...) {

##   unclassed <- x

##   class(unclassed) <- NULL

##   set_units(unclassed, ...)

## }

## 

## m <- set_units(1:3, m)

## km <- set_units(1:3, km)

## 

## tibble(

##   sci_int = set_num_opts(m + km, notation = "sci"),

##   digits_int = set_num_opts(km + m, digits = 4),

##   sci_ext = set_units(num(1:3 + 0.001, notation = "sci"), km)

## )

## 

## tibble(

##   sci_int = set_num_opts(m, notation = "sci") + km,

##   digits_int = set_num_opts(km, digits = 4) + m,

##   sci_ext = set_units(num(1:3, notation = "sci"), m) + km

## )

## xxx

## 

## ### formattable

## 

## xxx{r numbers-18, error = TRUE}

## library(formattable)

## 

## pillar_shaft.formattable <- function(x, ...) {

##   pillar::new_pillar_shaft_simple(format(x), align = "right")

## }

## 

## pillar_shaft.formattable_currency <- function(x, ...) {

##   formattable <- attr(x, "formattable", exact = TRUE)

## 

##   pillar_shaft(num(unclass(x), digits = formattable$digits))

## }

## 

## pillar_shaft.formattable_percent <- function(x, ...) {

##   formattable <- attr(x, "formattable", exact = TRUE)

## 

##   pillar_shaft(num(unclass(x), digits = formattable$digits, label = "%", scale = 100))

## }

## 

## pillar_shaft.formattable_scientific <- function(x, ...) {

##   pillar_shaft(num(unclass(x), notation = "sci"))

## }

## 

## type_sum.formattable <- function(x) {

##   formattable <- attr(x, "formattable", exact = TRUE)

## 

##   if (inherits(x, "formattable_currency")) {

##     I(sub("^formattable_", "", class(x)[[1]]))

##   } else if (inherits(x, "formattable_percent")) {

##     I("%")

##   } else {

##     abbreviate(sub("^formattable_", "", class(x)[[1]]), 4)

##   }

## }

## 

## num_currency(1:3 * 100 + 0.1)

## num_percent(1:3 * 0.1 + 0.001)

## num_scientific(1:3 * 0.1 + 0.001)

## 

## tibble(

##   currency = num_currency(1:3 * 100 + 0.1),

##   percent = num_percent(1:3 * 0.1 + 0.001),

##   scientific = num_scientific(1:3 * 0.1 + 0.001)

## )

## xxx

## 

## ### scales

## 

## xxx{r numbers-scales, error = TRUE}

## library(scales)

## 

## x <- num(1:10 / 100, label = "%", scale = 100)

## 

## scales::squish(x)

## 

## x < 0

## x < 0L

## 

## scales::cscale(x, scales::rescale_pal())

## xxx

## 

## ### ggplot2

## 

## xxx{r numbers-19}

## library(ggplot2)

## 

## scale_type.pillar_num <- function(x, ...) {

##   "continuous"

## }

## 

## data.frame(x = x, y = 1:10) %>%

##   ggplot(aes(x = x, y = y)) %>%

##   + geom_point()

## xxx

## 

## ## Rule-based decoration

## 

## 

## 

## xxx{r}

## library(dplyr)

## 

## data_units <-

##   palmerpenguins::penguins %>%

##   mutate(across(ends_with("_mm"), set_units, "mm")) %>%

##   mutate(across(ends_with("_g"), set_units, "g"))

## 

## data_units %>%

##   mutate(bill_area = bill_length_mm * bill_depth_mm, .after = island)

## xxx

## 

## xxx{r eval = FALSE}

## data_decor <-

##   data_units %>%

##   decorate(year, digits = 0) %>%

##   decorate(where(is.numeric), digits = 3)

## xxx

## 

## xxx{r eval = FALSE}

## data_decor %>%

##   mutate(bill_area = bill_length_mm * bill_depth_mm, .after = island)

## xxx

## 

## xxx{r echo = FALSE}

## data_units %>%

##   mutate(bill_area = bill_length_mm * bill_depth_mm, .after = island) %>%

##   mutate(across(year, set_num_opts, digits = 0)) %>%

##   mutate(across(where(is.numeric), set_num_opts, digits = 3))

## xxx

## 

#' Format multiple vectors in a tabular display
#'
#' @description
#' `r lifecycle::badge("soft-deprecated")`
#'
#' The vectors are formatted to fit horizontally into a user-supplied number of
#' characters per row.
#'
#' The `colonnade()` function doesn't process the input but returns an object
#' with a [format()] and a [print()] method.
#' The implementations call [squeeze()] to create [pillar] objects and fit them to a given width.
#'
#' @param x A list, which can contain matrices or data frames.
#'   If named, the names will be used as title for the pillars. Non-syntactic names
#'   will be escaped.
#' @param has_row_id Include a column indicating row IDs? Pass `"*"` to mark
#'   the row ID column with a star.
#' @param width Default width of the entire output, optional.
#' @inheritParams rlang::args_dots_empty
#' @keywords internal
#' @export
colonnade <- function(x, has_row_id = TRUE, width = NULL, ...) {
  deprecate_stop("1.9.0", "pillar::colonnade()", "pillar::tbl_format_setup()")

  if (!missing(...)) {
    check_dots_empty(action = warn)
  }

  # Reset local cache for each new colonnade
  num_colors(forget = TRUE)

  x <- flatten_colonnade(x)
  ret <- new_data_frame(x, has_row_id = has_row_id, class = "pillar_colonnade")
  ret <- set_width(ret, width)
  ret
}

flatten_colonnade <- function(x) {
  out <- map2(
    unname(x),
    names2(x),
    flatten_column
  )

  vec_rbind(
    !!!out,
    # .ptype = data_frame(names = list(), data = list())
    .ptype = data_frame(names = character(), data = list())
  )
}

flatten_column <- function(x, name) {
  if (name != "") {
    name <- tick_if_needed(name)
  }

  if (is.data.frame(x)) {
    flatten_df_column(x, name)
  } else if (is.matrix(x) && !inherits(x, c("Surv", "Surv2"))) {
    flatten_matrix_column(x, name)
  } else {
    # Length-one list, will be unlist()ed afterwards
    # data_frame(names = list(name), data = list(x))
    data_frame(names = name, data = list(x))
  }
}

flatten_df_column <- function(x, name) {
  if (length(x) == 0) {
    # data_frame(names = list(name), data = list(new_empty_col_sentinel(x)))
    data_frame(names = name, data = list(new_empty_col_sentinel(x)))
  } else {
    x <- flatten_colonnade(unclass(x))
    # x$names <- map(x$names, function(.x) c(name, .x))
    x$names <- paste0("$", x$names)
    x$names[[1]] <- paste0(name, x$names[[1]])
    x
  }
}

flatten_matrix_column <- function(x, name) {
  if (ncol(x) == 0) {
    data_frame(
      # names = list(c(name, "[,0]")),
      names = name,
      data = list(new_empty_col_sentinel(x))
    )
  } else {
    x_list <- map(seq_len(ncol(x)), function(i) x[, i])

    idx <- colnames(x)
    if (is.null(idx)) {
      idx <- seq_along(x_list)
    } else {
      idx <- encodeString(idx, quote = '"')
    }

    # names <- map(idx, function(.x) c(name, .x))
    names <- paste0("[,", idx, "]")
    names[[1]] <- paste0(name, names[[1]])

    data_frame(names = names, data = x_list)
  }
}

new_empty_col_sentinel <- function(type) {
  structure(list(type), class = c("pillar_empty_col"))
}

#' Squeeze a colonnade to a fixed width
#'
#' @description
#' `r lifecycle::badge("soft-deprecated")`
#'
#' The `squeeze()` function usually doesn't need to be called manually.
#' It returns an object suitable for printing and formatting at a fixed width
#' with additional information about omitted columns, which can be retrieved
#' via [extra_cols()].
#'
#' @keywords internal
#' @export
squeeze <- function(x, width = NULL, ...) {
  deprecate_stop("1.5.0", "pillar::squeeze()")

  squeeze_impl(x, width, ...)
}

squeeze_impl <- function(x, width = NULL, ...) {
  # Shortcut for zero-height corner case
  zero_height <- length(x$data) == 0L || length(x$data[[1]]) == 0L
  if (zero_height) {
    return(new_colonnade_squeezed(list(), colonnade = x, extra_cols = seq_along(x$data)))
  }

  if (is.null(width)) {
    width <- get_width(x)
  }

  if (is.null(width)) {
    width <- getOption("width")
  }

  rowid <- get_rowid_from_colonnade(x)
  if (is.null(rowid)) {
    rowid_width <- 0
  } else {
    rowid_width <- max(get_widths(rowid)) + 1L
  }

  col_widths <- colonnade_get_width(x, width, rowid_width)
  col_widths_shown <- col_widths[!safe_is_na(col_widths$tier), ]
  indexes <- split(seq_along(col_widths_shown$tier), col_widths_shown$tier)

  out <- map(indexes, function(i) {
    inner <- map2(col_widths_shown$pillar[i], col_widths_shown$width[i], pillar_format_parts)
    if (!is.null(rowid)) {
      inner <- c(list(pillar_format_parts(rowid, rowid_width - 1L)), inner)
    }
    inner
  })

  n_cols_shown <- nrow(col_widths_shown)
  extra_cols <- seq2(n_cols_shown + 1L, length(x$data))
  new_colonnade_squeezed(out, colonnade = x, extra_cols = extra_cols)
}

get_rowid_from_colonnade <- function(x) {
  has_title <- any(x$names != "")

  has_row_id <- attr(x, "has_row_id", exact = TRUE)
  if (!is_false(has_row_id) && length(x$data) > 0) {
    rowid <- rowidformat(
      length(x$data[[1]]),
      has_star = identical(has_row_id, "*"),
      has_title_row = has_title
    )
  } else {
    rowid <- NULL
  }

  rowid
}

new_colonnade_squeezed <- function(x, colonnade, extra_cols) {
  formatted_tiers <- map(x, format_colonnade_tier)
  formatted <- as_glue(as.character(unlist(formatted_tiers)))

  structure(
    list(formatted),
    extra_cols = colonnade[extra_cols, ],
    class = "pillar_squeezed_colonnade"
  )
}

format_colonnade_tier <- function(x) {
  "!!!!!DEBUG format_colonnade_tier(`v(x)`)"

  if (length(x) == 0) {
    return(character())
  }

  unlist(pmap(unname(x), paste))
}

#' @export
format.pillar_squeezed_colonnade <- function(x, ...) {
  x[[1]]
}

#' @export
print.pillar_squeezed_colonnade <- function(x, ...) {
  print(format(x, ...), ...)
  invisible(x)
}

# Method registration happens in .onLoad()
knit_print.pillar_squeezed_colonnade <- function(x, ...) {
  unlist(map(x, knit_print_squeezed_colonnade_tier))
}

knit_print_squeezed_colonnade_tier <- function(x) {
  # Hack
  header <- map_chr(map(x, `[[`, "capital_format"), `[[`, "title_format")
  col <- map(x, function(xx) c(xx[["capital_format"]][["type_format"]], xx[["shaft_format"]]))

  knitr::kable(as.data.frame(col), row.names = NA, col.names = header)
}

#' Retrieve information about columns that didn't fit the available width
#'
#' @description
#' `r lifecycle::badge("soft-deprecated")`
#'
#' Formatting a [colonnade] object may lead to some columns being omitted
#' due to width restrictions. This method returns a character vector that
#' describes each of the omitted columns.
#'
#' @param x The result of [squeeze()] on a [colonnade] object
#' @inheritParams rlang::args_dots_used
#' @keywords internal
#' @export
extra_cols <- function(x, ...) {
  deprecate_stop("1.5.0", "pillar::extra_cols()")

  if (!missing(...)) {
    check_dots_used(action = warn)
  }

  UseMethod("extra_cols")
}

#' @rdname extra_cols
#' @param n The number of extra columns to return; the returned vector will
#'   always contain as many elements as there are extra columns, but elements
#'   beyond `n` will be `NA`.
#' @export
extra_cols.pillar_squeezed_colonnade <- function(x, ..., n = Inf) {
  extra_cols_impl(x, n)
}

extra_cols_impl <- function(x, n = NULL) {
  extra_cols <- attr(x, "extra_cols", exact = TRUE)
  ret <- rep(NA_character_, length(extra_cols$data))

  if (is.null(n)) {
    n <- Inf
  }

  idx <- seq_len(min(length(extra_cols$data), n))
  ret[idx] <- map2_chr(extra_cols$data[idx], extra_cols$names[idx], format_abbrev, space = NBSP)
  ret
}

#' @export
format.pillar_colonnade <- function(x, ...) {
  format(squeeze_impl(x, ...))
}

#' @export
print.pillar_colonnade <- function(x, ...) {
  print(format(x, ...))
}

#' @rdname colonnade
#' @usage NULL
#' @aliases NULL
colonnade_get_width <- function(x, width, rowid_width) {
  #' @details
  #' Pillars may be distributed over multiple tiers if
  #' `width > getOption("width")`. In this case each tier is at most
  #' `getOption("width")` characters wide. The very first step of formatting
  #' is to determine how many tiers are shown at most, and the width of each
  #' tier.
  tier_widths <- get_tier_widths(width, length(x$data), rowid_width)

  #'
  #' To avoid unnecessary computation for showing very wide colonnades, a first
  #' pass tries to fit all capitals into the tiers.
  init_cols <- min(length(x$data), sum(floor((tier_widths + 1L) / (MIN_PILLAR_WIDTH + 1L))))
  capitals <- map2(x$data[seq_len(init_cols)], x$names[seq_len(init_cols)], pillar_capital)
  init_col_widths_df <- colonnade_compute_tiered_col_widths(capitals, tier_widths)
  pillar_shown <- init_col_widths_df$id[!safe_is_na(init_col_widths_df$tier)]
  if (length(pillar_shown) < init_cols) {
    # (Include one more pillar to indicate that the data is too wide.)
    pillar_shown <- c(pillar_shown, pillar_shown[length(pillar_shown)] + 1L)
  }

  #' For each pillar whose capital fits, it is then decided in which tier it is
  #' shown, if at all, and how much horizontal space it may use (either its
  #' minimum or its maximum width).
  shafts <- map(x$data[pillar_shown], pillar_shaft)
  pillars <- map2(capitals[pillar_shown], shafts, new_pillar_1e)
  col_widths_df <- colonnade_compute_tiered_col_widths(pillars, tier_widths)

  #' Remaining space is then distributed proportionally to pillars that do not
  #' use their desired width.
  colonnade_distribute_space_df(col_widths_df, tier_widths)
}

get_tier_widths <- function(width, ncol, rowid_width, tier_width = getOption("width")) {
  if (!is.finite(width)) {
    pos <- seq(0, length.out = ncol + 1L, by = tier_width)
  } else if (width < tier_width) {
    pos <- c(0L, width)
  } else {
    pos <- seq(0, width + tier_width - 1, by = tier_width)
  }

  widths <- diff(pos) - rowid_width
  widths[widths >= 1L]
}

colonnade_compute_tiered_col_widths <- function(pillars, tier_widths) {
  max_tier_width <- max(tier_widths)

  max_widths <- pmin(map_int(map(pillars, get_widths), max), max_tier_width)
  min_widths <- pmin(map_int(map(pillars, get_min_widths), max), max_widths)

  ret <- colonnade_compute_tiered_col_widths_df(max_widths, min_widths, tier_widths)
  ret$pillar <- pillars
  ret
}

#' @rdname colonnade
#' @usage NULL
#' @aliases NULL
colonnade_compute_tiered_col_widths_df <- function(max_widths, min_widths, tier_widths) {
  "!!!!!DEBUG colonnade_compute_tiered_col_widths_df(`v(tier_widths)`)"

  max_tier_width <- max(tier_widths)

  # Safety:
  max_widths <- pmin(max_widths, max_tier_width)
  min_widths <- pmin(min_widths, max_tier_width)

  id <- seq_along(max_widths)
  col_df <- data.frame(id, max_widths, min_widths, row.names = NULL)

  #' @details
  #' For fitting pillars in one or more tiers, first a check is made
  #' if all pillars fit with their maximum width (e.g.,
  #' `option(tibble.width = Inf)` or narrow colonnade).
  max_fit <- distribute_pillars(col_df$max_widths, tier_widths)
  #' If yes, this is the resulting fit, no more work needs to be done.
  if (all_pillars_fit(max_fit)) {
    return(max_fit)
  }

  #' Otherwise, if the maximum width is too wide, the same test
  #' is carried out with the minimum width.
  #' If this is still too wide, this is the resulting fit.
  min_fit <- distribute_pillars(col_df$min_widths, tier_widths)
  if (!all_pillars_fit(min_fit)) {
    return(min_fit)
  }

  #' Otherwise, some tiers from the start
  #' will contain pillars with their maximum width,
  #' one tier will contain some pillars with maximum and some with minimum width,
  #' and the remaining tiers contain pillars with their minimum width only.
  #'
  #' For this, we compute a "reverse minimum assignment".
  min_fit_rev <- distribute_pillars_rev(col_df$min_widths, tier_widths)

  combined_fit <- combine_pillar_distributions(max_fit, min_fit_rev, tier_widths)

  combined_fit$max_widths <- col_df$max_widths
  combined_fit
}

#' @rdname colonnade
#' @usage NULL
#' @aliases NULL
combine_pillar_distributions <- function(max_fit, min_fit_rev, tier_widths) {
  #' @details
  #' We determine the cut point where minimum and maximum assignment
  #' agree.
  #' The following strategy is applied:
  #'
  #' 1. First, we determine the tier in which the cut point lies.
  #'    This is the first instance of a column that ends up in the same tier
  #'    for both minimum and maximum assignment.
  cut_point_tier <- max_fit$tier[min(which(max_fit$tier == min_fit_rev$tier))]
  #' 2. A set of candidate cut points is derived.
  cut_point_candidates <- which(max_fit$tier == cut_point_tier)
  #' 3. We consult the column offsets. The last column where the minimum assignment
  #'    has a greater or equal offset than the maximum assignment is our latest
  #'    cut point.
  cut_point_candidate_idx <- which(max_fit$offset_after[cut_point_candidates] <= min_fit_rev$offset_after[cut_point_candidates])
  if (length(cut_point_candidate_idx) > 0) {
    cut_point <- cut_point_candidates[max(cut_point_candidate_idx)]
  } else {
    #'    If no such column exists, the cut point is the column just before our
    #'    first candidate.
    cut_point <- cut_point_candidates[[1]] - 1L
  }

  #' 4. Finally, we combine maximum and minimum reverse fits at the cut point.
  #'    We don't need to redistribute anything here.
  max_fit_cut <- max_fit[seq_len(cut_point), ]
  min_fit_cut <- min_fit_rev[seq2(cut_point + 1L, nrow(min_fit_rev)), ]
  rbind(max_fit_cut, min_fit_cut)
}

#' @rdname colonnade
#' @usage NULL
#' @aliases NULL
distribute_pillars <- function(widths, tier_widths) {
  tier <- rep(NA_integer_, length(widths))
  offset_after <- rep(NA_integer_, length(widths))
  current_tier <- 1L
  current_x <- 0L

  #' @details
  #' Fitting pillars into tiers is very similar to a word-wrapping algorithm.
  for (i in seq_along(widths)) {
    current_width <- widths[[i]]
    #' In a loop, new tiers are opened if the current tier overflows.
    if (current_x + current_width > tier_widths[[current_tier]]) {
      #' If a column is too wide to fit a single tier, it will never be
      #' displayed, and the colonnade will be truncated there.
      #' This case should never occur with reasonable display widths larger than
      #' 30 characters.
      if (current_width > tier_widths[[current_tier]]) {
        if (current_tier == length(tier_widths) || current_width > tier_widths[[current_tier + 1]]) {
          break
        }
      }

      current_tier <- current_tier + 1L
      current_x <- 0L
      #' Truncation also happens if all available tiers are filled.
      if (current_tier > length(tier_widths)) break
    }

    tier[[i]] <- current_tier
    current_x <- current_x + current_width
    offset_after[[i]] <- current_x
    current_x <- current_x + 1L
  }

  data_frame(id = seq_along(widths), tier = tier, width = widths, offset_after = offset_after)
}

distribute_pillars_rev <- function(widths, tier_widths) {
  ret <- distribute_pillars(rev(widths), rev(tier_widths))
  ret[2:4] <- ret[rev(seq_along(widths)), 2:4]
  tier <- length(tier_widths) + 1L - ret$tier
  ret$tier <- tier

  splits <- split(seq_along(tier), tier)
  tier_widths <- tier_widths[stats::na.omit(unique(tier))]

  new_offset_after <- unlist(map2(unname(splits), tier_widths, function(.x, .y) {
    new_offset_after <- cumsum(ret$width[.x] + 1)
    new_offset_after - max(new_offset_after) + .y
  }))

  ret$offset_after <- c(new_offset_after, rep(NA_integer_, sum(is.na(tier))))

  ret
}

all_pillars_fit <- function(tier_df) {
  rows <- nrow(tier_df)
  rows == 0 || !safe_any_na(tier_df$tier[[nrow(tier_df)]])
}

#' @rdname colonnade
#' @usage NULL
#' @aliases NULL
colonnade_distribute_space_df <- function(col_widths_df, tier_widths) {
  "!!!!!DEBUG colonnade_distribute_space_df(`v(tier_widths)`)"

  col_widths_split <- split(col_widths_df, col_widths_df$tier)
  tier_widths <- tier_widths[seq_along(col_widths_split)]
  col_widths_apply <- map2(col_widths_split, tier_widths, function(x, width) {
    x$width <- x$width + colonnade_distribute_space(x$width, x$max_widths, width)
    x
  })

  bind_rows(unname(col_widths_apply))
}

#' @rdname colonnade
#' @usage NULL
#' @aliases NULL
colonnade_distribute_space <- function(col_widths, max_widths, width) {
  if (any(is.na(col_widths))) {
    return(col_widths)
  }

  missing_space <- max_widths - col_widths
  # Shortcut to avoid division by zero
  if (all(missing_space == 0L)) {
    return(rep_along(col_widths, 0L))
  }

  #' @details
  #' The remaining space is distributed from left to right.
  #' Each column gains space proportional to the fraction of missing and
  #' remaining space,
  occupied_width <- sum(col_widths + 1L) - 1L
  remaining_width <- max(min(width - occupied_width, sum(missing_space)), 0L)
  added_space_prop <- missing_space / sum(missing_space) * remaining_width

  #' rounded down.
  added_space_ceil <- ceiling(added_space_prop)
  added_space_floor <- floor(added_space_prop)
  added_space_diff <- added_space_ceil - added_space_floor
  added_space <- ifelse(
    #' Any space remaining after rounding is distributed from left to right,
    #' one space per column.
    sum(added_space_floor) + cumsum(added_space_diff) <= remaining_width,
    added_space_ceil,
    added_space_floor
  )

  added_space
}

test_that("colonnade_compute_tiered_col_widths_df()", {
  expect_snapshot({
    colonnade_compute_tiered_col_widths_df(rep(30, 4), rep(15, 4), rep(60, 4))
    colonnade_compute_tiered_col_widths_df(rep(30, 4), rep(15, 4), rep(60, 3))
    colonnade_compute_tiered_col_widths_df(rep(30, 4), rep(15, 4), rep(60, 2))
    colonnade_compute_tiered_col_widths_df(rep(30, 5), rep(15, 5), rep(60, 3))
    colonnade_compute_tiered_col_widths_df(rep(30, 5), rep(15, 5), rep(60, 4))
    colonnade_compute_tiered_col_widths_df(rep(30, 10), rep(15, 10), rep(60, 5))
    colonnade_compute_tiered_col_widths_df(c(6, 5, 10), c(5, 5, 10), 21)
    colonnade_compute_tiered_col_widths_df(c(6, 5, 10), c(5, 5, 10), 22)
    colonnade_compute_tiered_col_widths_df(c(6, 5, 10), c(5, 5, 10), 23)
    colonnade_compute_tiered_col_widths_df(c(5, 6, 10), c(5, 5, 10), 21)
    colonnade_compute_tiered_col_widths_df(c(5, 6, 10), c(5, 5, 10), 22)
    colonnade_compute_tiered_col_widths_df(c(5, 6, 10), c(5, 5, 10), 23)
  })
})

test_that("distribute_pillars()", {
  expect_snapshot({
    distribute_pillars(1:3, 10)
    distribute_pillars(1:3, 5)
    distribute_pillars(1:3, c(5, 5))
    distribute_pillars(1:5, 7:9)
    distribute_pillars(3:5, 8:10)
    distribute_pillars(5:3, 9:8)
    distribute_pillars(c(5, 5), c(2, 8, 8))
    distribute_pillars(c(5, 5), c(8, 8, 2))
  })
})

test_that("distribute_pillars_rev()", {
  expect_snapshot({
    distribute_pillars_rev(1:3, 10)
    distribute_pillars_rev(1:3, 5)
    distribute_pillars_rev(1:3, c(5, 5))
    distribute_pillars_rev(1:5, 7:9)
    distribute_pillars_rev(3:5, 8:10)
    distribute_pillars_rev(5:3, 9:8)
    distribute_pillars_rev(c(5, 5), c(8, 8, 2))
    distribute_pillars_rev(c(5, 5), c(2, 8, 8))
  })
})

test_that("sanity check (1)", {
  expect_equal(cli::num_ansi_colors(), 1)
  expect_false(has_color())
  expect_equal(num_colors(), 1)

  expect_snapshot({
    cli::num_ansi_colors()
    has_color()
    num_colors()
    style_na("NA")
  })
})

test_that("output test", {
  skip_if(getRversion() < "4.0")

  x <- list(
    column_zero_one = 1:3 + 0.23,
    col_02 = letters[1:3],
    col_03 = factor(letters[1:3]),
    col_04 = ordered(letters[1:3])
  )
  expect_snapshot({
    colonnade(x, width = 4)
    colonnade(x, width = 5)
    colonnade(x, width = 6)
    colonnade(x, width = 7)
    colonnade(x, width = 8)
    colonnade(x, width = 9)
    colonnade(x, width = 10)
    colonnade(x, width = 11)
    colonnade(x, width = 12)
    colonnade(x, width = 13)
    colonnade(x, width = 14)
    colonnade(x, width = 15)
    colonnade(x, width = 16)
    colonnade(x, width = 17)
    colonnade(x, width = 18)
    colonnade(x, width = 19)
    colonnade(x, width = 20)
    colonnade(x, width = 21)
    colonnade(x, width = 22)
    colonnade(x, width = 23)
    colonnade(x, width = 24)
    colonnade(x, width = 25)
    colonnade(x, width = 26)
    colonnade(x, width = 27)
    colonnade(x, width = 28)
    colonnade(x, width = 29)
    colonnade(x, width = 30)
    colonnade(x, width = 31)
    colonnade(x, width = 32)
    colonnade(x, width = 33)
    colonnade(x, width = 34)
    colonnade(x, width = 35)
    colonnade(x, width = 36)
    colonnade(x, width = 37)
    colonnade(x, width = 38)
    colonnade(x, width = 39)
    colonnade(x, width = Inf)
  })

  expect_snapshot({
    colonnade(rep(list(paste(letters, collapse = " ")), 4), width = Inf)
  })

  # Spurious warnings on Windows
  suppressWarnings(
    expect_snapshot({
      as_glue(extra_cols_impl(squeeze_impl(colonnade(x), width = 10)))
    })
  )

  suppressWarnings(
    expect_snapshot({
      as_glue(extra_cols_impl(squeeze_impl(colonnade(x), width = 20)))
    })
  )

  suppressWarnings(
    expect_snapshot({
      as_glue(extra_cols_impl(squeeze_impl(colonnade(x), width = 30)))
    })
  )

  suppressWarnings(
    expect_snapshot({
      as_glue(extra_cols_impl(squeeze_impl(colonnade(x), width = 35)))
    })
  )

  expect_snapshot({
    as_glue(extra_cols_impl(squeeze_impl(colonnade(x), width = 40)))
  })
})

test_that("tests from tibble", {
  skip_if(getRversion() < "4.0")

  skip_if_not_installed("rlang", "0.4.11.9000")
  local_options(width = 80)

  expect_snapshot({
    colonnade(mtcars[1:8, ], has_row_id = "*", width = 30)
    colonnade(trees[1:5, ], width = 20)
    colonnade(trees[1:3, ], width = 10)
    colonnade(df_all, width = 30)
    colonnade(df_all, width = 300)
    options(width = 70)
    colonnade(df_all, width = 300)
    options(width = 60)
    colonnade(df_all, width = 300)
    options(width = 50)
    colonnade(df_all, width = 300)
    options(width = 40)
    colonnade(df_all, width = 300)
    options(width = 30)
    colonnade(df_all, width = 300)
    options(width = 20)
    colonnade(df_all, width = 300)
    list_with_ctl <- list(c("\n", '"'), factor(c("\n", "\n")))
    names(list_with_ctl) <- c("\n", "\r")
    colonnade(list_with_ctl, width = 30)
    colonnade(list(a = c("", " ", "a ", " a")), width = 30)
    colonnade(list("mean(x)" = 5, "var(x)" = 3), width = 30)
  })
})

test_that("empty", {
  skip_if(getRversion() < "4.0")

  expect_equal(
    format(colonnade(list(a = character(), b = logical()), width = 30)),
    as_glue(character())
  )
  expect_equal(
    format(colonnade(trees[1:5, character()], width = 30)),
    as_glue(character())
  )
})

test_that("NA names", {
  skip_if(getRversion() < "4.0")

  x <- list(`NA` = 1:3, set_to_NA = 4:6)
  names(x)[[2]] <- NA_character_
  expect_snapshot({
    colonnade(x, width = 30)
  })
})

test_that("sep argument", {
  skip_if(getRversion() < "4.0")

  x <- list(sep = 1:3)
  expect_snapshot({
    colonnade(x, width = 30)
    "dummy"
  })
})

test_that("color", {
  skip_if_not_installed("testthat", "3.1.1")

  local_colors()
  expect_equal(cli::num_ansi_colors(), 16)
  expect_true(has_color())
  expect_equal(num_colors(), 16)

  if (l10n_info()$`UTF-8`) {
    local_utf8()
    expect_true(cli::is_utf8_output())
    variant <- "unicode"
  } else {
    variant <- "ansi"
  }

  expect_snapshot(variant = variant, {
    cli::num_ansi_colors()
    has_color()
    num_colors()
    style_na("NA")
    style_neg("-1")
  })

  expect_snapshot(variant = variant, {
    style_na("NA")
  })

  xf <- colonnade(list(x = c((10^(-3:4)) * c(-1, 1), NA)))

  expect_snapshot(variant = variant, {
    print(xf)
    with_options(pillar.subtle_num = TRUE, print(xf))
    with_options(pillar.subtle = FALSE, print(xf))
    with_options(pillar.neg = FALSE, print(xf))
    with_options(pillar.subtle = FALSE, pillar.neg = FALSE, print(xf))
    with_options(pillar.bold = TRUE, print(xf))
  })

  skip_if(getRversion() < "4.0")

  expect_snapshot(variant = variant, {
    colonnade(list(a_very_long_column_name = 0), width = 15)
  })
})

test_that("sanity check (2)", {
  expect_equal(cli::num_ansi_colors(), 1)
  expect_false(has_color())
  expect_equal(num_colors(), 1)

  expect_snapshot({
    cli::num_ansi_colors()
    has_color()
    num_colors()
    style_na("NA")
  })
})

test_that("tibble columns", {
  skip_if(getRversion() < "4.0")

  x <- list(a = 1:3, b = data.frame(c = 4:6, d = 7:9))
  expect_snapshot({
    colonnade(x, width = 30)
  })
})

test_that("tibble columns (nested)", {
  skip_if(getRversion() < "4.0")

  x <- list(
    a = 1:3,
    b = structure(
      list(
        c = 4:6, d = 7:9,
        e = data.frame(f = 10:12, g = 13:15)
      ),
      class = "data.frame"
    )
  )
  expect_snapshot({
    colonnade(x, width = 40)
  })
})

test_that("tibble columns (empty)", {
  skip_if(getRversion() < "4.0")

  x <- list(
    a = 1:3,
    b = structure(
      list(
        c = 4:6, d = 7:9,
        e = data.frame(f = 10:12)[, 0],
        f = 10:12
      ),
      class = "data.frame"
    )
  )
  expect_snapshot({
    colonnade(x, width = 40)
  })
})

test_that("matrix columns (unnamed)", {
  skip_if(getRversion() < "4.0")

  x <- list(a = 1:3, b = matrix(4:9, ncol = 2))
  expect_snapshot({
    colonnade(x, width = 30)
  })
})

test_that("matrix columns (named)", {
  skip_if(getRversion() < "4.0")

  x <- list(a = 1:3, b = matrix(4:9, ncol = 2, dimnames = list(NULL, c("c", "d"))))
  expect_snapshot({
    colonnade(x, width = 30)
  })
})

test_that("matrix columns (empty)", {
  x <- list(a = 1:3, b = matrix(4:6, ncol = 1)[, 0], c = 4:6)
  expect_snapshot({
    colonnade(x, width = 30)
  })
})

context("format_character")

withr::with_options(list(pillar.bold = TRUE), {
  chartype_frame <- function() {
    chars <- character()
    desc <- character()

    chars[1] <- "\u0001\u001f"
    desc[1] <- "C0 control code"

    chars[2] <- "\a\b\f\n\r\t"
    desc[2] <- "Named control code"

    chars[3] <- "abcdefuvwxyz"
    desc[3] <- "ASCII"

    chars[4] <- "\u0080\u009f"
    desc[4] <- "C1 control code"

    chars[5] <- paste0(
      "\u00a0\u00a1\u00a2\u00a3\u00a4\u00a5",
      "\u00fa\u00fb\u00fc\u00fd\u00fe\u00ff"
    )
    desc[5] <- "Latin-1"

    chars[6] <- paste0(
      "\u0100\u0101\u0102\u0103\u0104\u0105",
      "\u0106\u0107\u0108\u0109\u010a\u010b"
    )
    desc[6] <- "Unicode"

    chars[7] <- "\uff01\uff02\uff03\uff04\uff05\uff06"
    desc[7] <- "Unicode wide"

    chars[8] <- "\ue00\u2029"
    desc[8] <- "Unicode control"

    chars[9] <- paste0(
      "x\u00adx\u200bx\u200cx\u200dx\u200ex\u200f",
      "x\u034fx\ufeffx", intToUtf8(0xE0001), "x",
      intToUtf8(0xE0020), "x", intToUtf8(0xE01EF), "x"
    )
    desc[9] <- "Unicode ignorable"

    chars[10] <- paste0(
      "a\u0300a\u0301a\u0302a\u0303a\u0304a\u0305",
      "a\u0306a\u0307a\u0308a\u0309a\u030aa\u030b"
    )
    desc[10] <- "Unicode mark"

    chars[11] <- paste0(
      intToUtf8(0x1F600), intToUtf8(0x1F601),
      intToUtf8(0x1F602), intToUtf8(0x1F603),
      intToUtf8(0x1F604), intToUtf8(0x1F483)
    )
    desc[11] <- "Emoji"

    chars[12] <- paste0("x", intToUtf8(0x10ffff), "x")
    desc[12] <- "Unassigned"

    chars[13] <- "\xfd\xfe\xff"
    desc[13] <- "Invalid"

    chars[14] <- "\\"
    desc[14] <- "Backslash"

    chars[15] <- '"'
    desc[15] <- "Quote"

    Encoding(chars) <- "UTF-8"

    data.frame(chars, desc, stringsAsFactors = FALSE)
  }

  test_that("output test", {
    expect_pillar_output(letters[1:5], filename = "letters.txt")
    expect_pillar_output(paste(letters, collapse = ""), filename = "letters-long.txt")
    expect_pillar_output(paste(letters, collapse = ""), width = 10, filename = "letters-long-10.txt")
    expect_pillar_output(paste(letters, collapse = ""), width = 3, filename = "letters-long-03.txt")
    expect_pillar_output(c(""), width = 5, filename = "spaces0.txt")
    expect_pillar_output(c(" "), width = 5, filename = "spaces.txt")
    expect_pillar_output(c(" a"), width = 5, filename = "spaces2.txt")
    expect_pillar_output(c("a "), width = 5, filename = "spaces3.txt")
    expect_pillar_output(c("a b"), width = 5, filename = "spaces4.txt")

    skip_on_os("windows")
    expect_pillar_output("\u6210\u4ea4\u65e5", title = "\u6210\u4ea4", filename = "deal1.txt")
    expect_pillar_output("\u6210\u4ea4", title = "\u6210\u4ea4\u65e5", filename = "deal2.txt")
    expect_pillar_output(1L, title = "\u6210\u4ea4\u65e5", filename = "deal3.txt")

    skip_on_os("solaris")
    # Spurious warnings on Windows and Solaris
    suppressWarnings(
      expect_pillar_output(xf = colonnade(chartype_frame()), width = 50, filename = "utf8.txt")
    )
  })
}) # withr::with_options(list(pillar.bold = TRUE), {

test_that("space distribution never removes space", {
  # and distributes space from left to right based on need
  expect_equal(
    colonnade_distribute_space(c(5, 6, 6), c(11, 8, 8), 18),
    c(0, 0, 0)
  )

  expect_equal(
    colonnade_distribute_space(c(5, 6, 6), c(11, 8, 8), 19),
    c(0, 0, 0)
  )

  expect_equal(
    colonnade_distribute_space(c(5, 6, 6), c(11, 8, 8), 20),
    c(1, 0, 0)
  )

  expect_equal(
    colonnade_distribute_space(c(5, 6, 6), c(11, 8, 8), 21),
    c(2, 0, 0)
  )

  expect_equal(
    colonnade_distribute_space(c(5, 6, 6), c(11, 8, 8), 22),
    c(2, 1, 0)
  )
})

test_that("sanity check (1)", {
  expect_equal(cli::num_ansi_colors(), 1)
  expect_false(has_color())
  expect_equal(num_colors(), 1)

  expect_snapshot({
    cli::num_ansi_colors()
    has_color()
    num_colors()
    style_na("NA")
  })
})

test_that("output test", {
  skip_if(getRversion() < "4.0")

  x <- list(
    column_zero_one = 1:3 + 0.23,
    col_02 = letters[1:3],
    col_03 = factor(letters[1:3]),
    col_04 = ordered(letters[1:3])
  )
  expect_snapshot({
    colonnade(x, width = 4)
    colonnade(x, width = 5)
    colonnade(x, width = 6)
    colonnade(x, width = 7)
    colonnade(x, width = 8)
    colonnade(x, width = 9)
    colonnade(x, width = 10)
    colonnade(x, width = 11)
    colonnade(x, width = 12)
    colonnade(x, width = 13)
    colonnade(x, width = 14)
    colonnade(x, width = 15)
    colonnade(x, width = 16)
    colonnade(x, width = 17)
    colonnade(x, width = 18)
    colonnade(x, width = 19)
    colonnade(x, width = 20)
    colonnade(x, width = 21)
    colonnade(x, width = 22)
    colonnade(x, width = 23)
    colonnade(x, width = 24)
    colonnade(x, width = 25)
    colonnade(x, width = 26)
    colonnade(x, width = 27)
    colonnade(x, width = 28)
    colonnade(x, width = 29)
    colonnade(x, width = 30)
    colonnade(x, width = 31)
    colonnade(x, width = 32)
    colonnade(x, width = 33)
    colonnade(x, width = 34)
    colonnade(x, width = 35)
    colonnade(x, width = 36)
    colonnade(x, width = 37)
    colonnade(x, width = 38)
    colonnade(x, width = 39)
    colonnade(x, width = Inf)
  })

  expect_snapshot({
    colonnade(rep(list(paste(letters, collapse = " ")), 4), width = Inf)
  })

  # Spurious warnings on Windows
  suppressWarnings(
    expect_snapshot({
      as_glue(extra_cols_impl(squeeze_impl(colonnade(x), width = 10)))
    })
  )

  suppressWarnings(
    expect_snapshot({
      as_glue(extra_cols_impl(squeeze_impl(colonnade(x), width = 20)))
    })
  )

  suppressWarnings(
    expect_snapshot({
      as_glue(extra_cols_impl(squeeze_impl(colonnade(x), width = 30)))
    })
  )

  suppressWarnings(
    expect_snapshot({
      as_glue(extra_cols_impl(squeeze_impl(colonnade(x), width = 35)))
    })
  )

  expect_snapshot({
    as_glue(extra_cols_impl(squeeze_impl(colonnade(x), width = 40)))
  })
})

test_that("tests from tibble", {
  skip_if(getRversion() < "4.0")

  skip_if_not_installed("rlang", "0.4.11.9000")
  local_options(width = 80)

  expect_snapshot({
    colonnade(mtcars[1:8, ], has_row_id = "*", width = 30)
    colonnade(trees[1:5, ], width = 20)
    colonnade(trees[1:3, ], width = 10)
    colonnade(df_all, width = 30)
    colonnade(df_all, width = 300)
    options(width = 70)
    colonnade(df_all, width = 300)
    options(width = 60)
    colonnade(df_all, width = 300)
    options(width = 50)
    colonnade(df_all, width = 300)
    options(width = 40)
    colonnade(df_all, width = 300)
    options(width = 30)
    colonnade(df_all, width = 300)
    options(width = 20)
    colonnade(df_all, width = 300)
    list_with_ctl <- list(c("\n", '"'), factor(c("\n", "\n")))
    names(list_with_ctl) <- c("\n", "\r")
    colonnade(list_with_ctl, width = 30)
    colonnade(list(a = c("", " ", "a ", " a")), width = 30)
    colonnade(list("mean(x)" = 5, "var(x)" = 3), width = 30)
  })
})

test_that("empty", {
  scoped_lifecycle_silence()
  skip_if(getRversion() < "4.0")

  expect_equal(
    format(colonnade(list(a = character(), b = logical()), width = 30)),
    as_glue(character())
  )
  expect_equal(
    format(colonnade(trees[1:5, character()], width = 30)),
    as_glue(character())
  )
})

test_that("NA names", {
  skip_if(getRversion() < "4.0")

  x <- list(`NA` = 1:3, set_to_NA = 4:6)
  names(x)[[2]] <- NA_character_
  expect_snapshot({
    colonnade(x, width = 30)
  })
})

test_that("sep argument", {
  skip_if(getRversion() < "4.0")

  x <- list(sep = 1:3)
  expect_snapshot({
    colonnade(x, width = 30)
    "dummy"
  })
})

test_that("color", {
  skip_if_not_installed("testthat", "3.1.1")

  local_colors()
  expect_equal(cli::num_ansi_colors(), 16)
  expect_true(has_color())
  expect_equal(num_colors(), 16)

  if (l10n_info()$`UTF-8`) {
    local_utf8()
    expect_true(cli::is_utf8_output())
  }

  expect_snapshot(variant = snapshot_variant("output-enc"), {
    cli::num_ansi_colors()
    has_color()
    num_colors()
    style_na("NA")
    style_neg("-1")
  })

  with_lifecycle_silence({
    xf <- colonnade(list(x = c((10^(-3:4)) * c(-1, 1), NA)))
  })

  expect_snapshot(variant = snapshot_variant("output-enc"), {
    print(xf)
    with_options(pillar.subtle_num = TRUE, print(xf))
    with_options(pillar.subtle = FALSE, print(xf))
    with_options(pillar.neg = FALSE, print(xf))
    with_options(pillar.subtle = FALSE, pillar.neg = FALSE, print(xf))
    with_options(pillar.bold = TRUE, print(xf))
  })

  skip_if(getRversion() < "4.0")

  expect_snapshot(variant = snapshot_variant("output-enc"), {
    colonnade(list(a_very_long_column_name = 0), width = 15)
  })
})

test_that("sanity check (2)", {
  expect_equal(cli::num_ansi_colors(), 1)
  expect_false(has_color())
  expect_equal(num_colors(), 1)

  expect_snapshot({
    cli::num_ansi_colors()
    has_color()
    num_colors()
    style_na("NA")
  })
})

test_that("tibble columns", {
  skip_if(getRversion() < "4.0")

  x <- list(a = 1:3, b = data.frame(c = 4:6, d = 7:9))
  expect_snapshot({
    colonnade(x, width = 30)
  })
})

test_that("tibble columns (nested)", {
  skip_if(getRversion() < "4.0")

  x <- list(
    a = 1:3,
    b = structure(
      list(
        c = 4:6, d = 7:9,
        e = data.frame(f = 10:12, g = 13:15)
      ),
      class = "data.frame"
    )
  )
  expect_snapshot({
    colonnade(x, width = 40)
  })
})

test_that("tibble columns (empty)", {
  skip_if(getRversion() < "4.0")

  x <- list(
    a = 1:3,
    b = structure(
      list(
        c = 4:6, d = 7:9,
        e = data.frame(f = 10:12)[, 0],
        f = 10:12
      ),
      class = "data.frame"
    )
  )
  expect_snapshot({
    colonnade(x, width = 40)
  })
})

test_that("matrix columns (unnamed)", {
  skip_if(getRversion() < "4.0")

  x <- list(a = 1:3, b = matrix(4:9, ncol = 2))
  expect_snapshot({
    colonnade(x, width = 30)
  })
})

test_that("matrix columns (named)", {
  skip_if(getRversion() < "4.0")

  x <- list(a = 1:3, b = matrix(4:9, ncol = 2, dimnames = list(NULL, c("c", "d"))))
  expect_snapshot({
    colonnade(x, width = 30)
  })
})

test_that("matrix columns (empty)", {
  x <- list(a = 1:3, b = matrix(4:6, ncol = 1)[, 0], c = 4:6)
  expect_snapshot({
    colonnade(x, width = 30)
  })
})

## 
## ## Integrating with other classes

## 
## ### gt

## 
## - Consumption via new `fmt_auto()`?

## 
## FIXME

## 
## ### units

## 
## xxx{r numbers-17}

## library(units)

## 
## set_units.pillar_num <- function(x, ...) {

##   unclassed <- x

##   class(unclassed) <- NULL

##   set_units(unclassed, ...)

## }

## 
## m <- set_units(1:3, m)

## km <- set_units(1:3, km)

## 
## tibble(

##   sci_int = set_num_opts(m + km, notation = "sci"),

##   digits_int = set_num_opts(km + m, digits = 4),

##   sci_ext = set_units(num(1:3 + 0.001, notation = "sci"), km)

## )

## 
## tibble(

##   sci_int = set_num_opts(m, notation = "sci") + km,

##   digits_int = set_num_opts(km, digits = 4) + m,

##   sci_ext = set_units(num(1:3, notation = "sci"), m) + km

## )

## xxx

## 
## ### formattable

## 
## xxx{r numbers-18, error = TRUE}

## library(formattable)

## 
## pillar_shaft.formattable <- function(x, ...) {

##   pillar::new_pillar_shaft_simple(format(x), align = "right")

## }

## 
## pillar_shaft.formattable_currency <- function(x, ...) {

##   formattable <- attr(x, "formattable", exact = TRUE)

## 
##   pillar_shaft(num(unclass(x), digits = formattable$digits))

## }

## 
## pillar_shaft.formattable_percent <- function(x, ...) {

##   formattable <- attr(x, "formattable", exact = TRUE)

## 
##   pillar_shaft(num(unclass(x), digits = formattable$digits, label = "%", scale = 100))

## }

## 
## pillar_shaft.formattable_scientific <- function(x, ...) {

##   pillar_shaft(num(unclass(x), notation = "sci"))

## }

## 
## type_sum.formattable <- function(x) {

##   formattable <- attr(x, "formattable", exact = TRUE)

## 
##   if (inherits(x, "formattable_currency")) {

##     I(sub("^formattable_", "", class(x)[[1]]))

##   } else if (inherits(x, "formattable_percent")) {

##     I("%")

##   } else {

##     abbreviate(sub("^formattable_", "", class(x)[[1]]), 4)

##   }

## }

## 
## num_currency(1:3 * 100 + 0.1)

## num_percent(1:3 * 0.1 + 0.001)

## num_scientific(1:3 * 0.1 + 0.001)

## 
## tibble(

##   currency = num_currency(1:3 * 100 + 0.1),

##   percent = num_percent(1:3 * 0.1 + 0.001),

##   scientific = num_scientific(1:3 * 0.1 + 0.001)

## )

## xxx

## 
## ### scales

## 
## xxx{r numbers-scales, error = TRUE}

## library(scales)

## 
## x <- num(1:10 / 100, label = "%", scale = 100)

## 
## scales::squish(x)

## 
## x < 0

## x < 0L

## 
## scales::cscale(x, scales::rescale_pal())

## xxx

## 
## ### ggplot2

## 
## xxx{r numbers-19}

## library(ggplot2)

## 
## scale_type.pillar_num <- function(x, ...) {

##   "continuous"

## }

## 
## data.frame(x = x, y = 1:10) %>%

##   ggplot(aes(x = x, y = y)) %>%

##   + geom_point()

## xxx

## 
## ## Rule-based decoration

## 
## 
## 
## xxx{r}

## library(dplyr)

## 
## data_units <-

##   palmerpenguins::penguins %>%

##   mutate(across(ends_with("_mm"), set_units, "mm")) %>%

##   mutate(across(ends_with("_g"), set_units, "g"))

## 
## data_units %>%

##   mutate(bill_area = bill_length_mm * bill_depth_mm, .after = island)

## xxx

## 
## xxx{r eval = FALSE}

## data_decor <-

##   data_units %>%

##   decorate(year, digits = 0) %>%

##   decorate(where(is.numeric), digits = 3)

## xxx

## 
## xxx{r eval = FALSE}

## data_decor %>%

##   mutate(bill_area = bill_length_mm * bill_depth_mm, .after = island)

## xxx

## 
## xxx{r echo = FALSE}

## data_units %>%

##   mutate(bill_area = bill_length_mm * bill_depth_mm, .after = island) %>%

##   mutate(across(year, set_num_opts, digits = 0)) %>%

##   mutate(across(where(is.numeric), set_num_opts, digits = 3))

## xxx

## 

#' Format the header of a tibble
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' For easier customization, the formatting of a tibble is split
#' into three components: header, body, and footer.
#' The `tbl_format_header()` method is responsible for formatting the header
#' of a tibble.
#'
#' Override this method if you need to change the appearance
#' of the entire header.
#' If you only need to change or extend the components shown in the header,
#' override or extend [tbl_sum()] for your class which is called by the
#' default method.
#'
#' @inheritParams tbl_format_body
#' @inherit tbl_format_body return
#'
#' @export
#' @examplesIf rlang::is_installed(c("palmerpenguins", "tibble"))
#' setup <- tbl_format_setup(palmerpenguins::penguins)
#' tbl_format_header(palmerpenguins::penguins, setup)
#'
#' # Shortcut for debugging
#' tbl_format_header(setup)
tbl_format_header <- function(x, setup, ...) {
  check_dots_empty()

  UseMethod("tbl_format_header")
}

#' @export
tbl_format_header.tbl <- function(x, setup, ...) {
  named_header <- setup$tbl_sum
  focus <- attr(x, "pillar_focus")
  if (!is.null(focus)) {
    named_header <- c(named_header, "Focus columns" = collapse(tick_if_needed(focus)))
  }

  if (all(names2(named_header) == "")) {
    header <- named_header
  } else {
    header <- paste0(
      align(paste0(names2(named_header), ":"), space = NBSP),
      # We add a space after the NBSP inserted by align()
      # so that wrapping occurs at the right location for very narrow outputs
      " ",
      named_header
    )
  }

  style_subtle(format_comment(header, width = setup$width))
}

#' @export
tbl_format_header.pillar_tbl_format_setup <- function(x, ...) {
  as_glue(c(
    cli::style_bold("<tbl_format_header(setup)>"),
    tbl_format_header(x$x, setup = x)
  ))
}

#' Customize the appearance of simple pillars in your tibble subclass
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Gain full control over the appearance of the pillars of your tibble subclass
#' in its body.
#' This method is intended for implementers of subclasses of the `"tbl"` class.
#' Users will rarely need them.
#'
#' @details
#' `ctl_new_pillar()` is called to construct pillars for regular (one-dimensional)
#' vectors.
#' The default implementation returns an object constructed with [pillar()].
#' Extend this method to modify the pillar components returned from the default
#' implementation.
#' Override this method to completely change the appearance of the pillars.
#' Components are created with [new_pillar_component()] or [pillar_component()].
#' In order to customize printing of row IDs, a method can be supplied for the
#' `ctl_new_rowid_pillar()` generic.
#'
#' All components must be of the same height.
#' This restriction may be levied in the future.
#'
#' Implementations should return `NULL` if none of the data
#' fits the available width.
#'
#' @inheritParams rlang::args_dots_empty
#' @param controller The object of class `"tbl"` currently printed.
#' @param x A simple (one-dimensional) vector.
#' @param width The available width, can be a vector for multiple tiers.
#' @param title The title, derived from the name of the column in the data.
#'
#' @seealso
#' See [ctl_new_pillar_list()] for creating pillar objects for compound columns:
#' packed data frames, matrices, or arrays.
#'
#' @export
#' @examplesIf rlang::is_installed("palmerpenguins") && requireNamespace("tibble")
#' # Create pillar objects
#' ctl_new_pillar(
#'   palmerpenguins::penguins,
#'   palmerpenguins::penguins$species[1:3],
#'   width = 60
#' )
#'
#' ctl_new_pillar(
#'   palmerpenguins::penguins,
#'   palmerpenguins::penguins$bill_length_mm[1:3],
#'   width = 60
#' )
#' @examples
#'
#' # Customize output
#' lines <- function(char = "-") {
#'   stopifnot(nchar(char) == 1)
#'   structure(char, class = "lines")
#' }
#'
#' format.lines <- function(x, width, ...) {
#'   paste(rep(x, width), collapse = "")
#' }
#'
#' ctl_new_pillar.line_tbl <- function(controller, x, width, ...) {
#'   out <- NextMethod()
#'   new_pillar(list(
#'     title = out$title,
#'     type = out$type,
#'     lines = new_pillar_component(list(lines("=")), width = 1),
#'     data = out$data
#'   ))
#' }
#'
#' ctl_new_rowid_pillar.line_tbl <- function(controller, x, width, ...) {
#'   out <- NextMethod()
#'   new_pillar(
#'     list(
#'       title = out$title,
#'       type = out$type,
#'       lines = new_pillar_component(list(lines("=")), width = 1),
#'       data = out$data
#'     ),
#'     width = as.integer(floor(log10(max(nrow(x), 1))) + 1)
#'   )
#' }
#'
#' vctrs::new_data_frame(
#'   list(a = 1:3, b = letters[1:3]),
#'   class = c("line_tbl", "tbl")
#' )
#'
#' ctl_new_rowid_pillar.roman_tbl <- function(controller, x, width, ...) {
#'   out <- NextMethod()
#'   rowid <- utils::as.roman(seq_len(nrow(x)))
#'   width <- max(nchar(as.character(rowid)))
#'   new_pillar(
#'     list(
#'       title = out$title,
#'       type = out$type,
#'       data = pillar_component(
#'         new_pillar_shaft(list(row_ids = rowid),
#'           width = width,
#'           class = "pillar_rif_shaft"
#'         )
#'       )
#'     ),
#'     width = width
#'   )
#' }
#'
#' vctrs::new_data_frame(
#'   list(a = 1:3, b = letters[1:3]),
#'   class = c("roman_tbl", "tbl")
#' )
#'
ctl_new_pillar <- function(controller, x, width, ..., title = NULL) {
  "!!!!DEBUG ctl_new_pillar(`v(width)`, `v(title)`)"

  check_dots_empty()

  UseMethod("ctl_new_pillar")
}

#' @export
ctl_new_pillar.tbl <- function(controller, x, width, ..., title = NULL) {
  "!!!!DEBUG ctl_new_pillar.tbl(`v(width)`, `v(title)`)"

  pillar(x, title, if (!is.null(width)) max0(width))
}

#' @param type String for specifying a row ID type. Current values in use are
#' `NULL` and `"*"`.
#' @rdname ctl_new_pillar
#' @export
ctl_new_rowid_pillar <- function(controller, x, width, ..., title = NULL, type = NULL) {
  "!!!!DEBUG ctl_new_rowid_pillar(`v(width)`, `v(title)`)"

  check_dots_empty()

  if (length(width) == 0) {
    return(NULL)
  }

  UseMethod("ctl_new_rowid_pillar")
}

#' @export
ctl_new_rowid_pillar.tbl <- function(controller, x, width, ..., title = NULL, type = NULL) {
  "!!!!DEBUG ctl_new_rowid_pillar.tbl(`v(width)`, `v(title)`)"

  template <- names(ctl_new_pillar(controller, vector(), width, title = title))

  if (!length(template)) {
    return(NULL)
  }

  data <- rif_shaft(nrow(x))

  out <- map(set_names(template), function(.x) "")

  if ("type" %in% template) {
    out$type <- pillar_component(rif_type(identical(type, "*")))
  }

  if ("data" %in% template) {
    out$data <- pillar_component(data)
  }

  new_pillar(out, width = get_width(data))
}

max0 <- function(x) {
  if (length(x) > 0) {
    max(x)
  } else {
    0L
  }
}

#' @aliases NULL pillar-package
#'
#' @description
#' `r lifecycle::badge("stable")`
#'
#' Formats tabular data in columns or rows
#' using the full range of colours provided by modern terminals.
#' Provides various generics for making every aspect of the display customizable.
#'
#' @seealso
#' - [pillar()] for formatting a single column,
#' - [print.tbl()] for formatting data-frame-like objects,
#' - [pillar_options] for a list of package options.
#'
#' @examplesIf rlang::is_installed("tibble")
#' pillar(1:3)
#' pillar(c(1, 2, 3))
#' pillar(factor(letters[1:3]), title = "letters")
#' tbl_format_setup(tibble::as_tibble(mtcars), width = 60)
"_PACKAGE"

format_title <- function(x, width) {
  out <- align(str_trunc(x, width))

  # HACK: Abbreviating text inbetween ticks
  ticked <- grepl("^`", x)
  if (!any(ticked)) {
    return(out)
  }

  ticked[which(get_extent(x[ticked]) <= width)] <- FALSE
  if (!any(ticked)) {
    return(out)
  }

  x_ticked <- x[ticked]
  rx <- "^`(.*)(`[^`]*)$"
  match <- gsub(rx, "\\1", x[ticked])
  rest <- gsub(rx, "\\2", x[ticked])

  short <- str_trunc(match, width + get_extent(match) - get_extent(x_ticked))
  out[ticked] <- align(paste0("`", short, rest))
  out
}

tick_names_if_needed <- function(x) {
  names(x) <- tick_if_needed(names(x))
  x
}

tick_if_needed <- function(x) {
  # Compatibility with R 3.4
  if (is.null(x)) {
    return(NULL)
  }
  needs_ticks <- !is_syntactic(x)
  x[needs_ticks] <- tick(x[needs_ticks])
  x
}

is_syntactic <- function(x) {
  # Compatibility with R 3.4
  if (is.null(x)) {
    return(logical())
  }
  ret <- make.names(x) == x
  ret[is.na(x)] <- FALSE
  ret
}

is_ambiguous_string <- function(x) {
  !is.na(x) & grepl("^$|^ | $|\\\\|\"", x)
}

tick <- function(x) {
  ifelse(is.na(x), "NA", encodeString(x, quote = "`"))
}

context("ticks")

test_that("title ticks without width restriction", {
  expect_equal(tick_if_needed("proper_title"), "proper_title")
  expect_equal(tick_if_needed("needs ticks"), "`needs ticks`")
  expect_equal(tick_if_needed("'ticks'"), "`'ticks'`")
  expect_equal(tick_if_needed("embedded\nnewline"), "`embedded\\nnewline`")
})

test_that("title ticks and width", {
  expect_equal(format_title("proper_title", 15), "proper_title")
  expect_equal(format_title("proper_title", 12), "proper_title")
  expect_equal(format_title("proper_title", 10), continue("proper_ti"))
  expect_equal(format_title("`a b`", 6), "`a b`")
  expect_equal(format_title("`a b`", 5), "`a b`")
  expect_equal(format_title("`a b`", 4), continue("`a "))
  expect_equal(format_title("`a b`", 3), continue("`a"))
})

test_that("title ticks without width restriction", {
  expect_equal(tick_if_needed("proper_title"), "proper_title")
  expect_equal(tick_if_needed("needs ticks"), "`needs ticks`")
  expect_equal(tick_if_needed("'ticks'"), "`'ticks'`")
  expect_equal(tick_if_needed("embedded\nnewline"), "`embedded\\nnewline`")
})

test_that("title ticks and width with footnote", {
  skip_if(!cli::is_utf8_output())

  expect_snapshot({
    format_title("proper_title", 15)
    format_title("proper_title", 12)
    format_title("proper_title", 10)
    format_title("`a b`", 6)
    format_title("`a b`", 5)
    format_title("`a b`", 4)
    format_title("`a b`", 3)
  })
})

test_that("title ticks and width", {
  expect_snapshot({
    format_title("proper_title", 15, footnote = FALSE)
    format_title("proper_title", 12, footnote = FALSE)
    format_title("proper_title", 10, footnote = FALSE)
    format_title("`a b`", 6, footnote = FALSE)
    format_title("`a b`", 5, footnote = FALSE)
    format_title("`a b`", 4, footnote = FALSE)
    format_title("`a b`", 3, footnote = FALSE)
    format_title("`a b`", 3, footnote = FALSE)
  })
})

style_star <- function(x) {
  style_hint(x)
}

rif_type <- function(has_star, ...) {
  star <- if (has_star) "*" else ""
  ret <- structure(
    list(
      star = star
    ),
    class = "pillar_rif_type"
  )
  ret <- set_width(ret, 1L)
  ret
}

#' @export
format.pillar_rif_type <- function(x, width = NULL, ...) {
  style_star(x$star)
}

show_output_in_terminal <- function() {
  system2("xterm", c("-e", shQuote("head tests/testthat/out/*; sleep 600")))
}

# A data frame with all major types
df_all <- list(
  a = c(1, 2.5, NA),
  b = c(1:2, NA),
  c = c(T, F, NA),
  d = c("a", "b", NA),
  e = factor(c("a", "b", NA)),
  f = as.Date("2015-12-09") + c(1:2, NA),
  g = as.POSIXct("2015-12-09 10:51:34.5678 UTC") + c(1:2, NA),
  h = as.list(c(1:2, NA)),
  i = list(list(1, 2:3), list(4:6), list(NA))
)

# A data frame with strings of varying lengths
long_str <- strrep("Abcdefghij", 5)
df_str <- map(rlang::set_names(1:50), function(i) substr(long_str, 1, i))

expect_pillar_output <- function(x = NULL, ..., filename, xp = NULL, xf = NULL,
                                 crayon = TRUE, output_width = 80L) {
  x <- rlang::enquo(x)
  dots <- rlang::enquos(...)
  xp <- rlang::enquo(xp)
  xf <- rlang::enquo(xf)
  object_quo <- rlang::quo(get_pillar_output_object(!!x, !!!dots, xp = !!xp, xf = !!xf))

  expect_pillar_output_utf8(object_quo, filename, output_width)
  expect_pillar_output_latin1(object_quo, filename, output_width)
}

expect_pillar_output_utf8 <- function(object_quo, filename, output_width) {
  if (l10n_info()$`UTF-8`) {
    expect_known_display(
      object = !!object_quo,
      file = file.path("out", filename),
      crayon = TRUE,
      width = output_width
    )

    expect_known_display(
      object = !!object_quo,
      file = file.path("bw-out", filename),
      crayon = FALSE,
      width = output_width
    )
  }
}

expect_pillar_output_latin1 <- function(object_quo, filename, output_width) {
  if (.Platform$OS.type == "windows") {
    expect_known_display(
      object = !!object_quo,
      file = file.path("out-native", filename),
      crayon = FALSE,
      width = output_width
    )
  }
}

get_pillar_output_object <- function(x = NULL, xp = NULL, xf = NULL, ...) {
  if (is.null(xf)) {
    if (is.null(xp)) {
      xp <- add_special(x)
    }
    xf <- pillar(xp, ...)
  }

  xf
}

#' `add_special()` is not exported, and used only for initializing default
#' values to `expect_pillar_output()`.
#' @rdname expect_pillar_output
add_special <- function(x) {
  if (inherits(x, "integer64")) {
    x <- c(x, bit64::NA_integer64_)
  } else {
    x <- x[seq2(1, length(x) + 1)]
    if (is.numeric(x) && is.double(x)) {
      x <- c(x, -Inf, Inf)
    }
  }
  x
}

continue <- function(x) {
  paste0(x, cli::symbol$continue)
}

without_color <- function(code) {
  old <- options(crayon.enabled = FALSE)
  has_color(forget = TRUE)
  on.exit({ options(old); has_color(forget = TRUE) })

  code
}

context("format_factor")

test_that("output test", {
  expect_pillar_output(xp = factor(c(letters[1:5], NA)), filename = "factor.txt")
  expect_pillar_output(xp = ordered(c(letters[1:5], NA)), filename = "ordered.txt")
  expect_pillar_output(xp = factor("a\nb"), filename = "escaped.txt")
})

context("title")

withr::with_options(list(pillar.bold = TRUE), {

  test_that("with and without title", {
    expect_pillar_output(10^(1:6), filename = "title-none.txt", crayon = FALSE)
    expect_pillar_output(10^(1:6), title = "crayon", filename = "title-crayon.txt")
    expect_pillar_output(10^(1:6), title = "short", filename = "title-short.txt", crayon = FALSE)
    expect_pillar_output(10^(1:6), title = "somewhat_wider", filename = "title-longer.txt", crayon = FALSE)
    expect_pillar_output(10^(1:6), title = "exactly_fifteen", filename = "title-fifteen.txt", crayon = FALSE)
    expect_pillar_output(10^(1:6), title = "absolutely_breaking_all_sensible_boundaries", width = 18, filename = "title-too-long.txt", crayon = FALSE)
  })
}) # withr::with_options(list(pillar.bold = TRUE), {

test_that("integer output will use scientific if necessary", {
  x <- 10000000L + 1:3
  expect_pillar_output(x, width = 6, filename = "integer-06.txt")
  expect_pillar_output(x, width = 7, filename = "integer-07.txt")
  expect_pillar_output(x, width = 8, filename = "integer-08.txt")
  expect_pillar_output(x, width = 9, filename = "integer-09.txt")
})

context("format_time")

test_that("output test", {
  expect_pillar_output(as.POSIXct("2017-07-28 18:04:35 +0200"), filename = "time.txt")
  expect_pillar_output(as.POSIXlt("2017-07-28 18:04:35 +0200"), filename = "time-posix.txt")
  withr::with_options(
    list(digits.secs = 4),
    expect_pillar_output(as.POSIXlt("2017-07-28 18:04:35 +0200"), filename = "time-digits-secs.txt")
  )

  testthat::skip_if(getRversion() < "3.3")
  expect_pillar_output(as.difftime(8:11, units = "secs"), filename = "difftime.txt")
})

#' @description
#' `new_pillar_shaft_simple()` provides an implementation of the `pillar_shaft`
#' class suitable for output that has a fixed formatting, which will be
#' truncated with a continuation character (ellipsis or `~`) if it doesn't fit
#' the available width.
#' By default, the required width is computed from the natural width of the
#' `formatted` argument.
#'
#' @details
#' The `formatted` argument may also contain ANSI escapes to change color
#' or other attributes of the text, provided e.g. by the \pkg{cli} package.
#'
#' @inheritParams tibble::char
#' @param ... Passed on to [new_pillar_shaft()].
#' @param formatted The data to show, an object coercible to [character].
#' @param align Alignment of the column.
#' @param na String to use as `NA` value, defaults to `"NA"` styled with
#'   [style_na()] with fallback if color is not available.
#' @param na_indent Indentation of `NA` values.
#' @param short_formatted If provided, a character vector of the same length as
#'   `formatted`, to be used when the available width is insufficient to show
#'   the full output.
#' @export
#' @rdname new_pillar_shaft
new_pillar_shaft_simple <- function(formatted, ..., width = NULL, align = "left",
                                    min_width = NULL, na = NULL, na_indent = 0L,
                                    shorten = c("back", "front", "mid", "abbreviate"),
                                    short_formatted = NULL) {
  formatted <- as.character(formatted)

  if (is.null(width)) {
    width <- get_max_extent(formatted)
  }

  if (is.null(na)) {
    na <- pillar_na()
  }

  if (missing(shorten)) {
    shorten <- NULL
  } else if (!is.null(shorten)) {
    shorten <- arg_match(shorten)
  }

  if (!is.null(short_formatted)) {
    # stopifnot(get_extent(short_formatted) <= get_extent(formatted))
    if (is.null(min_width)) {
      min_width <- get_max_extent(short_formatted)
    }
  }

  new_pillar_shaft(
    list(formatted),
    ...,
    width = width,
    min_width = min_width,
    align = align,
    na = na,
    na_indent = na_indent,
    shorten = shorten,
    short_formatted = short_formatted,
    class = "pillar_shaft_simple"
  )
}

#' @export
format.pillar_shaft_simple <- function(x, width, ...) {
  align <- attr(x, "align", exact = TRUE)
  shorten <- attr(x, "shorten", exact = TRUE)
  short_formatted <- attr(x, "short_formatted", exact = TRUE)
  desired_width <- get_width(x)
  shaft <- as.character(x[[1]])
  if (width < desired_width) {
    if (is.null(short_formatted)) {
      shaft <- str_trunc(shaft, width, shorten)
    } else {
      short_idx <- !is.na(shaft) & (get_extent(shaft) > width)
      shaft[short_idx] <- short_formatted[short_idx]
    }
  }

  shaft[is.na(shaft)] <- paste0(
    strrep(" ", attr(x, "na_indent", exact = TRUE)),
    attr(x, "na", exact = TRUE)
  )

  new_ornament(shaft, width = width, align = align)
}

MIN_PILLAR_WIDTH <- 5L

style_type <- function(x) {
  force(x)
  x <- style_subtle(x)
  crayon_italic(x)
}

#' Prepare a column type for formatting
#'
#' Calls [type_sum()] to format the type.
#' Call [format()] on the result to render column types.
#'
#' @param x A vector for which the type is to be retrieved.
#' @inheritParams rlang::args_dots_empty
#' @export
#' @examples
#' format(new_pillar_type("a"))
#' format(new_pillar_type(factor("a")))
new_pillar_type <- function(x, ...) {
  "!!!!DEBUG new_pillar_type(`v(class(x))`)"
  if (!missing(...)) {
    check_dots_empty(action = warn)
  }

  new_pillar_type_obj(get_pillar_type(x))
}

new_pillar_type_obj <- function(type) {
  # Must wrap in a list, because type_sum() can return a classed object
  ret <- structure(list(type), class = "pillar_type")
  extent <- get_extent(format_type_sum(type, NULL))
  ret <- set_width(ret, width = max(extent, MIN_PILLAR_WIDTH))
  ret <- set_min_width(ret, MIN_PILLAR_WIDTH)
  ret
}

get_pillar_type <- function(x) {
  type <- type_sum(x)
  if (length(type) == 0L) type <- "?"
  # Can return a classed object to be formatted by format_type_sum()
  type[] <- as.character(type[[1L]])
  type
}

#' @export
format.pillar_type <- function(x, width = NULL, ..., is_focus = FALSE) {
  out <- format_type_sum(x[[1]], width)
  if (is_focus) {
    out <- crayon_underline(out)
  }
  out
}

format_full_pillar_type <- function(x) {
  type <- get_pillar_type(x)
  format_type_sum(type, NULL)
}

#' Format a type summary
#'
#' Called on values returned from [type_sum()] for defining the description
#' in the capital.
#'
#' Two methods are implemented by default for this generic: the default method,
#' and the method for the `"AsIs"` class.
#' Return `I("type")` from your [type_sum()] implementation to format the type
#' without angle brackets.
#' For even more control over the formatting, implement your own method.
#'
#' @param x A return value from `type_sum()`
#' @param width The desired total width. If the returned string still is
#'   wider, it will be trimmed. Can be `NULL`.
#' @inheritParams rlang::args_dots_used
#'
#' @export
#' @examples
#' # Default method: show the type with angle brackets
#' writeLines(format_type_sum("dbl", width = NULL))
#' pillar(1)
#'
#' # AsIs method: show the type without angle brackets
#' type_sum.accel <- function(x) {
#'   I("kg m/s^2")
#' }
#'
#' # Typically done through NAMESPACE
#' # (perhaps with an @export directive in roxygen2)
#' registerS3method("type_sum", "accel", type_sum.accel)
#'
#' accel <- structure(9.81, class = "accel")
#' pillar(accel)
format_type_sum <- function(x, width, ...) {
  if (!missing(...)) {
    check_dots_used(action = warn)
  }

  UseMethod("format_type_sum")
}

#' @export
#' @rdname format_type_sum
format_type_sum.default <- function(x, width, ...) {
  if (!is.null(width) && width - 2 < get_extent(x)) {
    x <- substr(x, 1, max(width - 2, 0))
  }
  style_type(paste0("<", x, ">"))
}

#' @export
#' @rdname format_type_sum
format_type_sum.AsIs <- function(x, width, ...) {
  if (!is.null(width) && width < get_extent(x)) {
    x <- substr(x, 1, width)
  }
  style_type(unclass(x))
}

test_that("can format new_pillar_type()", {
  expect_equal(
    format(new_pillar_type(1:3)),
    format_full_pillar_type(1:3)
  )
  expect_equal(
    format(new_pillar_type(list(1, 2:3))),
    format_full_pillar_type(list(1, 2:3))
  )
})

#' Customize the appearance of compound pillars in your tibble subclass
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Gain full control over the appearance of the pillars of your tibble subclass
#' in its body.
#' This method is intended for implementers of subclasses of the `"tbl"` class.
#' Users will rarely need them, and we also expect the default implementation
#' to be sufficient for the vast majority of cases.
#'
#' @details
#' `ctl_new_pillar_list()` is called to construct a list of pillars.
#' If `x` is a regular (one-dimensional) vector, the list contains one pillar
#' constructed by [ctl_new_pillar()].
#' This method also works for compound columns: columns that are data frames,
#' matrices or arrays, with the following behavior:
#'
#' - If `width` is `NULL`, the method always returns a list of length one
#'   containing one pillar object that represents the first sub-column in this
#'   compound column.
#' - Otherwise, the returned list contains one pillar object for all sub-columns
#'   that can be fit in the available horizontal space.
#'   These pillar objects are obtained by calling `ctl_new_pillar_list()`
#'   with `width = NULL` on each sub-column until the available width is
#'   exhausted.
#'
#' This method is called to initiate the construction of all pillars
#' in the tibble to be printed.
#' To ensure that all packed columns that fit the available space are printed,
#' `ctl_new_pillar_list()` may be called twice on the same input:
#' once with `width = NULL`, and
#' once with `width` corresponding to the then known available space
#' and with `first_pillar` set to the pillar object constructed in the
#' first call.
#'
#' @inheritParams ctl_new_pillar
#' @param x A vector, can also be a data frame, matrix, or array.
#' @param width The available width, can be a vector for multiple tiers.
#'   If `NULL`, only the first pillar is instantiated.
#' @param first_pillar Can be passed to this method if the first pillar
#'   for a compound pillar (or the pillar itself for a simple pillar)
#'   has been constructed already.
#' @export
#' @examplesIf rlang::is_installed("palmerpenguins") && requireNamespace("tibble")
#' # Simple column
#' ctl_new_pillar_list(
#'   tibble::tibble(),
#'   palmerpenguins::penguins$weight[1:3],
#'   width = 10
#' )
#'
#' # Packed data frame: unknown width
#' ctl_new_pillar_list(
#'   tibble::tibble(),
#'   palmerpenguins::penguins[1:3, ],
#'   width = NULL
#' )
#'
#' # Packed data frame: known width
#' ctl_new_pillar_list(
#'   tibble::tibble(),
#'   palmerpenguins::penguins,
#'   width = 60
#' )
#'
#' # Deeply packed data frame with known width:
#' # showing only the first sub-column even if the width is sufficient
#' ctl_new_pillar_list(
#'   tibble::tibble(),
#'   tibble::tibble(x = tibble::tibble(b = 1, c = 2), y = 3),
#'   width = 60
#' )
#'
#' # Packed matrix: unknown width
#' ctl_new_pillar_list(tibble::tibble(), matrix(1:6, ncol = 2), width = NULL)
#'
#' # Packed matrix: known width
#' ctl_new_pillar_list(tibble::tibble(), matrix(1:6, ncol = 2), width = 60)
#'
#' # Packed array
#' ctl_new_pillar_list(tibble::tibble(), Titanic, width = 60)
ctl_new_pillar_list <- function(controller, x, width, ..., title = NULL, first_pillar = NULL) {
  "!!!!DEBUG ctl_new_pillar_list(`v(width)`, `v(title)`)"

  check_dots_empty()

  UseMethod("ctl_new_pillar_list")
}

#' @export
ctl_new_pillar_list.tbl <- function(controller, x, width, ..., title = NULL, first_pillar = NULL) {
  "!!!!DEBUG ctl_new_pillar_list.tbl(`v(width)`, `v(title)`)"

  if (is.data.frame(x)) {
    new_data_frame_pillar_list(x, controller, width, title = title, first_pillar = first_pillar)
  } else if (is.matrix(x) && !inherits(x, c("Surv", "Surv2"))) {
    new_matrix_pillar_list(x, controller, width, title = title, first_pillar = first_pillar)
  } else if (is.array(x) && length(dim(x)) > 2) {
    new_array_pillar_list(x, controller, width, title = title, first_pillar = first_pillar)
  } else {
    if (is.null(first_pillar)) {
      first_pillar <- ctl_new_pillar(controller, x, width, ..., title = prepare_title(title))
    }
    new_single_pillar_list(first_pillar, width)
  }
}

new_data_frame_pillar_list <- function(x, controller, width, title, first_pillar = NULL) {
  "!!!!!DEBUG new_data_frame_pillar_list(`v(width)`, `v(title)`)"

  if (ncol(x) == 0) {
    pillar <- pillar_from_shaft(
      new_pillar_title(prepare_title(title)),
      new_pillar_type(x),
      new_empty_shaft(nrow(x)),
      width
    )
    return(new_single_pillar_list(pillar, width))
  }

  max_n_pillars <- sum(width %/% 2)
  pillars <- vector("list", max_n_pillars)

  ticked_names <- tick_if_needed(names(x))

  for (i in seq_along(x)) {
    "!!!!!DEBUG i = `i`, width = `width`"

    if (i == 1 && !is.null(first_pillar)) {
      pillar <- first_pillar
    } else {
      # Call ctl_new_pillar_list(), return only the first sub-pillar
      # thanks to width = NULL
      new_pillars <- ctl_new_pillar_list(
        controller, x[[i]],
        width = NULL,
        title = c(title, ticked_names[[i]])
      )

      # Safety check:
      if (length(new_pillars) == 0) {
        # NULL return: doesn't fit
        break
      }

      pillar <- new_pillars[[1]]
    }

    if (is.null(pillar)) {
      # NULL return: doesn't fit
      break
    }

    if (is.null(width)) {
      return(list(pillar))
    }

    # Compute remaining width
    new_width <- deduct_width(width, pillar_get_min_width(pillar))
    if (is.null(new_width)) {
      # NULL return: current pillar doesn't fit
      break
    }

    pillars[[i]] <- pillar
    width <- new_width

    if (!is.null(title)) {
      title[] <- ""
    }
  }

  pillars <- compact(pillars)

  new_pillar_list(
    pillars,
    extra = names(x)[seq2(length(pillars) + 1, length(x))],
    remaining_width = width
  )
}

new_matrix_pillar_list <- function(x, controller, width, title, first_pillar = NULL) {
  if (ncol(x) == 0) {
    pillar <- pillar_from_shaft(
      new_pillar_title(prepare_title(title)),
      new_pillar_type(x),
      new_empty_shaft(nrow(x)),
      width
    )
    return(new_single_pillar_list(pillar, width))
  }

  max_n_pillars <- sum(width %/% 2)
  pillars <- vector("list", max_n_pillars)

  idx <- colnames(x)
  if (is.null(idx)) {
    idx <- seq_len(ncol(x))
  } else {
    idx <- encodeString(idx, quote = '"')
  }

  ticked_names <- paste0("[,", idx, "]")

  for (i in seq_along(ticked_names)) {
    if (i == 1 && !is.null(first_pillar)) {
      pillar <- first_pillar
    } else {
      # FIXME
      # sub_title <- c(title, ticked_names[[i]])
      sub_title <- ticked_names[[i]]
      if (!is.null(title)) {
        if (i > 1) {
          title[[length(title)]] <- ""
        }
        sub_title <- c(title, sub_title)
      }

      # Call ctl_new_pillar() only for objects that can fit
      pillar <- ctl_new_pillar(
        controller, x[, i],
        width = NULL,
        title = prepare_title(sub_title)
      )
    }

    if (is.null(pillar)) {
      # NULL return: doesn't fit
      break
    }

    if (is.null(width)) {
      return(list(pillar))
    }

    # Compute remaining width
    new_width <- deduct_width(width, pillar_get_min_width(pillar))
    if (is.null(new_width)) {
      # NULL return: current pillar doesn't fit
      break
    }

    pillars[[i]] <- pillar
    width <- new_width
  }

  pillars <- compact(pillars)
  new_pillar_list(
    pillars,
    extra = seq2(length(pillars) + 1, ncol(x)),
    remaining_width = width
  )
}

new_array_pillar_list <- function(x, controller, width, title, first_pillar = NULL) {
  if (!is.null(first_pillar)) {
    return(list(first_pillar))
  }

  first_slice <- head(as.vector(x), vec_size(x))

  body <- pillar_shaft(first_slice)

  pillar <- pillar_from_shaft(
    new_pillar_title(title),
    new_pillar_type(x),
    new_continuation_shaft(body),
    width
  )
  new_single_pillar_list(pillar, width)
}

# Can be rewritten with a repeat loop
deduct_width <- function(width, consumed_widths) {
  "!!!!!DEBUG deduct_width(`v(width)`, `v(consumed_widths)`)"

  if (length(consumed_widths) == 0) {
    # All sub-pillars distributed
    width
  } else if (length(width) == 0) {
    # Doesn't fit
    NULL
  } else if (width[[1]] >= consumed_widths[[1]]) {
    # Fits first tier
    deduct_width(
      c(max(width[[1]] - consumed_widths[[1]] - 1L, 0), width[-1]),
      consumed_widths[-1]
    )
  } else {
    # Doesn't fit first tier: try next tier
    deduct_width(width[-1], consumed_widths)
  }
}

new_single_pillar_list <- function(pillar, width) {
  pillar_width <- pillar_get_min_width(pillar)
  if (is.null(width)) {
    new_width <- NULL
  } else {
    new_width <- deduct_width(width, pillar_width)
    stopifnot(!is.null(new_width))
  }
  new_pillar_list(
    list(pillar),
    extra = NULL,
    remaining_width = new_width,
    simple = TRUE
  )
}

new_pillar_list <- function(pillar_list, extra, remaining_width, simple = FALSE) {
  structure(
    pillar_list,
    extra = extra,
    remaining_width = remaining_width,
    simple = simple
  )
}

prepare_title <- function(title) {
  n_title <- length(title)
  if (n_title == 0) {
    title
  } else if (grepl("^[[]", title[[n_title]])) {
    paste0(paste(title[-n_title], collapse = "$"), title[[n_title]])
  } else {
    paste(title, collapse = "$")
  }
}

#' Format multiple vectors in a tabular display
#'
#' @description
#' `r lifecycle::badge("soft-deprecated")`
#'
#' The vectors are formatted to fit horizontally into a user-supplied number of
#' characters per row.
#'
#' The `colonnade()` function doesn't process the input but returns an object
#' with a [format()] and a [print()] method.
#' The implementations call [squeeze()] to create [pillar] objects and fit them to a given width.
#'
#' @param x A list, which can contain matrices or data frames.
#'   If named, the names will be used as title for the pillars. Non-syntactic names
#'   will be escaped.
#' @param has_row_id Include a column indicating row IDs? Pass `"*"` to mark
#'   the row ID column with a star.
#' @param width Default width of the entire output, optional.
#' @inheritParams rlang::args_dots_empty
#' @keywords internal
#' @export
colonnade <- function(x, has_row_id = TRUE, width = NULL, ...) {
  deprecate_soft("1.7.0", "pillar::colonnade()", "pillar::tbl_format_setup()")

  if (!missing(...)) {
    check_dots_empty(action = warn)
  }

  # Reset local cache for each new colonnade
  num_colors(forget = TRUE)

  x <- flatten_colonnade(x)
  ret <- new_data_frame(x, has_row_id = has_row_id, class = "pillar_colonnade")
  ret <- set_width(ret, width)
  ret
}

flatten_colonnade <- function(x) {
  out <- map2(
    unname(x),
    names2(x),
    flatten_column
  )

  vec_rbind(
    !!!out,
    # .ptype = data_frame(names = list(), data = list())
    .ptype = data_frame(names = character(), data = list())
  )
}

flatten_column <- function(x, name) {
  if (name != "") {
    name <- tick_if_needed(name)
  }

  if (is.data.frame(x)) {
    flatten_df_column(x, name)
  } else if (is.matrix(x) && !inherits(x, c("Surv", "Surv2"))) {
    flatten_matrix_column(x, name)
  } else {
    # Length-one list, will be unlist()ed afterwards
    # data_frame(names = list(name), data = list(x))
    data_frame(names = name, data = list(x))
  }
}

flatten_df_column <- function(x, name) {
  if (length(x) == 0) {
    # data_frame(names = list(name), data = list(new_empty_col_sentinel(x)))
    data_frame(names = name, data = list(new_empty_col_sentinel(x)))
  } else {
    x <- flatten_colonnade(unclass(x))
    # x$names <- map(x$names, function(.x) c(name, .x))
    x$names <- paste0("$", x$names)
    x$names[[1]] <- paste0(name, x$names[[1]])
    x
  }
}

flatten_matrix_column <- function(x, name) {
  if (ncol(x) == 0) {
    data_frame(
      # names = list(c(name, "[,0]")),
      names = name,
      data = list(new_empty_col_sentinel(x))
    )
  } else {
    x_list <- map(seq_len(ncol(x)), function(i) x[, i])

    idx <- colnames(x)
    if (is.null(idx)) {
      idx <- seq_along(x_list)
    } else {
      idx <- encodeString(idx, quote = '"')
    }

    # names <- map(idx, function(.x) c(name, .x))
    names <- paste0("[,", idx, "]")
    names[[1]] <- paste0(name, names[[1]])

    data_frame(names = names, data = x_list)
  }
}

new_empty_col_sentinel <- function(type) {
  structure(list(type), class = c("pillar_empty_col"))
}

#' Squeeze a colonnade to a fixed width
#'
#' @description
#' `r lifecycle::badge("soft-deprecated")`
#'
#' The `squeeze()` function usually doesn't need to be called manually.
#' It returns an object suitable for printing and formatting at a fixed width
#' with additional information about omitted columns, which can be retrieved
#' via [extra_cols()].
#'
#' @keywords internal
#' @export
squeeze <- function(x, width = NULL, ...) {
  deprecate_soft("1.5.0", "pillar::squeeze()")

  squeeze_impl(x, width, ...)
}

squeeze_impl <- function(x, width = NULL, ...) {
  # Shortcut for zero-height corner case
  zero_height <- length(x$data) == 0L || length(x$data[[1]]) == 0L
  if (zero_height) {
    return(new_colonnade_squeezed(list(), colonnade = x, extra_cols = seq_along(x$data)))
  }

  if (is.null(width)) {
    width <- get_width(x)
  }

  if (is.null(width)) {
    width <- getOption("width")
  }

  rowid <- get_rowid_from_colonnade(x)
  if (is.null(rowid)) {
    rowid_width <- 0
  } else {
    rowid_width <- max(get_widths(rowid)) + 1L
  }

  col_widths <- colonnade_get_width(x, width, rowid_width)
  col_widths_shown <- col_widths[!safe_is_na(col_widths$tier), ]
  indexes <- split(seq_along(col_widths_shown$tier), col_widths_shown$tier)

  out <- map(indexes, function(i) {
    inner <- map2(col_widths_shown$pillar[i], col_widths_shown$width[i], pillar_format_parts)
    if (!is.null(rowid)) {
      inner <- c(list(pillar_format_parts(rowid, rowid_width - 1L)), inner)
    }
    inner
  })

  n_cols_shown <- nrow(col_widths_shown)
  extra_cols <- seq2(n_cols_shown + 1L, length(x$data))
  new_colonnade_squeezed(out, colonnade = x, extra_cols = extra_cols)
}

get_rowid_from_colonnade <- function(x) {
  has_title <- any(x$names != "")

  has_row_id <- attr(x, "has_row_id", exact = TRUE)
  if (!is_false(has_row_id) && length(x$data) > 0) {
    rowid <- rowidformat(
      length(x$data[[1]]),
      has_star = identical(has_row_id, "*"),
      has_title_row = has_title
    )
  } else {
    rowid <- NULL
  }

  rowid
}

new_colonnade_squeezed <- function(x, colonnade, extra_cols) {
  formatted_tiers <- map(x, format_colonnade_tier)
  formatted <- as_glue(as.character(unlist(formatted_tiers)))

  structure(
    list(formatted),
    extra_cols = colonnade[extra_cols, ],
    class = "pillar_squeezed_colonnade"
  )
}

format_colonnade_tier <- function(x) {
  "!!!!!DEBUG format_colonnade_tier(`v(x)`)"

  if (length(x) == 0) {
    return(character())
  }

  unlist(pmap(unname(x), paste))
}

#' @export
format.pillar_squeezed_colonnade <- function(x, ...) {
  x[[1]]
}

#' @export
print.pillar_squeezed_colonnade <- function(x, ...) {
  print(format(x, ...), ...)
  invisible(x)
}

# Method registration happens in .onLoad()
knit_print.pillar_squeezed_colonnade <- function(x, ...) {
  unlist(map(x, knit_print_squeezed_colonnade_tier))
}

knit_print_squeezed_colonnade_tier <- function(x) {
  # Hack
  header <- map_chr(map(x, `[[`, "capital_format"), `[[`, "title_format")
  col <- map(x, function(xx) c(xx[["capital_format"]][["type_format"]], xx[["shaft_format"]]))

  knitr::kable(as.data.frame(col), row.names = NA, col.names = header)
}

#' Retrieve information about columns that didn't fit the available width
#'
#' @description
#' `r lifecycle::badge("soft-deprecated")`
#'
#' Formatting a [colonnade] object may lead to some columns being omitted
#' due to width restrictions. This method returns a character vector that
#' describes each of the omitted columns.
#'
#' @param x The result of [squeeze()] on a [colonnade] object
#' @inheritParams rlang::args_dots_used
#' @keywords internal
#' @export
extra_cols <- function(x, ...) {
  deprecate_soft("1.5.0", "pillar::extra_cols()")

  if (!missing(...)) {
    check_dots_used(action = warn)
  }

  UseMethod("extra_cols")
}

#' @rdname extra_cols
#' @param n The number of extra columns to return; the returned vector will
#'   always contain as many elements as there are extra columns, but elements
#'   beyond `n` will be `NA`.
#' @export
extra_cols.pillar_squeezed_colonnade <- function(x, ..., n = Inf) {
  extra_cols_impl(x, n)
}

extra_cols_impl <- function(x, n = NULL) {
  extra_cols <- attr(x, "extra_cols", exact = TRUE)
  ret <- rep(NA_character_, length(extra_cols$data))

  if (is.null(n)) {
    n <- Inf
  }

  idx <- seq_len(min(length(extra_cols$data), n))
  ret[idx] <- map2_chr(extra_cols$data[idx], extra_cols$names[idx], format_abbrev, space = NBSP)
  ret
}

#' @export
format.pillar_colonnade <- function(x, ...) {
  format(squeeze_impl(x, ...))
}

#' @export
print.pillar_colonnade <- function(x, ...) {
  print(format(x, ...))
}

#' @rdname colonnade
#' @usage NULL
#' @aliases NULL
colonnade_get_width <- function(x, width, rowid_width) {
  #' @details
  #' Pillars may be distributed over multiple tiers if
  #' `width > getOption("width")`. In this case each tier is at most
  #' `getOption("width")` characters wide. The very first step of formatting
  #' is to determine how many tiers are shown at most, and the width of each
  #' tier.
  tier_widths <- get_tier_widths(width, length(x$data), rowid_width)

  #'
  #' To avoid unnecessary computation for showing very wide colonnades, a first
  #' pass tries to fit all capitals into the tiers.
  init_cols <- min(length(x$data), sum(floor((tier_widths + 1L) / (MIN_PILLAR_WIDTH + 1L))))
  capitals <- map2(x$data[seq_len(init_cols)], x$names[seq_len(init_cols)], pillar_capital)
  init_col_widths_df <- colonnade_compute_tiered_col_widths(capitals, tier_widths)
  pillar_shown <- init_col_widths_df$id[!safe_is_na(init_col_widths_df$tier)]
  if (length(pillar_shown) < init_cols) {
    # (Include one more pillar to indicate that the data is too wide.)
    pillar_shown <- c(pillar_shown, pillar_shown[length(pillar_shown)] + 1L)
  }

  #' For each pillar whose capital fits, it is then decided in which tier it is
  #' shown, if at all, and how much horizontal space it may use (either its
  #' minimum or its maximum width).
  shafts <- map(x$data[pillar_shown], pillar_shaft)
  pillars <- map2(capitals[pillar_shown], shafts, new_pillar_1e)
  col_widths_df <- colonnade_compute_tiered_col_widths(pillars, tier_widths)

  #' Remaining space is then distributed proportionally to pillars that do not
  #' use their desired width.
  colonnade_distribute_space_df(col_widths_df, tier_widths)
}

get_tier_widths <- function(width, ncol, rowid_width, tier_width = getOption("width")) {
  if (!is.finite(width)) {
    pos <- seq(0, length.out = ncol + 1L, by = tier_width)
  } else if (width < tier_width) {
    pos <- c(0L, width)
  } else {
    pos <- seq(0, width + tier_width - 1, by = tier_width)
  }

  widths <- diff(pos) - rowid_width
  widths[widths >= 1L]
}

colonnade_compute_tiered_col_widths <- function(pillars, tier_widths) {
  max_tier_width <- max(tier_widths)

  max_widths <- pmin(map_int(map(pillars, get_widths), max), max_tier_width)
  min_widths <- pmin(map_int(map(pillars, get_min_widths), max), max_widths)

  ret <- colonnade_compute_tiered_col_widths_df(max_widths, min_widths, tier_widths)
  ret$pillar <- pillars
  ret
}

#' @rdname colonnade
#' @usage NULL
#' @aliases NULL
colonnade_compute_tiered_col_widths_df <- function(max_widths, min_widths, tier_widths) {
  "!!!!!DEBUG colonnade_compute_tiered_col_widths_df(`v(tier_widths)`)"

  max_tier_width <- max(tier_widths)

  # Safety:
  max_widths <- pmin(max_widths, max_tier_width)
  min_widths <- pmin(min_widths, max_tier_width)

  id <- seq_along(max_widths)
  col_df <- data.frame(id, max_widths, min_widths, row.names = NULL)

  #' @details
  #' For fitting pillars in one or more tiers, first a check is made
  #' if all pillars fit with their maximum width (e.g.,
  #' `option(tibble.width = Inf)` or narrow colonnade).
  max_fit <- distribute_pillars(col_df$max_widths, tier_widths)
  #' If yes, this is the resulting fit, no more work needs to be done.
  if (all_pillars_fit(max_fit)) {
    return(max_fit)
  }

  #' Otherwise, if the maximum width is too wide, the same test
  #' is carried out with the minimum width.
  #' If this is still too wide, this is the resulting fit.
  min_fit <- distribute_pillars(col_df$min_widths, tier_widths)
  if (!all_pillars_fit(min_fit)) {
    return(min_fit)
  }

  #' Otherwise, some tiers from the start
  #' will contain pillars with their maximum width,
  #' one tier will contain some pillars with maximum and some with minimum width,
  #' and the remaining tiers contain pillars with their minimum width only.
  #'
  #' For this, we compute a "reverse minimum assignment".
  min_fit_rev <- distribute_pillars_rev(col_df$min_widths, tier_widths)

  combined_fit <- combine_pillar_distributions(max_fit, min_fit_rev, tier_widths)

  combined_fit$max_widths <- col_df$max_widths
  combined_fit
}

#' @rdname colonnade
#' @usage NULL
#' @aliases NULL
combine_pillar_distributions <- function(max_fit, min_fit_rev, tier_widths) {
  #' @details
  #' We determine the cut point where minimum and maximum assignment
  #' agree.
  #' The following strategy is applied:
  #'
  #' 1. First, we determine the tier in which the cut point lies.
  #'    This is the first instance of a column that ends up in the same tier
  #'    for both minimum and maximum assignment.
  cut_point_tier <- max_fit$tier[min(which(max_fit$tier == min_fit_rev$tier))]
  #' 2. A set of candidate cut points is derived.
  cut_point_candidates <- which(max_fit$tier == cut_point_tier)
  #' 3. We consult the column offsets. The last column where the minimum assignment
  #'    has a greater or equal offset than the maximum assignment is our latest
  #'    cut point.
  cut_point_candidate_idx <- which(max_fit$offset_after[cut_point_candidates] <= min_fit_rev$offset_after[cut_point_candidates])
  if (length(cut_point_candidate_idx) > 0) {
    cut_point <- cut_point_candidates[max(cut_point_candidate_idx)]
  } else {
    #'    If no such column exists, the cut point is the column just before our
    #'    first candidate.
    cut_point <- cut_point_candidates[[1]] - 1L
  }

  #' 4. Finally, we combine maximum and minimum reverse fits at the cut point.
  #'    We don't need to redistribute anything here.
  max_fit_cut <- max_fit[seq_len(cut_point), ]
  min_fit_cut <- min_fit_rev[seq2(cut_point + 1L, nrow(min_fit_rev)), ]
  rbind(max_fit_cut, min_fit_cut)
}

#' @rdname colonnade
#' @usage NULL
#' @aliases NULL
distribute_pillars <- function(widths, tier_widths) {
  tier <- rep(NA_integer_, length(widths))
  offset_after <- rep(NA_integer_, length(widths))
  current_tier <- 1L
  current_x <- 0L

  #' @details
  #' Fitting pillars into tiers is very similar to a word-wrapping algorithm.
  for (i in seq_along(widths)) {
    current_width <- widths[[i]]
    #' In a loop, new tiers are opened if the current tier overflows.
    if (current_x + current_width > tier_widths[[current_tier]]) {
      #' If a column is too wide to fit a single tier, it will never be
      #' displayed, and the colonnade will be truncated there.
      #' This case should never occur with reasonable display widths larger than
      #' 30 characters.
      if (current_width > tier_widths[[current_tier]]) {
        if (current_tier == length(tier_widths) || current_width > tier_widths[[current_tier + 1]]) {
          break
        }
      }

      current_tier <- current_tier + 1L
      current_x <- 0L
      #' Truncation also happens if all available tiers are filled.
      if (current_tier > length(tier_widths)) break
    }

    tier[[i]] <- current_tier
    current_x <- current_x + current_width
    offset_after[[i]] <- current_x
    current_x <- current_x + 1L
  }

  data_frame(id = seq_along(widths), tier = tier, width = widths, offset_after = offset_after)
}

distribute_pillars_rev <- function(widths, tier_widths) {
  ret <- distribute_pillars(rev(widths), rev(tier_widths))
  ret[2:4] <- ret[rev(seq_along(widths)), 2:4]
  tier <- length(tier_widths) + 1L - ret$tier
  ret$tier <- tier

  splits <- split(seq_along(tier), tier)
  tier_widths <- tier_widths[stats::na.omit(unique(tier))]

  new_offset_after <- unlist(map2(unname(splits), tier_widths, function(.x, .y) {
    new_offset_after <- cumsum(ret$width[.x] + 1)
    new_offset_after - max(new_offset_after) + .y
  }))

  ret$offset_after <- c(new_offset_after, rep(NA_integer_, sum(is.na(tier))))

  ret
}

all_pillars_fit <- function(tier_df) {
  rows <- nrow(tier_df)
  rows == 0 || !safe_any_na(tier_df$tier[[nrow(tier_df)]])
}

#' @rdname colonnade
#' @usage NULL
#' @aliases NULL
colonnade_distribute_space_df <- function(col_widths_df, tier_widths) {
  "!!!!!DEBUG colonnade_distribute_space_df(`v(tier_widths)`)"

  col_widths_split <- split(col_widths_df, col_widths_df$tier)
  tier_widths <- tier_widths[seq_along(col_widths_split)]
  col_widths_apply <- map2(col_widths_split, tier_widths, function(x, width) {
    x$width <- x$width + colonnade_distribute_space(x$width, x$max_widths, width)
    x
  })

  bind_rows(unname(col_widths_apply))
}

#' @rdname colonnade
#' @usage NULL
#' @aliases NULL
colonnade_distribute_space <- function(col_widths, max_widths, width) {
  if (any(is.na(col_widths))) {
    return(col_widths)
  }

  missing_space <- max_widths - col_widths
  # Shortcut to avoid division by zero
  if (all(missing_space == 0L)) {
    return(rep_along(col_widths, 0L))
  }

  #' @details
  #' The remaining space is distributed from left to right.
  #' Each column gains space proportional to the fraction of missing and
  #' remaining space,
  occupied_width <- sum(col_widths + 1L) - 1L
  remaining_width <- max(min(width - occupied_width, sum(missing_space)), 0L)
  added_space_prop <- missing_space / sum(missing_space) * remaining_width

  #' rounded down.
  added_space_ceil <- ceiling(added_space_prop)
  added_space_floor <- floor(added_space_prop)
  added_space_diff <- added_space_ceil - added_space_floor
  added_space <- ifelse(
    #' Any space remaining after rounding is distributed from left to right,
    #' one space per column.
    sum(added_space_floor) + cumsum(added_space_diff) <= remaining_width,
    added_space_ceil,
    added_space_floor
  )

  added_space
}

#' Format the header of a tibble
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' For easier customization, the formatting of a tibble is split
#' into three components: header, body, and footer.
#' The `tbl_format_header()` method is responsible for formatting the header
#' of a tibble.
#'
#' Override this method if you need to change the appearance
#' of the entire header.
#' If you only need to change or extend the components shown in the header,
#' override or extend [tbl_sum()] for your class which is called by the
#' default method.
#'
#' @inheritParams tbl_format_body
#' @inherit tbl_format_body return
#'
#' @export
#' @examplesIf rlang::is_installed("palmerpenguins")
#' setup <- tbl_format_setup(palmerpenguins::penguins)
#' tbl_format_header(palmerpenguins::penguins, setup)
#'
#' # Shortcut for debugging
#' tbl_format_header(setup)
tbl_format_header <- function(x, setup, ...) {
  check_dots_empty()

  UseMethod("tbl_format_header")
}

#' @export
tbl_format_header.tbl <- function(x, setup, ...) {
  named_header <- setup$tbl_sum
  focus <- attr(x, "pillar_focus")
  if (!is.null(focus)) {
    named_header <- c(named_header, "Focus columns" = collapse(tick_if_needed(focus)))
  }

  if (all(names2(named_header) == "")) {
    header <- named_header
  } else {
    header <- paste0(
      align(paste0(names2(named_header), ":"), space = NBSP),
      # We add a space after the NBSP inserted by align()
      # so that wrapping occurs at the right location for very narrow outputs
      " ",
      named_header
    )
  }

  style_subtle(format_comment(header, width = setup$width))
}

#' @export
tbl_format_header.pillar_tbl_format_setup <- function(x, ...) {
  as_glue(c(
    cli::style_bold("<tbl_format_header(setup)>"),
    tbl_format_header(x$x, setup = x)
  ))
}

#' @aliases NULL pillar-package
#'
#' @description
#' `r lifecycle::badge("stable")`
#'
#' Formats tabular data in columns or rows
#' using the full range of colours provided by modern terminals.
#' Provides various generics for making every aspect of the display customizable.
#'
#' @seealso
#' - [pillar()] for formatting a single column,
#' - [print.tbl()] for formatting data-frame-like objects,
#' - [pillar_options] for a list of package options.
#'
#' @examples
#' pillar(1:3)
#' pillar(c(1, 2, 3))
#' pillar(factor(letters[1:3]), title = "letters")
#' tbl_format_setup(tibble::as_tibble(mtcars), width = 60)
"_PACKAGE"

# Format numbers in scientific notation
#
# Uses colour and careful alignment.
#
# @seealso [format_decimal()]
# @inheritParams format_decimal
# @examples
# format_scientific(1.5:3.5)
# format_scientific(1e9)
format_scientific <- function(x, sigfig) {
  split_decimal(x, sigfig, scientific = TRUE)
}

format_exp <- function(x) {
  supernum(x$exp)
}

supernum <- function(x) {
  stopifnot(is.integer(x))

  num <- !is.na(x)
  if (!any(num)) return(rep_along(x, ""))

  neg <- num & x < 0
  if (any(neg)) {
    neg_chr <- ifelse(neg, "-", "+")
    neg_chr[!num] <- " "
  } else {
    neg_chr <- rep("", length(x))
  }

  digits <- as.character(abs(x))
  digits[!num] <- ""

  exp <- paste0(neg_chr, format(digits, justify = "right"))

  paste0(style_subtle(ifelse(num, "e", " ")), style_num(exp, neg))
}

## ---- include = FALSE---------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(pillar)

## -----------------------------------------------------------------------------
example_tbl <- function(class) {
  vctrs::new_data_frame(
    list(
      a = letters[1:3],
      b = data.frame(c = 1:3, d = 4:6 + 0.5)
    ),
    class = c(class, "tbl")
  )
}

## -----------------------------------------------------------------------------
example_tbl("default")

## -----------------------------------------------------------------------------
tbl_sum.default_header_extend <- function(x, ...) {
  default_header <- NextMethod()
  c(default_header, "New" = "A new header")
}

example_tbl("default_header_extend")

tbl_sum.default_header_replace <- function(x, ...) {
  c("Override" = "Replace all headers")
}

example_tbl("default_header_replace")

## -----------------------------------------------------------------------------
tbl_format_header.custom_header_replace <- function(x, setup, ...) {
  cli::style_italic(names(setup$tbl_sum), " = ", setup$tbl_sum)
}

example_tbl("custom_header_replace")

## -----------------------------------------------------------------------------
tbl_format_footer.custom_footer_extend <- function(x, setup, ...) {
  default_footer <- NextMethod()

  extra_info <- "and with extra info in the footer"
  extra_footer <- style_subtle(paste0("# ", cli::symbol$ellipsis, " ", extra_info))

  c(default_footer, extra_footer)
}

print(example_tbl("custom_footer_extend"), n = 2)

tbl_format_footer.custom_footer_replace <- function(x, setup, ...) {
  paste0("The table has ", setup$rows_total, " rows in total.")
}

print(example_tbl("custom_footer_replace"), n = 2)

## -----------------------------------------------------------------------------
tbl_format_setup.extra_info <- function(x, width, ...) {
  setup <- NextMethod()
  cells <- prod(dim(x))
  setup$cells <- cells
  setup$tbl_sum <- c(setup$tbl_sum, "Cells" = as.character(cells))
  setup
}

tbl_format_footer.extra_info <- function(x, setup, ...) {
  paste0("The table has ", setup$cells, " cells in total.")
}

example_tbl("extra_info")

## -----------------------------------------------------------------------------
ctl_new_rowid_pillar.pillar_roman <- function(controller, x, width, ...) {
  out <- NextMethod()
  rowid <- utils::as.roman(seq_len(nrow(x)))
  width <- max(nchar(as.character(rowid)))
  new_pillar(
    list(
      title = out$title,
      type = out$type,
      data = pillar_component(
        new_pillar_shaft(list(row_ids = rowid),
          width = width,
          class = "pillar_rif_shaft"
        )
      )
    ),
    width = width
  )
}

example_tbl("pillar_roman")

## -----------------------------------------------------------------------------
ctl_new_pillar.pillar_rule <- function(controller, x, width, ..., title = NULL) {
  out <- NextMethod()
  new_pillar(list(
    top_rule = new_pillar_component(list("========"), width = 8),
    title = out$title,
    type = out$type,
    mid_rule = new_pillar_component(list("--------"), width = 8),
    data = out$data,
    bottom_rule = new_pillar_component(list("========"), width = 8)
  ))
}

example_tbl("pillar_rule")

## -----------------------------------------------------------------------------
rule <- function(char = "-") {
  stopifnot(nchar(char) == 1)
  structure(char, class = "rule")
}

format.rule <- function(x, width, ...) {
  paste(rep(x, width), collapse = "")
}

ctl_new_pillar.pillar_rule_adaptive <- function(controller, x, width, ..., title = NULL) {
  out <- NextMethod()
  if (is.null(out)) {
    return(NULL)
  }

  new_pillar(list(
    top_rule = new_pillar_component(list(rule("=")), width = 1),
    title = out$title,
    type = out$type,
    mid_rule = new_pillar_component(list(rule("-")), width = 1),
    data = out$data,
    bottom_rule = new_pillar_component(list(rule("=")), width = 1)
  ))
}

example_tbl("pillar_rule_adaptive")

## -----------------------------------------------------------------------------
ctl_new_pillar_list.hide_df <- function(controller, x, width, ..., title = NULL) {
  if (!is.data.frame(x)) {
    return(NextMethod())
  }
  
  if (width < 8) {
    return(NULL)
  }

  list(new_pillar(
    list(
      title = pillar_component(new_pillar_title(title)),
      type = new_pillar_component(list("<hidden>"), width = 8),
      data = new_pillar_component(list(""), width = 1)
    ),
    width = 8
  ))
}

example_tbl("hide_df")

## -----------------------------------------------------------------------------
tbl_format_body.oldskool <- function(x, setup, ...) {
  capture.output(print.data.frame(setup$df))
}

print(example_tbl("oldskool"), n = 2)

#' Test helpers
#'
#' Expectation for packages that implement a data type with pillar support.
#' Allows storing the desired result in a file,
#' and comparing the output with the file contents.
#' Note that this expectation sets options that affect the formatting of the
#' pillar, see examples for usage.
#'
#' @inheritParams testthat::expect_output_file
#' @param object An object to check.
#' @param ... Unused.
#' @param width The width of the output.
#' @param crayon Color the output?
#' @export
#' @examples
#' file <- tempfile("pillar", fileext = ".txt")
#'
#' # The pillar is constructed after options have been set
#' # (need two runs because reference file doesn't exist during the first run)
#' suppressWarnings(tryCatch(
#'   expect_known_display(pillar(1:3), file, crayon = FALSE),
#'   expectation_failure = function(e) {}
#' ))
#' expect_known_display(pillar(1:3), file, crayon = FALSE)
#'
#' # Good: Use tidyeval to defer construction
#' pillar_quo <- rlang::quo(pillar(1:3))
#' expect_known_display(!!pillar_quo, file, crayon = FALSE)
#'
#' \dontrun{
#' # Bad: Options set in the active session may affect the display
#' integer_pillar <- pillar(1:3)
#' expect_known_display(integer_pillar, file, crayon = FALSE)
#' }
expect_known_display <- function(object, file, ..., width = 80L, crayon = TRUE) {

  object <- enquo(object)

  if (crayon) {
    old <- options(crayon.enabled = TRUE, crayon.colors = 16L)
    crayon::num_colors(forget = TRUE)
    has_color(forget = TRUE)
  } else {
    old <- options(crayon.enabled = FALSE)
    crayon::num_colors(forget = TRUE)
    has_color(forget = TRUE)
  }

  on.exit({
    options(old)
    crayon::num_colors(forget = TRUE)
    has_color(forget = TRUE)
  })

  testthat::expect_known_output(print(eval_tidy(object)), file, update = TRUE, width = width)
}

snapshot_variant <- function(...) {
  matrix <- c(...)

  bad_variants <- setdiff(matrix, c("os", "output-enc"))
  if (length(bad_variants) > 0) {
    abort(paste0("Unknown variant selector: ", bad_variants[[1]]))
  }

  variant <- NULL

  if ("os" %in% matrix) {
    variant <- c(variant, testthat_os())
  }

  if ("output-enc" %in% matrix) {
    if (cli::is_utf8_output()) {
      variant <- c(variant, "unicode")
    } else {
      variant <- c(variant, "ansi")
    }
  }

  if (is.null(variant)) {
    return(NULL)
  }
  paste(variant, collapse = "-")
}

testthat_os <- function() {
  os <- tolower(Sys.info()[["sysname"]])
  switch(os,
    darwin = "mac",
    sunos = "solaris",
    os
  )
}

format_title <- function(x, width, footnote = TRUE) {
  align(str_trunc(x, width, if (footnote) "untick_footnote" else "untick"))
}

tick_names_if_needed <- function(x) {
  names(x) <- tick_if_needed(names(x))
  x
}

tick_if_needed <- function(x) {
  # Compatibility with R 3.4
  if (is.null(x)) {
    return(NULL)
  }
  needs_ticks <- !is_syntactic(x)
  x[needs_ticks] <- tick(x[needs_ticks])
  x
}

is_syntactic <- function(x) {
  # Compatibility with R 3.4
  if (is.null(x)) {
    return(logical())
  }
  ret <- make.names(x) == x
  ret[is.na(x)] <- FALSE
  ret
}

is_ambiguous_string <- function(x) {
  !is.na(x) & grepl("^$|^ | $|\\\\|\"", x)
}

tick <- function(x) {
  ifelse(is.na(x), "NA", encodeString(x, quote = "`"))
}

#' @importFrom utils head
df_head <- function(x, n) {
  if (!is.data.frame(x)) {
    as.data.frame(head(x, n))
  } else {
    vec_head(as.data.frame(x), n)
  }
}

vec_head <- function(x, n) {
  n <- min(n, vec_size(x))
  vec_slice(x, seq_len(n))
}

cat_line <- function(...) {
  cat(..., "\n", sep = "")
}

#' @importFrom cli ansi_substr
str_trunc <- function(x, width, shorten = NULL) {
  if (all(is.infinite(width))) {
    return(x)
  }

  str_width <- get_extent(x)

  too_wide <- which(!is.na(x) & str_width > width)
  if (any(too_wide)) {
    x[too_wide] <- str_add_ellipsis(x[too_wide], str_width[too_wide], width, shorten)
  }

  x
}

str_add_ellipsis <- function(x, str_width, width, shorten) {
  if (length(x) == 0L) {
    return(x)
  }
  shorten <- shorten %||% "back"

  thing <- switch(shorten,
    back = {
      abbr <- ansi_substr(x, 1, width - 1)
      paste0(abbr, get_ellipsis())
    },
    untick = str_add_ellipsis_untick(x, str_width, width),
    untick_footnote = str_add_ellipsis_untick(x, str_width, width, footnote = TRUE),
    front = {
      abbr <- ansi_substr(x, str_width - width + 2, str_width)
      paste0(get_ellipsis(), abbr)
    },
    mid = {
      front_width <- ceiling(width / 2) - 1
      back_width <- width - front_width - 1
      abbr_front <- ansi_substr(x, 1, front_width)
      abbr_back <- ansi_substr(x, str_width - back_width + 1, str_width)
      paste0(abbr_front, get_ellipsis(), abbr_back)
    },
    abbreviate = {
      abbreviate(x, minlength = width, strict = TRUE)
    }
  )
  thing
}

str_add_ellipsis_untick <- function(x, str_width, width, footnote = FALSE) {
  stopifnot(length(x) == 1)
  stopifnot(str_width > width)

  if (footnote) {
    width <- width - 1L
  }

  # Removing ticks due to https://github.com/tidyverse/tibble/issues/838
  x_unticked <- gsub("`", "", x, fixed = TRUE)
  if (x_unticked != x) {
    x <- x_unticked
    str_width <- get_extent(x)
  }

  # Add ellipsis even if short enough after removal of ticks
  # TODO use ansi_strim()
  abbr <- ansi_substr(x, 1, width - 1L)
  abbr <- paste0(abbr, get_ellipsis())

  if (footnote) {
    # Placeholder, regular title can't end with this string,
    # we can use this to detect a footnote
    abbr <- paste0(abbr, "\u02df")
  }

  abbr
}

paste_with_space_if_needed <- function(x, y) {
  if (y == "") {
    x
  } else {
    paste(x, y)
  }
}

check_sigfig <- function(x) {
  stopifnot(is.numeric(x), length(x) == 1)
  x <- as.integer(x)
  if (x < 1L) {
    stop("Must show at least one significant figure", call. = FALSE)
  }

  x
}

slice <- function(df, index) {
  df[index, , drop = FALSE]
}

bind_rows <- function(x) {
  if (length(x) == 0) {
    return(data.frame())
  }
  eval_tidy(quo(rbind(!!!x)))
}

get_ellipsis <- function() {
  symbol$continue
}

is_latex_output <- function() {
  if (!("knitr" %in% loadedNamespaces())) {
    return(FALSE)
  }
  get("is_latex_output", asNamespace("knitr"))()
}

remove_as_is_class <- function(x) {
  if (all(class(x) == "AsIs")) {
    return(unclass(x))
  }
  class(x) <- setdiff(class(x), "AsIs")
  x
}

v <- function(x) {
  expr <- rlang::expr_deparse(substitute(x), width = Inf)
  paste0(expr, " = ", rlang::expr_deparse(x, width = 80)[[1]])
}

# Needed for R 3.4 and earlier
safe_is_na <- function(x) {
  if (is.null(x)) {
    return(logical())
  }
  is.na(x)
}

safe_any_na <- function(x) {
  anyNA(x)
}

vec_lead <- function(x, default = NULL) {
  n <- vec_size(x)
  if (n == 0) {
    return(x)
  }
  if (is.null(default)) {
    default <- vec_slice(x, NA_integer_)
  }
  vec_c(vec_slice(x, -1), default)
}

vec_lag <- function(x, default = vec_slice(x, NA_integer_)) {
  n <- vec_size(x)
  if (n == 0) {
    return(x)
  }
  if (is.null(default)) {
    default <- vec_slice(x, NA_integer_)
  }
  vec_c(default, vec_slice(x, -n))
}

super <- c(
  "\u00b9", "\u00b2", "\u00b3", "\u2074",
  "\u2075", "\u2076", "\u2077", "\u2078", "\u2079",
  "\u02df"
)

superdigit <- function(x) {
  if (cli::is_utf8_output()) {
    super[[min(x, 10)]]
  } else {
    if (x >= 10) "*" else as.character(x)
  }
}

superdigit_sep_default <- function() {
  if (cli::is_utf8_output()) {
    "\u200b"
  } else {
    ": "
  }
}

show_output_in_terminal <- function() {
  system2("xterm", c("-e", shQuote("head tests/testthat/out/*; sleep 600")))
}

# A data frame with all major types
df_all <- pillar:::new_tbl(list(
  a = c(1, 2.5, NA),
  b = c(1:2, NA),
  c = c(T, F, NA),
  d = c("a", "b", NA),
  e = factor(c("a", "b", NA)),
  f = as.Date("2015-12-09") + c(1:2, NA),
  g = as.POSIXct("2015-12-09 10:51:34.5678", tz = "UTC") + c(1:2, NA),
  h = as.list(c(1:2, NA)),
  i = list(list(1, 2:3), list(4:6), list(NA))
))

# A data frame with strings of varying lengths
long_str <- strrep("Abcdefghij", 5)
df_str <- pillar:::map(rlang::set_names(1:50), function(i) substr(long_str, 1, i))

#' `add_special()` is not exported, and used only for initializing default
#' values to `expect_pillar_output()`.
#' @rdname expect_pillar_output
add_special <- function(x) {
  if (inherits(x, "integer64")) {
    x <- c(x, bit64::NA_integer64_)
  } else {
    x <- x[seq2(1, length(x) + 1)]
    if (is.numeric(x) && is.double(x)) {
      x <- c(x, -Inf, Inf)
    }
  }
  x
}

continue <- function(x) {
  paste0(x, symbol$continue)
}

# from pkgdepends
local_colors <- function(.local_envir = parent.frame()) {
  # This is to restore crayon's cache. This runs after restoring the options.
  withr::defer(envir = .local_envir, {
    num_colors(forget = TRUE)
  })

  # We run this first, so this will run last by withr, to restore the
  # original options.
  withr::local_options(
    list(cli.num_colors = 16L),
    .local_envir = .local_envir
  )

  # Added safety
  num_colors(forget = TRUE)
}

local_utf8 <- function(enable = TRUE, .local_envir = parent.frame()) {
  withr::local_options(
    list(cli.unicode = enable),
    .local_envir = .local_envir
  )
}

test_that("tbl_format_footer() results", {
  local_colors()

  expect_snapshot({
    tbl_format_footer(tbl_format_setup(new_tbl(as.list(set_names(letters))), width = 80))

    tbl_format_footer(tbl_format_setup(new_tbl(as.list(set_names(letters))), width = 30))
  })
})

test_that("wrapping column names with spaces in the footer", {
  expect_snapshot({
    tbl_format_footer(tbl_format_setup(
      new_tbl(list2(!!!set_names(letters), !!paste(letters, collapse = " ") := 2)),
      width = 70
    ))
  })
})

test_that("overflow", {
  expect_snapshot({
    tbl_format_footer(tbl_format_setup(
      new_tbl(list2(!!!set_names(rep(letters, 6)))),
      width = 70
    ))
  })
})

test_that("max_footer_lines option", {
  new_footer_tbl <- function(prefix) {
    names <- outer(letters[1:2], letters, paste0)
    x <- setNames(as.list(seq_along(names)), paste0(prefix, names))
    new_tbl(x)
  }

  expect_identical(
    local({
      local_pillar_option_max_footer_lines(3)
      tbl_format_footer(tbl_format_setup(new_footer_tbl("")))
    }),
    tbl_format_footer(
      tbl_format_setup(new_footer_tbl(""), max_footer_lines = 3)
    )
  )

  expect_snapshot({
    tbl_format_footer(
      tbl_format_setup(new_footer_tbl(""))
    )
    tbl_format_footer(
      tbl_format_setup(new_footer_tbl("prefix_"))
    )
    tbl_format_footer(
      tbl_format_setup(new_footer_tbl("a_very_long_prefix_"))
    )

    tbl_format_footer(
      tbl_format_setup(new_footer_tbl(""), max_footer_lines = 3)
    )
    tbl_format_footer(
      tbl_format_setup(new_footer_tbl("prefix_"), max_footer_lines = 3)
    )
    tbl_format_footer(
      tbl_format_setup(new_footer_tbl("a_very_long_prefix_"), max_footer_lines = 3)
    )

    tbl_format_footer(
      tbl_format_setup(new_footer_tbl(""), max_footer_lines = Inf)
    )
    tbl_format_footer(
      tbl_format_setup(new_footer_tbl("prefix_"), max_footer_lines = Inf)
    )
    tbl_format_footer(
      tbl_format_setup(new_footer_tbl("a_very_long_prefix_"), max_footer_lines = Inf)
    )
  })
})

test_that("pillar.advice option (#575)", {
  local_options(pillar.advice = TRUE)
  expect_snapshot({
    tbl_format_footer(tbl_format_setup(new_tbl(as.list(set_names(letters))), width = 80))
  })
})

test_that("colnames() hint (#622)", {
  local_options(pillar.advice = TRUE)
  letters2 <- do.call(paste0, expand.grid(letters, letters))
  expect_snapshot({
    tbl_format_footer(tbl_format_setup(new_tbl(as.list(set_names(letters2))), width = 80))
  })
})

test_that("advice when interactive (#575)", {
  local_interactive()
  expect_snapshot({
    tbl_format_footer(tbl_format_setup(new_tbl(as.list(set_names(letters))), width = 80))
    tbl_format_footer(tbl_format_setup(new_tbl(list(a = 1:30)), width = 80))
  })
})

test_that("tbl_format_footer() results", {
  local_colors()

  expect_snapshot({
    tbl_format_footer(tbl_format_setup(new_tbl(as.list(set_names(letters))), width = 80))

    tbl_format_footer(tbl_format_setup(new_tbl(as.list(set_names(letters))), width = 30))
  })
})

test_that("wrapping column names with spaces in the footer", {
  expect_snapshot({
    tbl_format_footer(tbl_format_setup(
      new_tbl(list2(!!!set_names(letters), !!paste(letters, collapse = " ") := 2)),
      width = 70
    ))
  })
})

test_that("overflow", {
  expect_snapshot({
    tbl_format_footer(tbl_format_setup(
      new_tbl(list2(!!!set_names(rep(letters, 6)))),
      width = 70
    ))
  })
})

test_that("max_footer_lines option", {
  new_footer_tbl <- function(prefix) {
    names <- outer(letters[1:2], letters, paste0)
    x <- setNames(as.list(seq_along(names)), paste0(prefix, names))
    new_tbl(x)
  }

  expect_identical(
    local({
      local_pillar_option_max_footer_lines(3)
      tbl_format_footer(tbl_format_setup(new_footer_tbl("")))
    }),
    tbl_format_footer(
      tbl_format_setup(new_footer_tbl(""), max_footer_lines = 3)
    )
  )

  expect_snapshot({
    tbl_format_footer(
      tbl_format_setup(new_footer_tbl(""))
    )
    tbl_format_footer(
      tbl_format_setup(new_footer_tbl("prefix_"))
    )
    tbl_format_footer(
      tbl_format_setup(new_footer_tbl("a_very_long_prefix_"))
    )

    tbl_format_footer(
      tbl_format_setup(new_footer_tbl(""), max_footer_lines = 3)
    )
    tbl_format_footer(
      tbl_format_setup(new_footer_tbl("prefix_"), max_footer_lines = 3)
    )
    tbl_format_footer(
      tbl_format_setup(new_footer_tbl("a_very_long_prefix_"), max_footer_lines = 3)
    )

    tbl_format_footer(
      tbl_format_setup(new_footer_tbl(""), max_footer_lines = Inf)
    )
    tbl_format_footer(
      tbl_format_setup(new_footer_tbl("prefix_"), max_footer_lines = Inf)
    )
    tbl_format_footer(
      tbl_format_setup(new_footer_tbl("a_very_long_prefix_"), max_footer_lines = Inf)
    )
  })
})

test_that("tbl_format_body() results", {
  expect_snapshot({
    "Various column types"
    tbl_format_body(tbl_format_setup(df_all, width = 30))

    tbl_format_body(tbl_format_setup(df_all, width = 300))

    "POSIXct and POSIXlt"
    df <- new_tbl(list(x = as.POSIXct("2016-01-01 12:34:56 GMT") + 1:12))
    df$y <- as.POSIXlt(df$x)

    tbl_format_body(tbl_format_setup(df, width = 60L))

    "Colonnade"
    ctl_colonnade(list(a = 1:3), has_row_id = FALSE)

    # For coverage tests
    ctl_colonnade(list(a = 1:3))
  })
})

test_that("body: output for wide characters", {
  skip_if(!l10n_info()$`UTF-8`)

  if (getRversion() == "4.0.4") {
    # This seems to show differently on the Mac:
    # writeLines(evaluate::evaluate('c("\\u6210", "\\u6210")')[[1]]$src)
    skip_on_os("mac")
  }

  local_utf8()

  expect_snapshot({
    x <- c("\u6210\u4ea4\u65e5\u671f", "\u5408\u540c\u5f55\u5165\u65e5\u671f")
    df <- new_tbl(setNames(list(1:3, 4:6), x))
    print(df, n = 8L, width = 60L)
  })
})

test_that("integer64 output will use scientific if necessary", {
  skip_if_not_installed("bit64")

  # 0 tests for NA warning too
  x <- bit64::as.integer64(c(0, 1e11, 1e12) + 0:2)
  expect_snapshot({
    pillar(add_special(x), width = 6)
    pillar(add_special(x), width = 7)
    pillar(add_special(x), width = 8)
    pillar(add_special(x), width = 9)
    pillar(add_special(x), width = 10)
    pillar(add_special(x), width = 11)
    pillar(add_special(x), width = 12)
    pillar(add_special(x), width = 13)
    pillar(add_special(x), width = 14)
  })
})

test_that("integer64 output uses underlines correctly", {
  skip_if_not_installed("bit64")

  local_colors()

  expect_snapshot({
    pillar(bit64::as.integer64(c(1, 10000)))
  })
})

test_that("nanotime works (#378)", {
  skip_if_not_installed("nanotime")
  skip_if_not_installed("vctrs", "0.4.1.9000")

  # 0 tests for NA warning too
  x <- nanotime::nanotime('2011-12-05 08:30:00.000', format = "%Y-%m-%d %H:%M:%E9S", tz = "GMT")
  expect_snapshot({
    pillar(x, width = 16)
    pillar(x, width = 17)
    pillar(x, width = 18)
    pillar(x, width = 19)
    pillar(x, width = 20)
    pillar(x, width = 21)
    pillar(x, width = 22)
    pillar(x, width = 23)
    pillar(x, width = 24)
    pillar(x, width = 25)
    pillar(x, width = 26)
  })
})

test_that("integer64 output will use scientific if necessary", {
  skip_if_not_installed("bit64")

  # 0 tests for NA warning too
  x <- bit64::as.integer64(c(0, 1e11, 1e12) + 0:2)
  expect_snapshot({
    pillar(add_special(x), width = 6)
    pillar(add_special(x), width = 7)
    pillar(add_special(x), width = 8)
    pillar(add_special(x), width = 9)
    pillar(add_special(x), width = 10)
    pillar(add_special(x), width = 11)
    pillar(add_special(x), width = 12)
    pillar(add_special(x), width = 13)
    pillar(add_special(x), width = 14)
  })
})

test_that("integer64 output uses underlines correctly", {
  skip_if_not_installed("bit64")

  local_colors()

  expect_snapshot({
    pillar(bit64::as.integer64(c(1, 10000)))
  })
})

test_that("nanotime works (#378)", {
  skip_if_not_installed("nanotime")

  # 0 tests for NA warning too
  x <- nanotime::nanotime('2011-12-05 08:30:00.000', format = "%Y-%m-%d %H:%M:%E9S", tz = "GMT")
  expect_snapshot({
    pillar(x, width = 16)
    pillar(x, width = 17)
    pillar(x, width = 18)
    pillar(x, width = 19)
    pillar(x, width = 20)
    pillar(x, width = 21)
    pillar(x, width = 22)
    pillar(x, width = 23)
    pillar(x, width = 24)
    pillar(x, width = 25)
    pillar(x, width = 26)
  })
})

test_that("output test", {
  expect_snapshot({
    pillar(as.list(1:3))
    pillar(list(1:3, NULL))
    pillar(add_special(list(1:3)))
    pillar(list(trees), width = 10)
  })

  local_colors()

  expect_snapshot({
    pillar(list(data.frame()), width = 11)
    pillar(list(data.frame()), width = 12)
  })
})

test_that("output test", {
  expect_snapshot({
    ctl_colonnade(rep(list(paste(letters, collapse = " ")), 4), width = Inf)
  })
})

test_that("tests from tibble", {
  skip_if_not_installed("rlang", "0.4.11.9000")
  local_options(width = 80)

  expect_snapshot({
    ctl_colonnade(mtcars[1:8, ], has_row_id = "*", width = 30)
    ctl_colonnade(trees[1:5, ], width = 20)
    ctl_colonnade(trees[1:5, ], width = 10)
    ctl_colonnade(trees[1:3, ], width = 10)
    ctl_colonnade(df_all, width = 30)
    ctl_colonnade(df_all, width = 300)
    options(width = 70)
    ctl_colonnade(df_all, width = 300)
    options(width = 60)
    ctl_colonnade(df_all, width = 300)
    options(width = 50)
    ctl_colonnade(df_all, width = 300)
    options(width = 40)
    ctl_colonnade(df_all, width = 300)
    options(width = 30)
    ctl_colonnade(df_all, width = 300)
    options(width = 20)
    ctl_colonnade(df_all, width = 300)
    list_with_ctl <- list(c("\n", '"'), factor(c("\n", "\n")))
    names(list_with_ctl) <- c("\n", "\r")
    ctl_colonnade(list_with_ctl, width = 30)
    ctl_colonnade(list(a = c("", " ", "a ", " a")), width = 30)
    ctl_colonnade(list("mean(x)" = 5, "var(x)" = 3), width = 30)
  })
})

test_that("empty", {
  expect_equal(
    format(ctl_colonnade(list(a = character(), b = logical()), width = 30)$body),
    character()
  )
  expect_equal(
    format(ctl_colonnade(trees[1:5, character()], width = 30)$body),
    character()
  )
})

test_that("NA names", {
  expect_snapshot({
    x <- list(`NA` = 1:3, set_to_NA = 4:6)
    names(x)[[2]] <- NA_character_
    ctl_colonnade(x, width = 30)
  })
})

test_that("sep argument", {
  x <- list(sep = 1:3)
  expect_snapshot({
    ctl_colonnade(x, width = 30)
    "dummy"
  })
})

test_that("color", {
  skip_if_not_installed("testthat", "3.1.1")

  local_colors()
  expect_equal(cli::num_ansi_colors(), 16)

  if (l10n_info()$`UTF-8`) {
    local_utf8()
    expect_true(cli::is_utf8_output())
    variant <- "unicode"
  } else {
    variant <- "ansi"
  }

  expect_snapshot(variant = variant, {
    style_na("NA")
    style_neg("-1")
  })

  expect_snapshot(variant = variant, {
    xf <- function() ctl_colonnade(list(x = c((10^(-3:4)) * c(-1, 1), NA)))
    print(xf())
    with_options(pillar.subtle_num = TRUE, print(xf()))
    with_options(pillar.subtle = FALSE, print(xf()))
    with_options(pillar.neg = FALSE, print(xf()))
    with_options(pillar.subtle = FALSE, pillar.neg = FALSE, print(xf()))
    with_options(pillar.bold = TRUE, print(xf()))
  })

  expect_snapshot(variant = variant, {
    ctl_colonnade(list(a_very_long_column_name = 0), width = 20)
  })
})

# Run opposite test to snapshot output but not alter it
if (l10n_info()$`UTF-8`) {
  test_that("color, options: UTF-8 is FALSE", {
    skip("Symmetry")
  })
}

test_that("tibble columns", {
  x <- list(a = 1:3, b = data.frame(c = 4:6, d = 7:9))
  expect_snapshot({
    ctl_colonnade(x, width = 30)
  })
})

test_that("tibble columns (nested)", {
  x <- data_frame(
    a = 1:3,
    b = data_frame(
      c = 4:6, d = 7:9,
      e = data_frame(f = data_frame(g = 10:12, h = 13:15))
    )
  )
  expect_snapshot({
    ctl_colonnade(x, width = 40)
  })
})

test_that("tibble columns (empty)", {
  x <- data_frame(
    a = 1:3,
    b = data_frame(
      c = 4:6, d = 7:9,
      e = data_frame(f = 10:12)[, 0]
    ),
    c = 10:12
  )
  expect_snapshot({
    ctl_colonnade(x, width = 40)
  })
})

test_that("matrix columns (unnamed)", {
  x <- list(a = 1:3, b = matrix(4:9, ncol = 2))
  expect_snapshot({
    ctl_colonnade(x, width = 30)
  })
})

test_that("matrix columns (named)", {
  x <- list(a = 1:3, b = matrix(4:9, ncol = 2, dimnames = list(NULL, c("c", "d"))))
  expect_snapshot({
    ctl_colonnade(x, width = 30)
  })
})

test_that("matrix columns (empty)", {
  expect_snapshot({
    ctl_colonnade(
      list(a = 1:3, b = matrix(4:6, ncol = 1)[, 0], c = 4:6),
      width = 30
    )
  })
})

test_that("filling unused width (#331)", {
  new_foo <- function(x = character()) {
    vec_assert(x, character())
    new_vctr(x, class = "foo")
  }

  data <- new_tbl(list(
    month = month.name[1],
    sentences = new_foo(paste(letters, collapse = " ")),
    blah = paste(LETTERS, collapse = " ")
  ))

  pillar_shaft.foo <- function(x, ...) {
    full <- format(x)
    trunc <- format(paste0(substr(x, 1, 7), symbol$continue))
    pillar::new_pillar_shaft(
      list(full = full, trunc = trunc),
      width = pillar::get_max_extent(full),
      min_width = pillar::get_max_extent(trunc),
      class = "pillar_shaft_foo"
    )
  }

  format.pillar_shaft_foo <- function(x, width, ...) {
    if (pillar::get_max_extent(x$full) <= width) {
      ornament <- x$full
    } else {
      ornament <- x$trunc
    }

    pillar::new_ornament(ornament, align = "left")
  }

  local_methods(pillar_shaft.foo = pillar_shaft.foo, format.pillar_shaft_foo = format.pillar_shaft_foo)

  expect_snapshot({
    data
    options(width = 60)
    print(data)
  })
})

test_that("focus columns", {
  skip_if_not_installed("testthat", "3.1.1")

  local_colors()

  if (l10n_info()$`UTF-8`) {
    local_utf8()
    expect_true(cli::is_utf8_output())
    variant <- "unicode"
  } else {
    variant <- "ansi"
  }

  x <- new_tbl(list(a = new_tbl(list(x = 1, y = 2)), b = "long enough"))

  local_options(width = 80)

  expect_snapshot(variant = variant, {
    tbl_format_setup(x, width = 30, focus = "b")
    tbl_format_setup(x, width = 20, focus = "b")
    tbl_format_setup(x, width = 15, focus = "b")
    tbl_format_setup(x, width = 10, focus = "b")
    tbl_format_setup(x[2:1], width = 30, focus = "a")
    tbl_format_setup(x[2:1], width = 20, focus = "a")
    tbl_format_setup(x[2:1], width = 15, focus = "a")
    tbl_format_setup(x[2:1], width = 10, focus = "a")
    tbl_format_setup(x, width = 30, focus = c("a", "b"))
    tbl_format_setup(x, width = 20, focus = c("a", "b"))
    tbl_format_setup(x, width = 15, focus = c("a", "b"))
    tbl_format_setup(x, width = 10, focus = c("a", "b"))
    tbl_format_setup(x[2:1], width = 30, focus = c("a", "b"))
    tbl_format_setup(x[2:1], width = 20, focus = c("a", "b"))
    tbl_format_setup(x[2:1], width = 15, focus = c("a", "b"))
    tbl_format_setup(x[2:1], width = 10, focus = c("a", "b"))
  })
})

#' Deprecated functions
#' @keywords internal
#' @name deprecated
NULL

#' is_vector_s3()
#'
#' `is_vector_s3()` returns `TRUE` if an object represents an S3 vector.
#' The default implementation forwards to `vctrs::vec_is()`.
#'
#' @section Lifecycle:
#' `is_vector_s3()` is soft-deprecated, because [vctrs::vec_is()]
#' provides the same functionality based on a robust framework.
#' The generic will remain available for a while but is no longer used.
#' Users and implementers are encouraged to ensure that `vctrs::vec_is()`
#' is `TRUE` for their class.
#'
#' @export
#' @rdname deprecated
is_vector_s3 <- function(x) {
  deprecate_soft("1.4.4", "is_vector_s3()", "vctrs::vec_is()")

  UseMethod("is_vector_s3")
}

#' @export
is_vector_s3.default <- function(x) vctrs::vec_is(x)

MIN_PILLAR_WIDTH <- 5L

style_type <- function(x) {
  force(x)
  x <- style_subtle(x)
  crayon_italic(x)
}

#' Prepare a column type for formatting
#'
#' Calls [type_sum()] to format the type.
#' Call [format()] on the result to render column types.
#'
#' @param x A vector for which the type is to be retrieved.
#' @inheritParams rlang::args_dots_empty
#' @export
#' @examples
#' format(new_pillar_type("a"))
#' format(new_pillar_type(factor("a")))
new_pillar_type <- function(x, ...) {
  "!!!!DEBUG new_pillar_type(`v(class(x))`)"
  if (!missing(...)) {
    check_dots_empty(action = warn)
  }

  new_pillar_type_obj(get_pillar_type(x))
}

new_pillar_type_obj <- function(type) {
  # Must wrap in a list, because type_sum() can return a classed object
  ret <- structure(list(type), class = "pillar_type")
  extent <- get_extent(format_type_sum(type, NULL))
  ret <- set_width(ret, width = max(extent, MIN_PILLAR_WIDTH))
  ret <- set_min_width(ret, MIN_PILLAR_WIDTH)
  ret
}

get_pillar_type <- function(x) {
  type <- type_sum(x)
  if (length(type) == 0L) type <- "?"
  # Can return a classed object to be formatted by format_type_sum()
  type[] <- as.character(type[[1L]])
  type
}

#' @export
format.pillar_type <- function(x, width = NULL, ..., is_focus = FALSE) {
  out <- format_type_sum(x[[1]], width)
  if (is_focus) {
    out <- crayon_underline(out)
  }
  out
}

format_full_pillar_type <- function(x) {
  type <- get_pillar_type(x)
  format_type_sum(type, NULL)
}

#' Format a type summary
#'
#' Called on values returned from [type_sum()] for defining the description
#' in the capital.
#'
#' Two methods are implemented by default for this generic: the default method,
#' and the method for the `"AsIs"` class.
#' Return `I("type")` from your [type_sum()] implementation to format the type
#' without angle brackets.
#' For even more control over the formatting, implement your own method.
#'
#' @param x A return value from `type_sum()`
#' @param width The desired total width. If the returned string still is
#'   wider, it will be trimmed. Can be `NULL`.
#' @inheritParams rlang::args_dots_used
#'
#' @export
#' @examples
#' # Default method: show the type with angle brackets
#' format_type_sum(1, NULL)
#' pillar(1)
#'
#' # AsIs method: show the type without angle brackets
#' type_sum.accel <- function(x) {
#'   I("kg m/s^2")
#' }
#'
#' accel <- structure(9.81, class = "accel")
#' pillar(accel)
format_type_sum <- function(x, width, ...) {
  if (!missing(...)) {
    check_dots_used(action = warn)
  }

  UseMethod("format_type_sum")
}

# https://github.com/r-lib/pkgdown/issues/1540
type_sum.accel <- function(x) {
  I("kg m/s^2")
}

#' @export
#' @rdname format_type_sum
format_type_sum.default <- function(x, width, ...) {
  if (!is.null(width) && width - 2 < get_extent(x)) {
    x <- substr(x, 1, max(width - 2, 0))
  }
  style_type(paste0("<", x, ">"))
}

#' @export
#' @rdname format_type_sum
format_type_sum.AsIs <- function(x, width, ...) {
  if (!is.null(width) && width < get_extent(x)) {
    x <- substr(x, 1, width)
  }
  style_type(unclass(x))
}

as_override_type_sum_asis <- function(x) {
  structure(x, class = "override_type_sum_asis")
}

as_override_type_sum_custom <- function(x) {
  structure(x, class = "override_type_sum_custom")
}

local_override_type_sum <- function(frame = caller_env()) {
  local_methods(
    .frame = frame,
    type_sum.override_type_sum_asis = function(x, ...) {
      I("AsIs")
    },
    # Needed for head()
    `[.override_type_sum_asis` = function(x, ...) {
      as_override_type_sum_asis(NextMethod())
    },
    type_sum.override_type_sum_custom = function(x, ...) {
      structure("VeryLong", class = "override_format_type_sum")
    },
    # Needed for head()
    `[.override_type_sum_custom` = function(x, ...) {
      as_override_type_sum_custom(NextMethod())
    },
    format_type_sum.override_format_type_sum = function(x, ...) {
      "VeryLong"
    }
  )
}

test_that("works with glimpse", {
  local_override_type_sum()

  foo <- as_override_type_sum_asis(2011:2013)
  bar <- as_override_type_sum_custom(2011:2013)
  expect_equal(unclass(type_sum(foo)), "AsIs")
  expect_equal(unclass(type_sum(bar)), "VeryLong")

  expect_snapshot({
    glimpse(new_tbl(list(foo = foo, bar = bar)))
  })
})

test_that("can override", {
  local_override_type_sum()

  foo <- as_override_type_sum_asis(2011:2013)
  bar <- as_override_type_sum_custom(2011:2013)

  expect_snapshot({
    new_tbl(list(foo = foo, bar = bar))
    options(width = 13)
    new_tbl(list(foo = foo, bar = bar))
  })
})

style_empty <- function(n) {
  ""
}

new_empty_shaft <- function() {
  new_pillar_shaft(list(), class = "pillar_empty_shaft", width = 0)
}

#' @export
format.pillar_empty_shaft <- function(x, width, ...) {
  new_ornament(
    style_empty(),
    width = width
  )
}

#' Get a glimpse of your data
#'
#' @description
#' `glimpse()` is like a transposed version of `print()`:
#' columns run down the page, and data runs across.
#' This makes it possible to see every column in a data frame.
#' It's a little like [str()] applied to a data frame
#' but it tries to show you as much data as possible.
#' (And it always shows the underlying data, even when applied
#' to a remote data source.)
#'
#' See [format_glimpse()] for details on the formatting.
#'
#' @section S3 methods:
#' `glimpse` is an S3 generic with a customised method for `tbl`s and
#' `data.frames`, and a default method that calls [str()].
#'
#' @param x An object to glimpse at.
#' @param width Width of output: defaults to the setting of the
#'   `width` [option][pillar_options] (if finite)
#'   or the width of the console.
#' @param ... Unused, for extensibility.
#' @return x original x is (invisibly) returned, allowing `glimpse()` to be
#'   used within a data pipe line.
#' @export
#' @examples
#' glimpse(mtcars)
#'
#' @examplesIf rlang::is_installed("nycflights13")
#' glimpse(nycflights13::flights)
#'
glimpse <- function(x, width = NULL, ...) {
  UseMethod("glimpse")
}

#' @export
glimpse.tbl <- function(x, width = NULL, ...) {
  if (!is.null(width) && !is.finite(width)) {
    abort("`width` must be finite.")
  }
  width <- get_width_glimpse(width)

  cli::cat_line("Rows: ", big_mark(nrow(x)))

  # this is an overestimate, but shouldn't be too expensive.
  # every type needs at least three characters: "x, "
  rows <- as.integer(width / 3)
  df <- df_head(x, rows)
  cli::cat_line("Columns: ", big_mark(ncol(df)))

  summary <- tbl_sum(x)
  brief_summary <- summary[-1]

  if (has_length(brief_summary)) {
    cli::cat_line(names(brief_summary), ": ", brief_summary)
  }

  if (ncol(df) == 0) {
    return(invisible(x))
  }

  var_types <- map_chr(map(df, new_pillar_type), format)
  ticked_names <- format(new_pillar_title(tick_if_needed(names(df))))
  var_names <- paste0("$ ", align(ticked_names), " ", var_types, " ")

  # for some reason the offset was -2 in tibble but is now -1
  # so that the desired width is obtained
  data_width <- width - get_extent(var_names) - 1
  formatted <- map_chr(df, format_glimpse_1)
  truncated <- str_trunc(formatted, data_width)

  cli::cat_line(var_names, truncated)
  invisible(x)
}

#' @export
glimpse.data.frame <- glimpse.tbl

#' @importFrom utils str
#' @export
glimpse.default <- function(x, width = NULL, max.level = 3, ...) {
  str(x, width = get_width_glimpse(width), max.level = max.level, ...)
  invisible(x)
}

#' Format a vector for horizontal printing
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' This generic provides the logic for printing vectors in [glimpse()].
#'
#' The output strives to be as unambiguous as possible,
#' without compromising on readability.
#' In a list, to distinguish between vectors and nested lists,
#' the latter are surrounded by `[]` brackets.
#' Empty lists are shown as `[]`.
#' Vectors inside lists, of length not equal to one,
#' are surrounded by `<>` angle brackets.
#' Empty vectors are shown as `<>`.
#'
#' @return A character vector of the same length as `x`.
#' @inheritParams rlang::args_dots_used
#' @param x A vector.
#' @export
#' @examples
#' format_glimpse(1:3)
#'
#' # Lists use [], vectors inside lists use <>
#' format_glimpse(list(1:3))
#' format_glimpse(list(1, 2:3))
#' format_glimpse(list(list(1), list(2:3)))
#' format_glimpse(list(as.list(1), as.list(2:3)))
#' format_glimpse(list(character()))
#' format_glimpse(list(NULL))
#'
#' # Character strings are always quoted
#' writeLines(format_glimpse(letters[1:3]))
#' writeLines(format_glimpse(c("A", "B, C")))
#'
#' # Factors are quoted only when needed
#' writeLines(format_glimpse(factor(letters[1:3])))
#' writeLines(format_glimpse(factor(c("A", "B, C"))))
format_glimpse <- function(x, ...) {
  check_dots_used()
  UseMethod("format_glimpse")
}

# A variant with collapse and without checks, for format_glimpse.list()
format_glimpse_1 <- function(x, ...) {
  collapse(format_glimpse_(x, ...))
}

format_glimpse_ <- function(x, ...) {
  UseMethod("format_glimpse")
}


#' @export
format_glimpse.default <- function(x, ...) {
  dims <- dim(x)

  if (is.null(dims)) {
    out <- format(x, trim = TRUE, justify = "none")
    style_na_if(out, if (is.atomic(x)) is.na(x) else FALSE)
  } else {
    dims_out <- paste0(dims, collapse = " x ")
    paste0("<", class(x)[[1]], "[", dims_out, "]>")
  }
}

#' @export
format_glimpse.list <- function(x, ..., .inner = FALSE) {
  if (!.inner && length(x) == 0) {
    return("list()")
  }

  out <- map_chr(x, format_glimpse_1, .inner = TRUE)

  # Surround vectors by <>
  # Only surround inner lists by []
  list <- map_lgl(x, is.list)
  scalar <- rep_along(x, TRUE)
  scalar[!list] <- (map_int(x[!list], length) == 1L)
  out[!scalar] <- paste0("<", out[!scalar], ">")
  out[list] <- paste0("[", out[list], "]")

  out
}

#' @export
format_glimpse.character <- function(x, ...) {
  out <- encodeString(as.character(x), quote = '"')
  style_na_if(out, is.na(x))
}

#' @export
format_glimpse.factor <- function(x, ...) {
  if (any(grepl(",", levels(x), fixed = TRUE))) {
    out <- encodeString(as.character(x), quote = '"')
  } else {
    out <- format(x, trim = TRUE, justify = "none")
  }
  style_na_if(out, is.na(x))
}

test_that("format_glimpse() output test", {
  expect_snapshot({
    "# Atomic numbers"
    format_glimpse(1)
    format_glimpse(1:3)
    format_glimpse(NA)
    format_glimpse(TRUE)
    format_glimpse(logical())

    "# Strings"
    format_glimpse("1")
    format_glimpse(letters)
    format_glimpse(NA_character_)
    format_glimpse(character())

    "# Factors"
    format_glimpse(factor(c("1", "a")))
    format_glimpse(factor(c("foo", '"bar"')))
    format_glimpse(factor())

    "Add quotes around factor levels with comma"
    "so they don't appear as if they were two observations (#384)"
    format_glimpse(factor(c("foo, bar", "foo", '"bar"')))

    "# Lists"
    format_glimpse(list(1:3))
    format_glimpse(as.list(1:3))
    format_glimpse(list(1:3, 4))
    format_glimpse(list(1:3, 4:5))
    format_glimpse(list())
    format_glimpse(list(list()))
    format_glimpse(list(character()))
    format_glimpse(list(1:3, list(4)))
    format_glimpse(list(1:3, list(4:5)))
  })
})

test_that("glimpse(width = Inf) raises legible error", {
  expect_error(
    glimpse(mtcars, width = Inf)
  )
})

test_that("glimpse calls tbl_sum() (#550)", {
  skip_if(!l10n_info()$`UTF-8`) # capture_output_lines() forces native encoding

  local_override_tbl_sum()

  trees2 <- as_override_tbl_sum(trees)

  expect_output(
    glimpse(trees2),
    "Overridden: tbl_sum",
    fixed = TRUE
  )
})

test_that("output test for glimpse()", {
  skip_if_not_installed("tibble")
  local_unknown_rows()

  expect_snapshot({
    glimpse(as_tbl(mtcars), width = 70L)

    glimpse(as_tbl(trees), width = 70L)

    "No columns"
    glimpse(as_tbl(trees[integer()]), width = 70L)

    "Non-syntactic names"
    df <- tibble::tibble(!!!set_names(c(5, 3), c("mean(x)", "var(x)")))
    glimpse(df, width = 28)

    glimpse(as_tbl(df_all), width = 70L)

    "options(tibble.width = 50)"
    withr::with_options(
      list(tibble.width = 50),
      glimpse(as_tbl(df_all))
    )

    "options(tibble.width = 35)"
    withr::with_options(
      list(tibble.width = 35),
      glimpse(as_tbl(df_all))
    )

    "non-tibble"
    glimpse(5)

    trees2 <- as_unknown_rows(trees)
    glimpse(trees2, width = 70L)

    cyl <- unique(mtcars$cyl)
    data <- unname(split(mtcars, mtcars$cyl))
    nested_mtcars_df <- tibble::tibble(cyl, data)
    glimpse(nested_mtcars_df, width = 70L)

    data <- map(data, as_tbl)
    nested_mtcars_tbl <- tibble::tibble(cyl, data)
    glimpse(nested_mtcars_tbl, width = 70L)
  })
})

test_that("color test for missing values", {
  local_colors()

  expect_snapshot({
    "individual data types"
    format_glimpse(df_all$a)
    format_glimpse(df_all$b)
    format_glimpse(df_all$c)
    format_glimpse(df_all$d)
    format_glimpse(df_all$e)
    format_glimpse(df_all$f)
    format_glimpse(df_all$g)
    format_glimpse(df_all$h)
    format_glimpse(df_all$i)

    "tables"
    glimpse(df_all)
    glimpse(as.data.frame(df_all))
  })
})

test_that("format_glimpse() output test", {
  expect_snapshot({
    "# Atomic numbers"
    format_glimpse(1)
    format_glimpse(1:3)
    format_glimpse(NA)
    format_glimpse(TRUE)
    format_glimpse(logical())

    "# Strings"
    format_glimpse("1")
    format_glimpse(letters)
    format_glimpse(NA_character_)
    format_glimpse(character())

    "# Factors"
    format_glimpse(factor(c("1", "a")))
    format_glimpse(factor(c("foo", '"bar"')))
    format_glimpse(factor())

    "Add quotes around factor levels with comma"
    "so they don't appear as if they were two observations (#384)"
    format_glimpse(factor(c("foo, bar", "foo", '"bar"')))

    "# Lists"
    format_glimpse(list(1:3))
    format_glimpse(as.list(1:3))
    format_glimpse(list(1:3, 4))
    format_glimpse(list(1:3, 4:5))
    format_glimpse(list())
    format_glimpse(list(list()))
    format_glimpse(list(character()))
    format_glimpse(list(1:3, list(4)))
    format_glimpse(list(1:3, list(4:5)))
  })
})

test_that("glimpse(width = Inf) raises legible error", {
  expect_error(
    glimpse(mtcars, width = Inf)
  )
})

test_that("glimpse calls tbl_sum() (#550)", {
  skip_if(!l10n_info()$`UTF-8`) # capture_output_lines() forces native encoding

  local_override_tbl_sum()

  trees2 <- as_override_tbl_sum(trees)

  expect_output(
    glimpse(trees2),
    "Overridden: tbl_sum",
    fixed = TRUE
  )
})

test_that("output test for glimpse()", {
  local_unknown_rows()

  expect_snapshot({
    glimpse(as_tbl(mtcars), width = 70L)

    glimpse(as_tbl(trees), width = 70L)

    "No columns"
    glimpse(as_tbl(trees[integer()]), width = 70L)

    "Non-syntactic names"
    df <- tibble::tibble(!!!set_names(c(5, 3), c("mean(x)", "var(x)")))
    glimpse(df, width = 28)

    glimpse(as_tbl(df_all), width = 70L)

    "options(tibble.width = 50)"
    withr::with_options(
      list(tibble.width = 50),
      glimpse(as_tbl(df_all))
    )

    "options(tibble.width = 35)"
    withr::with_options(
      list(tibble.width = 35),
      glimpse(as_tbl(df_all))
    )

    "non-tibble"
    glimpse(5)

    trees2 <- as_unknown_rows(trees)
    glimpse(trees2, width = 70L)

    cyl <- unique(mtcars$cyl)
    data <- unname(split(mtcars, mtcars$cyl))
    nested_mtcars_df <- tibble::tibble(cyl, data)
    glimpse(nested_mtcars_df, width = 70L)

    data <- map(data, as_tbl)
    nested_mtcars_tbl <- tibble::tibble(cyl, data)
    glimpse(nested_mtcars_tbl, width = 70L)
  })
})

# Draw a sparkline line graph with Braille characters.
#
# @inheritParams spark_bar
# @examples
# \dontrun{
# x <- seq(0, 1, length = 10)
# spark_line(x)
# }
spark_line <- function(x) {
  stopifnot(is.numeric(x))

  y <- findInterval(x, seq(0, 1, length = 5), all.inside = TRUE)

  ind <- matrix(y, ncol = 2, byrow = TRUE)
  ind[, 2] <- ind[, 2] + 4

  chars <- apply(ind, 1, braille)
  new_vertical(paste0(chars, collapse = ""), class = c("pillar_spark_line", "pillar_spark"))
}

# https://en.wikipedia.org/wiki/Braille_Patterns
braille <- function(x) {
  # remap to braille sequence
  x <- c(7L, 3L, 2L, 1L, 8L, 6L, 5L, 4L)[x]

  raised <- 1:8 %in% x
  binary <- raised * 2^(0:7)

  # offset in hex is 2800
  val <- 10240 + sum(raised * 2^(0:7))

  intToUtf8(val)
}

# nocov start - compat-lifecycle (last updated: rlang 0.3.0.9000)

scoped_lifecycle_silence <- function(frame = rlang::caller_env()) {
  rlang::local_options(
    .frame = frame,
    lifecycle_verbosity = "quiet"
  )
}
with_lifecycle_silence <- function(expr) {
  scoped_lifecycle_silence()
  expr
}

scoped_lifecycle_warnings <- function(frame = rlang::caller_env()) {
  rlang::local_options(
    .frame = frame,
    lifecycle_verbosity = "warning"
  )
}
with_lifecycle_warnings <- function(expr) {
  scoped_lifecycle_warnings()
  expr
}

scoped_lifecycle_errors <- function(frame = rlang::caller_env()) {
  rlang::local_options(
    .frame = frame,
    lifecycle_verbosity = "error"
  )
}
with_lifecycle_errors <- function(expr) {
  scoped_lifecycle_errors()
  expr
}

# Enable once signal_superseded() reaches stable state
signal_superseded <- function(...) {}

foreign_caller_env <- function(my_env = ns_env()) {
  for (n in 2:10) {
    caller <- caller_env(n)
    if (!is_reference(env_parent(caller), my_env)) {
      return(caller)
    }
  }

  # Safety net
  caller
}

# nocov end

#' Provide a succinct summary of an object
#'
#' @description
#' `r lifecycle::badge("questioning")`
#'
#' @description
#' `type_sum()` gives a brief summary of object type. Objects that commonly
#' occur in a data frame should return a string with four or less characters.
#' For most inputs, the argument is forwarded to [vctrs::vec_ptype_abbr()].
#'
#' @section Lifecycle:
#' `type_sum()` is in the "questioning" stage, because [vctrs::vec_ptype_abbr()]
#' provides essentially the same functionality based on a robust framework.
#' The generic will remain available for a while.
#' Users and implementers are encouraged to consider `vctrs::vec_ptype_abbr()`
#' once it becomes stable.
#'
#' @param x an object to summarise. Generally only methods of atomic vectors
#'   and variants have been implemented.
#'
#' @export
type_sum <- function(x) UseMethod("type_sum")

#' @export
type_sum.ordered <- function(x) {
  # r-lib/vctrs#323:
  type_sum.default(x)
}

#' @export
type_sum.factor <- function(x) {
  # r-lib/vctrs#323:
  "fct"
}

#' @export
type_sum.default <- function(x) {
  if (is.object(x) || vctrs::vec_is(x)) return(vctrs::vec_ptype_abbr(x))

  switch(typeof(x),
    builtin = ,
    special = ,
    closure = "fn",
    environment = "env",
    symbol =
      if (is_missing(x)) {
        "missing"
      } else {
        "sym"
      },

    typeof(x)
  )
}

# Registered in .onLoad()
vec_ptype_abbr.pillar_empty_col <- function(x, ...) {
  vctrs::vec_ptype_abbr(x[[1]])
}

#' @description
#' `obj_sum()` also includes the size of the object if `vctrs::vec_is()`
#' is `TRUE`.
#'
#' @keywords internal
#' @examples
#' obj_sum(1:10)
#' obj_sum(matrix(1:10))
#' obj_sum(Sys.Date())
#' obj_sum(Sys.time())
#' obj_sum(mean)
#' @rdname type_sum
#' @export
obj_sum <- function(x) UseMethod("obj_sum")

#' @export
obj_sum.default <- function(x) {
  paste0(type_sum(x), size_sum(x))
}

#' @export
obj_sum.list <- function(x) {
  map_chr(x, obj_sum.default)
}

#' @export
obj_sum.POSIXlt <- function(x) {
  rep("POSIXlt", length(x))
}

#' @export
obj_sum.AsIs <- function(x) {
  paste0("I(", obj_sum(remove_as_is_class(x)), ")")
}

#' Format a numeric vector in a tibble
#'
#' These functions are reexported as [tibble::num()] and [tibble::set_num_opts()].
#'
#' @keywords internal
#' @export
num <- function(x, ...,
                sigfig = NULL, digits = NULL,
                label = NULL, scale = NULL,
                notation = c("fit", "dec", "sci", "eng", "si"),
                fixed_exponent = NULL,
                extra_sigfig = NULL) {
  stopifnot(is.numeric(x))
  stopifnot(is.null(digits) || is_integerish(digits))
  check_dots_empty()

  # FIXME: math and arith should also work for integers
  x[] <- as.numeric(x)

  if (missing(notation)) {
    notation <- NULL
  }

  # FIXME: new_vctr() overrides class attribute, doesn't support subclassing
  out <- set_num_opts(
    x,
    sigfig = sigfig,
    digits = digits,
    label = label,
    scale = scale,
    notation = notation,
    fixed_exponent = fixed_exponent,
    extra_sigfig = extra_sigfig
  )

  # FIXME: Include class(x) to support subclassing/mixin?
  # Needs vec_base(): vec_data() but only strips vctrs_vctr class?
  # Avoid inheriting from numeric (?): https://github.com/r-lib/vctrs/issues/1339
  new_class <- c("pillar_num", "pillar_vctr", "vctrs_vctr", "double")
  class(out) <- new_class

  out
}

#' @export
vec_ptype_full.pillar_num <- function(x, ...) {
  # No longer inherited in vctrs > 0.4.1,
  # https://github.com/r-lib/vctrs/pull/1571
  format(attr(x, "pillar", exact = TRUE))
}

#' @export
vec_ptype_abbr.pillar_num <- function(x, ...) {
  pillar_attr <- attr(x, "pillar", exact = TRUE)
  notation <- pillar_attr$notation
  if (is.null(notation)) {
    notation <- "num"
  }

  sigfig <- pillar_attr$sigfig
  digits <- pillar_attr$digits

  if (!is.null(digits)) {
    if (digits > 0) {
      out <- paste0(notation, ":.", digits, "!")
    } else {
      out <- paste0(notation, ":.", -digits)
    }
  } else if (!is.null(sigfig)) {
    out <- paste0(notation, ":", sigfig)
  } else {
    out <- notation
  }

  out
}

#' @export
format.pillar_num <- function(x, trim = FALSE, ...) {
  "!!!!DEBUG format.pillar_num()"

  shaft <- pillar_shaft(x)
  out <- format(shaft, width = get_width(shaft))
  if (trim) {
    attributes(out) <- NULL
  } else {
    out <- format(out, align = "right")
  }
  out
}

#' @method vec_arith pillar_num
#' @export
vec_arith.pillar_num <- function(op, x, y, ...) {
  "!!!!DEBUG vec_arith.pillar_num(`v(op)`)"

  UseMethod("vec_arith.pillar_num", y)
}
#' @method vec_arith.pillar_num default
#' @export
vec_arith.pillar_num.default <- function(op, x, y, ...) {
  "!!!!DEBUG vec_arith.pillar_num.default(`v(op)`)"
  out <- vec_arith_base(op, vec_proxy(x), vec_proxy(y))

  if (inherits(x, "pillar_num")) {
    vec_restore(out, x)
  } else {
    vec_restore(out, y)
  }
}
#' @method vec_arith.pillar_num MISSING
#' @export
vec_arith.pillar_num.MISSING <- function(op, x, y, ...) {
  "!!!!DEBUG vec_arith.pillar_num.MISSING(`v(op)`)"
  # FIXME
  out <- vec_arith_base(op, 0, vec_proxy(x))

  vec_restore(out, x)
}

#' @method vec_arith.numeric pillar_num
#' @export
vec_arith.numeric.pillar_num <- vec_arith.pillar_num.default


#' @export
vec_math.pillar_num <- function(.fn, .x, ...) {
  "!!!!DEBUG vec_math(`v(.fn)`)"

  out <- vec_math_base(.fn, vec_proxy(.x), ...)

  if (is.numeric(out)) {
    out <- vec_restore(out, .x)
  }

  out
}

#' @export
#' @rdname num
set_num_opts <- function(x, ...,
                         sigfig = NULL, digits = NULL,
                         label = NULL, scale = NULL,
                         notation = c("fit", "dec", "sci", "eng", "si"),
                         fixed_exponent = NULL,
                         extra_sigfig = NULL) {
  check_dots_empty()

  if (missing(notation)) {
    notation <- NULL
  } else if (!is.null(notation)) {
    notation <- arg_match(notation)
  }

  if (!is.null(fixed_exponent) && !is.null(notation)) {
    if (fixed_exponent && notation == "dec") {
      abort('Incomatible arguments: `notation = "dec"` and `fixed_exponent` given')
    }
  }

  if (!is.null(scale) && is.null(label)) {
    abort("Must set `label` if `scale` is provided.")
  }

  if (!is.null(digits) && !is.null(extra_sigfig)) {
    abort("Incompatible arguments: `extra_sigfig` and `digits`.")
  }

  pillar_attr <- structure(
    list(
      sigfig = sigfig,
      digits = digits,
      label = label,
      scale = scale,
      notation = notation,
      fixed_exponent = fixed_exponent,
      extra_sigfig = extra_sigfig
    ),
    class = c("pillar_num_attr", "pillar_vctr_attr", "tibble_vec_attr")
  )
  attr(x, "pillar") <- pillar_attr
  x
}

#' @export
format.pillar_num_attr <- function(x, ...) {
  notation <- x$notation
  if (is.null(notation)) {
    class <- "pillar_num"
  } else {
    class <- paste0("pillar_num(", notation, ")")
  }

  sigfig <- x$sigfig
  digits <- x$digits
  label <- x$label
  extra_sigfig <- x$extra_sigfig

  if (!is.null(digits)) {
    if (digits >= 0) {
      out <- paste0(class, ":.", digits, "!")
    } else {
      out <- paste0(class, ":.", -digits)
    }
  } else if (!is.null(sigfig)) {
    out <- paste0(class, ":", sigfig)

    if (isTRUE(extra_sigfig)) {
      out <- paste0(out, "*")
    }
  } else if (isTRUE(extra_sigfig)) {
    out <- paste0(class, "*")
  } else {
    out <- class
  }

  if (!is.null(label)) {
    out <- paste0(out, "{", label, "}")
  }

  scale <- x$scale

  if (!is.null(scale)) {
    out <- paste0(out, "*", scale)
  }

  fixed_exponent <- x$fixed_exponent

  if (!is.null(fixed_exponent)) {
    out <- paste0(out, "|", fixed_exponent)
  }

  out
}

#' @export
vec_ptype2.pillar_num.pillar_num <- function(x, y, ...) {
  x
}
#' @export
vec_ptype2.pillar_num.double <- function(x, y, ...) {
  x
}
#' @export
vec_ptype2.double.pillar_num <- function(x, y, ...) {
  y
}

#' @export
vec_ptype2.pillar_num.integer <- function(x, y, ...) {
  x
}
#' @export
vec_ptype2.integer.pillar_num <- function(x, y, ...) {
  y
}

#' @export
vec_cast.pillar_num.pillar_num <- function(x, to, ...) {
  pillar_x <- attr(x, "pillar", exact = TRUE)
  pillar_to <- attr(to, "pillar", exact = TRUE)

  pillar_x_label <- pillar_x$label
  pillar_to_label <- pillar_to$label

  if (!is.null(pillar_x_label) && !is.null(pillar_to_label)) {
    if (!identical(pillar_x$label, pillar_to$label)) {
      abort("Only `pillar_num` objects with the same label can be combined.")
    }
  }

  attr(x, "pillar") <- pillar_to
  x
}
#' @export
vec_cast.double.pillar_num <- function(x, to, ...) {
  vec_data(x)
}
#' @export
vec_cast.pillar_num.double <- function(x, to, ...) {
  out <- num(x)
  attr(out, "pillar") <- attr(to, "pillar")
  out
}
#' @export
vec_cast.pillar_num.integer <- function(x, to, ...) {
  out <- num(as.numeric(x))
  attr(out, "pillar") <- attr(to, "pillar")
  out
}

test_that("output test", {
  skip_if_not_installed("tibble")

  expect_snapshot({
    tibble::tibble(
      x0 = num(9:11 * 100 + 0.5, sigfig = 3),
      x1 = num(9:11 * 100 + 0.5, sigfig = 4),
      x2 = num(9:11 * 100 + 0.5, sigfig = 5),
    )

    # Maximum digits after the decimal points
    tibble::tibble(
      x3 = num(9:11 * 100 + 0.5, digits = 0),
      x4 = num(9:11 * 100 + 0.5, digits = -1),
      x5 = num(9:11 * 100 + 0.5, digits = -2),
    )

    # Exact digits after the decimal point, with label
    tibble::tibble(
      usd = num(9:11 * 100 + 0.5, digits = 2, label = "USD"),
      gbp = num(9:11 * 100 + 0.5, digits = 2, label = "£"),
      chf = num(9:11 * 100 + 0.5, digits = 2, label = "SFr")
    )

    # Scale
    tibble::tibble(
      small  = num(9:11 / 1000 + 0.00005, label = "%", scale = 100),
      medium = num(9:11 / 100 + 0.0005, label = "%", scale = 100),
      large  = num(9:11 / 10 + 0.005, label = "%", scale = 100)
    )

    # Notation
    tibble::tibble(
      sci = num(10^(-13:6), notation = "sci"),
      eng = num(10^(-13:6), notation = "eng"),
      dec = num(10^(-13:6), notation = "dec"),
      si  = num(10^(-13:6), notation = "si"),
    )

    # Fixed exponent notation
    tibble::tibble(
      scimin = num(10^(-7:6) * 123, notation = "sci", fixed_exponent = -Inf),
      engmin = num(10^(-7:6) * 123, notation = "eng", fixed_exponent = -Inf),
      simin  = num(10^(-7:6) * 123, notation = "si", fixed_exponent = -Inf)
    )

    tibble::tibble(
      scismall = num(10^(-7:6) * 123, notation = "sci", fixed_exponent = -3),
      engsmall = num(10^(-7:6) * 123, notation = "eng", fixed_exponent = -3),
      sismall  = num(10^(-7:6) * 123, notation = "si", fixed_exponent = -3)
    )

    tibble::tibble(
      scilarge = num(10^(-7:6) * 123, notation = "sci", fixed_exponent = 3),
      englarge = num(10^(-7:6) * 123, notation = "eng", fixed_exponent = 3),
      silarge  = num(10^(-7:6) * 123, notation = "si", fixed_exponent = 3)
    )

    tibble::tibble(
      scimax = num(10^(-7:6) * 123, notation = "sci", fixed_exponent = Inf),
      engmax = num(10^(-7:6) * 123, notation = "eng", fixed_exponent = Inf),
      simax  = num(10^(-7:6) * 123, notation = "si", fixed_exponent = Inf)
    )

    # Extra significant figures
    tibble::tibble(
      default = num(100 + 1:3 * 0.001),
      extra1 = num(100 + 1:3 * 0.001, extra_sigfig = TRUE),
      extra2 = num(100 + 1:3 * 0.0001, extra_sigfig = TRUE),
      extra3 = num(10000 + 1:3 * 0.00001, extra_sigfig = TRUE)
    )
  })
})

test_that("many digits", {
  expect_snapshot({
    num(123456789 * 10^(-9:0))
    num(123456789 * 10^(-9:1))
    num(123456789 * 10^(-9:1), notation = "dec")
    num(123456789 * 10^(-9:1), notation = "sci")
    num(123456789 * 10^(-9:1), notation = "eng")
    num(123456789 * 10^(-9:1), notation = "si")
    num(123456789 * 10^(-9:1), notation = "sci", fixed_exponent = -Inf)
    num(123456789 * 10^(-9:1), notation = "eng", fixed_exponent = -Inf)
    num(123456789 * 10^(-9:1), notation = "si", fixed_exponent = -Inf)
    num(123456789 * 10^(-9:1), notation = "sci", fixed_exponent = -3)
    num(123456789 * 10^(-9:1), notation = "sci", fixed_exponent = 3)
    num(123456789 * 10^(-9:1), notation = "sci", fixed_exponent = Inf)
  })
})

test_that("sigfig and digits", {
  expect_snapshot({
    num(c(578890.23, 240234.131, 40234.1))
    num(c(578890.23, 240234.131, 40234.1), sigfig = 6)
    num(c(578890.23, 240234.131, 40234.1), sigfig = 7)
    num(c(578890.23, 240234.131, 40234.1), sigfig = 8)
    num(c(578890.23, 240234.131, 40234.1), sigfig = 9)
    num(c(578890.23, 240234.131, 40234.1), digits = 2)
    num(c(578890.23, 240234.131, 40234.1), digits = 3)
    num(c(578890.23, 240234.131, 40234.1), digits = 4)
    num(c(578890.23, 240234.131, 40234.1), digits = -2)
    num(c(578890.23, 240234.131, 40234.1), digits = -3)
    num(c(578890.23, 240234.131, 40234.1), digits = -4)
  })
})

test_that("forced digits", {
  expect_snapshot({
    pillar(num(1:3, digits = 2))
    pillar(num(1:3, digits = 5))
  })
})

test_that("all NA", {
  expect_snapshot({
    pillar(num(NA_real_, digits = 2))
    pillar(num(NA_real_, notation = "si"))
    pillar(num(NA_real_, notation = "sci"))
    pillar(num(NA_real_, notation = "eng"))
    pillar(num(NA_real_, notation = "sci", fixed_exponent = -1))
    pillar(num(NA_real_, notation = "sci", fixed_exponent = -Inf))
  })
})

test_that("some NA", {
  expect_snapshot({
    pillar(num(c(NA_real_, 1000), digits = 2))
    pillar(num(c(NA_real_, 1000), notation = "si"))
    pillar(num(c(NA_real_, 1000), notation = "sci"))
    pillar(num(c(NA_real_, 1000), notation = "eng"))
    pillar(num(c(NA_real_, 1000), notation = "sci", fixed_exponent = -1))
    pillar(num(c(NA_real_, 1000), notation = "sci", fixed_exponent = -Inf))
  })
})

test_that("coercion", {
  expect_identical(
    vec_c(num(1), num(2)),
    num(as.numeric(1:2))
  )
  expect_identical(
    vec_c(num(1, sigfig = 4), num(2, sigfig = 2, notation = "si")),
    num(as.numeric(1:2), sigfig = 4)
  )
  expect_identical(
    vec_c(num(1, label = "a"), 2),
    num(as.numeric(1:2), label = "a")
  )
  expect_identical(
    vec_c(1, num(2, label = "a")),
    num(as.numeric(1:2), label = "a")
  )
  expect_error(
    vec_c(num(1, label = "a"), num(2, label = "b"))
  )
})

test_that("arithmetics", {
  expect_snapshot({
    num(1) + 2
    1 + num(2)
    1L + num(2)
    num(3.23456, sigfig = 4) - num(2)
    num(3, digits = 2) * num(4, sigfig = 2)
    -num(2)
  })
})

test_that("mathematics", {
  expect_snapshot({
    min(num(1:3, label = "$"))
    mean(num(1:3, notation = "eng"))
    sin(num(1:3, label = "%", scale = 100))
    sum(num(c(1:3, NA)), na.rm = TRUE)
  })
})

test_that("formatting", {
  expect_snapshot({
    format(num(-1:3))
    format(num(-1:3), trim = TRUE)
  })
})

test_that("attribute", {
  expect_snapshot({
    set_num_opts(1, sigfig = 2, fixed_exponent = -Inf)
    set_num_opts(1000, digits = 2, notation = "eng")
  })
})

test_that("output test", {
  expect_snapshot({
    tibble::tibble(
      x0 = num(9:11 * 100 + 0.5, sigfig = 3),
      x1 = num(9:11 * 100 + 0.5, sigfig = 4),
      x2 = num(9:11 * 100 + 0.5, sigfig = 5),
    )

    # Maximum digits after the decimal points
    tibble::tibble(
      x3 = num(9:11 * 100 + 0.5, digits = 0),
      x4 = num(9:11 * 100 + 0.5, digits = -1),
      x5 = num(9:11 * 100 + 0.5, digits = -2),
    )

    # Exact digits after the decimal point, with label
    tibble::tibble(
      usd = num(9:11 * 100 + 0.5, digits = 2, label = "USD"),
      gbp = num(9:11 * 100 + 0.5, digits = 2, label = "£"),
      chf = num(9:11 * 100 + 0.5, digits = 2, label = "SFr")
    )

    # Scale
    tibble::tibble(
      small  = num(9:11 / 1000 + 0.00005, label = "%", scale = 100),
      medium = num(9:11 / 100 + 0.0005, label = "%", scale = 100),
      large  = num(9:11 / 10 + 0.005, label = "%", scale = 100)
    )

    # Notation
    tibble::tibble(
      sci = num(10^(-13:6), notation = "sci"),
      eng = num(10^(-13:6), notation = "eng"),
      dec = num(10^(-13:6), notation = "dec"),
      si  = num(10^(-13:6), notation = "si"),
    )

    # Fixed exponent notation
    tibble::tibble(
      scimin = num(10^(-7:6) * 123, notation = "sci", fixed_exponent = -Inf),
      engmin = num(10^(-7:6) * 123, notation = "eng", fixed_exponent = -Inf),
      simin  = num(10^(-7:6) * 123, notation = "si", fixed_exponent = -Inf)
    )

    tibble::tibble(
      scismall = num(10^(-7:6) * 123, notation = "sci", fixed_exponent = -3),
      engsmall = num(10^(-7:6) * 123, notation = "eng", fixed_exponent = -3),
      sismall  = num(10^(-7:6) * 123, notation = "si", fixed_exponent = -3)
    )

    tibble::tibble(
      scilarge = num(10^(-7:6) * 123, notation = "sci", fixed_exponent = 3),
      englarge = num(10^(-7:6) * 123, notation = "eng", fixed_exponent = 3),
      silarge  = num(10^(-7:6) * 123, notation = "si", fixed_exponent = 3)
    )

    tibble::tibble(
      scimax = num(10^(-7:6) * 123, notation = "sci", fixed_exponent = Inf),
      engmax = num(10^(-7:6) * 123, notation = "eng", fixed_exponent = Inf),
      simax  = num(10^(-7:6) * 123, notation = "si", fixed_exponent = Inf)
    )

    # Extra significant figures
    tibble::tibble(
      default = num(100 + 1:3 * 0.001),
      extra1 = num(100 + 1:3 * 0.001, extra_sigfig = TRUE),
      extra2 = num(100 + 1:3 * 0.0001, extra_sigfig = TRUE),
      extra3 = num(10000 + 1:3 * 0.00001, extra_sigfig = TRUE)
    )
  })
})

test_that("many digits", {
  expect_snapshot({
    num(123456789 * 10^(-9:0))
    num(123456789 * 10^(-9:1))
    num(123456789 * 10^(-9:1), notation = "dec")
    num(123456789 * 10^(-9:1), notation = "sci")
    num(123456789 * 10^(-9:1), notation = "eng")
    num(123456789 * 10^(-9:1), notation = "si")
    num(123456789 * 10^(-9:1), notation = "sci", fixed_exponent = -Inf)
    num(123456789 * 10^(-9:1), notation = "eng", fixed_exponent = -Inf)
    num(123456789 * 10^(-9:1), notation = "si", fixed_exponent = -Inf)
    num(123456789 * 10^(-9:1), notation = "sci", fixed_exponent = -3)
    num(123456789 * 10^(-9:1), notation = "sci", fixed_exponent = 3)
    num(123456789 * 10^(-9:1), notation = "sci", fixed_exponent = Inf)
  })
})

test_that("sigfig and digits", {
  expect_snapshot({
    num(c(578890.23, 240234.131, 40234.1))
    num(c(578890.23, 240234.131, 40234.1), sigfig = 6)
    num(c(578890.23, 240234.131, 40234.1), sigfig = 7)
    num(c(578890.23, 240234.131, 40234.1), sigfig = 8)
    num(c(578890.23, 240234.131, 40234.1), sigfig = 9)
    num(c(578890.23, 240234.131, 40234.1), digits = 2)
    num(c(578890.23, 240234.131, 40234.1), digits = 3)
    num(c(578890.23, 240234.131, 40234.1), digits = 4)
    num(c(578890.23, 240234.131, 40234.1), digits = -2)
    num(c(578890.23, 240234.131, 40234.1), digits = -3)
    num(c(578890.23, 240234.131, 40234.1), digits = -4)
  })
})

test_that("forced digits", {
  expect_snapshot({
    pillar(num(1:3, digits = 2))
    pillar(num(1:3, digits = 5))
  })
})

test_that("all NA", {
  expect_snapshot({
    pillar(num(NA_real_, digits = 2))
    pillar(num(NA_real_, notation = "si"))
    pillar(num(NA_real_, notation = "sci"))
    pillar(num(NA_real_, notation = "eng"))
    pillar(num(NA_real_, notation = "sci", fixed_exponent = -1))
    pillar(num(NA_real_, notation = "sci", fixed_exponent = -Inf))
  })
})

test_that("some NA", {
  expect_snapshot({
    pillar(num(c(NA_real_, 1000), digits = 2))
    pillar(num(c(NA_real_, 1000), notation = "si"))
    pillar(num(c(NA_real_, 1000), notation = "sci"))
    pillar(num(c(NA_real_, 1000), notation = "eng"))
    pillar(num(c(NA_real_, 1000), notation = "sci", fixed_exponent = -1))
    pillar(num(c(NA_real_, 1000), notation = "sci", fixed_exponent = -Inf))
  })
})

test_that("coercion", {
  expect_identical(
    vec_c(num(1), num(2)),
    num(as.numeric(1:2))
  )
  expect_identical(
    vec_c(num(1, sigfig = 4), num(2, sigfig = 2, notation = "si")),
    num(as.numeric(1:2), sigfig = 4)
  )
  expect_identical(
    vec_c(num(1, label = "a"), 2),
    num(as.numeric(1:2), label = "a")
  )
  expect_identical(
    vec_c(1, num(2, label = "a")),
    num(as.numeric(1:2), label = "a")
  )
  expect_error(
    vec_c(num(1, label = "a"), num(2, label = "b"))
  )
})

test_that("arithmetics", {
  expect_snapshot({
    num(1) + 2
    1 + num(2)
    1L + num(2)
    num(3.23456, sigfig = 4) - num(2)
    num(3, digits = 2) * num(4, sigfig = 2)
    -num(2)
  })
})

test_that("mathematics", {
  expect_snapshot({
    min(num(1:3, label = "$"))
    mean(num(1:3, notation = "eng"))
    sin(num(1:3, label = "%", scale = 100))
  })
})

test_that("formatting", {
  expect_snapshot({
    format(num(-1:3))
    format(num(-1:3), trim = TRUE)
  })
})

test_that("attribute", {
  expect_snapshot({
    set_num_opts(1, sigfig = 2, fixed_exponent = -Inf)
    set_num_opts(1000, digits = 2, notation = "eng")
  })
})

test_that("ggplot2 snapshot tests", {
  skip_if_not_installed("ggplot2", "3.5.1.9000")
  skip_if_not_installed("vdiffr")
  skip_if_not_installed("tibble")

  data <- tibble::tibble(
    x = num((1:10) / 100, fixed_exponent = -3, notation = "eng"),
    y = num((1:10) / 100, scale = 100, label = "%"),
    z = num(10^(-5:4), notation = "si")
  )

  basic <-
    ggplot2::ggplot(data, ggplot2::aes(x, y)) +
    ggplot2::geom_point() +
    ggplot2::theme_minimal(36)

  vdiffr::expect_doppelganger("basic", basic)

  log_scale <-
    ggplot2::ggplot(data, ggplot2::aes(x, z)) +
    ggplot2::geom_point() +
    scale_y_num(transform = "log10") +
    ggplot2::theme_minimal(36)

  vdiffr::expect_doppelganger("log_scale", log_scale)
})

test_that("ggplot2 snapshot tests", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("vdiffr")
  skip_if_not_installed("tibble")

  data <- tibble::tibble(
    x = num((1:10) / 100, fixed_exponent = -3, notation = "eng"),
    y = num((1:10) / 100, scale = 100, label = "%"),
    z = num(10^(-5:4), notation = "si")
  )

  basic <-
    ggplot2::ggplot(data, ggplot2::aes(x, y)) +
    ggplot2::geom_point() +
    ggplot2::theme_minimal(36)

  vdiffr::expect_doppelganger("basic", basic)

  log_scale <-
    ggplot2::ggplot(data, ggplot2::aes(x, z)) +
    ggplot2::geom_point() +
    scale_y_num(trans = "log10") +
    ggplot2::theme_minimal(36)

  vdiffr::expect_doppelganger("log_scale", log_scale)
})

set_show_source_hooks <- function() {
  width <- 40

  set_show_source_opts_hook(width)
  set_show_source_source_hook()
}

# Defines a `show_source` knitr option. If this chunk option is set,
# the code should be a single function (use ::: for internal functions).
# The code will be replaced by the function definition, and not evaluated.
set_show_source_opts_hook <- function(width) {
  force(width)

  show_source_opts_hook <- function(options) {
    qualified_fun_name <- options$code
    fun_name <- gsub("^.*:::", "", qualified_fun_name)

    fun <- eval(parse(text = fun_name))
    code <- deparse(fun, control = "useSource")

    options$code <- paste0(fun_name, " <- ", paste0(code, collapse = "\n"))
    options$eval <- FALSE
    # Store for later reuse by source hook
    options$show_source <- qualified_fun_name
    options
  }

  knitr::opts_hooks$set(show_source = show_source_opts_hook)
}

# show_source chunks are included in a <details> tag.
set_show_source_source_hook <- function() {
  # Need to use a closure here to daisy-chain hooks and to keep state

  old_source_hook <- knitr::knit_hooks$get("source")

  show_source_source_hook <- function(x, options) {
    x <- old_source_hook(x, options)
    if (is.null(options$show_source)) {
      return(x)
    }

    qualified_fun_name <- options$show_source
    fun_name <- gsub("^.*:::", "", qualified_fun_name)

    paste0(
      "<details><summary>Source code of <code>",
      qualified_fun_name, "()</code></summary>",
      x, "</details>\n"
    )
  }

  knitr::knit_hooks$set(source = show_source_source_hook)
}

## ---- include = FALSE---------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

pillar:::set_show_source_hooks()

## ----setup--------------------------------------------------------------------
library(pillar)

## ----echo = FALSE, error = TRUE-----------------------------------------------
DiagrammeR::mermaid("format.mmd")

## -----------------------------------------------------------------------------
tbl <- tibble::tibble(a = 1:3, b = tibble::tibble(c = 4:6, d = 7:9), e = 10:12)
print(tbl, width = 23)
str(tbl)

## ----show_source = TRUE-------------------------------------------------------
#  print.tbl <- function (x, width = NULL, ..., n = NULL, max_extra_cols = NULL, 
#      max_footer_lines = NULL) 
#  {
#      print_tbl(x, width, ..., n = n, max_extra_cols = max_extra_cols, 
#          max_footer_lines = max_footer_lines)
#  }

## ----show_source = TRUE-------------------------------------------------------
#  format.tbl <- function (x, width = NULL, ..., n = NULL, max_extra_cols = NULL, 
#      max_footer_lines = NULL) 
#  {
#      format_tbl(x, width, ..., n = n, max_extra_cols = max_extra_cols, 
#          max_footer_lines = max_footer_lines)
#  }

## -----------------------------------------------------------------------------
setup <- tbl_format_setup(tbl, width = 24)
setup

## ----show_source = TRUE-------------------------------------------------------
#  tbl_format_setup <- function (x, width = NULL, ..., n = NULL, max_extra_cols = NULL, 
#      max_footer_lines = NULL, focus = NULL) 
#  {
#      "!!!!DEBUG tbl_format_setup()"
#      width <- get_width_print(width)
#      n <- get_n_print(n, nrow(x))
#      max_extra_cols <- get_max_extra_cols(max_extra_cols)
#      max_footer_lines <- get_max_footer_lines(max_footer_lines)
#      out <- tbl_format_setup_dispatch(x, width, ..., n = n, max_extra_cols = max_extra_cols, 
#          max_footer_lines = max_footer_lines, focus = focus)
#      return(out)
#      UseMethod("tbl_format_setup")
#  }

## ----show_source = TRUE-------------------------------------------------------
#  tbl_format_setup.tbl <- function (x, width, ..., n, max_extra_cols, max_footer_lines, 
#      focus) 
#  {
#      "!!!!DEBUG tbl_format_setup.tbl()"
#      rows <- nrow(x)
#      if (is.na(rows)) {
#          df <- df_head(x, n + 1)
#          if (nrow(df) <= n) {
#              rows <- nrow(df)
#          }
#          else {
#              df <- vec_head(df, n)
#          }
#      }
#      else {
#          df <- df_head(x, n)
#      }
#      if (is.na(rows)) {
#          needs_dots <- (nrow(df) >= n)
#      }
#      else {
#          needs_dots <- (rows > n)
#      }
#      if (needs_dots) {
#          rows_missing <- rows - n
#      }
#      else {
#          rows_missing <- 0
#      }
#      tbl_sum <- tbl_sum(x)
#      rownames(df) <- NULL
#      colonnade <- ctl_colonnade(df, has_row_id = if (.row_names_info(x) > 
#          0) 
#          "*"
#      else TRUE, width = width, controller = x, focus = focus)
#      body <- colonnade$body
#      extra_cols <- colonnade$extra_cols
#      extra_cols_total <- length(extra_cols)
#      if (extra_cols_total > max_extra_cols) {
#          length(extra_cols) <- max_extra_cols
#      }
#      abbrev_cols <- colonnade$abbrev_cols
#      new_tbl_format_setup(x = x, df = df, width = width, tbl_sum = tbl_sum, 
#          body = body, rows_missing = rows_missing, rows_total = rows, 
#          extra_cols = extra_cols, extra_cols_total = extra_cols_total, 
#          max_footer_lines = max_footer_lines, abbrev_cols = abbrev_cols)
#  }

## -----------------------------------------------------------------------------
tbl_format_header(tbl, setup)
tbl_format_body(tbl, setup)
tbl_format_footer(tbl, setup)

## -----------------------------------------------------------------------------
class(tbl_format_body(tbl, setup))
typeof(tbl_format_body(tbl, setup))

## ----show_source = TRUE-------------------------------------------------------
#  tbl_format_header.tbl <- function (x, setup, ...) 
#  {
#      named_header <- setup$tbl_sum
#      if (all(names2(named_header) == "")) {
#          header <- named_header
#      }
#      else {
#          header <- paste0(align(paste0(names2(named_header), ":"), 
#              space = NBSP), " ", named_header)
#      }
#      style_subtle(format_comment(header, width = setup$width))
#  }

## ----show_source = TRUE-------------------------------------------------------
#  tbl_format_body.tbl <- function (x, setup, ...) 
#  {
#      force(setup)
#      setup$body
#  }

## ----show_source = TRUE-------------------------------------------------------
#  tbl_format_footer.tbl <- function (x, setup, ...) 
#  {
#      footer <- format_footer(x, setup)
#      footer_comment <- wrap_footer(footer, setup)
#      style_subtle(footer_comment)
#  }

## -----------------------------------------------------------------------------
ctl_new_pillar_list(tbl, tbl$a, width = 20)
ctl_new_pillar_list(tbl, tbl$b, width = 20)

## ----show_source = TRUE-------------------------------------------------------
#  ctl_new_pillar_list.tbl <- function (controller, x, width, ..., title = NULL, first_pillar = NULL) 
#  {
#      "!!!!DEBUG ctl_new_pillar_list.tbl(`v(width)`, `v(title)`)"
#      if (is.data.frame(x)) {
#          new_data_frame_pillar_list(x, controller, width, title = title, 
#              first_pillar = first_pillar)
#      }
#      else if (is.matrix(x)) {
#          new_matrix_pillar_list(x, controller, width, title = title, 
#              first_pillar = first_pillar)
#      }
#      else if (is.array(x) && length(dim(x)) > 1) {
#          new_array_pillar_list(x, controller, width, title = title, 
#              first_pillar = first_pillar)
#      }
#      else {
#          if (is.null(first_pillar)) {
#              first_pillar <- ctl_new_pillar(controller, x, width, 
#                  ..., title = prepare_title(title))
#          }
#          new_single_pillar_list(first_pillar, width)
#      }
#  }

## -----------------------------------------------------------------------------
ctl_new_pillar(tbl, tbl$a, width = 20)

## ----show_source = TRUE-------------------------------------------------------
#  ctl_new_pillar.tbl <- function (controller, x, width, ..., title = NULL) 
#  {
#      "!!!!DEBUG ctl_new_pillar.tbl(`v(width)`, `v(title)`)"
#      pillar(x, title, if (!is.null(width)) 
#          max0(width))
#  }

## ----show_source = TRUE-------------------------------------------------------
#  pillar <- function (x, title = NULL, width = NULL, ...) 
#  {
#      "!!!!DEBUG pillar(`v(class(x))`, `v(title)`, `v(width)`)"
#      pillar_from_shaft(new_pillar_title(title), new_pillar_type(x), 
#          pillar_shaft(x, ...), width)
#  }

## ----show_source = TRUE-------------------------------------------------------
#  new_pillar <- function (components, ..., width = NULL, class = NULL, extra = deprecated()) 
#  {
#      "!!!!DEBUG new_pillar(`v(width)`, `v(class)`)"
#      if (is_present(extra)) {
#          deprecate_warn("1.7.0", "pillar::new_pillar(extra = )")
#      }
#      check_dots_empty()
#      if (length(components) > 0 && !is_named(components)) {
#          abort("All components must have names.")
#      }
#      structure(components, width = width, class = c(class, "pillar"))
#  }

## ----show_source = TRUE-------------------------------------------------------
#  format.pillar <- function (x, width = NULL, ...) 
#  {
#      if (is.null(width)) {
#          width <- get_width(x)
#      }
#      if (is.null(width)) {
#          widths <- pillar_get_widths(x)
#          width <- sum(widths) + length(widths) - 1
#      }
#      as_glue(pillar_format_parts_2(x, width)$aligned)
#  }

#' @importFrom utils head
df_head <- function(x, n) {
  if (!is.data.frame(x)) {
    as.data.frame(head(x, n))
  } else {
    vec_head(as.data.frame(x), n)
  }
}

vec_head <- function(x, n) {
  n <- min(n, vec_size(x))
  vec_slice(x, seq_len(n))
}

cat_line <- function(...) {
  cat(..., "\n", sep = "")
}

#' @importFrom utf8 utf8_width
#' @importFrom fansi strip_sgr substr2_ctl
str_trunc <- function(x, width, shorten = NULL) {
  if (all(is.infinite(width))) {
    return(x)
  }

  str_width <- get_extent(x)

  too_wide <- which(!is.na(x) & str_width > width)
  if (any(too_wide)) {
    x[too_wide] <- str_add_ellipsis(x[too_wide], str_width[too_wide], width, shorten)
  }

  x
}

str_add_ellipsis <- function(x, str_width, width, shorten) {
  if (is.null(shorten)) {
    shorten <- "back"
  }

  switch(shorten,
    back = {
      abbr <- substr2_ctl(x, 1, width - 1, type = "width")
      paste0(abbr, get_ellipsis())
    },
    untick = str_add_ellipsis_untick(x, str_width, width),
    untick_footnote = str_add_ellipsis_untick(x, str_width, width, footnote = TRUE),
    front = {
      abbr <- substr2_ctl(x, str_width - width + 2, str_width, type = "width")
      paste0(get_ellipsis(), abbr)
    },
    mid = {
      front_width <- ceiling(width / 2) - 1
      back_width <- width - front_width - 1
      abbr_front <- substr2_ctl(x, 1, front_width, type = "width")
      abbr_back <- substr2_ctl(x, str_width - back_width + 1, str_width, type = "width")
      paste0(abbr_front, get_ellipsis(), abbr_back)
    },
    abbreviate = {
      abbreviate(x, minlength = width, strict = TRUE)
    }
  )
}

str_add_ellipsis_untick <- function(x, str_width, width, footnote = FALSE) {
  stopifnot(length(x) == 1)
  stopifnot(str_width > width)

  if (footnote) {
    width <- width - 1L
  }

  # Removing ticks due to https://github.com/tidyverse/tibble/issues/838
  x_unticked <- gsub("`", "", x, fixed = TRUE)
  if (x_unticked != x) {
    x <- x_unticked
    str_width <- get_extent(x)
  }

  # Add ellipsis even if short enough after removal of ticks
  abbr <- substr2_ctl(x, 1, width - 1L, type = "width")
  abbr <- paste0(abbr, get_ellipsis())

  if (footnote) {
    # Placeholder, regular title can't end with this string,
    # we can use this to detect a footnote
    abbr <- paste0(abbr, "\u02df")
  }

  abbr
}

paste_with_space_if_needed <- function(x, y) {
  if (y == "") {
    x
  } else {
    paste(x, y)
  }
}

check_sigfig <- function(x) {
  stopifnot(is.numeric(x), length(x) == 1)
  x <- as.integer(x)
  if (x < 1L) {
    stop("Must show at least one significant figure", call. = FALSE)
  }

  x
}

slice <- function(df, index) {
  df[index, , drop = FALSE]
}

bind_rows <- function(x) {
  if (length(x) == 0) {
    return(data.frame())
  }
  eval_tidy(quo(rbind(!!!x)))
}

get_ellipsis <- function() {
  symbol$continue
}

is_latex_output <- function() {
  if (!("knitr" %in% loadedNamespaces())) {
    return(FALSE)
  }
  get("is_latex_output", asNamespace("knitr"))()
}

remove_as_is_class <- function(x) {
  if (all(class(x) == "AsIs")) {
    return(unclass(x))
  }
  class(x) <- setdiff(class(x), "AsIs")
  x
}

v <- function(x) {
  expr <- rlang::expr_deparse(substitute(x), width = Inf)
  paste0(expr, " = ", rlang::expr_deparse(x, width = 80)[[1]])
}

# Needed for R 3.4 and earlier
safe_is_na <- function(x) {
  if (is.null(x)) {
    return(logical())
  }
  is.na(x)
}

safe_any_na <- function(x) {
  anyNA(x)
}

vec_lead <- function(x, default = NULL) {
  n <- vec_size(x)
  if (n == 0) {
    return(x)
  }
  if (is.null(default)) {
    default <- vec_slice(x, NA_integer_)
  }
  vec_c(vec_slice(x, -1), default)
}

vec_lag <- function(x, default = vec_slice(x, NA_integer_)) {
  n <- vec_size(x)
  if (n == 0) {
    return(x)
  }
  if (is.null(default)) {
    default <- vec_slice(x, NA_integer_)
  }
  vec_c(default, vec_slice(x, -n))
}

super <- c(
  "\u00b9", "\u00b2", "\u00b3", "\u2074",
  "\u2075", "\u2076", "\u2077", "\u2078", "\u2079",
  "\u02df"
)

superdigit <- function(x) {
  if (cli::is_utf8_output()) {
    super[[min(x, 10)]]
  } else {
    if (x >= 10) "*" else as.character(x)
  }
}

superdigit_sep_default <- function() {
  if (cli::is_utf8_output()) {
    "\u200b"
  } else {
    ": "
  }
}

new_continuation_shaft <- function(parent) {
  extra_width <- get_extent(symbol$ellipsis) + 1L

  new_pillar_shaft(
    list(parent, extra_width),
    class = "pillar_continuation_shaft",
    width = get_width(parent) + extra_width,
    min_width = get_min_width(parent) + extra_width
  )
}

#' @export
format.pillar_continuation_shaft <- function(x, width, ...) {
  parent <- x[[1]]
  extra_width <- x[[2]]

  parent_format <- format(parent, width = width - extra_width)

  new_ornament(
    paste0(parent_format, " ", symbol$ellipsis),
    width = width,
    align = attr(parent_format, "align", exact = TRUE)
  )
}

get_width <- function(x) {
  attr(x, "width", exact = TRUE)
}

# Set width and minimum width information for an object
#
# Returns the input with updated `width` or `min_width` attributes.
#
# @param x Input to which assign a width or minimum width
# @param width,min_width The new width
set_width <- function(x, width) {
  if (is.null(width)) {
    return(x)
  }
  if (is.infinite(width)) {
    attr(x, "width") <- NA_integer_
  } else {
    attr(x, "width") <- as.integer(width)
  }
  x
}

get_widths <- function(x) {
  map_int(x, get_width)
}

get_min_width <- function(x) {
  attr(x, "min_width", exact = TRUE) %||% get_width(x)
}

# @rdname set_width
set_min_width <- function(x, min_width) {
  if (is.null(min_width)) {
    return(x)
  }
  attr(x, "min_width") <- as.integer(min_width)
  x
}

get_min_widths <- function(x) {
  map_int(x, get_min_width)
}

split_decimal_bw <- function(x, ...) {
  ret <- pillar_shaft(x, ...)
  # Hack: Pretend decimal format requires 100 characters
  ret$dec <- set_width(ret$dec, 100)
  ret <- set_width(ret, 100)
  ret <- set_min_width(ret, min(get_min_widths(ret)))
  format(ret, width = get_min_width(ret))
}

test_that("negative values displayed correct", {
  f <- split_decimal_bw(-0.123)
  expect_equal(unname(format(f)), "-1.23e-1")
})

test_that("exponents correct in presence of NA", {
  expect_snapshot({
    format(split_decimal_bw(c(NA, 1e-5)), justify = "right")
  })
})

test_that("corner cases", {
  expect_equal(split_decimal_bw(numeric()), character())
})

test_that("output test", {
  expect_snapshot({
    pillar(add_special(10^c(-9, -6, 3, 9)), width = 10)
    pillar(add_special((10^c(3, 9, 15, 22)) * c(-1, 1)), width = 10)
    pillar(add_special(1.25 * 10^(-309:-319)), width = 10)
  })
})

expect_scientific_width <- function(x) {
  get_formatted_width <- function(x) {
    get_max_extent(assemble_decimal(x))
  }

  expect_equal(
    get_formatted_width(split_decimal(!!x, 3)),
    get_width(split_decimal(x, 3))
  )
}

test_that("width computation", {
  expect_scientific_width(c(1, 10, 100))
  expect_scientific_width(c(0, NA))
  expect_scientific_width(c(1, NaN))
  expect_scientific_width(c(-12, 3))
  expect_scientific_width(c(-1, 23))
  expect_scientific_width(c(1.01, 10.1))
  expect_scientific_width(c(1.01, -10.1))
  expect_scientific_width(c(NA_integer_, NA_integer_))
  expect_scientific_width(c(1e5, 1e15, 1e150))
  expect_scientific_width(c(1e5, -1e15, 1e150))
  expect_scientific_width(c(1e5, 1e15, -1e150))
  expect_scientific_width(c(1e5, NaN))
  expect_scientific_width(c(1e5, 1.23e15, 1.23e150))
  expect_scientific_width(c(1e5, 1.234e15, 1.234e150))
  expect_scientific_width(c(1.2, -Inf))
  expect_scientific_width(c(1, Inf))
  expect_scientific_width(c(1e-5, 1e15))
})

context("format_scientific")

format_scientific_bw <- function(x, ...) {
  without_color({
    ret <- pillar_shaft(x, ...)
    # Hack: Pretend decimal format requires 100 characters
    ret$dec <- set_width(ret$dec, 100)
    ret <- set_width(ret, 100)
    ret <- set_min_width(ret, min(get_min_widths(ret)))
    format(ret, width = get_min_width(ret))
  })
}

test_that("negative values displayed correct", {
  f <- format_scientific_bw(-0.123)
  expect_equal(unname(format(f)), "-1.23e-1")
})

test_that("exponents correct in presence of NA", {
  f <- format_scientific_bw(c(NA, 1e-5))
  expect_equal(unname(format(f, justify = "right")), c("NA      ", " 1.00e-5"))
})

test_that("corner cases", {
  expect_equal(format_scientific_bw(numeric()), character())
})

test_that("output test", {
  expect_pillar_output(10^c(-9, -6, 3, 9), width = 10, filename = "scientific.txt")
  expect_pillar_output((10^c(3, 9, 15, 22)) * c(-1, 1), width = 10, filename = "scientific-short-neg.txt")
  expect_pillar_output(1.25 * 10^(-309:-319), width = 10, filename = "scientific-tiny.txt")
})

expect_scientific_width <- function(x) {
  get_formatted_width <- function(x) {
    get_max_extent(assemble_decimal(x))
  }

  expect_equal(
    get_formatted_width(format_scientific(!!x, 3)),
    get_width(format_scientific(x, 3))
  )
}

test_that("width computation", {
  expect_scientific_width(c(1, 10, 100))
  expect_scientific_width(c(0, NA))
  expect_scientific_width(c(1, NaN))
  expect_scientific_width(c(-12, 3))
  expect_scientific_width(c(-1, 23))
  expect_scientific_width(c(1.01, 10.1))
  expect_scientific_width(c(1.01, -10.1))
  expect_scientific_width(c(NA_integer_, NA_integer_))
  expect_scientific_width(c(1e5, 1e15, 1e150))
  expect_scientific_width(c(1e5, -1e15, 1e150))
  expect_scientific_width(c(1e5, 1e15, -1e150))
  expect_scientific_width(c(1e5, NaN))
  expect_scientific_width(c(1e5, 1.23e15, 1.23e150))
  expect_scientific_width(c(1e5, 1.234e15, 1.234e150))
  expect_scientific_width(c(1.2, -Inf))
  expect_scientific_width(c(1, Inf))
  expect_scientific_width(c(1e-5, 1e15))
})

context("format_numeric")

test_that("same pillar at different widths", {
  v <- 10^c(-9, -6, 3, 9)
  x <- pillar(v)
  expect_equal(get_min_width(x$shaft), 7)
  expect_equal(get_width(x$shaft), 7)
  expect_pillar_output(xp = v, width = 4, filename = "numeric-04.txt")
  expect_pillar_output(xp = v, width = 7, filename = "numeric-07.txt")
  expect_pillar_output(xp = v, width = 10, filename = "numeric-10.txt")
  expect_pillar_output(xp = v, width = 15, filename = "numeric-15.txt")
  expect_pillar_output(xp = v, width = 22, filename = "numeric-22.txt")
})

test_that("same pillar at different widths", {
  v <- 10^c(-5, -2, 3, 6)
  x <- pillar(v)
  expect_equal(get_min_width(x$data), 4)
  expect_equal(get_width(x$data), 13)
  expect_snapshot({
    pillar(v, width = 4)
    pillar(v, width = 7)
    pillar(v, width = 10)
    pillar(v, width = 15)
    pillar(v, width = 22)
  })
})

# Adapted from squeeze_impl()
ctl_colonnade <- function(x, has_row_id = TRUE, width = NULL,
                          controller = new_tbl(), focus = NULL) {
  "!!!!DEBUG ctl_colonnade()"

  x <- new_data_frame(x, names = names2(x))
  width <- get_width_print(width)

  stopifnot(all(focus %in% names(x)))

  n <- nrow(x)
  nc <- ncol(x)

  if (n == 0 || nc == 0) {
    return(new_colonnade_body(list(), extra_cols = x, abbrev_cols = character(), abbrev_col_idxs = numeric()))
  }

  if (is_false(has_row_id)) {
    rowid <- NULL
  } else {
    rowid <- ctl_new_rowid_pillar(controller, x, width,
      title = NULL,
      type = if (isTRUE(has_row_id)) NULL else has_row_id
    )
  }

  # Reserve space for rowid column in each tier
  if (is.null(rowid)) {
    rowid_width <- 0L
  } else {
    rowid_width <- get_width(rowid)
  }

  has_star <- identical(has_row_id, "*")

  # FIXME: Support for Inf?
  tier_widths <- get_tier_widths(width, nc, rowid_width + 1L)

  formatted_tiers <- list()
  extra_cols <- list(a = 1)[0]
  abbrev_cols <- character()
  abbrev_col_idxs <- numeric()

  on_tier <- function(formatted) {
    # writeLines(formatted)
    formatted_tiers <<- c(formatted_tiers, list(formatted))
  }

  on_extra_cols <- function(my_extra_cols) {
    # print(extra_cols)

    new_extra_cols <- pmap(my_extra_cols, function(x, title, cols) {
      out <- as.list(x)[cols]
      if (is.null(title)) {
        return(out)
      }

      if (length(out) > 1) {
        title_empty <- rep_along(title, "")
        new_names <- paste0(paste0(title_empty, "$", collapse = ""), names(out))
        new_names[[1]] <- paste0(paste0(title, "$", collapse = ""), names(out)[[1]])
        names(out) <- new_names
      } else {
        # Also account for the case of packed matrices here
        names(out) <- prepare_title(c(title, names(out)))
      }
      out
    })

    # Called at most once:
    stopifnot(length(extra_cols) == 0)
    extra_cols <<- unlist(new_extra_cols, recursive = FALSE)
  }

  on_abbrev_col <- function(title) {
    abbrev_cols <<- c(abbrev_cols, title)
  }

  on_get_n_abbrev_cols <- function() {
    length(abbrev_cols)
  }

  cb <- new_emit_tiers_callbacks(
    controller, rowid, rowid_width, has_star,
    on_tier, on_extra_cols, on_abbrev_col, on_get_n_abbrev_cols
  )

  # Side effect: populate formatted_tiers, extra_cols, and abbrev_cols
  do_emit_tiers(x, tier_widths, length(focus), cb, focus)

  new_colonnade_body(formatted_tiers, extra_cols, abbrev_cols, abbrev_col_idxs)
}

new_emit_tiers_callbacks <- function(controller, rowid, rowid_width, has_star,
                                     on_tier, on_extra_cols, on_abbrev_col, on_get_n_abbrev_cols) {
  list(
    controller = controller,
    rowid = rowid,
    rowid_width = rowid_width,
    has_star = has_star,
    on_tier = on_tier,
    on_extra_cols = on_extra_cols,
    on_abbrev_col = on_abbrev_col,
    on_get_n_abbrev_cols = on_get_n_abbrev_cols
  )
}

do_emit_tiers <- function(x, tier_widths, n_focus, cb, focus) {
  formatted_list <- NULL
  extra_cols <- data_frame(x = list(), title = list(), cols = list())

  on_start_tier <- function() {
    # message("on_start_tier()")
    formatted_list <<- list()
  }

  on_end_tier <- function() {
    # message("on_end_tier()")
    if (length(formatted_list) > 0) {
      if (!is.null(cb$rowid)) {
        formatted_list <- c(list(pillar_format_parts_2(cb$rowid, cb$rowid_width)), formatted_list)
      }

      aligned <- map(formatted_list, `[[`, "aligned")

      tier <- format_colonnade_tier_2(aligned, bidi = get_pillar_option_bidi())

      cb$on_tier(tier)
    }
    formatted_list <<- NULL
  }

  on_pillar <- function(formatted) {
    # message("pillar()")
    # print(formatted)
    # print(pillar, width = width)
    formatted_list <<- c(formatted_list, list(formatted))
  }

  on_extra_cols <- function(x, title, cols) {
    # message("extra_cols()")
    # print(title)
    # print(cols)
    new_extra_cols <- data_frame(
      x = list(x), title = list(title), cols = list(cols)
    )
    # Add to the back, extra columns are emitted in order:
    extra_cols <<- vec_rbind(extra_cols, new_extra_cols)
  }

  cb_pillars <- new_emit_pillars_callbacks(
    cb$controller,
    on_start_tier = on_start_tier,
    on_end_tier = on_end_tier,
    on_pillar = on_pillar,
    on_top_level_pillar = function(...) {},
    on_extra_cols = on_extra_cols,
    on_abbrev_col = cb$on_abbrev_col,
    on_get_n_abbrev_cols = cb$on_get_n_abbrev_cols
  )

  # Side effect: populate `extra_cols`,
  # aggregate and forward calls to higher-level callbacks in `cb`
  emit_pillars(x, tier_widths, cb_pillars, focus)

  cb$on_extra_cols(extra_cols)
}

new_emit_pillars_callbacks <- function(controller,
                                       on_start_tier,
                                       on_end_tier,
                                       on_pillar,
                                       on_top_level_pillar,
                                       on_extra_cols,
                                       on_abbrev_col,
                                       on_get_n_abbrev_cols) {
  list(
    controller = controller,
    on_start_tier = on_start_tier,
    on_end_tier = on_end_tier,
    on_pillar = on_pillar,
    on_top_level_pillar = on_top_level_pillar,
    on_extra_cols = on_extra_cols,
    on_abbrev_col = on_abbrev_col,
    on_get_n_abbrev_cols = on_get_n_abbrev_cols
  )
}

emit_pillars <- function(x, tier_widths, cb, focus) {
  cb$on_start_tier()
  do_emit_focus_pillars(x, tier_widths, cb, focus)
  cb$on_end_tier()
}

do_emit_focus_pillars <- function(x, tier_widths, cb, focus) {
  stopifnot(is.data.frame(x))

  focus <- sort(match(focus, names(x)))
  # Shortcut
  if (length(focus) == 0) {
    do_emit_pillars(x, tier_widths, cb)
    return()
  }

  focus_formatted_list <- list()
  focus_top_level_end_idx <- integer()
  focus_extra_cols <- list()

  on_focus_pillar <- function(formatted) {
    # message("pillar()")
    # print(formatted)
    # print(pillar, width = width)
    focus_formatted_list <<- c(focus_formatted_list, list(formatted))
  }

  on_focus_top_level_pillar <- function() {
    focus_top_level_end_idx <<- c(focus_top_level_end_idx, length(focus_formatted_list))
  }

  on_focus_extra_cols <- function(x, title, cols) {
    # message("extra_cols()")
    # print(title)
    # print(cols)
    new_extra_cols <- data_frame(x = list(x), title = list(title), cols = list(cols))
    n_focus_formatted_list <- length(focus_formatted_list)

    if (n_focus_formatted_list <= length(focus_extra_cols)) {
      focus_extra_cols[[n_focus_formatted_list]] <<- vec_rbind(focus_extra_cols[[n_focus_formatted_list]], new_extra_cols)
    } else {
      focus_extra_cols[[n_focus_formatted_list]] <<- new_extra_cols
    }
  }

  cb_focus <- new_emit_pillars_callbacks(
    controller = cb$controller,
    on_start_tier = function(...) {},
    on_end_tier = function(...) {},
    on_pillar = on_focus_pillar,
    on_top_level_pillar = on_focus_top_level_pillar,
    on_extra_cols = on_focus_extra_cols,
    on_abbrev_col = cb$on_abbrev_col,
    on_get_n_abbrev_cols = cb$on_get_n_abbrev_cols
  )

  # Side effect: populates focus_formatted_list and focus_extra_cols
  do_emit_pillars(x[focus], tier_widths, cb_focus, is_focus = TRUE)

  # Can't show focus pillars that don't fit, but need to iterate
  # to emit extra columns in the correct order
  length(focus_extra_cols) <- length(focus)
  n_focus_shown <- length(focus_top_level_end_idx)

  before_start_idx <- vec_lag(focus + 1L, default = 1L)
  before_end_idx <- focus - 1L

  focus_top_level_start_idx <- vec_lag(focus_top_level_end_idx + 1L, default = 1L)

  # Apply similar strategy as in do_emit_pillars(), but ensure that
  # focus pillars are shown
  widths_focus <- map_int(focus_formatted_list, `[[`, "max_extent")
  rev <- distribute_pillars_rev(widths_focus, tier_widths)
  stopifnot(!anyNA(rev$tier))

  # This indicates the limit until which we expand non-focus column
  # before the current focus column:
  rev$offset_before <- pmax(rev$offset_after - rev$width - 1L, 0L)
  rev_before <- rev[focus_top_level_start_idx, ]
  stopifnot(nrow(rev_before) == n_focus_shown)

  # This indicates how far the current focus column (with all sub-pillars)
  # can extend. We use it for convenience to use the same logic as
  # do_emit_pillars().
  rev_after <- rev[focus_top_level_end_idx, ]
  stopifnot(nrow(rev_after) == n_focus_shown)

  x_pos <- 0L
  tier_pos <- 1L

  for (col in seq_along(focus)) {
    # Emit extra columns for focus pillars before processing non-focus pillars,
    # to keep extra columns in order between focus and non-focus pillars:
    my_extra_cols <- focus_extra_cols[[col]]
    for (extra_cols_row in seq_len(NROW(my_extra_cols))) {
      cb$on_extra_cols(
        my_extra_cols$x[[extra_cols_row]],
        my_extra_cols$title[[extra_cols_row]],
        my_extra_cols$cols[[extra_cols_row]]
      )
    }

    start <- before_start_idx[[col]]
    end <- before_end_idx[[col]]

    # Emit non-focus pillars that fit: use offset_before
    if (start <= end) {
      sub_tier_widths <- compute_sub_tier_widths(
        tier_widths, x_pos, tier_pos,
        rev_before$offset_before[[col]], rev_before$tier[[col]]
      )

      adv <- advance_emit_pillars(x_pos, tier_pos, x[seq2(start, end)], sub_tier_widths, cb)
      x_pos <- adv$x_pos
      tier_pos <- adv$tier_pos
    }

    # Emit already formatted focus pillar(s):
    if (col <= length(focus_top_level_start_idx)) {
      focus_pillars <- seq2(focus_top_level_start_idx[[col]], focus_top_level_end_idx[[col]])
    } else {
      focus_pillars <- integer()
    }

    for (focus_pillar in focus_pillars) {
      # Deduct widths: use offset_after
      sub_tier_widths <- compute_sub_tier_widths(
        tier_widths, x_pos, tier_pos,
        rev_after$offset_after[[col]], rev_after$tier[[col]]
      )

      used <- compute_used_width(sub_tier_widths, widths_focus[[focus_pillar]])

      if (used$tiers > 0) {
        cb$on_end_tier()
        cb$on_start_tier()
      }

      cb$on_pillar(focus_formatted_list[[focus_pillar]])

      adv <- advance_pos(x_pos, tier_pos, used)
      x_pos <- adv$x_pos
      tier_pos <- adv$tier_pos
    }
  }

  # Emit pillars after focus pillar
  if (length(focus) > 0) {
    start <- focus[[length(focus)]] + 1L
  } else {
    start <- 1L
  }
  end <- length(x)

  # Emit non-focus pillars that fit: use offset_before
  if (start <= end) {
    sub_tier_widths <- compute_sub_tier_widths(
      tier_widths, x_pos, tier_pos,
      tier_widths[[length(tier_widths)]], length(tier_widths)
    )

    adv <- advance_emit_pillars(x_pos, tier_pos, x[seq2(start, end)], sub_tier_widths, cb)
    x_pos <- adv$x_pos
    tier_pos <- adv$tier_pos
  }
}

advance_emit_pillars <- function(x_pos, tier_pos, ...) {
  used <- do_emit_pillars(...)
  advance_pos(x_pos, tier_pos, used)
}

do_emit_pillars <- function(x, tier_widths, cb, title = NULL, first_pillar = NULL, parent_col_idx = NULL, is_focus = FALSE) {
  top_level <- is.null(first_pillar)

  # Only tweaking sub-title, because full title is needed for extra-cols
  sub_title <- title
  if (!is.null(sub_title)) {
    sub_title[-length(sub_title)][parent_col_idx[-1] != 1] <- ""
  }

  pillar_list <- ctl_new_pillar_list(cb$controller, x, width = tier_widths, title = sub_title, first_pillar = first_pillar)

  # Extra columns are known early on, and remain fixed
  extra <- attr(pillar_list, "extra")

  if (length(pillar_list) == 0) {
    emit_extra_cols(extra, x, title, cb)
    # Doesn't fit
    return(NULL)
  }

  # Simple pillar: fit and proceed
  if (isTRUE(attr(pillar_list, "simple"))) {
    pillar <- pillar_list[[1]]

    pillar_title <- pillar[["title"]]
    title_width <- get_width(pillar_title) %||% 0L

    formatted <- pillar_format_parts_2(
      pillar,
      max(tier_widths),
      is_focus,
      cb$on_get_n_abbrev_cols() + 1L
    )
    true_width <- formatted$max_extent
    stopifnot(true_width <= max(tier_widths))

    used <- compute_used_width(tier_widths, true_width)

    if (used$tiers > 0) {
      cb$on_end_tier()
      cb$on_start_tier()
    }

    cb$on_pillar(formatted)

    if (true_width < title_width) {
      cb$on_abbrev_col(format(pillar_title))
    }

    return(used)
  }

  # We can proceed cautiously to the next level if space permits.
  # For each sub-pillar we allow at most as much space so that
  # we can print all first components of all remaining pillars
  # with the minimum width
  min_widths <- map_int(pillar_list, pillar_get_min_width)
  rev <- distribute_pillars_rev(min_widths, tier_widths)
  stopifnot(!anyNA(rev$tier))

  x_pos <- 0L
  tier_pos <- 1L

  # Advance column by column
  for (col in seq_along(pillar_list)) {
    sub_tier_widths <- compute_sub_tier_widths(
      tier_widths, x_pos, tier_pos,
      rev$offset_after[[col]], rev$tier[[col]]
    )
    "!!!!!DEBUG sub_tier_widths"

    # Recurse
    used <- do_emit_pillars(
      x[[col]],
      sub_tier_widths,
      cb,
      c(title, tick_if_needed(names(x)[[col]])),
      pillar_list[[col]],
      c(parent_col_idx, if (!is.null(names(x))) col),
      is_focus
    )
    "!!!!!DEBUG used"

    adv <- advance_pos(x_pos, tier_pos, used)
    x_pos <- adv$x_pos
    tier_pos <- adv$tier_pos

    if (top_level) {
      cb$on_top_level_pillar()
    }
  }

  # We emit late to ensure that extra columns of compound pillars
  # appear before top-level extra columns.
  emit_extra_cols(extra, x, title, cb)

  list(tiers = tier_pos - 1L, width = x_pos)
}

compute_used_width <- function(tier_widths, width) {
  if (width <= max(tier_widths)) {
    # Handle tier break
    used_tier <- which(width <= tier_widths)[[1]]
  } else {
    used_tier <- which.max(tier_widths)
    width <- tier_widths[[used_tier]]
  }
  list(tiers = used_tier - 1L, width = width)
}

compute_sub_tier_widths <- function(tier_widths, x_pos, tier_pos, x_target, tier_target) {
  stopifnot(tier_pos <= tier_target)

  if (tier_pos == tier_target) {
    sub_tier_widths <- x_target - x_pos
  } else {
    sub_tier_widths <- c(
      tier_widths[[tier_pos]] - x_pos,
      tier_widths[seq2(tier_pos + 1L, tier_target - 1L)],
      x_target
    )
  }
  if (x_pos > 0) {
    sub_tier_widths[[1]] <- max(sub_tier_widths[[1]] - 1L, 0L)
  }
  stopifnot(sub_tier_widths >= 0)
  sub_tier_widths
}

advance_pos <- function(x_pos, tier_pos, used) {
  if (!is.null(used)) {
    if (used$tiers > 0) {
      x_pos <- used$width
      tier_pos <- tier_pos + used$tiers
    } else {
      if (x_pos > 0) {
        x_pos <- x_pos + 1L
      }
      x_pos <- x_pos + used$width
    }
  }
  list(x_pos = x_pos, tier_pos = tier_pos)
}

emit_extra_cols <- function(extra, x, title, cb) {
  if (length(extra) == 0) {
    return()
  }

  if (is.numeric(extra)) {
    if (length(extra) == 1) {
      extra <- paste0("[", extra, "]")
    } else {
      extra <- paste0("[", min(extra), ":", max(extra), "]")
    }
    x_extra <- set_names(list(x[1, ]), extra)
  } else {
    extra <- tick_if_needed(extra)
    x_extra <- tick_names_if_needed(x)
  }

  cb$on_extra_cols(x_extra, title, extra)
}

# Reference: https://www.w3.org/International/questions/qa-bidi-unicode-controls
fsi <- function(x) {
  paste0("\u2068", x, "\u2069")
}

lro <- function(x) {
  paste0("\u202d", x, "\u202c")
}

format_colonnade_tier_2 <- function(x, bidi = FALSE) {
  if (length(x) == 0) {
    return(character())
  }

  if (bidi) {
    x <- map(x, fsi)
  }

  out <- exec(paste, !!!x)

  if (bidi) {
    out <- lro(out)
  }

  out
}

new_colonnade_body <- function(x, extra_cols, abbrev_cols, abbrev_col_idxs = numeric()) {
  "!!!!!DEBUG new_colonnade_body()"

  body <- as_glue(as.character(unlist(x)))
  extra_cols <- as.list(extra_cols)

  list(
    body = body,
    extra_cols = extra_cols,
    abbrev_cols = abbrev_cols,
    abbrev_col_idxs = abbrev_col_idxs
  )
}

#' Get a glimpse of your data
#'
#' @description
#' `glimpse()` is like a transposed version of `print()`:
#' columns run down the page, and data runs across.
#' This makes it possible to see every column in a data frame.
#' It's a little like [str()] applied to a data frame
#' but it tries to show you as much data as possible.
#' (And it always shows the underlying data, even when applied
#' to a remote data source.)
#'
#' See [format_glimpse()] for details on the formatting.
#'
#' @section S3 methods:
#' `glimpse` is an S3 generic with a customised method for `tbl`s and
#' `data.frames`, and a default method that calls [str()].
#'
#' @param x An object to glimpse at.
#' @param width Width of output: defaults to the setting of the
#'   `width` [option][pillar_options] (if finite)
#'   or the width of the console.
#' @param ... Unused, for extensibility.
#' @return x original x is (invisibly) returned, allowing `glimpse()` to be
#'   used within a data pipe line.
#' @export
#' @examples
#' glimpse(mtcars)
#'
#' @examplesIf rlang::is_installed("nycflights13")
#' glimpse(nycflights13::flights)
#'
glimpse <- function(x, width = NULL, ...) {
  UseMethod("glimpse")
}

#' @export
glimpse.tbl <- function(x, width = NULL, ...) {
  if (!is.null(width) && !is.finite(width)) {
    abort("`width` must be finite.")
  }
  width <- get_width_glimpse(width)

  cli::cat_line("Rows: ", big_mark(nrow(x)))

  # this is an overestimate, but shouldn't be too expensive.
  # every type needs at least three characters: "x, "
  rows <- as.integer(width / 3)
  df <- df_head(x, rows)
  cli::cat_line("Columns: ", big_mark(ncol(df)))

  summary <- tbl_sum(x)
  brief_summary <- summary[-1]

  if (has_length(brief_summary)) {
    cli::cat_line(names(brief_summary), ": ", brief_summary)
  }

  if (ncol(df) == 0) {
    return(invisible(x))
  }

  var_types <- map_chr(map(df, new_pillar_type), format)
  ticked_names <- format(new_pillar_title(tick_if_needed(names(df))))
  var_names <- paste0("$ ", align(ticked_names), " ", var_types, " ")

  # for some reason the offset was -2 in tibble but is now -1
  # so that the desired width is obtained
  data_width <- width - get_extent(var_names) - 1
  formatted <- map_chr(df, format_glimpse_1)
  truncated <- str_trunc(formatted, data_width)

  cli::cat_line(var_names, truncated)
  invisible(x)
}

#' @export
glimpse.data.frame <- glimpse.tbl

#' @importFrom utils str
#' @export
glimpse.default <- function(x, width = NULL, max.level = 3, ...) {
  str(x, width = get_width_glimpse(width), max.level = max.level, ...)
  invisible(x)
}

#' Format a vector for horizontal printing
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' This generic provides the logic for printing vectors in [glimpse()].
#'
#' The output strives to be as unambiguous as possible,
#' without compromising on readability.
#' In a list, to distinguish between vectors and nested lists,
#' the latter are surrounded by `[]` brackets.
#' Empty lists are shown as `[]`.
#' Vectors inside lists, of length not equal to one,
#' are surrounded by `<>` angle brackets.
#' Empty vectors are shown as `<>`.
#'
#' @return A character vector of the same length as `x`.
#' @inheritParams rlang::args_dots_used
#' @param x A vector.
#' @export
#' @examples
#' format_glimpse(1:3)
#'
#' # Lists use [], vectors inside lists use <>
#' format_glimpse(list(1:3))
#' format_glimpse(list(1, 2:3))
#' format_glimpse(list(list(1), list(2:3)))
#' format_glimpse(list(as.list(1), as.list(2:3)))
#' format_glimpse(list(character()))
#' format_glimpse(list(NULL))
#'
#' # Character strings are always quoted
#' writeLines(format_glimpse(letters[1:3]))
#' writeLines(format_glimpse(c("A", "B, C")))
#'
#' # Factors are quoted only when needed
#' writeLines(format_glimpse(factor(letters[1:3])))
#' writeLines(format_glimpse(factor(c("A", "B, C"))))
format_glimpse <- function(x, ...) {
  check_dots_used()
  UseMethod("format_glimpse")
}

# A variant with collapse and without checks, for format_glimpse.list()
format_glimpse_1 <- function(x, ...) {
  collapse(format_glimpse_(x, ...))
}

format_glimpse_ <- function(x, ...) {
  UseMethod("format_glimpse")
}


#' @export
format_glimpse.default <- function(x, ...) {
  dims <- dim(x)

  if (!is.null(dims)) {
    dims_out <- paste0(dims, collapse = " x ")
    paste0("<", class(x)[[1]], "[", dims_out, "]>")
  } else {
    format(x, trim = TRUE, justify = "none")
  }
}

#' @export
format_glimpse.list <- function(x, ..., .inner = FALSE) {
  if (!.inner && length(x) == 0) {
    return("list()")
  }

  out <- map_chr(x, format_glimpse_1, .inner = TRUE)

  # Surround vectors by <>
  # Only surround inner lists by []
  list <- map_lgl(x, is.list)
  scalar <- rep_along(x, TRUE)
  scalar[!list] <- (map_int(x[!list], length) == 1L)
  out[!scalar] <- paste0("<", out[!scalar], ">")
  out[list] <- paste0("[", out[list], "]")

  out
}

#' @export
format_glimpse.character <- function(x, ...) {
  encodeString(x, quote = '"')
}

#' @export
format_glimpse.factor <- function(x, ...) {
  if (any(grepl(",", levels(x), fixed = TRUE))) {
    encodeString(as.character(x), quote = '"')
  } else {
    format(x, trim = TRUE, justify = "none")
  }
}

#' Set up formatting
#'
#' @description
#' `tbl_format_setup()` is called by [format.tbl()].
#' This method collects information that is common to the header, body,
#' and footer parts of a tibble.
#' Examples:
#'
#' - the dimensions sometimes are reported both in the header
#' and (implicitly) in the footer of a tibble;
#' - the columns shown in the body decide which columns are shown in the footer.
#'
#' This information is computed in `tbl_format_setup()`.
#' The result is passed on to the
#' [tbl_format_header()], [tbl_format_body()], and [tbl_format_footer()]
#' methods.
#' If you need to customize parts of the printed output independently,
#' override these methods instead.
#'
#' By checking the `setup` argument, you can return an object that is
#' suitable for a call to [tbl_format_header()] if `setup` is `NULL`.
#' In this case, the method is called a second time with the return value
#' of the first call as `setup`.
#'
#' @details
#' Extend this method to prepare information that is used
#' in several parts of the printed output of a tibble-like object,
#' or to collect additional arguments passed via `...` to
#' [print.tbl()] or [format.tbl()].
#'
#' We expect that `tbl_format_setup()` is extended only rarely,
#' and overridden only in exceptional circumstances, if at all.
#' If you override this method, you must also implement
#' [tbl_format_header()], [tbl_format_body()], and [tbl_format_footer()]
#' for your class.
#'
#' Implementing a method
#' allows to override printing and formatting of the entire object
#' without overriding the [print()] and [format()] methods directly.
#' This allows to keep the logic of the `width` and `n` arguments.
#'
#' @param x
#'   An object.
#' @param width
#'   Actual width for printing, a numeric greater than zero.
#'   This argument is mandatory for all implementations of this method.
#' @param ...
#'   Extra arguments to [print.tbl()] or [format.tbl()].
#' @param setup
#'   This generic is first called with `setup = NULL` .
#'   If the method _evaluates_ this argument, the return value
#'   will only be used in a call to [tbl_format_header()],
#'   and after that, a second call to this generic will be made
#'   with the return value of the first call as `setup`
#'   which then will be used in calls to [tbl_format_body()] and [tbl_format_footer()].
#'   This allows displaying the header before starting the computation
#'   required for the body and footer.
#' @param n
#'   Actual number of rows to print.
#'   No [options][pillar_options] should be considered
#'   by implementations of this method.
#' @param max_extra_cols
#'   Number of columns to print abbreviated information for,
#'   if the width is too small for the entire tibble.
#'   No [options][pillar_options] should be considered
#'   by implementations of this method.
#' @param max_footer_lines
#'   Maximum number of lines for the footer.
#'   No [options][pillar_options] should be considered
#'   by implementations of this method.
#' @param focus `r lifecycle::badge("experimental")`
#'
#'   Names of columns to show preferentially if space is tight.
#' @return
#'   An object that can be passed as `setup` argument to
#'   [tbl_format_header()], [tbl_format_body()], and [tbl_format_footer()].
#' @export
#' @examplesIf rlang::is_installed(c("palmerpenguins", "tibble"))
#' tbl_format_setup(palmerpenguins::penguins)
tbl_format_setup <- function(
  x,
  width = NULL,
  ...,
  setup = list(tbl_sum = tbl_sum(x)),
  n = NULL,
  max_extra_cols = NULL,
  max_footer_lines = NULL,
  focus = NULL
) {
  "!!!!DEBUG tbl_format_setup()"

  width <- get_width_print(width)

  n <- get_n_print(n, tbl_nrow(x))

  max_extra_cols <- get_max_extra_cols(max_extra_cols)
  max_footer_lines <- get_max_footer_lines(max_footer_lines)

  # Calls UseMethod("tbl_format_setup"),
  # allows using default values in S3 dispatch
  out <- tbl_format_setup_dispatch(
    x,
    width,
    ...,
    setup = setup,
    n = n,
    max_extra_cols = max_extra_cols,
    max_footer_lines = max_footer_lines,
    focus = focus
  )
  return(out)
  UseMethod("tbl_format_setup")
}

tbl_format_setup_dispatch <- function(x, width, ..., n, max_extra_cols, max_footer_lines, focus = NULL) {
  UseMethod("tbl_format_setup")
}

#' @details
#' The default method for the `"tbl"` class collects information for
#' standard printing for tibbles.
#' See [new_tbl_format_setup()] for details on the returned object.
#'
#' @rdname tbl_format_setup
#' @export
tbl_format_setup.tbl <- function(
  x,
  width,
  ...,
  setup,
  n,
  max_extra_cols,
  max_footer_lines,
  focus
) {
  "!!!!DEBUG tbl_format_setup.tbl()"

  if (is.null(setup)) {
    # Header with early exit
    tbl_sum <- tbl_sum(x)
    return(new_tbl_format_setup(width, tbl_sum, rows_total = NA_integer_))
  } else {
    tbl_sum <- setup$tbl_sum
  }

  # Number of rows
  rows <- tbl_nrow(x)

  lazy <- is.na(rows)
  if (lazy) {
    df <- as.data.frame(head(x, n + 1))
    if (nrow(df) <= n) {
      rows <- nrow(df)
    } else {
      df <- vec_head(df, n)
    }
  } else {
    df <- df_head(x, n)
  }

  if (is.na(rows)) {
    # Lazy table with too many rows
    needs_dots <- (nrow(df) >= n)
  } else {
    # Lazy table with few rows, or regular data frame
    needs_dots <- (rows > n)
  }

  if (needs_dots) {
    rows_missing <- rows - n
  } else {
    rows_missing <- 0L
  }

  # Body
  rownames(df) <- NULL

  colonnade <- ctl_colonnade(
    df,
    has_row_id = if (!lazy && .row_names_info(x) > 0) "*" else TRUE,
    width = width,
    controller = x,
    focus = focus
  )

  body <- colonnade$body

  # Extra columns
  extra_cols <- colonnade$extra_cols
  extra_cols_total <- length(extra_cols)

  if (extra_cols_total > max_extra_cols) {
    length(extra_cols) <- max_extra_cols
  }

  # Abbreviated columns
  abbrev_cols <- colonnade$abbrev_cols

  # Result
  new_tbl_format_setup(
    x = x,
    df = df,
    width = width,
    tbl_sum = tbl_sum,
    body = body,
    rows_missing = rows_missing,
    rows_total = rows,
    extra_cols = extra_cols,
    extra_cols_total = extra_cols_total,
    max_footer_lines = max_footer_lines,
    abbrev_cols = abbrev_cols
  )
}

#' Number of rows in a tbl object
#'
#' This generic will be called by [tbl_format_setup()] to determine the number
#' of rows in a tbl object.
#'
#' @param x A tbl object.
#' @inheritParams rlang::args_dots_empty
#' @export
tbl_nrow <- function(x, ...) {
  check_dots_empty0(...)
  UseMethod("tbl_nrow")
}

#' @export
tbl_nrow.tbl <- function(x, ...) {
  nrow(x)
}


#' Construct a setup object for formatting
#'
#' @description
#' The object returned from the default method of [tbl_format_setup()]
#' is an object with a `"class"` attribute and the elements described in the
#' "Parameters" section.
#'
#' Named elements can be added to such objects without affecting the behavior.
#' Do not modify existing elements.
#'
#' @param x The input object unchanged.
#' @param df A data frame representation of the intended output,
#'   trimmed to the desired number of rows.
#' @param width The `width` argument unchanged.
#' @param tbl_sum A named character vector, as returned from [tbl_sum()].
#' @param body A character vector with the formatted body,
#'   one element per line,
#' @param rows_missing The number of rows not shown from the body,
#'   `NA` if unknown.
#' @param rows_total The total number of rows in the data,
#'   `NA` if unknown.
#' @param extra_cols Columns that did not fit into the body,
#'   as a character vector of formatted column names and types.
#' @param extra_cols_total The total number of columns, may be larger than
#'   `length(extra_cols)`.
#' @param max_footer_lines The maximum number of lines in the footer.
#' @param abbrev_cols Formatted names of the columns that are shown abbreviated
#'   in the body.
#'
#' @keywords internal
new_tbl_format_setup <- function(
  width,
  tbl_sum,
  x = NULL,
  df = NULL,
  body = NULL,
  rows_missing = NULL,
  rows_total = NULL,
  extra_cols = NULL,
  extra_cols_total = NULL,
  max_footer_lines = NULL,
  abbrev_cols = NULL
) {
  trunc_info <- list(
    x = x,
    df = df,
    width = width,
    tbl_sum = tbl_sum,
    body = body,
    rows_missing = rows_missing,
    rows_total = rows_total,
    extra_cols = extra_cols,
    extra_cols_total = extra_cols_total,
    max_footer_lines = max_footer_lines,
    abbrev_cols = abbrev_cols
  )

  structure(trunc_info, class = "pillar_tbl_format_setup")
}

is_tbl_format_setup <- function(x) {
  inherits(x, "pillar_tbl_format_setup")
}

# For testing
#' @export
print.pillar_tbl_format_setup <- function(x, ...) {
  writeLines(format(x, ...))
  invisible(x)
}

#' @export
format.pillar_tbl_format_setup <- function(x, ...) {
  check_dots_empty()
  c(
    cli::style_bold("<pillar_tbl_format_setup>"),
    tbl_format_header(x),
    tbl_format_body(x),
    tbl_format_footer(x)
  )
}

test_that("compound pillars and multiple tiers", {
  local_options(width = 80)

  x <- new_tbl(list(a = 3, b = new_tbl(list(x = 1, y = 2))))

  expect_snapshot({
    options(width = 15)
    tbl_format_setup(x, width = 30)

    options(width = 10)
    tbl_format_setup(x, width = 30)
  })
})

test_that("compound pillars with zero columns (#402)", {
  expect_snapshot({
    tbl_format_setup(new_tbl(list(x = 1:2, y = matrix(integer(), ncol = 0, nrow = 2))))

    tbl_format_setup(new_tbl(list(x = 1:2, y = new_tbl(list(), n = 2L))))
  })
})

style_rowid <- function(x) {
  style_hint(x)
}

rif_shaft <- function(n, ...) {
  new_pillar_shaft(list(row_ids = seq_len(n)),
    width = as.integer(floor(log10(max(n, 1))) + 1),
    class = "pillar_rif_shaft"
  )
}

#' @export
format.pillar_rif_shaft <- function(x, width, ...) {
  new_ornament(
    style_rowid(format(x$row_ids, width = width)),
    width = width,
    align = "right"
  )
}

format_exp <- function(exp, si) {
  if (si) {
    si(exp)
  } else {
    supernum(exp)
  }
}

si <- function(x) {
  num <- !is.na(x)
  if (!any(num)) {
    return(rep_along(x, ""))
  }

  if (all(x[num] == 0L)) {
    return(rep_along(x, ""))
  }

  si_prefixes <- c(
    "y", "z", "a", "f", "p", "n", "\u00b5", "m", " ", "k", "M", "G", "T", "P", "E", "Z", "Y"
  )

  idx <- (x / 3) + 9
  idx[is.na(idx)] <- 9
  style_bold(si_prefixes[idx])
}

supernum <- function(x) {
  stopifnot(is.integer(x))

  num <- !is.na(x)
  if (!any(num)) {
    return(rep_along(x, ""))
  }

  neg <- num & x < 0
  if (any(neg)) {
    neg_chr <- ifelse(neg, "-", "+")
    neg_chr[!num] <- " "
  } else {
    neg_chr <- rep("", length(x))
  }

  digits <- as.character(abs(x))
  digits[!num] <- ""

  exp <- paste0(neg_chr, format(digits, justify = "right"))

  paste0(style_subtle(ifelse(num, "e", " ")), style_num(exp, neg))
}

# nocov start - expect_known_display()
#' Test helpers
#'
#' Use [testthat::expect_snapshot()] instead of `expect_known_display()`.
#'
#' @inheritParams testthat::expect_known_output
#' @param object An object to check.
#' @param ... Unused.
#' @param width The width of the output.
#' @param crayon Color the output?
#' @rdname deprecated
#' @export
expect_known_display <- function(object, file, ..., width = 80L, crayon = TRUE) {
  lifecycle::deprecate_stop(
    "1.7.0", "pillar::expect_known_display()",
    "testthat::expect_snapshot()"
  )
}
# nocov end

#' Format a numeric vector in a tibble
#'
#' These functions are reexported as [tibble::num()] and [tibble::set_num_opts()].
#'
#' @keywords internal
#' @export
num <- function(x, ...,
                sigfig = NULL, digits = NULL,
                label = NULL, scale = NULL,
                notation = c("fit", "dec", "sci", "eng", "si"),
                fixed_exponent = NULL,
                extra_sigfig = NULL) {
  stopifnot(is.numeric(x))
  stopifnot(is.null(digits) || is_integerish(digits))
  check_dots_empty()

  # FIXME: math and arith should also work for integers
  x[] <- as.numeric(x)

  if (missing(notation)) {
    notation <- NULL
  }

  # FIXME: new_vctr() overrides class attribute, doesn't support subclassing
  out <- set_num_opts(
    x,
    sigfig = sigfig,
    digits = digits,
    label = label,
    scale = scale,
    notation = notation,
    fixed_exponent = fixed_exponent,
    extra_sigfig = extra_sigfig
  )

  # FIXME: Include class(x) to support subclassing/mixin?
  # Needs vec_base(): vec_data() but only strips vctrs_vctr class?
  # Avoid inheriting from numeric (?): https://github.com/r-lib/vctrs/issues/1339
  new_class <- c("pillar_num", "pillar_vctr", "vctrs_vctr", "double")
  class(out) <- new_class

  out
}

#' @export
vec_ptype_abbr.pillar_num <- function(x, ...) {
  pillar_attr <- attr(x, "pillar", exact = TRUE)
  notation <- pillar_attr$notation
  if (is.null(notation)) {
    notation <- "num"
  }

  sigfig <- pillar_attr$sigfig
  digits <- pillar_attr$digits

  if (!is.null(digits)) {
    if (digits > 0) {
      out <- paste0(notation, ":.", digits, "!")
    } else {
      out <- paste0(notation, ":.", -digits)
    }
  } else if (!is.null(sigfig)) {
    out <- paste0(notation, ":", sigfig)
  } else {
    out <- notation
  }

  out
}

#' @export
format.pillar_num <- function(x, trim = FALSE, ...) {
  "!!!!DEBUG format.pillar_num()"

  shaft <- pillar_shaft(x)
  out <- format(shaft, width = get_width(shaft))
  if (trim) {
    attributes(out) <- NULL
  } else {
    out <- format(out, align = "right")
  }
  out
}

#' @method vec_arith pillar_num
#' @export
vec_arith.pillar_num <- function(op, x, y, ...) {
  "!!!!DEBUG vec_arith.pillar_num(`v(op)`)"

  UseMethod("vec_arith.pillar_num", y)
}
#' @method vec_arith.pillar_num default
#' @export
vec_arith.pillar_num.default <- function(op, x, y, ...) {
  "!!!!DEBUG vec_arith.pillar_num.default(`v(op)`)"
  stopifnot(is.numeric(x), is.numeric(y))
  out <- vec_arith_base(op, vec_proxy(x), vec_proxy(y))

  if (inherits(x, "pillar_num")) {
    vec_restore(out, x)
  } else {
    vec_restore(out, y)
  }
}
#' @method vec_arith.pillar_num MISSING
#' @export
vec_arith.pillar_num.MISSING <- function(op, x, y, ...) {
  "!!!!DEBUG vec_arith.pillar_num.MISSING(`v(op)`)"
  stopifnot(is.numeric(x))
  # FIXME
  out <- vec_arith_base(op, 0, vec_proxy(x))

  vec_restore(out, x)
}

#' @method vec_arith.numeric pillar_num
#' @export
vec_arith.numeric.pillar_num <- vec_arith.pillar_num.default


#' @export
vec_math.pillar_num <- function(op, x, ...) {
  "!!!!DEBUG vec_math(`v(op)`)"

  stopifnot(is.numeric(x))
  out <- vec_math_base(op, vec_proxy(x))

  if (is.numeric(out)) {
    out <- vec_restore(out, x)
  }

  out
}

#' @export
#' @rdname num
set_num_opts <- function(x, ...,
                         sigfig = NULL, digits = NULL,
                         label = NULL, scale = NULL,
                         notation = c("fit", "dec", "sci", "eng", "si"),
                         fixed_exponent = NULL,
                         extra_sigfig = NULL) {
  check_dots_empty()

  if (missing(notation)) {
    notation <- NULL
  } else if (!is.null(notation)) {
    notation <- arg_match(notation)
  }

  if (!is.null(fixed_exponent) && !is.null(notation)) {
    if (fixed_exponent && notation == "dec") {
      abort('Incomatible arguments: `notation = "dec"` and `fixed_exponent` given')
    }
  }

  if (!is.null(scale) && is.null(label)) {
    abort("Must set `label` if `scale` is provided.")
  }

  if (!is.null(digits) && !is.null(extra_sigfig)) {
    abort("Incompatible arguments: `extra_sigfig` and `digits`.")
  }

  pillar_attr <- structure(
    list(
      sigfig = sigfig,
      digits = digits,
      label = label,
      scale = scale,
      notation = notation,
      fixed_exponent = fixed_exponent,
      extra_sigfig = extra_sigfig
    ),
    class = c("pillar_num_attr", "pillar_vctr_attr", "tibble_vec_attr")
  )
  attr(x, "pillar") <- pillar_attr
  x
}

#' @export
format.pillar_num_attr <- function(x, ...) {
  notation <- x$notation
  if (is.null(notation)) {
    class <- "pillar_num"
  } else {
    class <- paste0("pillar_num(", notation, ")")
  }

  sigfig <- x$sigfig
  digits <- x$digits
  label <- x$label
  extra_sigfig <- x$extra_sigfig

  if (!is.null(digits)) {
    if (digits >= 0) {
      out <- paste0(class, ":.", digits, "!")
    } else {
      out <- paste0(class, ":.", -digits)
    }
  } else if (!is.null(sigfig)) {
    out <- paste0(class, ":", sigfig)

    if (isTRUE(extra_sigfig)) {
      out <- paste0(out, "*")
    }
  } else if (isTRUE(extra_sigfig)) {
    out <- paste0(class, "*")
  } else {
    out <- class
  }

  if (!is.null(label)) {
    out <- paste0(out, "{", label, "}")
  }

  scale <- x$scale

  if (!is.null(scale)) {
    out <- paste0(out, "*", scale)
  }

  fixed_exponent <- x$fixed_exponent

  if (!is.null(fixed_exponent)) {
    out <- paste0(out, "|", fixed_exponent)
  }

  out
}

#' @export
vec_ptype2.pillar_num.pillar_num <- function(x, y, ...) {
  x
}
#' @export
vec_ptype2.pillar_num.double <- function(x, y, ...) {
  x
}
#' @export
vec_ptype2.double.pillar_num <- function(x, y, ...) {
  y
}

#' @export
vec_ptype2.pillar_num.integer <- function(x, y, ...) {
  x
}
#' @export
vec_ptype2.integer.pillar_num <- function(x, y, ...) {
  y
}

#' @export
vec_cast.pillar_num.pillar_num <- function(x, to, ...) {
  pillar_x <- attr(x, "pillar", exact = TRUE)
  pillar_to <- attr(to, "pillar", exact = TRUE)

  pillar_x_label <- pillar_x$label
  pillar_to_label <- pillar_to$label

  if (!is.null(pillar_x_label) && !is.null(pillar_to_label)) {
    if (!identical(pillar_x$label, pillar_to$label)) {
      abort("Only `pillar_num` objects with the same label can be combined.")
    }
  }

  attr(x, "pillar") <- pillar_to
  x
}
#' @export
vec_cast.double.pillar_num <- function(x, to, ...) {
  vec_data(x)
}
#' @export
vec_cast.pillar_num.double <- function(x, to, ...) {
  vec_restore(vec_proxy(x), to)
}
#' @export
vec_cast.pillar_num.integer <- function(x, to, ...) {
  vec_restore(as.numeric(x), to)
}

new_foo_tbl <- function(..., class = NULL) {
  stopifnot(is.null(class))
  new_tbl(..., class = "foo_tbl")
}

local_foo_tbl <- function(frame = caller_env()) {
  local_methods(
    .frame = frame,
    tbl_sum.foo_tbl = function(x) {
      c(NextMethod(), foo = "bar")
    }
  )
}

test_that("tbl_format_header() results", {
  local_unknown_rows()
  local_foo_tbl()

  expect_snapshot({
    tbl_format_header(tbl_format_setup(as_tbl(mtcars), width = 80))

    tbl_format_header(tbl_format_setup(as_unknown_rows(trees), width = 30, n = 10))

    "Narrow"
    tbl_format_header(tbl_format_setup(as_tbl(mtcars), width = 10))

    "Custom tbl_sum()"
    tbl_format_header(tbl_format_setup(new_foo_tbl(), width = 30))

    "Focus columns"
    tbl_format_header(tbl_format_setup(new_tbl(trees, pillar_focus = "Volume"), width = 30))
  })
})

#' Components of a pillar
#'
#' `new_pillar_component()` constructs an object of class `"pillar_component"`.
#'
#' Objects of class `"pillar"` are internally a named lists of their components.
#' The default components are `title` (may be missing), `type`, and `data`.
#' Each component is a `"pillar_component"`.
#'
#' This class captures contents that can be fitted in a rectangle.
#' Each component consists of one or multiple cells that
#' are aligned horizontally (with one space in between) when printed.
#' Each cell has a maximum (i.e., desired) width and may have a minimum width
#' if the contents are compressible.
#' The component object stores the width of the cells as an attribute.
#'
#' @inheritParams ellipsis::dots_empty
#' @param x A bare list (for `new_pillar_component()`), or an object
#'   with attributes `"width"` and `"min_width"` attributes (for `pillar_component()`).
#' @param width,min_width Width and minimum width for the new component.
#'   If `min_width` is `NULL`, it is assumed to match `width`.
#' @export
#' @examples
#' new_pillar_component(list(letters[1:3]), width = 1)
#' pillar_component(new_pillar_title("letters"))
#' pillar_component(new_pillar_type(letters))
#' pillar_component(pillar_shaft(letters[1:3]))
new_pillar_component <- function(x, ..., width, min_width = NULL) {
  "!!!!DEBUG new_pillar_component(`v(x)`, `v(width)`, `v(min_width)`)"

  check_dots_empty()
  stopifnot(rlang::is_bare_list(x))
  stopifnot(is_integerish(width), length(width) == 1L)
  if (!is.null(min_width)) {
    stopifnot(is_integerish(min_width), length(min_width) == 1L)
  }

  structure(
    x,
    width = width,
    min_width = min_width,
    class = "pillar_component"
  )
}

#' pillar_component()
#'
#' `pillar_component()` is a convenience helper that wraps the input in a list
#' and extracts width and minimum width.
#'
#' @export
#' @rdname new_pillar_component
pillar_component <- function(x) {
  # FIXME: No longer need to wrap in a list, cell concept abandoned
  new_pillar_component(list(x), width = get_width(x), min_width = get_min_width(x))
}

pillar_get_widths <- function(x) {
  as.integer(exec(max, !!!map(x, get_width)))
}

pillar_get_min_widths <- function(x) {
  as.integer(exec(max, !!!map(x, get_min_width)))
}

pillar_format_parts_2 <- function(x, width, is_focus = FALSE) {
  widths <- pillar_get_widths(x)
  formatted <- map(x, function(.x) format(.x[[1L]], width = min(width, get_width(.x))))

  # FIXME: Support missing type component
  flat_focus_pos <- integer()
  if (is_focus) {
    type_idx <- which(names(x) == "type")
    if (length(type_idx) > 0) {
      formatted[[type_idx]] <- crayon_underline(formatted[[type_idx]])
    }
  }

  align <- attr(formatted[["data"]], "align", exact = TRUE) %||% "left"

  flat <- unlist(formatted)
  extent <- get_extent(flat)
  max_extent <- max(extent)
  if (max_extent > width) {
    flat <- str_trunc(flat, width)
    max_extent <- width
  }
  aligned <- align_impl(flat, max_extent, align, " ", extent)

  list(max_extent = max_extent, aligned = aligned, components = names(x))
}

test_that("rowids for line-style", {
  lines <- function(char = "-") {
    stopifnot(nchar(char) == 1)
    structure(char, class = "lines")
  }

  format_lines <- function(x, width, ...) {
    paste(rep(x, width), collapse = "")
  }

  ctl_new_pillar_line_tbl <- function(controller, x, width, ..., title = NULL) {
    out <- NextMethod()
    new_pillar(list(
      title = out$title,
      type = out$type,
      lines = new_pillar_component(list(lines("=")), width = 1),
      data = out$data
    ))
  }

  ctl_new_rowid_pillar_line_tbl <- function(controller, x, width, ...) {
    out <- NextMethod()
    new_pillar(
      list(
        title = out$title,
        type = out$type,
        lines = new_pillar_component(list(lines("=")), width = 1),
        data = out$data
      ),
      width = as.integer(floor(log10(max(nrow(x), 1))) + 1)
    )
  }

  local_methods(
    format.lines = format_lines,
    ctl_new_pillar.line_tbl = ctl_new_pillar_line_tbl,
    ctl_new_rowid_pillar.line_tbl = ctl_new_rowid_pillar_line_tbl
  )

  expect_snapshot({
    vctrs::new_data_frame(
      list(a = 1:3, b = letters[1:3]),
      class = c("line_tbl", "tbl")
    )
  })
})

test_that("roman rowids", {
  ctl_new_rowid_pillar_roman_tbl <- function(controller, x, width, ...) {
    out <- NextMethod()
    rowid <- utils::as.roman(seq_len(nrow(x)))
    width <- max(nchar(as.character(rowid)))
    new_pillar(
      list(
        title = out$title,
        type = out$type,
        data = pillar_component(
          new_pillar_shaft(list(row_ids = rowid),
            width = width,
            class = "pillar_rif_shaft"
          )
        )
      ),
      width = width
    )
  }

  local_methods(
    ctl_new_rowid_pillar.roman_tbl = ctl_new_rowid_pillar_roman_tbl
  )

  expect_snapshot({
    vctrs::new_data_frame(
      list(a = 1:3, b = letters[1:3]),
      class = c("roman_tbl", "tbl")
    )
  })
})

as_override_tbl_sum <- function(x) {
  structure(x, class = c("override_tbl_sum", class(x)))
}

local_override_tbl_sum <- function(frame = caller_env()) {
  local_methods(
    .frame = frame,
    tbl_sum.override_tbl_sum = function(x) {
      c(NextMethod(), "Overridden" = "tbl_sum")
    }
  )
}

## ---- include = FALSE---------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

pillar:::set_show_source_hooks()

## ----setup--------------------------------------------------------------------
library(pillar)

## ----echo = FALSE, error = TRUE-----------------------------------------------
DiagrammeR::mermaid("format.mmd")

## -----------------------------------------------------------------------------
tbl <- tibble::tibble(a = 1:3, b = tibble::tibble(c = 4:6, d = 7:9), e = 10:12)
print(tbl, width = 23)
str(tbl)

## ----show_source = TRUE-------------------------------------------------------
#  print.tbl <- function (x, width = NULL, ..., n = NULL, max_extra_cols = NULL, 
#      max_footer_lines = NULL) 
#  {
#      print_tbl(x, width, ..., n = n, max_extra_cols = max_extra_cols, 
#          max_footer_lines = max_footer_lines)
#  }

## ----show_source = TRUE-------------------------------------------------------
#  format.tbl <- function (x, width = NULL, ..., n = NULL, max_extra_cols = NULL, 
#      max_footer_lines = NULL) 
#  {
#      format_tbl(x, width, ..., n = n, max_extra_cols = max_extra_cols, 
#          max_footer_lines = max_footer_lines)
#  }

## -----------------------------------------------------------------------------
setup <- tbl_format_setup(tbl, width = 24)
setup

## ----show_source = TRUE-------------------------------------------------------
#  tbl_format_setup <- function (x, width = NULL, ..., n = NULL, max_extra_cols = NULL, 
#      max_footer_lines = NULL, focus = NULL) 
#  {
#      "!!!!DEBUG tbl_format_setup()"
#      width <- get_width_print(width)
#      n <- get_n_print(n, nrow(x))
#      max_extra_cols <- get_max_extra_cols(max_extra_cols)
#      max_footer_lines <- get_max_footer_lines(max_footer_lines)
#      out <- tbl_format_setup_dispatch(x, width, ..., n = n, max_extra_cols = max_extra_cols, 
#          max_footer_lines = max_footer_lines, focus = focus)
#      return(out)
#      UseMethod("tbl_format_setup")
#  }

## ----show_source = TRUE-------------------------------------------------------
#  tbl_format_setup.tbl <- function (x, width, ..., n, max_extra_cols, max_footer_lines, 
#      focus) 
#  {
#      "!!!!DEBUG tbl_format_setup.tbl()"
#      rows <- nrow(x)
#      if (is.na(rows)) {
#          df <- df_head(x, n + 1)
#          if (nrow(df) <= n) {
#              rows <- nrow(df)
#          }
#          else {
#              df <- vec_head(df, n)
#          }
#      }
#      else {
#          df <- df_head(x, n)
#      }
#      if (is.na(rows)) {
#          needs_dots <- (nrow(df) >= n)
#      }
#      else {
#          needs_dots <- (rows > n)
#      }
#      if (needs_dots) {
#          rows_missing <- rows - n
#      }
#      else {
#          rows_missing <- 0
#      }
#      tbl_sum <- tbl_sum(x)
#      rownames(df) <- NULL
#      colonnade <- ctl_colonnade(df, has_row_id = if (.row_names_info(x) > 
#          0) 
#          "*"
#      else TRUE, width = width, controller = x, focus = focus)
#      body <- colonnade$body
#      extra_cols <- colonnade$extra_cols
#      extra_cols_total <- length(extra_cols)
#      if (extra_cols_total > max_extra_cols) {
#          length(extra_cols) <- max_extra_cols
#      }
#      abbrev_cols <- colonnade$abbrev_cols
#      new_tbl_format_setup(x = x, df = df, width = width, tbl_sum = tbl_sum, 
#          body = body, rows_missing = rows_missing, rows_total = rows, 
#          extra_cols = extra_cols, extra_cols_total = extra_cols_total, 
#          max_footer_lines = max_footer_lines, abbrev_cols = abbrev_cols)
#  }

## -----------------------------------------------------------------------------
tbl_format_header(tbl, setup)
tbl_format_body(tbl, setup)
tbl_format_footer(tbl, setup)

## -----------------------------------------------------------------------------
class(tbl_format_body(tbl, setup))
typeof(tbl_format_body(tbl, setup))

## ----show_source = TRUE-------------------------------------------------------
#  tbl_format_header.tbl <- function (x, setup, ...) 
#  {
#      named_header <- setup$tbl_sum
#      focus <- attr(x, "pillar_focus")
#      if (!is.null(focus)) {
#          named_header <- c(named_header, `Focus columns` = collapse(tick_if_needed(focus)))
#      }
#      if (all(names2(named_header) == "")) {
#          header <- named_header
#      }
#      else {
#          header <- paste0(align(paste0(names2(named_header), ":"), 
#              space = NBSP), " ", named_header)
#      }
#      style_subtle(format_comment(header, width = setup$width))
#  }

## ----show_source = TRUE-------------------------------------------------------
#  tbl_format_body.tbl <- function (x, setup, ...) 
#  {
#      force(setup)
#      setup$body
#  }

## ----show_source = TRUE-------------------------------------------------------
#  tbl_format_footer.tbl <- function (x, setup, ...) 
#  {
#      footer <- format_footer(x, setup)
#      footer_comment <- wrap_footer(footer, setup)
#      footer_advice <- format_footer_advice(x, setup)
#      footer_advice_comment <- wrap_footer(footer_advice, setup, 
#          lines = 1, ellipsis = FALSE)
#      style_subtle(c(footer_comment, footer_advice_comment))
#  }

## -----------------------------------------------------------------------------
ctl_new_pillar_list(tbl, tbl$a, width = 20)
ctl_new_pillar_list(tbl, tbl$b, width = 20)

## ----show_source = TRUE-------------------------------------------------------
#  ctl_new_pillar_list.tbl <- function (controller, x, width, ..., title = NULL, first_pillar = NULL) 
#  {
#      "!!!!DEBUG ctl_new_pillar_list.tbl(`v(width)`, `v(title)`)"
#      if (is.data.frame(x)) {
#          new_data_frame_pillar_list(x, controller, width, title = title, 
#              first_pillar = first_pillar)
#      }
#      else if (is.matrix(x) && !inherits(x, c("Surv", "Surv2"))) {
#          new_matrix_pillar_list(x, controller, width, title = title, 
#              first_pillar = first_pillar)
#      }
#      else if (is.array(x) && length(dim(x)) > 2) {
#          new_array_pillar_list(x, controller, width, title = title, 
#              first_pillar = first_pillar)
#      }
#      else {
#          if (is.null(first_pillar)) {
#              first_pillar <- ctl_new_pillar(controller, x, width, 
#                  ..., title = prepare_title(title))
#          }
#          new_single_pillar_list(first_pillar, width)
#      }
#  }

## -----------------------------------------------------------------------------
ctl_new_pillar(tbl, tbl$a, width = 20)

## ----show_source = TRUE-------------------------------------------------------
#  ctl_new_pillar.tbl <- function (controller, x, width, ..., title = NULL) 
#  {
#      "!!!!DEBUG ctl_new_pillar.tbl(`v(width)`, `v(title)`)"
#      pillar(x, title, if (!is.null(width)) 
#          max0(width))
#  }

## ----show_source = TRUE-------------------------------------------------------
#  pillar <- function (x, title = NULL, width = NULL, ...) 
#  {
#      "!!!!DEBUG pillar(`v(class(x))`, `v(title)`, `v(width)`)"
#      pillar_from_shaft(new_pillar_title(title), new_pillar_type(x), 
#          pillar_shaft(x, ...), width)
#  }

## ----show_source = TRUE-------------------------------------------------------
#  new_pillar <- function (components, ..., width = NULL, class = NULL, extra = deprecated()) 
#  {
#      "!!!!DEBUG new_pillar(`v(width)`, `v(class)`)"
#      if (is_present(extra)) {
#          deprecate_warn("1.7.0", "pillar::new_pillar(extra = )")
#      }
#      check_dots_empty()
#      if (length(components) > 0 && !is_named(components)) {
#          abort("All components must have names.")
#      }
#      structure(components, width = width, class = c(class, "pillar"))
#  }

## ----show_source = TRUE-------------------------------------------------------
#  format.pillar <- function (x, width = NULL, ...) 
#  {
#      if (is.null(width)) {
#          width <- get_width(x)
#      }
#      if (is.null(width)) {
#          width <- pillar_get_width(x)
#      }
#      as_glue(pillar_format_parts_2(x, width)$aligned)
#  }

#' Constructor for column data
#'
#' @description
#' The `new_pillar_shaft()` constructor creates objects of the `"pillar_shaft"`
#' class.
#' This is a virtual or abstract class, you must specify the `class`
#' argument.
#' By convention, this should be a string that starts with `"pillar_shaft_"`.
#' See `vignette("extending", package = "tibble")` for usage examples.
#'
#' This method accepts a vector of arbitrary length and is expected to return an S3 object with the following properties:
#'
#' - It has an attribute `"width"`
#' - It can have an attribute `"min_width"`, if missing, `"width"` is used
#' - It must implement a method `format(x, width, ...)` that can be called with any value between `min_width` and `width`
#' - This method must return an object that inherits from `character` and has attributes `"align"` (with supported values `"left"`, `"right"`, and `"center"`) and `"width"`
#'
#' The function [new_pillar_shaft()] returns such an object, and also correctly formats `NA` values.
#' In many cases, the implementation of `pillar_shaft.your_class_name()` will format the data as a character vector (using color for emphasis) and simply call `new_pillar_shaft()`.
#' See `pillar:::pillar_shaft.numeric` for a code that allows changing the display depending on the available width.
#'
#' @param x An object
#' @param ... Additional attributes.
#' @param width The maximum column width.
#' @param min_width The minimum allowed column width, `width` if omitted.
#' @param type_sum `r lifecycle::badge("experimental")`
#'
#'   Override the type summary displayed at the top of the data.
#'   This argument, if given, takes precedence over the type summary provided by
#'   [type_sum()].
#' @param class The name of the subclass.
#' @param subclass Deprecated, pass the `class` argument instead.
#' @name new_pillar_shaft
#' @export
new_pillar_shaft <- function(x, ..., width = NULL, min_width = width,
                             type_sum = NULL, class = NULL, subclass = NULL) {
  if (!is.null(subclass)) {
    deprecate_stop("1.4.0", "pillar::new_pillar_shaft(subclass = )", "new_pillar_shaft(class = )")
    class <- subclass
  }

  stopifnot(is.character(class))
  stopifnot(length(class) > 0)
  stopifnot(is_bare_numeric(width, 1))

  ret <- structure(
    x,
    ...,
    type = if (!is.null(type_sum)) new_pillar_type_obj(type_sum),
    class = c(class, "pillar_shaft")
  )
  ret <- set_width(ret, width)
  ret <- set_min_width(ret, min_width)
  ret
}

#' Column data
#'
#' Internal class for formatting the data for a column.
#' `pillar_shaft()` is a coercion method that must be implemented
#' for your data type to display it in a tibble.
#'
#' @param x A vector to format
#' @inheritParams rlang::args_dots_used
#' @export
#' @examples
#' pillar_shaft(1:3)
#' pillar_shaft(1.5:3.5)
#' pillar_shaft(NA)
#' pillar_shaft(c(1:3, NA))
pillar_shaft <- function(x, ...) {
  "!!!!DEBUG pillar_shaft(`v(class(x))`)"

  if (!missing(...)) {
    check_dots_used(action = warn)
  }

  UseMethod("pillar_shaft")
}

#' @export
pillar_shaft.pillar_empty_col <- function(x, ...) {
  new_empty_shaft()
}

#' @param width Width for printing and formatting.
#' @export
#' @rdname pillar_shaft
print.pillar_shaft <- function(x, width = NULL, ...) {
  #' @description
  #' This class comes with a default method for [print()] that calls [format()].
  #' If `print()` is called without `width` argument, the natural width will be
  #' used when calling `format()`.
  #' Usually there's no need to implement this method for your subclass.
  if (is.null(width)) width <- get_width(x)
  print(format(x, width = width, ...))
}

#' @export
#' @rdname pillar_shaft
format.pillar_shaft <- function(x, width, ...) {
  #' @description
  #' Your subclass must implement `format()`, the default implementation just
  #' raises an error.
  #' Your `format()` method can assume a valid value for the `width` argument.
  stop("Please implement a format() method for class ", class(x)[[1]], call. = FALSE)
}

# Methods -----------------------------------------------------------------

#' @export
#' @rdname pillar_shaft
pillar_shaft.logical <- function(x, ...) {
  out <- rep(NA, length(x))
  out[x] <- "TRUE"
  out[!x] <- "FALSE"

  new_pillar_shaft_simple(out, width = 5, align = "left")
}

#' @export
#' @rdname pillar_shaft
#' @param sigfig
#'   Deprecated, use [num()] or [set_num_opts()] on the data instead.
pillar_shaft.numeric <- function(x, ..., sigfig = NULL) {
  pillar_attr <- attr(x, "pillar", exact = TRUE)

  if (is.null(pillar_attr) && !is.null(attr(x, "class", exact = TRUE))) {
    ret <- format(x)
    return(new_pillar_shaft_simple(ret, width = get_max_extent(ret), align = "left"))
  }

  pillar_shaft_number_attr(unclass(x), pillar_attr, sigfig)
}

pillar_shaft_number_attr <- function(data, pillar_attr, sigfig = NULL) {
  scale <- pillar_attr$scale
  if (!is.null(scale)) {
    data <- data * scale
  }

  pillar_shaft_number(
    data,
    sigfig %||% pillar_attr$sigfig,
    pillar_attr$digits,
    pillar_attr$notation,
    pillar_attr$fixed_exponent,
    pillar_attr$extra_sigfig
  )
}

pillar_shaft_number <- function(x, sigfig, digits, notation, fixed_exponent, extra_sigfig) {
  if (!is.null(digits)) {
    if (!is.numeric(digits) || length(digits) != 1) {
      abort("`digits` must be a number.")
    }
  }
  if (is.null(sigfig)) {
    sigfig <- get_pillar_option_sigfig()
  }

  if (isTRUE(extra_sigfig)) {
    sigfig <- sigfig + compute_extra_sigfig(x)
  }

  if (is.null(notation) || notation == "fit") {
    dec <- split_decimal(x, sigfig = sigfig, digits = digits)
    sci <- split_decimal(x, sigfig = sigfig, digits = digits, sci_mod = 1, fixed_exponent = fixed_exponent)

    max_dec_width <- get_pillar_option_max_dec_width()
    dec_width <- get_width(dec)
    "!!!!!!DEBUG `v(dec_width)`"

    if (dec_width > max_dec_width) {
      dec <- NULL
    }
  } else if (notation == "dec") {
    dec <- split_decimal(x, sigfig = sigfig, digits = digits)
    sci <- NULL
  } else if (notation == "sci") {
    sci <- split_decimal(
      x,
      sigfig = sigfig, digits = digits,
      sci_mod = 1,
      fixed_exponent = fixed_exponent
    )
    dec <- NULL
  } else if (notation == "eng") {
    sci <- split_decimal(
      x,
      sigfig = sigfig, digits = digits, sci_mod = 3,
      fixed_exponent = fixed_exponent
    )
    dec <- NULL
  } else if (notation == "si") {
    sci <- split_decimal(
      x,
      sigfig = sigfig, digits = digits, sci_mod = 3, si = TRUE,
      fixed_exponent = fixed_exponent
    )
    dec <- NULL
  } else {
    abort(paste0('Internal error: `notation = "', notation, '".'))
  }

  ret <- list()
  ret$dec <- dec
  ret$sci <- sci

  if (!is.null(ret$sci$unit) && ret$sci$unit != 0) {
    type_sum <- I(paste0(
      # Turn off subtle styling for units
      sub(" ", "", style_subtle(" ")),
      "[",
      format_exp(ret$sci$unit, (notation == "si")),
      "]"
    ))
  } else {
    type_sum <- NULL
  }

  new_pillar_shaft(
    ret,
    width = get_width(ret$dec %||% ret$sci),
    min_width = min(get_min_widths(ret)),
    type_sum = type_sum,
    class = "pillar_shaft_decimal"
  )
}

#' @export
pillar_shaft.integer64 <- function(x, ..., sigfig = NULL) {
  if (class(x)[[1]] != "integer64") {
    return(NextMethod())
  }

  pillar_shaft_number(x, sigfig, digits = NULL, notation = NULL, fixed_exponent = NULL, extra_sigfig = NULL)
}

#' @export
pillar_shaft.Surv <- function(x, ...) {
  new_pillar_shaft_simple(format(x), align = "right")
}

#' @export
pillar_shaft.Surv2 <- function(x, ...) {
  new_pillar_shaft_simple(format(x), align = "right")
}

#' @export
type_sum.Surv <- function(x) {
  "Surv"
}

#' @export
type_sum.Surv2 <- function(x) {
  "Surv2"
}

#' @export
#' @rdname pillar_shaft
pillar_shaft.Date <- function(x, ...) {
  x <- format(x, format = "%Y-%m-%d")

  new_pillar_shaft_simple(x, width = 10, align = "left")
}

#' @export
#' @rdname pillar_shaft
pillar_shaft.POSIXt <- function(x, ...) {
  width <- 19L
  digits_secs <- getOption("digits.secs", 0L)
  if (digits_secs > 0) {
    width <- width + min(digits_secs, 6) + 1L
  }

  date <- format(x, format = "%Y-%m-%d")
  time <- format(x, format = "%H:%M:%OS")

  # R < 4.3 don't support is.finite() for POSIXlt
  time_styled <- paste0(" ", style_subtle(time))
  if (inherits(x, "POSIXct")) {
    time_styled[!is.finite(x)] <- ""
  }

  datetime <- paste0(date, time_styled)
  datetime[is.na(x)] <- NA

  new_pillar_shaft_simple(datetime, width = width, align = "left")
}


#' @export
#' @rdname pillar_shaft
#' @param min_width
#'   Deprecated, use [char()] or [set_char_opts()] on the data instead.
pillar_shaft.character <- function(x, ..., min_width = NULL) {
  pillar_attr <- attr(x, "pillar", exact = TRUE)

  min_chars <- min_width %||% pillar_attr$min_chars

  x <- utf8::utf8_encode(x)
  out <- x

  # Add subtle quotes if necessary
  needs_quotes <- which(is_ambiguous_string(x))
  if (has_length(needs_quotes)) {
    quoted <- gsub('"', '\\"', x[needs_quotes], fixed = TRUE)
    out[needs_quotes] <- quoted
    valid <- paste0(style_subtle('"'), out[!is.na(x)], style_subtle('"'))
    out[!is.na(x)] <- valid
    na_indent <- 1
  } else {
    na_indent <- 0
  }

  # determine width based on width of characters in the vector
  if (is.null(min_chars)) {
    min_chars <- get_pillar_option_min_chars()
  }

  if (isS4(out)) {
    # Remove S4 bit:
    out <- unclass(out)[seq_along(out)]
  }

  pillar_shaft(as_glue(out), ..., min_width = min_chars, na_indent = na_indent, shorten = pillar_attr$shorten)
}

#' @export
#' @inheritParams new_pillar_shaft_simple
#' @rdname pillar_shaft
pillar_shaft.glue <- function(x, ..., min_width = NULL, na_indent = 0L, shorten = NULL) {
  min_width <- max(min_width, 3L)
  width <- get_max_extent(x)
  na <- pillar_na(use_brackets_if_no_color = TRUE)
  force(na_indent)

  new_pillar_shaft_simple(
    x,
    width = width, align = "left", min_width = min(width, min_width),
    na = na,
    na_indent = if (width > get_extent(na)) na_indent else 0,
    shorten = shorten
  )
}

#' @export
#' @rdname pillar_shaft
pillar_shaft.list <- function(x, ...) {
  summary <- map(x, obj_sum)

  formatted <- paste0("<", unlist(summary), ">")

  short <- map(summary, attr, "short")
  short_idx <- !map_lgl(short, is.null)
  short_formatted <- formatted
  short_formatted[short_idx] <- paste0("<", unlist(short[short_idx]), ">")

  width <- get_max_extent(formatted)
  min_width <- get_max_extent(short_formatted)

  new_pillar_shaft_simple(
    style_list(formatted),
    width = width,
    align = "left",
    min_width = min_width,
    short_formatted = style_list(short_formatted)
  )
}

#' @export
#' @rdname pillar_shaft
pillar_shaft.factor <- function(x, ...) {
  pillar_shaft(as.character(x), ...)
}

#' @export
#' @rdname pillar_shaft
pillar_shaft.AsIs <- function(x, ...) {
  pillar_shaft(remove_as_is_class(x))
}

#' @export
#' @rdname pillar_shaft
pillar_shaft.default <- function(x, ...) {
  #' @details
  #' The default method will currently format via [format()],
  #' but you should not rely on this behavior.
  pillar_shaft(as_glue(format(x)), ...)
}

#' Deprecated functions
#' @description
#' `r lifecycle::badge("deprecated")`
#' @keywords internal
#' @name deprecated
NULL

#' is_vector_s3()
#'
#' Use [vctrs::vec_is()] instead of `is_vector_s3()`.
#'
#' @export
#' @rdname deprecated
is_vector_s3 <- function(x) {
  deprecate_stop("1.7.0", "pillar::is_vector_s3()", "vctrs::vec_is()")
}

#' Set up formatting
#'
#' @description
#' `tbl_format_setup()` is called by [format.tbl()].
#' This method collects information that is common to the header, body,
#' and footer parts of a tibble.
#' Examples:
#'
#' - the dimensions sometimes are reported both in the header
#' and (implicitly) in the footer of a tibble;
#' - the columns shown in the body decide which columns are shown in the footer.
#'
#' This information is computed once in `tbl_format_setup()`.
#' The result is passed on to the
#' [tbl_format_header()], [tbl_format_body()], and [tbl_format_footer()]
#' methods.
#' If you need to customize parts of the printed output independently,
#' override these methods instead.
#'
#' @details
#' Extend this method to prepare information that is used
#' in several parts of the printed output of a tibble-like object,
#' or to collect additional arguments passed via `...` to
#' [print.tbl()] or [format.tbl()].
#'
#' We expect that `tbl_format_setup()` is extended only rarely,
#' and overridden only in exceptional circumstances, if at all.
#' If you override this method, you must also implement
#' [tbl_format_header()], [tbl_format_body()], and [tbl_format_footer()]
#' for your class.
#'
#' Implementing a method
#' allows to override printing and formatting of the entire object
#' without overriding the [print()] and [format()] methods directly.
#' This allows to keep the logic of the `width` and `n` arguments.
#'
#' @param x
#'   An object.
#' @param width
#'   Actual width for printing, a numeric greater than zero.
#'   This argument is mandatory for all implementations of this method.
#' @param ...
#'   Extra arguments to [print.tbl()] or [format.tbl()].
#' @param n
#'   Actual number of rows to print.
#'   No [options][pillar_options] should be considered
#'   by implementations of this method.
#' @param max_extra_cols
#'   Number of columns to print abbreviated information for,
#'   if the width is too small for the entire tibble.
#'   No [options][pillar_options] should be considered
#'   by implementations of this method.
#' @param max_footer_lines
#'   Maximum number of lines for the footer.
#'   No [options][pillar_options] should be considered
#'   by implementations of this method.
#' @param focus `r lifecycle::badge("experimental")`
#'
#'   Names of columns to show preferentially if space is tight.
#' @return
#'   An object that can be passed as `setup` argument to
#'   [tbl_format_header()], [tbl_format_body()], and [tbl_format_footer()].
#' @export
#' @examplesIf rlang::is_installed("palmerpenguins")
#' tbl_format_setup(palmerpenguins::penguins)
tbl_format_setup <- function(x, width = NULL, ...,
                             n = NULL, max_extra_cols = NULL,
                             max_footer_lines = NULL,
                             focus = NULL) {
  "!!!!DEBUG tbl_format_setup()"

  width <- get_width_print(width)

  n <- get_n_print(n, nrow(x))

  max_extra_cols <- get_max_extra_cols(max_extra_cols)
  max_footer_lines <- get_max_footer_lines(max_footer_lines)

  # Calls UseMethod("tbl_format_setup"),
  # allows using default values in S3 dispatch
  out <- tbl_format_setup_dispatch(
    x, width, ...,
    n = n, max_extra_cols = max_extra_cols, max_footer_lines = max_footer_lines,
    focus = focus
  )
  return(out)
  UseMethod("tbl_format_setup")
}

tbl_format_setup_dispatch <- function(x, width, ..., n, max_extra_cols, max_footer_lines, focus = NULL) {
  UseMethod("tbl_format_setup")
}

#' @details
#' The default method for the `"tbl"` class collects information for
#' standard printing for tibbles.
#' See [new_tbl_format_setup()] for details on the returned object.
#'
#' @rdname tbl_format_setup
#' @export
tbl_format_setup.tbl <- function(x, width, ...,
                                 n, max_extra_cols, max_footer_lines, focus) {
  "!!!!DEBUG tbl_format_setup.tbl()"

  # Number of rows
  rows <- nrow(x)

  if (is.na(rows)) {
    df <- df_head(x, n + 1)
    if (nrow(df) <= n) {
      rows <- nrow(df)
    } else {
      df <- vec_head(df, n)
    }
  } else {
    df <- df_head(x, n)
  }

  if (is.na(rows)) {
    # Lazy table with too many rows
    needs_dots <- (nrow(df) >= n)
  } else {
    # Lazy table with few rows, or regular data frame
    needs_dots <- (rows > n)
  }

  if (needs_dots) {
    rows_missing <- rows - n
  } else {
    rows_missing <- 0L
  }

  # Header
  tbl_sum <- tbl_sum(x)

  # Body
  rownames(df) <- NULL

  colonnade <- ctl_colonnade(
    df,
    has_row_id = if (.row_names_info(x) > 0) "*" else TRUE,
    width = width,
    controller = x,
    focus = focus
  )

  body <- colonnade$body

  # Extra columns
  extra_cols <- colonnade$extra_cols
  extra_cols_total <- length(extra_cols)

  if (extra_cols_total > max_extra_cols) {
    length(extra_cols) <- max_extra_cols
  }

  # Abbreviated columns
  abbrev_cols <- colonnade$abbrev_cols

  # Result
  new_tbl_format_setup(
    x = x,
    df = df,
    width = width,
    tbl_sum = tbl_sum,
    body = body,
    rows_missing = rows_missing,
    rows_total = rows,
    extra_cols = extra_cols,
    extra_cols_total = extra_cols_total,
    max_footer_lines = max_footer_lines,
    abbrev_cols = abbrev_cols
  )
}

#' Construct a setup object for formatting
#'
#' @description
#' The object returned from the default method of [tbl_format_setup()]
#' is an object with a `"class"` attribute and the elements described in the
#' "Parameters" section.
#'
#' Named elements can be added to such objects without affecting the behavior.
#' Do not modify existing elements.
#'
#' @param x The input object unchanged.
#' @param df A data frame representation of the intended output,
#'   trimmed to the desired number of rows.
#' @param width The `width` argument unchanged.
#' @param tbl_sum A named character vector, as returned from [tbl_sum()].
#' @param body A character vector with the formatted body,
#'   one element per line,
#' @param rows_missing The number of rows not shown from the body,
#'   `NA` if unknown.
#' @param rows_total The total number of rows in the data,
#'   `NA` if unknown.
#' @param extra_cols Columns that did not fit into the body,
#'   as a character vector of formatted column names and types.
#' @param extra_cols_total The total number of columns, may be larger than
#'   `length(extra_cols)`.
#' @param max_footer_lines The maximum number of lines in the footer.
#' @param abbrev_cols Formatted names of the columns that are shown abbreviated
#'   in the body.
#'
#' @keywords internal
new_tbl_format_setup <- function(x, df, width, tbl_sum, body,
                                 rows_missing, rows_total,
                                 extra_cols, extra_cols_total,
                                 max_footer_lines,
                                 abbrev_cols) {
  trunc_info <- list(
    x = x,
    df = df,
    width = width,
    tbl_sum = tbl_sum,
    body = body,
    rows_missing = rows_missing,
    rows_total = rows_total,
    extra_cols = extra_cols,
    extra_cols_total = extra_cols_total,
    max_footer_lines = max_footer_lines,
    abbrev_cols = abbrev_cols
  )

  structure(trunc_info, class = "pillar_tbl_format_setup")
}

is_tbl_format_setup <- function(x) {
  inherits(x, "pillar_tbl_format_setup")
}

# For testing
#' @export
print.pillar_tbl_format_setup <- function(x, ...) {
  writeLines(format(x, ...))
  invisible(x)
}

#' @export
format.pillar_tbl_format_setup <- function(x, ...) {
  check_dots_empty()
  c(
    cli::style_bold("<pillar_tbl_format_setup>"),
    tbl_format_header(x),
    tbl_format_body(x),
    tbl_format_footer(x)
  )
}

strrep <- NULL
strrep_compat <- function(x, times) {
  map_chr(
    times,
    function(n) {
      if (is.na(n)) {
        NA_character_
      } else {
        paste(rep(x, n), collapse = "")
      }
    }
  )
}

#' Calculate display width
#'
#' `get_extent()` calculates the display width for each string in a character
#' vector.
#'
#' @param x A character vector.
#' @export
#' @importFrom cli ansi_strip
#' @importFrom cli utf8_nchar
#' @examples
#' get_extent(c("abc", "de"))
#' get_extent("\u904b\u6c23")
get_extent <- function(x) {
  force(x)
  x <- ansi_strip(x)
  width <- utf8_nchar(x, "width")
  if (anyNA(width)) {
    is_na <- which(is.na(width))
    width[is_na] <- nchar(x[is_na], type = "width")
  }
  width
}

#' @description
#' `get_max_extent()` calculates the maximum display width of all strings in a
#' character vector, zero for empty vectors.
#' @export
#' @rdname get_extent
#' @examples
#' get_max_extent(c("abc", "de"))
get_max_extent <- function(x) {
  max(get_extent(x), 0L, na.rm = TRUE)
}

#' Alignment helper
#'
#' Facilitates easy alignment of strings within a character vector. Designed to
#' help implementers of formatters for custom data types.
#'
#' @param x A character vector
#' @param width The width that each string is padded to. If `NULL`, the maximum
#'   display width of the character vector is used (see [get_max_extent()]).
#' @param align How should strings be aligned? If `align = left` then padding
#'   appears on the `right`, and vice versa.
#' @param space What character should be used for the padding?
#' @export
#' @examples
#' align(c("abc", "de"), align = "left")
#' align(c("abc", "de"), align = "right")
align <- function(x, width = NULL, align = c("left", "right"), space = " ") {
  "!!!!!DEBUG align(`v(x)`, `v(class(x))`)"

  align <- match.arg(align)
  extent <- get_extent(x)
  if (is.null(width)) {
    width <- max(extent)
  }
  align_impl(x, width, align, space, extent)
}

align_impl <- function(x, width, align, space, extent) {
  spaces <- pmax(width - extent, 0L)
  if (align == "left") {
    paste0(x, strrep(space, spaces))
  } else {
    paste0(strrep(space, spaces), x)
  }
}

new_tbl <- function(x = list(), n = NULL, ..., class = NULL) {
  new_data_frame(x, n, class = c(class, "tbl"), ...)
}

as_tbl <- function(x, ...) {
  check_dots_empty()
  UseMethod("as_tbl")
}

#' @export
as_tbl.data.frame <- function(x, ...) {
  class(x) <- c("tbl", "data.frame")
  x
}

test_that("print_max", {
  value <- 15L

  orig <- get_pillar_option_print_max()
  expect_identical(orig, pillar_options$print_max())

  old <- set_pillar_option_print_max(value)
  expect_equal(get_pillar_option_print_max(), value)
  expect_equal(expect_invisible(set_pillar_option_print_max(old)), value)

  old <- pillar_options$print_max(value)
  expect_equal(pillar_options$print_max(), value)
  expect_equal(expect_invisible(pillar_options$print_max(old)), value)

  local({
    expect_equal(expect_invisible(local_pillar_option_print_max(value)), old)
    expect_equal(get_pillar_option_print_max(), value)
  })
  expect_equal(get_pillar_option_print_max(), orig)
})

test_that("print_min", {
  value <- 5L

  orig <- get_pillar_option_print_min()
  expect_identical(orig, pillar_options$print_min())

  old <- set_pillar_option_print_min(value)
  expect_equal(get_pillar_option_print_min(), value)
  expect_equal(expect_invisible(set_pillar_option_print_min(old)), value)

  old <- pillar_options$print_min(value)
  expect_equal(pillar_options$print_min(), value)
  expect_equal(expect_invisible(pillar_options$print_min(old)), value)

  local({
    expect_equal(expect_invisible(local_pillar_option_print_min(value)), old)
    expect_equal(get_pillar_option_print_min(), value)
  })
  expect_equal(get_pillar_option_print_min(), orig)
})

test_that("width", {
  value <- 160L

  orig <- get_pillar_option_width()
  expect_identical(orig, pillar_options$width())

  old <- set_pillar_option_width(value)
  expect_equal(get_pillar_option_width(), value)
  expect_equal(expect_invisible(set_pillar_option_width(old)), value)

  old <- pillar_options$width(value)
  expect_equal(pillar_options$width(), value)
  expect_equal(expect_invisible(pillar_options$width(old)), value)

  local({
    expect_equal(expect_invisible(local_pillar_option_width(value)), old)
    expect_equal(get_pillar_option_width(), value)
  })
  expect_equal(get_pillar_option_width(), orig)
})

test_that("max_footer_lines", {
  value <- 15L

  orig <- get_pillar_option_max_footer_lines()
  expect_identical(orig, pillar_options$max_footer_lines())

  old <- set_pillar_option_max_footer_lines(value)
  expect_equal(get_pillar_option_max_footer_lines(), value)
  expect_equal(expect_invisible(set_pillar_option_max_footer_lines(old)), value)

  old <- pillar_options$max_footer_lines(value)
  expect_equal(pillar_options$max_footer_lines(), value)
  expect_equal(expect_invisible(pillar_options$max_footer_lines(old)), value)

  local({
    expect_equal(expect_invisible(local_pillar_option_max_footer_lines(value)), old)
    expect_equal(get_pillar_option_max_footer_lines(), value)
  })
  expect_equal(get_pillar_option_max_footer_lines(), orig)
})

test_that("max_extra_cols", {
  value <- 30L

  orig <- get_pillar_option_max_extra_cols()
  expect_identical(orig, pillar_options$max_extra_cols())

  old <- set_pillar_option_max_extra_cols(value)
  expect_equal(get_pillar_option_max_extra_cols(), value)
  expect_equal(expect_invisible(set_pillar_option_max_extra_cols(old)), value)

  old <- pillar_options$max_extra_cols(value)
  expect_equal(pillar_options$max_extra_cols(), value)
  expect_equal(expect_invisible(pillar_options$max_extra_cols(old)), value)

  local({
    expect_equal(expect_invisible(local_pillar_option_max_extra_cols(value)), old)
    expect_equal(get_pillar_option_max_extra_cols(), value)
  })
  expect_equal(get_pillar_option_max_extra_cols(), orig)
})

test_that("bold", {
  value <- 0L

  orig <- get_pillar_option_bold()
  expect_identical(orig, pillar_options$bold())

  old <- set_pillar_option_bold(value)
  expect_equal(get_pillar_option_bold(), value)
  expect_equal(expect_invisible(set_pillar_option_bold(old)), value)

  old <- pillar_options$bold(value)
  expect_equal(pillar_options$bold(), value)
  expect_equal(expect_invisible(pillar_options$bold(old)), value)

  local({
    expect_equal(expect_invisible(local_pillar_option_bold(value)), old)
    expect_equal(get_pillar_option_bold(), value)
  })
  expect_equal(get_pillar_option_bold(), orig)
})

test_that("subtle", {
  value <- 0L

  orig <- get_pillar_option_subtle()
  expect_identical(orig, pillar_options$subtle())

  old <- set_pillar_option_subtle(value)
  expect_equal(get_pillar_option_subtle(), value)
  expect_equal(expect_invisible(set_pillar_option_subtle(old)), value)

  old <- pillar_options$subtle(value)
  expect_equal(pillar_options$subtle(), value)
  expect_equal(expect_invisible(pillar_options$subtle(old)), value)

  local({
    expect_equal(expect_invisible(local_pillar_option_subtle(value)), old)
    expect_equal(get_pillar_option_subtle(), value)
  })
  expect_equal(get_pillar_option_subtle(), orig)
})

test_that("subtle_num", {
  value <- 0L

  orig <- get_pillar_option_subtle_num()
  expect_identical(orig, pillar_options$subtle_num())

  old <- set_pillar_option_subtle_num(value)
  expect_equal(get_pillar_option_subtle_num(), value)
  expect_equal(expect_invisible(set_pillar_option_subtle_num(old)), value)

  old <- pillar_options$subtle_num(value)
  expect_equal(pillar_options$subtle_num(), value)
  expect_equal(expect_invisible(pillar_options$subtle_num(old)), value)

  local({
    expect_equal(expect_invisible(local_pillar_option_subtle_num(value)), old)
    expect_equal(get_pillar_option_subtle_num(), value)
  })
  expect_equal(get_pillar_option_subtle_num(), orig)
})

test_that("neg", {
  value <- 0L

  orig <- get_pillar_option_neg()
  expect_identical(orig, pillar_options$neg())

  old <- set_pillar_option_neg(value)
  expect_equal(get_pillar_option_neg(), value)
  expect_equal(expect_invisible(set_pillar_option_neg(old)), value)

  old <- pillar_options$neg(value)
  expect_equal(pillar_options$neg(), value)
  expect_equal(expect_invisible(pillar_options$neg(old)), value)

  local({
    expect_equal(expect_invisible(local_pillar_option_neg(value)), old)
    expect_equal(get_pillar_option_neg(), value)
  })
  expect_equal(get_pillar_option_neg(), orig)
})

test_that("sigfig", {
  value <- 5L

  orig <- get_pillar_option_sigfig()
  expect_identical(orig, pillar_options$sigfig())

  old <- set_pillar_option_sigfig(value)
  expect_equal(get_pillar_option_sigfig(), value)
  expect_equal(expect_invisible(set_pillar_option_sigfig(old)), value)

  old <- pillar_options$sigfig(value)
  expect_equal(pillar_options$sigfig(), value)
  expect_equal(expect_invisible(pillar_options$sigfig(old)), value)

  local({
    expect_equal(expect_invisible(local_pillar_option_sigfig(value)), old)
    expect_equal(get_pillar_option_sigfig(), value)
  })
  expect_equal(get_pillar_option_sigfig(), orig)
})

test_that("min_title_chars", {
  value <- 12L

  orig <- get_pillar_option_min_title_chars()
  expect_identical(orig, pillar_options$min_title_chars())

  old <- set_pillar_option_min_title_chars(value)
  expect_equal(get_pillar_option_min_title_chars(), value)
  expect_equal(expect_invisible(set_pillar_option_min_title_chars(old)), value)

  old <- pillar_options$min_title_chars(value)
  expect_equal(pillar_options$min_title_chars(), value)
  expect_equal(expect_invisible(pillar_options$min_title_chars(old)), value)

  local({
    expect_equal(expect_invisible(local_pillar_option_min_title_chars(value)), old)
    expect_equal(get_pillar_option_min_title_chars(), value)
  })
  expect_equal(get_pillar_option_min_title_chars(), orig)
})

test_that("min_chars", {
  value <- 5L

  orig <- get_pillar_option_min_chars()
  expect_identical(orig, pillar_options$min_chars())

  old <- set_pillar_option_min_chars(value)
  expect_equal(get_pillar_option_min_chars(), value)
  expect_equal(expect_invisible(set_pillar_option_min_chars(old)), value)

  old <- pillar_options$min_chars(value)
  expect_equal(pillar_options$min_chars(), value)
  expect_equal(expect_invisible(pillar_options$min_chars(old)), value)

  local({
    expect_equal(expect_invisible(local_pillar_option_min_chars(value)), old)
    expect_equal(get_pillar_option_min_chars(), value)
  })
  expect_equal(get_pillar_option_min_chars(), orig)
})

test_that("max_dec_width", {
  value <- 0L

  orig <- get_pillar_option_max_dec_width()
  expect_identical(orig, pillar_options$max_dec_width())

  old <- set_pillar_option_max_dec_width(value)
  expect_equal(get_pillar_option_max_dec_width(), value)
  expect_equal(expect_invisible(set_pillar_option_max_dec_width(old)), value)

  old <- pillar_options$max_dec_width(value)
  expect_equal(pillar_options$max_dec_width(), value)
  expect_equal(expect_invisible(pillar_options$max_dec_width(old)), value)

  local({
    expect_equal(expect_invisible(local_pillar_option_max_dec_width(value)), old)
    expect_equal(get_pillar_option_max_dec_width(), value)
  })
  expect_equal(get_pillar_option_max_dec_width(), orig)
})

test_that("bidi", {
  value <- TRUE

  orig <- get_pillar_option_bidi()
  expect_identical(orig, pillar_options$bidi())

  old <- set_pillar_option_bidi(value)
  expect_equal(get_pillar_option_bidi(), value)
  expect_equal(expect_invisible(set_pillar_option_bidi(old)), value)

  old <- pillar_options$bidi(value)
  expect_equal(pillar_options$bidi(), value)
  expect_equal(expect_invisible(pillar_options$bidi(old)), value)

  local({
    expect_equal(expect_invisible(local_pillar_option_bidi(value)), old)
    expect_equal(get_pillar_option_bidi(), value)
  })
  expect_equal(get_pillar_option_bidi(), orig)
})

test_that("print.tbl ignores max.print option", {
  trees2 <- as_tbl(trees)
  expect_output(
    withr::with_options(list(max.print = 3), print(trees2)),
    capture_output(print(trees2)),
    fixed = TRUE
  )
})

test_that("print.tbl uses tibble.width option", {
  mtcars2 <- as_tbl(mtcars)
  expect_output(
    withr::with_options(list(tibble.width = 40, dplyr.width = 50, width = 60), print(mtcars2)),
    capture_output(print(mtcars2, width = 40)),
    fixed = TRUE
  )
  expect_output(
    withr::with_options(list(dplyr.width = 50, width = 60), print(mtcars2)),
    capture_output(print(mtcars2, width = 50)),
    fixed = TRUE
  )
  expect_output(
    withr::with_options(list(width = 60), print(mtcars2)),
    capture_output(print(mtcars2, width = 60)),
    fixed = TRUE
  )
})

test_that("output test", {
  x <- new_tbl(list(
    column_zero_zero = 1:3 + 0.23,
    `col 01` = new_tbl(list(
      `col 02` = letters[1:3],
      `col 03` = LETTERS[1:3],
      `col 04` = matrix(1:9, nrow = 3, dimnames = list(letters[1:3], LETTERS[1:3]))
    )),
    `col 05` = ordered(letters[1:3])
  ))

  expect_snapshot({
    options(width = 100)
    tbl_format_setup(x, width = 4)
    tbl_format_setup(x, width = 5)
    tbl_format_setup(x, width = 6)
    tbl_format_setup(x, width = 7)
    tbl_format_setup(x, width = 8)
    tbl_format_setup(x, width = 9)
    tbl_format_setup(x, width = 10)
    tbl_format_setup(x, width = 11)
    tbl_format_setup(x, width = 12)
    tbl_format_setup(x, width = 13)
    tbl_format_setup(x, width = 14)
    tbl_format_setup(x, width = 15)
    tbl_format_setup(x, width = 16)
    tbl_format_setup(x, width = 17)
    tbl_format_setup(x, width = 18)
    tbl_format_setup(x, width = 19)
    tbl_format_setup(x, width = 20)
    tbl_format_setup(x, width = 21)
    tbl_format_setup(x, width = 22)
    tbl_format_setup(x, width = 23)
    tbl_format_setup(x, width = 24)
    tbl_format_setup(x, width = 25)
    tbl_format_setup(x, width = 26)
    tbl_format_setup(x, width = 27)
    tbl_format_setup(x, width = 28)
    tbl_format_setup(x, width = 29)
    tbl_format_setup(x, width = 30)
    tbl_format_setup(x, width = 31)
    tbl_format_setup(x, width = 32)
    tbl_format_setup(x, width = 33)
    tbl_format_setup(x, width = 34)
    tbl_format_setup(x, width = 35)
    tbl_format_setup(x, width = 36)
    tbl_format_setup(x, width = 37)
    tbl_format_setup(x, width = 38)
    tbl_format_setup(x, width = 39)
    tbl_format_setup(x, width = 40)
    tbl_format_setup(x, width = 41)
    tbl_format_setup(x, width = 42)
    tbl_format_setup(x, width = 43)
    tbl_format_setup(x, width = 44)
    tbl_format_setup(x, width = 45)
    tbl_format_setup(x, width = 46)
    tbl_format_setup(x, width = 47)
    tbl_format_setup(x, width = 48)
    tbl_format_setup(x, width = 49)
    tbl_format_setup(x, width = 50)
    tbl_format_setup(x, width = 51)
    tbl_format_setup(x, width = 52)
    tbl_format_setup(x, width = 53)
    tbl_format_setup(x, width = 54)
    tbl_format_setup(x, width = 55)
    tbl_format_setup(x, width = 56)
    tbl_format_setup(x, width = 57)
    tbl_format_setup(x, width = 58)
    tbl_format_setup(x, width = 59)
    tbl_format_setup(x, width = 60)
    tbl_format_setup(x, width = 61)
    tbl_format_setup(x, width = 62)
    tbl_format_setup(x, width = 63)
    tbl_format_setup(x, width = 64)
    tbl_format_setup(x, width = 65)
    tbl_format_setup(x, width = 66)
    tbl_format_setup(x, width = 67)
    tbl_format_setup(x, width = 68)
    tbl_format_setup(x, width = 69)
    tbl_format_setup(x, width = 70)
    tbl_format_setup(x, width = 71)
    tbl_format_setup(x, width = 72)
    tbl_format_setup(x, width = 73)
    tbl_format_setup(x, width = 74)
    tbl_format_setup(x, width = 75)
    tbl_format_setup(x, width = 76)
    tbl_format_setup(x, width = 77)
    tbl_format_setup(x, width = 78)
    tbl_format_setup(x, width = 79)
    tbl_format_setup(x, width = 80)
    tbl_format_setup(x, width = 81)
    tbl_format_setup(x, width = 82)
    tbl_format_setup(x, width = 83)
    tbl_format_setup(x, width = 84)
    tbl_format_setup(x, width = 85)
    tbl_format_setup(x, width = 86)
    tbl_format_setup(x, width = 87)
    tbl_format_setup(x, width = 88)
    tbl_format_setup(x, width = 89)
    tbl_format_setup(x, width = Inf)
  })
})

test_that("tbl_format_setup() results", {
  local_unknown_rows()

  expect_snapshot({
    tbl_format_setup(as_tbl(mtcars), width = 80)

    tbl_format_setup(as_tbl(mtcars), width = 30, n = 8)

    tbl_format_setup(as_tbl(trees), width = 30, n = 100)

    "Unknown rows"
    tbl_format_setup(as_unknown_rows(trees[1:9, ]), width = 30, n = 10)

    tbl_format_setup(as_unknown_rows(trees[1:10, ]), width = 30, n = 10)

    tbl_format_setup(as_unknown_rows(trees[1:11, ]), width = 30, n = 10)

    "No rows"
    tbl_format_setup(new_tbl(list(a = character(), b = logical())), width = 30)

    "No columns"
    tbl_format_setup(as_tbl(trees[character()]), width = 30, n = 5L)

    "No rows, unknown"
    tbl_format_setup(as_unknown_rows(trees[integer(), ]), width = 30, n = 5L)

    "No columns, unknown"
    tbl_format_setup(as_unknown_rows(trees[, character()]), width = 30, n = 5L)

    "Long table, unknown"
    tbl_format_setup(
      as_unknown_rows(new_tbl(list(a = seq.int(10000)))),
      n = 5L,
      width = 30L
    )
  })
})

test_that("tbl_format_setup() for footnotes", {
  expect_snapshot({
    tbl_format_setup(width = 73, as_tbl(data_frame(
      xxxabc = 1,
      xxxdef = 1,
      xxxghi = 1,
      xxxjkl = 1,
      xxxmno = 1,
      xxxpqr = 1,
      xxxstu = 1,
      xxxvwx = 1,
      xxxyza = 1,
      xxxbcd = 1,
      xxxefg = 1,
      xxxhij = 1,
      xxxklm = 1,
      xxxnop = 1
    )))
  })
})


test_that("tbl_format_setup() for footnotes with min_title_chars = 4", {
  local_pillar_option_min_title_chars(4)

  expect_snapshot({
    tbl_format_setup(width = 73, as_tbl(data_frame(
      xxxabc = 1,
      xxxdef = 1,
      xxxghi = 1,
      xxxjkl = 1,
      xxxmno = 1,
      xxxpqr = 1,
      xxxstu = 1,
      xxxvwx = 1,
      xxxyza = 1,
      xxxbcd = 1,
      xxxefg = 1,
      xxxhij = 1,
      xxxklm = 1,
      xxxnop = 1
    )))
  })
})

test_that("tbl_format_setup() for footnotes with UTF-8 output", {
  skip_if(!l10n_info()$`UTF-8`)

  local_utf8()

  expect_snapshot({
    tbl_format_setup(width = 73, as_tbl(data_frame(
      xxxabc = 1,
      xxxdef = 1,
      xxxghi = 1,
      xxxjkl = 1,
      xxxmno = 1,
      xxxpqr = 1,
      xxxstu = 1,
      xxxvwx = 1,
      xxxyza = 1,
      xxxbcd = 1,
      xxxefg = 1,
      xxxhij = 1,
      xxxklm = 1,
      xxxnop = 1
    )))
  })
})

#' Constructor for column data
#'
#' @description
#' The `new_pillar_shaft()` constructor creates objects of the `"pillar_shaft"`
#' class.
#' This is a virtual or abstract class, you must specify the `class`
#' argument.
#' By convention, this should be a string that starts with `"pillar_shaft_"`.
#' See `vignette("extending", package = "tibble")` for usage examples.
#'
#' This method accepts a vector of arbitrary length and is expected to return an S3 object with the following properties:
#'
#' - It has an attribute `"width"`
#' - It can have an attribute `"min_width"`, if missing, `"width"` is used
#' - It must implement a method `format(x, width, ...)` that can be called with any value between `min_width` and `width`
#' - This method must return an object that inherits from `character` and has attributes `"align"` (with supported values `"left"`, `"right"`, and `"center"`) and `"width"`
#'
#' The function [new_pillar_shaft()] returns such an object, and also correctly formats `NA` values.
#' In many cases, the implementation of `pillar_shaft.your_class_name()` will format the data as a character vector (using color for emphasis) and simply call `new_pillar_shaft()`.
#' See `pillar:::pillar_shaft.numeric` for a code that allows changing the display depending on the available width.
#'
#' @param x An object
#' @param ... Additional attributes.
#' @param width The maximum column width.
#' @param min_width The minimum allowed column width, `width` if omitted.
#' @param type_sum `r lifecycle::badge("experimental")`
#'
#'   Override the type summary displayed at the top of the data.
#'   This argument, if given, takes precedence over the type summary provided by
#'   [type_sum()].
#' @param class The name of the subclass.
#' @param subclass Deprecated, pass the `class` argument instead.
#' @name new_pillar_shaft
#' @export
new_pillar_shaft <- function(x, ..., width = NULL, min_width = width,
                             type_sum = NULL, class = NULL, subclass = NULL) {
  if (!is.null(subclass)) {
    deprecate_soft("1.4.0", "pillar::new_pillar_shaft(subclass = )", "new_pillar_shaft(class = )")
    class <- subclass
  }

  stopifnot(is.character(class))
  stopifnot(length(class) > 0)
  stopifnot(is_bare_numeric(width, 1))

  ret <- structure(
    x,
    ...,
    type = if (!is.null(type_sum)) new_pillar_type_obj(type_sum),
    class = c(class, "pillar_shaft")
  )
  ret <- set_width(ret, width)
  ret <- set_min_width(ret, min_width)
  ret
}

#' Column data
#'
#' Internal class for formatting the data for a column.
#' `pillar_shaft()` is a coercion method that must be implemented
#' for your data type to display it in a tibble.
#'
#' @param x A vector to format
#' @inheritParams rlang::args_dots_used
#' @export
#' @examples
#' pillar_shaft(1:3)
#' pillar_shaft(1.5:3.5)
#' pillar_shaft(NA)
#' pillar_shaft(c(1:3, NA))
pillar_shaft <- function(x, ...) {
  "!!!!DEBUG pillar_shaft(`v(class(x))`)"

  if (!missing(...)) {
    check_dots_used(action = warn)
  }

  UseMethod("pillar_shaft")
}

#' @export
pillar_shaft.pillar_empty_col <- function(x, ...) {
  new_empty_shaft()
}

#' @param width Width for printing and formatting.
#' @export
#' @rdname pillar_shaft
print.pillar_shaft <- function(x, width = NULL, ...) {
  #' @description
  #' This class comes with a default method for [print()] that calls [format()].
  #' If `print()` is called without `width` argument, the natural width will be
  #' used when calling `format()`.
  #' Usually there's no need to implement this method for your subclass.
  if (is.null(width)) width <- get_width(x)
  print(format(x, width = width, ...))
}

#' @export
#' @rdname pillar_shaft
format.pillar_shaft <- function(x, width, ...) {
  #' @description
  #' Your subclass must implement `format()`, the default implementation just
  #' raises an error.
  #' Your `format()` method can assume a valid value for the `width` argument.
  stop("Please implement a format() method for class ", class(x)[[1]], call. = FALSE)
}

# Methods -----------------------------------------------------------------

#' @export
#' @rdname pillar_shaft
pillar_shaft.logical <- function(x, ...) {
  out <- rep(NA, length(x))
  out[x] <- "TRUE"
  out[!x] <- "FALSE"

  new_pillar_shaft_simple(out, width = 5, align = "left")
}

#' @export
#' @rdname pillar_shaft
#' @param sigfig
#'   Deprecated, use [num()] or [set_num_opts()] on the data instead.
pillar_shaft.numeric <- function(x, ..., sigfig = NULL) {
  pillar_attr <- attr(x, "pillar", exact = TRUE)

  if (is.null(pillar_attr) && !is.null(attr(x, "class", exact = TRUE))) {
    ret <- format(x)
    return(new_pillar_shaft_simple(ret, width = get_max_extent(ret), align = "left"))
  }

  pillar_shaft_number_attr(unclass(x), pillar_attr, sigfig)
}

pillar_shaft_number_attr <- function(data, pillar_attr, sigfig = NULL) {
  scale <- pillar_attr$scale
  if (!is.null(scale)) {
    data <- data * scale
  }

  pillar_shaft_number(
    data,
    sigfig %||% pillar_attr$sigfig,
    pillar_attr$digits,
    pillar_attr$notation,
    pillar_attr$fixed_exponent,
    pillar_attr$extra_sigfig
  )
}

pillar_shaft_number <- function(x, sigfig, digits, notation, fixed_exponent, extra_sigfig) {
  if (!is.null(digits)) {
    if (!is.numeric(digits) || length(digits) != 1) {
      abort("`digits` must be a number.")
    }
  }
  if (is.null(sigfig)) {
    sigfig <- get_pillar_option_sigfig()
  }

  if (isTRUE(extra_sigfig)) {
    sigfig <- sigfig + compute_extra_sigfig(x)
  }

  if (is.null(notation) || notation == "fit") {
    dec <- split_decimal(x, sigfig = sigfig, digits = digits)
    sci <- split_decimal(x, sigfig = sigfig, digits = digits, sci_mod = 1, fixed_exponent = fixed_exponent)

    max_dec_width <- get_pillar_option_max_dec_width()
    dec_width <- get_width(dec)
    "!!!!!!DEBUG `v(dec_width)`"

    if (dec_width > max_dec_width) {
      dec <- NULL
    }
  } else if (notation == "dec") {
    dec <- split_decimal(x, sigfig = sigfig, digits = digits)
    sci <- NULL
  } else if (notation == "sci") {
    sci <- split_decimal(
      x,
      sigfig = sigfig, digits = digits,
      sci_mod = 1,
      fixed_exponent = fixed_exponent
    )
    dec <- NULL
  } else if (notation == "eng") {
    sci <- split_decimal(
      x,
      sigfig = sigfig, digits = digits, sci_mod = 3,
      fixed_exponent = fixed_exponent
    )
    dec <- NULL
  } else if (notation == "si") {
    sci <- split_decimal(
      x,
      sigfig = sigfig, digits = digits, sci_mod = 3, si = TRUE,
      fixed_exponent = fixed_exponent
    )
    dec <- NULL
  } else {
    abort(paste0('Internal error: `notation = "', notation, '".'))
  }

  ret <- list()
  ret$dec <- dec
  ret$sci <- sci

  if (!is.null(ret$sci$unit) && ret$sci$unit != 0) {
    type_sum <- I(paste0(
      # Turn off subtle styling for units
      sub(" ", "", style_subtle(" ")),
      "[",
      format_exp(ret$sci$unit, (notation == "si")),
      "]"
    ))
  } else {
    type_sum <- NULL
  }

  new_pillar_shaft(
    ret,
    width = get_width(ret$dec %||% ret$sci),
    min_width = min(get_min_widths(ret)),
    type_sum = type_sum,
    class = "pillar_shaft_decimal"
  )
}

#' @export
pillar_shaft.integer64 <- function(x, ..., sigfig = NULL) {
  if (class(x)[[1]] != "integer64") {
    return(NextMethod())
  }

  pillar_shaft_number(x, sigfig, digits = NULL, notation = NULL, fixed_exponent = NULL, extra_sigfig = NULL)
}

#' @export
pillar_shaft.Surv <- function(x, ...) {
  new_pillar_shaft_simple(format(x), align = "right")
}

#' @export
pillar_shaft.Surv2 <- function(x, ...) {
  new_pillar_shaft_simple(format(x), align = "right")
}

#' @export
type_sum.Surv <- function(x) {
  "Surv"
}

#' @export
type_sum.Surv2 <- function(x) {
  "Surv2"
}

#' @export
#' @rdname pillar_shaft
pillar_shaft.Date <- function(x, ...) {
  x <- format(x, format = "%Y-%m-%d")

  new_pillar_shaft_simple(x, width = 10, align = "left")
}

#' @export
#' @rdname pillar_shaft
pillar_shaft.POSIXt <- function(x, ...) {
  width <- 19L
  digits_secs <- getOption("digits.secs", 0L)
  if (digits_secs > 0) {
    width <- width + min(digits_secs, 6) + 1L
  }

  date <- format(x, format = "%Y-%m-%d")
  time <- format(x, format = "%H:%M:%OS")

  datetime <- paste0(date, " ", style_subtle(time))
  datetime[is.na(x)] <- NA

  new_pillar_shaft_simple(datetime, width = width, align = "left")
}


#' @export
#' @rdname pillar_shaft
#' @param min_width
#'   Deprecated, use [char()] or [set_char_opts()] on the data instead.
pillar_shaft.character <- function(x, ..., min_width = NULL) {
  pillar_attr <- attr(x, "pillar", exact = TRUE)

  min_chars <- min_width %||% pillar_attr$min_chars

  x <- utf8::utf8_encode(x)
  out <- x

  # Add subtle quotes if necessary
  needs_quotes <- which(is_ambiguous_string(x))
  if (has_length(needs_quotes)) {
    quoted <- gsub('"', '\\"', x[needs_quotes], fixed = TRUE)
    out[needs_quotes] <- quoted
    valid <- paste0(style_subtle('"'), out[!is.na(x)], style_subtle('"'))
    out[!is.na(x)] <- valid
    na_indent <- 1
  } else {
    na_indent <- 0
  }

  # determine width based on width of characters in the vector
  if (is.null(min_chars)) {
    min_chars <- get_pillar_option_min_chars()
  }

  pillar_shaft(as_glue(out), ..., min_width = min_chars, na_indent = na_indent, shorten = pillar_attr$shorten)
}

#' @export
#' @inheritParams new_pillar_shaft_simple
#' @rdname pillar_shaft
pillar_shaft.glue <- function(x, ..., min_width = NULL, na_indent = 0L, shorten = NULL) {
  min_width <- max(min_width, 3L)
  width <- get_max_extent(x)
  na <- pillar_na(use_brackets_if_no_color = TRUE)
  force(na_indent)

  new_pillar_shaft_simple(
    x,
    width = width, align = "left", min_width = min(width, min_width),
    na = na,
    na_indent = if (width > get_extent(na)) na_indent else 0,
    shorten = shorten
  )
}

#' @export
#' @rdname pillar_shaft
pillar_shaft.list <- function(x, ...) {
  summary <- map(x, obj_sum)

  formatted <- paste0("<", unlist(summary), ">")

  short <- map(summary, attr, "short")
  short_idx <- !map_lgl(short, is.null)
  short_formatted <- formatted
  short_formatted[short_idx] <- paste0("<", unlist(short[short_idx]), ">")

  width <- get_max_extent(formatted)
  min_width <- get_max_extent(short_formatted)

  new_pillar_shaft_simple(
    style_list(formatted),
    width = width,
    align = "left",
    min_width = min_width,
    short_formatted = short_formatted
  )
}

#' @export
#' @rdname pillar_shaft
pillar_shaft.factor <- function(x, ...) {
  pillar_shaft(as.character(x), ...)
}

#' @export
#' @rdname pillar_shaft
pillar_shaft.AsIs <- function(x, ...) {
  pillar_shaft(remove_as_is_class(x))
}

#' @export
#' @rdname pillar_shaft
pillar_shaft.default <- function(x, ...) {
  #' @details
  #' The default method will currently format via [format()],
  #' but you should not rely on this behavior.
  pillar_shaft(as_glue(format(x)), ...)
}

#' Formatting of tbl objects
#'
#' See [tibble::formatting] for details.
#'
#' @name format_tbl
#' @export
#' @keywords internal
print.tbl <- function(x, width = NULL, ..., n = NULL, max_extra_cols = NULL,
                      max_footer_lines = NULL) {
  print_tbl(
    x, width, ...,
    n = n, max_extra_cols = max_extra_cols, max_footer_lines = max_footer_lines
  )
}

print_tbl <- function(x, width = NULL, ...,
                      n_extra = NULL,
                      n = NULL, max_extra_cols = NULL, max_footer_lines = NULL) {
  if (!is.null(n_extra)) {
    deprecate_stop("1.6.2", "pillar::print(n_extra = )", "pillar::print(max_extra_cols = )")
  }

  # Printing happens as a side effect thanks to `transform = writeLines` .
  # For formatting, the default `transform = identity` returns the data instead.
  format_tbl(
    x,
    width = width, ...,
    n = n, max_extra_cols = max_extra_cols, max_footer_lines = max_footer_lines,
    transform = writeLines
  )

  invisible(x)
}

#' @export
#' @rdname format_tbl
format.tbl <- function(x, width = NULL, ...,
                       n = NULL, max_extra_cols = NULL, max_footer_lines = NULL) {
  format_tbl(
    x, width, ...,
    n = n, max_extra_cols = max_extra_cols, max_footer_lines = max_footer_lines
  )
}

format_tbl <- function(
  x,
  width = NULL,
  ...,
  n_extra = NULL,
  n = NULL,
  max_extra_cols = NULL,
  max_footer_lines = NULL,
  transform = identity
) {
  check_dots_empty(error = function(cnd) warn("`...` must be empty in `format.tbl()`", parent = cnd))

  if (!is.null(n_extra)) {
    deprecate_stop("1.6.2", "pillar::format(n_extra = )", "pillar::format(max_extra_cols = )")
  }

  # Reset local cache for each new output
  force(x)
  num_colors(forget = TRUE)

  # This is a bit of a hack to allow the setup function to be called twice
  # if the implementer is prepared to handle that.
  # We detect that by checking if the `setup` argument has been evaluated.
  setup_used <- FALSE

  # In either case, we expect a `setup` object that can be passed to `tbl_format_header()`
  # as a return from this call.
  setup <- tbl_format_setup(
    x,
    width = width,
    ...,
    setup = {
      # This construct updates the `setup_used` variable in the parent scope
      # when the `setup` argument is evaluated.
      setup_used <- TRUE
      NULL
    },
    n = n,
    max_extra_cols = max_extra_cols,
    max_footer_lines = max_footer_lines,
    focus = attr(x, "pillar_focus")
  )

  header <- transform(tbl_format_header(x, setup))

  # If the implementation did not request the `setup` argument in the first call,
  # the default behavior before 1.9.1 is used: the first call already
  # has returned the full setup object.
  # Otherwise, we assume that a second call is required, and we pass it the
  # setup object returned from the first call.
  if (setup_used) {
    setup <- tbl_format_setup(
      x,
      width = width,
      ...,
      setup = setup,
      n = n,
      max_extra_cols = max_extra_cols,
      max_footer_lines = max_footer_lines,
      focus = attr(x, "pillar_focus")
    )
  }

  # In either case, the `setup` object is now complete and can be used to format the body
  # and the footer.
  body <- transform(tbl_format_body(x, setup))
  footer <- transform(tbl_format_footer(x, setup))
  c(header, body, footer)
}

format_comment <- function(x, width) {
  if (length(x) == 0L) {
    return(character())
  }
  map_chr(x, wrap, prefix = "# ", width = min(width, cli::console_width()))
}

# wrap --------------------------------------------------------------------

NBSP <- "\U00A0"

wrap <- function(..., indent = 0, prefix = "", width) {
  x <- paste0(..., collapse = "")
  wrapped <- strwrap2(x, width - get_extent(prefix), indent)
  wrapped <- paste0(prefix, wrapped)
  wrapped <- gsub(NBSP, " ", wrapped)

  paste0(wrapped, collapse = "\n")
}

strwrap2 <- function(x, width, indent) {
  cli::ansi_strwrap(x, width = max(width, 0), indent = indent, exdent = indent + 2, simplify = FALSE)
}

#' Scale that supports formatted numbers
#'
#' @description
#'  `r lifecycle::badge("experimental")`
#'
#' This scale is used by default in ggplot2 with columns created with [num()].
#'
#' @inheritDotParams ggplot2::continuous_scale -scale_name
#' @param guide,position Passed on to [ggplot2::continuous_scale()]
#' @param rescaler,super Must remain `NULL`.
#'
#' @keywords internal
#' @export
scale_x_num <- function(..., position = "bottom", guide = ggplot2::waiver(),
                        rescaler = NULL, super = NULL) {
  stopifnot(is.null(rescaler))
  stopifnot(is.null(super))
  check_installed("ggplot2")
  ggplot2::continuous_scale(
    aesthetics = c(
      "x", "xmin", "xmax", "xend", "xintercept", "xmin_final", "xmax_final",
      "xlower", "xmiddle", "xupper"
    ),
    palette = "position_c",
    ...,
    guide = guide,
    position = position,
    rescaler = scales::rescale,
    super = MakeScaleContinuousPositionNum()
  )
}

#' @rdname scale_x_num
#' @export
scale_y_num <- function(..., guide = ggplot2::waiver(),
                        rescaler = NULL, super = NULL) {
  stopifnot(is.null(rescaler))
  stopifnot(is.null(super))
  check_installed("ggplot2")
  ggplot2::continuous_scale(
    aesthetics = c(
      "y", "ymin", "ymax", "yend", "yintercept", "ymin_final", "ymax_final",
      "lower", "middle", "upper"
    ),
    palette = "position_c",
    ...,
    guide = guide,
    rescaler = scales::rescale,
    super = MakeScaleContinuousPositionNum()
  )
}

MakeScaleContinuousPositionNum <- function() {
  ggplot2::ggproto("ScaleContinuousPositionNum", ggplot2::ScaleContinuousPosition,
    ptype = NULL,
    train = function(self, x) {
      self$ptype <- vec_ptype2(x, self$ptype)
      ggplot2::ggproto_parent(ggplot2::ScaleContinuousPosition, self)$train(x)
    },
    get_breaks = function(self, limits = self$get_limits()) {
      out <- ggplot2::ggproto_parent(ggplot2::ScaleContinuousPosition, self)$get_breaks(limits)
      vec_cast(out, self$ptype)
    },
    get_labels = function(self, breaks = self$get_breaks()) {
      out <- ggplot2::ggproto_parent(ggplot2::ScaleContinuousPosition, self)$get_labels(breaks)
      ansi_strip(out)
    },
    make_title = function(self, title) {
      out <- ggplot2::ggproto_parent(ggplot2::ScaleContinuousPosition, self)$make_title(title)
      pillar_attr <- attr(self$ptype, "pillar", exact = TRUE)
      if (!is.null(pillar_attr$fixed_exponent)) {
        shaft <- pillar_shaft_number_attr(numeric(), pillar_attr)
        if (pillar_attr$notation == "si") {
          type <- attr(shaft, "type")
          if (!is.null(type)) {
            out <- paste0(out, " ", ansi_strip(type[[1]]))
          }
        } else {
          # paste0() doesn't work here, paste() works like paste0()
          out <- quo_squash(quo(paste(!!out, " [", 10^!!shaft$sci$unit, "]")))
        }
      } else {
        label <- pillar_attr$label
        if (!is.null(label)) {
          out <- paste0(out, " [", label, "]")
        }
      }
      out
    }
  )
}

# registered in .onLoad()
scale_type.pillar_num <- function(x) c("num", "continuous")

# registered in .onLoad()
rescale.pillar_num <- function(x,
                               to = c(0, 1),
                               from = range(x, na.rm = TRUE, finite = TRUE),
                               ...) {
  out <- scales::rescale(vec_data(x), to, from, ...)
  vec_cast(out, x)
}

#' Components of a pillar
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' `new_pillar_component()` constructs an object of class `"pillar_component"`.
#' It is used by custom [ctl_new_pillar()] methods to create pillars with
#' nonstandard components.
#'
#' @details
#' Objects of class `"pillar"` are internally a named lists of their components.
#' The default components for pillars created by [pillar()] are:
#' `title` (may be missing), `type`, and `data`.
#' Each component is a `"pillar_component"` object.
#'
#' This class captures contents that can be fitted in a simple column.
#' Compound columns are represented by multiple pillar objects, each with their
#' own components.
#'
#' @inheritParams rlang::args_dots_empty
#' @param x A bare list of length one (for `new_pillar_component()`),
#'   or an object with `"width"` and `"min_width"` attributes
#'   (for `pillar_component()`).
#' @param width,min_width Width and minimum width for the new component.
#'   If `min_width` is `NULL`, it is assumed to match `width`.
#' @export
#' @examples
#' new_pillar_component(list(letters[1:3]), width = 1)
#' pillar_component(new_pillar_title("letters"))
#' pillar_component(new_pillar_type(letters))
#' pillar_component(pillar_shaft(letters[1:3]))
new_pillar_component <- function(x, ..., width, min_width = NULL) {
  "!!!!DEBUG new_pillar_component(`v(x)`, `v(width)`, `v(min_width)`)"

  check_dots_empty()
  stopifnot(rlang::is_bare_list(x))
  stopifnot(length(x) == 1)
  stopifnot(is_integerish(width), length(width) == 1L)
  if (!is.null(min_width)) {
    stopifnot(is_integerish(min_width), length(min_width) == 1L)
  }

  structure(
    x,
    width = as.integer(width),
    min_width = as.integer(min_width %||% width),
    class = "pillar_component"
  )
}

#' pillar_component()
#'
#' `pillar_component()` is a convenience helper that wraps the input in a list
#' and extracts width and minimum width.
#'
#' @export
#' @rdname new_pillar_component
pillar_component <- function(x) {
  # FIXME: No longer need to wrap in a list, cell concept abandoned
  new_pillar_component(list(x), width = get_width(x), min_width = get_min_width(x))
}

pillar_get_width <- function(x) {
  as.integer(max(map_int(x, get_width)))
}

pillar_get_min_width <- function(x) {
  as.integer(max(map_int(x, get_min_width)))
}

pillar_format_parts_2 <- function(x, width, is_focus = FALSE, footnote_idx = 1L) {
  formatted <- map(x, function(.x) format(
    .x[[1L]],
    width = min(width, get_width(.x)),
    is_focus = is_focus,
    footnote_idx = footnote_idx
  ))

  align <- attr(formatted[["data"]], "align", exact = TRUE) %||% "left"

  flat <- unlist(formatted)
  extent <- get_extent(flat)
  max_extent <- max(extent)
  if (max_extent > width) {
    flat <- str_trunc(flat, width)
    max_extent <- width
  }
  aligned <- align_impl(flat, max_extent, align, " ", extent)

  list(max_extent = max_extent, aligned = aligned, components = names(x))
}

#' @export
pillar_shaft.pillar_vctr <- function(x, ...) {
  # still seems necessary
  pillar_shaft(unclass(x))
}

#' @export
vec_ptype_full.pillar_vctr <- function(x, ...) {
  format(attr(x, "pillar", exact = TRUE))
}

#' @export
print.pillar_vctr <- function(x, ..., max = NULL) {
  if (is.null(max)) {
    max <- getOption("max.print")
  }

  xx <- vec_head(x, max)
  size <- vec_size(x)

  obj_print_header.pillar_vctr(xx, ..., .size = size)
  obj_print_data.pillar_vctr(xx, ..., .size = size)
  obj_print_footer.pillar_vctr(xx, ..., .size = size)
  invisible(x)
}

#' @export
obj_print_header.pillar_vctr <- function(x, ..., .size) {
  writeLines(paste0("<", vec_ptype_full(x), "[", .size, "]>"))
  pillar_attr <- attr(x, "pillar")
  if (!is.null(pillar_attr$fixed_exponent)) {
    shaft <- pillar_shaft_number_attr(numeric(), pillar_attr)
    type <- attr(shaft, "type")
    if (!is.null(type)) {
      writeLines(paste0("Fixed exponent: ", type[[1]]))
    }
  }
  invisible(x)
}

#' @export
obj_print_data.pillar_vctr <- function(x, ..., .size) {
  if (length(x) == 0) {
    return(invisible(x))
  }

  # FIXME: base::print.default() can't use color, roll own implementation?
  out <- stats::setNames(ansi_strip(format(x)), names(x))
  print(out, quote = FALSE, max = vec_size(x))
  invisible(x)
}

#' @export
obj_print_footer.pillar_vctr <- function(x, ..., .size) {
  delta <- .size - vec_size(x)

  if (delta <= 0) {
    return(invisible(x))
  }

  writeLines(style_subtle(pre_dots(paste0("and ", delta, " more"))))
  invisible(x)
}

#' @export
print.pillar_vctr_attr <- function(x, ...) {
  writeLines(format(x))
  invisible(x)
}

#' @export
vec_proxy_compare.pillar_vctr <- function(x, ...) {
  vec_data(x)
}
#' @export
vec_proxy_order.pillar_vctr <- function(x, ...) {
  vec_data(x)
}

#' Provide a succinct summary of an object
#'
#' `type_sum()` gives a brief summary of object type. Objects that commonly
#' occur in a data frame should return a string with four or less characters.
#' For most inputs, the argument is forwarded to [vctrs::vec_ptype_abbr()].
#'
#' When formatting a pillar,
#' `type_sum()` will be called on a slice of the column vector.
#' The formatted type should only depend on the type and not on the data,
#' to avoid confusion.
#'
#' @param x an object to summarise. Generally only methods of atomic vectors
#'   and variants have been implemented.
#'
#' @export
type_sum <- function(x) {
  UseMethod("type_sum")
}

#' @export
type_sum.ordered <- function(x) {
  # r-lib/vctrs#323:
  type_sum.default(x)
}

#' @export
type_sum.factor <- function(x) {
  # r-lib/vctrs#323:
  "fct"
}

#' @export
type_sum.default <- function(x) {
  pillar_attr <- attr(x, "pillar", exact = TRUE)

  label <- pillar_attr$label
  if (!is.null(label)) {
    return(I(label))
  }

  if (is.object(x) || vec_is(x)) {
    return(vec_ptype_abbr(x))
  }

  switch(typeof(x),
    builtin = ,
    special = ,
    closure = "fn",
    environment = "env",
    symbol =
      if (is_missing(x)) {
        "missing"
      } else {
        "sym"
      },
    typeof(x)
  )
}

#' @export
vec_ptype_abbr.pillar_empty_col <- function(x, ...) {
  vec_ptype_abbr(x[[1]])
}

#' @description
#' `obj_sum()` also includes the size (but not the shape) of the object
#' if [vctrs::vec_is()] is `TRUE`.
#' It should always return a string (a character vector of length one).
#' As of pillar v1.6.1, the default method forwards to [vctrs::vec_ptype_abbr()]
#' for vectors and to [type_sum()] for other objects.
#' Previous versions always forwarded to [type_sum()].
#' An attribute named `"short"` in the return value will be picked up by
#' the [pillar_shaft()] method for lists, and used if space is limited.
#'
#' @examples
#' obj_sum(1:10)
#' obj_sum(matrix(1:10))
#' obj_sum(data.frame(a = 1))
#' obj_sum(Sys.Date())
#' obj_sum(Sys.time())
#' obj_sum(mean)
#' @rdname type_sum
#' @export
obj_sum <- function(x) {
  UseMethod("obj_sum")
}

#' @export
obj_sum.default <- function(x) {
  if (!vec_is(x)) {
    type_sum(x)
  } else {
    abbr <- vec_ptype_abbr(x, suffix_shape = FALSE)

    out <- paste(abbr, size_sum(x))
    if (is.array(x)) {
      short <- paste0(abbr, "[", symbol$ellipsis, "]")
    } else {
      short <- abbr
    }

    structure(out, short = short)
  }
}

#' @export
obj_sum.AsIs <- function(x) {
  paste0("I(", obj_sum(remove_as_is_class(x)), ")")
}

#' @description
#' `size_sum()` is called by `obj_sum()` to format the size of the object.
#' It should always return a string (a character vector of length one),
#' it can be an empty string `""` to omit size information,
#' this is what the default method does for scalars.
#'
#' @keywords internal
#' @examples
#'
#' size_sum(1:10)
#' size_sum(trees)
#' size_sum(Titanic)
#' @rdname type_sum
#' @export
size_sum <- function(x) {
  UseMethod("size_sum")
}

#' @export
size_sum.default <- function(x) {
  if (!vec_is(x)) {
    return("")
  }

  paste0("[", dim_desc(x), "]")
}

# obj_sum ----------------------------------------------------------------

test_that("forwards to vec_ptype_abbr() for S4", {
  x <- methods::setClass("A")
  expect_equal(obj_sum(x), vec_ptype_abbr(x))
})

test_that("forwards to vec_ptype_abbr() for S3", {
  x <- structure(list(), class = c("a", "b", "c"))
  expect_equal(obj_sum(x), vec_ptype_abbr(x))
})

test_that("NULL handled specially", {
  expect_equal(obj_sum(NULL), "NULL")
})

test_that("data frames and common data vectors have size summary", {
  local_override_size_sum()

  expect_obj_sum_is_ptype <- function(x) {
    obj_sum <- obj_sum(x)
    attr(obj_sum, "short") <- NULL

    ptype <- vec_ptype_abbr(x, suffix_shape = FALSE)

    expect_equal(obj_sum, !!paste_with_space_if_needed(ptype, size_sum(x)))
  }

  expect_obj_sum_is_ptype(mtcars)
  expect_obj_sum_is_ptype(factor(1:3))
  expect_obj_sum_is_ptype(ordered(1:3))
  expect_obj_sum_is_ptype(Sys.Date() + 1:3)
  expect_obj_sum_is_ptype(Sys.time() + 1:3)
  expect_obj_sum_is_ptype(Sys.time() - Sys.time() + 1:3)
  expect_obj_sum_is_ptype(as.POSIXlt(Sys.time() + 1:3))
  expect_obj_sum_is_ptype(list(1, 2:3))
  expect_obj_sum_is_ptype(matrix(1:10, ncol = 2))
  expect_obj_sum_is_ptype(Titanic)
  expect_obj_sum_is_ptype(as_override_size_sum(1:3))
})

test_that("object summary has short version", {
  local_override_size_sum()

  expect_obj_sum_short_is_ptype <- function(x, suffix = "") {
    obj_sum <- attr(obj_sum(x), "short")

    ptype <- vec_ptype_abbr(x, suffix_shape = FALSE)

    expect_equal(obj_sum, !!paste0(ptype, suffix))
  }

  expect_obj_sum_short_is_ptype(mtcars)
  expect_obj_sum_short_is_ptype(factor(1:3))
  expect_obj_sum_short_is_ptype(ordered(1:3))
  expect_obj_sum_short_is_ptype(Sys.Date() + 1:3)
  expect_obj_sum_short_is_ptype(Sys.time() + 1:3)
  expect_obj_sum_short_is_ptype(Sys.time() - Sys.time() + 1:3)
  expect_obj_sum_short_is_ptype(as.POSIXlt(Sys.time() + 1:3))
  expect_obj_sum_short_is_ptype(list(1, 2:3))
  expect_obj_sum_short_is_ptype(matrix(1:10, ncol = 2), paste0("[", symbol$ellipsis, "]"))
  expect_obj_sum_short_is_ptype(Titanic, paste0("[", symbol$ellipsis, "]"))
  expect_obj_sum_short_is_ptype(as_override_size_sum(1:3))
})


# type_sum ----------------------------------------------------------------

test_that("less common objects get abbreviations", {
  expect_equal(type_sum(environment()), "env")
  expect_equal(type_sum(environment), "fn")
  expect_equal(type_sum(list), "fn")
  expect_equal(type_sum(quote(foo)), "sym")
  expect_equal(type_sum(expr()), "missing")
})

#' Calculate display width
#'
#' `get_extent()` calculates the display width for each string in a character
#' vector.
#'
#' @param x A character vector.
#' @export
#' @importFrom fansi strip_sgr
#' @importFrom utf8 utf8_width
#' @examples
#' get_extent(c("abc", "de"))
#' get_extent("\u904b\u6c23")
get_extent <- function(x) {
  force(x)
  x <- strip_sgr(x, warn = FALSE)
  width <- utf8_width(x, encode = FALSE, utf8 = TRUE)
  if (anyNA(width)) {
    is_na <- which(is.na(width))
    width[is_na] <- nchar(x[is_na], type = "width")
  }
  width
}

#' @description
#' `get_max_extent()` calculates the maximum display width of all strings in a
#' character vector, zero for empty vectors.
#' @export
#' @rdname get_extent
#' @examples
#' get_max_extent(c("abc", "de"))
get_max_extent <- function(x) {
  max(get_extent(x), 0L, na.rm = TRUE)
}

#' Alignment helper
#'
#' Facilitates easy alignment of strings within a character vector. Designed to
#' help implementers of formatters for custom data types.
#'
#' @param x A character vector
#' @param width The width that each string is padded to. If `NULL`, the maximum
#'   display width of the character vector is used (see [get_max_extent()]).
#' @param align How should strings be aligned? If `align = left` then padding
#'   appears on the `right`, and vice versa.
#' @param space What character should be used for the padding?
#' @export
#' @examples
#' align(c("abc", "de"), align = "left")
#' align(c("abc", "de"), align = "right")
align <- function(x, width = NULL, align = c("left", "right"), space = " ") {
  "!!!!!DEBUG align(`v(x)`, `v(class(x))`)"

  align <- match.arg(align)
  extent <- get_extent(x)
  if (is.null(width)) {
    width <- max(extent)
  }
  align_impl(x, width, align, space, extent)
}

align_impl <- function(x, width, align, space, extent) {
  spaces <- pmax(width - extent, 0L)
  if (align == "left") {
    paste0(x, strrep(space, spaces))
  } else {
    paste0(strrep(space, spaces), x)
  }
}

#' Format the footer of a tibble
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' For easier customization, the formatting of a tibble is split
#' into three components: header, body, and footer.
#' The `tbl_format_footer()` method is responsible for formatting the footer
#' of a tibble.
#'
#' Override or extend this method if you need to change the appearance
#' of the footer.
#' The default implementation adds information about rows and columns
#' that are not shown in the body.
#'
#' @inheritParams tbl_format_body
#' @inherit tbl_format_body return
#'
#' @export
#' @examplesIf rlang::is_installed(c("palmerpenguins", "tibble"))
#' setup <- tbl_format_setup(palmerpenguins::penguins)
#' tbl_format_footer(palmerpenguins::penguins, setup)
#'
#' # Shortcut for debugging
#' tbl_format_footer(setup)
tbl_format_footer <- function(x, setup, ...) {
  check_dots_empty()

  UseMethod("tbl_format_footer")
}

#' @export
tbl_format_footer.pillar_tbl_format_setup <- function(x, ...) {
  as_glue(c(
    cli::style_bold("<tbl_format_footer(setup)>"),
    tbl_format_footer(x$x, setup = x)
  ))
}

#' @export
tbl_format_footer.tbl <- function(x, setup, ...) {
  footer <- format_footer(x, setup)
  footer_comment <- wrap_footer_bullet(footer, setup)
  footer_advice <- format_footer_advice(x, setup)
  footer_advice_comment <- wrap_footer_bullet(
    footer_advice,
    setup,
    lines = 1,
    ellipsis = FALSE,
    bullet = symbol$info
  )
  style_subtle(c(footer_comment, footer_advice_comment))
}

format_footer <- function(x, setup) {
  extra_rows <- format_footer_extra_rows(x, setup)
  abbrev_cols <- format_footer_abbrev_cols(x, setup)
  extra_cols <- format_footer_extra_cols(x, setup)

  compact(list(extra_rows, abbrev_cols, extra_cols))
}

format_footer_extra_rows <- function(x, setup) {
  if (length(setup$df) != 0) {
    if (is.na(setup$rows_missing)) {
      c("more", "rows")
    } else if (setup$rows_missing > 0) {
      c(big_mark(setup$rows_missing), "more", pluralise_n("row(s)", setup$rows_missing))
    }
  } else {
    rows_body <- nrow(setup$df)
    if (is.na(setup$rows_total) && rows_body > 0) {
      c("at", "least", big_mark(rows_body), pluralise_n("row(s)", rows_body), "total")
    }
  }
}

format_footer_abbrev_cols <- function(x, setup) {
  abbrev_cols <- setup$abbrev_cols
  abbrev_cols_total <- length(abbrev_cols)
  if (abbrev_cols_total == 0) {
    return(NULL)
  }

  abbrev_cols <- paste0(
    map_chr(seq_along(abbrev_cols), superdigit),
    pillar_options$superdigit_sep(),
    abbrev_cols
  )

  idx_all_but_last <- seq_len(abbrev_cols_total - 1)
  abbrev_cols[idx_all_but_last] <- paste0(abbrev_cols[idx_all_but_last], ",")

  c(
    "abbreviated",
    pluralise("name(s):", abbrev_cols),
    abbrev_cols
  )
}

format_footer_extra_cols <- function(x, setup) {
  extra_cols <- setup$extra_cols
  if (length(extra_cols) == 0) {
    return(NULL)
  }

  extra_cols_total <- setup$extra_cols_total

  vars <- format_extra_vars(extra_cols, extra_cols_total)
  c(
    big_mark(extra_cols_total),
    if (!identical(setup$rows_total, 0L) && nrow(setup$df) > 0) "more",
    pluralise_n("variable(s):", extra_cols_total),
    vars
  )
}

format_extra_vars <- function(extra_cols, extra_cols_total) {
  out <- unname(map2_chr(extra_cols, names(extra_cols), format_abbrev))

  if (extra_cols_total > length(extra_cols)) {
    out <- c(out, symbol$ellipsis)
  }

  out[-length(out)] <- paste0(out[-length(out)], ",")
  out
}

format_footer_advice <- function(x, setup) {
  if (!isTRUE(pillar_options$advice())) {
    return()
  }

  if (setup$extra_cols_total > length(setup$extra_cols)) {
    cols <- "`colnames()` to see all variable names"
  } else {
    cols <- NULL
  }

  if (is.na(setup$rows_missing) || setup$rows_missing > 0) {
    rows <- "`print(n = ...)` to see more rows"
  } else {
    rows <- NULL
  }

  advice <- enum_collapse(c(rows, cols))

  if (length(advice) == 0) {
    return()
  }

  paste0("Use ", paste(advice, collapse = " "))
}

wrap_footer_bullet <- function(footers,
                               setup,
                               lines = setup$max_footer_lines,
                               ellipsis = TRUE,
                               bullet = symbol$info) {
  out <- character()

  for (footer in footers) {
    if (lines <= 0) {
      break
    }
    current <- wrap_footer_bullet_one(footer, setup, lines, ellipsis, bullet)
    out <- c(out, current)
    lines <- lines - length(current)
  }

  out
}

wrap_footer_bullet_one <- function(footer,
                                   setup,
                                   lines,
                                   ellipsis,
                                   bullet) {
  # When asking for width = 80, use at most 79 characters
  max_extent <- setup$width - 1L

  tier_widths <- get_footer_tier_widths_bullets(
    footer, max_extent, lines, ellipsis
  )

  # show optuput even if too wide
  widths <- pmin(get_extent(footer), max_extent - 4L)
  wrap <- colonnade_compute_tiered_col_widths_df(widths, widths, tier_widths)

  # truncate output that doesn't fit
  truncated <- anyNA(wrap$tier)
  split <- split(footer[wrap$id], wrap$tier)
  if (ellipsis && truncated && length(split) > 0) {
    split[[length(split)]] <- c(split[[length(split)]], symbol$ellipsis)
  }
  split <- imap(split, function(.x, .y) {
    if (.y == 1) {
      header <- bullet
    } else {
      header <- " "
    }
    c("#", header, .x)
  })

  map_chr(split, paste, collapse = " ")
}

get_footer_tier_widths_bullets <- function(footer, max_extent, n_tiers, ellipsis = TRUE) {
  if (ellipsis) {
    extra_width <- get_extent(symbol$ellipsis) + 1L # space, ellipsis
  } else {
    extra_width <- FALSE
  }

  n_tiers <- min(length(footer), n_tiers)

  c(
    rep(max_extent - 4, n_tiers - 1),
    max_extent - 4 - extra_width
  )
}

pre_dots <- function(x) {
  if (length(x) > 0) {
    paste0(symbol$ellipsis, " ", x)
  } else {
    character()
  }
}

collapse <- function(x) paste(x, collapse = ", ")

split_lines <- function(x) {
  # Avoid .ptype argument to vec_c()
  if (is_empty(x)) {
    return(character())
  }

  unlist(strsplit(x, "\n", fixed = TRUE))
}

enum_collapse <- function(x) {
  if (length(x) > 1) {
    x_len <- length(x)
    idx_all_but_last <- seq2(1, x_len - 1)
    x[idx_all_but_last] <- map(x[idx_all_but_last], function(x) {
      x[[length(x)]] <- paste0(x[[length(x)]], ",")
      x
    })
    x <- c(x[idx_all_but_last], "and", x[x_len])
  }

  x
}

make_option_impl <- function(getter, option_name = NULL, env = caller_env()) {
  getter_body <- enexpr(getter)

  if (is.null(option_name)) {
    # Assuming that the call is getOption()
    option_name <- getter_body[[2]]
    stopifnot(is.character(option_name))
  }
  name <- sub(paste0(utils::packageName(env), "."), "", option_name, fixed = TRUE)
  getter_name <- paste0("get_", utils::packageName(env), "_option_", name)
  local_setter_name <- paste0("local_", utils::packageName(env), "_option_", name)
  setter_name <- paste0("set_", utils::packageName(env), "_option_", name)

  local_setter_body <- expr({
    out <- !!call2("local_options", !!option_name := sym("value"), .frame = sym("env"))
    !!call2(getter_name)
    invisible(out[[1]])
  })

  setter_body <- expr({
    out <- !!call2("options", !!option_name := sym("value"))
    !!call2(getter_name)
    invisible(out[[1]])
  })

  body <- expr({
    if (missing(!!sym("value"))) {
      if (!missing(local)) {
        abort("Can't pass `local` argument if `value` is missing.")
      }
      !!getter_body
    } else if (local) !!local_setter_body
    else !!setter_body
  })

  args <- pairlist2(value = , local = FALSE, env = quote(caller_env()))

  assign(getter_name, new_function(list(), getter_body, env = env), env)
  assign(local_setter_name, new_function(args[c(1, 3)], local_setter_body, env = env), env)
  assign(setter_name, new_function(args[1], setter_body, env = env), env)

  new_function(args, body, env = env)
}

# nocov start
keep_empty <- function(fun) {
  function(x) {
    ret <- rep_along(x, "")
    update <- which(is.na(x) | x != "")
    ret[update] <- fun(x[update])
    ret
  }
}
# nocov end

#' Styling helpers
#'
#' Functions that allow implementers of formatters for custom data types to
#' maintain a consistent style with the default data types.
#'
#' `style_subtle()` is affected by the `subtle` [option][pillar_options].
#'
#' @param x The character vector to style.
#' @export
#' @seealso [pillar_options] for a list of options
#' @examples
#' style_subtle("text")
style_subtle <- keep_empty(function(x) {
  force(x)
  if (isTRUE(get_pillar_option_subtle())) {
    crayon_grey_0.6(x)
  } else {
    x
  }
})

#' @rdname style_subtle
#' @details
#' `style_subtle_num()` is affected by the
#' `subtle_num` [option][pillar_options],
#' which is `FALSE` by default.
#'
#' @export
#' @examples
#' style_subtle_num(0.01 * 1:3, c(TRUE, FALSE, TRUE))
style_subtle_num <- function(x, negative) {
  if (isTRUE(get_pillar_option_subtle_num())) {
    style_subtle(x)
  } else {
    ifelse(negative, style_neg(x), x)
  }
}

style_hint <- keep_empty(function(x) {
  force(x)
  if (isTRUE(get_pillar_option_subtle())) {
    crayon_grey_0.8(x)
  } else {
    x
  }
})

style_spark_na <- function(x) {
  crayon_yellow(x)
}

#' @details
#' `style_bold()` is affected by the `bold` [option][pillar_options],
#' which is `FALSE` by default.
#'
#' @rdname style_subtle
#' @export
#' @examples
#' style_bold("Petal.Width")
style_bold <- keep_empty(function(x) {
  if (isTRUE(get_pillar_option_bold())) {
    crayon_bold(x)
  } else {
    x
  }
})

#' @rdname style_subtle
#' @export
#' @examples
#' style_na("NA")
style_na <- function(x) {
  crayon_red(x)
}

style_na_if <- function(x, p) {
  idx <- which(p)
  if (length(idx) == 0) {
    return(x)
  }
  x[p] <- style_na(x[idx[[1]]])
  x
}

#' @details
#' `style_neg()` is affected by the `pillar.neg` [option][pillar_options].
#'
#' @rdname style_subtle
#' @export
#' @examples
#' style_neg("123")
style_neg <- keep_empty(function(x) {
  if (isTRUE(get_pillar_option_neg())) {
    crayon_red(x)
  } else {
    x
  }
})

pillar_na <- function(use_brackets_if_no_color = FALSE) {
  if (use_brackets_if_no_color && !has_color()) {
    "<NA>"
  } else {
    style_na("NA")
  }
}

style_list <- function(x) {
  style_subtle(x)
}

# Only check if we have color support once per session
num_colors <- local({
  num_colors <- NULL
  function(forget = FALSE) {
    if (is.null(num_colors) || forget) {
      num_colors <<- cli::num_ansi_colors()
    }
    num_colors
  }
})

has_color <- function() {
  num_colors() > 1
}

# nocov start
# Crayon functions call cli::num_ansi_colors() every call
make_style_fast <- function(...) {
  # Force has_color to be true when making styles
  # Use prefix to avoid problems with empty NAMESPACE
  rlang::local_options(cli.num_colors = 16L)

  style_16 <- attr(cli::make_ansi_style(..., colors = 16), "_styles")[[1]]
  open_16 <- style_16$open
  close_16 <- style_16$close

  style_256 <- attr(cli::make_ansi_style(..., colors = 256), "_styles")[[1]]
  open_256 <- style_256$open
  close_256 <- style_256$close

  function(...) {
    colors <- num_colors()
    if (colors <= 1) {
      paste0(...)
    } else if (colors >= 256) {
      paste0(open_256, ..., close_256)
    } else {
      paste0(open_16, ..., close_16)
    }
  }
}

# Placeholders, assigned in .onLoad()
crayon_underline <- function(...) {}
crayon_italic <- function(...) {}
crayon_red <- function(...) {}
crayon_yellow <- function(...) {}
crayon_bold <- function(...) {}
crayon_grey_0.6 <- function(...) {}
crayon_grey_0.8 <- function(...) {}

assign_crayon_styles <- function() {
  crayon_underline <<- make_style_fast("underline")
  crayon_italic <<- make_style_fast("italic")
  crayon_red <<- make_style_fast("red")
  crayon_yellow <<- make_style_fast("yellow")
  crayon_bold <<- make_style_fast("bold")
  crayon_grey_0.6 <<- make_style_fast(grDevices::grey(0.6), grey = TRUE)
  crayon_grey_0.8 <<- make_style_fast(grDevices::grey(0.8), grey = TRUE)
}
# nocov end

#' Formatting of tbl objects
#'
#' See [tibble::formatting] for details.
#'
#' @name format_tbl
#' @export
#' @keywords internal
print.tbl <- function(x, width = NULL, ..., n = NULL, max_extra_cols = NULL,
                      max_footer_lines = NULL) {
  print_tbl(
    x, width, ...,
    n = n, max_extra_cols = max_extra_cols, max_footer_lines = max_footer_lines
  )
}

print_tbl <- function(x, width = NULL, ...,
                      n_extra = NULL,
                      n = NULL, max_extra_cols = NULL, max_footer_lines = NULL) {
  if (!is.null(n_extra)) {
    deprecate_soft(
      "1.6.2", "pillar::print(n_extra = )", "pillar::print(max_extra_cols = )",
      user_env = caller_env(2)
    )
    if (is.null(max_extra_cols)) {
      max_extra_cols <- n_extra
    }
  }

  writeLines(format(
    x,
    width = width, ...,
    n = n, max_extra_cols = max_extra_cols, max_footer_lines = max_footer_lines
  ))
  invisible(x)
}

#' @export
#' @rdname format_tbl
format.tbl <- function(x, width = NULL, ...,
                       n = NULL, max_extra_cols = NULL, max_footer_lines = NULL) {
  format_tbl(
    x, width, ...,
    n = n, max_extra_cols = max_extra_cols, max_footer_lines = max_footer_lines
  )
}

format_tbl <- function(x, width = NULL, ...,
                       n_extra = NULL,
                       n = NULL, max_extra_cols = NULL, max_footer_lines = NULL) {
  check_dots_empty(action = signal)

  if (!is.null(n_extra)) {
    deprecate_soft(
      "1.6.2", "pillar::format(n_extra = )", "pillar::format(max_extra_cols = )",
      user_env = caller_env(2)
    )
    if (is.null(max_extra_cols)) {
      max_extra_cols <- n_extra
    }
  }

  # Reset local cache for each new output
  force(x)
  num_colors(forget = TRUE)

  setup <- tbl_format_setup(x,
    width = width, ...,
    n = n,
    max_extra_cols = max_extra_cols,
    max_footer_lines = max_footer_lines,
    focus = attr(x, "pillar_focus")
  )

  header <- tbl_format_header(x, setup)
  body <- tbl_format_body(x, setup)
  footer <- tbl_format_footer(x, setup)
  c(header, body, footer)
}

format_comment <- function(x, width) {
  if (length(x) == 0L) {
    return(character())
  }
  map_chr(x, wrap, prefix = "# ", width = min(width, cli::console_width()))
}

# wrap --------------------------------------------------------------------

NBSP <- "\U00A0"

wrap <- function(..., indent = 0, prefix = "", width) {
  x <- paste0(..., collapse = "")
  wrapped <- strwrap2(x, width - get_extent(prefix), indent)
  wrapped <- paste0(prefix, wrapped)
  wrapped <- gsub(NBSP, " ", wrapped)

  paste0(wrapped, collapse = "\n")
}

strwrap2 <- function(x, width, indent) {
  fansi::strwrap_ctl(x, width = max(width, 0), indent = indent, exdent = indent + 2)
}

# Format numbers in decimal notation
#
# This formatting system is designed to make it as easy as possible to
# compare columns of numbers. Significant digits are coloured black or red
# (for positive and negative numbers) and non-significant digits are coloured
# in paler gray.
#
# @return A list with at least the following elements:
# * `neg`: negative sign or space, `TRUE` if needed
# * `lhs`: whole number
# * `dec`: decimal point, `TRUE` if needed
# * `rhs`: remainder of number
#
# @param x A numeric vector
# @param sigfig Number of significant figures to display.
# @param digits Number of digits after the decimal point, incompatible with
#   `sigfig`. A negative number means the maximum number of digits to display.
# @param sci_mod Set to 1 for scientific or to 3 for engineering display.
# @param si Set to `TRUE` for SI notation, requires `sci_mod = 3`
# @param fixed Set to `TRUE` to use the same exponent everywhere.
# @seealso [split_decimal()]
# @examples
# split_decimal(1.5:3.5)
# split_decimal(1.5:3.5, sci_mod = 1)
# split_decimal(1e9)
# split_decimal(1e9, sci_mod = 1)
split_decimal <- function(x, sigfig, digits = NULL, sci_mod = NULL, si = FALSE,
                          fixed_exponent = NULL) {
  "!!!!!!DEBUG split_decimal(`v(x)`, `v(sigfig)`, `v(digits)`, `v(sci_mod)`, `v(si)`, `v(fixed_exponent)`"

  stopifnot(is.numeric(x))
  sigfig <- check_sigfig(sigfig)

  num <- is.finite(x)
  "!!!!!!DEBUG `v(num)`"
  dec <- num

  # Do we need negative signs?
  neg <- !is.na(x) & x < 0
  "!!!!!!DEBUG `v(neg)`"

  abs_x <- abs(x)
  mnt <- abs_x
  "!!!!!!DEBUG `v(mnt)`"

  if (!is.null(sci_mod)) {
    exp <- fix_exp(num, compute_exp(mnt, sigfig, digits), fixed_exponent, sci_mod, si)
    "!!!!!!DEBUG `v(exp)`"
    unit <- attr(exp, "unit")

    # Must divide by 10^exp, because 10^-exp may not be representable
    # for very large values of exp
    mnt_idx <- which(num & mnt != 0)
    mnt[mnt_idx] <- safe_divide_10_to(mnt[mnt_idx], exp[mnt_idx])
    "!!!!!!DEBUG `v(mnt)`"
  } else {
    exp <- 0
    "!!!!!!DEBUG `v(exp)`"
    unit <- NULL
  }

  if (is.null(sci_mod) || !is.null(fixed_exponent)) {
    exp_display <- rep_along(x, NA_integer_)
  } else {
    exp_display <- exp
  }

  if (is.null(digits)) {
    "!!!!!!DEBUG `v(sigfig)`"
    min_sigfig <- compute_min_sigfig(mnt)
    round_mnt <- safe_signif(mnt, pmax(sigfig, min_sigfig, na.rm = TRUE))
    rhs_digits <- compute_rhs_digits(mnt, sigfig)
  } else if (digits >= 0) {
    "!!!!!!DEBUG `v(digits)`"
    round_mnt <- round(mnt, digits)
    rhs_digits <- digits
  } else {
    "!!!!!!DEBUG `v(-digits)`"
    round_mnt <- round(mnt, -digits)
    trunc_mnt <- trunc(mnt)
    rhs_digits <- compute_rhs_digits(mnt - trunc_mnt, -digits, trunc_mnt)
  }

  "!!!!!!DEBUG `v(round_mnt)`"
  "!!!!!!DEBUG `v(rhs_digits)`"

  lhs <- trunc(round_mnt)
  "!!!!!!DEBUG `v(lhs)`"

  rhs <- round_mnt - lhs
  "!!!!!!DEBUG `v(rhs)`"

  if (is.null(digits) || digits < 0) {
    "!!!!!!DEBUG `v(lhs * 10^exp - abs_x)`"
    reset_dec <- (mnt == 0 | (rhs == 0 & within_tolerance(lhs * 10^exp, abs_x)))
    "!!!!!!DEBUG `v(reset_dec)`"

    dec[reset_dec] <- FALSE
    "!!!!!!DEBUG `v(dec)`"
  }

  ret <- list(
    sigfig = sigfig,
    num = num,
    neg = neg,
    # integer64 doesn't support format(trim = FALSE)
    # trimws() is unnecessarily slow
    lhs = sub("^ +", "", format(lhs, scientific = FALSE)),
    lhs_zero = (lhs == 0),
    rhs = rhs,
    rhs_digits = rhs_digits,
    dec = dec,
    exp = exp_display,
    unit = unit,
    si = si
  )

  set_width(ret, get_decimal_width(ret))
}

fix_exp <- function(num, exp, fixed_exponent, sci_mod, si) {
  "!!!!!!DEBUG fix_exp(`v(num)`, `v(exp)`, `v(fixed_exponent)`, `v(sci_mod)`, `v(si)`"
  if (!is.null(fixed_exponent)) {
    if (is.finite(fixed_exponent)) {
      exp <- fixed_exponent
    } else if (all(is.na(exp))) {
      exp <- NA_real_
    } else if (fixed_exponent < 0) {
      exp <- min(exp, na.rm = TRUE)
    } else {
      exp <- max(exp, na.rm = TRUE)
    }
    "!!!!!!DEBUG `v(exp)`"
    exp <- as.integer(round(exp))
    exp <- structure(rep_along(num, exp), unit = exp %|% 0L)
    exp[!num] <- NA_integer_
    "!!!!!!DEBUG `v(exp)`"
  }

  if (sci_mod != 1) {
    exp[] <- as.integer(round(floor(exp / sci_mod) * sci_mod))
    "!!!!!!DEBUG `v(exp)`"
  }
  if (si) {
    # Truncate very small and very large exponents
    exp[] <- pmin(pmax(exp, -24L), 24L)
    "!!!!!!DEBUG `v(exp)`"
  }

  exp
}

get_decimal_width <- function(x) {
  exp <- x$exp[!is.na(x$exp)]

  if (x$si) {
    exp_digits <- any(exp != 0)
  } else {
    exp_digits <- any(exp < 0) + max(2 + trunc(log10(abs(exp) + 0.5)), 0)
  }

  max(x$neg + nchar(x$lhs), 0) +
    any(x$dec, na.rm = TRUE) +
    max(x$rhs_digits, 0) +
    exp_digits
}

safe_signif <- function(x, digits) {
  if (length(x) == 0L) {
    return(numeric())
  }
  signif(x, digits)
}

safe_divide_10_to <- function(x, y) {
  # Computes x / 10^y in a robust way

  10^(log10(x) - y)
}

eps_2 <- 2 * .Machine$double.eps

within_tolerance <- function(x, y) {
  "!!!!!!DEBUG within_tolerance(`v(x)`, `v(y)`)"
  l2x <- round(log2(x))
  "!!!!!!DEBUG `v(l2x)`"
  l2y <- round(log2(y))
  "!!!!!!DEBUG `v(l2y)`"

  equal <- (l2x == l2y)
  equal[is.na(equal)] <- FALSE
  out <- equal

  # Work around integer64 problem
  equal[x == y] <- FALSE
  "!!!!!!DEBUG `v(abs((x[equal] - y[equal]) * 2 ^ -l2x[equal]))`"
  out[equal] <- abs((x[equal] - y[equal]) * 2^-l2x[equal]) <= eps_2
  out
}

compute_rhs_digits <- function(x, sigfig, offset = rep_along(x, 0)) {
  "!!!!!!DEBUG compute_rhs_digits(`v(x)`, `v(sigfig)`)"
  # If already bigger than sigfig, can round to zero.
  # Otherwise ensure we have sigfig digits shown
  exp <- compute_exp(x, sigfig)
  exp[is.na(exp)] <- Inf
  "!!!!!!DEBUG `v(exp)"
  rhs_digits <- rep_along(x, 0)
  "!!!!!!DEBUG `v(rhs_digits)"

  if (!is.integer(x) && !all(x == trunc(x), na.rm = TRUE)) {
    has_rhs <- (exp <= sigfig)
    rhs_digits[has_rhs] <- sigfig - 1 - exp[has_rhs]

    to_check <- rhs_digits > 0
    while (any(to_check, na.rm = TRUE)) {
      "!!!!!!DEBUG `v(to_check)"
      "!!!!!!DEBUG `v(rhs_digits)"

      which_to_check <- which(to_check)
      val <- (x[which_to_check] + offset[which_to_check]) * 10^(rhs_digits[which_to_check] - 1)
      "!!!!!!DEBUG `v(val)"
      "!!!!!!DEBUG `v(val - round(val))"

      resid_zero <- within_tolerance(val, round(val))
      resid_zero[is.na(resid_zero)] <- FALSE

      rhs_digits[which_to_check][resid_zero] <-
        rhs_digits[which_to_check][resid_zero] - 1

      to_check[which_to_check][!resid_zero] <- FALSE
      to_check[rhs_digits == 0] <- FALSE
    }
  }

  "!!!!!!DEBUG `v(rhs_digits)"
  rhs_digits
}

compute_min_sigfig <- function(x) {
  ret <- rep_along(x, NA_integer_)
  nonzero <- which(x != 0 & is.finite(x))
  ret[nonzero] <- as.integer(floor(log10(x[nonzero]))) + 1L
  ret
}

compute_extra_sigfig <- function(x) {
  x <- sort(abs(x))
  delta <- diff(x)
  x <- x[-1]

  keep <- which((delta != 0) & is.finite(delta))
  if (length(keep) == 0) {
    return(0)
  }

  ceiling(log10(max(x[keep] / delta[keep]))) - 1
}

LOG_10 <- log(10)

compute_exp <- function(x, sigfig, digits) {
  if (is.null(sigfig)) {
    sigfig <- abs(digits)
  }

  # With 3 significant digits:
  # 0.9994 -> 0.999 -> exp == -1
  # 0.9995 -> 1.00 -> exp == 0
  # This means that x is divided by 0.9995 in this example
  # before computing log10().
  # Division before log is the same as subtraction after log.
  # Using log1p for numerical stability.
  offset <- log1p(-5 * 10^(-sigfig - 1)) / LOG_10

  ret <- rep_along(x, NA_integer_)
  nonzero <- which(x != 0 & is.finite(x))
  ret[nonzero] <- as.integer(floor(log10(x[nonzero]) - offset))
  ret
}

format_mantissa <- function(x) {
  lhs <- format_lhs(x)
  dec <- format_dec(x)
  rhs <- format_rhs(x)
  paste0(lhs, dec, rhs)
}

format_neg <- function(s) {
  neg <- s$neg
  neg_col <- ifelse(neg, "-", "")
  neg_col
}

format_lhs <- function(s) {
  neg <- s$neg
  num <- s$num
  lhs_zero <- s$lhs_zero

  lhs_str <- s$lhs
  lhs_split <- strsplit(lhs_str, "", fixed = TRUE)
  lhs_width <- lengths(lhs_split)

  lhs_split_underlined <- map(lhs_split, underline_3_back)

  lhs_sig <- map(pmin(lhs_width, s$sigfig), seq_len)
  lhs_insig <- map(lhs_sig, `-`)

  lhs_split_sig <- map2(lhs_split_underlined, lhs_sig, `[`)
  lhs_split_non <- map2(lhs_split_underlined, lhs_insig, `[`)

  lhs_sig <- map_chr(lhs_split_sig, paste, collapse = "")
  lhs_non <- map_chr(lhs_split_non, paste, collapse = "")

  # as.character() to support corner case of length zero
  lhs_col <- as.character(ifelse(num,
    paste0(
      style_num(lhs_sig, neg, !lhs_zero),
      style_subtle_num(lhs_non, neg)
    ),
    style_na(lhs_str)
  ))

  lhs_col <- paste0(format_neg(s), lhs_col)
  # No alignment here, result needs to be right-aligned
  lhs_col
}

underline_3_back <- function(x) {
  idx <- which(trunc((seq_along(x) - length(x)) / 3) %% 2 == 1)
  x[idx] <- crayon_underline(x[idx])
  x
}

format_dec <- function(s) {
  neg <- s$neg
  dec <- s$dec
  lhs_zero <- s$lhs_zero

  # Decimal column
  if (any(dec)) {
    dec_col <- ifelse(dec, style_num(".", neg, !lhs_zero), " ")
  } else {
    dec_col <- rep_along(neg, "")
  }
  dec_col
}

format_rhs <- function(s) {
  neg <- s$neg
  dec <- s$dec
  lhs_zero <- s$lhs_zero
  rhs_num <- s$rhs_num
  rhs_digits <- s$rhs_digits

  # Digits on RHS of .
  rhs_num <- sprintf("%.0f", abs(round(s$rhs * 10^(s$rhs_digits))))
  rhs_num[rhs_num == "0"] <- ""

  n_zeros <- pmax(0, rhs_digits - get_extent(rhs_num))
  rhs_zero <- strrep("0", n_zeros)

  rhs_split <- strsplit(paste0(rhs_zero, rhs_num), "", fixed = TRUE)
  rhs_split_underlined <- map(rhs_split, underline_3)

  rhs_is_zero <- map(n_zeros, seq_len)
  rhs_is_nonzero <- map2(n_zeros + 1L, rhs_digits, seq2)

  rhs_split_underlined_zero <- map2(rhs_split_underlined, rhs_is_zero, `[`)
  rhs_split_underlined_num <- map2(rhs_split_underlined, rhs_is_nonzero, `[`)

  rhs_underlined_zero <- map_chr(rhs_split_underlined_zero, paste, collapse = "")
  rhs_underlined_num <- map_chr(rhs_split_underlined_num, paste, collapse = "")

  rhs_col <- ifelse(dec,
    paste0(
      style_num(rhs_underlined_zero, neg, !lhs_zero),
      style_num(rhs_underlined_num, neg)
    ),
    ""
  )

  # ensure all same width
  rhs_col <- align(rhs_col, max(rhs_digits, 0L, na.rm = TRUE), "left")

  rhs_col
}

underline_3 <- function(x) {
  idx <- which(trunc((seq_along(x) - 1) / 3) %% 2 == 1)
  x[idx] <- crayon_underline(x[idx])
  x
}

#' @export
#' @param negative,significant Logical vector the same length as `x` that
#'   indicate if the values are negative and significant, respectively
#' @rdname style_subtle
#' @examples
#' style_num(
#'   c("123", "456"),
#'   negative = c(TRUE, FALSE)
#' )
#' style_num(
#'   c("123", "456"),
#'   negative = c(TRUE, FALSE),
#'   significant = c(FALSE, FALSE)
#' )
style_num <- function(x, negative, significant = rep_along(x, TRUE)) {
  ifelse(significant, ifelse(negative, style_neg(x), x), style_subtle_num(x, negative))
}

assemble_decimal <- function(x) {
  mantissa <- format_mantissa(x)
  exp <- format_exp(x$exp, x$si)

  paste0(mantissa, exp)
}

#' @export
format.pillar_shaft_decimal <- function(x, width, ...) {
  if (is.null(x$dec) || width < get_width(x$dec)) {
    fmt <- x$sci
  } else {
    fmt <- x$dec
  }

  if (length(fmt$num) == 0L) {
    return(character())
  }

  if (width < get_min_width(fmt)) {
    stop(
      "Need at least width ", get_min_width(x), ", requested ", width, ".",
      call. = FALSE
    )
  }

  row <- assemble_decimal(fmt)

  used_width <- get_max_extent(row)
  row <- paste0(strrep(" ", width - used_width), row)
  new_ornament(row, width = width, align = "right")
}

split_decimal_3 <- function(x, sigfig = 3, ...) {
  split_decimal(x, sigfig = sigfig, ...)
}

test_that("compute_rhs_digits() works", {
  x <- c(NA, NaN, Inf, 0, 1, 100, 1e10, 0.001, 1e-14)
  expect_equal(
    compute_rhs_digits(x, 3),
    c(0, 0, 0, 0, 0, 0, 0, 3, 14)
  )
  expect_equal(
    compute_rhs_digits(x + 1e-8, 3),
    c(0, 0, 0, 8, 2, 0, 0, 5, 10)
  )
  expect_equal(
    compute_rhs_digits(x, 7),
    c(0, 0, 0, 0, 0, 0, 0, 3, 14)
  )
  expect_equal(
    compute_rhs_digits(x, 7 + 1e-8),
    c(0, 0, 0, 0, 6, 4, 0, 9, 20)
  )
})

test_that("compute_exp() returns NA if not relevant", {
  x <- c(NA, NaN, Inf, 0, 1, 100, 0.001)
  expect_equal(compute_exp(x, 6), c(NA, NA, NA, NA, 0, 2, -3))
})

test_that("compute_exp() respectis significant digits", {
  x <- c(0.9, 0.99, 0.999, 0.99949, 0.9995, 0.99951, 0.9999, 0.99999, 0.999999)
  expect_equal(compute_exp(x, 3), c(-1, -1, -1, -1, 0, 0, 0, 0, 0))
})

test_that("special values appear in LHS", {
  x <- c(NA, NaN, Inf)
  f <- split_decimal_3(x)

  expect_equal(format_lhs(f), format(x, trim = TRUE))
})

test_that("all-positive values get nothing in neg", {
  f <- split_decimal_3(c(0, Inf))
  expect_equal(format_neg(f), c("", ""))
})

test_that("negative values get - in neg", {
  f <- split_decimal_3(c(-Inf, Inf))
  expect_equal(format_neg(f), c("-", ""))
})

test_that("trailing zeros removed if whole decimal fraction", {
  f <- split_decimal_3(c(1.5, 0.5))
  expect_equal(format_lhs(f), c("1", "0"))
  expect_equal(format_rhs(f), c("5", "5"))
})

test_that("trailing zeros pad to sigfigs if data not shown", {
  f <- split_decimal_3(c(1.5, 0.5) + 1e-6)
  expect_equal(format_lhs(f), c("1", "0"))
  expect_equal(format_rhs(f), c("50 ", "500"))
})

test_that("sigfigs split between lhs and rhs", {
  x <- c(1.43, 10.43, 100.43)
  f <- split_decimal_3(x)

  expect_equal(format_lhs(f), as.character(trunc(x)))
  expect_equal(format_rhs(f), c("43", "4 ", "  "))
})

test_that("leading 0 added to rhs", {
  f <- split_decimal_3(1.01)

  expect_equal(format_lhs(f), "1")
  expect_equal(format_rhs(f), "01")
})

test_that("values rounded up as expect", {
  f <- split_decimal_3(c(18.9, 18.99))

  expect_equal(format_lhs(f), c("18", "19"))
  expect_equal(format_rhs(f), c("9", "0"))
})

test_that("values on LHS not rounded", {
  f <- format_lhs(split_decimal(123456, 3))
  expect_equal(f, "123456")
})

test_that("corner cases", {
  expect_equal(format_lhs(split_decimal_3(numeric())), character())
})

test_that("output test", {
  expect_snapshot({
    pillar(add_special((10^(-3:4)) * c(-1, 1)))
    pillar(add_special(1.23456 * 10^(-3:3)))
  })
  withr::with_options(
    list(pillar.sigfig = 5),
    expect_snapshot({
      pillar(add_special((10^(c(-2:4, 0L))) * c(-1, 1)))
    })
  )
  withr::with_options(
    list(pillar.sigfig = 7),
    expect_snapshot({
      pillar(add_special((10^(0:-5)) * c(-1, 1)), width = 20)
    })
  )
  expect_snapshot({
    pillar(add_special((10^(4:-4)) + 1e-6), width = 20)
  })
})

expect_decimal_width <- function(x) {
  get_formatted_width <- function(x) {
    get_max_extent(assemble_decimal(x))
  }

  expect_equal(
    get_formatted_width(split_decimal(!!x, 3)),
    !!get_width(split_decimal(x, 3))
  )
}

test_that("width computation", {
  expect_decimal_width(c(1, 10, 100))
  expect_decimal_width(c(0, NA))
  expect_decimal_width(c(1, NaN))
  expect_decimal_width(c(-12, 3))
  expect_decimal_width(c(-1, 23))
  expect_decimal_width(c(1.01, 10.1))
  expect_decimal_width(c(1.01, -10.1))
  expect_decimal_width(c(NA_integer_, NA_integer_))
  expect_decimal_width(c(1.234, 1.2345))
  expect_decimal_width(c(1.2, -Inf))
  expect_decimal_width(c(1, Inf))
})

#' Scale that supports formatted numbers
#'
#' @description
#'  `r lifecycle::badge("experimental")`
#'
#' This scale is used by default in ggplot2 with columns created with [num()].
#'
#' @inheritDotParams ggplot2::continuous_scale
#' @param guide,position Passed on to [ggplot2::continuous_scale()]
#' @param rescaler,super Must remain `NULL`.
#'
#' @keywords internal
#' @export
scale_x_num <- function(..., position = "bottom", guide = ggplot2::waiver(),
                        rescaler = NULL, super = NULL) {
  stopifnot(is.null(rescaler))
  stopifnot(is.null(super))
  stopifnot(is_installed("ggplot2"))
  ggplot2::continuous_scale(
    c(
      "x", "xmin", "xmax", "xend", "xintercept", "xmin_final", "xmax_final",
      "xlower", "xmiddle", "xupper"
    ),
    "position_c", identity,
    ...,
    guide = guide,
    position = position,
    rescaler = scales::rescale,
    super = MakeScaleContinuousPositionNum()
  )
}

#' @rdname scale_x_num
#' @export
scale_y_num <- function(..., guide = ggplot2::waiver(),
                        rescaler = NULL, super = NULL) {
  stopifnot(is.null(rescaler))
  stopifnot(is.null(super))
  stopifnot(is_installed("ggplot2"))
  ggplot2::continuous_scale(
    c(
      "y", "ymin", "ymax", "yend", "yintercept", "ymin_final", "ymax_final",
      "lower", "middle", "upper"
    ),
    "position_c", identity,
    ...,
    guide = guide,
    rescaler = scales::rescale,
    super = MakeScaleContinuousPositionNum()
  )
}

MakeScaleContinuousPositionNum <- function() {
  ggplot2::ggproto("ScaleContinuousPositionNum", ggplot2::ScaleContinuousPosition,
    ptype = NULL,
    train = function(self, x) {
      self$ptype <- vec_ptype2(x, self$ptype)
      ggplot2::ggproto_parent(ggplot2::ScaleContinuousPosition, self)$train(x)
    },
    get_breaks = function(self, limits = self$get_limits()) {
      out <- ggplot2::ggproto_parent(ggplot2::ScaleContinuousPosition, self)$get_breaks(limits)
      vec_cast(out, self$ptype)
    },
    get_labels = function(self, breaks = self$get_breaks()) {
      out <- ggplot2::ggproto_parent(ggplot2::ScaleContinuousPosition, self)$get_labels(breaks)
      fansi::strip_sgr(out)
    },
    make_title = function(self, title) {
      out <- ggplot2::ggproto_parent(ggplot2::ScaleContinuousPosition, self)$make_title(title)
      pillar_attr <- attr(self$ptype, "pillar", exact = TRUE)
      if (!is.null(pillar_attr$fixed_exponent)) {
        shaft <- pillar_shaft_number_attr(numeric(), pillar_attr)
        if (pillar_attr$notation == "si") {
          type <- attr(shaft, "type")
          if (!is.null(type)) {
            out <- paste0(out, " ", cli::ansi_strip(type[[1]]))
          }
        } else {
          # paste0() doesn't work here, paste() works like paste0()
          out <- quo_squash(quo(paste(!!out, " [", 10^!!shaft$sci$unit, "]")))
        }
      } else {
        label <- pillar_attr$label
        if (!is.null(label)) {
          out <- paste0(out, " [", label, "]")
        }
      }
      out
    }
  )
}

# registered in .onLoad()
scale_type.pillar_num <- function(x) c("num", "continuous")

# registered in .onLoad()
rescale.pillar_num <- function(x,
                               to = c(0, 1),
                               from = range(x, na.rm = TRUE, finite = TRUE),
                               ...) {
  out <- scales::rescale(vec_data(x), to, from, ...)
  vec_cast(out, x)
}

#' Format dimensions
#'
#' Multi-dimensional objects are formatted as `a x b x ...`, for vectors the
#' length is returned.
#'
#' @param x The object to format the dimensions for
#'
#' @export
#' @examples
#' dim_desc(1:10)
#' dim_desc(Titanic)
dim_desc <- function(x) {
  dim <- dim2(x)
  format_dim <- big_mark(dim)
  format_dim[is.na(dim)] <- "??"
  paste0(format_dim, collapse = spaces_around(mult_sign()))
}

# https://github.com/r-lib/vctrs/pull/467
dim2 <- function(x) {
  dim(x) %||% vec_size(x)
}

mult_sign <- function() {
  symbol$times
}

spaces_around <- function(x) {
  paste0(" ", x, " ")
}

# function for the thousand separator,
# returns "," unless it's used for the decimal point, in which case returns "."
big_mark <- function(x) {
  # The thousand separator,
  # "," unless it's used for the decimal point, in which case "."
  mark <- if (identical(getOption("OutDec"), ",")) "." else ","
  ret <- formatC(x, big.mark = mark, format = "d", preserve.width = "individual")
  ret[is.na(x)] <- "??"
  ret
}

test_that("dim2() falls back to vec_size()", {
  expect_equal(dim2(1:3), 3)
  expect_equal(dim2(Titanic), dim(Titanic))
})

test_that("Using perceived dimensions (#167)", {
  expect_equal(dim_desc(as.POSIXlt(Sys.time() + 1:10)), "10")
  expect_equal(dim_desc(as.POSIXlt(Sys.time() + 1:17)), "17")
})

test_that("big_mark() works for large numbers", {
  expect_match(big_mark(123), "123")
  expect_match(big_mark(123456), "123.456")
  expect_match(big_mark(123456789), "123.456.789")
})

#' Format the body of a tibble
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' For easier customization, the formatting of a tibble is split
#' into three components: header, body, and footer.
#' The `tbl_format_body()` method is responsible for formatting the body
#' of a tibble.
#'
#' Override this method if you need to change the appearance of all parts
#' of the body.
#' If you only need to change the appearance of a single data type,
#' override [vctrs::vec_ptype_abbr()] and [pillar_shaft()] for this data type.
#'
#' @inheritParams rlang::args_dots_empty
#' @param x A tibble-like object.
#' @param setup A setup object returned from [tbl_format_setup()].
#'
#' @return
#'   A character vector.
#'
#' @export
#' @examplesIf rlang::is_installed(c("palmerpenguins", "tibble"))
#' setup <- tbl_format_setup(palmerpenguins::penguins)
#' tbl_format_body(palmerpenguins::penguins, setup)
#'
#' # Shortcut for debugging
#' tbl_format_body(setup)
tbl_format_body <- function(x, setup, ...) {
  check_dots_empty()

  UseMethod("tbl_format_body")
}

#' @export
tbl_format_body.tbl <- function(x, setup, ...) {
  force(setup)
  setup$body
}

#' @export
tbl_format_body.pillar_tbl_format_setup <- function(x, ...) {
  as_glue(c(
    cli::style_bold("<tbl_format_body(setup)>"),
    tbl_format_body(x$x, setup = x)
  ))
}

#' @export
pillar_shaft.pillar_vctr <- function(x, ...) {
  # still seems necessary
  pillar_shaft(unclass(x))
}

#' @export
vec_ptype_full.pillar_vctr <- function(x, ...) {
  format(attr(x, "pillar", exact = TRUE))
}

#' @export
print.pillar_vctr <- function(x, ..., max = NULL) {
  if (is.null(max)) {
    max <- getOption("max.print")
  }

  xx <- vec_head(x, max)
  size <- vec_size(x)

  obj_print_header.pillar_vctr(xx, ..., .size = size)
  obj_print_data.pillar_vctr(xx, ..., .size = size)
  obj_print_footer.pillar_vctr(xx, ..., .size = size)
  invisible(x)
}

#' @export
obj_print_header.pillar_vctr <- function(x, ..., .size) {
  writeLines(paste0("<", vec_ptype_full(x), "[", .size, "]>"))
  pillar_attr <- attr(x, "pillar")
  if (!is.null(pillar_attr$fixed_exponent)) {
    shaft <- pillar_shaft_number_attr(numeric(), pillar_attr)
    type <- attr(shaft, "type")
    if (!is.null(type)) {
      writeLines(paste0("Fixed exponent: ", type[[1]]))
    }
  }
  invisible(x)
}

#' @export
obj_print_data.pillar_vctr <- function(x, ..., .size) {
  if (length(x) == 0) {
    return(invisible(x))
  }

  # FIXME: base::print.default() can't use color, roll own implementation?
  out <- stats::setNames(strip_sgr(format(x), warn = FALSE), names(x))
  print(out, quote = FALSE, max = vec_size(x))
  invisible(x)
}

#' @export
obj_print_footer.pillar_vctr <- function(x, ..., .size) {
  delta <- .size - vec_size(x)

  if (delta <= 0) {
    return(invisible(x))
  }

  writeLines(style_subtle(pre_dots(paste0("and ", delta, " more"))))
  invisible(x)
}

#' @export
print.pillar_vctr_attr <- function(x, ...) {
  writeLines(format(x))
  invisible(x)
}

#' @export
vec_proxy_compare.pillar_vctr <- function(x, ...) {
  vec_data(x)
}
#' @export
vec_proxy_order.pillar_vctr <- function(x, ...) {
  vec_data(x)
}

pluralise <- function(message, objects) {
  pluralise_n(message, length(objects))
}

pluralise_n <- function(message, n) {
  stopifnot(n >= 0)


  # Don't strip parens if they have a space in between
  # (but not if the space comes before the closing paren)

  if (n == 1) {
    # strip [
    message <- gsub("\\[([^\\] ]* *)\\]", "\\1", message, perl = TRUE)
    # remove ( and its content
    message <- gsub("\\([^\\) ]* *\\)", "", message, perl = TRUE)
  } else {
    # strip (
    message <- gsub("\\(([^\\) ]* *)\\)", "\\1", message, perl = TRUE)
    # remove [ and its content
    message <- gsub("\\[[^\\] ]* *\\]", "", message, perl = TRUE)
  }

  message
}

#' Format the footer of a tibble
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' For easier customization, the formatting of a tibble is split
#' into three components: header, body, and footer.
#' The `tbl_format_footer()` method is responsible for formatting the footer
#' of a tibble.
#'
#' Override or extend this method if you need to change the appearance
#' of the footer.
#' The default implementation adds information about rows and columns
#' that are not shown in the body.
#'
#' @inheritParams tbl_format_body
#' @inherit tbl_format_body return
#'
#' @export
#' @examplesIf rlang::is_installed("palmerpenguins")
#' setup <- tbl_format_setup(palmerpenguins::penguins)
#' tbl_format_footer(palmerpenguins::penguins, setup)
#'
#' # Shortcut for debugging
#' tbl_format_footer(setup)
tbl_format_footer <- function(x, setup, ...) {
  check_dots_empty()

  UseMethod("tbl_format_footer")
}

#' @export
tbl_format_footer.pillar_tbl_format_setup <- function(x, ...) {
  as_glue(c(
    cli::style_bold("<tbl_format_footer(setup)>"),
    tbl_format_footer(x$x, setup = x)
  ))
}

#' @export
tbl_format_footer.tbl <- function(x, setup, ...) {
  footer <- format_footer(x, setup)
  footer_comment <- wrap_footer(footer, setup)
  footer_advice <- format_footer_advice(x, setup)
  footer_advice_comment <- wrap_footer(footer_advice, setup, lines = 1, ellipsis = FALSE)
  style_subtle(c(footer_comment, footer_advice_comment))
}

format_footer <- function(x, setup) {
  extra_rows <- format_footer_extra_rows(x, setup)
  abbrev_cols <- format_footer_abbrev_cols(x, setup)
  extra_cols <- format_footer_extra_cols(x, setup)

  footer <- compact(list(extra_rows, extra_cols, abbrev_cols))
  if (length(footer) == 0) {
    return(character())
  }

  footer_enum <- enum_collapse(footer)

  extra <- unlist(footer_enum, recursive = FALSE)

  c("with", extra)
}

format_footer_extra_rows <- function(x, setup) {
  if (ncol(setup$x) != 0) {
    if (is.na(setup$rows_missing)) {
      c("more", "rows")
    } else if (setup$rows_missing > 0) {
      c(big_mark(setup$rows_missing), "more", pluralise_n("row(s)", setup$rows_missing))
    }
  } else {
    rows_body <- nrow(setup$df)
    if (is.na(setup$rows_total) && rows_body > 0) {
      c("at", "least", big_mark(rows_body), pluralise_n("row(s)", rows_body), "total")
    }
  }
}

format_footer_abbrev_cols <- function(x, setup) {
  abbrev_cols <- setup$abbrev_cols
  abbrev_cols_total <- length(abbrev_cols)
  if (abbrev_cols_total == 0) {
    return(NULL)
  }

  abbrev_cols <- paste0(
    map_chr(seq_along(abbrev_cols), superdigit),
    pillar_options$superdigit_sep(),
    abbrev_cols
  )

  idx_all_but_last <- seq_len(abbrev_cols_total - 1)
  abbrev_cols[idx_all_but_last] <- paste0(abbrev_cols[idx_all_but_last], ",")

  c(
    "abbreviated", "variable",
    pluralise("name(s)", abbrev_cols),
    abbrev_cols
  )
}

format_footer_extra_cols <- function(x, setup) {
  extra_cols <- setup$extra_cols
  if (length(extra_cols) == 0) {
    return(NULL)
  }

  extra_cols_total <- setup$extra_cols_total

  vars <- format_extra_vars(extra_cols, extra_cols_total)
  c(
    big_mark(extra_cols_total),
    if (!identical(setup$rows_total, 0L) && nrow(setup$df) > 0) "more",
    pluralise_n("variable(s):", extra_cols_total),
    vars
  )
}

format_extra_vars <- function(extra_cols, extra_cols_total) {
  out <- imap(extra_cols, format_abbrev)

  if (extra_cols_total > length(extra_cols)) {
    out <- c(out, symbol$ellipsis)
  }

  out[-length(out)] <- paste0(out[-length(out)], ",")
  out
}

format_footer_advice <- function(x, setup) {
  if (length(setup$extra_cols) > 0) {
    cols <- "`colnames()` to see all variable names"
  } else {
    cols <- NULL
  }

  if (is.na(setup$rows_missing) || setup$rows_missing > 0) {
    rows <- "`print(n = ...)` to see more rows"
  } else {
    rows <- NULL
  }

  advice <- enum_collapse(c(rows, cols))

  if (length(advice) == 0) {
    return()
  }

  paste0(symbol$info, " Use ", paste(advice, collapse = " "))
}

wrap_footer <- function(footer, setup, lines = setup$max_footer_lines, ellipsis = TRUE) {
  if (length(footer) == 0) {
    return(character())
  }

  # When asking for width = 80, use at most 79 characters
  max_extent <- setup$width - 1L

  tier_widths <- get_footer_tier_widths(
    footer, max_extent, lines, ellipsis
  )

  # show optuput even if too wide
  widths <- pmin(get_extent(footer), max_extent - 4L)
  wrap <- colonnade_compute_tiered_col_widths_df(widths, widths, tier_widths)

  # truncate output that doesn't fit
  truncated <- anyNA(wrap$tier)
  split <- split(footer[wrap$id], wrap$tier)
  if (ellipsis && truncated && length(split) > 0) {
    split[[length(split)]] <- c(split[[length(split)]], symbol$ellipsis)
  }
  split <- imap(split, function(.x, .y) {
    if (!ellipsis) {
      header <- NULL
    } else if (.y == 1) {
      header <- symbol$ellipsis
    } else {
      header <- " "
    }
    c("#", header, .x)
  })

  map_chr(split, paste, collapse = " ")
}

get_footer_tier_widths <- function(footer, max_extent, n_tiers, ellipsis = TRUE) {
  if (ellipsis) {
    extra_width <- get_extent(symbol$ellipsis) + 1L # space, ellipsis
  } else {
    extra_width <- FALSE
  }

  n_tiers <- min(length(footer), n_tiers)

  if (n_tiers == 1) {
    max_extent - 2 - 2 * extra_width
  } else {
    c(
      max_extent - 2 - extra_width,
      rep(max_extent - 4, n_tiers - 2),
      max_extent - 4 - extra_width
    )
  }
}

pre_dots <- function(x) {
  if (length(x) > 0) {
    paste0(symbol$ellipsis, " ", x)
  } else {
    character()
  }
}

collapse <- function(x) paste(x, collapse = ", ")

split_lines <- function(x) {
  # Avoid .ptype argument to vec_c()
  if (is_empty(x)) {
    return(character())
  }

  unlist(strsplit(x, "\n", fixed = TRUE))
}

enum_collapse <- function(x) {
  if (length(x) > 1) {
    x_len <- length(x)
    idx_all_but_last <- seq2(1, x_len - 1)
    x[idx_all_but_last] <- map(x[idx_all_but_last], function(x) {
      x[[length(x)]] <- paste0(x[[length(x)]], ",")
      x
    })
    x <- c(x[idx_all_but_last], "and", x[x_len])
  }

  x
}

# nocov start
keep_empty <- function(fun) {
  function(x) {
    ret <- rep_along(x, "")
    update <- which(is.na(x) | x != "")
    ret[update] <- fun(x[update])
    ret
  }
}
# nocov end

#' Styling helpers
#'
#' Functions that allow implementers of formatters for custom data types to
#' maintain a consistent style with the default data types.
#'
#' `style_subtle()` is affected by the `subtle` [option][pillar_options].
#'
#' @param x The character vector to style.
#' @export
#' @seealso [pillar_options] for a list of options
#' @examples
#' style_subtle("text")
style_subtle <- keep_empty(function(x) {
  force(x)
  if (isTRUE(get_pillar_option_subtle())) {
    crayon_grey_0.6(x)
  } else {
    x
  }
})

#' @rdname style_subtle
#' @details
#' `style_subtle_num()` is affected by the
#' `subtle_num` [option][pillar_options],
#' which is `FALSE` by default.
#'
#' @export
#' @examples
#' style_subtle_num(0.01 * 1:3, c(TRUE, FALSE, TRUE))
style_subtle_num <- function(x, negative) {
  if (isTRUE(get_pillar_option_subtle_num())) {
    style_subtle(x)
  } else {
    ifelse(negative, style_neg(x), x)
  }
}

style_hint <- keep_empty(function(x) {
  force(x)
  if (isTRUE(get_pillar_option_subtle())) {
    crayon_grey_0.8(x)
  } else {
    x
  }
})

style_spark_na <- function(x) {
  crayon_yellow(x)
}

#' @details
#' `style_bold()` is affected by the `bold` [option][pillar_options],
#' which is `FALSE` by default.
#'
#' @rdname style_subtle
#' @export
#' @examples
#' style_bold("Petal.Width")
style_bold <- keep_empty(function(x) {
  if (isTRUE(get_pillar_option_bold())) {
    crayon_bold(x)
  } else {
    x
  }
})

#' @rdname style_subtle
#' @export
#' @examples
#' style_na("NA")
style_na <- function(x) {
  crayon_red(x)
}

#' @details
#' `style_neg()` is affected by the `pillar.neg` [option][pillar_options].
#'
#' @rdname style_subtle
#' @export
#' @examples
#' style_neg("123")
style_neg <- keep_empty(function(x) {
  if (isTRUE(get_pillar_option_neg())) {
    crayon_red(x)
  } else {
    x
  }
})

pillar_na <- function(use_brackets_if_no_color = FALSE) {
  if (use_brackets_if_no_color && !has_color()) {
    "<NA>"
  } else {
    style_na("NA")
  }
}

style_list <- function(x) {
  style_subtle(x)
}

# Only check if we have color support once per session
num_colors <- local({
  num_colors <- NULL
  function(forget = FALSE) {
    if (is.null(num_colors) || forget) {
      num_colors <<- cli::num_ansi_colors()
    }
    num_colors
  }
})

has_color <- function() {
  num_colors() > 1
}

# nocov start
# Crayon functions call cli::num_ansi_colors() every call
make_style_fast <- function(...) {
  # Force has_color to be true when making styles
  # Use prefix to avoid problems with empty NAMESPACE
  rlang::local_options(cli.num_colors = 16L)

  style_16 <- attr(cli::make_ansi_style(..., colors = 16), "_styles")[[1]]
  open_16 <- style_16$open
  close_16 <- style_16$close

  style_256 <- attr(cli::make_ansi_style(..., colors = 256), "_styles")[[1]]
  open_256 <- style_256$open
  close_256 <- style_256$close

  function(...) {
    colors <- num_colors()
    if (colors <= 1) {
      paste0(...)
    } else if (colors >= 256) {
      paste0(open_256, ..., close_256)
    } else {
      paste0(open_16, ..., close_16)
    }
  }
}

# Placeholders, assigned in .onLoad()
crayon_underline <- function(...) {}
crayon_italic <- function(...) {}
crayon_red <- function(...) {}
crayon_yellow <- function(...) {}
crayon_bold <- function(...) {}
crayon_grey_0.6 <- function(...) {}
crayon_grey_0.8 <- function(...) {}

assign_crayon_styles <- function() {
  crayon_underline <<- make_style_fast("underline")
  crayon_italic <<- make_style_fast("italic")
  crayon_red <<- make_style_fast("red")
  crayon_yellow <<- make_style_fast("yellow")
  crayon_bold <<- make_style_fast("bold")
  crayon_grey_0.6 <<- make_style_fast(grDevices::grey(0.6), grey = TRUE)
  crayon_grey_0.8 <<- make_style_fast(grDevices::grey(0.8), grey = TRUE)
}
# nocov end

as_unknown_rows <- function(x) {
  x <- as_tbl(x)
  class(x) <- c("unknown_rows", "tbl")
  x
}

local_unknown_rows <- function(frame = caller_env()) {
  local_methods(
    .frame = frame,
    as.data.frame.unknown_rows = function(x, ...) {
      class(x) <- "data.frame"
      x
    },
    dim.unknown_rows = function(x) {
      c(NA_integer_, length(x))
    },
    head.unknown_rows = function(x, n) {
      class(x) <- "data.frame"
      head(x, n)
    }
  )
}

# Format numbers in decimal notation
#
# This formatting system is designed to make it as easy as possible to
# compare columns of numbers. Significant digits are coloured black or red
# (for positive and negative numbers) and non-significant digits are coloured
# in paler gray.
#
# @return A list with at least the following elements:
# * `neg`: negative sign or space, `TRUE` if needed
# * `lhs`: whole number
# * `dec`: decimal point, `TRUE` if needed
# * `rhs`: remainder of number
#
# @param x A numeric vector
# @param sigfig Number of significant figures to display.
# @param digits Number of digits after the decimal point, incompatible with
#   `sigfig`. A negative number means the maximum number of digits to display.
# @param sci_mod Set to 1 for scientific or to 3 for engineering display.
# @param si Set to `TRUE` for SI notation, requires `sci_mod = 3`
# @param fixed Set to `TRUE` to use the same exponent everywhere.
# @seealso [split_decimal()]
# @examples
# split_decimal(1.5:3.5)
# split_decimal(1.5:3.5, sci_mod = 1)
# split_decimal(1e9)
# split_decimal(1e9, sci_mod = 1)
split_decimal <- function(x, sigfig, digits = NULL, sci_mod = NULL, si = FALSE,
                          fixed_exponent = NULL) {
  "!!!!!!DEBUG split_decimal(`v(x)`, `v(sigfig)`, `v(digits)`, `v(sci_mod)`, `v(si)`, `v(fixed_exponent)`"

  stopifnot(is.numeric(x))
  sigfig <- check_sigfig(sigfig)

  num <- is.finite(x)
  "!!!!!!DEBUG `v(num)`"
  dec <- num

  # Do we need negative signs?
  neg <- !is.na(x) & x < 0
  "!!!!!!DEBUG `v(neg)`"

  abs_x <- abs(x)
  mnt <- abs_x
  "!!!!!!DEBUG `v(mnt)`"

  if (!is.null(sci_mod)) {
    exp <- fix_exp(num, compute_exp(mnt, sigfig, digits), fixed_exponent, sci_mod, si)
    "!!!!!!DEBUG `v(exp)`"
    unit <- attr(exp, "unit")

    # Must divide by 10^exp, because 10^-exp may not be representable
    # for very large values of exp
    mnt_idx <- which(num & mnt != 0)
    mnt[mnt_idx] <- safe_divide_10_to(mnt[mnt_idx], exp[mnt_idx])
    "!!!!!!DEBUG `v(mnt)`"
  } else {
    exp <- 0
    "!!!!!!DEBUG `v(exp)`"
    unit <- NULL
  }

  if (is.null(sci_mod) || !is.null(fixed_exponent)) {
    exp_display <- rep_along(x, NA_integer_)
  } else {
    exp_display <- exp
  }

  if (is.null(digits)) {
    "!!!!!!DEBUG `v(sigfig)`"
    min_sigfig <- compute_min_sigfig(mnt)
    round_mnt <- safe_signif(mnt, pmax(sigfig, min_sigfig, na.rm = TRUE))
    rhs_digits <- compute_rhs_digits(mnt, sigfig)
  } else if (digits >= 0) {
    "!!!!!!DEBUG `v(digits)`"
    round_mnt <- round(mnt, digits)
    rhs_digits <- digits
  } else {
    "!!!!!!DEBUG `v(-digits)`"
    round_mnt <- round(mnt, -digits)
    trunc_mnt <- trunc(mnt)
    rhs_digits <- compute_rhs_digits(mnt - trunc_mnt, -digits, trunc_mnt)
  }

  "!!!!!!DEBUG `v(round_mnt)`"
  "!!!!!!DEBUG `v(rhs_digits)`"

  lhs <- trunc(round_mnt)
  "!!!!!!DEBUG `v(lhs)`"

  rhs <- round_mnt - lhs
  "!!!!!!DEBUG `v(rhs)`"

  if (is.null(digits) || digits < 0) {
    "!!!!!!DEBUG `v(lhs * 10^exp - abs_x)`"
    reset_dec <- (mnt == 0 | (rhs == 0 & within_tolerance(lhs * 10^exp, abs_x)))
    "!!!!!!DEBUG `v(reset_dec)`"

    dec[reset_dec] <- FALSE
    "!!!!!!DEBUG `v(dec)`"
  }

  ret <- list(
    sigfig = sigfig,
    num = num,
    neg = neg,
    # integer64 doesn't support format(trim = FALSE)
    # trimws() is unnecessarily slow
    lhs = sub("^ +", "", format(lhs, scientific = FALSE)),
    lhs_zero = (lhs == 0),
    rhs = rhs,
    rhs_digits = rhs_digits,
    dec = dec,
    exp = exp_display,
    unit = unit,
    si = si
  )

  set_width(ret, get_decimal_width(ret))
}

fix_exp <- function(num, exp, fixed_exponent, sci_mod, si) {
  "!!!!!!DEBUG fix_exp(`v(num)`, `v(exp)`, `v(fixed_exponent)`, `v(sci_mod)`, `v(si)`"
  if (!is.null(fixed_exponent)) {
    if (is.finite(fixed_exponent)) {
      exp <- fixed_exponent
    } else if (all(is.na(exp))) {
      exp <- NA_real_
    } else if (fixed_exponent < 0) {
      exp <- min(exp, na.rm = TRUE)
    } else {
      exp <- max(exp, na.rm = TRUE)
    }
    "!!!!!!DEBUG `v(exp)`"
    exp <- as.integer(round(exp))
    exp <- structure(rep_along(num, exp), unit = exp %|% 0L)
    exp[!num] <- NA_integer_
    "!!!!!!DEBUG `v(exp)`"
  }

  if (sci_mod != 1) {
    exp[] <- as.integer(round(floor(exp / sci_mod) * sci_mod))
    "!!!!!!DEBUG `v(exp)`"
  }
  if (si) {
    # Truncate very small and very large exponents
    exp[] <- pmin(pmax(exp, -24L), 24L)
    "!!!!!!DEBUG `v(exp)`"
  }

  exp
}

get_decimal_width <- function(x) {
  exp <- x$exp[!is.na(x$exp)]

  if (x$si) {
    exp_digits <- any(exp != 0)
  } else {
    exp_digits <- any(exp < 0) + max(2 + trunc(log10(abs(exp) + 0.5)), 0)
  }

  max(x$neg + nchar(x$lhs), 0) +
    any(x$dec, na.rm = TRUE) +
    max(x$rhs_digits, 0) +
    exp_digits
}

safe_signif <- function(x, digits) {
  if (length(x) == 0L) {
    return(numeric())
  }
  signif(x, digits)
}

safe_divide_10_to <- function(x, y) {
  # Computes x / 10^y in a robust way

  x / (10^y)
}

eps_2 <- 2 * .Machine$double.eps

within_tolerance <- function(x, y) {
  "!!!!!!DEBUG within_tolerance(`v(x)`, `v(y)`)"
  l2x <- round(log2(x))
  "!!!!!!DEBUG `v(l2x)`"
  l2y <- round(log2(y))
  "!!!!!!DEBUG `v(l2y)`"

  equal <- (l2x == l2y)
  equal[is.na(equal)] <- FALSE
  out <- equal

  # Work around integer64 problem
  equal[x == y] <- FALSE
  "!!!!!!DEBUG `v(abs((x[equal] - y[equal]) * 2 ^ -l2x[equal]))`"
  out[equal] <- abs((x[equal] - y[equal]) * 2^-l2x[equal]) <= eps_2
  out
}

compute_rhs_digits <- function(x, sigfig, offset = rep_along(x, 0)) {
  "!!!!!!DEBUG compute_rhs_digits(`v(x)`, `v(sigfig)`)"
  # If already bigger than sigfig, can round to zero.
  # Otherwise ensure we have sigfig digits shown
  exp <- compute_exp(x, sigfig)
  exp[is.na(exp)] <- Inf
  "!!!!!!DEBUG `v(exp)"
  rhs_digits <- rep_along(x, 0)
  "!!!!!!DEBUG `v(rhs_digits)"

  if (!is.integer(x) && !all(x == trunc(x), na.rm = TRUE)) {
    has_rhs <- (exp <= sigfig)
    rhs_digits[has_rhs] <- sigfig - 1 - exp[has_rhs]

    to_check <- rhs_digits > 0
    while (any(to_check, na.rm = TRUE)) {
      "!!!!!!DEBUG `v(to_check)"
      "!!!!!!DEBUG `v(rhs_digits)"

      which_to_check <- which(to_check)
      val <- (x[which_to_check] + offset[which_to_check]) * 10^(rhs_digits[which_to_check] - 1)
      "!!!!!!DEBUG `v(val)"
      "!!!!!!DEBUG `v(val - round(val))"

      resid_zero <- within_tolerance(val, round(val))
      resid_zero[is.na(resid_zero)] <- FALSE

      rhs_digits[which_to_check][resid_zero] <-
        rhs_digits[which_to_check][resid_zero] - 1

      to_check[which_to_check][!resid_zero] <- FALSE
      to_check[rhs_digits == 0] <- FALSE
    }
  }

  "!!!!!!DEBUG `v(rhs_digits)"
  rhs_digits
}

compute_min_sigfig <- function(x) {
  ret <- rep_along(x, NA_integer_)
  nonzero <- which(x != 0 & is.finite(x))
  ret[nonzero] <- as.integer(floor(log10(x[nonzero]))) + 1L
  ret
}

compute_extra_sigfig <- function(x) {
  x <- sort(abs(x))
  delta <- diff(x)
  x <- x[-1]

  keep <- which((delta != 0) & is.finite(delta))
  if (length(keep) == 0) {
    return(0)
  }

  ceiling(log10(max(x[keep] / delta[keep]))) - 1
}

LOG_10 <- log(10)

compute_exp <- function(x, sigfig, digits) {
  if (is.null(sigfig)) {
    sigfig <- abs(digits)
  }

  # With 3 significant digits:
  # 0.9994 -> 0.999 -> exp == -1
  # 0.9995 -> 1.00 -> exp == 0
  # This means that x is divided by 0.9995 in this example
  # before computing log10().
  # Division before log is the same as subtraction after log.
  # Using log1p for numerical stability.
  offset <- log1p(-5 * 10^(-sigfig - 1)) / LOG_10

  ret <- rep_along(x, NA_integer_)
  nonzero <- which(x != 0 & is.finite(x))
  ret[nonzero] <- as.integer(floor(log10(x[nonzero]) - offset))
  ret
}

format_mantissa <- function(x) {
  lhs <- format_lhs(x)
  dec <- format_dec(x)
  rhs <- format_rhs(x)
  paste0(lhs, dec, rhs)
}

format_neg <- function(s) {
  neg <- s$neg
  neg_col <- ifelse(neg, "-", "")
  neg_col
}

format_lhs <- function(s) {
  neg <- s$neg
  num <- s$num
  lhs_zero <- s$lhs_zero

  lhs_str <- s$lhs
  lhs_split <- strsplit(lhs_str, "", fixed = TRUE)
  lhs_width <- lengths(lhs_split)

  lhs_split_underlined <- map(lhs_split, underline_3_back)

  lhs_sig <- map(pmin(lhs_width, s$sigfig), seq_len)
  lhs_insig <- map(lhs_sig, `-`)

  lhs_split_sig <- map2(lhs_split_underlined, lhs_sig, `[`)
  lhs_split_non <- map2(lhs_split_underlined, lhs_insig, `[`)

  lhs_sig <- map_chr(lhs_split_sig, paste, collapse = "")
  lhs_non <- map_chr(lhs_split_non, paste, collapse = "")

  # as.character() to support corner case of length zero
  lhs_col <- as.character(ifelse(num,
    paste0(
      style_num(lhs_sig, neg, !lhs_zero),
      style_subtle_num(lhs_non, neg)
    ),
    style_na(lhs_str)
  ))

  lhs_col <- paste0(format_neg(s), lhs_col)
  # No alignment here, result needs to be right-aligned
  lhs_col
}

underline_3_back <- function(x) {
  idx <- which(trunc((seq_along(x) - length(x)) / 3) %% 2 == 1)
  x[idx] <- crayon_underline(x[idx])
  x
}

format_dec <- function(s) {
  neg <- s$neg
  dec <- s$dec
  lhs_zero <- s$lhs_zero

  # Decimal column
  if (any(dec)) {
    dec_col <- ifelse(dec, style_num(".", neg, !lhs_zero), " ")
  } else {
    dec_col <- rep_along(neg, "")
  }
  dec_col
}

format_rhs <- function(s) {
  neg <- s$neg
  dec <- s$dec
  lhs_zero <- s$lhs_zero
  rhs_num <- s$rhs_num
  rhs_digits <- s$rhs_digits

  # Digits on RHS of .
  rhs_num <- sprintf("%.0f", abs(round(s$rhs * 10^(s$rhs_digits))))
  rhs_num[rhs_num == "0"] <- ""

  n_zeros <- pmax(0, rhs_digits - get_extent(rhs_num))
  rhs_zero <- strrep("0", n_zeros)

  rhs_split <- strsplit(paste0(rhs_zero, rhs_num), "", fixed = TRUE)
  rhs_split_underlined <- map(rhs_split, underline_3)

  rhs_is_zero <- map(n_zeros, seq_len)
  rhs_is_nonzero <- map2(n_zeros + 1L, rhs_digits, seq2)

  rhs_split_underlined_zero <- map2(rhs_split_underlined, rhs_is_zero, `[`)
  rhs_split_underlined_num <- map2(rhs_split_underlined, rhs_is_nonzero, `[`)

  rhs_underlined_zero <- map_chr(rhs_split_underlined_zero, paste, collapse = "")
  rhs_underlined_num <- map_chr(rhs_split_underlined_num, paste, collapse = "")

  rhs_col <- ifelse(dec,
    paste0(
      style_num(rhs_underlined_zero, neg, !lhs_zero),
      style_num(rhs_underlined_num, neg)
    ),
    ""
  )

  # ensure all same width
  rhs_col <- align(rhs_col, max(rhs_digits, 0L, na.rm = TRUE), "left")

  rhs_col
}

underline_3 <- function(x) {
  idx <- which(trunc((seq_along(x) - 1) / 3) %% 2 == 1)
  x[idx] <- crayon_underline(x[idx])
  x
}

#' @export
#' @param negative,significant Logical vector the same length as `x` that
#'   indicate if the values are negative and significant, respectively
#' @rdname style_subtle
#' @examples
#' style_num(
#'   c("123", "456"),
#'   negative = c(TRUE, FALSE)
#' )
#' style_num(
#'   c("123", "456"),
#'   negative = c(TRUE, FALSE),
#'   significant = c(FALSE, FALSE)
#' )
style_num <- function(x, negative, significant = rep_along(x, TRUE)) {
  ifelse(significant, ifelse(negative, style_neg(x), x), style_subtle_num(x, negative))
}

assemble_decimal <- function(x) {
  mantissa <- format_mantissa(x)
  exp <- format_exp(x$exp, x$si)

  paste0(mantissa, exp)
}

#' @export
format.pillar_shaft_decimal <- function(x, width, ...) {
  if (is.null(x$dec) || width < get_width(x$dec)) {
    fmt <- x$sci
  } else {
    fmt <- x$dec
  }

  if (length(fmt$num) == 0L) {
    return(character())
  }

  if (width < get_min_width(fmt)) {
    stop(
      "Need at least width ", get_min_width(x), ", requested ", width, ".",
      call. = FALSE
    )
  }

  row <- assemble_decimal(fmt)

  used_width <- get_max_extent(row)
  row <- paste0(strrep(" ", width - used_width), row)
  new_ornament(row, width = width, align = "right")
}

#' Format the body of a tibble
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' For easier customization, the formatting of a tibble is split
#' into three components: header, body, and footer.
#' The `tbl_format_body()` method is responsible for formatting the body
#' of a tibble.
#'
#' Override this method if you need to change the appearance of all parts
#' of the body.
#' If you only need to change the appearance of a single data type,
#' override [vctrs::vec_ptype_abbr()] and [pillar_shaft()] for this data type.
#'
#' @inheritParams rlang::args_dots_empty
#' @param x A tibble-like object.
#' @param setup A setup object returned from [tbl_format_setup()].
#'
#' @return
#'   A character vector.
#'
#' @export
#' @examplesIf rlang::is_installed("palmerpenguins")
#' setup <- tbl_format_setup(palmerpenguins::penguins)
#' tbl_format_body(palmerpenguins::penguins, setup)
#'
#' # Shortcut for debugging
#' tbl_format_body(setup)
tbl_format_body <- function(x, setup, ...) {
  check_dots_empty()

  UseMethod("tbl_format_body")
}

#' @export
tbl_format_body.tbl <- function(x, setup, ...) {
  force(setup)
  setup$body
}

#' @export
tbl_format_body.pillar_tbl_format_setup <- function(x, ...) {
  as_glue(c(
    cli::style_bold("<tbl_format_body(setup)>"),
    tbl_format_body(x$x, setup = x)
  ))
}

# nocov start - compat-purrr (last updated: rlang 0.3.2.9000)

# This file serves as a reference for compatibility functions for
# purrr. They are not drop-in replacements but allow a similar style
# of programming. This is useful in cases where purrr is too heavy a
# package to depend on. Please find the most recent version in rlang's
# repository.

map <- function(.x, .f, ...) {
  lapply(.x, .f, ...)
}
map_mold <- function(.x, .f, .mold, ...) {
  out <- vapply(.x, .f, .mold, ..., USE.NAMES = FALSE)
  names(out) <- names(.x)
  out
}
map_lgl <- function(.x, .f, ...) {
  map_mold(.x, .f, logical(1), ...)
}
map_int <- function(.x, .f, ...) {
  map_mold(.x, .f, integer(1), ...)
}
map_dbl <- function(.x, .f, ...) {
  map_mold(.x, .f, double(1), ...)
}
map_chr <- function(.x, .f, ...) {
  map_mold(.x, .f, character(1), ...)
}
map_cpl <- function(.x, .f, ...) {
  map_mold(.x, .f, complex(1), ...)
}

walk <- function(.x, .f, ...) {
  map(.x, .f, ...)
  invisible(.x)
}

pluck <- function(.x, .f) {
  map(.x, `[[`, .f)
}
pluck_lgl <- function(.x, .f) {
  map_lgl(.x, `[[`, .f)
}
pluck_int <- function(.x, .f) {
  map_int(.x, `[[`, .f)
}
pluck_dbl <- function(.x, .f) {
  map_dbl(.x, `[[`, .f)
}
pluck_chr <- function(.x, .f) {
  map_chr(.x, `[[`, .f)
}
pluck_cpl <- function(.x, .f) {
  map_cpl(.x, `[[`, .f)
}

map2 <- function(.x, .y, .f, ...) {
  out <- mapply(.f, .x, .y, MoreArgs = list(...), SIMPLIFY = FALSE)
  if (length(out) == length(.x)) {
    set_names(out, names(.x))
  } else {
    set_names(out, NULL)
  }
}
map2_lgl <- function(.x, .y, .f, ...) {
  as.vector(map2(.x, .y, .f, ...), "logical")
}
map2_int <- function(.x, .y, .f, ...) {
  as.vector(map2(.x, .y, .f, ...), "integer")
}
map2_dbl <- function(.x, .y, .f, ...) {
  as.vector(map2(.x, .y, .f, ...), "double")
}
map2_chr <- function(.x, .y, .f, ...) {
  as.vector(map2(.x, .y, .f, ...), "character")
}
map2_cpl <- function(.x, .y, .f, ...) {
  as.vector(map2(.x, .y, .f, ...), "complex")
}

args_recycle <- function(args) {
  lengths <- map_int(args, length)
  n <- max(lengths)

  stopifnot(all(lengths == 1L | lengths == n))
  to_recycle <- lengths == 1L
  args[to_recycle] <- map(args[to_recycle], function(x) rep.int(x, n))

  args
}
pmap <- function(.l, .f, ...) {
  args <- args_recycle(.l)
  do.call("mapply", c(
    FUN = list(quote(.f)),
    args, MoreArgs = quote(list(...)),
    SIMPLIFY = FALSE, USE.NAMES = FALSE
  ))
}

probe <- function(.x, .p, ...) {
  if (is_logical(.p)) {
    stopifnot(length(.p) == length(.x))
    .p
  } else {
    map_lgl(.x, .p, ...)
  }
}

keep <- function(.x, .f, ...) {
  .x[probe(.x, .f, ...)]
}
discard <- function(.x, .p, ...) {
  sel <- probe(.x, .p, ...)
  .x[is.na(sel) | !sel]
}
map_if <- function(.x, .p, .f, ...) {
  matches <- probe(.x, .p)
  .x[matches] <- map(.x[matches], .f, ...)
  .x
}

compact <- function(.x) {
  Filter(length, .x)
}

transpose <- function(.l) {
  inner_names <- names(.l[[1]])
  if (is.null(inner_names)) {
    fields <- seq_along(.l[[1]])
  } else {
    fields <- set_names(inner_names)
  }

  map(fields, function(i) {
    map(.l, .subset2, i)
  })
}

every <- function(.x, .p, ...) {
  for (i in seq_along(.x)) {
    if (!rlang::is_true(.p(.x[[i]], ...))) {
      return(FALSE)
    }
  }
  TRUE
}
some <- function(.x, .p, ...) {
  for (i in seq_along(.x)) {
    if (rlang::is_true(.p(.x[[i]], ...))) {
      return(TRUE)
    }
  }
  FALSE
}
negate <- function(.p) {
  function(...) !.p(...)
}

reduce <- function(.x, .f, ..., .init) {
  f <- function(x, y) .f(x, y, ...)
  Reduce(f, .x, init = .init)
}
reduce_right <- function(.x, .f, ..., .init) {
  f <- function(x, y) .f(y, x, ...)
  Reduce(f, .x, init = .init, right = TRUE)
}
accumulate <- function(.x, .f, ..., .init) {
  f <- function(x, y) .f(x, y, ...)
  Reduce(f, .x, init = .init, accumulate = TRUE)
}
accumulate_right <- function(.x, .f, ..., .init) {
  f <- function(x, y) .f(y, x, ...)
  Reduce(f, .x, init = .init, right = TRUE, accumulate = TRUE)
}

detect <- function(.x, .f, ..., .right = FALSE, .p = is_true) {
  for (i in index(.x, .right)) {
    if (.p(.f(.x[[i]], ...))) {
      return(.x[[i]])
    }
  }
  NULL
}
detect_index <- function(.x, .f, ..., .right = FALSE, .p = is_true) {
  for (i in index(.x, .right)) {
    if (.p(.f(.x[[i]], ...))) {
      return(i)
    }
  }
  0L
}
index <- function(x, right = FALSE) {
  idx <- seq_along(x)
  if (right) {
    idx <- rev(idx)
  }
  idx
}

imap <- function(.x, .f, ...) {
  map2(.x, vec_index(.x), .f, ...)
}
vec_index <- function(x) {
  names(x) %||% seq_along(x)
}


# nocov end

as_override_size_sum <- function(x) {
  structure(x, class = "override_size_sum")
}

local_override_size_sum <- function(frame = caller_env()) {
  local_methods(
    .frame = frame,
    size_sum.override_size_sum = function(x, ...) {
      "SC"
    }
  )
}

test_that("smoke test", {
  local_override_size_sum()

  v <- new_list_of(list(1L, 2:4), integer())
  expect_snapshot({
    pillar(v, width = 15)
  })

  v <- new_list_of(list(as_override_size_sum(1:3)), as_override_size_sum(integer()))
  expect_snapshot({
    pillar(v, width = 30)
  })
})

---
title: "Printing a tibble: Control and data flow"
output: html_vignette
vignette: >
  %\VignetteIndexEntry{Printing a tibble: Control and data flow}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  eval = rlang::is_installed("tibble"),
  comment = "#>"
)

pillar:::set_show_source_hooks()
```

```{r setup}
library(pillar)
```

What happens when a tibble is printed?
This vignette documents the control flow and the data flow, explains the design choices, and shows the default implementation for the `"tbl"` class.
It is mainly of interest for implementers of table subclasses.
Customizing the formatting of a vector class in a tibble is described in `vignette("pillar", package = "vctrs")`.
The different customization options are showcased in `vignette("extending")`.


## Requirements

- Fit into pre-specified width, distributing across multiple tiers if necessary

- Optionally shrink and stretch individual columns

- Header, body and footer for the tibble

    - Avoid recomputation of information

- Custom components for the pillars in a tibble, top-aligned

    - The container, not the column vectors, determine the appearance

- Customization of the entire output and of the pillars

- Support for data frame columns (packed data frames) and matrix/array columns

- Pillars are always shown from left to right, no "holes" in the colonnade

    - If the first column consumes all available space, the remaining columns are not shown, even if they all would fit if the first column is omitted.

- Printing pillars should take time proportional to the number of characters printed, and be "fast enough".



## Overview

The overall control and data flow are illustrated in the diagram below.
Boxes are functions and methods.
Solid lines are function calls.
Dotted lines represent information that a function obtains via argument or (in the case of options) queries actively.

```{r echo = FALSE, error = TRUE}
text <- paste(
  readLines("format.mmd"),
  collapse = "\n"
)
DiagrammeR::mermaid(text)
```

The pillar package uses debugme for debugging.
Activating debugging for pillar is another way to track the control flow, see `vignette("debugme")` for details.


## Initialization

A tibble is a list of columns of class `"tbl_df"` and `"tbl"`.
Printing is designed to work for non-data-frame table-like objects such as lazy tables.
The `print.tbl()` method calls `format()` for the object and prints the output.

```{r}
tbl <- tibble::tibble(a = 1:3, b = tibble::tibble(c = 4:6, d = 7:9), e = 10:12)
print(tbl, width = 23)
str(tbl)
```


```{r show_source = TRUE}
pillar:::print.tbl
```

The `format.tbl()` method creates a setup object, and uses that object to format header, body and footer.

```{r show_source = TRUE}
pillar:::format.tbl
```

While it's possible to extend or override these methods for your `"tbl"` subclass, often overriding the more specialized methods shown below is sufficient.


## Setup

Most of the work for formatting actually happens in `tbl_format_setup()`.
The desired output width is baked into the setup object and must be available when calling.
Setup objects print like a tibble but with a clear separation of header, body, and footer.

```{r}
setup <- tbl_format_setup(tbl, width = 24)
setup
```

A setup object is required here to avoid computing information twice.
For instance, the dimensions shown in the header or the extra columns displayed in the footer are available only after the body has been computed.

The generic dispatches over the container, so that you can override it if necessary.
It is responsible for assigning default values to arguments before passing them on to the method.

```{r show_source = TRUE}
tbl_format_setup
```


The default implementation converts the input to a data frame via `as.data.frame(head(x))`, and returns an object constructed with `new_tbl_format_setup()` that contains the data frame and additional information.
If you override this method, e.g. to incorporate more information, you can add new items to the default setup object, but you should not overwrite existing items.

```{r show_source = TRUE}
pillar:::tbl_format_setup.tbl
```

At the core, the internal function `ctl_colonnade()` composes the body.
Its functionality and the customization points it offers are detailed in the "Colonnade" section below.


## Header, body, footer

The components of a tibble are formatted with `tbl_format_*()` generics, which also dispatch on the container to allow extension or overrides.
They return a character vector, with one element per line printed.
The setup object is required.

```{r}
tbl_format_header(tbl, setup)
tbl_format_body(tbl, setup)
tbl_format_footer(tbl, setup)
```

(The body is returned as a classed object with a `print()` method, it is still a `character()` under the hood.)

```{r}
class(tbl_format_body(tbl, setup))
typeof(tbl_format_body(tbl, setup))
```

Since most of the work already has been carried out in `tbl_format_setup()`, the default implementations mostly consist of code that styles and wraps the output.

```{r show_source = TRUE}
pillar:::tbl_format_header.tbl
```

```{r show_source = TRUE}
pillar:::tbl_format_body.tbl
```

```{r show_source = TRUE}
pillar:::tbl_format_footer.tbl
```


## Colonnade

The internal function `ctl_colonnade()` composes the body.
It performs the following tasks:

1. Create a pillar object for every top-level column that fits, using the minimum width, via `ctl_new_pillar_list()`, `ctl_new_pillar()` and ultimately `pillar()` and `pillar_shaft()`
1. Determine the number of tiers and the width for each tier
1. Distribute the pillars across the tiers, assigning a width to each pillar.
1. Format each pillar via its `format()` function, passing the now known width.
1. Combine the formatted pillars horizontally.
1. Combine the tiers vertically.
1. Return the formatted body, and the columns that could not fit.

In the following, the first and the fourth steps are discussed.

## Creating pillar objects

The initial tibble is passed to `ctl_new_pillar_list()`, which eventually calls `ctl_new_pillar()` once or several times.
For each top-level column, one pillar object is constructed.
The loop is terminated when the available width is exhausted even considering the minimum width.


### Pillar lists

The `ctl_new_pillar_list()` generic dispatches on the container:

```{r}
ctl_new_pillar_list(tbl, tbl$a, width = 20)
ctl_new_pillar_list(tbl, tbl$b, width = 20)
```

In a tibble, each column can be a data frame, matrix, or even array itself, such columns are called *compound columns*.
Such columns are decomposed into sub-pillars and returned as a list of pillars.
Regular vectors are forwarded to `ctl_new_pillar()` and returned as list of length one.
Implementers of `"tbl"` subclasses will rarely if ever need to extend or override this method.

```{r show_source = TRUE}
pillar:::ctl_new_pillar_list.tbl
```


### Simple pillars

The `ctl_new_pillar()` method is called for columns that are not data frames or arrays, and also dispatches over the container.

```{r}
ctl_new_pillar(tbl, tbl$a, width = 20)
```

```{r show_source = TRUE}
pillar:::ctl_new_pillar.tbl
```

The default method calls `pillar()` directly, passing the maximum width available.

```{r show_source = TRUE}
pillar
```

Formatting for title and type is provided by `new_pillar_title()` and `new_pillar_type()`.
The body can be customized by implementing `pillar_shaft()` for a vector class, see `vignette("pillar", package = "vctrs")` for details.
If title or type don't fit the available width, `pillar_shaft()` is never called.

This function now returns `NULL` if the width is insufficient to contain the data.
It is possible to change the appearance of pillars by overriding or extending `ctl_new_pillar()`.


### Components

Pillar objects share the same structure and are ultimately constructed with `new_pillar()`.

```{r show_source = TRUE}
new_pillar
```

A pillar is stored as a list of components.
Each pillar represents only one simple (atomic) column, compound columns are always represented as multiple pillar objects.


## Formatting pillars

When a pillar object is constructed, it has a minimum and a desired (maximum) width.
Because it depends on the number and width of other pillar objects that may not be even constructed, the final width is not known yet.
It is passed to `format()`, which uses the desired width if empty:

```{r show_source = TRUE}
pillar:::format.pillar
```

---
title: "Controlling display of numbers"
output: html_vignette
vignette: >
  %\VignetteIndexEntry{Controlling display of numbers}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

This content has moved to `vignette("numbers", package = "tibble")`.

`````{asis echo = FALSE}

## Integrating with other classes

### gt

- Consumption via new `fmt_auto()`?

FIXME

### units

xxx{r numbers-17}
library(units)

set_units.pillar_num <- function(x, ...) {
  unclassed <- x
  class(unclassed) <- NULL
  set_units(unclassed, ...)
}

m <- set_units(1:3, m)
km <- set_units(1:3, km)

tibble(
  sci_int = set_num_opts(m + km, notation = "sci"),
  digits_int = set_num_opts(km + m, digits = 4),
  sci_ext = set_units(num(1:3 + 0.001, notation = "sci"), km)
)

tibble(
  sci_int = set_num_opts(m, notation = "sci") + km,
  digits_int = set_num_opts(km, digits = 4) + m,
  sci_ext = set_units(num(1:3, notation = "sci"), m) + km
)
xxx

### formattable

xxx{r numbers-18, error = TRUE}
library(formattable)

pillar_shaft.formattable <- function(x, ...) {
  pillar::new_pillar_shaft_simple(format(x), align = "right")
}

pillar_shaft.formattable_currency <- function(x, ...) {
  formattable <- attr(x, "formattable", exact = TRUE)

  pillar_shaft(num(unclass(x), digits = formattable$digits))
}

pillar_shaft.formattable_percent <- function(x, ...) {
  formattable <- attr(x, "formattable", exact = TRUE)

  pillar_shaft(num(unclass(x), digits = formattable$digits, label = "%", scale = 100))
}

pillar_shaft.formattable_scientific <- function(x, ...) {
  pillar_shaft(num(unclass(x), notation = "sci"))
}

type_sum.formattable <- function(x) {
  formattable <- attr(x, "formattable", exact = TRUE)

  if (inherits(x, "formattable_currency")) {
    I(sub("^formattable_", "", class(x)[[1]]))
  } else if (inherits(x, "formattable_percent")) {
    I("%")
  } else {
    abbreviate(sub("^formattable_", "", class(x)[[1]]), 4)
  }
}

num_currency(1:3 * 100 + 0.1)
num_percent(1:3 * 0.1 + 0.001)
num_scientific(1:3 * 0.1 + 0.001)

tibble(
  currency = num_currency(1:3 * 100 + 0.1),
  percent = num_percent(1:3 * 0.1 + 0.001),
  scientific = num_scientific(1:3 * 0.1 + 0.001)
)
xxx

### scales

xxx{r numbers-scales, error = TRUE}
library(scales)

x <- num(1:10 / 100, label = "%", scale = 100)

scales::squish(x)

x < 0
x < 0L

scales::cscale(x, scales::rescale_pal())
xxx

### ggplot2

xxx{r numbers-19}
library(ggplot2)

scale_type.pillar_num <- function(x, ...) {
  "continuous"
}

data.frame(x = x, y = 1:10) %>%
  ggplot(aes(x = x, y = y)) %>%
  + geom_point()
xxx

## Rule-based decoration



xxx{r}
library(dplyr)

data_units <-
  palmerpenguins::penguins %>%
  mutate(across(ends_with("_mm"), set_units, "mm")) %>%
  mutate(across(ends_with("_g"), set_units, "g"))

data_units %>%
  mutate(bill_area = bill_length_mm * bill_depth_mm, .after = island)
xxx

xxx{r eval = FALSE}
data_decor <-
  data_units %>%
  decorate(year, digits = 0) %>%
  decorate(where(is.numeric), digits = 3)
xxx

xxx{r eval = FALSE}
data_decor %>%
  mutate(bill_area = bill_length_mm * bill_depth_mm, .after = island)
xxx

xxx{r echo = FALSE}
data_units %>%
  mutate(bill_area = bill_length_mm * bill_depth_mm, .after = island) %>%
  mutate(across(year, set_num_opts, digits = 0)) %>%
  mutate(across(where(is.numeric), set_num_opts, digits = 3))
xxx

`````

---
title: "Custom formatting"
output: html_vignette
vignette: >
  %\VignetteIndexEntry{Custom formatting}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(pillar)
```

How to customize the printed output of a `"tbl"` subclass?
This vignette shows the various customization options.
Customizing the formatting of a vector class in a tibble is described in `vignette("pillar", package = "vctrs")`.
An overview over the control and data flow is given in `vignette("printing")`.

This vignette assumes that the reader is familiar with S3 classes, methods, and inheritance.
The ["S3" chapter](https://adv-r.hadley.nz/s3.html) of Hadley Wickham's "Advanced R" is a good start.

To make use of pillar's printing capabilities, create a class that inherits from `"tbl"`, like tibble (classes `"tbl_df"` and `"tbl"`), dbplyr lazy tables (`"tbl_lazy"` and `"tbl"`) and sf spatial data frames (`"sf"`, `"tbl_df"` and `"tbl"`).
Because we are presenting various customization options, we create a constructor for an example data frame with arbitrary subclass.

```{r}
example_tbl <- function(class = NULL) {
  vctrs::new_data_frame(
    list(
      a = letters[1:3],
      b = data.frame(c = 1:3, d = 4:6 + 0.5)
    ),
    class = c(class, "tbl")
  )
}
```

The `"default"` class doesn't have any customizations yet, and prints like a regular tibble.

```{r}
example_tbl()
```


## Header

### Tweak header

The easiest customization consists of tweaking the header.
Implement a `tbl_sum()` method to extend or replace the information shown in the header, keeping the original formatting.

```{r}
tbl_sum.default_header_extend <- function(x, ...) {
  default_header <- NextMethod()
  c(default_header, "New" = "A new header")
}

example_tbl("default_header_extend")

tbl_sum.default_header_replace <- function(x, ...) {
  c("Override" = "Replace all headers")
}

example_tbl("default_header_replace")
```


### Restyle header

To style the header in a different way, implement a `tbl_format_header()` method.
The implementation is responsible for the entire formatting and styling, including the leading hash.

```{r}
tbl_format_header.custom_header_replace <- function(x, setup, ...) {
  cli::style_italic(names(setup$tbl_sum), " = ", setup$tbl_sum)
}

example_tbl("custom_header_replace")
```


## Footer

### Restyle footer

Similarly, to add information the footer, or to replace it entirely, implement a `tbl_format_footer()` method.
Here, as in all `tbl_format_*()` methods, you can use the information contained in the setup object, see `?new_tbl_format_setup` for the available fields.
Again, the implementation is responsible for the entire formatting and styling, including the leading hash if needed.

```{r}
tbl_format_footer.custom_footer_extend <- function(x, setup, ...) {
  default_footer <- NextMethod()

  extra_info <- "and with extra info in the footer"
  extra_footer <- style_subtle(paste0("# ", cli::symbol$ellipsis, " ", extra_info))

  c(default_footer, extra_footer)
}

print(example_tbl("custom_footer_extend"), n = 2)

tbl_format_footer.custom_footer_replace <- function(x, setup, ...) {
  paste0("The table has ", setup$rows_total, " rows in total.")
}

print(example_tbl("custom_footer_replace"), n = 2)
```


### Compute additional info beforehand

If the same information needs to be displayed in several parts (e.g., in both header and footer), it is useful to compute it in `tbl_format_setup()` and store it in the setup object.
New elements may be added to the setup object, existing elements should not be overwritten.
Exception: the `tbl_sum` element contains the output of `tbl_sum()` and can be enhanced with additional elements.

```{r}
tbl_format_setup.extra_info <- function(x, width, ...) {
  setup <- NextMethod()
  cells <- prod(dim(x))
  setup$cells <- cells
  setup$tbl_sum <- c(setup$tbl_sum, "Cells" = as.character(cells))
  setup
}

tbl_format_footer.extra_info <- function(x, setup, ...) {
  paste0("The table has ", setup$cells, " cells in total.")
}

example_tbl("extra_info")
```


## Row IDs

By implementing the generic `ctl_new_rowid_pillar()`, printing of the row ID column can be customized. In order to print Roman instead of Arabic numerals, one could use `utils::as.roman()` to generate the corresponding sequence and build up a row ID pillar using `new_pillar()` and associated methods as has been introduced previously.

```{r}
ctl_new_rowid_pillar.pillar_roman <- function(controller, x, width, ...) {
  out <- NextMethod()
  rowid <- utils::as.roman(seq_len(nrow(x)))
  width <- max(nchar(as.character(rowid)))
  new_pillar(
    list(
      title = out$title,
      type = out$type,
      data = pillar_component(
        new_pillar_shaft(list(row_ids = rowid),
          width = width,
          class = "pillar_rif_shaft"
        )
      )
    ),
    width = width
  )
}

example_tbl("pillar_roman")
```


## Body

### Tweak pillar composition

Pillars consist of components, see `?new_pillar_component` for details.
Extend or override the `ctl_new_pillar()` method to alter the appearance.
The example below adds table rules of constant width to the output.

```{r}
ctl_new_pillar.pillar_rule <- function(controller, x, width, ..., title = NULL) {
  out <- NextMethod()
  new_pillar(list(
    top_rule = new_pillar_component(list("========"), width = 8),
    title = out$title,
    type = out$type,
    mid_rule = new_pillar_component(list("--------"), width = 8),
    data = out$data,
    bottom_rule = new_pillar_component(list("========"), width = 8)
  ))
}

example_tbl("pillar_rule")
```

To make the width adaptive, we implement a `"rule"` class with a `format()` method that formats rules to prespecified widths.

```{r}
rule <- function(char = "-") {
  stopifnot(nchar(char) == 1)
  structure(char, class = "rule")
}

format.rule <- function(x, width, ...) {
  paste(rep(x, width), collapse = "")
}

ctl_new_pillar.pillar_rule_adaptive <- function(controller, x, width, ..., title = NULL) {
  out <- NextMethod()
  if (is.null(out)) {
    return(NULL)
  }

  new_pillar(list(
    top_rule = new_pillar_component(list(rule("=")), width = 1),
    title = out$title,
    type = out$type,
    mid_rule = new_pillar_component(list(rule("-")), width = 1),
    data = out$data,
    bottom_rule = new_pillar_component(list(rule("=")), width = 1)
  ))
}

example_tbl("pillar_rule_adaptive")
```


### Tweak display of compound pillars

Compound pillars are created by `ctl_new_pillar_list()` for columns that contain a data frame, a matrix or an array.
The default implementation also calls `ctl_new_pillar()` shown above.
The (somewhat artificial) example hides all data frame columns in a column with the type `"<hidden>"`.

```{r}
ctl_new_pillar_list.hide_df <- function(controller, x, width, ..., title = NULL) {
  if (!is.data.frame(x)) {
    return(NextMethod())
  }

  if (width < 8) {
    return(NULL)
  }

  list(new_pillar(
    list(
      title = pillar_component(new_pillar_title(title)),
      type = new_pillar_component(list("<hidden>"), width = 8),
      data = new_pillar_component(list(""), width = 1)
    ),
    width = 8
  ))
}

example_tbl("hide_df")
```


### Restyle body

Last but not least, it is also possible to completely alter the display of the body by overriding `tbl_format_body()`.
The example below uses plain data frame output for a tibble.

```{r}
tbl_format_body.oldskool <- function(x, setup, ...) {
  capture.output(print.data.frame(setup$df))
}

print(example_tbl("oldskool"), n = 2)
```

Note that default printed output is computed in `tbl_format_setup()`, this takes a considerable amount of time.
If you really need to change the output for the entire body, consider providing your own `tbl_format_setup()` method.

---
title: "Printing a tibble: Tracking control flow"
output: html_vignette
vignette: >
  %\VignetteIndexEntry{Printing a tibble: Tracking control flow}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

What happens when a tibble is printed?
This vignette shows the actual control flow, powered by the [debugme package](https://github.com/r-lib/debugme#readme).
See `vignette("printing")` for a detailed description of the control and data flows.

To activate debugging, we need to set an environment variable before loading the pillar package.
As a result, debug strings in pillar's code are replaced with calls to a logging function.

By default, debugme also shows timestamps.
This is disabled to keep the output stable.

```{r eval = FALSE}
Sys.setenv(DEBUGME = "!!!!pillar")
Sys.setenv(DEBUGME_SHOW_TIMESTAMP = "no")

library(pillar)

tbl <- tibble::tibble(a = 1:3, b = tibble::tibble(c = 4:6, d = 7:9), e = 10:12)
print(tbl, width = 23)
```

```{r echo = FALSE}
writeLines(readLines("debugme.txt"))
```

---
title: "Comparison with data frames"
output: html_vignette
vignette: >
  %\VignetteIndexEntry{Comparison with data frames}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

This content has moved to `vignette("digits", package = "tibble")`.

---
title: "Custom formatting"
output: html_vignette
vignette: >
  %\VignetteIndexEntry{Custom formatting}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(pillar)
```

How to customize the printed output of a `"tbl"` subclass?
This vignette shows the various customization options.
Customizing the formatting of a vector class in a tibble is described in `vignette("pillar", package = "vctrs")`.
An overview over the control and data flow is given in `vignette("printing")`.

This vignette assumes that the reader is familiar with S3 classes, methods, and inheritance.
The ["S3" chapter](https://adv-r.hadley.nz/s3.html) of Hadley Wickham's "Advanced R" is a good start.

To make use of pillar's printing capabilities, create a class that inherits from `"tbl"`, like tibble (classes `"tbl_df"` and `"tbl"`), dbplyr lazy tables (`"tbl_lazy"` and `"tbl"`) and sf spatial data frames (`"sf"`, `"tbl_df"` and `"tbl"`).
Because we are presenting various customization options, we create a constructor for an example data frame with arbitrary subclass.

```{r}
example_tbl <- function(class) {
  vctrs::new_data_frame(
    list(
      a = letters[1:3],
      b = data.frame(c = 1:3, d = 4:6 + 0.5)
    ),
    class = c(class, "tbl")
  )
}
```

The `"default"` class doesn't have any customizations yet, and prints like a regular tibble.

```{r}
example_tbl("default")
```


## Header

### Tweak header

The easiest customization consists of tweaking the header.
Implement a `tbl_sum()` method to extend or replace the information shown in the header, keeping the original formatting.

```{r}
tbl_sum.default_header_extend <- function(x, ...) {
  default_header <- NextMethod()
  c(default_header, "New" = "A new header")
}

example_tbl("default_header_extend")

tbl_sum.default_header_replace <- function(x, ...) {
  c("Override" = "Replace all headers")
}

example_tbl("default_header_replace")
```


### Restyle header

To style the header in a different way, implement a `tbl_format_header()` method.
The implementation is responsible for the entire formatting and styling, including the leading hash.

```{r}
tbl_format_header.custom_header_replace <- function(x, setup, ...) {
  cli::style_italic(names(setup$tbl_sum), " = ", setup$tbl_sum)
}

example_tbl("custom_header_replace")
```


## Footer

### Restyle footer

Similarly, to add information the footer, or to replace it entirely, implement a `tbl_format_footer()` method.
Here, as in all `tbl_format_*()` methods, you can use the information contained in the setup object, see `?new_tbl_format_setup` for the available fields.
Again, the implementation is responsible for the entire formatting and styling, including the leading hash if needed.

```{r}
tbl_format_footer.custom_footer_extend <- function(x, setup, ...) {
  default_footer <- NextMethod()

  extra_info <- "and with extra info in the footer"
  extra_footer <- style_subtle(paste0("# ", cli::symbol$ellipsis, " ", extra_info))

  c(default_footer, extra_footer)
}

print(example_tbl("custom_footer_extend"), n = 2)

tbl_format_footer.custom_footer_replace <- function(x, setup, ...) {
  paste0("The table has ", setup$rows_total, " rows in total.")
}

print(example_tbl("custom_footer_replace"), n = 2)
```


### Compute additional info beforehand

If the same information needs to be displayed in several parts (e.g., in both header and footer), it is useful to compute it in `tbl_format_setup()` and store it in the setup object.
New elements may be added to the setup object, existing elements should not be overwritten.
Exception: the `tbl_sum` element contains the output of `tbl_sum()` and can be enhanced with additional elements.

```{r}
tbl_format_setup.extra_info <- function(x, width, ...) {
  setup <- NextMethod()
  cells <- prod(dim(x))
  setup$cells <- cells
  setup$tbl_sum <- c(setup$tbl_sum, "Cells" = as.character(cells))
  setup
}

tbl_format_footer.extra_info <- function(x, setup, ...) {
  paste0("The table has ", setup$cells, " cells in total.")
}

example_tbl("extra_info")
```


## Row IDs

By implementing the generic `ctl_new_rowid_pillar()`, printing of the row ID column can be customized. In order to print Roman instead of Arabic numerals, one could use `utils::as.roman()` to generate the corresponding sequence and build up a row ID pillar using `new_pillar()` and associated methods as has been introduced previously.

```{r}
ctl_new_rowid_pillar.pillar_roman <- function(controller, x, width, ...) {
  out <- NextMethod()
  rowid <- utils::as.roman(seq_len(nrow(x)))
  width <- max(nchar(as.character(rowid)))
  new_pillar(
    list(
      title = out$title,
      type = out$type,
      data = pillar_component(
        new_pillar_shaft(list(row_ids = rowid),
          width = width,
          class = "pillar_rif_shaft"
        )
      )
    ),
    width = width
  )
}

example_tbl("pillar_roman")
```


## Body

### Tweak pillar composition

Pillars consist of components, see `?new_pillar_component` for details.
Extend or override the `ctl_new_pillar()` method to alter the appearance.
The example below adds table rules of constant width to the output.

```{r}
ctl_new_pillar.pillar_rule <- function(controller, x, width, ..., title = NULL) {
  out <- NextMethod()
  new_pillar(list(
    top_rule = new_pillar_component(list("========"), width = 8),
    title = out$title,
    type = out$type,
    mid_rule = new_pillar_component(list("--------"), width = 8),
    data = out$data,
    bottom_rule = new_pillar_component(list("========"), width = 8)
  ))
}

example_tbl("pillar_rule")
```

To make the width adaptive, we implement a `"rule"` class with a `format()` method that formats rules to prespecified widths.

```{r}
rule <- function(char = "-") {
  stopifnot(nchar(char) == 1)
  structure(char, class = "rule")
}

format.rule <- function(x, width, ...) {
  paste(rep(x, width), collapse = "")
}

ctl_new_pillar.pillar_rule_adaptive <- function(controller, x, width, ..., title = NULL) {
  out <- NextMethod()
  if (is.null(out)) {
    return(NULL)
  }

  new_pillar(list(
    top_rule = new_pillar_component(list(rule("=")), width = 1),
    title = out$title,
    type = out$type,
    mid_rule = new_pillar_component(list(rule("-")), width = 1),
    data = out$data,
    bottom_rule = new_pillar_component(list(rule("=")), width = 1)
  ))
}

example_tbl("pillar_rule_adaptive")
```


### Tweak display of compound pillars

Compound pillars are created by `ctl_new_pillar_list()` for columns that contain a data frame, a matrix or an array.
The default implementation also calls `ctl_new_pillar()` shown above.
The (somewhat artificial) example hides all data frame columns in a column with the type `"<hidden>"`.

```{r}
ctl_new_pillar_list.hide_df <- function(controller, x, width, ..., title = NULL) {
  if (!is.data.frame(x)) {
    return(NextMethod())
  }
  
  if (width < 8) {
    return(NULL)
  }

  list(new_pillar(
    list(
      title = pillar_component(new_pillar_title(title)),
      type = new_pillar_component(list("<hidden>"), width = 8),
      data = new_pillar_component(list(""), width = 1)
    ),
    width = 8
  ))
}

example_tbl("hide_df")
```


### Restyle body

Last but not least, it is also possible to completely alter the display of the body by overriding `tbl_format_body()`.
The example below uses plain data frame output for a tibble.

```{r}
tbl_format_body.oldskool <- function(x, setup, ...) {
  capture.output(print.data.frame(setup$df))
}

print(example_tbl("oldskool"), n = 2)
```

Note that default printed output is computed in `tbl_format_setup()`, this takes a considerable amount of time.
If you really need to change the output for the entire body, consider providing your own `tbl_format_setup()` method.

---
title: "Custom formatting"
output: html_vignette
vignette: >
  %\VignetteIndexEntry{Custom formatting}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(pillar)
```

How to customize the printed output of a `"tbl"` subclass?
This vignette shows the various customization options.
Customizing the formatting of a vector class in a tibble is described in `vignette("pillar", package = "vctrs")`.
An overview over the control and data flow is given in `vignette("printing")`.

This vignette assumes that the reader is familiar with S3 classes, methods, and inheritance.
The ["S3" chapter](https://adv-r.hadley.nz/s3.html) of Hadley Wickham's "Advanced R" is a good start.

To make use of pillar's printing capabilities, create a class that inherits from `"tbl"`, like tibble (classes `"tbl_df"` and `"tbl"`), dbplyr lazy tables (`"tbl_lazy"` and `"tbl"`) and sf spatial data frames (`"sf"`, `"tbl_df"` and `"tbl"`).
Because we are presenting various customization options, we create a constructor for an example data frame with arbitrary subclass.

```{r}
example_tbl <- function(class) {
  vctrs::new_data_frame(
    list(
      a = letters[1:3],
      b = data.frame(c = 1:3, d = 4:6 + 0.5)
    ),
    class = c(class, "tbl")
  )
}
```

The `"default"` class doesn't have any customizations yet, and prints like a regular tibble.

```{r}
example_tbl("default")
```


## Header

### Tweak header

The easiest customization consists of tweaking the header.
Implement a `tbl_sum()` method to extend or replace the information shown in the header, keeping the original formatting.

```{r}
tbl_sum.default_header_extend <- function(x, ...) {
  default_header <- NextMethod()
  c(default_header, "New" = "A new header")
}

example_tbl("default_header_extend")

tbl_sum.default_header_replace <- function(x, ...) {
  c("Override" = "Replace all headers")
}

example_tbl("default_header_replace")
```


### Restyle header

To style the header in a different way, implement a `tbl_format_header()` method.
The implementation is responsible for the entire formatting and styling, including the leading hash.

```{r}
tbl_format_header.custom_header_replace <- function(x, setup, ...) {
  cli::style_italic(names(setup$tbl_sum), " = ", setup$tbl_sum)
}

example_tbl("custom_header_replace")
```


## Footer

### Restyle footer

Similarly, to add information the footer, or to replace it entirely, implement a `tbl_format_footer()` method.
Here, as in all `tbl_format_*()` methods, you can use the information contained in the setup object, see `?new_tbl_format_setup` for the available fields.
Again, the implementation is responsible for the entire formatting and styling, including the leading hash if needed.

```{r}
tbl_format_footer.custom_footer_extend <- function(x, setup, ...) {
  default_footer <- NextMethod()

  extra_info <- "and with extra info in the footer"
  extra_footer <- style_subtle(paste0("# ", cli::symbol$ellipsis, " ", extra_info))

  c(default_footer, extra_footer)
}

print(example_tbl("custom_footer_extend"), n = 2)

tbl_format_footer.custom_footer_replace <- function(x, setup, ...) {
  paste0("The table has ", setup$rows_total, " rows in total.")
}

print(example_tbl("custom_footer_replace"), n = 2)
```


### Compute additional info beforehand

If the same information needs to be displayed in several parts (e.g., in both header and footer), it is useful to compute it in `tbl_format_setup()` and store it in the setup object.
New elements may be added to the setup object, existing elements should not be overwritten.
Exception: the `tbl_sum` element contains the output of `tbl_sum()` and can be enhanced with additional elements.

```{r}
tbl_format_setup.extra_info <- function(x, width, ...) {
  setup <- NextMethod()
  cells <- prod(dim(x))
  setup$cells <- cells
  setup$tbl_sum <- c(setup$tbl_sum, "Cells" = as.character(cells))
  setup
}

tbl_format_footer.extra_info <- function(x, setup, ...) {
  paste0("The table has ", setup$cells, " cells in total.")
}

example_tbl("extra_info")
```


## Row IDs

By implementing the generic `ctl_new_rowid_pillar()`, printing of the row ID column can be customized. In order to print Roman instead of Arabic numerals, one could use `utils::as.roman()` to generate the corresponding sequence and build up a row ID pillar using `new_pillar()` and associated methods as has been introduced previously.

```{r}
ctl_new_rowid_pillar.pillar_roman <- function(controller, x, width, ...) {
  out <- NextMethod()
  rowid <- utils::as.roman(seq_len(nrow(x)))
  width <- max(nchar(as.character(rowid)))
  new_pillar(
    list(
      title = out$title,
      type = out$type,
      data = pillar_component(
        new_pillar_shaft(list(row_ids = rowid),
          width = width,
          class = "pillar_rif_shaft"
        )
      )
    ),
    width = width
  )
}

example_tbl("pillar_roman")
```


## Body

### Tweak pillar composition

Pillars consist of components, see `?new_pillar_component` for details.
Extend or override the `ctl_new_pillar()` method to alter the appearance.
The example below adds table rules of constant width to the output.

```{r}
ctl_new_pillar.pillar_rule <- function(controller, x, width, ..., title = NULL) {
  out <- NextMethod()
  new_pillar(list(
    top_rule = new_pillar_component(list("========"), width = 8),
    title = out$title,
    type = out$type,
    mid_rule = new_pillar_component(list("--------"), width = 8),
    data = out$data,
    bottom_rule = new_pillar_component(list("========"), width = 8)
  ))
}

example_tbl("pillar_rule")
```

To make the width adaptive, we implement a `"rule"` class with a `format()` method that formats rules to prespecified widths.

```{r}
rule <- function(char = "-") {
  stopifnot(nchar(char) == 1)
  structure(char, class = "rule")
}

format.rule <- function(x, width, ...) {
  paste(rep(x, width), collapse = "")
}

ctl_new_pillar.pillar_rule_adaptive <- function(controller, x, width, ..., title = NULL) {
  out <- NextMethod()
  if (is.null(out)) {
    return(NULL)
  }

  new_pillar(list(
    top_rule = new_pillar_component(list(rule("=")), width = 1),
    title = out$title,
    type = out$type,
    mid_rule = new_pillar_component(list(rule("-")), width = 1),
    data = out$data,
    bottom_rule = new_pillar_component(list(rule("=")), width = 1)
  ))
}

example_tbl("pillar_rule_adaptive")
```


### Tweak display of compound pillars

Compound pillars are created by `ctl_new_pillar_list()` for columns that contain a data frame, a matrix or an array.
The default implementation also calls `ctl_new_pillar()` shown above.
The (somewhat artificial) example hides all data frame columns in a column with the type `"<hidden>"`.

```{r}
ctl_new_pillar_list.hide_df <- function(controller, x, width, ..., title = NULL) {
  if (!is.data.frame(x)) {
    return(NextMethod())
  }

  if (width < 8) {
    return(NULL)
  }

  list(new_pillar(
    list(
      title = pillar_component(new_pillar_title(title)),
      type = new_pillar_component(list("<hidden>"), width = 8),
      data = new_pillar_component(list(""), width = 1)
    ),
    width = 8
  ))
}

example_tbl("hide_df")
```


### Restyle body

Last but not least, it is also possible to completely alter the display of the body by overriding `tbl_format_body()`.
The example below uses plain data frame output for a tibble.

```{r}
tbl_format_body.oldskool <- function(x, setup, ...) {
  capture.output(print.data.frame(setup$df))
}

print(example_tbl("oldskool"), n = 2)
```

Note that default printed output is computed in `tbl_format_setup()`, this takes a considerable amount of time.
If you really need to change the output for the entire body, consider providing your own `tbl_format_setup()` method.

# output test

    Code
      pillar(add_special(letters[1:5]))
    Output
      <pillar>
      <chr>
      a    
      b    
      c    
      d    
      e    
      <NA> 
    Code
      pillar(add_special(paste(letters, collapse = "")))
    Output
      <pillar>
      <chr>                     
      abcdefghijklmnopqrstuvwxyz
      <NA>                      
    Code
      pillar(add_special(paste(letters, collapse = "")), width = 10)
    Output
      <pillar>
      <chr>     
      abcdefghi~
      <NA>      
    Code
      pillar(add_special(paste(letters, collapse = "")), width = 3)
    Output
      NULL
    Code
      pillar(add_special(c("")), width = 5)
    Output
      <pillar>
      <chr>
      ""   
      <NA> 
    Code
      pillar(add_special(c(" ")), width = 5)
    Output
      <pillar>
      <chr>
      " "  
      <NA> 
    Code
      pillar(add_special(c(" a")), width = 5)
    Output
      <pillar>
      <chr>
      " a" 
      <NA> 
    Code
      pillar(add_special(c("a ")), width = 5)
    Output
      <pillar>
      <chr>
      "a " 
      <NA> 
    Code
      pillar(add_special(c("a b")), width = 5)
    Output
      <pillar>
      <chr>
      a b  
      <NA> 
    Code
      pillar(add_special(c("\t")), width = 10)
    Output
      <pillar>
      <chr>
      "\t" 
      <NA> 
    Code
      pillar(add_special(c("a\nb")), width = 10)
    Output
      <pillar>
      <chr> 
      "a\nb"
       <NA> 
    Code
      pillar(add_special(c("a\001b")), width = 10)
    Output
      <pillar>
      <chr>     
      "a\u0001b"
       <NA>     

# output test (not on Windows)

    Code
      pillar(add_special("成交日"), title = "成交")
    Output
      <pillar>
      成交  
      <chr> 
      成交日
      <NA>  
    Code
      pillar(add_special("成交"), title = "成交日")
    Output
      <pillar>
      成交日
      <chr> 
      成交  
      <NA>  
    Code
      pillar(add_special(1L), title = "成交日")
    Output
      <pillar>
      成交日
       <int>
           1
          NA

---

    Code
      ctl_colonnade(chartype_frame(), width = 50)
    Output
      $body
         chars          desc              
         <chr>          <chr>             
       1 "\u0001\u001f" C0 control code   
       2 "\a\b\f\n\r\t" Named control code
       3 "abcdefuvwxyz" ASCII             
       4 "\u0080\u009f" C1 control code   
       5 " ¡¢£¤¥úûüýþÿ" Latin-1           
       6 "ĀāĂăĄąĆćĈĉĊċ" Unicode           
       7 "！＂＃＄％＆" Unicode wide      
       8 "\u0e00\u2029" Unicode control   
       9 "x­x​x‌x‍x‎x‏x͏x﻿x󠀁x󠀠x󠇯x" Unicode ignorable 
      10 "àáâãāa̅ăȧäảåa̋" Unicode mark      
      11 "😀😁😂😃😄💃" Emoji             
      12 "x\U0010ffffx" Unassigned        
      13 "\xfd\xfe\xff" Invalid           
      14 "\\"           Backslash         
      15 "\""           Quote             
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      

# strings with varying widths

    Code
      ctl_colonnade(df_str[c(28L, 34L, 16L, 29L, 47L, 25L, 42L, 27L, 44L, 20L, 14L,
        36L, 43L, 41L, 26L, 45L, 22L, 9L, 13L, 32L, 31L, 12L, 19L, 48L, 49L, 35L, 3L,
        11L, 23L, 24L, 40L, 15L, 38L, 10L, 46L, 5L, 50L, 18L, 21L, 6L, 30L, 2L, 7L,
        1L, 4L, 8L, 17L, 33L, 39L, 37L)], width = {
        options(width = 54)
        516
      })
    Output
      $body
        `28`                        
        <chr>                       
      1 AbcdefghijAbcdefghijAbcdefgh
        `34`                               `16`            
        <chr>                              <chr>           
      1 AbcdefghijAbcdefghijAbcdefghijAbcd AbcdefghijAbcdef
        `29`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefghi
        `47`                                           
        <chr>                                          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefg
        `25`                      `42`     `27`  `44`  `20` 
        <chr>                     <chr>    <chr> <chr> <chr>
      1 AbcdefghijAbcdefghijAbcde Abcdefg~ Abcd~ Abcd~ Abcd~
        `14`       `36`  `43`  `41`  `26`  `45`  `22`  `9`  
        <chr>      <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefghi~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `13`       `32`  `31`  `12`  `19`  `48`  `49`  `35` 
        <chr>      <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefghi~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `3`   `11`       `23`  `24`  `40`  `15`  `38`  `10` 
        <chr> <chr>      <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abc   Abcdefghi~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `46`       `5`   `50`  `18`  `21`  `6`   `30`  `2`  
        <chr>      <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefghi~ Abcde Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Ab   
        `7`     `1`   `4`   `8`      `17`  `33`  `39`  `37` 
        <chr>   <chr> <chr> <chr>    <chr> <chr> <chr> <chr>
      1 Abcdefg A     Abcd  Abcdefgh Abcd~ Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(28L, 41L, 12L, 29L, 13L, 43L, 24L, 50L, 48L, 35L, 44L,
        21L, 33L, 45L, 47L, 34L, 25L, 14L, 18L, 23L, 7L, 3L, 42L, 36L, 11L, 2L, 20L,
        31L, 1L, 4L, 38L, 9L, 27L, 40L, 32L, 17L, 6L, 49L, 16L, 19L, 15L, 22L, 39L,
        10L, 46L, 5L, 30L, 8L, 26L, 37L)], width = {
        options(width = 42)
        1365
      })
    Output
      $body
        `28`                        
        <chr>                       
      1 AbcdefghijAbcdefghijAbcdefgh
        `41`                                    
        <chr>                                   
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghi~
        `12`        
        <chr>       
      1 AbcdefghijAb
        `29`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefghi
        `13`         
        <chr>        
      1 AbcdefghijAbc
        `43`                                    
        <chr>                                   
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghi~
        `24`                    
        <chr>                   
      1 AbcdefghijAbcdefghijAbcd
        `50`                                    
        <chr>                                   
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghi~
        `48`                                    
        <chr>                                   
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghi~
        `35`                               
        <chr>                              
      1 AbcdefghijAbcdefghijAbcdefghijAbcde
        `44`                                    
        <chr>                                   
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghi~
        `21`                 
        <chr>                
      1 AbcdefghijAbcdefghijA
        `33`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAbc
        `45`                                    
        <chr>                                   
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghi~
        `47`                                    
        <chr>                                   
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghi~
        `34`                              
        <chr>                             
      1 AbcdefghijAbcdefghijAbcdefghijAbcd
        `25`                      `14`          
        <chr>                     <chr>         
      1 AbcdefghijAbcdefghijAbcde AbcdefghijAbcd
        `18`              
        <chr>             
      1 AbcdefghijAbcdefgh
        `23`                    `7`     `3`  
        <chr>                   <chr>   <chr>
      1 AbcdefghijAbcdefghijAbc Abcdefg Abc  
        `42`                                    
        <chr>                                   
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghi~
        `36`                                
        <chr>                               
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef
        `11`        `2`   `20`                
        <chr>       <chr> <chr>               
      1 AbcdefghijA Ab    AbcdefghijAbcdefghij
        `31`                            `1`  
        <chr>                           <chr>
      1 AbcdefghijAbcdefghijAbcdefghijA A    
        `4`  
        <chr>
      1 Abcd 
        `38`                                  
        <chr>                                 
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `9`       `27`                       
        <chr>     <chr>                      
      1 Abcdefghi AbcdefghijAbcdefghijAbcdefg
        `40`                                    
        <chr>                                   
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghij
        `32`                            
        <chr>                           
      1 AbcdefghijAbcdefghijAbcdefghijAb
        `17`              `6`   
        <chr>             <chr> 
      1 AbcdefghijAbcdefg Abcdef
        `49`                                    
        <chr>                                   
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghi~
        `16`             `19`               
        <chr>            <chr>              
      1 AbcdefghijAbcdef AbcdefghijAbcdefghi
        `15`            `22`         `39`  `10` 
        <chr>           <chr>        <chr> <chr>
      1 AbcdefghijAbcde AbcdefghijA~ Abcd~ Abcd~
        `46`       `5`   `30`  `8`   `26`  `37` 
        <chr>      <chr> <chr> <chr> <chr> <chr>
      1 Abcdefghi~ Abcde Abcd~ Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(40L, 17L, 13L, 23L, 22L, 2L, 18L, 3L, 29L, 45L, 14L, 19L,
        33L, 37L, 47L, 43L, 44L, 10L, 31L, 27L, 34L, 35L, 41L, 21L, 4L, 25L, 38L, 48L,
        9L, 24L, 26L, 39L, 20L, 36L, 42L, 16L, 6L, 11L, 7L, 12L, 1L, 46L, 15L, 5L, 8L,
        50L, 32L, 30L, 49L, 28L)], width = {
        options(width = 39)
        934
      })
    Output
      $body
        `40`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `17`              `13`         
        <chr>             <chr>        
      1 AbcdefghijAbcdefg AbcdefghijAbc
        `23`                   
        <chr>                  
      1 AbcdefghijAbcdefghijAbc
        `22`                   `2`  
        <chr>                  <chr>
      1 AbcdefghijAbcdefghijAb Ab   
        `18`               `3`  
        <chr>              <chr>
      1 AbcdefghijAbcdefgh Abc  
        `29`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefghi
        `45`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `14`           `19`               
        <chr>          <chr>              
      1 AbcdefghijAbcd AbcdefghijAbcdefghi
        `33`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAbc
        `37`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefg
        `47`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `43`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `44`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `10`      
        <chr>     
      1 Abcdefghij
        `31`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghijA
        `27`                       
        <chr>                      
      1 AbcdefghijAbcdefghijAbcdefg
        `34`                              
        <chr>                             
      1 AbcdefghijAbcdefghijAbcdefghijAbcd
        `35`                               
        <chr>                              
      1 AbcdefghijAbcdefghijAbcdefghijAbcde
        `41`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `21`                  `4`   `25`     
        <chr>                 <chr> <chr>    
      1 AbcdefghijAbcdefghijA Abcd  Abcdefgh~
        `38`    `48`  `9`   `24`  `26`  `39` 
        <chr>   <chr> <chr> <chr> <chr> <chr>
      1 Abcdef~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `20`    `36`  `42`  `16`  `6`   `11` 
        <chr>   <chr> <chr> <chr> <chr> <chr>
      1 Abcdef~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `7`     `12`  `1`   `46`  `15`  `5`  
        <chr>   <chr> <chr> <chr> <chr> <chr>
      1 Abcdefg Abcd~ A     Abcd~ Abcd~ Abcde
        `8`     `50`  `32`  `30`  `49`  `28` 
        <chr>   <chr> <chr> <chr> <chr> <chr>
      1 Abcdef~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(11L, 36L, 17L, 14L, 31L, 35L, 23L, 13L, 6L, 44L, 45L,
        22L, 21L, 18L, 33L, 10L, 43L, 2L, 46L, 34L, 3L, 19L, 1L, 38L, 9L, 37L, 5L, 8L,
        25L, 49L, 27L, 29L, 15L, 39L, 24L, 40L, 48L, 26L, 47L, 42L, 41L, 12L, 28L,
        30L, 7L, 16L, 4L, 50L, 20L, 32L)], width = {
        options(width = 32)
        565
      })
    Output
      $body
        `11`       
        <chr>      
      1 AbcdefghijA
        `36`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `17`             
        <chr>            
      1 AbcdefghijAbcdefg
        `14`          
        <chr>         
      1 AbcdefghijAbcd
        `31`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `35`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `23`                   
        <chr>                  
      1 AbcdefghijAbcdefghijAbc
        `13`          `6`   
        <chr>         <chr> 
      1 AbcdefghijAbc Abcdef
        `44`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `45`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `22`         `21`  `18`  `33` 
        <chr>        <chr> <chr> <chr>
      1 AbcdefghijA~ Abcd~ Abcd~ Abcd~
        `10`   `43`  `2`   `46`  `34` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Ab    Abcd~ Abcd~
        `3`   `19`   `1`   `38`  `9`  
        <chr> <chr>  <chr> <chr> <chr>
      1 Abc   Abcde~ A     Abcd~ Abcd~
        `37`   `5`   `8`   `25`  `49` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcde Abcd~ Abcd~ Abcd~
        `27`   `29`  `15`  `39`  `24` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `40`   `48`  `26`  `47`  `42` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `41`   `12`  `28`  `30`  `7`  
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `16`   `4`   `50`  `20`  `32` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd  Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(18L, 46L, 11L, 43L, 31L, 47L, 48L, 44L, 50L, 15L, 28L,
        33L, 13L, 4L, 22L, 3L, 37L, 32L, 40L, 9L, 25L, 16L, 45L, 23L, 21L, 6L, 49L,
        36L, 27L, 38L, 14L, 34L, 8L, 24L, 29L, 1L, 12L, 2L, 20L, 17L, 35L, 5L, 19L,
        30L, 7L, 26L, 42L, 41L, 39L, 10L)], width = {
        options(width = 35)
        1121
      })
    Output
      $body
        `18`              
        <chr>             
      1 AbcdefghijAbcdefgh
        `46`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAb~
        `11`       
        <chr>      
      1 AbcdefghijA
        `43`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAb~
        `31`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghijA
        `47`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAb~
        `48`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAb~
        `44`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAb~
        `50`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAb~
        `15`           
        <chr>          
      1 AbcdefghijAbcde
        `28`                        
        <chr>                       
      1 AbcdefghijAbcdefghijAbcdefgh
        `33`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAbc
        `13`          `4`  
        <chr>         <chr>
      1 AbcdefghijAbc Abcd 
        `22`                   `3`  
        <chr>                  <chr>
      1 AbcdefghijAbcdefghijAb Abc  
        `37`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAb~
        `32`                            
        <chr>                           
      1 AbcdefghijAbcdefghijAbcdefghijAb
        `40`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAb~
        `9`      
        <chr>    
      1 Abcdefghi
        `25`                     
        <chr>                    
      1 AbcdefghijAbcdefghijAbcde
        `16`            
        <chr>           
      1 AbcdefghijAbcdef
        `45`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAb~
        `23`                   
        <chr>                  
      1 AbcdefghijAbcdefghijAbc
        `21`                  `6`   
        <chr>                 <chr> 
      1 AbcdefghijAbcdefghijA Abcdef
        `49`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAb~
        `36`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAb~
        `27`                       
        <chr>                      
      1 AbcdefghijAbcdefghijAbcdefg
        `38`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAb~
        `14`          
        <chr>         
      1 AbcdefghijAbcd
        `34`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAb~
        `8`      `24`               `29` 
        <chr>    <chr>              <chr>
      1 Abcdefgh AbcdefghijAbcdefg~ Abcd~
        `1`   `12`      `2`   `20`  `17` 
        <chr> <chr>     <chr> <chr> <chr>
      1 A     Abcdefgh~ Ab    Abcd~ Abcd~
        `35`      `5`   `19`  `30`  `7`  
        <chr>     <chr> <chr> <chr> <chr>
      1 Abcdefgh~ Abcde Abcd~ Abcd~ Abcd~
        `26`      `42`  `41`  `39`  `10` 
        <chr>     <chr> <chr> <chr> <chr>
      1 Abcdefgh~ Abcd~ Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(43L, 1L, 3L, 15L, 28L, 12L, 46L, 34L, 31L, 7L, 11L, 4L,
        44L, 8L, 9L, 5L, 36L, 22L, 17L, 39L, 18L, 45L, 37L, 13L, 29L, 6L, 30L, 16L,
        20L, 10L, 19L, 26L, 33L, 40L, 35L, 48L, 38L, 25L, 2L, 47L, 42L, 41L, 27L, 14L,
        21L, 24L, 50L, 49L, 23L, 32L)], width = {
        options(width = 32)
        446
      })
    Output
      $body
        `43`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `1`   `3`   `15`           
        <chr> <chr> <chr>          
      1 A     Abc   AbcdefghijAbcde
        `28`                        
        <chr>                       
      1 AbcdefghijAbcdefghijAbcdefgh
        `12`        
        <chr>       
      1 AbcdefghijAb
        `46`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `34`               `31`  `7`  
        <chr>              <chr> <chr>
      1 AbcdefghijAbcdefg~ Abcd~ Abcd~
        `11`   `4`   `44`  `8`   `9`  
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd  Abcd~ Abcd~ Abcd~
        `5`   `36`   `22`  `17`  `39` 
        <chr> <chr>  <chr> <chr> <chr>
      1 Abcde Abcde~ Abcd~ Abcd~ Abcd~
        `18`   `45`  `37`  `13`  `29` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `6`    `30`  `16`  `20`  `10` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcdef Abcd~ Abcd~ Abcd~ Abcd~
        `19`   `26`  `33`  `40`  `35` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `48`   `38`  `25`  `2`   `47` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Ab    Abcd~
        `42`   `41`  `27`  `14`  `21` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `24`   `50`  `49`  `23`  `32` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(37L, 46L, 21L, 3L, 16L, 39L, 34L, 33L, 10L, 17L, 19L,
        36L, 45L, 49L, 11L, 50L, 14L, 29L, 44L, 13L, 30L, 38L, 32L, 40L, 42L, 1L, 31L,
        41L, 7L, 23L, 35L, 28L, 6L, 25L, 2L, 9L, 12L, 15L, 5L, 18L, 20L, 27L, 43L, 8L,
        47L, 4L, 48L, 24L, 26L, 22L)], width = {
        options(width = 31)
        1166
      })
    Output
      $body
        `37`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `46`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `21`                  `3`  
        <chr>                 <chr>
      1 AbcdefghijAbcdefghijA Abc  
        `16`            
        <chr>           
      1 AbcdefghijAbcdef
        `39`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `34`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `33`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `10`       `17`             
        <chr>      <chr>            
      1 Abcdefghij AbcdefghijAbcdefg
        `19`               
        <chr>              
      1 AbcdefghijAbcdefghi
        `36`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `45`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `49`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `11`       
        <chr>      
      1 AbcdefghijA
        `50`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `14`          
        <chr>         
      1 AbcdefghijAbcd
        `29`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefghi
        `44`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `13`         
        <chr>        
      1 AbcdefghijAbc
        `30`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `38`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `32`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `40`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `42`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `1`  
        <chr>
      1 A    
        `31`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `41`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `7`    
        <chr>  
      1 Abcdefg
        `23`                   
        <chr>                  
      1 AbcdefghijAbcdefghijAbc
        `35`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `28`                        
        <chr>                       
      1 AbcdefghijAbcdefghijAbcdefgh
        `6`   
        <chr> 
      1 Abcdef
        `25`                     
        <chr>                    
      1 AbcdefghijAbcdefghijAbcde
        `2`   `9`       `12`        
        <chr> <chr>     <chr>       
      1 Ab    Abcdefghi AbcdefghijAb
        `15`            `5`  
        <chr>           <chr>
      1 AbcdefghijAbcde Abcde
        `18`              
        <chr>             
      1 AbcdefghijAbcdefgh
        `20`                
        <chr>               
      1 AbcdefghijAbcdefghij
        `27`        `43`  `8`   `47` 
        <chr>       <chr> <chr> <chr>
      1 Abcdefghij~ Abcd~ Abcd~ Abcd~
        `4`   `48`  `24`  `26`  `22` 
        <chr> <chr> <chr> <chr> <chr>
      1 Abcd  Abcd~ Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(31L, 39L, 40L, 30L, 10L, 21L, 9L, 16L, 46L, 25L, 15L,
        24L, 3L, 50L, 35L, 1L, 12L, 34L, 48L, 4L, 29L, 23L, 37L, 36L, 28L, 43L, 11L,
        17L, 32L, 8L, 41L, 13L, 44L, 7L, 38L, 26L, 33L, 20L, 19L, 2L, 18L, 49L, 27L,
        47L, 22L, 14L, 6L, 5L, 45L, 42L)], width = {
        options(width = 58)
        546
      })
    Output
      $body
        `31`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghijA
        `39`                                   
        <chr>                                  
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghi
        `40`                                    
        <chr>                                   
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghij
        `30`                           `10`      
        <chr>                          <chr>     
      1 AbcdefghijAbcdefghijAbcdefghij Abcdefghij
        `21`                  `9`       `16`            
        <chr>                 <chr>     <chr>           
      1 AbcdefghijAbcdefghijA Abcdefghi AbcdefghijAbcdef
        `46`                       `25`  `15`  `24`  `3`   `50` 
        <chr>                      <chr> <chr> <chr> <chr> <chr>
      1 AbcdefghijAbcdefghijAbcde~ Abcd~ Abcd~ Abcd~ Abc   Abcd~
        `35`     `1`   `12`  `34`  `48`  `4`   `29`  `23`  `37` 
        <chr>    <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefg~ A     Abcd~ Abcd~ Abcd~ Abcd  Abcd~ Abcd~ Abcd~
        `36`     `28`  `43`  `11`  `17`  `32`  `8`   `41`  `13` 
        <chr>    <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefg~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `44`     `7`   `38`  `26`  `33`  `20`  `19`  `2`   `18` 
        <chr>    <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefg~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Ab    Abcd~
        `49`     `27`  `47`  `22`  `14`  `6`   `5`   `45`  `42` 
        <chr>    <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefg~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcde Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(43L, 21L, 41L, 48L, 22L, 25L, 2L, 8L, 1L, 24L, 6L, 39L,
        38L, 20L, 49L, 45L, 47L, 12L, 9L, 13L, 36L, 26L, 44L, 11L, 46L, 28L, 7L, 18L,
        50L, 16L, 29L, 30L, 4L, 23L, 17L, 40L, 33L, 14L, 27L, 19L, 34L, 32L, 3L, 37L,
        15L, 10L, 5L, 35L, 31L, 42L)], width = {
        options(width = 57)
        1035
      })
    Output
      $body
        `43`                                       
        <chr>                                      
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbc
        `21`                 
        <chr>                
      1 AbcdefghijAbcdefghijA
        `41`                                     
        <chr>                                    
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijA
        `48`                                            
        <chr>                                           
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `22`                   `25`                      `2`  
        <chr>                  <chr>                     <chr>
      1 AbcdefghijAbcdefghijAb AbcdefghijAbcdefghijAbcde Ab   
        `8`      `1`   `24`                     `6`   
        <chr>    <chr> <chr>                    <chr> 
      1 Abcdefgh A     AbcdefghijAbcdefghijAbcd Abcdef
        `39`                                   
        <chr>                                  
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghi
        `38`                                  
        <chr>                                 
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `20`                
        <chr>               
      1 AbcdefghijAbcdefghij
        `49`                                             
        <chr>                                            
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefghi
        `45`                                         
        <chr>                                        
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcde
        `47`                                           
        <chr>                                          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefg
        `12`         `9`       `13`         
        <chr>        <chr>     <chr>        
      1 AbcdefghijAb Abcdefghi AbcdefghijAbc
        `36`                                
        <chr>                               
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef
        `26`                      
        <chr>                     
      1 AbcdefghijAbcdefghijAbcdef
        `44`                                         `11`      
        <chr>                                        <chr>     
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcd Abcdefghi~
        `46`          `28`  `7`   `18`  `50`  `16`  `29`  `30` 
        <chr>         <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 AbcdefghijAb~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `4`   `23`    `17`  `40`  `33`  `14`  `27`  `19`  `34` 
        <chr> <chr>   <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcd  Abcdef~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `32`    `3`   `37`  `15`  `10`  `5`   `35`  `31`  `42` 
        <chr>   <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdef~ Abc   Abcd~ Abcd~ Abcd~ Abcde Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(40L, 6L, 25L, 5L, 26L, 17L, 19L, 2L, 11L, 34L, 45L, 24L,
        22L, 44L, 35L, 7L, 4L, 49L, 1L, 36L, 12L, 41L, 39L, 13L, 48L, 27L, 18L, 30L,
        42L, 28L, 3L, 46L, 21L, 20L, 16L, 29L, 50L, 10L, 9L, 8L, 47L, 31L, 14L, 38L,
        33L, 32L, 43L, 23L, 15L, 37L)], width = {
        options(width = 33)
        1217
      })
    Output
      $body
        `40`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghij~
        `6`   
        <chr> 
      1 Abcdef
        `25`                      `5`  
        <chr>                     <chr>
      1 AbcdefghijAbcdefghijAbcde Abcde
        `26`                      
        <chr>                     
      1 AbcdefghijAbcdefghijAbcdef
        `17`             
        <chr>            
      1 AbcdefghijAbcdefg
        `19`                `2`  
        <chr>               <chr>
      1 AbcdefghijAbcdefghi Ab   
        `11`       
        <chr>      
      1 AbcdefghijA
        `34`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghij~
        `45`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghij~
        `24`                    
        <chr>                   
      1 AbcdefghijAbcdefghijAbcd
        `22`                  
        <chr>                 
      1 AbcdefghijAbcdefghijAb
        `44`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghij~
        `35`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghij~
        `7`     `4`  
        <chr>   <chr>
      1 Abcdefg Abcd 
        `49`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghij~
        `1`  
        <chr>
      1 A    
        `36`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghij~
        `12`        
        <chr>       
      1 AbcdefghijAb
        `41`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghij~
        `39`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghij~
        `13`         
        <chr>        
      1 AbcdefghijAbc
        `48`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghij~
        `27`                       
        <chr>                      
      1 AbcdefghijAbcdefghijAbcdefg
        `18`              
        <chr>             
      1 AbcdefghijAbcdefgh
        `30`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghij
        `42`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghij~
        `28`                        
        <chr>                       
      1 AbcdefghijAbcdefghijAbcdefgh
        `3`  
        <chr>
      1 Abc  
        `46`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghij~
        `21`                 
        <chr>                
      1 AbcdefghijAbcdefghijA
        `20`                
        <chr>               
      1 AbcdefghijAbcdefghij
        `16`            
        <chr>           
      1 AbcdefghijAbcdef
        `29`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefghi
        `50`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghij~
        `10`       `9`       `8`     
        <chr>      <chr>     <chr>   
      1 Abcdefghij Abcdefghi Abcdefgh
        `47`    `31`  `14`  `38`  `33` 
        <chr>   <chr> <chr> <chr> <chr>
      1 Abcdef~ Abcd~ Abcd~ Abcd~ Abcd~
        `32`    `43`  `23`  `15`  `37` 
        <chr>   <chr> <chr> <chr> <chr>
      1 Abcdef~ Abcd~ Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(43L, 23L, 22L, 11L, 6L, 26L, 48L, 17L, 7L, 42L, 36L, 21L,
        35L, 50L, 13L, 19L, 29L, 8L, 15L, 4L, 2L, 27L, 49L, 47L, 30L, 31L, 25L, 28L,
        46L, 12L, 32L, 39L, 24L, 10L, 45L, 5L, 37L, 14L, 40L, 20L, 41L, 44L, 33L, 18L,
        38L, 3L, 1L, 34L, 16L, 9L)], width = {
        options(width = 32)
        770
      })
    Output
      $body
        `43`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `23`                   
        <chr>                  
      1 AbcdefghijAbcdefghijAbc
        `22`                  
        <chr>                 
      1 AbcdefghijAbcdefghijAb
        `11`        `6`   
        <chr>       <chr> 
      1 AbcdefghijA Abcdef
        `26`                      
        <chr>                     
      1 AbcdefghijAbcdefghijAbcdef
        `48`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `17`              `7`    
        <chr>             <chr>  
      1 AbcdefghijAbcdefg Abcdefg
        `42`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `36`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `21`                 
        <chr>                
      1 AbcdefghijAbcdefghijA
        `35`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `50`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `13`         
        <chr>        
      1 AbcdefghijAbc
        `19`               
        <chr>              
      1 AbcdefghijAbcdefghi
        `29`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefghi
        `8`      `15`            `4`  
        <chr>    <chr>           <chr>
      1 Abcdefgh AbcdefghijAbcde Abcd 
        `2`  
        <chr>
      1 Ab   
        `27`                       
        <chr>                      
      1 AbcdefghijAbcdefghijAbcdefg
        `49`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `47`                     `30` 
        <chr>                    <chr>
      1 AbcdefghijAbcdefghijAbc~ Abcd~
        `31`   `25`  `28`  `46`  `12` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `32`   `39`  `24`  `10`  `45` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `5`   `37`   `14`  `40`  `20` 
        <chr> <chr>  <chr> <chr> <chr>
      1 Abcde Abcde~ Abcd~ Abcd~ Abcd~
        `41`   `44`  `33`  `18`  `38` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `3`   `1`   `34`   `16`  `9`  
        <chr> <chr> <chr>  <chr> <chr>
      1 Abc   A     Abcde~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(5L, 24L, 43L, 46L, 35L, 39L, 18L, 26L, 8L, 27L, 2L, 50L,
        6L, 14L, 29L, 47L, 9L, 16L, 36L, 4L, 13L, 41L, 3L, 28L, 37L, 33L, 38L, 31L,
        34L, 19L, 42L, 32L, 1L, 45L, 15L, 7L, 11L, 49L, 23L, 12L, 48L, 20L, 21L, 44L,
        25L, 10L, 22L, 30L, 40L, 17L)], width = {
        options(width = 46)
        1439
      })
    Output
      $body
        `5`   `24`                    
        <chr> <chr>                   
      1 Abcde AbcdefghijAbcdefghijAbcd
        `43`                                       
        <chr>                                      
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbc
        `46`                                        
        <chr>                                       
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbc~
        `35`                               
        <chr>                              
      1 AbcdefghijAbcdefghijAbcdefghijAbcde
        `39`                                   
        <chr>                                  
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghi
        `18`              
        <chr>             
      1 AbcdefghijAbcdefgh
        `26`                       `8`     
        <chr>                      <chr>   
      1 AbcdefghijAbcdefghijAbcdef Abcdefgh
        `27`                        `2`  
        <chr>                       <chr>
      1 AbcdefghijAbcdefghijAbcdefg Ab   
        `50`                                        
        <chr>                                       
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbc~
        `6`    `14`          
        <chr>  <chr>         
      1 Abcdef AbcdefghijAbcd
        `29`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefghi
        `47`                                        
        <chr>                                       
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbc~
        `9`       `16`            
        <chr>     <chr>           
      1 Abcdefghi AbcdefghijAbcdef
        `36`                                 `4`  
        <chr>                                <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef Abcd 
        `13`         
        <chr>        
      1 AbcdefghijAbc
        `41`                                     
        <chr>                                    
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijA
        `3`   `28`                        
        <chr> <chr>                       
      1 Abc   AbcdefghijAbcdefghijAbcdefgh
        `37`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefg
        `33`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAbc
        `38`                                  
        <chr>                                 
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `31`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghijA
        `34`                              
        <chr>                             
      1 AbcdefghijAbcdefghijAbcdefghijAbcd
        `19`               
        <chr>              
      1 AbcdefghijAbcdefghi
        `42`                                      
        <chr>                                     
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAb
        `32`                             `1`  
        <chr>                            <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAb A    
        `45`                                        
        <chr>                                       
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbc~
        `15`            `7`     `11`       
        <chr>           <chr>   <chr>      
      1 AbcdefghijAbcde Abcdefg AbcdefghijA
        `49`                                        
        <chr>                                       
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbc~
        `23`                    `12`        
        <chr>                   <chr>       
      1 AbcdefghijAbcdefghijAbc AbcdefghijAb
        `48`                                        
        <chr>                                       
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbc~
        `20`                 `21`                 
        <chr>                <chr>                
      1 AbcdefghijAbcdefghij AbcdefghijAbcdefghijA
        `44`     `25`  `10`  `22`  `30`  `40`  `17` 
        <chr>    <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefg~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(7L, 44L, 19L, 21L, 18L, 35L, 23L, 50L, 33L, 37L, 25L,
        26L, 10L, 39L, 2L, 47L, 42L, 14L, 9L, 41L, 45L, 6L, 4L, 11L, 24L, 43L, 32L,
        3L, 38L, 5L, 49L, 27L, 17L, 8L, 22L, 40L, 12L, 15L, 1L, 28L, 31L, 29L, 13L,
        48L, 34L, 36L, 30L, 20L, 16L, 46L)], width = {
        options(width = 52)
        1065
      })
    Output
      $body
        `7`    
        <chr>  
      1 Abcdefg
        `44`                                        
        <chr>                                       
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcd
        `19`                `21`                 
        <chr>               <chr>                
      1 AbcdefghijAbcdefghi AbcdefghijAbcdefghijA
        `18`              
        <chr>             
      1 AbcdefghijAbcdefgh
        `35`                               
        <chr>                              
      1 AbcdefghijAbcdefghijAbcdefghijAbcde
        `23`                   
        <chr>                  
      1 AbcdefghijAbcdefghijAbc
        `50`                                              
        <chr>                                             
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefghij
        `33`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAbc
        `37`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefg
        `25`                     
        <chr>                    
      1 AbcdefghijAbcdefghijAbcde
        `26`                       `10`      
        <chr>                      <chr>     
      1 AbcdefghijAbcdefghijAbcdef Abcdefghij
        `39`                                    `2`  
        <chr>                                   <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghi Ab   
        `47`                                           
        <chr>                                          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefg
        `42`                                      
        <chr>                                     
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAb
        `14`           `9`      
        <chr>          <chr>    
      1 AbcdefghijAbcd Abcdefghi
        `41`                                     
        <chr>                                    
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijA
        `45`                                         
        <chr>                                        
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcde
        `6`    `4`   `11`        `24`                `43` 
        <chr>  <chr> <chr>       <chr>               <chr>
      1 Abcdef Abcd  AbcdefghijA AbcdefghijAbcdefgh~ Abcd~
        `32`     `3`   `38`  `5`   `49`  `27`  `17`  `8`  
        <chr>    <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefg~ Abc   Abcd~ Abcde Abcd~ Abcd~ Abcd~ Abcd~
        `22`     `40`  `12`  `15`  `1`   `28`  `31`  `29` 
        <chr>    <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefg~ Abcd~ Abcd~ Abcd~ A     Abcd~ Abcd~ Abcd~
        `13`     `48`  `34`  `36`  `30`  `20`  `16`  `46` 
        <chr>    <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefg~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(38L, 18L, 23L, 36L, 35L, 20L, 44L, 19L, 13L, 41L, 31L,
        7L, 11L, 29L, 2L, 14L, 26L, 46L, 40L, 45L, 9L, 34L, 33L, 22L, 1L, 17L, 28L,
        10L, 21L, 30L, 47L, 49L, 6L, 12L, 4L, 25L, 32L, 15L, 43L, 24L, 48L, 3L, 37L,
        50L, 42L, 39L, 16L, 8L, 27L, 5L)], width = {
        options(width = 35)
        393
      })
    Output
      $body
        `38`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAb~
        `18`              
        <chr>             
      1 AbcdefghijAbcdefgh
        `23`                  `36`  `35` 
        <chr>                 <chr> <chr>
      1 AbcdefghijAbcdefghij~ Abcd~ Abcd~
        `20`      `44`  `19`  `13`  `41` 
        <chr>     <chr> <chr> <chr> <chr>
      1 Abcdefgh~ Abcd~ Abcd~ Abcd~ Abcd~
        `31`      `7`   `11`  `29`  `2`  
        <chr>     <chr> <chr> <chr> <chr>
      1 Abcdefgh~ Abcd~ Abcd~ Abcd~ Ab   
        `14`      `26`  `46`  `40`  `45` 
        <chr>     <chr> <chr> <chr> <chr>
      1 Abcdefgh~ Abcd~ Abcd~ Abcd~ Abcd~
        `9`       `34`  `33`  `22`  `1`  
        <chr>     <chr> <chr> <chr> <chr>
      1 Abcdefghi Abcd~ Abcd~ Abcd~ A    
        `17`      `28`  `10`  `21`  `30` 
        <chr>     <chr> <chr> <chr> <chr>
      1 Abcdefgh~ Abcd~ Abcd~ Abcd~ Abcd~
        `47`      `49`  `6`   `12`  `4`  
        <chr>     <chr> <chr> <chr> <chr>
      1 Abcdefgh~ Abcd~ Abcd~ Abcd~ Abcd 
        `25`      `32`  `15`  `43`  `24` 
        <chr>     <chr> <chr> <chr> <chr>
      1 Abcdefgh~ Abcd~ Abcd~ Abcd~ Abcd~
        `48`      `3`   `37`  `50`  `42` 
        <chr>     <chr> <chr> <chr> <chr>
      1 Abcdefgh~ Abc   Abcd~ Abcd~ Abcd~
        `39`      `16`  `8`   `27`  `5`  
        <chr>     <chr> <chr> <chr> <chr>
      1 Abcdefgh~ Abcd~ Abcd~ Abcd~ Abcde
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(22L, 9L, 11L, 26L, 19L, 16L, 32L, 25L, 1L, 30L, 31L, 6L,
        24L, 10L, 39L, 21L, 50L, 7L, 29L, 12L, 46L, 43L, 15L, 35L, 20L, 40L, 49L, 38L,
        36L, 48L, 34L, 3L, 8L, 4L, 27L, 42L, 44L, 33L, 45L, 18L, 5L, 2L, 13L, 47L,
        28L, 17L, 37L, 14L, 41L, 23L)], width = {
        options(width = 41)
        999
      })
    Output
      $body
        `22`                   `9`      
        <chr>                  <chr>    
      1 AbcdefghijAbcdefghijAb Abcdefghi
        `11`        `26`                      
        <chr>       <chr>                     
      1 AbcdefghijA AbcdefghijAbcdefghijAbcdef
        `19`                `16`            
        <chr>               <chr>           
      1 AbcdefghijAbcdefghi AbcdefghijAbcdef
        `32`                            
        <chr>                           
      1 AbcdefghijAbcdefghijAbcdefghijAb
        `25`                      `1`  
        <chr>                     <chr>
      1 AbcdefghijAbcdefghijAbcde A    
        `30`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghij
        `31`                            `6`   
        <chr>                           <chr> 
      1 AbcdefghijAbcdefghijAbcdefghijA Abcdef
        `24`                     `10`      
        <chr>                    <chr>     
      1 AbcdefghijAbcdefghijAbcd Abcdefghij
        `39`                                   
        <chr>                                  
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghi
        `21`                 
        <chr>                
      1 AbcdefghijAbcdefghijA
        `50`                                   
        <chr>                                  
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefgh~
        `7`     `29`                         
        <chr>   <chr>                        
      1 Abcdefg AbcdefghijAbcdefghijAbcdefghi
        `12`        
        <chr>       
      1 AbcdefghijAb
        `46`                                   
        <chr>                                  
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefgh~
        `43`                                   
        <chr>                                  
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefgh~
        `15`           
        <chr>          
      1 AbcdefghijAbcde
        `35`                               
        <chr>                              
      1 AbcdefghijAbcdefghijAbcdefghijAbcde
        `20`                
        <chr>               
      1 AbcdefghijAbcdefghij
        `40`                                   
        <chr>                                  
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefgh~
        `49`                                   
        <chr>                                  
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefgh~
        `38`                                  
        <chr>                                 
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `36`                  `48`  `34`  `3`  
        <chr>                 <chr> <chr> <chr>
      1 AbcdefghijAbcdefghij~ Abcd~ Abcd~ Abc  
        `8`      `4`   `27`   `42`  `44`  `33` 
        <chr>    <chr> <chr>  <chr> <chr> <chr>
      1 Abcdefgh Abcd  Abcde~ Abcd~ Abcd~ Abcd~
        `45`      `18`  `5`   `2`   `13`  `47` 
        <chr>     <chr> <chr> <chr> <chr> <chr>
      1 Abcdefgh~ Abcd~ Abcde Ab    Abcd~ Abcd~
        `28`      `17`  `37`  `14`  `41`  `23` 
        <chr>     <chr> <chr> <chr> <chr> <chr>
      1 Abcdefgh~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      

# smoke test

    Code
      pillar(v, width = 15)
    Output
      <pillar>
      <list<int>>
              [1]
              [3]

---

    Code
      pillar(v, width = 30)
    Output
      <pillar>
      <list<ovrrd_s_>>
                    SC

# output test (not on Windows)

    Code
      colonnade(chartype_frame(), width = 50)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
         chars          desc              
         <chr>          <chr>             
       1 "\u0001\u001f" C0 control code   
       2 "\a\b\f\n\r\t" Named control code
       3 "abcdefuvwxyz" ASCII             
       4 "\u0080\u009f" C1 control code   
       5 " ¡¢£¤¥úûüýþÿ" Latin-1           
       6 "ĀāĂăĄąĆćĈĉĊċ" Unicode           
       7 "！＂＃＄％＆" Unicode wide      
       8 "\u0e00\u2029" Unicode control   
       9 "x­x​x‌x‍x‎x‏x͏x﻿x󠀁x󠀠x󠇯x" Unicode ignorable 
      10 "àáâãāa̅ăȧäảåa̋" Unicode mark      
      11 "😀😁😂😃😄💃" Emoji             
      12 "x\U0010ffffx" Unassigned        
      13 "\xfd\xfe\xff" Invalid           
      14 "\\"           Backslash         
      15 "\""           Quote             

# output test

    Code
      pillar(add_special(as.POSIXct("2017-07-28 18:04:35 +0200")))
    Output
      <pillar>
      <dttm>             
      2017-07-28 18:04:35
      NA                 
    Code
      pillar(add_special(as.POSIXlt("2017-07-28 18:04:35 +0200")))
    Output
      <pillar>
      <dttm>             
      2017-07-28 18:04:35
      NA                 

---

    Code
      pillar(add_special(as.POSIXlt("2017-07-28 18:04:35 +0200")))
    Output
      <pillar>
      <dttm>                  
      2017-07-28 18:04:35.0000
      NA                      

---

    Code
      pillar(add_special(as.difftime(8:11, units = "secs")))
    Output
      <pillar>
      <drtn> 
       8 secs
       9 secs
      10 secs
      11 secs
      NA secs

# tbl_format_footer() results

    Code
      tbl_format_footer(tbl_format_setup(new_tbl(as.list(set_names(letters))), width = 80))
    Output
      [1m<tbl_format_footer(setup)>[22m
      [90m# i 13 more variables: n <chr>, o <chr>, p <chr>, q <chr>, r <chr>, s <chr>,[39m
      [90m#   t <chr>, u <chr>, v <chr>, w <chr>, x <chr>, y <chr>, z <chr>[39m
    Code
      tbl_format_footer(tbl_format_setup(new_tbl(as.list(set_names(letters))), width = 30))
    Output
      [1m<tbl_format_footer(setup)>[22m
      [90m# i 22 more variables:[39m
      [90m#   e <chr>, f <chr>,[39m
      [90m#   g <chr>, h <chr>,[39m
      [90m#   i <chr>, j <chr>,[39m
      [90m#   k <chr>, l <chr>,[39m
      [90m#   m <chr>, n <chr>,[39m
      [90m#   o <chr>, p <chr>, ...[39m

# wrapping column names with spaces in the footer

    Code
      tbl_format_footer(tbl_format_setup(new_tbl(list2(!!!set_names(letters), !!paste(
        letters, collapse = " ") := 2)), width = 70))
    Output
      <tbl_format_footer(setup)>
      # i 16 more variables: l <chr>, m <chr>, n <chr>, o <chr>, p <chr>,
      #   q <chr>, r <chr>, s <chr>, t <chr>, u <chr>, v <chr>, w <chr>,
      #   x <chr>, y <chr>, z <chr>,
      #   `a b c d e f g h i j k l m n o p q r s t u v w x y z` <dbl>

# overflow

    Code
      tbl_format_footer(tbl_format_setup(new_tbl(list2(!!!set_names(rep(letters, 6)))),
      width = 70))
    Output
      <tbl_format_footer(setup)>
      # i 145 more variables: l <chr>, m <chr>, n <chr>, o <chr>, p <chr>,
      #   q <chr>, r <chr>, s <chr>, t <chr>, u <chr>, v <chr>, w <chr>,
      #   x <chr>, y <chr>, z <chr>, a <chr>, b <chr>, c <chr>, d <chr>,
      #   e <chr>, f <chr>, g <chr>, h <chr>, i <chr>, j <chr>, k <chr>,
      #   l <chr>, m <chr>, n <chr>, o <chr>, p <chr>, q <chr>, r <chr>,
      #   s <chr>, t <chr>, u <chr>, v <chr>, w <chr>, x <chr>, y <chr>,
      #   z <chr>, a <chr>, b <chr>, c <chr>, d <chr>, e <chr>, ...

# max_footer_lines option

    Code
      tbl_format_footer(tbl_format_setup(new_footer_tbl("")))
    Output
      <tbl_format_footer(setup)>
      # i 39 more variables: bg <int>, ah <int>, bh <int>, ai <int>, bi <int>,
      #   aj <int>, bj <int>, ak <int>, bk <int>, al <int>, bl <int>, am <int>,
      #   bm <int>, an <int>, bn <int>, ao <int>, bo <int>, ap <int>, bp <int>,
      #   aq <int>, bq <int>, ar <int>, br <int>, as <int>, bs <int>, at <int>,
      #   bt <int>, au <int>, bu <int>, av <int>, bv <int>, aw <int>, bw <int>,
      #   ax <int>, bx <int>, ay <int>, by <int>, az <int>, bz <int>
    Code
      tbl_format_footer(tbl_format_setup(new_footer_tbl("prefix_")))
    Output
      <tbl_format_footer(setup)>
      # i 45 more variables: prefix_bd <int>, prefix_ae <int>, prefix_be <int>,
      #   prefix_af <int>, prefix_bf <int>, prefix_ag <int>, prefix_bg <int>,
      #   prefix_ah <int>, prefix_bh <int>, prefix_ai <int>, prefix_bi <int>,
      #   prefix_aj <int>, prefix_bj <int>, prefix_ak <int>, prefix_bk <int>,
      #   prefix_al <int>, prefix_bl <int>, prefix_am <int>, prefix_bm <int>,
      #   prefix_an <int>, prefix_bn <int>, prefix_ao <int>, prefix_bo <int>,
      #   prefix_ap <int>, prefix_bp <int>, prefix_aq <int>, prefix_bq <int>, ...
    Code
      tbl_format_footer(tbl_format_setup(new_footer_tbl("a_very_long_prefix_")))
    Output
      <tbl_format_footer(setup)>
      # i 49 more variables: a_very_long_prefix_bb <int>,
      #   a_very_long_prefix_ac <int>, a_very_long_prefix_bc <int>,
      #   a_very_long_prefix_ad <int>, a_very_long_prefix_bd <int>,
      #   a_very_long_prefix_ae <int>, a_very_long_prefix_be <int>,
      #   a_very_long_prefix_af <int>, a_very_long_prefix_bf <int>,
      #   a_very_long_prefix_ag <int>, a_very_long_prefix_bg <int>,
      #   a_very_long_prefix_ah <int>, a_very_long_prefix_bh <int>, ...
    Code
      tbl_format_footer(tbl_format_setup(new_footer_tbl(""), max_footer_lines = 3))
    Output
      <tbl_format_footer(setup)>
      # i 39 more variables: bg <int>, ah <int>, bh <int>, ai <int>, bi <int>,
      #   aj <int>, bj <int>, ak <int>, bk <int>, al <int>, bl <int>, am <int>,
      #   bm <int>, an <int>, bn <int>, ao <int>, bo <int>, ap <int>, bp <int>, ...
    Code
      tbl_format_footer(tbl_format_setup(new_footer_tbl("prefix_"), max_footer_lines = 3))
    Output
      <tbl_format_footer(setup)>
      # i 45 more variables: prefix_bd <int>, prefix_ae <int>, prefix_be <int>,
      #   prefix_af <int>, prefix_bf <int>, prefix_ag <int>, prefix_bg <int>,
      #   prefix_ah <int>, prefix_bh <int>, prefix_ai <int>, prefix_bi <int>, ...
    Code
      tbl_format_footer(tbl_format_setup(new_footer_tbl("a_very_long_prefix_"),
      max_footer_lines = 3))
    Output
      <tbl_format_footer(setup)>
      # i 49 more variables: a_very_long_prefix_bb <int>,
      #   a_very_long_prefix_ac <int>, a_very_long_prefix_bc <int>,
      #   a_very_long_prefix_ad <int>, a_very_long_prefix_bd <int>, ...
    Code
      tbl_format_footer(tbl_format_setup(new_footer_tbl(""), max_footer_lines = Inf))
    Output
      <tbl_format_footer(setup)>
      # i 39 more variables: bg <int>, ah <int>, bh <int>, ai <int>, bi <int>,
      #   aj <int>, bj <int>, ak <int>, bk <int>, al <int>, bl <int>, am <int>,
      #   bm <int>, an <int>, bn <int>, ao <int>, bo <int>, ap <int>, bp <int>,
      #   aq <int>, bq <int>, ar <int>, br <int>, as <int>, bs <int>, at <int>,
      #   bt <int>, au <int>, bu <int>, av <int>, bv <int>, aw <int>, bw <int>,
      #   ax <int>, bx <int>, ay <int>, by <int>, az <int>, bz <int>
    Code
      tbl_format_footer(tbl_format_setup(new_footer_tbl("prefix_"), max_footer_lines = Inf))
    Output
      <tbl_format_footer(setup)>
      # i 45 more variables: prefix_bd <int>, prefix_ae <int>, prefix_be <int>,
      #   prefix_af <int>, prefix_bf <int>, prefix_ag <int>, prefix_bg <int>,
      #   prefix_ah <int>, prefix_bh <int>, prefix_ai <int>, prefix_bi <int>,
      #   prefix_aj <int>, prefix_bj <int>, prefix_ak <int>, prefix_bk <int>,
      #   prefix_al <int>, prefix_bl <int>, prefix_am <int>, prefix_bm <int>,
      #   prefix_an <int>, prefix_bn <int>, prefix_ao <int>, prefix_bo <int>,
      #   prefix_ap <int>, prefix_bp <int>, prefix_aq <int>, prefix_bq <int>,
      #   prefix_ar <int>, prefix_br <int>, prefix_as <int>, prefix_bs <int>,
      #   prefix_at <int>, prefix_bt <int>, prefix_au <int>, prefix_bu <int>,
      #   prefix_av <int>, prefix_bv <int>, prefix_aw <int>, prefix_bw <int>,
      #   prefix_ax <int>, prefix_bx <int>, prefix_ay <int>, prefix_by <int>,
      #   prefix_az <int>, prefix_bz <int>
    Code
      tbl_format_footer(tbl_format_setup(new_footer_tbl("a_very_long_prefix_"),
      max_footer_lines = Inf))
    Output
      <tbl_format_footer(setup)>
      # i 49 more variables: a_very_long_prefix_bb <int>,
      #   a_very_long_prefix_ac <int>, a_very_long_prefix_bc <int>,
      #   a_very_long_prefix_ad <int>, a_very_long_prefix_bd <int>,
      #   a_very_long_prefix_ae <int>, a_very_long_prefix_be <int>,
      #   a_very_long_prefix_af <int>, a_very_long_prefix_bf <int>,
      #   a_very_long_prefix_ag <int>, a_very_long_prefix_bg <int>,
      #   a_very_long_prefix_ah <int>, a_very_long_prefix_bh <int>,
      #   a_very_long_prefix_ai <int>, a_very_long_prefix_bi <int>,
      #   a_very_long_prefix_aj <int>, a_very_long_prefix_bj <int>,
      #   a_very_long_prefix_ak <int>, a_very_long_prefix_bk <int>,
      #   a_very_long_prefix_al <int>, a_very_long_prefix_bl <int>,
      #   a_very_long_prefix_am <int>, a_very_long_prefix_bm <int>,
      #   a_very_long_prefix_an <int>, a_very_long_prefix_bn <int>,
      #   a_very_long_prefix_ao <int>, a_very_long_prefix_bo <int>,
      #   a_very_long_prefix_ap <int>, a_very_long_prefix_bp <int>,
      #   a_very_long_prefix_aq <int>, a_very_long_prefix_bq <int>,
      #   a_very_long_prefix_ar <int>, a_very_long_prefix_br <int>,
      #   a_very_long_prefix_as <int>, a_very_long_prefix_bs <int>,
      #   a_very_long_prefix_at <int>, a_very_long_prefix_bt <int>,
      #   a_very_long_prefix_au <int>, a_very_long_prefix_bu <int>,
      #   a_very_long_prefix_av <int>, a_very_long_prefix_bv <int>,
      #   a_very_long_prefix_aw <int>, a_very_long_prefix_bw <int>,
      #   a_very_long_prefix_ax <int>, a_very_long_prefix_bx <int>,
      #   a_very_long_prefix_ay <int>, a_very_long_prefix_by <int>,
      #   a_very_long_prefix_az <int>, a_very_long_prefix_bz <int>

# pillar.advice option (#575)

    Code
      tbl_format_footer(tbl_format_setup(new_tbl(as.list(set_names(letters))), width = 80))
    Output
      <tbl_format_footer(setup)>
      # i 13 more variables: n <chr>, o <chr>, p <chr>, q <chr>, r <chr>, s <chr>,
      #   t <chr>, u <chr>, v <chr>, w <chr>, x <chr>, y <chr>, z <chr>

# colnames() hint (#622)

    Code
      tbl_format_footer(tbl_format_setup(new_tbl(as.list(set_names(letters2))),
      width = 80))
    Output
      <tbl_format_footer(setup)>
      # i 663 more variables: na <chr>, oa <chr>, pa <chr>, qa <chr>, ra <chr>,
      #   sa <chr>, ta <chr>, ua <chr>, va <chr>, wa <chr>, xa <chr>, ya <chr>,
      #   za <chr>, ab <chr>, bb <chr>, cb <chr>, db <chr>, eb <chr>, fb <chr>,
      #   gb <chr>, hb <chr>, ib <chr>, jb <chr>, kb <chr>, lb <chr>, mb <chr>,
      #   nb <chr>, ob <chr>, pb <chr>, qb <chr>, rb <chr>, sb <chr>, tb <chr>,
      #   ub <chr>, vb <chr>, wb <chr>, xb <chr>, yb <chr>, zb <chr>, ac <chr>,
      #   bc <chr>, cc <chr>, dc <chr>, ec <chr>, fc <chr>, gc <chr>, hc <chr>, ...
      # i Use `colnames()` to see all variable names

# advice when interactive (#575)

    Code
      tbl_format_footer(tbl_format_setup(new_tbl(as.list(set_names(letters))), width = 80))
    Output
      <tbl_format_footer(setup)>
      # i 13 more variables: n <chr>, o <chr>, p <chr>, q <chr>, r <chr>, s <chr>,
      #   t <chr>, u <chr>, v <chr>, w <chr>, x <chr>, y <chr>, z <chr>
    Code
      tbl_format_footer(tbl_format_setup(new_tbl(list(a = 1:30)), width = 80))
    Output
      <tbl_format_footer(setup)>
      # i 20 more rows
      # i Use `print(n = ...)` to see more rows

# output test

    Code
      colonnade(x, width = 4)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Code
      colonnade(x, width = 5)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Code
      colonnade(x, width = 6)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Code
      colonnade(x, width = 7)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        colu~
        <dbl>
      1  1.23
      2  2.23
      3  3.23
    Code
      colonnade(x, width = 8)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        colum~
         <dbl>
      1   1.23
      2   2.23
      3   3.23
    Code
      colonnade(x, width = 9)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column~
          <dbl>
      1    1.23
      2    2.23
      3    3.23
    Code
      colonnade(x, width = 10)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_~
           <dbl>
      1     1.23
      2     2.23
      3     3.23
    Code
      colonnade(x, width = 11)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_z~
            <dbl>
      1      1.23
      2      2.23
      3      3.23
    Code
      colonnade(x, width = 12)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_ze~
             <dbl>
      1       1.23
      2       2.23
      3       3.23
    Code
      colonnade(x, width = 13)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zer~
              <dbl>
      1        1.23
      2        2.23
      3        3.23
    Code
      colonnade(x, width = 14)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero~
               <dbl>
      1         1.23
      2         2.23
      3         3.23
    Code
      colonnade(x, width = 15)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_~
                <dbl>
      1          1.23
      2          2.23
      3          3.23
    Code
      colonnade(x, width = 16)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_o~
                 <dbl>
      1           1.23
      2           2.23
      3           3.23
    Code
      colonnade(x, width = 17)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one
                  <dbl>
      1            1.23
      2            2.23
      3            3.23
    Code
      colonnade(x, width = 18)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one
                  <dbl>
      1            1.23
      2            2.23
      3            3.23
    Code
      colonnade(x, width = 19)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one
                  <dbl>
      1            1.23
      2            2.23
      3            3.23
    Code
      colonnade(x, width = 20)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one
                  <dbl>
      1            1.23
      2            2.23
      3            3.23
    Code
      colonnade(x, width = 21)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one
                  <dbl>
      1            1.23
      2            2.23
      3            3.23
    Code
      colonnade(x, width = 22)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one
                  <dbl>
      1            1.23
      2            2.23
      3            3.23
    Code
      colonnade(x, width = 23)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one
                  <dbl>
      1            1.23
      2            2.23
      3            3.23
    Code
      colonnade(x, width = 24)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one col_02
                  <dbl> <chr> 
      1            1.23 a     
      2            2.23 b     
      3            3.23 c     
    Code
      colonnade(x, width = 25)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one col_02
                  <dbl> <chr> 
      1            1.23 a     
      2            2.23 b     
      3            3.23 c     
    Code
      colonnade(x, width = 26)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one col_02
                  <dbl> <chr> 
      1            1.23 a     
      2            2.23 b     
      3            3.23 c     
    Code
      colonnade(x, width = 27)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one col_02
                  <dbl> <chr> 
      1            1.23 a     
      2            2.23 b     
      3            3.23 c     
    Code
      colonnade(x, width = 28)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one col_02
                  <dbl> <chr> 
      1            1.23 a     
      2            2.23 b     
      3            3.23 c     
    Code
      colonnade(x, width = 29)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one col_02
                  <dbl> <chr> 
      1            1.23 a     
      2            2.23 b     
      3            3.23 c     
    Code
      colonnade(x, width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one col_02
                  <dbl> <chr> 
      1            1.23 a     
      2            2.23 b     
      3            3.23 c     
    Code
      colonnade(x, width = 31)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one col_02 col_03
                  <dbl> <chr>  <fct> 
      1            1.23 a      a     
      2            2.23 b      b     
      3            3.23 c      c     
    Code
      colonnade(x, width = 32)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one col_02 col_03
                  <dbl> <chr>  <fct> 
      1            1.23 a      a     
      2            2.23 b      b     
      3            3.23 c      c     
    Code
      colonnade(x, width = 33)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one col_02 col_03
                  <dbl> <chr>  <fct> 
      1            1.23 a      a     
      2            2.23 b      b     
      3            3.23 c      c     
    Code
      colonnade(x, width = 34)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one col_02 col_03
                  <dbl> <chr>  <fct> 
      1            1.23 a      a     
      2            2.23 b      b     
      3            3.23 c      c     
    Code
      colonnade(x, width = 35)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one col_02 col_03
                  <dbl> <chr>  <fct> 
      1            1.23 a      a     
      2            2.23 b      b     
      3            3.23 c      c     
    Code
      colonnade(x, width = 36)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one col_02 col_03
                  <dbl> <chr>  <fct> 
      1            1.23 a      a     
      2            2.23 b      b     
      3            3.23 c      c     
    Code
      colonnade(x, width = 37)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one col_02 col_03
                  <dbl> <chr>  <fct> 
      1            1.23 a      a     
      2            2.23 b      b     
      3            3.23 c      c     
    Code
      colonnade(x, width = 38)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one col_02 col_03 col_04
                  <dbl> <chr>  <fct>  <ord> 
      1            1.23 a      a      a     
      2            2.23 b      b      b     
      3            3.23 c      c      c     
    Code
      colonnade(x, width = 39)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one col_02 col_03 col_04
                  <dbl> <chr>  <fct>  <ord> 
      1            1.23 a      a      a     
      2            2.23 b      b      b     
      3            3.23 c      c      c     
    Code
      colonnade(x, width = Inf)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one col_02 col_03 col_04
                  <dbl> <chr>  <fct>  <ord> 
      1            1.23 a      a      a     
      2            2.23 b      b      b     
      3            3.23 c      c      c     

---

    Code
      colonnade(rep(list(paste(letters, collapse = " ")), 4), width = Inf)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        <chr>                                              
      1 a b c d e f g h i j k l m n o p q r s t u v w x y z
        <chr>                                              
      1 a b c d e f g h i j k l m n o p q r s t u v w x y z
        <chr>                                              
      1 a b c d e f g h i j k l m n o p q r s t u v w x y z
        <chr>                                              
      1 a b c d e f g h i j k l m n o p q r s t u v w x y z

---

    Code
      as_glue(extra_cols_impl(squeeze_impl(colonnade(x), width = 10)))
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
      col_02 <chr>
      col_03 <fct>
      col_04 <ord>

---

    Code
      as_glue(extra_cols_impl(squeeze_impl(colonnade(x), width = 20)))
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
      col_02 <chr>
      col_03 <fct>
      col_04 <ord>

---

    Code
      as_glue(extra_cols_impl(squeeze_impl(colonnade(x), width = 30)))
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
      col_03 <fct>
      col_04 <ord>

---

    Code
      as_glue(extra_cols_impl(squeeze_impl(colonnade(x), width = 35)))
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
      col_04 <ord>

---

    Code
      as_glue(extra_cols_impl(squeeze_impl(colonnade(x), width = 40)))
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.

# tests from tibble

    Code
      colonnade(mtcars[1:8, ], has_row_id = "*", width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
          mpg   cyl  disp    hp
      * <dbl> <dbl> <dbl> <dbl>
      1  21       6  160    110
      2  21       6  160    110
      3  22.8     4  108     93
      4  21.4     6  258    110
      5  18.7     8  360    175
      6  18.1     6  225    105
      7  14.3     8  360    245
      8  24.4     4  147.    62
    Code
      colonnade(trees[1:5, ], width = 20)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        Girth Height
        <dbl>  <dbl>
      1   8.3     70
      2   8.6     65
      3   8.8     63
      4  10.5     72
      5  10.7     81
    Code
      colonnade(trees[1:3, ], width = 10)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        Girth
        <dbl>
      1   8.3
      2   8.6
      3   8.8
    Code
      colonnade(df_all, width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a     b c     d    
        <dbl> <int> <lgl> <chr>
      1   1       1 TRUE  a    
      2   2.5     2 FALSE b    
      3  NA      NA NA    <NA> 
    Code
      colonnade(df_all, width = 300)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a     b c     d     e     f          g                   h        
        <dbl> <int> <lgl> <chr> <fct> <date>     <dttm>              <list>   
      1   1       1 TRUE  a     a     2015-12-10 2015-12-09 10:51:35 <int [1]>
      2   2.5     2 FALSE b     b     2015-12-11 2015-12-09 10:51:36 <int [1]>
      3  NA      NA NA    <NA>  <NA>  NA         NA                  <int [1]>
        i         
        <list>    
      1 <list [2]>
      2 <list [1]>
      3 <list [1]>
    Code
      options(width = 70)
      colonnade(df_all, width = 300)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a     b c     d     e     f          g                  
        <dbl> <int> <lgl> <chr> <fct> <date>     <dttm>             
      1   1       1 TRUE  a     a     2015-12-10 2015-12-09 10:51:35
      2   2.5     2 FALSE b     b     2015-12-11 2015-12-09 10:51:36
      3  NA      NA NA    <NA>  <NA>  NA         NA                 
        h         i         
        <list>    <list>    
      1 <int [1]> <list [2]>
      2 <int [1]> <list [1]>
      3 <int [1]> <list [1]>
    Code
      options(width = 60)
      colonnade(df_all, width = 300)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a     b c     d     e     f         
        <dbl> <int> <lgl> <chr> <fct> <date>    
      1   1       1 TRUE  a     a     2015-12-10
      2   2.5     2 FALSE b     b     2015-12-11
      3  NA      NA NA    <NA>  <NA>  NA        
        g                   h         i         
        <dttm>              <list>    <list>    
      1 2015-12-09 10:51:35 <int [1]> <list [2]>
      2 2015-12-09 10:51:36 <int [1]> <list [1]>
      3 NA                  <int [1]> <list [1]>
    Code
      options(width = 50)
      colonnade(df_all, width = 300)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a     b c     d     e     f         
        <dbl> <int> <lgl> <chr> <fct> <date>    
      1   1       1 TRUE  a     a     2015-12-10
      2   2.5     2 FALSE b     b     2015-12-11
      3  NA      NA NA    <NA>  <NA>  NA        
        g                   h         i         
        <dttm>              <list>    <list>    
      1 2015-12-09 10:51:35 <int [1]> <list [2]>
      2 2015-12-09 10:51:36 <int [1]> <list [1]>
      3 NA                  <int [1]> <list [1]>
    Code
      options(width = 40)
      colonnade(df_all, width = 300)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a     b c     d     e    
        <dbl> <int> <lgl> <chr> <fct>
      1   1       1 TRUE  a     a    
      2   2.5     2 FALSE b     b    
      3  NA      NA NA    <NA>  <NA> 
        f          g                  
        <date>     <dttm>             
      1 2015-12-10 2015-12-09 10:51:35
      2 2015-12-11 2015-12-09 10:51:36
      3 NA         NA                 
        h         i         
        <list>    <list>    
      1 <int [1]> <list [2]>
      2 <int [1]> <list [1]>
      3 <int [1]> <list [1]>
    Code
      options(width = 30)
      colonnade(df_all, width = 300)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a     b c     d    
        <dbl> <int> <lgl> <chr>
      1   1       1 TRUE  a    
      2   2.5     2 FALSE b    
      3  NA      NA NA    <NA> 
        e     f         
        <fct> <date>    
      1 a     2015-12-10
      2 b     2015-12-11
      3 <NA>  NA        
        g                  
        <dttm>             
      1 2015-12-09 10:51:35
      2 2015-12-09 10:51:36
      3 NA                 
        h         i         
        <list>    <list>    
      1 <int [1]> <list [2]>
      2 <int [1]> <list [1]>
      3 <int [1]> <list [1]>
    Code
      options(width = 20)
      colonnade(df_all, width = 300)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a     b c    
        <dbl> <int> <lgl>
      1   1       1 TRUE 
      2   2.5     2 FALSE
      3  NA      NA NA   
        d     e    
        <chr> <fct>
      1 a     a    
      2 b     b    
      3 <NA>  <NA> 
        f         
        <date>    
      1 2015-12-10
      2 2015-12-11
      3 NA        
        g                 
        <dttm>            
      1 2015-12-09 10:51:~
      2 2015-12-09 10:51:~
      3 NA                
        h        
        <list>   
      1 <int [1]>
      2 <int [1]>
      3 <int [1]>
        i         
        <list>    
      1 <list [2]>
      2 <list [1]>
      3 <list [1]>
    Code
      list_with_ctl <- list(c("\n", "\""), factor(c("\n", "\n")))
      names(list_with_ctl) <- c("\n", "\r")
      colonnade(list_with_ctl, width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        `\n`  `\r` 
        <chr> <fct>
      1 "\n"  "\n" 
      2 "\""  "\n" 
    Code
      colonnade(list(a = c("", " ", "a ", " a")), width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        a    
        <chr>
      1 ""   
      2 " "  
      3 "a " 
      4 " a" 
    Code
      colonnade(list(`mean(x)` = 5, `var(x)` = 3), width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        `mean(x)` `var(x)`
            <dbl>    <dbl>
      1         5        3

# NA names

    Code
      colonnade(x, width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
         `NA` <int>
        <int> <int>
      1     1     4
      2     2     5
      3     3     6

# sep argument

    Code
      colonnade(x, width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
          sep
        <int>
      1     1
      2     2
      3     3
    Code
      # dummy

# tibble columns

    Code
      colonnade(x, width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a   b$c    $d
        <int> <int> <int>
      1     1     4     7
      2     2     5     8
      3     3     6     9

# tibble columns (nested)

    Code
      colonnade(x, width = 40)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a   b$c    $d  $e$f   $$g
        <int> <int> <int> <int> <int>
      1     1     4     7    10    13
      2     2     5     8    11    14
      3     3     6     9    12    15

# tibble columns (empty)

    Code
      colonnade(x, width = 40)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a   b$c    $d $e          $f
        <int> <int> <int> <df[,0]> <int>
      1     1     4     7             10
      2     2     5     8             11
      3     3     6     9             12

# matrix columns (unnamed)

    Code
      colonnade(x, width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a b[,1]  [,2]
        <int> <int> <int>
      1     1     4     7
      2     2     5     8
      3     3     6     9

# matrix columns (named)

    Code
      colonnade(x, width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a b[,"c"] [,"d"]
        <int>   <int>  <int>
      1     1       4      7
      2     2       5      8
      3     3       6      9

# matrix columns (empty)

    Code
      colonnade(x, width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a b             c
        <int> <int[,0]> <int>
      1     1               4
      2     2               5
      3     3               6

# tbl_format_footer() results

    Code
      tbl_format_footer(tbl_format_setup(new_tbl(as.list(set_names(letters))), width = 80))
    Output
      [1m<tbl_format_footer(setup)>[22m
      [90m# ... with 13 more variables: n <chr>, o <chr>, p <chr>, q <chr>, r <chr>,[39m
      [90m#   s <chr>, t <chr>, u <chr>, v <chr>, w <chr>, x <chr>, y <chr>, z <chr>[39m
      [90m# i Use `colnames()` to see all variable names[39m
    Code
      tbl_format_footer(tbl_format_setup(new_tbl(as.list(set_names(letters))), width = 30))
    Output
      [1m<tbl_format_footer(setup)>[22m
      [90m# ... with 22 more variables:[39m
      [90m#   e <chr>, f <chr>,[39m
      [90m#   g <chr>, h <chr>,[39m
      [90m#   i <chr>, j <chr>,[39m
      [90m#   k <chr>, l <chr>,[39m
      [90m#   m <chr>, n <chr>,[39m
      [90m#   o <chr>, p <chr>, ...[39m
      [90m# i Use `colnames()` to see all variable names[39m

# wrapping column names with spaces in the footer

    Code
      tbl_format_footer(tbl_format_setup(new_tbl(list2(!!!set_names(letters), !!paste(
        letters, collapse = " ") := 2)), width = 70))
    Output
      <tbl_format_footer(setup)>
      # ... with 16 more variables: l <chr>, m <chr>, n <chr>, o <chr>,
      #   p <chr>, q <chr>, r <chr>, s <chr>, t <chr>, u <chr>, v <chr>,
      #   w <chr>, x <chr>, y <chr>, z <chr>,
      #   `a b c d e f g h i j k l m n o p q r s t u v w x y z` <dbl>
      # i Use `colnames()` to see all variable names

# overflow

    Code
      tbl_format_footer(tbl_format_setup(new_tbl(list2(!!!set_names(rep(letters, 6)))),
      width = 70))
    Output
      <tbl_format_footer(setup)>
      # ... with 145 more variables: l <chr>, m <chr>, n <chr>, o <chr>,
      #   p <chr>, q <chr>, r <chr>, s <chr>, t <chr>, u <chr>, v <chr>,
      #   w <chr>, x <chr>, y <chr>, z <chr>, a <chr>, b <chr>, c <chr>,
      #   d <chr>, e <chr>, f <chr>, g <chr>, h <chr>, i <chr>, j <chr>,
      #   k <chr>, l <chr>, m <chr>, n <chr>, o <chr>, p <chr>, q <chr>,
      #   r <chr>, s <chr>, t <chr>, u <chr>, v <chr>, w <chr>, x <chr>,
      #   y <chr>, z <chr>, a <chr>, b <chr>, c <chr>, d <chr>, ...
      # i Use `colnames()` to see all variable names

# max_footer_lines option

    Code
      tbl_format_footer(tbl_format_setup(new_footer_tbl("")))
    Output
      <tbl_format_footer(setup)>
      # ... with 39 more variables: bg <int>, ah <int>, bh <int>, ai <int>, bi <int>,
      #   aj <int>, bj <int>, ak <int>, bk <int>, al <int>, bl <int>, am <int>,
      #   bm <int>, an <int>, bn <int>, ao <int>, bo <int>, ap <int>, bp <int>,
      #   aq <int>, bq <int>, ar <int>, br <int>, as <int>, bs <int>, at <int>,
      #   bt <int>, au <int>, bu <int>, av <int>, bv <int>, aw <int>, bw <int>,
      #   ax <int>, bx <int>, ay <int>, by <int>, az <int>, bz <int>
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_footer(tbl_format_setup(new_footer_tbl("prefix_")))
    Output
      <tbl_format_footer(setup)>
      # ... with 43 more variables: prefix_be <int>, prefix_af <int>,
      #   prefix_bf <int>, prefix_ag <int>, prefix_bg <int>, prefix_ah <int>,
      #   prefix_bh <int>, prefix_ai <int>, prefix_bi <int>, prefix_aj <int>,
      #   prefix_bj <int>, prefix_ak <int>, prefix_bk <int>, prefix_al <int>,
      #   prefix_bl <int>, prefix_am <int>, prefix_bm <int>, prefix_an <int>,
      #   prefix_bn <int>, prefix_ao <int>, prefix_bo <int>, prefix_ap <int>,
      #   prefix_bp <int>, prefix_aq <int>, prefix_bq <int>, prefix_ar <int>, ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_footer(tbl_format_setup(new_footer_tbl("a_very_long_prefix_")))
    Output
      <tbl_format_footer(setup)>
      # ... with 43 more variables: a_very_long_prefix_be <int>,
      #   a_very_long_prefix_af <int>, a_very_long_prefix_bf <int>,
      #   a_very_long_prefix_ag <int>, a_very_long_prefix_bg <int>,
      #   a_very_long_prefix_ah <int>, a_very_long_prefix_bh <int>,
      #   a_very_long_prefix_ai <int>, a_very_long_prefix_bi <int>,
      #   a_very_long_prefix_aj <int>, a_very_long_prefix_bj <int>,
      #   a_very_long_prefix_ak <int>, a_very_long_prefix_bk <int>, ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_footer(tbl_format_setup(new_footer_tbl(""), max_footer_lines = 3))
    Output
      <tbl_format_footer(setup)>
      # ... with 39 more variables: bg <int>, ah <int>, bh <int>, ai <int>, bi <int>,
      #   aj <int>, bj <int>, ak <int>, bk <int>, al <int>, bl <int>, am <int>,
      #   bm <int>, an <int>, bn <int>, ao <int>, bo <int>, ap <int>, bp <int>, ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_footer(tbl_format_setup(new_footer_tbl("prefix_"), max_footer_lines = 3))
    Output
      <tbl_format_footer(setup)>
      # ... with 43 more variables: prefix_be <int>, prefix_af <int>,
      #   prefix_bf <int>, prefix_ag <int>, prefix_bg <int>, prefix_ah <int>,
      #   prefix_bh <int>, prefix_ai <int>, prefix_bi <int>, prefix_aj <int>, ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_footer(tbl_format_setup(new_footer_tbl("a_very_long_prefix_"),
      max_footer_lines = 3))
    Output
      <tbl_format_footer(setup)>
      # ... with 43 more variables: a_very_long_prefix_be <int>,
      #   a_very_long_prefix_af <int>, a_very_long_prefix_bf <int>,
      #   a_very_long_prefix_ag <int>, a_very_long_prefix_bg <int>, ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_footer(tbl_format_setup(new_footer_tbl(""), max_footer_lines = Inf))
    Output
      <tbl_format_footer(setup)>
      # ... with 39 more variables: bg <int>, ah <int>, bh <int>, ai <int>, bi <int>,
      #   aj <int>, bj <int>, ak <int>, bk <int>, al <int>, bl <int>, am <int>,
      #   bm <int>, an <int>, bn <int>, ao <int>, bo <int>, ap <int>, bp <int>,
      #   aq <int>, bq <int>, ar <int>, br <int>, as <int>, bs <int>, at <int>,
      #   bt <int>, au <int>, bu <int>, av <int>, bv <int>, aw <int>, bw <int>,
      #   ax <int>, bx <int>, ay <int>, by <int>, az <int>, bz <int>
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_footer(tbl_format_setup(new_footer_tbl("prefix_"), max_footer_lines = Inf))
    Output
      <tbl_format_footer(setup)>
      # ... with 43 more variables: prefix_be <int>, prefix_af <int>,
      #   prefix_bf <int>, prefix_ag <int>, prefix_bg <int>, prefix_ah <int>,
      #   prefix_bh <int>, prefix_ai <int>, prefix_bi <int>, prefix_aj <int>,
      #   prefix_bj <int>, prefix_ak <int>, prefix_bk <int>, prefix_al <int>,
      #   prefix_bl <int>, prefix_am <int>, prefix_bm <int>, prefix_an <int>,
      #   prefix_bn <int>, prefix_ao <int>, prefix_bo <int>, prefix_ap <int>,
      #   prefix_bp <int>, prefix_aq <int>, prefix_bq <int>, prefix_ar <int>,
      #   prefix_br <int>, prefix_as <int>, prefix_bs <int>, prefix_at <int>,
      #   prefix_bt <int>, prefix_au <int>, prefix_bu <int>, prefix_av <int>,
      #   prefix_bv <int>, prefix_aw <int>, prefix_bw <int>, prefix_ax <int>,
      #   prefix_bx <int>, prefix_ay <int>, prefix_by <int>, prefix_az <int>,
      #   prefix_bz <int>, and abbreviated variable names 1: prefix_bb, 2: prefix_ac,
      #   3: prefix_bc, 4: prefix_ad, 5: prefix_bd, 6: prefix_ae
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_footer(tbl_format_setup(new_footer_tbl("a_very_long_prefix_"),
      max_footer_lines = Inf))
    Output
      <tbl_format_footer(setup)>
      # ... with 43 more variables: a_very_long_prefix_be <int>,
      #   a_very_long_prefix_af <int>, a_very_long_prefix_bf <int>,
      #   a_very_long_prefix_ag <int>, a_very_long_prefix_bg <int>,
      #   a_very_long_prefix_ah <int>, a_very_long_prefix_bh <int>,
      #   a_very_long_prefix_ai <int>, a_very_long_prefix_bi <int>,
      #   a_very_long_prefix_aj <int>, a_very_long_prefix_bj <int>,
      #   a_very_long_prefix_ak <int>, a_very_long_prefix_bk <int>,
      #   a_very_long_prefix_al <int>, a_very_long_prefix_bl <int>,
      #   a_very_long_prefix_am <int>, a_very_long_prefix_bm <int>,
      #   a_very_long_prefix_an <int>, a_very_long_prefix_bn <int>,
      #   a_very_long_prefix_ao <int>, a_very_long_prefix_bo <int>,
      #   a_very_long_prefix_ap <int>, a_very_long_prefix_bp <int>,
      #   a_very_long_prefix_aq <int>, a_very_long_prefix_bq <int>,
      #   a_very_long_prefix_ar <int>, a_very_long_prefix_br <int>,
      #   a_very_long_prefix_as <int>, a_very_long_prefix_bs <int>,
      #   a_very_long_prefix_at <int>, a_very_long_prefix_bt <int>,
      #   a_very_long_prefix_au <int>, a_very_long_prefix_bu <int>,
      #   a_very_long_prefix_av <int>, a_very_long_prefix_bv <int>,
      #   a_very_long_prefix_aw <int>, a_very_long_prefix_bw <int>,
      #   a_very_long_prefix_ax <int>, a_very_long_prefix_bx <int>,
      #   a_very_long_prefix_ay <int>, a_very_long_prefix_by <int>,
      #   a_very_long_prefix_az <int>, a_very_long_prefix_bz <int>, and abbreviated
      #   variable names 1: a_very_long_prefix_aa, 2: a_very_long_prefix_ba,
      #   3: a_very_long_prefix_ab, 4: a_very_long_prefix_bb,
      #   5: a_very_long_prefix_ac, 6: a_very_long_prefix_bc,
      #   7: a_very_long_prefix_ad, 8: a_very_long_prefix_bd,
      #   9: a_very_long_prefix_ae
      # i Use `colnames()` to see all variable names

# ornament

    Code
      new_ornament(c("abc", "de"))
    Output
      <pillar_ornament>
      abc
      de 
    Code
      new_ornament(c("abc", "de"), align = "right")
    Output
      <pillar_ornament>
      abc
       de
    Code
      new_ornament(c("abc", "de"), width = 5, align = "right")
    Output
      <pillar_ornament>
        abc
         de
    Code
      format(new_ornament(c("abc", "de")))
    Output
      [1] "abc" "de "
    Code
      format(new_ornament(c("abc", "de")), width = 5)
    Output
      [1] "abc  " "de   "

# vertical

    Code
      as_glue(c("abc", "de"))
    Output
      abc
      de

# exponents correct in presence of NA

    Code
      format(split_decimal_bw(c(NA, 1e-05)), justify = "right")
    Output
      [1] "NA   " " 1e-5"

# output test

    Code
      pillar(add_special(10^c(-9, -6, 3, 9)), width = 10)
    Output
      <pillar>
        <dbl>
         1e-9
         1e-6
         1e+3
         1e+9
        NA   
      -Inf   
       Inf   
    Code
      pillar(add_special((10^c(3, 9, 15, 22)) * c(-1, 1)), width = 10)
    Output
      <pillar>
        <dbl>
        -1e 3
         1e 9
        -1e15
         1e22
        NA   
      -Inf   
       Inf   
    Code
      pillar(add_special(1.25 * 10^(-309:-319)), width = 10)
    Output
      NULL

# output test

    Code
      options(width = 100)
      tbl_format_setup(x, width = 4)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A
      #   data
      #   frame:
      #   3
      #   x
      #   3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # i 3 more variables: column_zero_zero <dbl>, `col 01` <tbl[,3]>, `col 05` <ord>
    Code
      tbl_format_setup(x, width = 5)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A
      #   data
      #   frame:
      #   3
      #   x
      #   3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # i 3
      #   more
      #   variables:
      #   column_zero_zero <dbl>,
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 6)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A
      #   data
      #   frame:
      #   3
      #   x
      #   3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # i 3
      #   more
      #   variables:
      #   column_zero_zero <dbl>,
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 7)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A
      #   data
      #   frame:
      #   3
      #   x
      #   3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # i 3
      #   more
      #   variables:
      #   column_zero_zero <dbl>,
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 8)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A
      #   data
      #   frame:
      #   3 x
      #   3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # i 3
      #   more
      #   variables:
      #   column_zero_zero <dbl>,
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 9)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data
      #   frame:
      #   3 x
      #   3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # i 3
      #   more
      #   variables:
      #   column_zero_zero <dbl>,
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 10)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data
      #   frame:
      #   3 x 3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # i 3
      #   more
      #   variables:
      #   column_zero_zero <dbl>,
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 11)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data
      #   frame:
      #   3 x 3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # i 3 more
      #   variables:
      #   column_zero_zero <dbl>,
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 12)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data
      #   frame:
      #   3 x 3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # i 3 more
      #   variables:
      #   column_zero_zero <dbl>,
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 13)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data
      #   frame: 3
      #   x 3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # i 3 more
      #   variables:
      #   column_zero_zero <dbl>,
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 14)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data
      #   frame: 3
      #   x 3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # i 3 more
      #   variables:
      #   column_zero_zero <dbl>,
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 15)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data
      #   frame: 3 x
      #   3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # i 3 more
      #   variables:
      #   column_zero_zero <dbl>,
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 16)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame:
      #   3 x 3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # i 3 more
      #   variables:
      #   column_zero_zero <dbl>,
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 17)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame:
      #   3 x 3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # i 3 more
      #   variables:
      #   column_zero_zero <dbl>,
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 18)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3
      #   x 3
      <tbl_format_body(setup)>
        column_zero_zero
                   <dbl>
      1             1.23
      2             2.23
      3             3.23
      <tbl_format_footer(setup)>
      # i 2 more
      #   variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 19)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3
      #   x 3
      <tbl_format_body(setup)>
        column_zero_zero
                   <dbl>
      1             1.23
      2             2.23
      3             3.23
      <tbl_format_footer(setup)>
      # i 2 more
      #   variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 20)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x
      #   3
      <tbl_format_body(setup)>
        column_zero_zero
                   <dbl>
      1             1.23
      2             2.23
      3             3.23
      <tbl_format_footer(setup)>
      # i 2 more
      #   variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 21)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x
      #   3
      <tbl_format_body(setup)>
        column_zero_zero
                   <dbl>
      1             1.23
      2             2.23
      3             3.23
      <tbl_format_footer(setup)>
      # i 2 more
      #   variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 22)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero
                   <dbl>
      1             1.23
      2             2.23
      3             3.23
      <tbl_format_footer(setup)>
      # i 2 more variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 23)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero
                   <dbl>
      1             1.23
      2             2.23
      3             3.23
      <tbl_format_footer(setup)>
      # i 2 more variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 24)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero
                   <dbl>
      1             1.23
      2             2.23
      3             3.23
      <tbl_format_footer(setup)>
      # i 2 more variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 25)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero
                   <dbl>
      1             1.23
      2             2.23
      3             3.23
      <tbl_format_footer(setup)>
      # i 2 more variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 26)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero
                   <dbl>
      1             1.23
      2             2.23
      3             3.23
      <tbl_format_footer(setup)>
      # i 2 more variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 27)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero
                   <dbl>
      1             1.23
      2             2.23
      3             3.23
      <tbl_format_footer(setup)>
      # i 2 more variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 28)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero
                   <dbl>
      1             1.23
      2             2.23
      3             3.23
      <tbl_format_footer(setup)>
      # i 2 more variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 29)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero
                   <dbl>
      1             1.23
      2             2.23
      3             3.23
      <tbl_format_footer(setup)>
      # i 2 more variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 30)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero
                   <dbl>
      1             1.23
      2             2.23
      3             3.23
      <tbl_format_footer(setup)>
      # i 2 more variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 31)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero
                   <dbl>
      1             1.23
      2             2.23
      3             3.23
      <tbl_format_footer(setup)>
      # i 2 more variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 32)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero
                   <dbl>
      1             1.23
      2             2.23
      3             3.23
      <tbl_format_footer(setup)>
      # i 2 more variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 33)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero
                   <dbl>
      1             1.23
      2             2.23
      3             3.23
      <tbl_format_footer(setup)>
      # i 2 more variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 34)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero
                   <dbl>
      1             1.23
      2             2.23
      3             3.23
      <tbl_format_footer(setup)>
      # i 2 more variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 35)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero
                   <dbl>
      1             1.23
      2             2.23
      3             3.23
      <tbl_format_footer(setup)>
      # i 2 more variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 36)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02`
                   <dbl> <chr>            
      1             1.23 a                
      2             2.23 b                
      3             3.23 c                
      <tbl_format_footer(setup)>
      # i 3 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 37)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02`
                   <dbl> <chr>            
      1             1.23 a                
      2             2.23 b                
      3             3.23 c                
      <tbl_format_footer(setup)>
      # i 3 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 38)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02`
                   <dbl> <chr>            
      1             1.23 a                
      2             2.23 b                
      3             3.23 c                
      <tbl_format_footer(setup)>
      # i 3 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 39)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02`
                   <dbl> <chr>            
      1             1.23 a                
      2             2.23 b                
      3             3.23 c                
      <tbl_format_footer(setup)>
      # i 3 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>,
      #   `col 05` <ord>
    Code
      tbl_format_setup(x, width = 40)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02`
                   <dbl> <chr>            
      1             1.23 a                
      2             2.23 b                
      3             3.23 c                
      <tbl_format_footer(setup)>
      # i 3 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>, `col 05` <ord>
    Code
      tbl_format_setup(x, width = 41)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02`
                   <dbl> <chr>            
      1             1.23 a                
      2             2.23 b                
      3             3.23 c                
      <tbl_format_footer(setup)>
      # i 3 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>, `col 05` <ord>
    Code
      tbl_format_setup(x, width = 42)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02`
                   <dbl> <chr>            
      1             1.23 a                
      2             2.23 b                
      3             3.23 c                
      <tbl_format_footer(setup)>
      # i 3 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>, `col 05` <ord>
    Code
      tbl_format_setup(x, width = 43)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02`
                   <dbl> <chr>            
      1             1.23 a                
      2             2.23 b                
      3             3.23 c                
      <tbl_format_footer(setup)>
      # i 3 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>, `col 05` <ord>
    Code
      tbl_format_setup(x, width = 44)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02`
                   <dbl> <chr>            
      1             1.23 a                
      2             2.23 b                
      3             3.23 c                
      <tbl_format_footer(setup)>
      # i 3 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>, `col 05` <ord>
    Code
      tbl_format_setup(x, width = 45)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` `col 05`
                   <dbl> <chr>             <ord>   
      1             1.23 a                 a       
      2             2.23 b                 b       
      3             3.23 c                 c       
      <tbl_format_footer(setup)>
      # i 2 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 46)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` `col 05`
                   <dbl> <chr>             <ord>   
      1             1.23 a                 a       
      2             2.23 b                 b       
      3             3.23 c                 c       
      <tbl_format_footer(setup)>
      # i 2 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 47)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` `col 05`
                   <dbl> <chr>             <ord>   
      1             1.23 a                 a       
      2             2.23 b                 b       
      3             3.23 c                 c       
      <tbl_format_footer(setup)>
      # i 2 more variables: `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 48)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` `col 05`
                   <dbl> <chr>             <ord>   
      1             1.23 a                 a       
      2             2.23 b                 b       
      3             3.23 c                 c       
      <tbl_format_footer(setup)>
      # i 2 more variables: `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 49)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` `col 05`
                   <dbl> <chr>             <ord>   
      1             1.23 a                 a       
      2             2.23 b                 b       
      3             3.23 c                 c       
      <tbl_format_footer(setup)>
      # i 2 more variables: `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 50)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` `col 05`
                   <dbl> <chr>             <ord>   
      1             1.23 a                 a       
      2             2.23 b                 b       
      3             3.23 c                 c       
      <tbl_format_footer(setup)>
      # i 2 more variables: `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 51)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` `col 05`
                   <dbl> <chr>             <ord>   
      1             1.23 a                 a       
      2             2.23 b                 b       
      3             3.23 c                 c       
      <tbl_format_footer(setup)>
      # i 2 more variables: `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 52)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` `col 05`
                   <dbl> <chr>             <ord>   
      1             1.23 a                 a       
      2             2.23 b                 b       
      3             3.23 c                 c       
      <tbl_format_footer(setup)>
      # i 2 more variables: `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 53)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` `col 05`
                   <dbl> <chr>             <ord>   
      1             1.23 a                 a       
      2             2.23 b                 b       
      3             3.23 c                 c       
      <tbl_format_footer(setup)>
      # i 2 more variables: `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 54)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` `col 05`
                   <dbl> <chr>             <ord>   
      1             1.23 a                 a       
      2             2.23 b                 b       
      3             3.23 c                 c       
      <tbl_format_footer(setup)>
      # i 2 more variables: `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 55)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` `col 05`
                   <dbl> <chr>             <chr>     <ord>   
      1             1.23 a                 A         a       
      2             2.23 b                 B         b       
      3             3.23 c                 C         c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 56)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` `col 05`
                   <dbl> <chr>             <chr>     <ord>   
      1             1.23 a                 A         a       
      2             2.23 b                 B         b       
      3             3.23 c                 C         c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 57)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` `col 05`
                   <dbl> <chr>             <chr>     <ord>   
      1             1.23 a                 A         a       
      2             2.23 b                 B         b       
      3             3.23 c                 C         c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 58)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` `col 05`
                   <dbl> <chr>             <chr>     <ord>   
      1             1.23 a                 A         a       
      2             2.23 b                 B         b       
      3             3.23 c                 C         c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 59)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` `col 05`
                   <dbl> <chr>             <chr>     <ord>   
      1             1.23 a                 A         a       
      2             2.23 b                 B         b       
      3             3.23 c                 C         c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 60)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` `col 05`
                   <dbl> <chr>             <chr>     <ord>   
      1             1.23 a                 A         a       
      2             2.23 b                 B         b       
      3             3.23 c                 C         c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 61)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` `col 05`
                   <dbl> <chr>             <chr>     <ord>   
      1             1.23 a                 A         a       
      2             2.23 b                 B         b       
      3             3.23 c                 C         c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 62)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` `col 05`
                   <dbl> <chr>             <chr>     <ord>   
      1             1.23 a                 A         a       
      2             2.23 b                 B         b       
      3             3.23 c                 C         c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 63)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` `col 05`
                   <dbl> <chr>             <chr>     <ord>   
      1             1.23 a                 A         a       
      2             2.23 b                 B         b       
      3             3.23 c                 C         c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 64)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` `col 05`
                   <dbl> <chr>             <chr>     <ord>   
      1             1.23 a                 A         a       
      2             2.23 b                 B         b       
      3             3.23 c                 C         c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 65)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` `col 05`
                   <dbl> <chr>             <chr>     <ord>   
      1             1.23 a                 A         a       
      2             2.23 b                 B         b       
      3             3.23 c                 C         c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 66)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` `col 05`
                   <dbl> <chr>             <chr>     <ord>   
      1             1.23 a                 A         a       
      2             2.23 b                 B         b       
      3             3.23 c                 C         c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 67)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` `col 05`
                   <dbl> <chr>             <chr>     <ord>   
      1             1.23 a                 A         a       
      2             2.23 b                 B         b       
      3             3.23 c                 C         c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 68)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` `col 05`
                   <dbl> <chr>             <chr>     <ord>   
      1             1.23 a                 A         a       
      2             2.23 b                 B         b       
      3             3.23 c                 C         c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 69)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` `col 05`
                   <dbl> <chr>             <chr>     <ord>   
      1             1.23 a                 A         a       
      2             2.23 b                 B         b       
      3             3.23 c                 C         c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 70)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` `col 05`
                   <dbl> <chr>             <chr>     <ord>   
      1             1.23 a                 A         a       
      2             2.23 b                 B         b       
      3             3.23 c                 C         c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04` <int[,3]>
    Code
      tbl_format_setup(x, width = 71)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] `col 05`
                   <dbl> <chr>             <chr>               <int> <ord>   
      1             1.23 a                 A                       1 a       
      2             2.23 b                 B                       2 b       
      3             3.23 c                 C                       3 c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04`[2:3] <int>
    Code
      tbl_format_setup(x, width = 72)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] `col 05`
                   <dbl> <chr>             <chr>               <int> <ord>   
      1             1.23 a                 A                       1 a       
      2             2.23 b                 B                       2 b       
      3             3.23 c                 C                       3 c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04`[2:3] <int>
    Code
      tbl_format_setup(x, width = 73)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] `col 05`
                   <dbl> <chr>             <chr>               <int> <ord>   
      1             1.23 a                 A                       1 a       
      2             2.23 b                 B                       2 b       
      3             3.23 c                 C                       3 c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04`[2:3] <int>
    Code
      tbl_format_setup(x, width = 74)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] `col 05`
                   <dbl> <chr>             <chr>               <int> <ord>   
      1             1.23 a                 A                       1 a       
      2             2.23 b                 B                       2 b       
      3             3.23 c                 C                       3 c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04`[2:3] <int>
    Code
      tbl_format_setup(x, width = 75)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] `col 05`
                   <dbl> <chr>             <chr>               <int> <ord>   
      1             1.23 a                 A                       1 a       
      2             2.23 b                 B                       2 b       
      3             3.23 c                 C                       3 c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04`[2:3] <int>
    Code
      tbl_format_setup(x, width = 76)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] `col 05`
                   <dbl> <chr>             <chr>               <int> <ord>   
      1             1.23 a                 A                       1 a       
      2             2.23 b                 B                       2 b       
      3             3.23 c                 C                       3 c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04`[2:3] <int>
    Code
      tbl_format_setup(x, width = 77)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] `col 05`
                   <dbl> <chr>             <chr>               <int> <ord>   
      1             1.23 a                 A                       1 a       
      2             2.23 b                 B                       2 b       
      3             3.23 c                 C                       3 c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04`[2:3] <int>
    Code
      tbl_format_setup(x, width = 78)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] `col 05`
                   <dbl> <chr>             <chr>               <int> <ord>   
      1             1.23 a                 A                       1 a       
      2             2.23 b                 B                       2 b       
      3             3.23 c                 C                       3 c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04`[2:3] <int>
    Code
      tbl_format_setup(x, width = 79)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] $[,"B"] `col 05`
                   <dbl> <chr>             <chr>               <int>   <int> <ord>   
      1             1.23 a                 A                       1       4 a       
      2             2.23 b                 B                       2       5 b       
      3             3.23 c                 C                       3       6 c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04`[3] <int>
    Code
      tbl_format_setup(x, width = 80)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] $[,"B"] `col 05`
                   <dbl> <chr>             <chr>               <int>   <int> <ord>   
      1             1.23 a                 A                       1       4 a       
      2             2.23 b                 B                       2       5 b       
      3             3.23 c                 C                       3       6 c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04`[3] <int>
    Code
      tbl_format_setup(x, width = 81)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] $[,"B"] `col 05`
                   <dbl> <chr>             <chr>               <int>   <int> <ord>   
      1             1.23 a                 A                       1       4 a       
      2             2.23 b                 B                       2       5 b       
      3             3.23 c                 C                       3       6 c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04`[3] <int>
    Code
      tbl_format_setup(x, width = 82)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] $[,"B"] `col 05`
                   <dbl> <chr>             <chr>               <int>   <int> <ord>   
      1             1.23 a                 A                       1       4 a       
      2             2.23 b                 B                       2       5 b       
      3             3.23 c                 C                       3       6 c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04`[3] <int>
    Code
      tbl_format_setup(x, width = 83)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] $[,"B"] `col 05`
                   <dbl> <chr>             <chr>               <int>   <int> <ord>   
      1             1.23 a                 A                       1       4 a       
      2             2.23 b                 B                       2       5 b       
      3             3.23 c                 C                       3       6 c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04`[3] <int>
    Code
      tbl_format_setup(x, width = 84)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] $[,"B"] `col 05`
                   <dbl> <chr>             <chr>               <int>   <int> <ord>   
      1             1.23 a                 A                       1       4 a       
      2             2.23 b                 B                       2       5 b       
      3             3.23 c                 C                       3       6 c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04`[3] <int>
    Code
      tbl_format_setup(x, width = 85)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] $[,"B"] `col 05`
                   <dbl> <chr>             <chr>               <int>   <int> <ord>   
      1             1.23 a                 A                       1       4 a       
      2             2.23 b                 B                       2       5 b       
      3             3.23 c                 C                       3       6 c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04`[3] <int>
    Code
      tbl_format_setup(x, width = 86)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] $[,"B"] `col 05`
                   <dbl> <chr>             <chr>               <int>   <int> <ord>   
      1             1.23 a                 A                       1       4 a       
      2             2.23 b                 B                       2       5 b       
      3             3.23 c                 C                       3       6 c       
      <tbl_format_footer(setup)>
      # i 1 more variable: `col 01`$`col 04`[3] <int>
    Code
      tbl_format_setup(x, width = 87)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] $[,"B"] $[,"C"] `col 05`
                   <dbl> <chr>             <chr>               <int>   <int>   <int> <ord>   
      1             1.23 a                 A                       1       4       7 a       
      2             2.23 b                 B                       2       5       8 b       
      3             3.23 c                 C                       3       6       9 c       
      <tbl_format_footer(setup)>
    Code
      tbl_format_setup(x, width = 88)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] $[,"B"] $[,"C"] `col 05`
                   <dbl> <chr>             <chr>               <int>   <int>   <int> <ord>   
      1             1.23 a                 A                       1       4       7 a       
      2             2.23 b                 B                       2       5       8 b       
      3             3.23 c                 C                       3       6       9 c       
      <tbl_format_footer(setup)>
    Code
      tbl_format_setup(x, width = 89)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] $[,"B"] $[,"C"] `col 05`
                   <dbl> <chr>             <chr>               <int>   <int>   <int> <ord>   
      1             1.23 a                 A                       1       4       7 a       
      2             2.23 b                 B                       2       5       8 b       
      3             3.23 c                 C                       3       6       9 c       
      <tbl_format_footer(setup)>
    Code
      tbl_format_setup(x, width = Inf)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] $[,"B"] $[,"C"] `col 05`
                   <dbl> <chr>             <chr>               <int>   <int>   <int> <ord>   
      1             1.23 a                 A                       1       4       7 a       
      2             2.23 b                 B                       2       5       8 b       
      3             3.23 c                 C                       3       6       9 c       
      <tbl_format_footer(setup)>

# tbl_format_setup() results

    Code
      tbl_format_setup(as_tbl(mtcars), width = 80)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 32 x 11
      <tbl_format_body(setup)>
           mpg   cyl  disp    hp  drat    wt  qsec    vs    am  gear  carb
       * <dbl> <dbl> <dbl> <dbl> <dbl> <dbl> <dbl> <dbl> <dbl> <dbl> <dbl>
       1  21       6  160    110  3.9   2.62  16.5     0     1     4     4
       2  21       6  160    110  3.9   2.88  17.0     0     1     4     4
       3  22.8     4  108     93  3.85  2.32  18.6     1     1     4     1
       4  21.4     6  258    110  3.08  3.22  19.4     1     0     3     1
       5  18.7     8  360    175  3.15  3.44  17.0     0     0     3     2
       6  18.1     6  225    105  2.76  3.46  20.2     1     0     3     1
       7  14.3     8  360    245  3.21  3.57  15.8     0     0     3     4
       8  24.4     4  147.    62  3.69  3.19  20       1     0     4     2
       9  22.8     4  141.    95  3.92  3.15  22.9     1     0     4     2
      10  19.2     6  168.   123  3.92  3.44  18.3     1     0     4     4
      <tbl_format_footer(setup)>
      # i 22 more rows
    Code
      tbl_format_setup(as_tbl(mtcars), width = 30, n = 8)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 32 x 11
      <tbl_format_body(setup)>
          mpg   cyl  disp    hp
      * <dbl> <dbl> <dbl> <dbl>
      1  21       6  160    110
      2  21       6  160    110
      3  22.8     4  108     93
      4  21.4     6  258    110
      5  18.7     8  360    175
      6  18.1     6  225    105
      7  14.3     8  360    245
      8  24.4     4  147.    62
      <tbl_format_footer(setup)>
      # i 24 more rows
      # i 7 more variables:
      #   drat <dbl>, wt <dbl>,
      #   qsec <dbl>, vs <dbl>,
      #   am <dbl>, gear <dbl>,
      #   carb <dbl>
    Code
      tbl_format_setup(as_tbl(trees), width = 30, n = 100)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 31 x 3
      <tbl_format_body(setup)>
         Girth Height Volume
         <dbl>  <dbl>  <dbl>
       1   8.3     70   10.3
       2   8.6     65   10.3
       3   8.8     63   10.2
       4  10.5     72   16.4
       5  10.7     81   18.8
       6  10.8     83   19.7
       7  11       66   15.6
       8  11       75   18.2
       9  11.1     80   22.6
      10  11.2     75   19.9
      11  11.3     79   24.2
      12  11.4     76   21  
      13  11.4     76   21.4
      14  11.7     69   21.3
      15  12       75   19.1
      16  12.9     74   22.2
      17  12.9     85   33.8
      18  13.3     86   27.4
      19  13.7     71   25.7
      20  13.8     64   24.9
      21  14       78   34.5
      22  14.2     80   31.7
      23  14.5     74   36.3
      24  16       72   38.3
      25  16.3     77   42.6
      26  17.3     81   55.4
      27  17.5     82   55.7
      28  17.9     80   58.3
      29  18       80   51.5
      30  18       80   51  
      31  20.6     87   77  
      <tbl_format_footer(setup)>
    Code
      # Unknown rows
      tbl_format_setup(as_unknown_rows(trees[1:9, ]), width = 30, n = 10)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: ?? x 3
      <tbl_format_body(setup)>
        Girth Height Volume
        <dbl>  <dbl>  <dbl>
      1   8.3     70   10.3
      2   8.6     65   10.3
      3   8.8     63   10.2
      4  10.5     72   16.4
      5  10.7     81   18.8
      6  10.8     83   19.7
      7  11       66   15.6
      8  11       75   18.2
      9  11.1     80   22.6
      <tbl_format_footer(setup)>
    Code
      tbl_format_setup(as_unknown_rows(trees[1:10, ]), width = 30, n = 10)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: ?? x 3
      <tbl_format_body(setup)>
         Girth Height Volume
         <dbl>  <dbl>  <dbl>
       1   8.3     70   10.3
       2   8.6     65   10.3
       3   8.8     63   10.2
       4  10.5     72   16.4
       5  10.7     81   18.8
       6  10.8     83   19.7
       7  11       66   15.6
       8  11       75   18.2
       9  11.1     80   22.6
      10  11.2     75   19.9
      <tbl_format_footer(setup)>
    Code
      tbl_format_setup(as_unknown_rows(trees[1:11, ]), width = 30, n = 10)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: ?? x 3
      <tbl_format_body(setup)>
         Girth Height Volume
         <dbl>  <dbl>  <dbl>
       1   8.3     70   10.3
       2   8.6     65   10.3
       3   8.8     63   10.2
       4  10.5     72   16.4
       5  10.7     81   18.8
       6  10.8     83   19.7
       7  11       66   15.6
       8  11       75   18.2
       9  11.1     80   22.6
      10  11.2     75   19.9
      <tbl_format_footer(setup)>
      # i more rows
    Code
      # No rows
      tbl_format_setup(new_tbl(list(a = character(), b = logical())), width = 30)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 0 x 2
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # i 2 variables: a <chr>,
      #   b <lgl>
    Code
      # No columns
      tbl_format_setup(as_tbl(trees[character()]), width = 30, n = 5L)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 31 x 0
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
    Code
      # No rows, unknown
      tbl_format_setup(as_unknown_rows(trees[integer(), ]), width = 30, n = 5L)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: ?? x 3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # i 3 variables: Girth <dbl>,
      #   Height <dbl>,
      #   Volume <dbl>
    Code
      # No columns, unknown
      tbl_format_setup(as_unknown_rows(trees[, character()]), width = 30, n = 5L)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: ?? x 0
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # i at least 5 rows total
    Code
      # Long table, unknown
      tbl_format_setup(as_unknown_rows(new_tbl(list(a = seq.int(10000)))), n = 5L,
      width = 30L)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: ?? x 1
      <tbl_format_body(setup)>
            a
        <int>
      1     1
      2     2
      3     3
      4     4
      5     5
      <tbl_format_footer(setup)>
      # i more rows

# tbl_format_setup() for footnotes

    Code
      tbl_format_setup(width = 73, as_tbl(data_frame(xxxabc = 1, xxxdef = 1, xxxghi = 1,
        xxxjkl = 1, xxxmno = 1, xxxpqr = 1, xxxstu = 1, xxxvwx = 1, xxxyza = 1,
        xxxbcd = 1, xxxefg = 1, xxxhij = 1, xxxklm = 1, xxxnop = 1)))
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 1 x 14
      <tbl_format_body(setup)>
        xxxabc xxxdef xxxghi xxxjkl xxxmno xxxpqr xxxstu xxxvwx xxxyza xxxbcd
         <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>
      1      1      1      1      1      1      1      1      1      1      1
      <tbl_format_footer(setup)>
      # i 4 more variables: xxxefg <dbl>, xxxhij <dbl>, xxxklm <dbl>,
      #   xxxnop <dbl>

# tbl_format_setup() for footnotes with min_title_chars = 4

    Code
      tbl_format_setup(width = 73, as_tbl(data_frame(xxxabc = 1, xxxdef = 1, xxxghi = 1,
        xxxjkl = 1, xxxmno = 1, xxxpqr = 1, xxxstu = 1, xxxvwx = 1, xxxyza = 1,
        xxxbcd = 1, xxxefg = 1, xxxhij = 1, xxxklm = 1, xxxnop = 1)))
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 1 x 14
      <tbl_format_body(setup)>
        xxxabc xxxdef xxxghi xxxjkl xxxmno xxxpqr xxxstu xxxvwx xxxyza xxxbcd
         <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>
      1      1      1      1      1      1      1      1      1      1      1
      <tbl_format_footer(setup)>
      # i 4 more variables: xxxefg <dbl>, xxxhij <dbl>, xxxklm <dbl>,
      #   xxxnop <dbl>

# tbl_format_setup() for footnotes with UTF-8 output

    Code
      tbl_format_setup(width = 73, as_tbl(data_frame(xxxabc = 1, xxxdef = 1, xxxghi = 1,
        xxxjkl = 1, xxxmno = 1, xxxpqr = 1, xxxstu = 1, xxxvwx = 1, xxxyza = 1,
        xxxbcd = 1, xxxefg = 1, xxxhij = 1, xxxklm = 1, xxxnop = 1)))
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 1 × 14
      <tbl_format_body(setup)>
        xxxabc xxxdef xxxghi xxxjkl xxxmno xxxpqr xxxstu xxxvwx xxxyza xxxbcd
         <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>
      1      1      1      1      1      1      1      1      1      1      1
      <tbl_format_footer(setup)>
      # ℹ 4 more variables: xxxefg <dbl>, xxxhij <dbl>, xxxklm <dbl>,
      #   xxxnop <dbl>

# color

    Code
      style_na("NA")
    Output
      [1] "\033[31mNA\033[39m"
    Code
      style_neg("-1")
    Output
      [1] "\033[31m-1\033[39m"

---

    Code
      xf <- (function() ctl_colonnade(list(x = c((10^(-3:4)) * c(-1, 1), NA))))
      print(xf())
    Output
      $body
                x
            [3m[90m<dbl>[39m[23m
      [90m1[39m    -[31m0[39m[31m.[39m[31m00[39m[31m1[39m
      [90m2[39m     0.01 
      [90m3[39m    -[31m0[39m[31m.[39m[31m1[39m  
      [90m4[39m     1    
      [90m5[39m   -[31m10[39m    
      [90m6[39m   100    
      [90m7[39m -[31m[4m1[24m00[39m[31m0[39m    
      [90m8[39m [4m1[24m[4m0[24m000    
      [90m9[39m    [31mNA[39m    
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      with_options(pillar.subtle_num = TRUE, print(xf()))
    Output
      $body
                x
            [3m[90m<dbl>[39m[23m
      [90m1[39m    -[90m0[39m[90m.[39m[90m00[39m[31m1[39m
      [90m2[39m     [90m0[39m[90m.[39m[90m0[39m1 
      [90m3[39m    -[90m0[39m[90m.[39m[31m1[39m  
      [90m4[39m     1    
      [90m5[39m   -[31m10[39m    
      [90m6[39m   100    
      [90m7[39m -[31m[4m1[24m00[39m[90m0[39m    
      [90m8[39m [4m1[24m[4m0[24m0[90m00[39m    
      [90m9[39m    [31mNA[39m    
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      with_options(pillar.subtle = FALSE, print(xf()))
    Output
      $body
                x
            [3m<dbl>[23m
      1    -[31m0[39m[31m.[39m[31m00[39m[31m1[39m
      2     0.01 
      3    -[31m0[39m[31m.[39m[31m1[39m  
      4     1    
      5   -[31m10[39m    
      6   100    
      7 -[31m[4m1[24m00[39m[31m0[39m    
      8 [4m1[24m[4m0[24m000    
      9    [31mNA[39m    
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      with_options(pillar.neg = FALSE, print(xf()))
    Output
      $body
                x
            [3m[90m<dbl>[39m[23m
      [90m1[39m    -0.001
      [90m2[39m     0.01 
      [90m3[39m    -0.1  
      [90m4[39m     1    
      [90m5[39m   -10    
      [90m6[39m   100    
      [90m7[39m -[4m1[24m000    
      [90m8[39m [4m1[24m[4m0[24m000    
      [90m9[39m    [31mNA[39m    
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      with_options(pillar.subtle = FALSE, pillar.neg = FALSE, print(xf()))
    Output
      $body
                x
            [3m<dbl>[23m
      1    -0.001
      2     0.01 
      3    -0.1  
      4     1    
      5   -10    
      6   100    
      7 -[4m1[24m000    
      8 [4m1[24m[4m0[24m000    
      9    [31mNA[39m    
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      with_options(pillar.bold = TRUE, print(xf()))
    Output
      $body
                [1mx[22m
            [3m[90m<dbl>[39m[23m
      [90m1[39m    -[31m0[39m[31m.[39m[31m00[39m[31m1[39m
      [90m2[39m     0.01 
      [90m3[39m    -[31m0[39m[31m.[39m[31m1[39m  
      [90m4[39m     1    
      [90m5[39m   -[31m10[39m    
      [90m6[39m   100    
      [90m7[39m -[31m[4m1[24m00[39m[31m0[39m    
      [90m8[39m [4m1[24m[4m0[24m000    
      [90m9[39m    [31mNA[39m    
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      

---

    Code
      ctl_colonnade(list(a_very_long_column_name = 0), width = 20)
    Output
      $body
      
      $extra_cols
      $extra_cols$a_very_long_column_name
      [1] 0
      
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      

# focus columns

    Code
      tbl_format_setup(x, width = 30, focus = "b")
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data frame: 1 × 2[39m
      [1m<tbl_format_body(setup)>[22m
          a$x    $y b          
        [3m[90m<dbl>[39m[23m [3m[90m<dbl>[39m[23m [4m[3m[90m<chr>[39m[23m[24m      
      [90m1[39m     1     2 long enough
      [1m<tbl_format_footer(setup)>[22m
    Code
      tbl_format_setup(x, width = 20, focus = "b")
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data frame: 1 ×
      #   2[39m
      [1m<tbl_format_body(setup)>[22m
          a$x b          
        [3m[90m<dbl>[39m[23m [4m[3m[90m<chr>[39m[23m[24m      
      [90m1[39m     1 long enough
      [1m<tbl_format_footer(setup)>[22m
      [90m# ℹ 1 more[39m
      [90m#   variable:[39m
      [90m#   a$y <dbl>[39m
    Code
      tbl_format_setup(x, width = 15, focus = "b")
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data
      #   frame: 1 ×
      #   2[39m
      [1m<tbl_format_body(setup)>[22m
        b          
        [4m[3m[90m<chr>[39m[23m[24m      
      [90m1[39m long enough
      [1m<tbl_format_footer(setup)>[22m
      [90m# ℹ 1 more[39m
      [90m#   variable:[39m
      [90m#   a <tbl[,2]>[39m
    Code
      tbl_format_setup(x, width = 10, focus = "b")
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data
      #   frame:
      #   1 × 2[39m
      [1m<tbl_format_body(setup)>[22m
        b       
        [4m[3m[90m<chr>[39m[23m[24m   
      [90m1[39m long en…
      [1m<tbl_format_footer(setup)>[22m
      [90m# ℹ 1[39m
      [90m#   more[39m
      [90m#   variable:[39m
      [90m#   a <tbl[,2]>[39m
    Code
      tbl_format_setup(x[2:1], width = 30, focus = "a")
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data frame: 1 × 2[39m
      [1m<tbl_format_body(setup)>[22m
        b             a$x    $y
        [3m[90m<chr>[39m[23m       [4m[3m[90m<dbl>[39m[23m[24m [4m[3m[90m<dbl>[39m[23m[24m
      [90m1[39m long enough     1     2
      [1m<tbl_format_footer(setup)>[22m
    Code
      tbl_format_setup(x[2:1], width = 20, focus = "a")
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data frame: 1 ×
      #   2[39m
      [1m<tbl_format_body(setup)>[22m
        b        a$x    $y
        [3m[90m<chr>[39m[23m  [4m[3m[90m<dbl>[39m[23m[24m [4m[3m[90m<dbl>[39m[23m[24m
      [90m1[39m long …     1     2
      [1m<tbl_format_footer(setup)>[22m
    Code
      tbl_format_setup(x[2:1], width = 15, focus = "a")
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data
      #   frame: 1 ×
      #   2[39m
      [1m<tbl_format_body(setup)>[22m
          a$x    $y
        [4m[3m[90m<dbl>[39m[23m[24m [4m[3m[90m<dbl>[39m[23m[24m
      [90m1[39m     1     2
      [1m<tbl_format_footer(setup)>[22m
      [90m# ℹ 1 more[39m
      [90m#   variable:[39m
      [90m#   b <chr>[39m
    Code
      tbl_format_setup(x[2:1], width = 10, focus = "a")
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data
      #   frame:
      #   1 × 2[39m
      [1m<tbl_format_body(setup)>[22m
          a$x
        [4m[3m[90m<dbl>[39m[23m[24m
      [90m1[39m     1
      [1m<tbl_format_footer(setup)>[22m
      [90m# ℹ 2[39m
      [90m#   more[39m
      [90m#   variables:[39m
      [90m#   a$y <dbl>,[39m
      [90m#   b <chr>[39m
    Code
      tbl_format_setup(x, width = 30, focus = c("a", "b"))
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data frame: 1 × 2[39m
      [1m<tbl_format_body(setup)>[22m
          a$x    $y b          
        [4m[3m[90m<dbl>[39m[23m[24m [4m[3m[90m<dbl>[39m[23m[24m [4m[3m[90m<chr>[39m[23m[24m      
      [90m1[39m     1     2 long enough
      [1m<tbl_format_footer(setup)>[22m
    Code
      tbl_format_setup(x, width = 20, focus = c("a", "b"))
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data frame: 1 ×
      #   2[39m
      [1m<tbl_format_body(setup)>[22m
          a$x    $y b     
        [4m[3m[90m<dbl>[39m[23m[24m [4m[3m[90m<dbl>[39m[23m[24m [4m[3m[90m<chr>[39m[23m[24m 
      [90m1[39m     1     2 long …
      [1m<tbl_format_footer(setup)>[22m
    Code
      tbl_format_setup(x, width = 15, focus = c("a", "b"))
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data
      #   frame: 1 ×
      #   2[39m
      [1m<tbl_format_body(setup)>[22m
          a$x b      
        [4m[3m[90m<dbl>[39m[23m[24m [4m[3m[90m<chr>[39m[23m[24m  
      [90m1[39m     1 long e…
      [1m<tbl_format_footer(setup)>[22m
      [90m# ℹ 1 more[39m
      [90m#   variable:[39m
      [90m#   a$y <dbl>[39m
    Code
      tbl_format_setup(x, width = 10, focus = c("a", "b"))
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data
      #   frame:
      #   1 × 2[39m
      [1m<tbl_format_body(setup)>[22m
          a$x
        [4m[3m[90m<dbl>[39m[23m[24m
      [90m1[39m     1
      [1m<tbl_format_footer(setup)>[22m
      [90m# ℹ 2[39m
      [90m#   more[39m
      [90m#   variables:[39m
      [90m#   a$y <dbl>,[39m
      [90m#   b <chr>[39m
    Code
      tbl_format_setup(x[2:1], width = 30, focus = c("a", "b"))
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data frame: 1 × 2[39m
      [1m<tbl_format_body(setup)>[22m
        b             a$x    $y
        [4m[3m[90m<chr>[39m[23m[24m       [4m[3m[90m<dbl>[39m[23m[24m [4m[3m[90m<dbl>[39m[23m[24m
      [90m1[39m long enough     1     2
      [1m<tbl_format_footer(setup)>[22m
    Code
      tbl_format_setup(x[2:1], width = 20, focus = c("a", "b"))
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data frame: 1 ×
      #   2[39m
      [1m<tbl_format_body(setup)>[22m
        b             a$x
        [4m[3m[90m<chr>[39m[23m[24m       [4m[3m[90m<dbl>[39m[23m[24m
      [90m1[39m long enough     1
      [1m<tbl_format_footer(setup)>[22m
      [90m# ℹ 1 more[39m
      [90m#   variable:[39m
      [90m#   a$y <dbl>[39m
    Code
      tbl_format_setup(x[2:1], width = 15, focus = c("a", "b"))
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data
      #   frame: 1 ×
      #   2[39m
      [1m<tbl_format_body(setup)>[22m
        b         a$x
        [4m[3m[90m<chr>[39m[23m[24m   [4m[3m[90m<dbl>[39m[23m[24m
      [90m1[39m long e…     1
      [1m<tbl_format_footer(setup)>[22m
      [90m# ℹ 1 more[39m
      [90m#   variable:[39m
      [90m#   a$y <dbl>[39m
    Code
      tbl_format_setup(x[2:1], width = 10, focus = c("a", "b"))
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data
      #   frame:
      #   1 × 2[39m
      [1m<tbl_format_body(setup)>[22m
        b       
        [4m[3m[90m<chr>[39m[23m[24m   
      [90m1[39m long en…
      [1m<tbl_format_footer(setup)>[22m
      [90m# ℹ 1[39m
      [90m#   more[39m
      [90m#   variable:[39m
      [90m#   a <tbl[,2]>[39m

# rowids for line-style

    Code
      vctrs::new_data_frame(list(a = 1:3, b = letters[1:3]), class = c("line_tbl",
        "tbl"))
    Output
      # A data frame: 3 x 2
            a b    
        <int> <chr>
      =     = =    
      1     1 a    
      2     2 b    
      3     3 c    

# roman rowids

    Code
      vctrs::new_data_frame(list(a = 1:3, b = letters[1:3]), class = c("roman_tbl",
        "tbl"))
    Output
      # A data frame: 3 x 2
              a b    
          <int> <chr>
      I       1 a    
      II      2 b    
      III     3 c    

# S4 character class (tidyverse/tibble#1367)

    Code
      pillar(DBI::SQL("x"))
    Output
      <pillar>
      <SQL>
      x    

# Infinite times (#645)

    Code
      pillar(.POSIXct(Inf))
    Output
      <pillar>
      <dttm>
      Inf   

# output test

    Code
      options(width = 100)
      tbl_format_setup(x, width = 4)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A
      #   data
      #   frame:
      #   3
      #   x
      #   3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      #   with 3 more variables: column_zero_zero <dbl>, `col 01` <tbl[,3]>, `col 05` <ord>
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 5)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A
      #   data
      #   frame:
      #   3
      #   x
      #   3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      #   with
      #   3
      #   more
      #   variables:
      #   column_zero_zero <dbl>,
      #   `col 01` <tbl[,3]>, ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 6)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A
      #   data
      #   frame:
      #   3
      #   x
      #   3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      #   with
      #   3
      #   more
      #   variables:
      #   column_zero_zero <dbl>,
      #   `col 01` <tbl[,3]>, ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 7)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A
      #   data
      #   frame:
      #   3
      #   x
      #   3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      #   with
      #   3
      #   more
      #   variables:
      #   column_zero_zero <dbl>,
      #   `col 01` <tbl[,3]>, ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 8)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A
      #   data
      #   frame:
      #   3 x
      #   3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      #   with
      #   3
      #   more
      #   variables:
      #   column_zero_zero <dbl>,
      #   `col 01` <tbl[,3]>, ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 9)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data
      #   frame:
      #   3 x
      #   3
      <tbl_format_body(setup)>
        colum~1
          <dbl>
      1    1.23
      2    2.23
      3    3.23
      <tbl_format_footer(setup)>
      #   with
      #   2
      #   more
      #   variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>, ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 10)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data
      #   frame:
      #   3 x 3
      <tbl_format_body(setup)>
        column~1
           <dbl>
      1     1.23
      2     2.23
      3     3.23
      <tbl_format_footer(setup)>
      #   with
      #   2
      #   more
      #   variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>, ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 11)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data
      #   frame:
      #   3 x 3
      <tbl_format_body(setup)>
        column_~1
            <dbl>
      1      1.23
      2      2.23
      3      3.23
      <tbl_format_footer(setup)>
      # ... with
      #   2 more
      #   variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>,
      #   and
      #   abbreviated ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 12)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data
      #   frame:
      #   3 x 3
      <tbl_format_body(setup)>
        column_z~1
             <dbl>
      1       1.23
      2       2.23
      3       3.23
      <tbl_format_footer(setup)>
      # ... with
      #   2 more
      #   variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>,
      #   and
      #   abbreviated ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 13)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data
      #   frame: 3
      #   x 3
      <tbl_format_body(setup)>
        column_ze~1
              <dbl>
      1        1.23
      2        2.23
      3        3.23
      <tbl_format_footer(setup)>
      # ... with 2
      #   more
      #   variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>,
      #   and
      #   abbreviated ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 14)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data
      #   frame: 3
      #   x 3
      <tbl_format_body(setup)>
        column_zer~1
               <dbl>
      1         1.23
      2         2.23
      3         3.23
      <tbl_format_footer(setup)>
      # ... with 2
      #   more
      #   variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>,
      #   and
      #   abbreviated ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 15)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data
      #   frame: 3 x
      #   3
      <tbl_format_body(setup)>
        column_zero~1
                <dbl>
      1          1.23
      2          2.23
      3          3.23
      <tbl_format_footer(setup)>
      # ... with 2
      #   more
      #   variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>,
      #   and
      #   abbreviated ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 16)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame:
      #   3 x 3
      <tbl_format_body(setup)>
        column_zero_~1
                 <dbl>
      1           1.23
      2           2.23
      3           3.23
      <tbl_format_footer(setup)>
      # ... with 2
      #   more
      #   variables:
      #   `col 01` <tbl[,3]>,
      #   `col 05` <ord>,
      #   and
      #   abbreviated ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 17)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame:
      #   3 x 3
      <tbl_format_body(setup)>
        colum~1 col 0~2
          <dbl> <chr>  
      1    1.23 a      
      2    2.23 b      
      3    3.23 c      
      <tbl_format_footer(setup)>
      # ... with 3
      #   more
      #   variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>,
      #   `col 05` <ord>,
      #   and ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 18)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3
      #   x 3
      <tbl_format_body(setup)>
        column~1 col 0~2
           <dbl> <chr>  
      1     1.23 a      
      2     2.23 b      
      3     3.23 c      
      <tbl_format_footer(setup)>
      # ... with 3 more
      #   variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>,
      #   `col 05` <ord>,
      #   and
      #   abbreviated ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 19)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3
      #   x 3
      <tbl_format_body(setup)>
        column_~1 col 0~2
            <dbl> <chr>  
      1      1.23 a      
      2      2.23 b      
      3      3.23 c      
      <tbl_format_footer(setup)>
      # ... with 3 more
      #   variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>,
      #   `col 05` <ord>,
      #   and
      #   abbreviated ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 20)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x
      #   3
      <tbl_format_body(setup)>
        column_z~1 col 0~2
             <dbl> <chr>  
      1       1.23 a      
      2       2.23 b      
      3       3.23 c      
      <tbl_format_footer(setup)>
      # ... with 3 more
      #   variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>,
      #   `col 05` <ord>,
      #   and abbreviated
      #   variable ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 21)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x
      #   3
      <tbl_format_body(setup)>
        column_ze~1 col 0~2
              <dbl> <chr>  
      1        1.23 a      
      2        2.23 b      
      3        3.23 c      
      <tbl_format_footer(setup)>
      # ... with 3 more
      #   variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>,
      #   `col 05` <ord>,
      #   and abbreviated
      #   variable ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 22)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zer~1 col 0~2
               <dbl> <chr>  
      1         1.23 a      
      2         2.23 b      
      3         3.23 c      
      <tbl_format_footer(setup)>
      # ... with 3 more
      #   variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>,
      #   `col 05` <ord>,
      #   and abbreviated
      #   variable ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 23)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero~1 col 0~2
                <dbl> <chr>  
      1          1.23 a      
      2          2.23 b      
      3          3.23 c      
      <tbl_format_footer(setup)>
      # ... with 3 more
      #   variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>,
      #   `col 05` <ord>,
      #   and abbreviated
      #   variable names ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 24)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_~1 col 0~2
                 <dbl> <chr>  
      1           1.23 a      
      2           2.23 b      
      3           3.23 c      
      <tbl_format_footer(setup)>
      # ... with 3 more
      #   variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>,
      #   `col 05` <ord>, and
      #   abbreviated
      #   variable names ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 25)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        colum~1 col 0~2 col 0~3
          <dbl> <chr>   <ord>  
      1    1.23 a       a      
      2    2.23 b       b      
      3    3.23 c       c      
      <tbl_format_footer(setup)>
      # ... with 2 more
      #   variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>,
      #   and abbreviated
      #   variable names
      #   1: column_zero_zero, ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 26)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column~1 col 0~2 col 0~3
           <dbl> <chr>   <ord>  
      1     1.23 a       a      
      2     2.23 b       b      
      3     3.23 c       c      
      <tbl_format_footer(setup)>
      # ... with 2 more
      #   variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>,
      #   and abbreviated
      #   variable names
      #   1: column_zero_zero, ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 27)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_~1 col 0~2 col 0~3
            <dbl> <chr>   <ord>  
      1      1.23 a       a      
      2      2.23 b       b      
      3      3.23 c       c      
      <tbl_format_footer(setup)>
      # ... with 2 more
      #   variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>,
      #   and abbreviated
      #   variable names
      #   1: column_zero_zero, ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 28)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_z~1 col 0~2 col 0~3
             <dbl> <chr>   <ord>  
      1       1.23 a       a      
      2       2.23 b       b      
      3       3.23 c       c      
      <tbl_format_footer(setup)>
      # ... with 2 more
      #   variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>,
      #   and abbreviated
      #   variable names
      #   1: column_zero_zero, ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 29)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_ze~1 col 0~2 col 0~3
              <dbl> <chr>   <ord>  
      1        1.23 a       a      
      2        2.23 b       b      
      3        3.23 c       c      
      <tbl_format_footer(setup)>
      # ... with 2 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>, and
      #   abbreviated variable
      #   names
      #   1: column_zero_zero,
      #   2: `col 01`$`col 02`, ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 30)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zer~1 col 0~2 col 0~3
               <dbl> <chr>   <ord>  
      1         1.23 a       a      
      2         2.23 b       b      
      3         3.23 c       c      
      <tbl_format_footer(setup)>
      # ... with 2 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>, and
      #   abbreviated variable
      #   names
      #   1: column_zero_zero,
      #   2: `col 01`$`col 02`, ...
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 31)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero~1 col 0~2 col 0~3
                <dbl> <chr>   <ord>  
      1          1.23 a       a      
      2          2.23 b       b      
      3          3.23 c       c      
      <tbl_format_footer(setup)>
      # ... with 2 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>, and
      #   abbreviated variable names
      #   1: column_zero_zero,
      #   2: `col 01`$`col 02`,
      #   3: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 32)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_~1 col 0~2 col 0~3
                 <dbl> <chr>   <ord>  
      1           1.23 a       a      
      2           2.23 b       b      
      3           3.23 c       c      
      <tbl_format_footer(setup)>
      # ... with 2 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>, and
      #   abbreviated variable names
      #   1: column_zero_zero,
      #   2: `col 01`$`col 02`,
      #   3: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 33)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_z~1 col 0~2 col 0~3
                  <dbl> <chr>   <ord>  
      1            1.23 a       a      
      2            2.23 b       b      
      3            3.23 c       c      
      <tbl_format_footer(setup)>
      # ... with 2 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>, and
      #   abbreviated variable names
      #   1: column_zero_zero,
      #   2: `col 01`$`col 02`,
      #   3: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 34)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 0~1 col 0~2
                   <dbl> <chr>   <ord>  
      1             1.23 a       a      
      2             2.23 b       b      
      3             3.23 c       c      
      <tbl_format_footer(setup)>
      # ... with 2 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>, and
      #   abbreviated variable names
      #   1: `col 01`$`col 02`,
      #   2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 35)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01~1 col 0~2
                   <dbl> <chr>    <ord>  
      1             1.23 a        a      
      2             2.23 b        b      
      3             3.23 c        c      
      <tbl_format_footer(setup)>
      # ... with 2 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>, and
      #   abbreviated variable names
      #   1: `col 01`$`col 02`,
      #   2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 36)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01$~1 col 0~2
                   <dbl> <chr>     <ord>  
      1             1.23 a         a      
      2             2.23 b         b      
      3             3.23 c         c      
      <tbl_format_footer(setup)>
      # ... with 2 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>, and
      #   abbreviated variable names
      #   1: `col 01`$`col 02`,
      #   2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 37)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01$c~1 col 0~2
                   <dbl> <chr>      <ord>  
      1             1.23 a          a      
      2             2.23 b          b      
      3             3.23 c          c      
      <tbl_format_footer(setup)>
      # ... with 2 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>, and
      #   abbreviated variable names
      #   1: `col 01`$`col 02`,
      #   2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 38)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01$co~1 col 0~2
                   <dbl> <chr>       <ord>  
      1             1.23 a           a      
      2             2.23 b           b      
      3             3.23 c           c      
      <tbl_format_footer(setup)>
      # ... with 2 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>, and
      #   abbreviated variable names
      #   1: `col 01`$`col 02`, 2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 39)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01$col~1 col 0~2
                   <dbl> <chr>        <ord>  
      1             1.23 a            a      
      2             2.23 b            b      
      3             3.23 c            c      
      <tbl_format_footer(setup)>
      # ... with 2 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>, and
      #   abbreviated variable names
      #   1: `col 01`$`col 02`, 2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 40)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01$col ~1 col 0~2
                   <dbl> <chr>         <ord>  
      1             1.23 a             a      
      2             2.23 b             b      
      3             3.23 c             c      
      <tbl_format_footer(setup)>
      # ... with 2 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>, and
      #   abbreviated variable names
      #   1: `col 01`$`col 02`, 2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 41)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01$col 0~1 col 0~2
                   <dbl> <chr>          <ord>  
      1             1.23 a              a      
      2             2.23 b              b      
      3             3.23 c              c      
      <tbl_format_footer(setup)>
      # ... with 2 more variables:
      #   `col 01`$`col 03` <chr>,
      #   $`col 04` <int[,3]>, and abbreviated
      #   variable names 1: `col 01`$`col 02`,
      #   2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 42)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 0~1 $col ~2 col 0~3
                   <dbl> <chr>   <chr>   <ord>  
      1             1.23 a       A       a      
      2             2.23 b       B       b      
      3             3.23 c       C       c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable:
      #   `col 01`$`col 04` <int[,3]>, and
      #   abbreviated variable names
      #   1: `col 01`$`col 02`, 2: $`col 03`,
      #   3: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 43)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01~1 $col ~2 col 0~3
                   <dbl> <chr>    <chr>   <ord>  
      1             1.23 a        A       a      
      2             2.23 b        B       b      
      3             3.23 c        C       c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable:
      #   `col 01`$`col 04` <int[,3]>, and
      #   abbreviated variable names
      #   1: `col 01`$`col 02`, 2: $`col 03`,
      #   3: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 44)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01$~1 $col ~2 col 0~3
                   <dbl> <chr>     <chr>   <ord>  
      1             1.23 a         A       a      
      2             2.23 b         B       b      
      3             3.23 c         C       c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable:
      #   `col 01`$`col 04` <int[,3]>, and
      #   abbreviated variable names
      #   1: `col 01`$`col 02`, 2: $`col 03`,
      #   3: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 45)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01$c~1 $col ~2 col 0~3
                   <dbl> <chr>      <chr>   <ord>  
      1             1.23 a          A       a      
      2             2.23 b          B       b      
      3             3.23 c          C       c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable:
      #   `col 01`$`col 04` <int[,3]>, and
      #   abbreviated variable names
      #   1: `col 01`$`col 02`, 2: $`col 03`,
      #   3: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 46)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01$co~1 $col ~2 col 0~3
                   <dbl> <chr>       <chr>   <ord>  
      1             1.23 a           A       a      
      2             2.23 b           B       b      
      3             3.23 c           C       c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable:
      #   `col 01`$`col 04` <int[,3]>, and
      #   abbreviated variable names
      #   1: `col 01`$`col 02`, 2: $`col 03`,
      #   3: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 47)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01$col~1 $col ~2 col 0~3
                   <dbl> <chr>        <chr>   <ord>  
      1             1.23 a            A       a      
      2             2.23 b            B       b      
      3             3.23 c            C       c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable:
      #   `col 01`$`col 04` <int[,3]>, and
      #   abbreviated variable names
      #   1: `col 01`$`col 02`, 2: $`col 03`,
      #   3: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 48)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01$col ~1 $col ~2 col 0~3
                   <dbl> <chr>         <chr>   <ord>  
      1             1.23 a             A       a      
      2             2.23 b             B       b      
      3             3.23 c             C       c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable:
      #   `col 01`$`col 04` <int[,3]>, and
      #   abbreviated variable names
      #   1: `col 01`$`col 02`, 2: $`col 03`,
      #   3: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 49)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01$col 0~1 $col ~2 col 0~3
                   <dbl> <chr>          <chr>   <ord>  
      1             1.23 a              A       a      
      2             2.23 b              B       b      
      3             3.23 c              C       c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable:
      #   `col 01`$`col 04` <int[,3]>, and abbreviated
      #   variable names 1: `col 01`$`col 02`,
      #   2: $`col 03`, 3: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 50)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 0~1 $col ~2 $col ~3 col 0~4
                   <dbl> <chr>   <chr>     <int> <ord>  
      1             1.23 a       A             1 a      
      2             2.23 b       B             2 b      
      3             3.23 c       C             3 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable:
      #   `col 01`$`col 04`[2:3] <int>, and abbreviated
      #   variable names 1: `col 01`$`col 02`,
      #   2: $`col 03`, 3: $`col 04`[,"A"], 4: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 51)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01~1 $col ~2 $col ~3 col 0~4
                   <dbl> <chr>    <chr>     <int> <ord>  
      1             1.23 a        A             1 a      
      2             2.23 b        B             2 b      
      3             3.23 c        C             3 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable:
      #   `col 01`$`col 04`[2:3] <int>, and abbreviated
      #   variable names 1: `col 01`$`col 02`,
      #   2: $`col 03`, 3: $`col 04`[,"A"], 4: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 52)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01$~1 $col ~2 $col ~3 col 0~4
                   <dbl> <chr>     <chr>     <int> <ord>  
      1             1.23 a         A             1 a      
      2             2.23 b         B             2 b      
      3             3.23 c         C             3 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable:
      #   `col 01`$`col 04`[2:3] <int>, and abbreviated
      #   variable names 1: `col 01`$`col 02`,
      #   2: $`col 03`, 3: $`col 04`[,"A"], 4: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 53)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01$c~1 $col ~2 $col ~3 col 0~4
                   <dbl> <chr>      <chr>     <int> <ord>  
      1             1.23 a          A             1 a      
      2             2.23 b          B             2 b      
      3             3.23 c          C             3 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable:
      #   `col 01`$`col 04`[2:3] <int>, and abbreviated
      #   variable names 1: `col 01`$`col 02`,
      #   2: $`col 03`, 3: $`col 04`[,"A"], 4: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 54)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01$co~1 $col ~2 $col ~3 col 0~4
                   <dbl> <chr>       <chr>     <int> <ord>  
      1             1.23 a           A             1 a      
      2             2.23 b           B             2 b      
      3             3.23 c           C             3 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable:
      #   `col 01`$`col 04`[2:3] <int>, and abbreviated
      #   variable names 1: `col 01`$`col 02`,
      #   2: $`col 03`, 3: $`col 04`[,"A"], 4: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 55)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01$col~1 $col ~2 $col ~3 col 0~4
                   <dbl> <chr>        <chr>     <int> <ord>  
      1             1.23 a            A             1 a      
      2             2.23 b            B             2 b      
      3             3.23 c            C             3 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable:
      #   `col 01`$`col 04`[2:3] <int>, and abbreviated
      #   variable names 1: `col 01`$`col 02`, 2: $`col 03`,
      #   3: $`col 04`[,"A"], 4: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 56)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01$col ~1 $col ~2 $col ~3 col 0~4
                   <dbl> <chr>         <chr>     <int> <ord>  
      1             1.23 a             A             1 a      
      2             2.23 b             B             2 b      
      3             3.23 c             C             3 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable:
      #   `col 01`$`col 04`[2:3] <int>, and abbreviated
      #   variable names 1: `col 01`$`col 02`, 2: $`col 03`,
      #   3: $`col 04`[,"A"], 4: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 57)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01$col 0~1 $col ~2 $col ~3 col 0~4
                   <dbl> <chr>          <chr>     <int> <ord>  
      1             1.23 a              A             1 a      
      2             2.23 b              B             2 b      
      3             3.23 c              C             3 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable:
      #   `col 01`$`col 04`[2:3] <int>, and abbreviated
      #   variable names 1: `col 01`$`col 02`, 2: $`col 03`,
      #   3: $`col 04`[,"A"], 4: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 58)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01$col 02~1 $col ~2 $col ~3 col 0~4
                   <dbl> <chr>           <chr>     <int> <ord>  
      1             1.23 a               A             1 a      
      2             2.23 b               B             2 b      
      3             3.23 c               C             3 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable: `col 01`$`col 04`[2:3] <int>,
      #   and abbreviated variable names 1: `col 01`$`col 02`,
      #   2: $`col 03`, 3: $`col 04`[,"A"], 4: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 59)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero col 01$col 02~1 $col 0~2 $col ~3 col 0~4
                   <dbl> <chr>           <chr>      <int> <ord>  
      1             1.23 a               A              1 a      
      2             2.23 b               B              2 b      
      3             3.23 c               C              3 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable: `col 01`$`col 04`[2:3] <int>,
      #   and abbreviated variable names 1: `col 01`$`col 02`,
      #   2: $`col 03`, 3: $`col 04`[,"A"], 4: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 60)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $col ~1 $col ~2 col 0~3
                   <dbl> <chr>             <chr>     <int> <ord>  
      1             1.23 a                 A             1 a      
      2             2.23 b                 B             2 b      
      3             3.23 c                 C             3 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable: `col 01`$`col 04`[2:3] <int>,
      #   and abbreviated variable names 1: $`col 03`,
      #   2: $`col 04`[,"A"], 3: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 61)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $col 0~1 $col ~2 col 0~3
                   <dbl> <chr>             <chr>      <int> <ord>  
      1             1.23 a                 A              1 a      
      2             2.23 b                 B              2 b      
      3             3.23 c                 C              3 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable: `col 01`$`col 04`[2:3] <int>,
      #   and abbreviated variable names 1: $`col 03`,
      #   2: $`col 04`[,"A"], 3: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 62)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col ~1 col 0~2
                   <dbl> <chr>             <chr>       <int> <ord>  
      1             1.23 a                 A               1 a      
      2             2.23 b                 B               2 b      
      3             3.23 c                 C               3 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable: `col 01`$`col 04`[2:3] <int>, and
      #   abbreviated variable names 1: $`col 04`[,"A"],
      #   2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 63)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 0~1 col 0~2
                   <dbl> <chr>             <chr>        <int> <ord>  
      1             1.23 a                 A                1 a      
      2             2.23 b                 B                2 b      
      3             3.23 c                 C                3 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable: `col 01`$`col 04`[2:3] <int>, and
      #   abbreviated variable names 1: $`col 04`[,"A"], 2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 64)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 04~1 col 0~2
                   <dbl> <chr>             <chr>         <int> <ord>  
      1             1.23 a                 A                 1 a      
      2             2.23 b                 B                 2 b      
      3             3.23 c                 C                 3 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable: `col 01`$`col 04`[2:3] <int>, and
      #   abbreviated variable names 1: $`col 04`[,"A"], 2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 65)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 04[~1 col 0~2
                   <dbl> <chr>             <chr>          <int> <ord>  
      1             1.23 a                 A                  1 a      
      2             2.23 b                 B                  2 b      
      3             3.23 c                 C                  3 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable: `col 01`$`col 04`[2:3] <int>, and
      #   abbreviated variable names 1: $`col 04`[,"A"], 2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 66)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 04[,~1 col 0~2
                   <dbl> <chr>             <chr>           <int> <ord>  
      1             1.23 a                 A                   1 a      
      2             2.23 b                 B                   2 b      
      3             3.23 c                 C                   3 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable: `col 01`$`col 04`[2:3] <int>, and
      #   abbreviated variable names 1: $`col 04`[,"A"], 2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 67)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 04[,"~1 col 0~2
                   <dbl> <chr>             <chr>            <int> <ord>  
      1             1.23 a                 A                    1 a      
      2             2.23 b                 B                    2 b      
      3             3.23 c                 C                    3 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable: `col 01`$`col 04`[2:3] <int>, and
      #   abbreviated variable names 1: $`col 04`[,"A"], 2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 68)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 04[,"A~1 col 0~2
                   <dbl> <chr>             <chr>             <int> <ord>  
      1             1.23 a                 A                     1 a      
      2             2.23 b                 B                     2 b      
      3             3.23 c                 C                     3 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable: `col 01`$`col 04`[2:3] <int>, and
      #   abbreviated variable names 1: $`col 04`[,"A"], 2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 69)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 04[,"A"~1 col 0~2
                   <dbl> <chr>             <chr>              <int> <ord>  
      1             1.23 a                 A                      1 a      
      2             2.23 b                 B                      2 b      
      3             3.23 c                 C                      3 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable: `col 01`$`col 04`[2:3] <int>, and
      #   abbreviated variable names 1: $`col 04`[,"A"], 2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 70)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col ~1 $[,"B"] col 0~2
                   <dbl> <chr>             <chr>       <int>   <int> <ord>  
      1             1.23 a                 A               1       4 a      
      2             2.23 b                 B               2       5 b      
      3             3.23 c                 C               3       6 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable: `col 01`$`col 04`[3] <int>, and
      #   abbreviated variable names 1: $`col 04`[,"A"], 2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 71)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 0~1 $[,"B"] col 0~2
                   <dbl> <chr>             <chr>        <int>   <int> <ord>  
      1             1.23 a                 A                1       4 a      
      2             2.23 b                 B                2       5 b      
      3             3.23 c                 C                3       6 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable: `col 01`$`col 04`[3] <int>, and
      #   abbreviated variable names 1: $`col 04`[,"A"], 2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 72)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 04~1 $[,"B"] col 0~2
                   <dbl> <chr>             <chr>         <int>   <int> <ord>  
      1             1.23 a                 A                 1       4 a      
      2             2.23 b                 B                 2       5 b      
      3             3.23 c                 C                 3       6 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable: `col 01`$`col 04`[3] <int>, and abbreviated
      #   variable names 1: $`col 04`[,"A"], 2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 73)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 04[~1 $[,"B"] col 0~2
                   <dbl> <chr>             <chr>          <int>   <int> <ord>  
      1             1.23 a                 A                  1       4 a      
      2             2.23 b                 B                  2       5 b      
      3             3.23 c                 C                  3       6 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable: `col 01`$`col 04`[3] <int>, and abbreviated
      #   variable names 1: $`col 04`[,"A"], 2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 74)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 04[,~1 $[,"B"] col 0~2
                   <dbl> <chr>             <chr>           <int>   <int> <ord>  
      1             1.23 a                 A                   1       4 a      
      2             2.23 b                 B                   2       5 b      
      3             3.23 c                 C                   3       6 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable: `col 01`$`col 04`[3] <int>, and abbreviated
      #   variable names 1: $`col 04`[,"A"], 2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 75)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 04[,"~1 $[,"B"] col 0~2
                   <dbl> <chr>             <chr>            <int>   <int> <ord>  
      1             1.23 a                 A                    1       4 a      
      2             2.23 b                 B                    2       5 b      
      3             3.23 c                 C                    3       6 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable: `col 01`$`col 04`[3] <int>, and abbreviated
      #   variable names 1: $`col 04`[,"A"], 2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 76)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 04[,"A~1 $[,"B"] col 0~2
                   <dbl> <chr>             <chr>             <int>   <int> <ord>  
      1             1.23 a                 A                     1       4 a      
      2             2.23 b                 B                     2       5 b      
      3             3.23 c                 C                     3       6 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable: `col 01`$`col 04`[3] <int>, and abbreviated
      #   variable names 1: $`col 04`[,"A"], 2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 77)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 04[,"A"~1 $[,"B"] col 0~2
                   <dbl> <chr>             <chr>              <int>   <int> <ord>  
      1             1.23 a                 A                      1       4 a      
      2             2.23 b                 B                      2       5 b      
      3             3.23 c                 C                      3       6 c      
      <tbl_format_footer(setup)>
      # ... with 1 more variable: `col 01`$`col 04`[3] <int>, and abbreviated
      #   variable names 1: $`col 04`[,"A"], 2: `col 05`
      # i Use `colnames()` to see all variable names
    Code
      tbl_format_setup(x, width = 78)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col ~1 $[,"B"] $[,"C"] col 0~2
                   <dbl> <chr>             <chr>       <int>   <int>   <int> <ord>  
      1             1.23 a                 A               1       4       7 a      
      2             2.23 b                 B               2       5       8 b      
      3             3.23 c                 C               3       6       9 c      
      <tbl_format_footer(setup)>
      # ... with abbreviated variable names 1: $`col 04`[,"A"], 2: `col 05`
    Code
      tbl_format_setup(x, width = 79)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 0~1 $[,"B"] $[,"C"] col 0~2
                   <dbl> <chr>             <chr>        <int>   <int>   <int> <ord>  
      1             1.23 a                 A                1       4       7 a      
      2             2.23 b                 B                2       5       8 b      
      3             3.23 c                 C                3       6       9 c      
      <tbl_format_footer(setup)>
      # ... with abbreviated variable names 1: $`col 04`[,"A"], 2: `col 05`
    Code
      tbl_format_setup(x, width = 80)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 04~1 $[,"B"] $[,"C"] col 0~2
                   <dbl> <chr>             <chr>         <int>   <int>   <int> <ord>  
      1             1.23 a                 A                 1       4       7 a      
      2             2.23 b                 B                 2       5       8 b      
      3             3.23 c                 C                 3       6       9 c      
      <tbl_format_footer(setup)>
      # ... with abbreviated variable names 1: $`col 04`[,"A"], 2: `col 05`
    Code
      tbl_format_setup(x, width = 81)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 04[~1 $[,"B"] $[,"C"] col 0~2
                   <dbl> <chr>             <chr>          <int>   <int>   <int> <ord>  
      1             1.23 a                 A                  1       4       7 a      
      2             2.23 b                 B                  2       5       8 b      
      3             3.23 c                 C                  3       6       9 c      
      <tbl_format_footer(setup)>
      # ... with abbreviated variable names 1: $`col 04`[,"A"], 2: `col 05`
    Code
      tbl_format_setup(x, width = 82)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 04[,~1 $[,"B"] $[,"C"] col 0~2
                   <dbl> <chr>             <chr>           <int>   <int>   <int> <ord>  
      1             1.23 a                 A                   1       4       7 a      
      2             2.23 b                 B                   2       5       8 b      
      3             3.23 c                 C                   3       6       9 c      
      <tbl_format_footer(setup)>
      # ... with abbreviated variable names 1: $`col 04`[,"A"], 2: `col 05`
    Code
      tbl_format_setup(x, width = 83)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 04[,"~1 $[,"B"] $[,"C"] col 0~2
                   <dbl> <chr>             <chr>            <int>   <int>   <int> <ord>  
      1             1.23 a                 A                    1       4       7 a      
      2             2.23 b                 B                    2       5       8 b      
      3             3.23 c                 C                    3       6       9 c      
      <tbl_format_footer(setup)>
      # ... with abbreviated variable names 1: $`col 04`[,"A"], 2: `col 05`
    Code
      tbl_format_setup(x, width = 84)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 04[,"A~1 $[,"B"] $[,"C"] col 0~2
                   <dbl> <chr>             <chr>             <int>   <int>   <int> <ord>  
      1             1.23 a                 A                     1       4       7 a      
      2             2.23 b                 B                     2       5       8 b      
      3             3.23 c                 C                     3       6       9 c      
      <tbl_format_footer(setup)>
      # ... with abbreviated variable names 1: $`col 04`[,"A"], 2: `col 05`
    Code
      tbl_format_setup(x, width = 85)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $col 04[,"A"~1 $[,"B"] $[,"C"] col 0~2
                   <dbl> <chr>             <chr>              <int>   <int>   <int> <ord>  
      1             1.23 a                 A                      1       4       7 a      
      2             2.23 b                 B                      2       5       8 b      
      3             3.23 c                 C                      3       6       9 c      
      <tbl_format_footer(setup)>
      # ... with abbreviated variable names 1: $`col 04`[,"A"], 2: `col 05`
    Code
      tbl_format_setup(x, width = 86)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] $[,"B"] $[,"C"] col 0~1
                   <dbl> <chr>             <chr>               <int>   <int>   <int> <ord>  
      1             1.23 a                 A                       1       4       7 a      
      2             2.23 b                 B                       2       5       8 b      
      3             3.23 c                 C                       3       6       9 c      
      <tbl_format_footer(setup)>
      # ... with abbreviated variable name 1: `col 05`
    Code
      tbl_format_setup(x, width = 87)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] $[,"B"] $[,"C"] `col 05`
                   <dbl> <chr>             <chr>               <int>   <int>   <int> <ord>   
      1             1.23 a                 A                       1       4       7 a       
      2             2.23 b                 B                       2       5       8 b       
      3             3.23 c                 C                       3       6       9 c       
      <tbl_format_footer(setup)>
    Code
      tbl_format_setup(x, width = 88)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] $[,"B"] $[,"C"] `col 05`
                   <dbl> <chr>             <chr>               <int>   <int>   <int> <ord>   
      1             1.23 a                 A                       1       4       7 a       
      2             2.23 b                 B                       2       5       8 b       
      3             3.23 c                 C                       3       6       9 c       
      <tbl_format_footer(setup)>
    Code
      tbl_format_setup(x, width = 89)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] $[,"B"] $[,"C"] `col 05`
                   <dbl> <chr>             <chr>               <int>   <int>   <int> <ord>   
      1             1.23 a                 A                       1       4       7 a       
      2             2.23 b                 B                       2       5       8 b       
      3             3.23 c                 C                       3       6       9 c       
      <tbl_format_footer(setup)>
    Code
      tbl_format_setup(x, width = Inf)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 3 x 3
      <tbl_format_body(setup)>
        column_zero_zero `col 01`$`col 02` $`col 03` $`col 04`[,"A"] $[,"B"] $[,"C"] `col 05`
                   <dbl> <chr>             <chr>               <int>   <int>   <int> <ord>   
      1             1.23 a                 A                       1       4       7 a       
      2             2.23 b                 B                       2       5       8 b       
      3             3.23 c                 C                       3       6       9 c       
      <tbl_format_footer(setup)>

# tbl_format_setup() results

    Code
      tbl_format_setup(as_tbl(mtcars), width = 80)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 32 x 11
      <tbl_format_body(setup)>
           mpg   cyl  disp    hp  drat    wt  qsec    vs    am  gear  carb
       * <dbl> <dbl> <dbl> <dbl> <dbl> <dbl> <dbl> <dbl> <dbl> <dbl> <dbl>
       1  21       6  160    110  3.9   2.62  16.5     0     1     4     4
       2  21       6  160    110  3.9   2.88  17.0     0     1     4     4
       3  22.8     4  108     93  3.85  2.32  18.6     1     1     4     1
       4  21.4     6  258    110  3.08  3.22  19.4     1     0     3     1
       5  18.7     8  360    175  3.15  3.44  17.0     0     0     3     2
       6  18.1     6  225    105  2.76  3.46  20.2     1     0     3     1
       7  14.3     8  360    245  3.21  3.57  15.8     0     0     3     4
       8  24.4     4  147.    62  3.69  3.19  20       1     0     4     2
       9  22.8     4  141.    95  3.92  3.15  22.9     1     0     4     2
      10  19.2     6  168.   123  3.92  3.44  18.3     1     0     4     4
      <tbl_format_footer(setup)>
      # ... with 22 more rows
      # i Use `print(n = ...)` to see more rows
    Code
      tbl_format_setup(as_tbl(mtcars), width = 30, n = 8)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 32 x 11
      <tbl_format_body(setup)>
          mpg   cyl  disp    hp
      * <dbl> <dbl> <dbl> <dbl>
      1  21       6  160    110
      2  21       6  160    110
      3  22.8     4  108     93
      4  21.4     6  258    110
      5  18.7     8  360    175
      6  18.1     6  225    105
      7  14.3     8  360    245
      8  24.4     4  147.    62
      <tbl_format_footer(setup)>
      # ... with 24 more rows, and
      #   7 more variables:
      #   drat <dbl>, wt <dbl>,
      #   qsec <dbl>, vs <dbl>,
      #   am <dbl>, gear <dbl>,
      #   carb <dbl>
      # i Use `print(n = ...)` to see more rows, and `colnames()` to see all variable names
    Code
      tbl_format_setup(as_tbl(trees), width = 30, n = 100)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 31 x 3
      <tbl_format_body(setup)>
         Girth Height Volume
         <dbl>  <dbl>  <dbl>
       1   8.3     70   10.3
       2   8.6     65   10.3
       3   8.8     63   10.2
       4  10.5     72   16.4
       5  10.7     81   18.8
       6  10.8     83   19.7
       7  11       66   15.6
       8  11       75   18.2
       9  11.1     80   22.6
      10  11.2     75   19.9
      11  11.3     79   24.2
      12  11.4     76   21  
      13  11.4     76   21.4
      14  11.7     69   21.3
      15  12       75   19.1
      16  12.9     74   22.2
      17  12.9     85   33.8
      18  13.3     86   27.4
      19  13.7     71   25.7
      20  13.8     64   24.9
      21  14       78   34.5
      22  14.2     80   31.7
      23  14.5     74   36.3
      24  16       72   38.3
      25  16.3     77   42.6
      26  17.3     81   55.4
      27  17.5     82   55.7
      28  17.9     80   58.3
      29  18       80   51.5
      30  18       80   51  
      31  20.6     87   77  
      <tbl_format_footer(setup)>
    Code
      # Unknown rows
      tbl_format_setup(as_unknown_rows(trees[1:9, ]), width = 30, n = 10)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: ?? x 3
      <tbl_format_body(setup)>
        Girth Height Volume
      * <dbl>  <dbl>  <dbl>
      1   8.3     70   10.3
      2   8.6     65   10.3
      3   8.8     63   10.2
      4  10.5     72   16.4
      5  10.7     81   18.8
      6  10.8     83   19.7
      7  11       66   15.6
      8  11       75   18.2
      9  11.1     80   22.6
      <tbl_format_footer(setup)>
    Code
      tbl_format_setup(as_unknown_rows(trees[1:10, ]), width = 30, n = 10)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: ?? x 3
      <tbl_format_body(setup)>
         Girth Height Volume
       * <dbl>  <dbl>  <dbl>
       1   8.3     70   10.3
       2   8.6     65   10.3
       3   8.8     63   10.2
       4  10.5     72   16.4
       5  10.7     81   18.8
       6  10.8     83   19.7
       7  11       66   15.6
       8  11       75   18.2
       9  11.1     80   22.6
      10  11.2     75   19.9
      <tbl_format_footer(setup)>
    Code
      tbl_format_setup(as_unknown_rows(trees[1:11, ]), width = 30, n = 10)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: ?? x 3
      <tbl_format_body(setup)>
         Girth Height Volume
       * <dbl>  <dbl>  <dbl>
       1   8.3     70   10.3
       2   8.6     65   10.3
       3   8.8     63   10.2
       4  10.5     72   16.4
       5  10.7     81   18.8
       6  10.8     83   19.7
       7  11       66   15.6
       8  11       75   18.2
       9  11.1     80   22.6
      10  11.2     75   19.9
      <tbl_format_footer(setup)>
      # ... with more rows
      # i Use `print(n = ...)` to see more rows
    Code
      # No rows
      tbl_format_setup(new_tbl(list(a = character(), b = logical())), width = 30)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 0 x 2
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # ... with 2 variables:
      #   a <chr>, b <lgl>
      # i Use `colnames()` to see all variable names
    Code
      # No columns
      tbl_format_setup(as_tbl(trees[character()]), width = 30, n = 5L)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 31 x 0
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # i Use `print(n = ...)` to see more rows
    Code
      # No rows, unknown
      tbl_format_setup(as_unknown_rows(trees[integer(), ]), width = 30, n = 5L)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: ?? x 3
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # ... with 3 variables:
      #   Girth <dbl>,
      #   Height <dbl>,
      #   Volume <dbl>
      # i Use `colnames()` to see all variable names
    Code
      # No columns, unknown
      tbl_format_setup(as_unknown_rows(trees[, character()]), width = 30, n = 5L)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: ?? x 0
      <tbl_format_body(setup)>
      <tbl_format_footer(setup)>
      # ... with at least 5 rows
      #   total
      # i Use `print(n = ...)` to see more rows
    Code
      # Long table, unknown
      tbl_format_setup(as_unknown_rows(new_tbl(list(a = seq.int(10000)))), n = 5L,
      width = 30L)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: ?? x 1
      <tbl_format_body(setup)>
            a
        <int>
      1     1
      2     2
      3     3
      4     4
      5     5
      <tbl_format_footer(setup)>
      # ... with more rows
      # i Use `print(n = ...)` to see more rows

# tbl_format_setup() for footnotes

    Code
      tbl_format_setup(width = 73, as_tbl(data_frame(xxxabc = 1, xxxdef = 1, xxxghi = 1,
        xxxjkl = 1, xxxmno = 1, xxxpqr = 1, xxxstu = 1, xxxvwx = 1, xxxyza = 1,
        xxxbcd = 1, xxxefg = 1, xxxhij = 1, xxxklm = 1, xxxnop = 1)))
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 1 x 14
      <tbl_format_body(setup)>
        xxxabc xxxdef xxxghi xxxjkl xxxmno xxxpqr xxxstu xxxvwx xxxyza xxxbcd
         <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>
      1      1      1      1      1      1      1      1      1      1      1
      <tbl_format_footer(setup)>
      # ... with 4 more variables: xxxefg <dbl>, xxxhij <dbl>, xxxklm <dbl>,
      #   xxxnop <dbl>
      # i Use `colnames()` to see all variable names

# tbl_format_setup() for footnotes with min_title_chars = 4

    Code
      tbl_format_setup(width = 73, as_tbl(data_frame(xxxabc = 1, xxxdef = 1, xxxghi = 1,
        xxxjkl = 1, xxxmno = 1, xxxpqr = 1, xxxstu = 1, xxxvwx = 1, xxxyza = 1,
        xxxbcd = 1, xxxefg = 1, xxxhij = 1, xxxklm = 1, xxxnop = 1)))
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 1 x 14
      <tbl_format_body(setup)>
        xxxabc xxxdef xxxghi xxxjkl xxxmno xxxpqr xxxstu xxxvwx xxxyza xxxbcd
         <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>
      1      1      1      1      1      1      1      1      1      1      1
      <tbl_format_footer(setup)>
      # ... with 4 more variables: xxxefg <dbl>, xxxhij <dbl>, xxxklm <dbl>,
      #   xxxnop <dbl>
      # i Use `colnames()` to see all variable names

# tbl_format_setup() for footnotes with UTF-8 output

    Code
      tbl_format_setup(width = 73, as_tbl(data_frame(xxxabc = 1, xxxdef = 1, xxxghi = 1,
        xxxjkl = 1, xxxmno = 1, xxxpqr = 1, xxxstu = 1, xxxvwx = 1, xxxyza = 1,
        xxxbcd = 1, xxxefg = 1, xxxhij = 1, xxxklm = 1, xxxnop = 1)))
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 1 × 14
      <tbl_format_body(setup)>
        xxxabc xxxdef xxxghi xxxjkl xxxmno xxxpqr xxxstu xxxvwx xxxyza xxxbcd
         <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>  <dbl>
      1      1      1      1      1      1      1      1      1      1      1
      <tbl_format_footer(setup)>
      # … with 4 more variables: xxxefg <dbl>, xxxhij <dbl>, xxxklm <dbl>,
      #   xxxnop <dbl>
      # ℹ Use `colnames()` to see all variable names

# output test

    Code
      rowidformat(3)
    Output
      <pillar>
       
      1
      2
      3
    Code
      rowidformat(12, has_title_row = TRUE, has_star = TRUE)
    Output
      <pillar>
        
       *
       1
       2
       3
       4
       5
       6
       7
       8
       9
      10
      11
      12

# colonnade_compute_tiered_col_widths_df()

    Code
      colonnade_compute_tiered_col_widths_df(rep(30, 4), rep(15, 4), rep(60, 4))
    Output
        id tier width offset_after
      1  1    1    30           30
      2  2    2    30           30
      3  3    3    30           30
      4  4    4    30           30
    Code
      colonnade_compute_tiered_col_widths_df(rep(30, 4), rep(15, 4), rep(60, 3))
    Output
        id tier width offset_after max_widths
      1  1    1    30           30         30
      2  2    2    30           30         30
      3  3    3    30           30         30
      4  4    3    15           60         30
    Code
      colonnade_compute_tiered_col_widths_df(rep(30, 4), rep(15, 4), rep(60, 2))
    Output
        id tier width offset_after max_widths
      1  1    1    30           30         30
      2  2    2    15           28         30
      3  3    2    15           44         30
      4  4    2    15           60         30
    Code
      colonnade_compute_tiered_col_widths_df(rep(30, 5), rep(15, 5), rep(60, 3))
    Output
        id tier width offset_after max_widths
      1  1    1    30           30         30
      2  2    2    30           30         30
      3  3    3    15           28         30
      4  4    3    15           44         30
      5  5    3    15           60         30
    Code
      colonnade_compute_tiered_col_widths_df(rep(30, 5), rep(15, 5), rep(60, 4))
    Output
        id tier width offset_after max_widths
      1  1    1    30           30         30
      2  2    2    30           30         30
      3  3    3    30           30         30
      4  4    4    30           30         30
      5  5    4    15           60         30
    Code
      colonnade_compute_tiered_col_widths_df(rep(30, 10), rep(15, 10), rep(60, 5))
    Output
         id tier width offset_after max_widths
      1   1    1    30           30         30
      2   2    2    30           30         30
      3   3    3    30           30         30
      4   4    3    15           60         30
      5   5    4    15           28         30
      6   6    4    15           44         30
      7   7    4    15           60         30
      8   8    5    15           28         30
      9   9    5    15           44         30
      10 10    5    15           60         30
    Code
      colonnade_compute_tiered_col_widths_df(c(6, 5, 10), c(5, 5, 10), 21)
    Output
        id tier width offset_after
      1  1    1     5            5
      2  2    1     5           11
      3  3   NA    10           NA
    Code
      colonnade_compute_tiered_col_widths_df(c(6, 5, 10), c(5, 5, 10), 22)
    Output
        id tier width offset_after max_widths
      1  1    1     5            5          6
      2  2    1     5           11          5
      3  3    1    10           22         10
    Code
      colonnade_compute_tiered_col_widths_df(c(6, 5, 10), c(5, 5, 10), 23)
    Output
        id tier width offset_after
      1  1    1     6            6
      2  2    1     5           12
      3  3    1    10           23
    Code
      colonnade_compute_tiered_col_widths_df(c(5, 6, 10), c(5, 5, 10), 21)
    Output
        id tier width offset_after
      1  1    1     5            5
      2  2    1     5           11
      3  3   NA    10           NA
    Code
      colonnade_compute_tiered_col_widths_df(c(5, 6, 10), c(5, 5, 10), 22)
    Output
        id tier width offset_after max_widths
      1  1    1     5            5          5
      2  2    1     5           11          6
      3  3    1    10           22         10
    Code
      colonnade_compute_tiered_col_widths_df(c(5, 6, 10), c(5, 5, 10), 23)
    Output
        id tier width offset_after
      1  1    1     5            5
      2  2    1     6           12
      3  3    1    10           23

# distribute_pillars()

    Code
      distribute_pillars(1:3, 10)
    Output
        id tier width offset_after
      1  1    1     1            1
      2  2    1     2            4
      3  3    1     3            8
    Code
      distribute_pillars(1:3, 5)
    Output
        id tier width offset_after
      1  1    1     1            1
      2  2    1     2            4
      3  3   NA     3           NA
    Code
      distribute_pillars(1:3, c(5, 5))
    Output
        id tier width offset_after
      1  1    1     1            1
      2  2    1     2            4
      3  3    2     3            3
    Code
      distribute_pillars(1:5, 7:9)
    Output
        id tier width offset_after
      1  1    1     1            1
      2  2    1     2            4
      3  3    2     3            3
      4  4    2     4            8
      5  5    3     5            5
    Code
      distribute_pillars(3:5, 8:10)
    Output
        id tier width offset_after
      1  1    1     3            3
      2  2    1     4            8
      3  3    2     5            5
    Code
      distribute_pillars(5:3, 9:8)
    Output
        id tier width offset_after
      1  1    1     5            5
      2  2    2     4            4
      3  3    2     3            8
    Code
      distribute_pillars(c(5, 5), c(2, 8, 8))
    Output
        id tier width offset_after
      1  1    2     5            5
      2  2    3     5            5
    Code
      distribute_pillars(c(5, 5), c(8, 8, 2))
    Output
        id tier width offset_after
      1  1    1     5            5
      2  2    2     5            5

# distribute_pillars_rev()

    Code
      distribute_pillars_rev(1:3, 10)
    Output
        id tier width offset_after
      1  1    1     1            3
      2  2    1     2            6
      3  3    1     3           10
    Code
      distribute_pillars_rev(1:3, 5)
    Output
        id tier width offset_after
      1  1   NA     1            5
      2  2   NA     2           NA
      3  3    1     3           NA
    Code
      distribute_pillars_rev(1:3, c(5, 5))
    Output
        id tier width offset_after
      1  1    1     1            2
      2  2    1     2            5
      3  3    2     3            5
    Code
      distribute_pillars_rev(1:5, 7:9)
    Output
        id tier width offset_after
      1  1    1     1            4
      2  2    1     2            7
      3  3    2     3            3
      4  4    2     4            8
      5  5    3     5            9
    Code
      distribute_pillars_rev(3:5, 8:10)
    Output
        id tier width offset_after
      1  1    2     3            9
      2  2    3     4            4
      3  3    3     5           10
    Code
      distribute_pillars_rev(5:3, 9:8)
    Output
        id tier width offset_after
      1  1    1     5            9
      2  2    2     4            4
      3  3    2     3            8
    Code
      distribute_pillars_rev(c(5, 5), c(8, 8, 2))
    Output
        id tier width offset_after
      1  1    1     5            8
      2  2    2     5            8
    Code
      distribute_pillars_rev(c(5, 5), c(2, 8, 8))
    Output
        id tier width offset_after
      1  1    2     5            8
      2  2    3     5            8

# ctl_new_pillar()

    Code
      ctl_new_pillar(new_tbl(), 1:3, width = 20, title = "a")
    Output
      <pillar>
          a
      <int>
          1
          2
          3

# ctl_new_pillar_list()

    Code
      ctl_new_pillar_list(new_tbl(), 1:3, width = 20, title = "a")
    Output
      [[1]]
      <pillar>
          a
      <int>
          1
          2
          3
      
      attr(,"remaining_width")
      [1] 14
      attr(,"simple")
      [1] TRUE
    Code
      ctl_new_pillar_list(new_tbl(), trees[1:3, ], width = 8, title = "a")
    Output
      [[1]]
      <pillar>
      a$Girth
        <dbl>
          8.3
          8.6
          8.8
      
      attr(,"extra")
      [1] "Height" "Volume"
      attr(,"remaining_width")
      [1] 0
      attr(,"simple")
      [1] FALSE
    Code
      ctl_new_pillar_list(new_tbl(), trees[1:3, ], width = 20, title = "a")
    Output
      [[1]]
      <pillar>
      a$Girth
        <dbl>
          8.3
          8.6
          8.8
      
      [[2]]
      <pillar>
      $Height
        <dbl>
           70
           65
           63
      
      attr(,"extra")
      [1] "Volume"
      attr(,"remaining_width")
      [1] 4
      attr(,"simple")
      [1] FALSE
    Code
      ctl_new_pillar_list(new_tbl(), as.matrix(trees[1:3, ]), width = 8, title = "a")
    Output
      list()
      attr(,"extra")
      [1] 1 2 3
      attr(,"remaining_width")
      [1] 8
      attr(,"simple")
      [1] FALSE
    Code
      ctl_new_pillar_list(new_tbl(), as.matrix(trees[1:3, ]), width = 20, title = "a")
    Output
      [[1]]
      <pillar>
      a[,"Girth"]
            <dbl>
              8.3
              8.6
              8.8
      
      attr(,"extra")
      [1] 2 3
      attr(,"remaining_width")
      [1] 8
      attr(,"simple")
      [1] FALSE
    Code
      ctl_new_pillar_list(new_tbl(), matrix(1:6, ncol = 2), width = 8, title = "a")
    Output
      [[1]]
      <pillar>
      a[,1]
      <int>
          1
          2
          3
      
      attr(,"extra")
      [1] 2
      attr(,"remaining_width")
      [1] 2
      attr(,"simple")
      [1] FALSE
    Code
      ctl_new_pillar_list(new_tbl(), matrix(1:6, ncol = 2), width = 20, title = "a")
    Output
      [[1]]
      <pillar>
      a[,1]
      <int>
          1
          2
          3
      
      [[2]]
      <pillar>
       [,2]
      <int>
          4
          5
          6
      
      attr(,"extra")
      integer(0)
      attr(,"remaining_width")
      [1] 8
      attr(,"simple")
      [1] FALSE
    Code
      ctl_new_pillar_list(new_tbl(), matrix(1:6, ncol = 3), width = 10, title = "a")
    Output
      [[1]]
      <pillar>
      a[,1]
      <int>
          1
          2
      
      attr(,"extra")
      [1] 2 3
      attr(,"remaining_width")
      [1] 4
      attr(,"simple")
      [1] FALSE

# ctl_new_pillar_list() for tables

    Code
      ctl_new_pillar_list(new_tbl(), Titanic, width = 20, title = "a")
    Output
      [[1]]
      <pillar>
                    a
      <table[,2,2,2]>
                0 ...
                0 ...
               35 ...
                0 ...
      
      attr(,"remaining_width")
      [1] 13
      attr(,"simple")
      [1] TRUE

# compound pillars and multiple tiers

    Code
      options(width = 15)
      tbl_format_setup(x, width = 30)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 1 x 2
      <tbl_format_body(setup)>
            a   b$x
        <dbl> <dbl>
      1     3     1
           $y
        <dbl>
      1     2
      <tbl_format_footer(setup)>
    Code
      options(width = 10)
      tbl_format_setup(x, width = 30)
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 1 x 2
      <tbl_format_body(setup)>
            a
        <dbl>
      1     3
          b$x
        <dbl>
      1     1
           $y
        <dbl>
      1     2
      <tbl_format_footer(setup)>

# compound pillars with zero columns (#402)

    Code
      tbl_format_setup(new_tbl(list(x = 1:2, y = matrix(integer(), ncol = 0, nrow = 2))))
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 2 x 2
      <tbl_format_body(setup)>
            x y        
        <int> <int[,0]>
      1     1          
      2     2          
      <tbl_format_footer(setup)>
    Code
      tbl_format_setup(new_tbl(list(x = 1:2, y = new_tbl(list(), n = 2L))))
    Output
      <pillar_tbl_format_setup>
      <tbl_format_header(setup)>
      # A data frame: 2 x 2
      <tbl_format_body(setup)>
            x y        
        <int> <tbl[,0]>
      1     1          
      2     2          
      <tbl_format_footer(setup)>

# survival output

    Code
      pillar(x, width = 20)
    Output
      <pillar>
      <Surv>
        306 
        455 
       1010+
        210 
        883 
       1022+

---

    Code
      pillar(x, width = 20)
    Output
      <pillar>
      <Surv2>
        306:2
        455:2
       1010+ 
        210:2
        883:2
       1022+ 

# integer output will use scientific if necessary

    Code
      pillar(add_special(x), width = 6)
    Output
      NULL
    Code
      pillar(add_special(x), width = 7)
    Output
      <pillar>
        <int>
       1.00e7
       1.00e7
       1.00e7
      NA     
    Code
      pillar(add_special(x), width = 8)
    Output
      <pillar>
         <int>
      10000001
      10000002
      10000003
            NA
    Code
      pillar(add_special(x), width = 9)
    Output
      <pillar>
         <int>
      10000001
      10000002
      10000003
            NA

# tbl_format_header() results

    Code
      tbl_format_header(tbl_format_setup(as_tbl(mtcars), width = 80))
    Output
      <tbl_format_header(setup)>
      # A data frame: 32 x 11
    Code
      tbl_format_header(tbl_format_setup(as_unknown_rows(trees), width = 30, n = 10))
    Output
      <tbl_format_header(setup)>
      # A data frame: ?? x 3
    Code
      # Narrow
      tbl_format_header(tbl_format_setup(as_tbl(mtcars), width = 10))
    Output
      <tbl_format_header(setup)>
      # A data
      #   frame:
      #   32 x
      #   11
    Code
      # Custom tbl_sum()
      tbl_format_header(tbl_format_setup(new_foo_tbl(), width = 30))
    Output
      <tbl_format_header(setup)>
      # A data frame: 0 x 0
      # foo:          bar
    Code
      # Focus columns
      tbl_format_header(tbl_format_setup(new_tbl(trees, pillar_focus = "Volume"),
      width = 30))
    Output
      <tbl_format_header(setup)>
      # A data frame:  31 x 3
      # Focus columns: Volume

# integer64 output will use scientific if necessary

    Code
      pillar(add_special(x), width = 6)
    Output
      <pillar>
      <int6>
        0   
        1e11
        1e12
       NA   
    Code
      pillar(add_special(x), width = 7)
    Output
      <pillar>
      <int64>
         0   
         1e11
         1e12
        NA   
    Code
      pillar(add_special(x), width = 8)
    Output
      <pillar>
       <int64>
          0   
          1e11
          1e12
         NA   
    Code
      pillar(add_special(x), width = 9)
    Output
      <pillar>
        <int64>
           0   
           1e11
           1e12
          NA   
    Code
      pillar(add_special(x), width = 10)
    Output
      <pillar>
         <int64>
            0   
            1e11
            1e12
           NA   
    Code
      pillar(add_special(x), width = 11)
    Output
      <pillar>
          <int64>
             0   
             1e11
             1e12
            NA   
    Code
      pillar(add_special(x), width = 12)
    Output
      <pillar>
           <int64>
              0   
              1e11
              1e12
             NA   
    Code
      pillar(add_special(x), width = 13)
    Output
      <pillar>
            <int64>
                  0
       100000000001
      1000000000002
                 NA
    Code
      pillar(add_special(x), width = 14)
    Output
      <pillar>
            <int64>
                  0
       100000000001
      1000000000002
                 NA

# integer64 output uses underlines correctly

    Code
      pillar(bit64::as.integer64(c(1, 10000)))
    Output
      <pillar>
      [3m[90m<int64>[39m[23m
            1
        [4m1[24m[4m0[24m000

# nanotime works (#378)

    Code
      pillar(x, width = 16)
    Output
      <pillar>
      <nanotime>      
      2011-12-05T08:3~
    Code
      pillar(x, width = 17)
    Output
      <pillar>
      <nanotime>       
      2011-12-05T08:30~
    Code
      pillar(x, width = 18)
    Output
      <pillar>
      <nanotime>        
      2011-12-05T08:30:~
    Code
      pillar(x, width = 19)
    Output
      <pillar>
      <nanotime>         
      2011-12-05T08:30:0~
    Code
      pillar(x, width = 20)
    Output
      <pillar>
      <nanotime>          
      2011-12-05T08:30:00~
    Code
      pillar(x, width = 21)
    Output
      <pillar>
      <nanotime>           
      2011-12-05T08:30:00+~
    Code
      pillar(x, width = 22)
    Output
      <pillar>
      <nanotime>            
      2011-12-05T08:30:00+0~
    Code
      pillar(x, width = 23)
    Output
      <pillar>
      <nanotime>             
      2011-12-05T08:30:00+00~
    Code
      pillar(x, width = 24)
    Output
      <pillar>
      <nanotime>              
      2011-12-05T08:30:00+00:~
    Code
      pillar(x, width = 25)
    Output
      <pillar>
      <nanotime>               
      2011-12-05T08:30:00+00:00
    Code
      pillar(x, width = 26)
    Output
      <pillar>
      <nanotime>               
      2011-12-05T08:30:00+00:00

# strings with varying widths

    Code
      options(width = 59)
      colonnade(df_str[c(12L, 33L, 36L, 7L, 41L, 3L, 18L, 23L, 13L, 44L, 14L, 16L,
        25L, 21L, 19L, 45L, 43L, 29L, 1L, 30L, 22L, 27L, 15L, 47L, 28L, 31L, 10L, 50L,
        4L, 40L, 42L, 8L, 6L, 9L, 24L, 48L, 38L, 37L, 34L, 49L, 46L, 2L, 32L, 35L,
        39L, 11L, 17L, 5L, 26L, 20L)], width = 1382)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        `12`         `33`                             
        <chr>        <chr>                            
      1 AbcdefghijAb AbcdefghijAbcdefghijAbcdefghijAbc
        `36`                                 `7`    
        <chr>                                <chr>  
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef Abcdefg
        `41`                                      `3`  
        <chr>                                     <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijA Abc  
        `18`               `23`                    `13`         
        <chr>              <chr>                   <chr>        
      1 AbcdefghijAbcdefgh AbcdefghijAbcdefghijAbc AbcdefghijAbc
        `44`                                        
        <chr>                                       
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcd
        `14`           `16`             `25`                     
        <chr>          <chr>            <chr>                    
      1 AbcdefghijAbcd AbcdefghijAbcdef AbcdefghijAbcdefghijAbcde
        `21`                  `19`               
        <chr>                 <chr>              
      1 AbcdefghijAbcdefghijA AbcdefghijAbcdefghi
        `45`                                         
        <chr>                                        
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcde
        `43`                                       
        <chr>                                      
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbc
        `29`                          `1`  
        <chr>                         <chr>
      1 AbcdefghijAbcdefghijAbcdefghi A    
        `30`                           `22`                  
        <chr>                          <chr>                 
      1 AbcdefghijAbcdefghijAbcdefghij AbcdefghijAbcdefghijAb
        `27`                        `15`           
        <chr>                       <chr>          
      1 AbcdefghijAbcdefghijAbcdefg AbcdefghijAbcde
        `47`                                           
        <chr>                                          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefg
        `28`                        
        <chr>                       
      1 AbcdefghijAbcdefghijAbcdefgh
        `31`                            `10`      
        <chr>                           <chr>     
      1 AbcdefghijAbcdefghijAbcdefghijA Abcdefghij
        `50`                                               `4`  
        <chr>                                              <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefghij Abcd 
        `40`                                    
        <chr>                                   
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghij
        `42`                                       `8`     
        <chr>                                      <chr>   
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAb Abcdefgh
        `6`    `9`       `24`                    
        <chr>  <chr>     <chr>                   
      1 Abcdef Abcdefghi AbcdefghijAbcdefghijAbcd
        `48`                                            
        <chr>                                           
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `38`                                  
        <chr>                                 
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `37`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefg
        `34`                               `49`        `46`      
        <chr>                              <chr>       <chr>     
      1 AbcdefghijAbcdefghijAbcdefghijAbcd Abcdefghij~ Abcdefghi~
        `2`   `32`   `35`   `39`   `11`   `17`  `5`   `26`  `20` 
        <chr> <chr>  <chr>  <chr>  <chr>  <chr> <chr> <chr> <chr>
      1 Ab    Abcde~ Abcde~ Abcde~ Abcde~ Abcd~ Abcde Abcd~ Abcd~
    Code
      options(width = 54)
      colonnade(df_str[c(40L, 28L, 7L, 16L, 48L, 6L, 21L, 1L, 20L, 17L, 47L, 45L, 29L,
        41L, 49L, 34L, 4L, 39L, 18L, 36L, 26L, 38L, 10L, 8L, 5L, 15L, 44L, 24L, 46L,
        14L, 25L, 27L, 3L, 37L, 35L, 12L, 9L, 13L, 22L, 33L, 42L, 11L, 19L, 50L, 23L,
        30L, 32L, 2L, 43L, 31L)], width = 837)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        `40`                                    
        <chr>                                   
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghij
        `28`                         `7`    
        <chr>                        <chr>  
      1 AbcdefghijAbcdefghijAbcdefgh Abcdefg
        `16`            
        <chr>           
      1 AbcdefghijAbcdef
        `48`                                            
        <chr>                                           
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `6`    `21`                  `1`  
        <chr>  <chr>                 <chr>
      1 Abcdef AbcdefghijAbcdefghijA A    
        `20`                 `17`             
        <chr>                <chr>            
      1 AbcdefghijAbcdefghij AbcdefghijAbcdefg
        `47`                                           
        <chr>                                          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefg
        `45`                                         
        <chr>                                        
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcde
        `29`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefghi
        `41`                                     
        <chr>                                    
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijA
        `49`                                             
        <chr>                                            
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefghi
        `34`                               `4`   `39`       
        <chr>                              <chr> <chr>      
      1 AbcdefghijAbcdefghijAbcdefghijAbcd Abcd  Abcdefghij~
        `18`   `36`    `26`   `38`   `10`  `8`   `5`   `15` 
        <chr>  <chr>   <chr>  <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcdef~ Abcde~ Abcde~ Abcd~ Abcd~ Abcde Abcd~
        `44`    `24`   `46`    `14`  `25`  `27`  `3`   `37` 
        <chr>   <chr>  <chr>   <chr> <chr> <chr> <chr> <chr>
      1 Abcdef~ Abcde~ Abcdef~ Abcd~ Abcd~ Abcd~ Abc   Abcd~
        `35`    `12`   `9`   `13`  `22`  `33`   `42`   `11` 
        <chr>   <chr>  <chr> <chr> <chr> <chr>  <chr>  <chr>
      1 Abcdef~ Abcde~ Abcd~ Abcd~ Abcd~ Abcde~ Abcde~ Abcd~
        `19`   `50`    `23`   `30`   `32`  `2`   `43`  `31` 
        <chr>  <chr>   <chr>  <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcdef~ Abcde~ Abcde~ Abcd~ Ab    Abcd~ Abcd~
    Code
      options(width = 32)
      colonnade(df_str[c(47L, 42L, 4L, 46L, 9L, 34L, 19L, 39L, 8L, 32L, 36L, 12L, 29L,
        5L, 15L, 11L, 31L, 27L, 33L, 28L, 43L, 6L, 13L, 22L, 14L, 16L, 35L, 50L, 38L,
        7L, 23L, 45L, 40L, 3L, 2L, 24L, 41L, 10L, 30L, 25L, 17L, 26L, 48L, 37L, 49L,
        1L, 18L, 21L, 44L, 20L)], width = 455)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        `47`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `42`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `4`  
        <chr>
      1 Abcd 
        `46`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `9`      
        <chr>    
      1 Abcdefghi
        `34`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `19`    `39`     `8`   `32`   
        <chr>   <chr>    <chr> <chr>  
      1 Abcdef~ Abcdefg~ Abcd~ Abcdef~
        `36`   `12`  `29`  `5`   `15` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcde Abcd~
        `11`   `31`  `27`  `33`  `28` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `43`   `6`   `13`  `22`  `14` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `16`   `35`  `50`  `38`  `7`  
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `23`   `45`  `40`  `3`   `2`  
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abc   Ab   
        `24`   `41`  `10`  `30`  `25` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `17`   `26`  `48`  `37`  `49` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `1`   `18`   `21`  `44`  `20` 
        <chr> <chr>  <chr> <chr> <chr>
      1 A     Abcde~ Abcd~ Abcd~ Abcd~
    Code
      options(width = 55)
      colonnade(df_str[c(41L, 4L, 25L, 31L, 8L, 22L, 19L, 10L, 29L, 21L, 34L, 5L, 26L,
        36L, 47L, 46L, 2L, 24L, 27L, 39L, 28L, 43L, 32L, 30L, 48L, 44L, 6L, 20L, 13L,
        15L, 18L, 42L, 9L, 12L, 37L, 45L, 16L, 40L, 11L, 14L, 38L, 1L, 7L, 3L, 23L,
        35L, 50L, 17L, 49L, 33L)], width = 855)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        `41`                                      `4`  
        <chr>                                     <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijA Abcd 
        `25`                     
        <chr>                    
      1 AbcdefghijAbcdefghijAbcde
        `31`                            `8`     
        <chr>                           <chr>   
      1 AbcdefghijAbcdefghijAbcdefghijA Abcdefgh
        `22`                   `19`                `10`      
        <chr>                  <chr>               <chr>     
      1 AbcdefghijAbcdefghijAb AbcdefghijAbcdefghi Abcdefghij
        `29`                          `21`                 
        <chr>                         <chr>                
      1 AbcdefghijAbcdefghijAbcdefghi AbcdefghijAbcdefghijA
        `34`                               `5`  
        <chr>                              <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcd Abcde
        `26`                      
        <chr>                     
      1 AbcdefghijAbcdefghijAbcdef
        `36`                                
        <chr>                               
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef
        `47`                                           
        <chr>                                          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefg
        `46`                                           `2`  
        <chr>                                          <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdef Ab   
        `24`                     `27`                       
        <chr>                    <chr>                      
      1 AbcdefghijAbcdefghijAbcd AbcdefghijAbcdefghijAbcdefg
        `39`                                   
        <chr>                                  
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghi
        `28`                         `43`          `32`      
        <chr>                        <chr>         <chr>     
      1 AbcdefghijAbcdefghijAbcdefgh AbcdefghijAb~ Abcdefghi~
        `30`  `48`  `44`  `6`   `20`  `13`  `15`  `18`  `42` 
        <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `9`   `12`  `37`  `45`  `16`  `40`  `11`  `14`  `38` 
        <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `1`   `7`   `3`   `23`  `35`  `50`  `17`  `49`  `33` 
        <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 A     Abcd~ Abc   Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
    Code
      options(width = 54)
      colonnade(df_str[c(27L, 22L, 9L, 23L, 16L, 19L, 25L, 31L, 44L, 1L, 28L, 46L,
        12L, 20L, 43L, 37L, 5L, 2L, 18L, 41L, 26L, 33L, 11L, 49L, 24L, 35L, 4L, 47L,
        30L, 7L, 34L, 3L, 32L, 42L, 10L, 45L, 38L, 39L, 48L, 14L, 6L, 17L, 36L, 50L,
        40L, 13L, 8L, 21L, 15L, 29L)], width = 552)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        `27`                        `22`                  
        <chr>                       <chr>                 
      1 AbcdefghijAbcdefghijAbcdefg AbcdefghijAbcdefghijAb
        `9`       `23`                    `16`            
        <chr>     <chr>                   <chr>           
      1 Abcdefghi AbcdefghijAbcdefghijAbc AbcdefghijAbcdef
        `19`                `25`                     
        <chr>               <chr>                    
      1 AbcdefghijAbcdefghi AbcdefghijAbcdefghijAbcde
        `31`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghijA
        `44`                                         `1`  
        <chr>                                        <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcd A    
        `28`                        
        <chr>                       
      1 AbcdefghijAbcdefghijAbcdefgh
        `46`      `12`   `20`   `43`     `37`    `5`   `2`  
        <chr>     <chr>  <chr>  <chr>    <chr>   <chr> <chr>
      1 Abcdefgh~ Abcde~ Abcde~ Abcdefg~ Abcdef~ Abcde Ab   
        `18`   `41`   `26`   `33`   `11`  `49`   `24`  `35` 
        <chr>  <chr>  <chr>  <chr>  <chr> <chr>  <chr> <chr>
      1 Abcde~ Abcde~ Abcde~ Abcde~ Abcd~ Abcde~ Abcd~ Abcd~
        `4`   `47`    `30`   `7`    `34`  `3`   `32`  `42`  
        <chr> <chr>   <chr>  <chr>  <chr> <chr> <chr> <chr> 
      1 Abcd  Abcdef~ Abcde~ Abcde~ Abcd~ Abc   Abcd~ Abcde~
        `10`   `45`    `38`   `39`  `48`   `14`  `6`   `17` 
        <chr>  <chr>   <chr>  <chr> <chr>  <chr> <chr> <chr>
      1 Abcde~ Abcdef~ Abcde~ Abcd~ Abcde~ Abcd~ Abcd~ Abcd~
        `36`   `50`    `40`    `13`  `8`   `21`  `15`  `29` 
        <chr>  <chr>   <chr>   <chr> <chr> <chr> <chr> <chr>
      1 Abcde~ Abcdef~ Abcdef~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
    Code
      options(width = 49)
      colonnade(df_str[c(32L, 24L, 18L, 25L, 26L, 13L, 33L, 2L, 50L, 38L, 37L, 16L,
        27L, 9L, 28L, 49L, 34L, 15L, 17L, 35L, 22L, 47L, 3L, 21L, 23L, 41L, 5L, 1L,
        14L, 46L, 30L, 31L, 44L, 4L, 7L, 40L, 43L, 12L, 29L, 8L, 36L, 45L, 11L, 20L,
        10L, 6L, 19L, 48L, 39L, 42L)], width = 1031)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        `32`                            
        <chr>                           
      1 AbcdefghijAbcdefghijAbcdefghijAb
        `24`                     `18`              
        <chr>                    <chr>             
      1 AbcdefghijAbcdefghijAbcd AbcdefghijAbcdefgh
        `25`                     
        <chr>                    
      1 AbcdefghijAbcdefghijAbcde
        `26`                       `13`         
        <chr>                      <chr>        
      1 AbcdefghijAbcdefghijAbcdef AbcdefghijAbc
        `33`                              `2`  
        <chr>                             <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbc Ab   
        `50`                                           
        <chr>                                          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdef~
        `38`                                  
        <chr>                                 
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `37`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefg
        `16`             `27`                       
        <chr>            <chr>                      
      1 AbcdefghijAbcdef AbcdefghijAbcdefghijAbcdefg
        `9`       `28`                        
        <chr>     <chr>                       
      1 Abcdefghi AbcdefghijAbcdefghijAbcdefgh
        `49`                                           
        <chr>                                          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdef~
        `34`                              
        <chr>                             
      1 AbcdefghijAbcdefghijAbcdefghijAbcd
        `15`            `17`             
        <chr>           <chr>            
      1 AbcdefghijAbcde AbcdefghijAbcdefg
        `35`                               
        <chr>                              
      1 AbcdefghijAbcdefghijAbcdefghijAbcde
        `22`                  
        <chr>                 
      1 AbcdefghijAbcdefghijAb
        `47`                                           
        <chr>                                          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefg
        `3`   `21`                 
        <chr> <chr>                
      1 Abc   AbcdefghijAbcdefghijA
        `23`                   
        <chr>                  
      1 AbcdefghijAbcdefghijAbc
        `41`                                     
        <chr>                                    
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijA
        `5`   `1`   `14`  `46`  `30`  `31`  `44`  `4`  
        <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcde A     Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd 
        `7`   `40`  `43`  `12`  `29`  `8`   `36`  `45` 
        <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `11`  `20`  `10`  `6`   `19`  `48`  `39`  `42` 
        <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
    Code
      options(width = 38)
      colonnade(df_str[c(44L, 34L, 49L, 9L, 15L, 16L, 1L, 10L, 40L, 29L, 26L, 22L, 4L,
        43L, 20L, 17L, 46L, 33L, 35L, 32L, 2L, 12L, 8L, 37L, 23L, 39L, 7L, 18L, 36L,
        42L, 6L, 30L, 19L, 25L, 5L, 21L, 47L, 50L, 28L, 11L, 31L, 14L, 24L, 27L, 45L,
        41L, 38L, 3L, 13L, 48L)], width = 429)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        `44`                                
        <chr>                               
      1 AbcdefghijAbcdefghijAbcdefghijAbcde~
        `34`                              
        <chr>                             
      1 AbcdefghijAbcdefghijAbcdefghijAbcd
        `49`                                
        <chr>                               
      1 AbcdefghijAbcdefghijAbcdefghijAbcde~
        `9`       `15`           
        <chr>     <chr>          
      1 Abcdefghi AbcdefghijAbcde
        `16`             `1`   `10`      
        <chr>            <chr> <chr>     
      1 AbcdefghijAbcdef A     Abcdefghij
        `40`   `29`  `26`  `22`  `4`   `43` 
        <chr>  <chr> <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd  Abcd~
        `20`   `17`  `46`  `33`  `35`  `32` 
        <chr>  <chr> <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `2`   `12`   `8`   `37`  `23`  `39` 
        <chr> <chr>  <chr> <chr> <chr> <chr>
      1 Ab    Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `7`    `18`  `36`  `42`  `6`   `30` 
        <chr>  <chr> <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `19`   `25`  `5`   `21`  `47`  `50` 
        <chr>  <chr> <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcde Abcd~ Abcd~ Abcd~
        `28`   `11`  `31`  `14`  `24`  `27` 
        <chr>  <chr> <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `45`   `41`  `38`  `3`   `13`  `48` 
        <chr>  <chr> <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abc   Abcd~ Abcd~
    Code
      options(width = 54)
      colonnade(df_str[c(21L, 26L, 8L, 22L, 41L, 24L, 13L, 5L, 47L, 37L, 4L, 42L, 19L,
        34L, 11L, 43L, 38L, 3L, 33L, 20L, 31L, 2L, 18L, 48L, 27L, 44L, 9L, 35L, 30L,
        6L, 49L, 10L, 1L, 16L, 46L, 29L, 12L, 14L, 45L, 36L, 15L, 39L, 50L, 23L, 17L,
        28L, 7L, 32L, 40L, 25L)], width = 633)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        `21`                  `26`                      
        <chr>                 <chr>                     
      1 AbcdefghijAbcdefghijA AbcdefghijAbcdefghijAbcdef
        `8`      `22`                  
        <chr>    <chr>                 
      1 Abcdefgh AbcdefghijAbcdefghijAb
        `41`                                     
        <chr>                                    
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijA
        `24`                     `13`          `5`  
        <chr>                    <chr>         <chr>
      1 AbcdefghijAbcdefghijAbcd AbcdefghijAbc Abcde
        `47`                                           
        <chr>                                          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefg
        `37`                                  `4`  
        <chr>                                 <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefg Abcd 
        `42`                                      
        <chr>                                     
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAb
        `19`                `34`   `11`   `43`   `38`  `3`  
        <chr>               <chr>  <chr>  <chr>  <chr> <chr>
      1 AbcdefghijAbcdefghi Abcde~ Abcde~ Abcde~ Abcd~ Abc  
        `33`   `20`   `31`   `2`   `18`  `48`   `27`  `44`  
        <chr>  <chr>  <chr>  <chr> <chr> <chr>  <chr> <chr> 
      1 Abcde~ Abcde~ Abcde~ Ab    Abcd~ Abcde~ Abcd~ Abcde~
        `9`    `35`    `30`   `6`   `49`   `10`  `1`   `16` 
        <chr>  <chr>   <chr>  <chr> <chr>  <chr> <chr> <chr>
      1 Abcde~ Abcdef~ Abcde~ Abcd~ Abcde~ Abcd~ A     Abcd~
        `46`    `29`   `12`   `14`  `45`   `36`  `15`  `39` 
        <chr>   <chr>  <chr>  <chr> <chr>  <chr> <chr> <chr>
      1 Abcdef~ Abcde~ Abcde~ Abcd~ Abcde~ Abcd~ Abcd~ Abcd~
        `50`    `23`   `17`   `28`   `7`   `32`  `40`  `25` 
        <chr>   <chr>  <chr>  <chr>  <chr> <chr> <chr> <chr>
      1 Abcdef~ Abcde~ Abcde~ Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
    Code
      options(width = 39)
      colonnade(df_str[c(23L, 49L, 13L, 37L, 3L, 25L, 36L, 4L, 9L, 7L, 27L, 48L, 12L,
        10L, 50L, 14L, 38L, 39L, 46L, 22L, 28L, 8L, 21L, 44L, 32L, 40L, 31L, 1L, 29L,
        34L, 35L, 33L, 19L, 15L, 41L, 20L, 47L, 18L, 16L, 45L, 6L, 5L, 24L, 26L, 43L,
        11L, 42L, 30L, 17L, 2L)], width = 1496)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        `23`                   
        <chr>                  
      1 AbcdefghijAbcdefghijAbc
        `49`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `13`         
        <chr>        
      1 AbcdefghijAbc
        `37`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefg
        `3`   `25`                     
        <chr> <chr>                    
      1 Abc   AbcdefghijAbcdefghijAbcde
        `36`                                
        <chr>                               
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef
        `4`   `9`       `7`    
        <chr> <chr>     <chr>  
      1 Abcd  Abcdefghi Abcdefg
        `27`                       
        <chr>                      
      1 AbcdefghijAbcdefghijAbcdefg
        `48`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `12`         `10`      
        <chr>        <chr>     
      1 AbcdefghijAb Abcdefghij
        `50`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `14`          
        <chr>         
      1 AbcdefghijAbcd
        `38`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `39`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `46`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `22`                  
        <chr>                 
      1 AbcdefghijAbcdefghijAb
        `28`                         `8`     
        <chr>                        <chr>   
      1 AbcdefghijAbcdefghijAbcdefgh Abcdefgh
        `21`                 
        <chr>                
      1 AbcdefghijAbcdefghijA
        `44`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `32`                            
        <chr>                           
      1 AbcdefghijAbcdefghijAbcdefghijAb
        `40`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `31`                            `1`  
        <chr>                           <chr>
      1 AbcdefghijAbcdefghijAbcdefghijA A    
        `29`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefghi
        `34`                              
        <chr>                             
      1 AbcdefghijAbcdefghijAbcdefghijAbcd
        `35`                               
        <chr>                              
      1 AbcdefghijAbcdefghijAbcdefghijAbcde
        `33`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAbc
        `19`                `15`           
        <chr>               <chr>          
      1 AbcdefghijAbcdefghi AbcdefghijAbcde
        `41`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `20`                
        <chr>               
      1 AbcdefghijAbcdefghij
        `47`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `18`               `16`            
        <chr>              <chr>           
      1 AbcdefghijAbcdefgh AbcdefghijAbcdef
        `45`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `6`    `5`   `24`                    
        <chr>  <chr> <chr>                   
      1 Abcdef Abcde AbcdefghijAbcdefghijAbcd
        `26`                      
        <chr>                     
      1 AbcdefghijAbcdefghijAbcdef
        `43`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `11`       
        <chr>      
      1 AbcdefghijA
        `42`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `30`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghij
        `17`              `2`  
        <chr>             <chr>
      1 AbcdefghijAbcdefg Ab   
    Code
      options(width = 31)
      colonnade(df_str[c(45L, 14L, 49L, 24L, 22L, 31L, 42L, 18L, 16L, 47L, 25L, 4L,
        37L, 8L, 26L, 21L, 50L, 5L, 41L, 30L, 2L, 33L, 34L, 3L, 44L, 19L, 43L, 6L,
        32L, 29L, 20L, 1L, 13L, 11L, 40L, 12L, 48L, 23L, 9L, 15L, 46L, 36L, 27L, 35L,
        28L, 10L, 7L, 39L, 17L, 38L)], width = 493)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        `45`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `14`          
        <chr>         
      1 AbcdefghijAbcd
        `49`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `24`                    
        <chr>                   
      1 AbcdefghijAbcdefghijAbcd
        `22`                  
        <chr>                 
      1 AbcdefghijAbcdefghijAb
        `31`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `42`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `18`      `16`    `47`       
        <chr>     <chr>   <chr>      
      1 Abcdefgh~ Abcdef~ Abcdefghij~
        `25`  `4`   `37`  `8`   `26` 
        <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd  Abcd~ Abcd~ Abcd~
        `21`  `50`  `5`   `41`  `30` 
        <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abcde Abcd~ Abcd~
        `2`   `33`  `34`  `3`   `44` 
        <chr> <chr> <chr> <chr> <chr>
      1 Ab    Abcd~ Abcd~ Abc   Abcd~
        `19`  `43`  `6`   `32`  `29` 
        <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `20`  `1`   `13`  `11`  `40` 
        <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ A     Abcd~ Abcd~ Abcd~
        `12`  `48`  `23`  `9`   `15` 
        <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `46`  `36`  `27`  `35`  `28` 
        <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `10`  `7`   `39`  `17`  `38` 
        <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
    Code
      options(width = 52)
      colonnade(df_str[c(38L, 46L, 17L, 11L, 24L, 18L, 16L, 39L, 50L, 42L, 6L, 13L,
        37L, 29L, 41L, 47L, 9L, 33L, 44L, 31L, 45L, 36L, 28L, 5L, 10L, 30L, 20L, 1L,
        14L, 43L, 49L, 23L, 26L, 21L, 32L, 19L, 34L, 15L, 48L, 4L, 7L, 35L, 40L, 8L,
        22L, 3L, 25L, 12L, 27L, 2L)], width = 1130)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        `38`                                  
        <chr>                                 
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `46`                                          
        <chr>                                         
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdef
        `17`              `11`       
        <chr>             <chr>      
      1 AbcdefghijAbcdefg AbcdefghijA
        `24`                     `18`              
        <chr>                    <chr>             
      1 AbcdefghijAbcdefghijAbcd AbcdefghijAbcdefgh
        `16`            
        <chr>           
      1 AbcdefghijAbcdef
        `39`                                   
        <chr>                                  
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghi
        `50`                                              
        <chr>                                             
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefghij
        `42`                                       `6`   
        <chr>                                      <chr> 
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAb Abcdef
        `13`         
        <chr>        
      1 AbcdefghijAbc
        `37`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefg
        `29`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefghi
        `41`                                     
        <chr>                                    
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijA
        `47`                                           
        <chr>                                          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefg
        `9`       `33`                             
        <chr>     <chr>                            
      1 Abcdefghi AbcdefghijAbcdefghijAbcdefghijAbc
        `44`                                        
        <chr>                                       
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcd
        `31`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghijA
        `45`                                         
        <chr>                                        
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcde
        `36`                                
        <chr>                               
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef
        `28`                         `5`   `10`   `30`    
        <chr>                        <chr> <chr>  <chr>   
      1 AbcdefghijAbcdefghijAbcdefgh Abcde Abcde~ Abcdefg~
        `20`   `1`   `14`   `43`   `49`  `23`  `26`  `21` 
        <chr>  <chr> <chr>  <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ A     Abcde~ Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `32`   `19`   `34`   `15`  `48`  `4`   `7`   `35` 
        <chr>  <chr>  <chr>  <chr> <chr> <chr> <chr> <chr>
      1 Abcde~ Abcde~ Abcde~ Abcd~ Abcd~ Abcd  Abcd~ Abcd~
        `40`    `8`    `22`  `3`   `25`  `12`  `27`  `2`  
        <chr>   <chr>  <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdef~ Abcde~ Abcd~ Abc   Abcd~ Abcd~ Abcd~ Ab   
    Code
      options(width = 58)
      colonnade(df_str[c(17L, 28L, 29L, 27L, 20L, 31L, 43L, 30L, 32L, 48L, 10L, 50L,
        13L, 12L, 36L, 21L, 46L, 33L, 25L, 35L, 1L, 5L, 16L, 34L, 18L, 42L, 3L, 11L,
        40L, 26L, 37L, 7L, 39L, 6L, 4L, 19L, 8L, 45L, 14L, 24L, 23L, 2L, 47L, 9L, 49L,
        41L, 38L, 22L, 44L, 15L)], width = 1310)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        `17`              `28`                        
        <chr>             <chr>                       
      1 AbcdefghijAbcdefg AbcdefghijAbcdefghijAbcdefgh
        `29`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefghi
        `27`                        `20`                
        <chr>                       <chr>               
      1 AbcdefghijAbcdefghijAbcdefg AbcdefghijAbcdefghij
        `31`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghijA
        `43`                                       
        <chr>                                      
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbc
        `30`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghij
        `32`                            
        <chr>                           
      1 AbcdefghijAbcdefghijAbcdefghijAb
        `48`                                            
        <chr>                                           
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `10`      
        <chr>     
      1 Abcdefghij
        `50`                                              
        <chr>                                             
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefghij
        `13`          `12`        
        <chr>         <chr>       
      1 AbcdefghijAbc AbcdefghijAb
        `36`                                
        <chr>                               
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef
        `21`                 
        <chr>                
      1 AbcdefghijAbcdefghijA
        `46`                                          
        <chr>                                         
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdef
        `33`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAbc
        `25`                     
        <chr>                    
      1 AbcdefghijAbcdefghijAbcde
        `35`                                `1`   `5`  
        <chr>                               <chr> <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcde A     Abcde
        `16`             `34`                              
        <chr>            <chr>                             
      1 AbcdefghijAbcdef AbcdefghijAbcdefghijAbcdefghijAbcd
        `18`              
        <chr>             
      1 AbcdefghijAbcdefgh
        `42`                                       `3`  
        <chr>                                      <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAb Abc  
        `11`        `40`           `26`       `37`         `7`  
        <chr>       <chr>          <chr>      <chr>        <chr>
      1 AbcdefghijA AbcdefghijAbc~ Abcdefghi~ AbcdefghijA~ Abcd~
        `39`   `6`    `4`   `19`   `8`   `45`  `14`  `24`  `23` 
        <chr>  <chr>  <chr> <chr>  <chr> <chr> <chr> <chr> <chr>
      1 Abcde~ Abcdef Abcd  Abcde~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `2`   `47`   `9`    `49`   `41`  `38`  `22`  `44`  `15` 
        <chr> <chr>  <chr>  <chr>  <chr> <chr> <chr> <chr> <chr>
      1 Ab    Abcde~ Abcde~ Abcde~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
    Code
      options(width = 47)
      colonnade(df_str[c(1L, 26L, 20L, 12L, 49L, 16L, 24L, 4L, 15L, 47L, 8L, 11L, 14L,
        50L, 17L, 2L, 44L, 30L, 36L, 45L, 25L, 38L, 18L, 29L, 5L, 13L, 3L, 23L, 48L,
        40L, 34L, 22L, 39L, 33L, 27L, 7L, 19L, 10L, 37L, 6L, 35L, 46L, 31L, 41L, 43L,
        28L, 42L, 32L, 21L, 9L)], width = 484)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        `1`   `26`                      
        <chr> <chr>                     
      1 A     AbcdefghijAbcdefghijAbcdef
        `20`                 `12`        
        <chr>                <chr>       
      1 AbcdefghijAbcdefghij AbcdefghijAb
        `49`                                         
        <chr>                                        
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcd~
        `16`             `24`                    
        <chr>            <chr>                   
      1 AbcdefghijAbcdef AbcdefghijAbcdefghijAbcd
        `4`  
        <chr>
      1 Abcd 
        `15`   `47`    `8`   `11`  `14`  `50`   `17` 
        <chr>  <chr>   <chr> <chr> <chr> <chr>  <chr>
      1 Abcde~ Abcdef~ Abcd~ Abcd~ Abcd~ Abcde~ Abcd~
        `2`   `44`   `30`   `36`   `45`   `25`  `38` 
        <chr> <chr>  <chr>  <chr>  <chr>  <chr> <chr>
      1 Ab    Abcde~ Abcde~ Abcde~ Abcde~ Abcd~ Abcd~
        `18`   `29`   `5`   `13`   `3`   `23`  `48`  
        <chr>  <chr>  <chr> <chr>  <chr> <chr> <chr> 
      1 Abcde~ Abcde~ Abcde Abcde~ Abc   Abcd~ Abcde~
        `40`   `34`   `22`   `39`   `33`  `27`  `7`  
        <chr>  <chr>  <chr>  <chr>  <chr> <chr> <chr>
      1 Abcde~ Abcde~ Abcde~ Abcde~ Abcd~ Abcd~ Abcd~
        `19`   `10`   `37`   `6`   `35`  `46`   `31` 
        <chr>  <chr>  <chr>  <chr> <chr> <chr>  <chr>
      1 Abcde~ Abcde~ Abcde~ Abcd~ Abcd~ Abcde~ Abcd~
        `41`   `43`   `28`   `42`   `32`  `21`  `9`  
        <chr>  <chr>  <chr>  <chr>  <chr> <chr> <chr>
      1 Abcde~ Abcde~ Abcde~ Abcde~ Abcd~ Abcd~ Abcd~
    Code
      options(width = 55)
      colonnade(df_str[c(6L, 49L, 26L, 45L, 25L, 15L, 31L, 20L, 21L, 44L, 23L, 48L,
        37L, 36L, 5L, 43L, 11L, 14L, 13L, 39L, 16L, 12L, 4L, 18L, 42L, 3L, 10L, 28L,
        40L, 24L, 29L, 17L, 35L, 47L, 2L, 38L, 34L, 9L, 7L, 8L, 50L, 33L, 32L, 27L,
        46L, 19L, 22L, 41L, 30L, 1L)], width = 779)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        `6`   
        <chr> 
      1 Abcdef
        `49`                                             
        <chr>                                            
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefghi
        `26`                      
        <chr>                     
      1 AbcdefghijAbcdefghijAbcdef
        `45`                                         
        <chr>                                        
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcde
        `25`                      `15`           
        <chr>                     <chr>          
      1 AbcdefghijAbcdefghijAbcde AbcdefghijAbcde
        `31`                            `20`                
        <chr>                           <chr>               
      1 AbcdefghijAbcdefghijAbcdefghijA AbcdefghijAbcdefghij
        `21`                 
        <chr>                
      1 AbcdefghijAbcdefghijA
        `44`                                        
        <chr>                                       
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcd
        `23`                   
        <chr>                  
      1 AbcdefghijAbcdefghijAbc
        `48`                                            
        <chr>                                           
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `37`                                  `36`           
        <chr>                                 <chr>          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefg AbcdefghijAbcd~
        `5`   `43`  `11`  `14`  `13`  `39`  `16`  `12`  `4`  
        <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcde Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd 
        `18`  `42`  `3`   `10`  `28`  `40`  `24`  `29`  `17` 
        <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abc   Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `35`  `47`  `2`   `38`  `34`  `9`   `7`   `8`   `50` 
        <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Ab    Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `33`  `32`  `27`  `46`  `19`  `22`  `41`  `30`  `1`  
        <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ A    
    Code
      options(width = 46)
      colonnade(df_str[c(38L, 42L, 41L, 10L, 40L, 11L, 27L, 9L, 17L, 37L, 46L, 13L,
        36L, 18L, 31L, 20L, 39L, 12L, 44L, 33L, 50L, 34L, 26L, 32L, 23L, 30L, 29L,
        21L, 4L, 49L, 19L, 25L, 3L, 6L, 15L, 14L, 43L, 48L, 8L, 22L, 1L, 2L, 45L, 35L,
        16L, 5L, 47L, 28L, 24L, 7L)], width = 694)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        `38`                                  
        <chr>                                 
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `42`                                      
        <chr>                                     
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAb
        `41`                                     
        <chr>                                    
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijA
        `10`      
        <chr>     
      1 Abcdefghij
        `40`                                    
        <chr>                                   
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghij
        `11`        `27`                       
        <chr>       <chr>                      
      1 AbcdefghijA AbcdefghijAbcdefghijAbcdefg
        `9`       `17`             
        <chr>     <chr>            
      1 Abcdefghi AbcdefghijAbcdefg
        `37`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefg
        `46`                                        
        <chr>                                       
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbc~
        `13`         
        <chr>        
      1 AbcdefghijAbc
        `36`              `18`       `31`           
        <chr>             <chr>      <chr>          
      1 AbcdefghijAbcdef~ Abcdefghi~ AbcdefghijAbcd~
        `20`   `39`   `12`   `44`  `33`  `50`  `34` 
        <chr>  <chr>  <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcde~ Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `26`   `32`   `23`   `30`  `29`  `21`  `4`  
        <chr>  <chr>  <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcde~ Abcde~ Abcd~ Abcd~ Abcd~ Abcd 
        `49`    `19`   `25`  `3`   `6`   `15`  `14` 
        <chr>   <chr>  <chr> <chr> <chr> <chr> <chr>
      1 Abcdef~ Abcde~ Abcd~ Abc   Abcd~ Abcd~ Abcd~
        `43`   `48`   `8`    `22`  `1`   `2`   `45` 
        <chr>  <chr>  <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcde~ Abcde~ Abcd~ A     Ab    Abcd~
        `35`   `16`   `5`   `47`   `28`  `24`  `7`  
        <chr>  <chr>  <chr> <chr>  <chr> <chr> <chr>
      1 Abcde~ Abcde~ Abcde Abcde~ Abcd~ Abcd~ Abcd~

# ctl_new_pillar()

    Code
      ctl_new_pillar(new_tbl(), 1:3, width = 20, title = "a")
    Output
      <pillar>
          a
      <int>
          1
          2
          3

# ctl_new_pillar_list()

    Code
      ctl_new_pillar_list(new_tbl(), 1:3, width = 20, title = "a")
    Output
      [[1]]
      <pillar>
          a
      <int>
          1
          2
          3
      
      attr(,"remaining_width")
      [1] 14
      attr(,"simple")
      [1] TRUE
    Code
      ctl_new_pillar_list(new_tbl(), trees[1:3, ], width = 8, title = "a")
    Output
      [[1]]
      <pillar>
      a$Girth
        <dbl>
          8.3
          8.6
          8.8
      
      attr(,"extra")
      [1] "Height" "Volume"
      attr(,"remaining_width")
      [1] 0
      attr(,"simple")
      [1] FALSE
    Code
      ctl_new_pillar_list(new_tbl(), trees[1:3, ], width = 20, title = "a")
    Output
      [[1]]
      <pillar>
      a$Girth
        <dbl>
          8.3
          8.6
          8.8
      
      [[2]]
      <pillar>
      $Height
        <dbl>
           70
           65
           63
      
      attr(,"extra")
      [1] "Volume"
      attr(,"remaining_width")
      [1] 4
      attr(,"simple")
      [1] FALSE
    Code
      ctl_new_pillar_list(new_tbl(), as.matrix(trees[1:3, ]), width = 8, title = "a")
    Output
      [[1]]
      <pillar>
      a[,"Girth"]
            <dbl>
              8.3
              8.6
              8.8
      
      attr(,"extra")
      [1] 2 3
      attr(,"remaining_width")
      [1] 0
      attr(,"simple")
      [1] FALSE
    Code
      ctl_new_pillar_list(new_tbl(), as.matrix(trees[1:3, ]), width = 20, title = "a")
    Output
      [[1]]
      <pillar>
      a[,"Girth"]
            <dbl>
              8.3
              8.6
              8.8
      
      [[2]]
      <pillar>
      [,"Height"]
            <dbl>
               70
               65
               63
      
      attr(,"extra")
      [1] 3
      attr(,"remaining_width")
      [1] 4
      attr(,"simple")
      [1] FALSE
    Code
      ctl_new_pillar_list(new_tbl(), matrix(1:6, ncol = 2), width = 8, title = "a")
    Output
      [[1]]
      <pillar>
      a[,1]
      <int>
          1
          2
          3
      
      attr(,"extra")
      [1] 2
      attr(,"remaining_width")
      [1] 2
      attr(,"simple")
      [1] FALSE
    Code
      ctl_new_pillar_list(new_tbl(), matrix(1:6, ncol = 2), width = 20, title = "a")
    Output
      [[1]]
      <pillar>
      a[,1]
      <int>
          1
          2
          3
      
      [[2]]
      <pillar>
       [,2]
      <int>
          4
          5
          6
      
      attr(,"extra")
      integer(0)
      attr(,"remaining_width")
      [1] 8
      attr(,"simple")
      [1] FALSE
    Code
      ctl_new_pillar_list(new_tbl(), matrix(1:6, ncol = 3), width = 10, title = "a")
    Output
      [[1]]
      <pillar>
      a[,1]
      <int>
          1
          2
      
      attr(,"extra")
      [1] 2 3
      attr(,"remaining_width")
      [1] 4
      attr(,"simple")
      [1] FALSE

# ctl_new_pillar_list() for tables

    Code
      ctl_new_pillar_list(new_tbl(), Titanic, width = 20, title = "a")
    Output
      [[1]]
      <pillar>
                    a
      <table[,2,2,2]>
                0 ...
                0 ...
               35 ...
                0 ...
      
      attr(,"remaining_width")
      [1] 13
      attr(,"simple")
      [1] TRUE

# output test

    Code
      ctl_colonnade(rep(list(paste(letters, collapse = " ")), 4), width = Inf)
    Output
      $body
        ``                                                 
        <chr>                                              
      1 a b c d e f g h i j k l m n o p q r s t u v w x y z
        ``                                                 
        <chr>                                              
      1 a b c d e f g h i j k l m n o p q r s t u v w x y z
        ``                                                 
        <chr>                                              
      1 a b c d e f g h i j k l m n o p q r s t u v w x y z
        ``                                                 
        <chr>                                              
      1 a b c d e f g h i j k l m n o p q r s t u v w x y z
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      

# tests from tibble

    Code
      ctl_colonnade(mtcars[1:8, ], has_row_id = "*", width = 30)
    Output
      $body
          mpg   cyl  disp    hp
      * <dbl> <dbl> <dbl> <dbl>
      1  21       6  160    110
      2  21       6  160    110
      3  22.8     4  108     93
      4  21.4     6  258    110
      5  18.7     8  360    175
      6  18.1     6  225    105
      7  14.3     8  360    245
      8  24.4     4  147.    62
      
      $extra_cols
      $extra_cols$drat
      [1] 3.90 3.90 3.85 3.08 3.15 2.76 3.21 3.69
      
      $extra_cols$wt
      [1] 2.620 2.875 2.320 3.215 3.440 3.460 3.570 3.190
      
      $extra_cols$qsec
      [1] 16.46 17.02 18.61 19.44 17.02 20.22 15.84 20.00
      
      $extra_cols$vs
      [1] 0 0 1 1 0 1 0 1
      
      $extra_cols$am
      [1] 1 1 1 0 0 0 0 0
      
      $extra_cols$gear
      [1] 4 4 4 3 3 3 3 4
      
      $extra_cols$carb
      [1] 4 4 1 1 2 1 4 2
      
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(trees[1:5, ], width = 20)
    Output
      $body
        Girth Height
        <dbl>  <dbl>
      1   8.3     70
      2   8.6     65
      3   8.8     63
      4  10.5     72
      5  10.7     81
      
      $extra_cols
      $extra_cols$Volume
      [1] 10.3 10.3 10.2 16.4 18.8
      
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(trees[1:5, ], width = 10)
    Output
      $body
        Girth
        <dbl>
      1   8.3
      2   8.6
      3   8.8
      4  10.5
      5  10.7
      
      $extra_cols
      $extra_cols$Height
      [1] 70 65 63 72 81
      
      $extra_cols$Volume
      [1] 10.3 10.3 10.2 16.4 18.8
      
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(trees[1:3, ], width = 10)
    Output
      $body
        Girth
        <dbl>
      1   8.3
      2   8.6
      3   8.8
      
      $extra_cols
      $extra_cols$Height
      [1] 70 65 63
      
      $extra_cols$Volume
      [1] 10.3 10.3 10.2
      
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_all, width = 30)
    Output
      $body
            a     b c     d    
        <dbl> <int> <lgl> <chr>
      1   1       1 TRUE  a    
      2   2.5     2 FALSE b    
      3  NA      NA NA    <NA> 
      
      $extra_cols
      $extra_cols$e
      [1] a    b    <NA>
      Levels: a b
      
      $extra_cols$f
      [1] "2015-12-10" "2015-12-11" NA          
      
      $extra_cols$g
      [1] "2015-12-09 10:51:35 UTC" "2015-12-09 10:51:36 UTC"
      [3] NA                       
      
      $extra_cols$h
      $extra_cols$h[[1]]
      [1] 1
      
      $extra_cols$h[[2]]
      [1] 2
      
      $extra_cols$h[[3]]
      [1] NA
      
      
      $extra_cols$i
      $extra_cols$i[[1]]
      $extra_cols$i[[1]][[1]]
      [1] 1
      
      $extra_cols$i[[1]][[2]]
      [1] 2 3
      
      
      $extra_cols$i[[2]]
      $extra_cols$i[[2]][[1]]
      [1] 4 5 6
      
      
      $extra_cols$i[[3]]
      $extra_cols$i[[3]][[1]]
      [1] NA
      
      
      
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_all, width = 300)
    Output
      $body
            a     b c     d     e     f          g                   h        
        <dbl> <int> <lgl> <chr> <fct> <date>     <dttm>              <list>   
      1   1       1 TRUE  a     a     2015-12-10 2015-12-09 10:51:35 <int [1]>
      2   2.5     2 FALSE b     b     2015-12-11 2015-12-09 10:51:36 <int [1]>
      3  NA      NA NA    <NA>  <NA>  NA         NA                  <int [1]>
        i         
        <list>    
      1 <list [2]>
      2 <list [1]>
      3 <list [1]>
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      options(width = 70)
      ctl_colonnade(df_all, width = 300)
    Output
      $body
            a     b c     d     e     f          g                  
        <dbl> <int> <lgl> <chr> <fct> <date>     <dttm>             
      1   1       1 TRUE  a     a     2015-12-10 2015-12-09 10:51:35
      2   2.5     2 FALSE b     b     2015-12-11 2015-12-09 10:51:36
      3  NA      NA NA    <NA>  <NA>  NA         NA                 
        h         i         
        <list>    <list>    
      1 <int [1]> <list [2]>
      2 <int [1]> <list [1]>
      3 <int [1]> <list [1]>
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      options(width = 60)
      ctl_colonnade(df_all, width = 300)
    Output
      $body
            a     b c     d     e     f         
        <dbl> <int> <lgl> <chr> <fct> <date>    
      1   1       1 TRUE  a     a     2015-12-10
      2   2.5     2 FALSE b     b     2015-12-11
      3  NA      NA NA    <NA>  <NA>  NA        
        g                   h         i         
        <dttm>              <list>    <list>    
      1 2015-12-09 10:51:35 <int [1]> <list [2]>
      2 2015-12-09 10:51:36 <int [1]> <list [1]>
      3 NA                  <int [1]> <list [1]>
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      options(width = 50)
      ctl_colonnade(df_all, width = 300)
    Output
      $body
            a     b c     d     e     f         
        <dbl> <int> <lgl> <chr> <fct> <date>    
      1   1       1 TRUE  a     a     2015-12-10
      2   2.5     2 FALSE b     b     2015-12-11
      3  NA      NA NA    <NA>  <NA>  NA        
        g                   h         i         
        <dttm>              <list>    <list>    
      1 2015-12-09 10:51:35 <int [1]> <list [2]>
      2 2015-12-09 10:51:36 <int [1]> <list [1]>
      3 NA                  <int [1]> <list [1]>
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      options(width = 40)
      ctl_colonnade(df_all, width = 300)
    Output
      $body
            a     b c     d     e    
        <dbl> <int> <lgl> <chr> <fct>
      1   1       1 TRUE  a     a    
      2   2.5     2 FALSE b     b    
      3  NA      NA NA    <NA>  <NA> 
        f          g                  
        <date>     <dttm>             
      1 2015-12-10 2015-12-09 10:51:35
      2 2015-12-11 2015-12-09 10:51:36
      3 NA         NA                 
        h         i         
        <list>    <list>    
      1 <int [1]> <list [2]>
      2 <int [1]> <list [1]>
      3 <int [1]> <list [1]>
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      options(width = 30)
      ctl_colonnade(df_all, width = 300)
    Output
      $body
            a     b c     d    
        <dbl> <int> <lgl> <chr>
      1   1       1 TRUE  a    
      2   2.5     2 FALSE b    
      3  NA      NA NA    <NA> 
        e     f         
        <fct> <date>    
      1 a     2015-12-10
      2 b     2015-12-11
      3 <NA>  NA        
        g                  
        <dttm>             
      1 2015-12-09 10:51:35
      2 2015-12-09 10:51:36
      3 NA                 
        h         i         
        <list>    <list>    
      1 <int [1]> <list [2]>
      2 <int [1]> <list [1]>
      3 <int [1]> <list [1]>
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      options(width = 20)
      ctl_colonnade(df_all, width = 300)
    Output
      $body
            a     b c    
        <dbl> <int> <lgl>
      1   1       1 TRUE 
      2   2.5     2 FALSE
      3  NA      NA NA   
        d     e    
        <chr> <fct>
      1 a     a    
      2 b     b    
      3 <NA>  <NA> 
        f         
        <date>    
      1 2015-12-10
      2 2015-12-11
      3 NA        
      
      $extra_cols
      $extra_cols$g
      [1] "2015-12-09 10:51:35 UTC"
      [2] "2015-12-09 10:51:36 UTC"
      [3] NA                       
      
      $extra_cols$h
      $extra_cols$h[[1]]
      [1] 1
      
      $extra_cols$h[[2]]
      [1] 2
      
      $extra_cols$h[[3]]
      [1] NA
      
      
      $extra_cols$i
      $extra_cols$i[[1]]
      $extra_cols$i[[1]][[1]]
      [1] 1
      
      $extra_cols$i[[1]][[2]]
      [1] 2 3
      
      
      $extra_cols$i[[2]]
      $extra_cols$i[[2]][[1]]
      [1] 4 5 6
      
      
      $extra_cols$i[[3]]
      $extra_cols$i[[3]][[1]]
      [1] NA
      
      
      
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      list_with_ctl <- list(c("\n", "\""), factor(c("\n", "\n")))
      names(list_with_ctl) <- c("\n", "\r")
      ctl_colonnade(list_with_ctl, width = 30)
    Output
      $body
        `\n`  `\r` 
        <chr> <fct>
      1 "\n"  "\n" 
      2 "\""  "\n" 
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(list(a = c("", " ", "a ", " a")), width = 30)
    Output
      $body
        a    
        <chr>
      1 ""   
      2 " "  
      3 "a " 
      4 " a" 
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(list(`mean(x)` = 5, `var(x)` = 3), width = 30)
    Output
      $body
        `mean(x)` `var(x)`
            <dbl>    <dbl>
      1         5        3
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      

# NA names

    Code
      x <- list(`NA` = 1:3, set_to_NA = 4:6)
      names(x)[[2]] <- NA_character_
      ctl_colonnade(x, width = 30)
    Output
      $body
         `NA`    ``
        <int> <int>
      1     1     4
      2     2     5
      3     3     6
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      

# sep argument

    Code
      ctl_colonnade(x, width = 30)
    Output
      $body
          sep
        <int>
      1     1
      2     2
      3     3
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      # dummy

# tibble columns

    Code
      ctl_colonnade(x, width = 30)
    Output
      $body
            a   b$c    $d
        <int> <int> <int>
      1     1     4     7
      2     2     5     8
      3     3     6     9
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      

# tibble columns (nested)

    Code
      ctl_colonnade(x, width = 40)
    Output
      $body
            a   b$c    $d $e$f$g  $$$h
        <int> <int> <int>  <int> <int>
      1     1     4     7     10    13
      2     2     5     8     11    14
      3     3     6     9     12    15
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      

# tibble columns (empty)

    Code
      ctl_colonnade(x, width = 40)
    Output
      $body
            a   b$c    $d $e           c
        <int> <int> <int> <df[,0]> <int>
      1     1     4     7             10
      2     2     5     8             11
      3     3     6     9             12
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      

# matrix columns (unnamed)

    Code
      ctl_colonnade(x, width = 30)
    Output
      $body
            a b[,1]  [,2]
        <int> <int> <int>
      1     1     4     7
      2     2     5     8
      3     3     6     9
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      

# matrix columns (named)

    Code
      ctl_colonnade(x, width = 30)
    Output
      $body
            a b[,"c"] [,"d"]
        <int>   <int>  <int>
      1     1       4      7
      2     2       5      8
      3     3       6      9
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      

# matrix columns (empty)

    Code
      ctl_colonnade(list(a = 1:3, b = matrix(4:6, ncol = 1)[, 0], c = 4:6), width = 30)
    Output
      $body
            a b             c
        <int> <int[,0]> <int>
      1     1               4
      2     2               5
      3     3               6
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      

# filling unused width (#331)

    Code
      data
    Output
      # A data frame: 1 x 3
        month   sentences                                           blah              
        <chr>   <foo>                                               <chr>             
      1 January a b c d e f g h i j k l m n o p q r s t u v w x y z A B C D E F G H I~
    Code
      options(width = 60)
      print(data)
    Output
      # A data frame: 1 x 3
        month   sentences blah                                    
        <chr>   <foo>     <chr>                                   
      1 January a b c d~  A B C D E F G H I J K L M N O P Q R S T~

# works with glimpse

    Code
      glimpse(new_tbl(list(foo = foo, bar = bar)))
    Output
      Rows: 3
      Columns: 2
      $ foo AsIs 2011, 2012, 2013
      $ bar VeryLong 2011, 2012, 2013

# can override

    Code
      new_tbl(list(foo = foo, bar = bar))
    Output
      # A data frame: 3 x 2
        foo  bar     
        AsIs VeryLong
      1 2011 2011    
      2 2012 2012    
      3 2013 2013    
    Code
      options(width = 13)
      new_tbl(list(foo = foo, bar = bar))
    Output
      # A data
      #   frame: 3
      #   x 2
        foo  bar   
        AsIs VeryL~
      1 2011 2011  
      2 2012 2012  
      3 2013 2013  

# strings with varying widths

    Code
      ctl_colonnade(df_str[c(12L, 33L, 36L, 7L, 41L, 3L, 18L, 23L, 13L, 44L, 14L, 16L,
        25L, 21L, 19L, 45L, 43L, 29L, 1L, 30L, 22L, 27L, 15L, 47L, 28L, 31L, 10L, 50L,
        4L, 40L, 42L, 8L, 6L, 9L, 24L, 48L, 38L, 37L, 34L, 49L, 46L, 2L, 32L, 35L,
        39L, 11L, 17L, 5L, 26L, 20L)], width = {
        options(width = 59)
        1382
      })
    Output
      $body
        `12`         `33`                             
        <chr>        <chr>                            
      1 AbcdefghijAb AbcdefghijAbcdefghijAbcdefghijAbc
        `36`                                 `7`    
        <chr>                                <chr>  
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef Abcdefg
        `41`                                      `3`  
        <chr>                                     <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijA Abc  
        `18`               `23`                    `13`         
        <chr>              <chr>                   <chr>        
      1 AbcdefghijAbcdefgh AbcdefghijAbcdefghijAbc AbcdefghijAbc
        `44`                                        
        <chr>                                       
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcd
        `14`           `16`             `25`                     
        <chr>          <chr>            <chr>                    
      1 AbcdefghijAbcd AbcdefghijAbcdef AbcdefghijAbcdefghijAbcde
        `21`                  `19`               
        <chr>                 <chr>              
      1 AbcdefghijAbcdefghijA AbcdefghijAbcdefghi
        `45`                                         
        <chr>                                        
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcde
        `43`                                       
        <chr>                                      
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbc
        `29`                          `1`  
        <chr>                         <chr>
      1 AbcdefghijAbcdefghijAbcdefghi A    
        `30`                           `22`                  
        <chr>                          <chr>                 
      1 AbcdefghijAbcdefghijAbcdefghij AbcdefghijAbcdefghijAb
        `27`                        `15`           
        <chr>                       <chr>          
      1 AbcdefghijAbcdefghijAbcdefg AbcdefghijAbcde
        `47`                                           
        <chr>                                          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefg
        `28`                        
        <chr>                       
      1 AbcdefghijAbcdefghijAbcdefgh
        `31`                            `10`      
        <chr>                           <chr>     
      1 AbcdefghijAbcdefghijAbcdefghijA Abcdefghij
        `50`                                               `4`  
        <chr>                                              <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefghij Abcd 
        `40`                                    
        <chr>                                   
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghij
        `42`                                       `8`     
        <chr>                                      <chr>   
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAb Abcdefgh
        `6`    `9`       `24`                    
        <chr>  <chr>     <chr>                   
      1 Abcdef Abcdefghi AbcdefghijAbcdefghijAbcd
        `48`                                            
        <chr>                                           
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `38`                                  
        <chr>                                 
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `37`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefg
        `34`                               `49`             `46` 
        <chr>                              <chr>            <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcd AbcdefghijAbcde~ Abcd~
        `2`   `32`      `35`  `39`  `11`  `17`  `5`   `26`  `20` 
        <chr> <chr>     <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Ab    Abcdefgh~ Abcd~ Abcd~ Abcd~ Abcd~ Abcde Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(40L, 28L, 7L, 16L, 48L, 6L, 21L, 1L, 20L, 17L, 47L, 45L,
        29L, 41L, 49L, 34L, 4L, 39L, 18L, 36L, 26L, 38L, 10L, 8L, 5L, 15L, 44L, 24L,
        46L, 14L, 25L, 27L, 3L, 37L, 35L, 12L, 9L, 13L, 22L, 33L, 42L, 11L, 19L, 50L,
        23L, 30L, 32L, 2L, 43L, 31L)], width = {
        options(width = 54)
        837
      })
    Output
      $body
        `40`                                    
        <chr>                                   
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghij
        `28`                         `7`    
        <chr>                        <chr>  
      1 AbcdefghijAbcdefghijAbcdefgh Abcdefg
        `16`            
        <chr>           
      1 AbcdefghijAbcdef
        `48`                                            
        <chr>                                           
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `6`    `21`                  `1`  
        <chr>  <chr>                 <chr>
      1 Abcdef AbcdefghijAbcdefghijA A    
        `20`                 `17`             
        <chr>                <chr>            
      1 AbcdefghijAbcdefghij AbcdefghijAbcdefg
        `47`                                           
        <chr>                                          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefg
        `45`                                         
        <chr>                                        
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcde
        `29`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefghi
        `41`                                     
        <chr>                                    
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijA
        `49`                                             
        <chr>                                            
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefghi
        `34`                               `4`   `39`       
        <chr>                              <chr> <chr>      
      1 AbcdefghijAbcdefghijAbcdefghijAbcd Abcd  Abcdefghij~
        `18`       `36`  `26`  `38`  `10`  `8`   `5`   `15` 
        <chr>      <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefghi~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcde Abcd~
        `44`       `24`  `46`  `14`  `25`  `27`  `3`   `37` 
        <chr>      <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefghi~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abc   Abcd~
        `35`       `12`  `9`   `13`  `22`  `33`  `42`  `11` 
        <chr>      <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefghi~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `19`       `50`  `23`  `30`  `32`  `2`   `43`  `31` 
        <chr>      <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefghi~ Abcd~ Abcd~ Abcd~ Abcd~ Ab    Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(47L, 42L, 4L, 46L, 9L, 34L, 19L, 39L, 8L, 32L, 36L, 12L,
        29L, 5L, 15L, 11L, 31L, 27L, 33L, 28L, 43L, 6L, 13L, 22L, 14L, 16L, 35L, 50L,
        38L, 7L, 23L, 45L, 40L, 3L, 2L, 24L, 41L, 10L, 30L, 25L, 17L, 26L, 48L, 37L,
        49L, 1L, 18L, 21L, 44L, 20L)], width = {
        options(width = 32)
        455
      })
    Output
      $body
        `47`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `42`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `4`  
        <chr>
      1 Abcd 
        `46`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `9`      
        <chr>    
      1 Abcdefghi
        `34`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghi~
        `19`         `39`  `8`   `32` 
        <chr>        <chr> <chr> <chr>
      1 AbcdefghijA~ Abcd~ Abcd~ Abcd~
        `36`   `12`  `29`  `5`   `15` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcde Abcd~
        `11`   `31`  `27`  `33`  `28` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `43`   `6`   `13`  `22`  `14` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `16`   `35`  `50`  `38`  `7`  
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `23`   `45`  `40`  `3`   `2`  
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abc   Ab   
        `24`   `41`  `10`  `30`  `25` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `17`   `26`  `48`  `37`  `49` 
        <chr>  <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `1`   `18`   `21`  `44`  `20` 
        <chr> <chr>  <chr> <chr> <chr>
      1 A     Abcde~ Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(41L, 4L, 25L, 31L, 8L, 22L, 19L, 10L, 29L, 21L, 34L, 5L,
        26L, 36L, 47L, 46L, 2L, 24L, 27L, 39L, 28L, 43L, 32L, 30L, 48L, 44L, 6L, 20L,
        13L, 15L, 18L, 42L, 9L, 12L, 37L, 45L, 16L, 40L, 11L, 14L, 38L, 1L, 7L, 3L,
        23L, 35L, 50L, 17L, 49L, 33L)], width = {
        options(width = 55)
        855
      })
    Output
      $body
        `41`                                      `4`  
        <chr>                                     <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijA Abcd 
        `25`                     
        <chr>                    
      1 AbcdefghijAbcdefghijAbcde
        `31`                            `8`     
        <chr>                           <chr>   
      1 AbcdefghijAbcdefghijAbcdefghijA Abcdefgh
        `22`                   `19`                `10`      
        <chr>                  <chr>               <chr>     
      1 AbcdefghijAbcdefghijAb AbcdefghijAbcdefghi Abcdefghij
        `29`                          `21`                 
        <chr>                         <chr>                
      1 AbcdefghijAbcdefghijAbcdefghi AbcdefghijAbcdefghijA
        `34`                               `5`  
        <chr>                              <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcd Abcde
        `26`                      
        <chr>                     
      1 AbcdefghijAbcdefghijAbcdef
        `36`                                
        <chr>                               
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef
        `47`                                           
        <chr>                                          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefg
        `46`                                           `2`  
        <chr>                                          <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdef Ab   
        `24`                     `27`                       
        <chr>                    <chr>                      
      1 AbcdefghijAbcdefghijAbcd AbcdefghijAbcdefghijAbcdefg
        `39`                                   
        <chr>                                  
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghi
        `28`                         `43`               `32` 
        <chr>                        <chr>              <chr>
      1 AbcdefghijAbcdefghijAbcdefgh AbcdefghijAbcdefg~ Abcd~
        `30`  `48`  `44`  `6`   `20`  `13`  `15`  `18`  `42` 
        <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `9`   `12`  `37`  `45`  `16`  `40`  `11`  `14`  `38` 
        <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `1`   `7`   `3`   `23`  `35`  `50`  `17`  `49`  `33` 
        <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 A     Abcd~ Abc   Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(27L, 22L, 9L, 23L, 16L, 19L, 25L, 31L, 44L, 1L, 28L, 46L,
        12L, 20L, 43L, 37L, 5L, 2L, 18L, 41L, 26L, 33L, 11L, 49L, 24L, 35L, 4L, 47L,
        30L, 7L, 34L, 3L, 32L, 42L, 10L, 45L, 38L, 39L, 48L, 14L, 6L, 17L, 36L, 50L,
        40L, 13L, 8L, 21L, 15L, 29L)], width = {
        options(width = 54)
        552
      })
    Output
      $body
        `27`                        `22`                  
        <chr>                       <chr>                 
      1 AbcdefghijAbcdefghijAbcdefg AbcdefghijAbcdefghijAb
        `9`       `23`                    `16`            
        <chr>     <chr>                   <chr>           
      1 Abcdefghi AbcdefghijAbcdefghijAbc AbcdefghijAbcdef
        `19`                `25`                     
        <chr>               <chr>                    
      1 AbcdefghijAbcdefghi AbcdefghijAbcdefghijAbcde
        `31`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghijA
        `44`                                         `1`  
        <chr>                                        <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcd A    
        `28`                         `46`                   
        <chr>                        <chr>                  
      1 AbcdefghijAbcdefghijAbcdefgh AbcdefghijAbcdefghijAb~
        `12`         `20`            `43`  `37`  `5`   `2`  
        <chr>        <chr>           <chr> <chr> <chr> <chr>
      1 AbcdefghijAb AbcdefghijAbcd~ Abcd~ Abcd~ Abcde Ab   
        `18`       `41`  `26`  `33`  `11`  `49`  `24`  `35` 
        <chr>      <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefghi~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `4`   `47`       `30`  `7`   `34`  `3`   `32`  `42` 
        <chr> <chr>      <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcd  Abcdefghi~ Abcd~ Abcd~ Abcd~ Abc   Abcd~ Abcd~
        `10`       `45`  `38`  `39`  `48`  `14`  `6`   `17` 
        <chr>      <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefghij Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `36`       `50`  `40`  `13`  `8`   `21`  `15`  `29` 
        <chr>      <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefghi~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(32L, 24L, 18L, 25L, 26L, 13L, 33L, 2L, 50L, 38L, 37L,
        16L, 27L, 9L, 28L, 49L, 34L, 15L, 17L, 35L, 22L, 47L, 3L, 21L, 23L, 41L, 5L,
        1L, 14L, 46L, 30L, 31L, 44L, 4L, 7L, 40L, 43L, 12L, 29L, 8L, 36L, 45L, 11L,
        20L, 10L, 6L, 19L, 48L, 39L, 42L)], width = {
        options(width = 49)
        1031
      })
    Output
      $body
        `32`                            
        <chr>                           
      1 AbcdefghijAbcdefghijAbcdefghijAb
        `24`                     `18`              
        <chr>                    <chr>             
      1 AbcdefghijAbcdefghijAbcd AbcdefghijAbcdefgh
        `25`                     
        <chr>                    
      1 AbcdefghijAbcdefghijAbcde
        `26`                       `13`         
        <chr>                      <chr>        
      1 AbcdefghijAbcdefghijAbcdef AbcdefghijAbc
        `33`                              `2`  
        <chr>                             <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbc Ab   
        `50`                                           
        <chr>                                          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdef~
        `38`                                  
        <chr>                                 
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `37`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefg
        `16`             `27`                       
        <chr>            <chr>                      
      1 AbcdefghijAbcdef AbcdefghijAbcdefghijAbcdefg
        `9`       `28`                        
        <chr>     <chr>                       
      1 Abcdefghi AbcdefghijAbcdefghijAbcdefgh
        `49`                                           
        <chr>                                          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdef~
        `34`                              
        <chr>                             
      1 AbcdefghijAbcdefghijAbcdefghijAbcd
        `15`            `17`             
        <chr>           <chr>            
      1 AbcdefghijAbcde AbcdefghijAbcdefg
        `35`                               
        <chr>                              
      1 AbcdefghijAbcdefghijAbcdefghijAbcde
        `22`                  
        <chr>                 
      1 AbcdefghijAbcdefghijAb
        `47`                                           
        <chr>                                          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefg
        `3`   `21`                 
        <chr> <chr>                
      1 Abc   AbcdefghijAbcdefghijA
        `23`                   
        <chr>                  
      1 AbcdefghijAbcdefghijAbc
        `41`                                      `5`  
        <chr>                                     <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijA Abcde
        `1`   `14`        `46`  `30`  `31`  `44`  `4`  
        <chr> <chr>       <chr> <chr> <chr> <chr> <chr>
      1 A     Abcdefghij~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd 
        `7`   `40`  `43`  `12`  `29`  `8`   `36`  `45` 
        <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `11`  `20`  `10`  `6`   `19`  `48`  `39`  `42` 
        <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(44L, 34L, 49L, 9L, 15L, 16L, 1L, 10L, 40L, 29L, 26L, 22L,
        4L, 43L, 20L, 17L, 46L, 33L, 35L, 32L, 2L, 12L, 8L, 37L, 23L, 39L, 7L, 18L,
        36L, 42L, 6L, 30L, 19L, 25L, 5L, 21L, 47L, 50L, 28L, 11L, 31L, 14L, 24L, 27L,
        45L, 41L, 38L, 3L, 13L, 48L)], width = {
        options(width = 38)
        429
      })
    Output
      $body
        `44`                                
        <chr>                               
      1 AbcdefghijAbcdefghijAbcdefghijAbcde~
        `34`                              
        <chr>                             
      1 AbcdefghijAbcdefghijAbcdefghijAbcd
        `49`                                
        <chr>                               
      1 AbcdefghijAbcdefghijAbcdefghijAbcde~
        `9`       `15`           
        <chr>     <chr>          
      1 Abcdefghi AbcdefghijAbcde
        `16`             `1`   `10`      
        <chr>            <chr> <chr>     
      1 AbcdefghijAbcdef A     Abcdefghij
        `40`   `29`  `26`  `22`  `4`   `43` 
        <chr>  <chr> <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd  Abcd~
        `20`   `17`  `46`  `33`  `35`  `32` 
        <chr>  <chr> <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `2`   `12`   `8`   `37`  `23`  `39` 
        <chr> <chr>  <chr> <chr> <chr> <chr>
      1 Ab    Abcde~ Abcd~ Abcd~ Abcd~ Abcd~
        `7`    `18`  `36`  `42`  `6`   `30` 
        <chr>  <chr> <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `19`   `25`  `5`   `21`  `47`  `50` 
        <chr>  <chr> <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcde Abcd~ Abcd~ Abcd~
        `28`   `11`  `31`  `14`  `24`  `27` 
        <chr>  <chr> <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `45`   `41`  `38`  `3`   `13`  `48` 
        <chr>  <chr> <chr> <chr> <chr> <chr>
      1 Abcde~ Abcd~ Abcd~ Abc   Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(21L, 26L, 8L, 22L, 41L, 24L, 13L, 5L, 47L, 37L, 4L, 42L,
        19L, 34L, 11L, 43L, 38L, 3L, 33L, 20L, 31L, 2L, 18L, 48L, 27L, 44L, 9L, 35L,
        30L, 6L, 49L, 10L, 1L, 16L, 46L, 29L, 12L, 14L, 45L, 36L, 15L, 39L, 50L, 23L,
        17L, 28L, 7L, 32L, 40L, 25L)], width = {
        options(width = 54)
        633
      })
    Output
      $body
        `21`                  `26`                      
        <chr>                 <chr>                     
      1 AbcdefghijAbcdefghijA AbcdefghijAbcdefghijAbcdef
        `8`      `22`                  
        <chr>    <chr>                 
      1 Abcdefgh AbcdefghijAbcdefghijAb
        `41`                                     
        <chr>                                    
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijA
        `24`                     `13`          `5`  
        <chr>                    <chr>         <chr>
      1 AbcdefghijAbcdefghijAbcd AbcdefghijAbc Abcde
        `47`                                           
        <chr>                                          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefg
        `37`                                  `4`  
        <chr>                                 <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefg Abcd 
        `42`                                      
        <chr>                                     
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAb
        `19`                `34`     `11`  `43`  `38`  `3`  
        <chr>               <chr>    <chr> <chr> <chr> <chr>
      1 AbcdefghijAbcdefghi Abcdefg~ Abcd~ Abcd~ Abcd~ Abc  
        `33`       `20`  `31`  `2`   `18`  `48`  `27`  `44` 
        <chr>      <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefghi~ Abcd~ Abcd~ Ab    Abcd~ Abcd~ Abcd~ Abcd~
        `9`       `35`   `30`  `6`   `49`  `10`  `1`   `16` 
        <chr>     <chr>  <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefghi Abcde~ Abcd~ Abcd~ Abcd~ Abcd~ A     Abcd~
        `46`       `29`  `12`  `14`  `45`  `36`  `15`  `39` 
        <chr>      <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefghi~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `50`       `23`  `17`  `28`  `7`   `32`  `40`  `25` 
        <chr>      <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefghi~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(23L, 49L, 13L, 37L, 3L, 25L, 36L, 4L, 9L, 7L, 27L, 48L,
        12L, 10L, 50L, 14L, 38L, 39L, 46L, 22L, 28L, 8L, 21L, 44L, 32L, 40L, 31L, 1L,
        29L, 34L, 35L, 33L, 19L, 15L, 41L, 20L, 47L, 18L, 16L, 45L, 6L, 5L, 24L, 26L,
        43L, 11L, 42L, 30L, 17L, 2L)], width = {
        options(width = 39)
        1496
      })
    Output
      $body
        `23`                   
        <chr>                  
      1 AbcdefghijAbcdefghijAbc
        `49`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `13`         
        <chr>        
      1 AbcdefghijAbc
        `37`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefg
        `3`   `25`                     
        <chr> <chr>                    
      1 Abc   AbcdefghijAbcdefghijAbcde
        `36`                                
        <chr>                               
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef
        `4`   `9`       `7`    
        <chr> <chr>     <chr>  
      1 Abcd  Abcdefghi Abcdefg
        `27`                       
        <chr>                      
      1 AbcdefghijAbcdefghijAbcdefg
        `48`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `12`         `10`      
        <chr>        <chr>     
      1 AbcdefghijAb Abcdefghij
        `50`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `14`          
        <chr>         
      1 AbcdefghijAbcd
        `38`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `39`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `46`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `22`                  
        <chr>                 
      1 AbcdefghijAbcdefghijAb
        `28`                         `8`     
        <chr>                        <chr>   
      1 AbcdefghijAbcdefghijAbcdefgh Abcdefgh
        `21`                 
        <chr>                
      1 AbcdefghijAbcdefghijA
        `44`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `32`                            
        <chr>                           
      1 AbcdefghijAbcdefghijAbcdefghijAb
        `40`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `31`                            `1`  
        <chr>                           <chr>
      1 AbcdefghijAbcdefghijAbcdefghijA A    
        `29`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefghi
        `34`                              
        <chr>                             
      1 AbcdefghijAbcdefghijAbcdefghijAbcd
        `35`                               
        <chr>                              
      1 AbcdefghijAbcdefghijAbcdefghijAbcde
        `33`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAbc
        `19`                `15`           
        <chr>               <chr>          
      1 AbcdefghijAbcdefghi AbcdefghijAbcde
        `41`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `20`                
        <chr>               
      1 AbcdefghijAbcdefghij
        `47`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `18`               `16`            
        <chr>              <chr>           
      1 AbcdefghijAbcdefgh AbcdefghijAbcdef
        `45`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `6`    `5`   `24`                    
        <chr>  <chr> <chr>                   
      1 Abcdef Abcde AbcdefghijAbcdefghijAbcd
        `26`                      
        <chr>                     
      1 AbcdefghijAbcdefghijAbcdef
        `43`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `11`       
        <chr>      
      1 AbcdefghijA
        `42`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef~
        `30`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghij
        `17`              `2`  
        <chr>             <chr>
      1 AbcdefghijAbcdefg Ab   
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(45L, 14L, 49L, 24L, 22L, 31L, 42L, 18L, 16L, 47L, 25L,
        4L, 37L, 8L, 26L, 21L, 50L, 5L, 41L, 30L, 2L, 33L, 34L, 3L, 44L, 19L, 43L, 6L,
        32L, 29L, 20L, 1L, 13L, 11L, 40L, 12L, 48L, 23L, 9L, 15L, 46L, 36L, 27L, 35L,
        28L, 10L, 7L, 39L, 17L, 38L)], width = {
        options(width = 31)
        493
      })
    Output
      $body
        `45`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `14`          
        <chr>         
      1 AbcdefghijAbcd
        `49`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `24`                    
        <chr>                   
      1 AbcdefghijAbcdefghijAbcd
        `22`                  
        <chr>                 
      1 AbcdefghijAbcdefghijAb
        `31`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `42`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefgh~
        `18`              `16`  `47` 
        <chr>             <chr> <chr>
      1 AbcdefghijAbcdef~ Abcd~ Abcd~
        `25`  `4`   `37`  `8`   `26` 
        <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd  Abcd~ Abcd~ Abcd~
        `21`  `50`  `5`   `41`  `30` 
        <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abcde Abcd~ Abcd~
        `2`   `33`  `34`  `3`   `44` 
        <chr> <chr> <chr> <chr> <chr>
      1 Ab    Abcd~ Abcd~ Abc   Abcd~
        `19`  `43`  `6`   `32`  `29` 
        <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `20`  `1`   `13`  `11`  `40` 
        <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ A     Abcd~ Abcd~ Abcd~
        `12`  `48`  `23`  `9`   `15` 
        <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `46`  `36`  `27`  `35`  `28` 
        <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `10`  `7`   `39`  `17`  `38` 
        <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(38L, 46L, 17L, 11L, 24L, 18L, 16L, 39L, 50L, 42L, 6L,
        13L, 37L, 29L, 41L, 47L, 9L, 33L, 44L, 31L, 45L, 36L, 28L, 5L, 10L, 30L, 20L,
        1L, 14L, 43L, 49L, 23L, 26L, 21L, 32L, 19L, 34L, 15L, 48L, 4L, 7L, 35L, 40L,
        8L, 22L, 3L, 25L, 12L, 27L, 2L)], width = {
        options(width = 52)
        1130
      })
    Output
      $body
        `38`                                  
        <chr>                                 
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `46`                                          
        <chr>                                         
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdef
        `17`              `11`       
        <chr>             <chr>      
      1 AbcdefghijAbcdefg AbcdefghijA
        `24`                     `18`              
        <chr>                    <chr>             
      1 AbcdefghijAbcdefghijAbcd AbcdefghijAbcdefgh
        `16`            
        <chr>           
      1 AbcdefghijAbcdef
        `39`                                   
        <chr>                                  
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghi
        `50`                                              
        <chr>                                             
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefghij
        `42`                                       `6`   
        <chr>                                      <chr> 
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAb Abcdef
        `13`         
        <chr>        
      1 AbcdefghijAbc
        `37`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefg
        `29`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefghi
        `41`                                     
        <chr>                                    
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijA
        `47`                                           
        <chr>                                          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefg
        `9`       `33`                             
        <chr>     <chr>                            
      1 Abcdefghi AbcdefghijAbcdefghijAbcdefghijAbc
        `44`                                        
        <chr>                                       
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcd
        `31`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghijA
        `45`                                         
        <chr>                                        
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcde
        `36`                                
        <chr>                               
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef
        `28`                         `5`   `10`      `30` 
        <chr>                        <chr> <chr>     <chr>
      1 AbcdefghijAbcdefghijAbcdefgh Abcde Abcdefgh~ Abcd~
        `20`     `1`   `14`  `43`  `49`  `23`  `26`  `21` 
        <chr>    <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefg~ A     Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `32`     `19`  `34`  `15`  `48`  `4`   `7`   `35` 
        <chr>    <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefg~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd  Abcd~ Abcd~
        `40`     `8`   `22`  `3`   `25`  `12`  `27`  `2`  
        <chr>    <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefg~ Abcd~ Abcd~ Abc   Abcd~ Abcd~ Abcd~ Ab   
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(17L, 28L, 29L, 27L, 20L, 31L, 43L, 30L, 32L, 48L, 10L,
        50L, 13L, 12L, 36L, 21L, 46L, 33L, 25L, 35L, 1L, 5L, 16L, 34L, 18L, 42L, 3L,
        11L, 40L, 26L, 37L, 7L, 39L, 6L, 4L, 19L, 8L, 45L, 14L, 24L, 23L, 2L, 47L, 9L,
        49L, 41L, 38L, 22L, 44L, 15L)], width = {
        options(width = 58)
        1310
      })
    Output
      $body
        `17`              `28`                        
        <chr>             <chr>                       
      1 AbcdefghijAbcdefg AbcdefghijAbcdefghijAbcdefgh
        `29`                         
        <chr>                        
      1 AbcdefghijAbcdefghijAbcdefghi
        `27`                        `20`                
        <chr>                       <chr>               
      1 AbcdefghijAbcdefghijAbcdefg AbcdefghijAbcdefghij
        `31`                           
        <chr>                          
      1 AbcdefghijAbcdefghijAbcdefghijA
        `43`                                       
        <chr>                                      
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbc
        `30`                          
        <chr>                         
      1 AbcdefghijAbcdefghijAbcdefghij
        `32`                            
        <chr>                           
      1 AbcdefghijAbcdefghijAbcdefghijAb
        `48`                                            
        <chr>                                           
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `10`      
        <chr>     
      1 Abcdefghij
        `50`                                              
        <chr>                                             
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefghij
        `13`          `12`        
        <chr>         <chr>       
      1 AbcdefghijAbc AbcdefghijAb
        `36`                                
        <chr>                               
      1 AbcdefghijAbcdefghijAbcdefghijAbcdef
        `21`                 
        <chr>                
      1 AbcdefghijAbcdefghijA
        `46`                                          
        <chr>                                         
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdef
        `33`                             
        <chr>                            
      1 AbcdefghijAbcdefghijAbcdefghijAbc
        `25`                     
        <chr>                    
      1 AbcdefghijAbcdefghijAbcde
        `35`                                `1`   `5`  
        <chr>                               <chr> <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcde A     Abcde
        `16`             `34`                              
        <chr>            <chr>                             
      1 AbcdefghijAbcdef AbcdefghijAbcdefghijAbcdefghijAbcd
        `18`              
        <chr>             
      1 AbcdefghijAbcdefgh
        `42`                                       `3`  
        <chr>                                      <chr>
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAb Abc  
        `11`        `40`                       `26`  `37`  `7`  
        <chr>       <chr>                      <chr> <chr> <chr>
      1 AbcdefghijA AbcdefghijAbcdefghijAbcde~ Abcd~ Abcd~ Abcd~
        `39`     `6`   `4`   `19`  `8`   `45`  `14`  `24`  `23` 
        <chr>    <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefg~ Abcd~ Abcd  Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `2`   `47`     `9`   `49`  `41`  `38`  `22`  `44`  `15` 
        <chr> <chr>    <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Ab    Abcdefg~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(1L, 26L, 20L, 12L, 49L, 16L, 24L, 4L, 15L, 47L, 8L, 11L,
        14L, 50L, 17L, 2L, 44L, 30L, 36L, 45L, 25L, 38L, 18L, 29L, 5L, 13L, 3L, 23L,
        48L, 40L, 34L, 22L, 39L, 33L, 27L, 7L, 19L, 10L, 37L, 6L, 35L, 46L, 31L, 41L,
        43L, 28L, 42L, 32L, 21L, 9L)], width = {
        options(width = 47)
        484
      })
    Output
      $body
        `1`   `26`                      
        <chr> <chr>                     
      1 A     AbcdefghijAbcdefghijAbcdef
        `20`                 `12`        
        <chr>                <chr>       
      1 AbcdefghijAbcdefghij AbcdefghijAb
        `49`                                         
        <chr>                                        
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcd~
        `16`             `24`                    
        <chr>            <chr>                   
      1 AbcdefghijAbcdef AbcdefghijAbcdefghijAbcd
        `4`   `15`            `47`                   
        <chr> <chr>           <chr>                  
      1 Abcd  AbcdefghijAbcde AbcdefghijAbcdefghijAb~
        `8`      `11`        `14`         `50`  `17` 
        <chr>    <chr>       <chr>        <chr> <chr>
      1 Abcdefgh AbcdefghijA AbcdefghijA~ Abcd~ Abcd~
        `2`   `44`      `30`  `36`  `45`  `25`  `38` 
        <chr> <chr>     <chr> <chr> <chr> <chr> <chr>
      1 Ab    Abcdefgh~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `18`      `29`  `5`   `13`  `3`   `23`  `48` 
        <chr>     <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefgh~ Abcd~ Abcde Abcd~ Abc   Abcd~ Abcd~
        `40`      `34`  `22`  `39`  `33`  `27`  `7`  
        <chr>     <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefgh~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `19`      `10`  `37`  `6`   `35`  `46`  `31` 
        <chr>     <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefgh~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `41`      `43`  `28`  `42`  `32`  `21`  `9`  
        <chr>     <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefgh~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(6L, 49L, 26L, 45L, 25L, 15L, 31L, 20L, 21L, 44L, 23L,
        48L, 37L, 36L, 5L, 43L, 11L, 14L, 13L, 39L, 16L, 12L, 4L, 18L, 42L, 3L, 10L,
        28L, 40L, 24L, 29L, 17L, 35L, 47L, 2L, 38L, 34L, 9L, 7L, 8L, 50L, 33L, 32L,
        27L, 46L, 19L, 22L, 41L, 30L, 1L)], width = {
        options(width = 55)
        779
      })
    Output
      $body
        `6`   
        <chr> 
      1 Abcdef
        `49`                                             
        <chr>                                            
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefghi
        `26`                      
        <chr>                     
      1 AbcdefghijAbcdefghijAbcdef
        `45`                                         
        <chr>                                        
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcde
        `25`                      `15`           
        <chr>                     <chr>          
      1 AbcdefghijAbcdefghijAbcde AbcdefghijAbcde
        `31`                            `20`                
        <chr>                           <chr>               
      1 AbcdefghijAbcdefghijAbcdefghijA AbcdefghijAbcdefghij
        `21`                 
        <chr>                
      1 AbcdefghijAbcdefghijA
        `44`                                        
        <chr>                                       
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcd
        `23`                   
        <chr>                  
      1 AbcdefghijAbcdefghijAbc
        `48`                                            
        <chr>                                           
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `37`                                  `36`           
        <chr>                                 <chr>          
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefg AbcdefghijAbcd~
        `5`   `43`  `11`  `14`  `13`  `39`  `16`  `12`  `4`  
        <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcde Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd 
        `18`  `42`  `3`   `10`  `28`  `40`  `24`  `29`  `17` 
        <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abc   Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `35`  `47`  `2`   `38`  `34`  `9`   `7`   `8`   `50` 
        <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Ab    Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `33`  `32`  `27`  `46`  `19`  `22`  `41`  `30`  `1`  
        <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ A    
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      ctl_colonnade(df_str[c(38L, 42L, 41L, 10L, 40L, 11L, 27L, 9L, 17L, 37L, 46L,
        13L, 36L, 18L, 31L, 20L, 39L, 12L, 44L, 33L, 50L, 34L, 26L, 32L, 23L, 30L,
        29L, 21L, 4L, 49L, 19L, 25L, 3L, 6L, 15L, 14L, 43L, 48L, 8L, 22L, 1L, 2L, 45L,
        35L, 16L, 5L, 47L, 28L, 24L, 7L)], width = {
        options(width = 46)
        694
      })
    Output
      $body
        `38`                                  
        <chr>                                 
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefgh
        `42`                                      
        <chr>                                     
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAb
        `41`                                     
        <chr>                                    
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijA
        `10`      
        <chr>     
      1 Abcdefghij
        `40`                                    
        <chr>                                   
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghij
        `11`        `27`                       
        <chr>       <chr>                      
      1 AbcdefghijA AbcdefghijAbcdefghijAbcdefg
        `9`       `17`             
        <chr>     <chr>            
      1 Abcdefghi AbcdefghijAbcdefg
        `37`                                 
        <chr>                                
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefg
        `46`                                        
        <chr>                                       
      1 AbcdefghijAbcdefghijAbcdefghijAbcdefghijAbc~
        `13`         
        <chr>        
      1 AbcdefghijAbc
        `36`                             `18`  `31` 
        <chr>                            <chr> <chr>
      1 AbcdefghijAbcdefghijAbcdefghijA~ Abcd~ Abcd~
        `20`     `39`  `12`  `44`  `33`  `50`  `34` 
        <chr>    <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefg~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~
        `26`     `32`  `23`  `30`  `29`  `21`  `4`  
        <chr>    <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefg~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd~ Abcd 
        `49`     `19`  `25`  `3`   `6`   `15`  `14` 
        <chr>    <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefg~ Abcd~ Abcd~ Abc   Abcd~ Abcd~ Abcd~
        `43`     `48`  `8`   `22`  `1`   `2`   `45` 
        <chr>    <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefg~ Abcd~ Abcd~ Abcd~ A     Ab    Abcd~
        `35`     `16`  `5`   `47`  `28`  `24`  `7`  
        <chr>    <chr> <chr> <chr> <chr> <chr> <chr>
      1 Abcdefg~ Abcd~ Abcde Abcd~ Abcd~ Abcd~ Abcd~
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      

# title ticks and width

    Code
      format_title("proper_title", 15, footnote = FALSE)
    Output
      [1] "proper_title"
    Code
      format_title("proper_title", 12, footnote = FALSE)
    Output
      [1] "proper_title"
    Code
      format_title("proper_title", 10, footnote = FALSE)
    Output
      [1] "proper_ti~"
    Code
      format_title("`a b`", 6, footnote = FALSE)
    Output
      [1] "`a b`"
    Code
      format_title("`a b`", 5, footnote = FALSE)
    Output
      [1] "`a b`"
    Code
      format_title("`a b`", 4, footnote = FALSE)
    Output
      [1] "a b~"
    Code
      format_title("`a b`", 3, footnote = FALSE)
    Output
      [1] "a ~"
    Code
      format_title("`a b`", 3, footnote = FALSE)
    Output
      [1] "a ~"

# output test

    Code
      pillar(factor(c(letters[1:5], NA)))
    Output
      <pillar>
      <fct>
      a    
      b    
      c    
      d    
      e    
      <NA> 
    Code
      pillar(ordered(c(letters[1:5], NA)))
    Output
      <pillar>
      <ord>
      a    
      b    
      c    
      d    
      e    
      <NA> 
    Code
      pillar(factor("a\nb"))
    Output
      <pillar>
      <fct> 
      "a\nb"

# integer64 output will use scientific if necessary

    Code
      pillar(add_special(x), width = 6)
    Output
      <pillar>
      <int6>
        0   
        1e11
        1e12
       NA   
    Code
      pillar(add_special(x), width = 7)
    Output
      <pillar>
      <int64>
         0   
         1e11
         1e12
        NA   
    Code
      pillar(add_special(x), width = 8)
    Output
      <pillar>
       <int64>
          0   
          1e11
          1e12
         NA   
    Code
      pillar(add_special(x), width = 9)
    Output
      <pillar>
        <int64>
           0   
           1e11
           1e12
          NA   
    Code
      pillar(add_special(x), width = 10)
    Output
      <pillar>
         <int64>
            0   
            1e11
            1e12
           NA   
    Code
      pillar(add_special(x), width = 11)
    Output
      <pillar>
          <int64>
             0   
             1e11
             1e12
            NA   
    Code
      pillar(add_special(x), width = 12)
    Output
      <pillar>
           <int64>
              0   
              1e11
              1e12
             NA   
    Code
      pillar(add_special(x), width = 13)
    Output
      <pillar>
            <int64>
                  0
       100000000001
      1000000000002
                 NA
    Code
      pillar(add_special(x), width = 14)
    Output
      <pillar>
            <int64>
                  0
       100000000001
      1000000000002
                 NA

# integer64 output uses underlines correctly

    Code
      pillar(bit64::as.integer64(c(1, 10000)))
    Output
      <pillar>
      [3m[90m<int64>[39m[23m
            1
        [4m1[24m[4m0[24m000

# nanotime works (#378)

    Code
      pillar(x, width = 16)
    Output
      <pillar>
      <int64>         
      2011-12-05T08:3~
    Code
      pillar(x, width = 17)
    Output
      <pillar>
      <int64>          
      2011-12-05T08:30~
    Code
      pillar(x, width = 18)
    Output
      <pillar>
      <int64>           
      2011-12-05T08:30:~
    Code
      pillar(x, width = 19)
    Output
      <pillar>
      <int64>            
      2011-12-05T08:30:0~
    Code
      pillar(x, width = 20)
    Output
      <pillar>
      <int64>             
      2011-12-05T08:30:00~
    Code
      pillar(x, width = 21)
    Output
      <pillar>
      <int64>              
      2011-12-05T08:30:00+~
    Code
      pillar(x, width = 22)
    Output
      <pillar>
      <int64>               
      2011-12-05T08:30:00+0~
    Code
      pillar(x, width = 23)
    Output
      <pillar>
      <int64>                
      2011-12-05T08:30:00+00~
    Code
      pillar(x, width = 24)
    Output
      <pillar>
      <int64>                 
      2011-12-05T08:30:00+00:~
    Code
      pillar(x, width = 25)
    Output
      <pillar>
      <int64>                  
      2011-12-05T08:30:00+00:00
    Code
      pillar(x, width = 26)
    Output
      <pillar>
      <int64>                  
      2011-12-05T08:30:00+00:00

# with and without title

    Code
      pillar(add_special(10^(1:6)))
    Output
      <pillar>
        <dbl>
           10
          100
         1000
        10000
       100000
      1000000
           NA
         -Inf
          Inf
    Code
      pillar(add_special(10^(1:6)), title = "crayon")
    Output
      <pillar>
       crayon
        <dbl>
           10
          100
         1000
        10000
       100000
      1000000
           NA
         -Inf
          Inf
    Code
      pillar(add_special(10^(1:6)), title = "short")
    Output
      <pillar>
        short
        <dbl>
           10
          100
         1000
        10000
       100000
      1000000
           NA
         -Inf
          Inf
    Code
      pillar(add_special(10^(1:6)), title = "somewhat_wider")
    Output
      <pillar>
      somewhat_wider
               <dbl>
                  10
                 100
                1000
               10000
              100000
             1000000
                  NA
                -Inf
                 Inf
    Code
      pillar(add_special(10^(1:6)), title = "exactly_fifteen")
    Output
      <pillar>
      exactly_fifteen
                <dbl>
                   10
                  100
                 1000
                10000
               100000
              1000000
                   NA
                 -Inf
                  Inf
    Code
      pillar(add_special(10^(1:6)), title = "absolutely_breaking_all_sensible_boundaries",
      width = 18)
    Output
      NULL

# same pillar at different widths

    Code
      pillar(v, width = 4)
    Output
      NULL
    Code
      pillar(v, width = 7)
    Output
      <pillar>
        <dbl>
         1e-5
         1e-2
         1e+3
         1e+6
    Code
      pillar(v, width = 10)
    Output
      <pillar>
           <dbl>
            1e-5
            1e-2
            1e+3
            1e+6
    Code
      pillar(v, width = 15)
    Output
      <pillar>
              <dbl>
            0.00001
            0.01   
         1000      
      1000000      
    Code
      pillar(v, width = 22)
    Output
      <pillar>
              <dbl>
            0.00001
            0.01   
         1000      
      1000000      

# output test

    Code
      pillar(as.list(1:3))
    Output
      <pillar>
      <list>   
      <int [1]>
      <int [1]>
      <int [1]>
    Code
      pillar(list(1:3, NULL))
    Output
      <pillar>
      <list>   
      <int [3]>
      <NULL>   
    Code
      pillar(add_special(list(1:3)))
    Output
      <pillar>
      <list>   
      <int [3]>
      <NULL>   
    Code
      pillar(list(trees), width = 10)
    Output
      <pillar>
      <list>
      <df>  

---

    Code
      pillar(list(data.frame()), width = 11)
    Output
      <pillar>
      [3m[90m<list>[39m[23m
      [90m<df>[39m  
    Code
      pillar(list(data.frame()), width = 12)
    Output
      <pillar>
      [3m[90m<list>[39m[23m      
      [90m<df [0 x 0]>[39m

# color

    Code
      style_na("NA")
    Output
      [1] "\033[31mNA\033[39m"
    Code
      style_neg("-1")
    Output
      [1] "\033[31m-1\033[39m"

---

    Code
      xf <- (function() ctl_colonnade(list(x = c((10^(-3:4)) * c(-1, 1), NA))))
      print(xf())
    Output
      $body
                x
            [3m[90m<dbl>[39m[23m
      [90m1[39m    -[31m0[39m[31m.[39m[31m00[39m[31m1[39m
      [90m2[39m     0.01 
      [90m3[39m    -[31m0[39m[31m.[39m[31m1[39m  
      [90m4[39m     1    
      [90m5[39m   -[31m10[39m    
      [90m6[39m   100    
      [90m7[39m -[31m[4m1[24m00[39m[31m0[39m    
      [90m8[39m [4m1[24m[4m0[24m000    
      [90m9[39m    [31mNA[39m    
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      with_options(pillar.subtle_num = TRUE, print(xf()))
    Output
      $body
                x
            [3m[90m<dbl>[39m[23m
      [90m1[39m    -[90m0[39m[90m.[39m[90m00[39m[31m1[39m
      [90m2[39m     [90m0[39m[90m.[39m[90m0[39m1 
      [90m3[39m    -[90m0[39m[90m.[39m[31m1[39m  
      [90m4[39m     1    
      [90m5[39m   -[31m10[39m    
      [90m6[39m   100    
      [90m7[39m -[31m[4m1[24m00[39m[90m0[39m    
      [90m8[39m [4m1[24m[4m0[24m0[90m00[39m    
      [90m9[39m    [31mNA[39m    
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      with_options(pillar.subtle = FALSE, print(xf()))
    Output
      $body
                x
            [3m<dbl>[23m
      1    -[31m0[39m[31m.[39m[31m00[39m[31m1[39m
      2     0.01 
      3    -[31m0[39m[31m.[39m[31m1[39m  
      4     1    
      5   -[31m10[39m    
      6   100    
      7 -[31m[4m1[24m00[39m[31m0[39m    
      8 [4m1[24m[4m0[24m000    
      9    [31mNA[39m    
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      with_options(pillar.neg = FALSE, print(xf()))
    Output
      $body
                x
            [3m[90m<dbl>[39m[23m
      [90m1[39m    -0.001
      [90m2[39m     0.01 
      [90m3[39m    -0.1  
      [90m4[39m     1    
      [90m5[39m   -10    
      [90m6[39m   100    
      [90m7[39m -[4m1[24m000    
      [90m8[39m [4m1[24m[4m0[24m000    
      [90m9[39m    [31mNA[39m    
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      with_options(pillar.subtle = FALSE, pillar.neg = FALSE, print(xf()))
    Output
      $body
                x
            [3m<dbl>[23m
      1    -0.001
      2     0.01 
      3    -0.1  
      4     1    
      5   -10    
      6   100    
      7 -[4m1[24m000    
      8 [4m1[24m[4m0[24m000    
      9    [31mNA[39m    
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      
    Code
      with_options(pillar.bold = TRUE, print(xf()))
    Output
      $body
                [1mx[22m
            [3m[90m<dbl>[39m[23m
      [90m1[39m    -[31m0[39m[31m.[39m[31m00[39m[31m1[39m
      [90m2[39m     0.01 
      [90m3[39m    -[31m0[39m[31m.[39m[31m1[39m  
      [90m4[39m     1    
      [90m5[39m   -[31m10[39m    
      [90m6[39m   100    
      [90m7[39m -[31m[4m1[24m00[39m[31m0[39m    
      [90m8[39m [4m1[24m[4m0[24m000    
      [90m9[39m    [31mNA[39m    
      
      $extra_cols
      list()
      
      $abbrev_cols
      character(0)
      
      $abbrev_col_idxs
      numeric(0)
      

---

    Code
      ctl_colonnade(list(a_very_long_column_name = 0), width = 20)
    Output
      $body
        a_very_long_colu~1
                     [3m[90m<dbl>[39m[23m
      [90m1[39m                  0
      
      $extra_cols
      list()
      
      $abbrev_cols
      [1] "a_very_long_column_name"
      
      $abbrev_col_idxs
      numeric(0)
      

# focus columns

    Code
      tbl_format_setup(x, width = 30, focus = "b")
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data frame: 1 x 2[39m
      [1m<tbl_format_body(setup)>[22m
          a$x    $y b          
        [3m[90m<dbl>[39m[23m [3m[90m<dbl>[39m[23m [4m[3m[90m<chr>[39m[23m[24m      
      [90m1[39m     1     2 long enough
      [1m<tbl_format_footer(setup)>[22m
    Code
      tbl_format_setup(x, width = 20, focus = "b")
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data frame: 1 x
      #   2[39m
      [1m<tbl_format_body(setup)>[22m
          a$x b          
        [3m[90m<dbl>[39m[23m [4m[3m[90m<chr>[39m[23m[24m      
      [90m1[39m     1 long enough
      [1m<tbl_format_footer(setup)>[22m
      [90m# ... with 1 more[39m
      [90m#   variable:[39m
      [90m#   a$y <dbl>[39m
      [90m# i Use `colnames()` to see all variable names[39m
    Code
      tbl_format_setup(x, width = 15, focus = "b")
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data
      #   frame: 1 x
      #   2[39m
      [1m<tbl_format_body(setup)>[22m
        b          
        [4m[3m[90m<chr>[39m[23m[24m      
      [90m1[39m long enough
      [1m<tbl_format_footer(setup)>[22m
      [90m# ... with 1[39m
      [90m#   more[39m
      [90m#   variable:[39m
      [90m#   a <tbl[,2]>[39m
      [90m# i Use `colnames()` to see all variable names[39m
    Code
      tbl_format_setup(x, width = 10, focus = "b")
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data
      #   frame:
      #   1 x 2[39m
      [1m<tbl_format_body(setup)>[22m
        b       
        [4m[3m[90m<chr>[39m[23m[24m   
      [90m1[39m long en~
      [1m<tbl_format_footer(setup)>[22m
      [90m#   with[39m
      [90m#   1[39m
      [90m#   more[39m
      [90m#   variable: ...[39m
      [90m# i Use `colnames()` to see all variable names[39m
    Code
      tbl_format_setup(x[2:1], width = 30, focus = "a")
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data frame: 1 x 2[39m
      [1m<tbl_format_body(setup)>[22m
        b             a$x    $y
        [3m[90m<chr>[39m[23m       [4m[3m[90m<dbl>[39m[23m[24m [4m[3m[90m<dbl>[39m[23m[24m
      [90m1[39m long enough     1     2
      [1m<tbl_format_footer(setup)>[22m
    Code
      tbl_format_setup(x[2:1], width = 20, focus = "a")
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data frame: 1 x
      #   2[39m
      [1m<tbl_format_body(setup)>[22m
        b        a$x    $y
        [3m[90m<chr>[39m[23m  [4m[3m[90m<dbl>[39m[23m[24m [4m[3m[90m<dbl>[39m[23m[24m
      [90m1[39m long ~     1     2
      [1m<tbl_format_footer(setup)>[22m
    Code
      tbl_format_setup(x[2:1], width = 15, focus = "a")
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data
      #   frame: 1 x
      #   2[39m
      [1m<tbl_format_body(setup)>[22m
          a$x    $y
        [4m[3m[90m<dbl>[39m[23m[24m [4m[3m[90m<dbl>[39m[23m[24m
      [90m1[39m     1     2
      [1m<tbl_format_footer(setup)>[22m
      [90m# ... with 1[39m
      [90m#   more[39m
      [90m#   variable:[39m
      [90m#   b <chr>[39m
      [90m# i Use `colnames()` to see all variable names[39m
    Code
      tbl_format_setup(x[2:1], width = 10, focus = "a")
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data
      #   frame:
      #   1 x 2[39m
      [1m<tbl_format_body(setup)>[22m
          a$x
        [4m[3m[90m<dbl>[39m[23m[24m
      [90m1[39m     1
      [1m<tbl_format_footer(setup)>[22m
      [90m#   with[39m
      [90m#   2[39m
      [90m#   more[39m
      [90m#   variables:[39m
      [90m#   a$y <dbl>, ...[39m
      [90m# i Use `colnames()` to see all variable names[39m
    Code
      tbl_format_setup(x, width = 30, focus = c("a", "b"))
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data frame: 1 x 2[39m
      [1m<tbl_format_body(setup)>[22m
          a$x    $y b          
        [4m[3m[90m<dbl>[39m[23m[24m [4m[3m[90m<dbl>[39m[23m[24m [4m[3m[90m<chr>[39m[23m[24m      
      [90m1[39m     1     2 long enough
      [1m<tbl_format_footer(setup)>[22m
    Code
      tbl_format_setup(x, width = 20, focus = c("a", "b"))
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data frame: 1 x
      #   2[39m
      [1m<tbl_format_body(setup)>[22m
          a$x    $y b     
        [4m[3m[90m<dbl>[39m[23m[24m [4m[3m[90m<dbl>[39m[23m[24m [4m[3m[90m<chr>[39m[23m[24m 
      [90m1[39m     1     2 long ~
      [1m<tbl_format_footer(setup)>[22m
    Code
      tbl_format_setup(x, width = 15, focus = c("a", "b"))
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data
      #   frame: 1 x
      #   2[39m
      [1m<tbl_format_body(setup)>[22m
          a$x b      
        [4m[3m[90m<dbl>[39m[23m[24m [4m[3m[90m<chr>[39m[23m[24m  
      [90m1[39m     1 long e~
      [1m<tbl_format_footer(setup)>[22m
      [90m# ... with 1[39m
      [90m#   more[39m
      [90m#   variable:[39m
      [90m#   a$y <dbl>[39m
      [90m# i Use `colnames()` to see all variable names[39m
    Code
      tbl_format_setup(x, width = 10, focus = c("a", "b"))
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data
      #   frame:
      #   1 x 2[39m
      [1m<tbl_format_body(setup)>[22m
          a$x
        [4m[3m[90m<dbl>[39m[23m[24m
      [90m1[39m     1
      [1m<tbl_format_footer(setup)>[22m
      [90m#   with[39m
      [90m#   2[39m
      [90m#   more[39m
      [90m#   variables:[39m
      [90m#   a$y <dbl>, ...[39m
      [90m# i Use `colnames()` to see all variable names[39m
    Code
      tbl_format_setup(x[2:1], width = 30, focus = c("a", "b"))
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data frame: 1 x 2[39m
      [1m<tbl_format_body(setup)>[22m
        b             a$x    $y
        [4m[3m[90m<chr>[39m[23m[24m       [4m[3m[90m<dbl>[39m[23m[24m [4m[3m[90m<dbl>[39m[23m[24m
      [90m1[39m long enough     1     2
      [1m<tbl_format_footer(setup)>[22m
    Code
      tbl_format_setup(x[2:1], width = 20, focus = c("a", "b"))
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data frame: 1 x
      #   2[39m
      [1m<tbl_format_body(setup)>[22m
        b             a$x
        [4m[3m[90m<chr>[39m[23m[24m       [4m[3m[90m<dbl>[39m[23m[24m
      [90m1[39m long enough     1
      [1m<tbl_format_footer(setup)>[22m
      [90m# ... with 1 more[39m
      [90m#   variable:[39m
      [90m#   a$y <dbl>[39m
      [90m# i Use `colnames()` to see all variable names[39m
    Code
      tbl_format_setup(x[2:1], width = 15, focus = c("a", "b"))
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data
      #   frame: 1 x
      #   2[39m
      [1m<tbl_format_body(setup)>[22m
        b         a$x
        [4m[3m[90m<chr>[39m[23m[24m   [4m[3m[90m<dbl>[39m[23m[24m
      [90m1[39m long e~     1
      [1m<tbl_format_footer(setup)>[22m
      [90m# ... with 1[39m
      [90m#   more[39m
      [90m#   variable:[39m
      [90m#   a$y <dbl>[39m
      [90m# i Use `colnames()` to see all variable names[39m
    Code
      tbl_format_setup(x[2:1], width = 10, focus = c("a", "b"))
    Output
      [1m<pillar_tbl_format_setup>[22m
      [1m<tbl_format_header(setup)>[22m
      [90m# A data
      #   frame:
      #   1 x 2[39m
      [1m<tbl_format_body(setup)>[22m
        b       
        [4m[3m[90m<chr>[39m[23m[24m   
      [90m1[39m long en~
      [1m<tbl_format_footer(setup)>[22m
      [90m#   with[39m
      [90m#   1[39m
      [90m#   more[39m
      [90m#   variable: ...[39m
      [90m# i Use `colnames()` to see all variable names[39m

# sanity check (1)

    Code
      cli::num_ansi_colors()
    Output
      [1] 1
    Code
      has_color()
    Output
      [1] FALSE
    Code
      num_colors()
    Output
      [1] 1
    Code
      style_na("NA")
    Output
      [1] "NA"

# output test

    Code
      colonnade(x, width = 4)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Code
      colonnade(x, width = 5)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Code
      colonnade(x, width = 6)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Code
      colonnade(x, width = 7)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        colu~
        <dbl>
      1  1.23
      2  2.23
      3  3.23
    Code
      colonnade(x, width = 8)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        colum~
         <dbl>
      1   1.23
      2   2.23
      3   3.23
    Code
      colonnade(x, width = 9)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column~
          <dbl>
      1    1.23
      2    2.23
      3    3.23
    Code
      colonnade(x, width = 10)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_~
           <dbl>
      1     1.23
      2     2.23
      3     3.23
    Code
      colonnade(x, width = 11)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_z~
            <dbl>
      1      1.23
      2      2.23
      3      3.23
    Code
      colonnade(x, width = 12)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_ze~
             <dbl>
      1       1.23
      2       2.23
      3       3.23
    Code
      colonnade(x, width = 13)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column~
          <dbl>
      1    1.23
      2    2.23
      3    3.23
    Code
      colonnade(x, width = 14)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column~
          <dbl>
      1    1.23
      2    2.23
      3    3.23
    Code
      colonnade(x, width = 15)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column~
          <dbl>
      1    1.23
      2    2.23
      3    3.23
    Code
      colonnade(x, width = 16)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column~ col_02
          <dbl> <chr> 
      1    1.23 a     
      2    2.23 b     
      3    3.23 c     
    Code
      colonnade(x, width = 17)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_~ col_02
           <dbl> <chr> 
      1     1.23 a     
      2     2.23 b     
      3     3.23 c     
    Code
      colonnade(x, width = 18)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_z~ col_02
            <dbl> <chr> 
      1      1.23 a     
      2      2.23 b     
      3      3.23 c     
    Code
      colonnade(x, width = 19)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column~ col_02
          <dbl> <chr> 
      1    1.23 a     
      2    2.23 b     
      3    3.23 c     
    Code
      colonnade(x, width = 20)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column~ col_02
          <dbl> <chr> 
      1    1.23 a     
      2    2.23 b     
      3    3.23 c     
    Code
      colonnade(x, width = 21)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column~ col_02
          <dbl> <chr> 
      1    1.23 a     
      2    2.23 b     
      3    3.23 c     
    Code
      colonnade(x, width = 22)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column~ col_02
          <dbl> <chr> 
      1    1.23 a     
      2    2.23 b     
      3    3.23 c     
    Code
      colonnade(x, width = 23)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column~ col_02 col_03
          <dbl> <chr>  <fct> 
      1    1.23 a      a     
      2    2.23 b      b     
      3    3.23 c      c     
    Code
      colonnade(x, width = 24)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_~ col_02 col_03
           <dbl> <chr>  <fct> 
      1     1.23 a      a     
      2     2.23 b      b     
      3     3.23 c      c     
    Code
      colonnade(x, width = 25)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column~ col_02 col_03
          <dbl> <chr>  <fct> 
      1    1.23 a      a     
      2    2.23 b      b     
      3    3.23 c      c     
    Code
      colonnade(x, width = 26)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column~ col_02 col_03
          <dbl> <chr>  <fct> 
      1    1.23 a      a     
      2    2.23 b      b     
      3    3.23 c      c     
    Code
      colonnade(x, width = 27)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column~ col_02 col_03
          <dbl> <chr>  <fct> 
      1    1.23 a      a     
      2    2.23 b      b     
      3    3.23 c      c     
    Code
      colonnade(x, width = 28)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column~ col_02 col_03
          <dbl> <chr>  <fct> 
      1    1.23 a      a     
      2    2.23 b      b     
      3    3.23 c      c     
    Code
      colonnade(x, width = 29)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column~ col_02 col_03
          <dbl> <chr>  <fct> 
      1    1.23 a      a     
      2    2.23 b      b     
      3    3.23 c      c     
    Code
      colonnade(x, width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column~ col_02 col_03 col_04
          <dbl> <chr>  <fct>  <ord> 
      1    1.23 a      a      a     
      2    2.23 b      b      b     
      3    3.23 c      c      c     
    Code
      colonnade(x, width = 31)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_~ col_02 col_03 col_04
           <dbl> <chr>  <fct>  <ord> 
      1     1.23 a      a      a     
      2     2.23 b      b      b     
      3     3.23 c      c      c     
    Code
      colonnade(x, width = 32)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_z~ col_02 col_03 col_04
            <dbl> <chr>  <fct>  <ord> 
      1      1.23 a      a      a     
      2      2.23 b      b      b     
      3      3.23 c      c      c     
    Code
      colonnade(x, width = 33)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_ze~ col_02 col_03 col_04
             <dbl> <chr>  <fct>  <ord> 
      1       1.23 a      a      a     
      2       2.23 b      b      b     
      3       3.23 c      c      c     
    Code
      colonnade(x, width = 34)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zer~ col_02 col_03 col_04
              <dbl> <chr>  <fct>  <ord> 
      1        1.23 a      a      a     
      2        2.23 b      b      b     
      3        3.23 c      c      c     
    Code
      colonnade(x, width = 35)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero~ col_02 col_03 col_04
               <dbl> <chr>  <fct>  <ord> 
      1         1.23 a      a      a     
      2         2.23 b      b      b     
      3         3.23 c      c      c     
    Code
      colonnade(x, width = 36)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_~ col_02 col_03 col_04
                <dbl> <chr>  <fct>  <ord> 
      1          1.23 a      a      a     
      2          2.23 b      b      b     
      3          3.23 c      c      c     
    Code
      colonnade(x, width = 37)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_o~ col_02 col_03 col_04
                 <dbl> <chr>  <fct>  <ord> 
      1           1.23 a      a      a     
      2           2.23 b      b      b     
      3           3.23 c      c      c     
    Code
      colonnade(x, width = 38)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one col_02 col_03 col_04
                  <dbl> <chr>  <fct>  <ord> 
      1            1.23 a      a      a     
      2            2.23 b      b      b     
      3            3.23 c      c      c     
    Code
      colonnade(x, width = 39)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one col_02 col_03 col_04
                  <dbl> <chr>  <fct>  <ord> 
      1            1.23 a      a      a     
      2            2.23 b      b      b     
      3            3.23 c      c      c     
    Code
      colonnade(x, width = Inf)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        column_zero_one col_02 col_03 col_04
                  <dbl> <chr>  <fct>  <ord> 
      1            1.23 a      a      a     
      2            2.23 b      b      b     
      3            3.23 c      c      c     

---

    Code
      colonnade(rep(list(paste(letters, collapse = " ")), 4), width = Inf)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        <chr>                                              
      1 a b c d e f g h i j k l m n o p q r s t u v w x y z
        <chr>                                              
      1 a b c d e f g h i j k l m n o p q r s t u v w x y z
        <chr>                                              
      1 a b c d e f g h i j k l m n o p q r s t u v w x y z
        <chr>                                              
      1 a b c d e f g h i j k l m n o p q r s t u v w x y z

---

    Code
      as_glue(extra_cols_impl(squeeze_impl(colonnade(x), width = 10)))
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
      col_02 <chr>
      col_03 <fct>
      col_04 <ord>

---

    Code
      as_glue(extra_cols_impl(squeeze_impl(colonnade(x), width = 20)))
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
      col_03 <fct>
      col_04 <ord>

---

    Code
      as_glue(extra_cols_impl(squeeze_impl(colonnade(x), width = 30)))
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.

---

    Code
      as_glue(extra_cols_impl(squeeze_impl(colonnade(x), width = 35)))
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.

---

    Code
      as_glue(extra_cols_impl(squeeze_impl(colonnade(x), width = 40)))
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.

# tests from tibble

    Code
      colonnade(mtcars[1:8, ], has_row_id = "*", width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
          mpg   cyl  disp    hp
      * <dbl> <dbl> <dbl> <dbl>
      1  21       6  160    110
      2  21       6  160    110
      3  22.8     4  108     93
      4  21.4     6  258    110
      5  18.7     8  360    175
      6  18.1     6  225    105
      7  14.3     8  360    245
      8  24.4     4  147.    62
    Code
      colonnade(trees[1:5, ], width = 20)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        Girth Height
        <dbl>  <dbl>
      1   8.3     70
      2   8.6     65
      3   8.8     63
      4  10.5     72
      5  10.7     81
    Code
      colonnade(trees[1:3, ], width = 10)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        Girth
        <dbl>
      1   8.3
      2   8.6
      3   8.8
    Code
      colonnade(df_all, width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a     b c     d    
        <dbl> <int> <lgl> <chr>
      1   1       1 TRUE  a    
      2   2.5     2 FALSE b    
      3  NA      NA NA    <NA> 
    Code
      colonnade(df_all, width = 300)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a     b c     d     e     f          g                   h        
        <dbl> <int> <lgl> <chr> <fct> <date>     <dttm>              <list>   
      1   1       1 TRUE  a     a     2015-12-10 2015-12-09 10:51:35 <int [1]>
      2   2.5     2 FALSE b     b     2015-12-11 2015-12-09 10:51:36 <int [1]>
      3  NA      NA NA    <NA>  <NA>  NA         NA                  <int [1]>
        i         
        <list>    
      1 <list [2]>
      2 <list [1]>
      3 <list [1]>
    Code
      options(width = 70)
      colonnade(df_all, width = 300)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a     b c     d     e     f          g                  
        <dbl> <int> <lgl> <chr> <fct> <date>     <dttm>             
      1   1       1 TRUE  a     a     2015-12-10 2015-12-09 10:51:35
      2   2.5     2 FALSE b     b     2015-12-11 2015-12-09 10:51:36
      3  NA      NA NA    <NA>  <NA>  NA         NA                 
        h         i         
        <list>    <list>    
      1 <int [1]> <list [2]>
      2 <int [1]> <list [1]>
      3 <int [1]> <list [1]>
    Code
      options(width = 60)
      colonnade(df_all, width = 300)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a     b c     d     e     f         
        <dbl> <int> <lgl> <chr> <fct> <date>    
      1   1       1 TRUE  a     a     2015-12-10
      2   2.5     2 FALSE b     b     2015-12-11
      3  NA      NA NA    <NA>  <NA>  NA        
        g                   h         i         
        <dttm>              <list>    <list>    
      1 2015-12-09 10:51:35 <int [1]> <list [2]>
      2 2015-12-09 10:51:36 <int [1]> <list [1]>
      3 NA                  <int [1]> <list [1]>
    Code
      options(width = 50)
      colonnade(df_all, width = 300)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a     b c     d     e     f         
        <dbl> <int> <lgl> <chr> <fct> <date>    
      1   1       1 TRUE  a     a     2015-12-10
      2   2.5     2 FALSE b     b     2015-12-11
      3  NA      NA NA    <NA>  <NA>  NA        
        g                   h         i         
        <dttm>              <list>    <list>    
      1 2015-12-09 10:51:35 <int [1]> <list [2]>
      2 2015-12-09 10:51:36 <int [1]> <list [1]>
      3 NA                  <int [1]> <list [1]>
    Code
      options(width = 40)
      colonnade(df_all, width = 300)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a     b c     d     e    
        <dbl> <int> <lgl> <chr> <fct>
      1   1       1 TRUE  a     a    
      2   2.5     2 FALSE b     b    
      3  NA      NA NA    <NA>  <NA> 
        f          g                  
        <date>     <dttm>             
      1 2015-12-10 2015-12-09 10:51:35
      2 2015-12-11 2015-12-09 10:51:36
      3 NA         NA                 
        h         i         
        <list>    <list>    
      1 <int [1]> <list [2]>
      2 <int [1]> <list [1]>
      3 <int [1]> <list [1]>
    Code
      options(width = 30)
      colonnade(df_all, width = 300)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a     b c     d    
        <dbl> <int> <lgl> <chr>
      1   1       1 TRUE  a    
      2   2.5     2 FALSE b    
      3  NA      NA NA    <NA> 
        e     f         
        <fct> <date>    
      1 a     2015-12-10
      2 b     2015-12-11
      3 <NA>  NA        
        g                  
        <dttm>             
      1 2015-12-09 10:51:35
      2 2015-12-09 10:51:36
      3 NA                 
        h         i         
        <list>    <list>    
      1 <int [1]> <list [2]>
      2 <int [1]> <list [1]>
      3 <int [1]> <list [1]>
    Code
      options(width = 20)
      colonnade(df_all, width = 300)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a     b c    
        <dbl> <int> <lgl>
      1   1       1 TRUE 
      2   2.5     2 FALSE
      3  NA      NA NA   
        d     e    
        <chr> <fct>
      1 a     a    
      2 b     b    
      3 <NA>  <NA> 
        f         
        <date>    
      1 2015-12-10
      2 2015-12-11
      3 NA        
        g                 
        <dttm>            
      1 2015-12-09 10:51:~
      2 2015-12-09 10:51:~
      3 NA                
        h        
        <list>   
      1 <int [1]>
      2 <int [1]>
      3 <int [1]>
        i         
        <list>    
      1 <list [2]>
      2 <list [1]>
      3 <list [1]>
    Code
      list_with_ctl <- list(c("\n", "\""), factor(c("\n", "\n")))
      names(list_with_ctl) <- c("\n", "\r")
      colonnade(list_with_ctl, width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        `\n`  `\r` 
        <chr> <fct>
      1 "\n"  "\n" 
      2 "\""  "\n" 
    Code
      colonnade(list(a = c("", " ", "a ", " a")), width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        a    
        <chr>
      1 ""   
      2 " "  
      3 "a " 
      4 " a" 
    Code
      colonnade(list(`mean(x)` = 5, `var(x)` = 3), width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
        `mean(x)` `var(x)`
            <dbl>    <dbl>
      1         5        3

# NA names

    Code
      colonnade(x, width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
         `NA` <int>
        <int> <int>
      1     1     4
      2     2     5
      3     3     6

# sep argument

    Code
      colonnade(x, width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
          sep
        <int>
      1     1
      2     2
      3     3
    Code
      # dummy

# sanity check (2)

    Code
      cli::num_ansi_colors()
    Output
      [1] 1
    Code
      has_color()
    Output
      [1] FALSE
    Code
      num_colors()
    Output
      [1] 1
    Code
      style_na("NA")
    Output
      [1] "NA"

# tibble columns

    Code
      colonnade(x, width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a   b$c    $d
        <int> <int> <int>
      1     1     4     7
      2     2     5     8
      3     3     6     9

# tibble columns (nested)

    Code
      colonnade(x, width = 40)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a   b$c    $d  $e$f   $$g
        <int> <int> <int> <int> <int>
      1     1     4     7    10    13
      2     2     5     8    11    14
      3     3     6     9    12    15

# tibble columns (empty)

    Code
      colonnade(x, width = 40)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a   b$c    $d $e          $f
        <int> <int> <int> <df[,0]> <int>
      1     1     4     7             10
      2     2     5     8             11
      3     3     6     9             12

# matrix columns (unnamed)

    Code
      colonnade(x, width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a b[,1]  [,2]
        <int> <int> <int>
      1     1     4     7
      2     2     5     8
      3     3     6     9

# matrix columns (named)

    Code
      colonnade(x, width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a b[,"c"] [,"d"]
        <int>   <int>  <int>
      1     1       4      7
      2     2       5      8
      3     3       6      9

# matrix columns (empty)

    Code
      colonnade(x, width = 30)
    Condition
      Warning:
      `colonnade()` was deprecated in pillar 1.7.0.
      Please use `tbl_format_setup()` instead.
    Output
            a b             c
        <int> <int[,0]> <int>
      1     1               4
      2     2               5
      3     3               6

# max argument

    Code
      print(num(1:10), max = 5)
    Output
      <pillar_num[10]>
      [1] 1 2 3 4 5
      ... and 5 more
    Code
      print(char(letters), max = 10)
    Output
      <pillar_char[26]>
       [1] a b c d e f g h i j
      ... and 16 more

# max.print option

    Code
      num(1:10)
    Output
      <pillar_num[10]>
      [1] 1 2 3
      ... and 7 more
    Code
      char(letters)
    Output
      <pillar_char[26]>
      [1] a b c
      ... and 23 more

# output test

    Code
      pillar(I(1:3))
    Output
      <pillar>
      <I<int>>
             1
             2
             3
    Code
      pillar(I(list(1, 1:2, 1:3)))
    Output
      <pillar>
      <I<list>>
      <dbl [1]>
      <int [2]>
      <int [3]>

# with and without title

    Code
      pillar(add_special(10^(1:6)))
    Output
      <pillar>
        <dbl>
           10
          100
         1000
        10000
       100000
      1000000
           NA
         -Inf
          Inf
    Code
      pillar(add_special(10^(1:6)), title = "crayon")
    Output
      <pillar>
       crayon
        <dbl>
           10
          100
         1000
        10000
       100000
      1000000
           NA
         -Inf
          Inf
    Code
      pillar(add_special(10^(1:6)), title = "short")
    Output
      <pillar>
        short
        <dbl>
           10
          100
         1000
        10000
       100000
      1000000
           NA
         -Inf
          Inf
    Code
      pillar(add_special(10^(1:6)), title = "somewhat_wider")
    Output
      <pillar>
      somewhat_wider
               <dbl>
                  10
                 100
                1000
               10000
              100000
             1000000
                  NA
                -Inf
                 Inf
    Code
      pillar(add_special(10^(1:6)), title = "exactly_fifteen")
    Output
      <pillar>
      exactly_fifteen
                <dbl>
                   10
                  100
                 1000
                10000
               100000
              1000000
                   NA
                 -Inf
                  Inf
    Code
      pillar(add_special(10^(1:6)), title = "absolutely_breaking_all_sensible_boundaries",
      width = 18)
    Output
      <pillar>
      absolutely_break~1
                   <dbl>
                      10
                     100
                    1000
                   10000
                  100000
                 1000000
                      NA
                    -Inf
                     Inf

# output test

    Code
      pillar(as.list(1:3))
    Output
      <pillar>
      <list>   
      <int [1]>
      <int [1]>
      <int [1]>
    Code
      pillar(list(1:3, NULL))
    Output
      <pillar>
      <list>   
      <int [3]>
      <NULL>   
    Code
      pillar(add_special(list(1:3)))
    Output
      <pillar>
      <list>   
      <int [3]>
      <NULL>   
    Code
      pillar(list(trees), width = 10)
    Output
      <pillar>
      <list>
      <df>  

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shaft-.R, R/shaft-simple.R
\name{new_pillar_shaft}
\alias{new_pillar_shaft}
\alias{new_pillar_shaft_simple}
\title{Constructor for column data}
\usage{
new_pillar_shaft(
  x,
  ...,
  width = NULL,
  min_width = width,
  type_sum = NULL,
  class = NULL,
  subclass = NULL
)

new_pillar_shaft_simple(
  formatted,
  ...,
  width = NULL,
  align = "left",
  min_width = NULL,
  na = NULL,
  na_indent = 0L,
  shorten = c("back", "front", "mid", "abbreviate"),
  short_formatted = NULL
)
}
\arguments{
\item{x}{An object}

\item{...}{Passed on to \code{\link[=new_pillar_shaft]{new_pillar_shaft()}}.}

\item{width}{The maximum column width.}

\item{min_width}{The minimum allowed column width, \code{width} if omitted.}

\item{type_sum}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Override the type summary displayed at the top of the data.
This argument, if given, takes precedence over the type summary provided by
\code{\link[=type_sum]{type_sum()}}.}

\item{class}{The name of the subclass.}

\item{subclass}{Deprecated, pass the \code{class} argument instead.}

\item{formatted}{The data to show, an object coercible to \link{character}.}

\item{align}{Alignment of the column.}

\item{na}{String to use as \code{NA} value, defaults to \code{"NA"} styled with
\code{\link[=style_na]{style_na()}} with fallback if color is not available.}

\item{na_indent}{Indentation of \code{NA} values.}

\item{shorten}{How to abbreviate the data if necessary:
\itemize{
\item \code{"back"} (default): add an ellipsis at the end
\item \code{"front"}: add an ellipsis at the front
\item \code{"mid"}: add an ellipsis in the middle
\item \code{"abbreviate"}: use \code{\link[=abbreviate]{abbreviate()}}
}}

\item{short_formatted}{If provided, a character vector of the same length as
\code{formatted}, to be used when the available width is insufficient to show
the full output.}
}
\description{
The \code{new_pillar_shaft()} constructor creates objects of the \code{"pillar_shaft"}
class.
This is a virtual or abstract class, you must specify the \code{class}
argument.
By convention, this should be a string that starts with \code{"pillar_shaft_"}.
See \code{vignette("extending", package = "tibble")} for usage examples.

This method accepts a vector of arbitrary length and is expected to return an S3 object with the following properties:
\itemize{
\item It has an attribute \code{"width"}
\item It can have an attribute \code{"min_width"}, if missing, \code{"width"} is used
\item It must implement a method \code{format(x, width, ...)} that can be called with any value between \code{min_width} and \code{width}
\item This method must return an object that inherits from \code{character} and has attributes \code{"align"} (with supported values \code{"left"}, \code{"right"}, and \code{"center"}) and \code{"width"}
}

The function \code{\link[=new_pillar_shaft]{new_pillar_shaft()}} returns such an object, and also correctly formats \code{NA} values.
In many cases, the implementation of \code{pillar_shaft.your_class_name()} will format the data as a character vector (using color for emphasis) and simply call \code{new_pillar_shaft()}.
See \code{pillar:::pillar_shaft.numeric} for a code that allows changing the display depending on the available width.

\code{new_pillar_shaft_simple()} provides an implementation of the \code{pillar_shaft}
class suitable for output that has a fixed formatting, which will be
truncated with a continuation character (ellipsis or \code{~}) if it doesn't fit
the available width.
By default, the required width is computed from the natural width of the
\code{formatted} argument.
}
\details{
The \code{formatted} argument may also contain ANSI escapes to change color
or other attributes of the text, provided e.g. by the \pkg{cli} package.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ctl_pillar_component.R
\name{new_pillar_component}
\alias{new_pillar_component}
\alias{pillar_component}
\title{Components of a pillar}
\usage{
new_pillar_component(x, ..., width, min_width = NULL)

pillar_component(x)
}
\arguments{
\item{x}{A bare list of length one (for \code{new_pillar_component()}),
or an object with \code{"width"} and \code{"min_width"} attributes
(for \code{pillar_component()}).}

\item{...}{These dots are for future extensions and must be empty.}

\item{width, min_width}{Width and minimum width for the new component.
If \code{min_width} is \code{NULL}, it is assumed to match \code{width}.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

\code{new_pillar_component()} constructs an object of class \code{"pillar_component"}.
It is used by custom \code{\link[=ctl_new_pillar]{ctl_new_pillar()}} methods to create pillars with
nonstandard components.

\code{pillar_component()} is a convenience helper that wraps the input in a list
and extracts width and minimum width.
}
\details{
Objects of class \code{"pillar"} are internally a named lists of their components.
The default components for pillars created by \code{\link[=pillar]{pillar()}} are:
\code{title} (may be missing), \code{type}, and \code{data}.
Each component is a \code{"pillar_component"} object.

This class captures contents that can be fitted in a simple column.
Compound columns are represented by multiple pillar objects, each with their
own components.
}
\examples{
new_pillar_component(list(letters[1:3]), width = 1)
pillar_component(new_pillar_title("letters"))
pillar_component(new_pillar_type(letters))
pillar_component(pillar_shaft(letters[1:3]))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multi.R
\name{extra_cols}
\alias{extra_cols}
\alias{extra_cols.pillar_squeezed_colonnade}
\title{Retrieve information about columns that didn't fit the available width}
\usage{
extra_cols(x, ...)

\method{extra_cols}{pillar_squeezed_colonnade}(x, ..., n = Inf)
}
\arguments{
\item{x}{The result of \code{\link[=squeeze]{squeeze()}} on a \link{colonnade} object}

\item{...}{Arguments passed to methods.}

\item{n}{The number of extra columns to return; the returned vector will
always contain as many elements as there are extra columns, but elements
beyond \code{n} will be \code{NA}.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#soft-deprecated}{\figure{lifecycle-soft-deprecated.svg}{options: alt='[Soft-deprecated]'}}}{\strong{[Soft-deprecated]}}

Formatting a \link{colonnade} object may lead to some columns being omitted
due to width restrictions. This method returns a character vector that
describes each of the omitted columns.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multi.R
\name{squeeze}
\alias{squeeze}
\title{Squeeze a colonnade to a fixed width}
\usage{
squeeze(x, width = NULL, ...)
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#soft-deprecated}{\figure{lifecycle-soft-deprecated.svg}{options: alt='[Soft-deprecated]'}}}{\strong{[Soft-deprecated]}}

The \code{squeeze()} function usually doesn't need to be called manually.
It returns an object suitable for printing and formatting at a fixed width
with additional information about omitted columns, which can be retrieved
via \code{\link[=extra_cols]{extra_cols()}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/char.R
\name{char}
\alias{char}
\alias{set_char_opts}
\title{Format a character vector in a tibble}
\usage{
char(
  x,
  ...,
  min_chars = NULL,
  shorten = c("back", "front", "mid", "abbreviate")
)

set_char_opts(
  x,
  ...,
  min_chars = NULL,
  shorten = c("back", "front", "mid", "abbreviate")
)
}
\description{
These functions are reexported as \code{\link[tibble:char]{tibble::char()}} and \code{\link[tibble:char]{tibble::set_char_opts()}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type-sum.R
\name{type_sum}
\alias{type_sum}
\alias{obj_sum}
\title{Provide a succinct summary of an object}
\usage{
type_sum(x)

obj_sum(x)
}
\arguments{
\item{x}{an object to summarise. Generally only methods of atomic vectors
and variants have been implemented.}
}
\description{
\ifelse{html}{\out{<a href='https://www.tidyverse.org/lifecycle/#questioning'><img src='figures/lifecycle-questioning.svg' alt='Questioning lifecycle'></a>}}{\strong{Questioning}}

\code{type_sum()} gives a brief summary of object type. Objects that commonly
occur in a data frame should return a string with four or less characters.
For most inputs, the argument is forwarded to \code{\link[vctrs:vec_ptype_full]{vctrs::vec_ptype_abbr()}}.

\code{obj_sum()} also includes the size of the object if \code{vctrs::vec_is()}
is \code{TRUE}.
}
\section{Lifecycle}{

\code{type_sum()} is in the "questioning" stage, because \code{\link[vctrs:vec_ptype_full]{vctrs::vec_ptype_abbr()}}
provides essentially the same functionality based on a robust framework.
The generic will remain available for a while.
Users and implementers are encouraged to consider \code{vctrs::vec_ptype_abbr()}
once it becomes stable.
}

\examples{
obj_sum(1:10)
obj_sum(matrix(1:10))
obj_sum(Sys.Date())
obj_sum(Sys.time())
obj_sum(mean)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ornament.R
\name{new_ornament}
\alias{new_ornament}
\title{Helper to define the contents of a pillar}
\usage{
new_ornament(x, width = NULL, align = NULL)
}
\arguments{
\item{x}{A character vector with formatting,
can use ANYI styles e.g provided by the \pkg{cli} package.}

\item{width}{An optional width of the resulting pillar, computed from \code{x} if
missing}

\item{align}{Alignment, one of \code{"left"} or \code{"right"}}
}
\description{
This function is useful if your data renders differently depending on the
available width. In this case, implement the \code{\link[=pillar_shaft]{pillar_shaft()}} method for your
class to return a subclass of "pillar_shaft" and have the \code{\link[=format]{format()}} method
for this subclass call \code{new_ornament()}. See the implementation of
\code{pillar_shaft.numeric()} and \code{format.pillar_shaft_decimal()} for an example.
}
\examples{
new_ornament(c("abc", "de"), align = "right")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tbl-format-footer.R
\name{tbl_format_footer}
\alias{tbl_format_footer}
\title{Format the footer of a tibble}
\usage{
tbl_format_footer(x, setup, ...)
}
\arguments{
\item{x}{A tibble-like object.}

\item{setup}{A setup object returned from \code{\link[=tbl_format_setup]{tbl_format_setup()}}.}

\item{...}{These dots are for future extensions and must be empty.}
}
\value{
A character vector.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

For easier customization, the formatting of a tibble is split
into three components: header, body, and footer.
The \code{tbl_format_footer()} method is responsible for formatting the footer
of a tibble.

Override or extend this method if you need to change the appearance
of the footer.
The default implementation adds information about rows and columns
that are not shown in the body.
}
\examples{
\dontshow{if (rlang::is_installed(c("palmerpenguins", "tibble"))) withAutoprint(\{ # examplesIf}
setup <- tbl_format_setup(palmerpenguins::penguins)
tbl_format_footer(palmerpenguins::penguins, setup)

# Shortcut for debugging
tbl_format_footer(setup)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tbl-format-footer.R
\name{tbl_format_footer}
\alias{tbl_format_footer}
\title{Format the footer of a tibble}
\usage{
tbl_format_footer(x, setup, ...)
}
\arguments{
\item{x}{A tibble-like object.}

\item{setup}{A setup object returned from \code{\link[=tbl_format_setup]{tbl_format_setup()}}.}

\item{...}{These dots are for future extensions and must be empty.}
}
\value{
A character vector.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

For easier customization, the formatting of a tibble is split
into three components: header, body, and footer.
The \code{tbl_format_footer()} method is responsible for formatting the footer
of a tibble.

Override or extend this method if you need to change the appearance
of the footer.
The default implementation adds information about rows and columns
that are not shown in the body.
}
\examples{
\dontshow{if (rlang::is_installed("palmerpenguins")) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
setup <- tbl_format_setup(palmerpenguins::penguins)
tbl_format_footer(palmerpenguins::penguins, setup)

# Shortcut for debugging
tbl_format_footer(setup)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extent.R
\name{align}
\alias{align}
\title{Alignment helper}
\usage{
align(x, width = NULL, align = c("left", "right"), space = " ")
}
\arguments{
\item{x}{A character vector}

\item{width}{The width that each string is padded to. If \code{NULL}, the maximum
display width of the character vector is used (see \code{\link[=get_max_extent]{get_max_extent()}}).}

\item{align}{How should strings be aligned? If \code{align = left} then padding
appears on the \code{right}, and vice versa.}

\item{space}{What character should be used for the padding?}
}
\description{
Facilitates easy alignment of strings within a character vector. Designed to
help implementers of formatters for custom data types.
}
\examples{
align(c("abc", "de"), align = "left")
align(c("abc", "de"), align = "right")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type-sum.R
\name{type_sum}
\alias{type_sum}
\alias{obj_sum}
\alias{size_sum}
\title{Provide a succinct summary of an object}
\usage{
type_sum(x)

obj_sum(x)

size_sum(x)
}
\arguments{
\item{x}{an object to summarise. Generally only methods of atomic vectors
and variants have been implemented.}
}
\description{
\code{type_sum()} gives a brief summary of object type. Objects that commonly
occur in a data frame should return a string with four or less characters.
For most inputs, the argument is forwarded to \code{\link[vctrs:vec_ptype_full]{vctrs::vec_ptype_abbr()}}.

\code{obj_sum()} also includes the size (but not the shape) of the object
if \code{\link[vctrs:vec_assert]{vctrs::vec_is()}} is \code{TRUE}.
It should always return a string (a character vector of length one).
As of pillar v1.6.1, the default method forwards to \code{\link[vctrs:vec_ptype_full]{vctrs::vec_ptype_abbr()}}
for vectors and to \code{\link[=type_sum]{type_sum()}} for other objects.
Previous versions always forwarded to \code{\link[=type_sum]{type_sum()}}.
An attribute named \code{"short"} in the return value will be picked up by
the \code{\link[=pillar_shaft]{pillar_shaft()}} method for lists, and used if space is limited.

\code{size_sum()} is called by \code{obj_sum()} to format the size of the object.
It should always return a string (a character vector of length one),
it can be an empty string \code{""} to omit size information,
this is what the default method does for scalars.
}
\details{
When formatting a pillar,
\code{type_sum()} will be called on a slice of the column vector.
The formatted type should only depend on the type and not on the data,
to avoid confusion.
}
\examples{
obj_sum(1:10)
obj_sum(matrix(1:10))
obj_sum(data.frame(a = 1))
obj_sum(Sys.Date())
obj_sum(Sys.time())
obj_sum(mean)

size_sum(1:10)
size_sum(trees)
size_sum(Titanic)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/testthat.R
\name{expect_known_display}
\alias{expect_known_display}
\title{Test helpers}
\usage{
expect_known_display(object, file, ..., width = 80L, crayon = TRUE)
}
\arguments{
\item{object}{An object to check.}

\item{file}{File path where known value/output will be stored.}

\item{...}{Unused.}

\item{width}{The width of the output.}

\item{crayon}{Color the output?}
}
\description{
Expectation for packages that implement a data type with pillar support.
Allows storing the desired result in a file,
and comparing the output with the file contents.
Note that this expectation sets options that affect the formatting of the
pillar, see examples for usage.
}
\examples{
file <- tempfile("pillar", fileext = ".txt")

# The pillar is constructed after options have been set
# (need two runs because reference file doesn't exist during the first run)
suppressWarnings(tryCatch(
  expect_known_display(pillar(1:3), file, crayon = FALSE),
  expectation_failure = function(e) {}
))
expect_known_display(pillar(1:3), file, crayon = FALSE)

# Good: Use tidyeval to defer construction
pillar_quo <- rlang::quo(pillar(1:3))
expect_known_display(!!pillar_quo, file, crayon = FALSE)

\dontrun{
# Bad: Options set in the active session may affect the display
integer_pillar <- pillar(1:3)
expect_known_display(integer_pillar, file, crayon = FALSE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multi.R
\name{colonnade}
\alias{colonnade}
\title{Format multiple vectors in a tabular display}
\usage{
colonnade(x, has_row_id = TRUE, width = NULL, ...)
}
\arguments{
\item{x}{A list, which can contain matrices or data frames.
If named, the names will be used as title for the pillars. Non-syntactic names
will be escaped.}

\item{has_row_id}{Include a column indicating row IDs? Pass \code{"*"} to mark
the row ID column with a star.}

\item{width}{Default width of the entire output, optional.}

\item{...}{These dots are for future extensions and must be empty.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#soft-deprecated}{\figure{lifecycle-soft-deprecated.svg}{options: alt='[Soft-deprecated]'}}}{\strong{[Soft-deprecated]}}

The vectors are formatted to fit horizontally into a user-supplied number of
characters per row.

The \code{colonnade()} function doesn't process the input but returns an object
with a \code{\link[=format]{format()}} and a \code{\link[=print]{print()}} method.
The implementations call \code{\link[=squeeze]{squeeze()}} to create \link{pillar} objects and fit them to a given width.
}
\details{
Pillars may be distributed over multiple tiers if
\code{width > getOption("width")}. In this case each tier is at most
\code{getOption("width")} characters wide. The very first step of formatting
is to determine how many tiers are shown at most, and the width of each
tier.

To avoid unnecessary computation for showing very wide colonnades, a first
pass tries to fit all capitals into the tiers.
For each pillar whose capital fits, it is then decided in which tier it is
shown, if at all, and how much horizontal space it may use (either its
minimum or its maximum width).
Remaining space is then distributed proportionally to pillars that do not
use their desired width.

For fitting pillars in one or more tiers, first a check is made
if all pillars fit with their maximum width (e.g.,
\code{option(tibble.width = Inf)} or narrow colonnade).
If yes, this is the resulting fit, no more work needs to be done.
Otherwise, if the maximum width is too wide, the same test
is carried out with the minimum width.
If this is still too wide, this is the resulting fit.
Otherwise, some tiers from the start
will contain pillars with their maximum width,
one tier will contain some pillars with maximum and some with minimum width,
and the remaining tiers contain pillars with their minimum width only.

For this, we compute a "reverse minimum assignment".

We determine the cut point where minimum and maximum assignment
agree.
The following strategy is applied:
\enumerate{
\item First, we determine the tier in which the cut point lies.
This is the first instance of a column that ends up in the same tier
for both minimum and maximum assignment.
\item A set of candidate cut points is derived.
\item We consult the column offsets. The last column where the minimum assignment
has a greater or equal offset than the maximum assignment is our latest
cut point.
If no such column exists, the cut point is the column just before our
first candidate.
\item Finally, we combine maximum and minimum reverse fits at the cut point.
We don't need to redistribute anything here.
}

Fitting pillars into tiers is very similar to a word-wrapping algorithm.
In a loop, new tiers are opened if the current tier overflows.
If a column is too wide to fit a single tier, it will never be
displayed, and the colonnade will be truncated there.
This case should never occur with reasonable display widths larger than
30 characters.
Truncation also happens if all available tiers are filled.

The remaining space is distributed from left to right.
Each column gains space proportional to the fraction of missing and
remaining space,
rounded down.
Any space remaining after rounding is distributed from left to right,
one space per column.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/num.R
\name{num}
\alias{num}
\alias{set_num_opts}
\title{Format a numeric vector in a tibble}
\usage{
num(
  x,
  ...,
  sigfig = NULL,
  digits = NULL,
  label = NULL,
  scale = NULL,
  notation = c("fit", "dec", "sci", "eng", "si"),
  fixed_exponent = NULL,
  extra_sigfig = NULL
)

set_num_opts(
  x,
  ...,
  sigfig = NULL,
  digits = NULL,
  label = NULL,
  scale = NULL,
  notation = c("fit", "dec", "sci", "eng", "si"),
  fixed_exponent = NULL,
  extra_sigfig = NULL
)
}
\description{
These functions are reexported as \code{\link[tibble:num]{tibble::num()}} and \code{\link[tibble:num]{tibble::set_num_opts()}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ctl_pillar.R
\name{new_pillar}
\alias{new_pillar}
\title{Construct a custom pillar object}
\usage{
new_pillar(components, ..., width = NULL, class = NULL, extra = deprecated())
}
\arguments{
\item{components}{A named list of components constructed with \code{\link[=pillar_component]{pillar_component()}}.}

\item{...}{These dots are for future extensions and must be empty.}

\item{width}{Default width, optional.}

\item{class}{Name of subclass.}

\item{extra}{Deprecated.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

\code{new_pillar()} is the low-level constructor for pillar objects.
It supports arbitrary components.
See \code{\link[=pillar]{pillar()}} for the high-level constructor with default components.
}
\details{
Arbitrary components are supported.
If your tibble subclass needs more or different components in its pillars,
override or extend \code{\link[=ctl_new_pillar]{ctl_new_pillar()}} and perhaps \code{\link[=ctl_new_pillar_list]{ctl_new_pillar_list()}}.
}
\examples{
lines <- function(char = "-") {
  stopifnot(nchar(char) == 1)
  structure(char, class = "lines")
}

format.lines <- function(x, width, ...) {
  paste(rep(x, width), collapse = "")
}

new_pillar(list(
  title = pillar_component(new_ornament(c("abc", "de"), align = "right")),
  lines = new_pillar_component(list(lines("=")), width = 1)
))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sigfig.R, R/styles.R
\name{style_num}
\alias{style_num}
\alias{style_subtle}
\alias{style_subtle_num}
\alias{style_bold}
\alias{style_na}
\alias{style_neg}
\title{Styling helpers}
\usage{
style_num(x, negative, significant = rep_along(x, TRUE))

style_subtle(x)

style_subtle_num(x, negative)

style_bold(x)

style_na(x)

style_neg(x)
}
\arguments{
\item{x}{The character vector to style.}

\item{negative, significant}{Logical vector the same length as \code{x} that
indicate if the values are negative and significant, respectively}
}
\description{
Functions that allow implementers of formatters for custom data types to
maintain a consistent style with the default data types.
}
\details{
\code{style_subtle()} is affected by the \code{subtle} \link[=pillar_options]{option}.

\code{style_subtle_num()} is affected by the
\code{subtle_num} \link[=pillar_options]{option},
which is \code{FALSE} by default.

\code{style_bold()} is affected by the \code{bold} \link[=pillar_options]{option},
which is \code{FALSE} by default.

\code{style_neg()} is affected by the \code{pillar.neg} \link[=pillar_options]{option}.
}
\examples{
style_num(
  c("123", "456"),
  negative = c(TRUE, FALSE)
)
style_num(
  c("123", "456"),
  negative = c(TRUE, FALSE),
  significant = c(FALSE, FALSE)
)
style_subtle("text")
style_subtle_num(0.01 * 1:3, c(TRUE, FALSE, TRUE))
style_bold("Petal.Width")
style_na("NA")
style_neg("123")
}
\seealso{
\link{pillar_options} for a list of options
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pillar.R
\name{pillar}
\alias{pillar}
\title{Format a vector suitable for tabular display}
\usage{
pillar(x, title = NULL, width = NULL, ...)
}
\arguments{
\item{x}{A vector to format.}

\item{title}{An optional title for the column. The title will be
used "as is", no quoting will be applied.}

\item{width}{Default width, optional.}

\item{...}{Passed on to \code{\link[=pillar_shaft]{pillar_shaft()}}.}
}
\description{
\ifelse{html}{\out{<a href='https://www.tidyverse.org/lifecycle/#stable'><img src='figures/lifecycle-stable.svg' alt='Stable lifecycle'></a>}}{\strong{Stable}}

\code{pillar()} formats a vector using one row for a title (if given),
one row for the type, and \code{length(x)} rows for the data.
}
\details{
A pillar consists of a \emph{capital} and a \emph{shaft}.

The capital is constructed using the (currently internal)
\code{pillar_capital()} function, which uses the \code{title} argument
and calls \code{\link[=type_sum]{type_sum()}} to format the type.

For the shaft, the \code{\link[=pillar_shaft]{pillar_shaft()}} generic is called with the object.
The returned value is stored and processed with \code{\link[=format]{format()}} when displaying the pillar.
The call to \code{format()} has a valid \code{width} argument.
Depending on the implementation,
the output representation can be computed eagerly right away (as done with \code{\link[=new_pillar_shaft_simple]{new_pillar_shaft_simple()}}),
or only when \code{format()} is called.
The latter allows for adaptive shortening of the output depending on the available width,
see \code{pillar:::pillar_shaft.numeric} for an example.
}
\examples{
x <- 123456789 * (10 ^ c(-1, -3, -5, NA, -8, -10))
pillar(x)
pillar(-x)
pillar(runif(10))
pillar(rcauchy(20))

# Special values are highlighted
pillar(c(runif(5), NA, NaN, Inf, -Inf))

# Very wide ranges will be displayed in scientific format
pillar(c(1e10, 1e-10), width = 20)
pillar(c(1e10, 1e-10))

x <- c(FALSE, NA, FALSE, FALSE, TRUE, FALSE, FALSE, TRUE, FALSE, TRUE)
pillar(x)

x <- c("This is string is rather long", NA, "?", "Short")
pillar(x)
pillar(x, width = 30)
pillar(x, width = 5)

date <- as.Date("2017-05-15")
pillar(date + c(1, NA, 3:5))
pillar(as.POSIXct(date) + c(30, NA, 600, 3600, 86400))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tbl-format-setup.R
\name{tbl_format_setup}
\alias{tbl_format_setup}
\alias{tbl_format_setup.tbl}
\title{Set up formatting}
\usage{
tbl_format_setup(
  x,
  width = NULL,
  ...,
  setup = list(tbl_sum = tbl_sum(x)),
  n = NULL,
  max_extra_cols = NULL,
  max_footer_lines = NULL,
  focus = NULL
)

\method{tbl_format_setup}{tbl}(
  x,
  width,
  ...,
  setup,
  n,
  max_extra_cols,
  max_footer_lines,
  focus
)
}
\arguments{
\item{x}{An object.}

\item{width}{Actual width for printing, a numeric greater than zero.
This argument is mandatory for all implementations of this method.}

\item{...}{Extra arguments to \code{\link[=print.tbl]{print.tbl()}} or \code{\link[=format.tbl]{format.tbl()}}.}

\item{setup}{This generic is first called with \code{setup = NULL} .
If the method \emph{evaluates} this argument, the return value
will only be used in a call to \code{\link[=tbl_format_header]{tbl_format_header()}},
and after that, a second call to this generic will be made
with the return value of the first call as \code{setup}
which then will be used in calls to \code{\link[=tbl_format_body]{tbl_format_body()}} and \code{\link[=tbl_format_footer]{tbl_format_footer()}}.
This allows displaying the header before starting the computation
required for the body and footer.}

\item{n}{Actual number of rows to print.
No \link[=pillar_options]{options} should be considered
by implementations of this method.}

\item{max_extra_cols}{Number of columns to print abbreviated information for,
if the width is too small for the entire tibble.
No \link[=pillar_options]{options} should be considered
by implementations of this method.}

\item{max_footer_lines}{Maximum number of lines for the footer.
No \link[=pillar_options]{options} should be considered
by implementations of this method.}

\item{focus}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Names of columns to show preferentially if space is tight.}
}
\value{
An object that can be passed as \code{setup} argument to
\code{\link[=tbl_format_header]{tbl_format_header()}}, \code{\link[=tbl_format_body]{tbl_format_body()}}, and \code{\link[=tbl_format_footer]{tbl_format_footer()}}.
}
\description{
\code{tbl_format_setup()} is called by \code{\link[=format.tbl]{format.tbl()}}.
This method collects information that is common to the header, body,
and footer parts of a tibble.
Examples:
\itemize{
\item the dimensions sometimes are reported both in the header
and (implicitly) in the footer of a tibble;
\item the columns shown in the body decide which columns are shown in the footer.
}

This information is computed in \code{tbl_format_setup()}.
The result is passed on to the
\code{\link[=tbl_format_header]{tbl_format_header()}}, \code{\link[=tbl_format_body]{tbl_format_body()}}, and \code{\link[=tbl_format_footer]{tbl_format_footer()}}
methods.
If you need to customize parts of the printed output independently,
override these methods instead.

By checking the \code{setup} argument, you can return an object that is
suitable for a call to \code{\link[=tbl_format_header]{tbl_format_header()}} if \code{setup} is \code{NULL}.
In this case, the method is called a second time with the return value
of the first call as \code{setup}.
}
\details{
Extend this method to prepare information that is used
in several parts of the printed output of a tibble-like object,
or to collect additional arguments passed via \code{...} to
\code{\link[=print.tbl]{print.tbl()}} or \code{\link[=format.tbl]{format.tbl()}}.

We expect that \code{tbl_format_setup()} is extended only rarely,
and overridden only in exceptional circumstances, if at all.
If you override this method, you must also implement
\code{\link[=tbl_format_header]{tbl_format_header()}}, \code{\link[=tbl_format_body]{tbl_format_body()}}, and \code{\link[=tbl_format_footer]{tbl_format_footer()}}
for your class.

Implementing a method
allows to override printing and formatting of the entire object
without overriding the \code{\link[=print]{print()}} and \code{\link[=format]{format()}} methods directly.
This allows to keep the logic of the \code{width} and \code{n} arguments.

The default method for the \code{"tbl"} class collects information for
standard printing for tibbles.
See \code{\link[=new_tbl_format_setup]{new_tbl_format_setup()}} for details on the returned object.
}
\examples{
\dontshow{if (rlang::is_installed(c("palmerpenguins", "tibble"))) withAutoprint(\{ # examplesIf}
tbl_format_setup(palmerpenguins::penguins)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tbl-format-setup.R
\name{tbl_format_setup}
\alias{tbl_format_setup}
\alias{tbl_format_setup.tbl}
\title{Set up formatting}
\usage{
tbl_format_setup(
  x,
  width = NULL,
  ...,
  n = NULL,
  max_extra_cols = NULL,
  max_footer_lines = NULL,
  focus = NULL
)

\method{tbl_format_setup}{tbl}(x, width, ..., n, max_extra_cols, max_footer_lines, focus)
}
\arguments{
\item{x}{An object.}

\item{width}{Actual width for printing, a numeric greater than zero.
This argument is mandatory for all implementations of this method.}

\item{...}{Extra arguments to \code{\link[=print.tbl]{print.tbl()}} or \code{\link[=format.tbl]{format.tbl()}}.}

\item{n}{Actual number of rows to print.
No \link[=pillar_options]{options} should be considered
by implementations of this method.}

\item{max_extra_cols}{Number of columns to print abbreviated information for,
if the width is too small for the entire tibble.
No \link[=pillar_options]{options} should be considered
by implementations of this method.}

\item{max_footer_lines}{Maximum number of lines for the footer.
No \link[=pillar_options]{options} should be considered
by implementations of this method.}

\item{focus}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Names of columns to show preferentially if space is tight.}
}
\value{
An object that can be passed as \code{setup} argument to
\code{\link[=tbl_format_header]{tbl_format_header()}}, \code{\link[=tbl_format_body]{tbl_format_body()}}, and \code{\link[=tbl_format_footer]{tbl_format_footer()}}.
}
\description{
\code{tbl_format_setup()} is called by \code{\link[=format.tbl]{format.tbl()}}.
This method collects information that is common to the header, body,
and footer parts of a tibble.
Examples:
\itemize{
\item the dimensions sometimes are reported both in the header
and (implicitly) in the footer of a tibble;
\item the columns shown in the body decide which columns are shown in the footer.
}

This information is computed once in \code{tbl_format_setup()}.
The result is passed on to the
\code{\link[=tbl_format_header]{tbl_format_header()}}, \code{\link[=tbl_format_body]{tbl_format_body()}}, and \code{\link[=tbl_format_footer]{tbl_format_footer()}}
methods.
If you need to customize parts of the printed output independently,
override these methods instead.
}
\details{
Extend this method to prepare information that is used
in several parts of the printed output of a tibble-like object,
or to collect additional arguments passed via \code{...} to
\code{\link[=print.tbl]{print.tbl()}} or \code{\link[=format.tbl]{format.tbl()}}.

We expect that \code{tbl_format_setup()} is extended only rarely,
and overridden only in exceptional circumstances, if at all.
If you override this method, you must also implement
\code{\link[=tbl_format_header]{tbl_format_header()}}, \code{\link[=tbl_format_body]{tbl_format_body()}}, and \code{\link[=tbl_format_footer]{tbl_format_footer()}}
for your class.

Implementing a method
allows to override printing and formatting of the entire object
without overriding the \code{\link[=print]{print()}} and \code{\link[=format]{format()}} methods directly.
This allows to keep the logic of the \code{width} and \code{n} arguments.

The default method for the \code{"tbl"} class collects information for
standard printing for tibbles.
See \code{\link[=new_tbl_format_setup]{new_tbl_format_setup()}} for details on the returned object.
}
\examples{
\dontshow{if (rlang::is_installed("palmerpenguins")) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
tbl_format_setup(palmerpenguins::penguins)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{deprecated}
\alias{deprecated}
\alias{is_vector_s3}
\title{Deprecated functions}
\usage{
is_vector_s3(x)
}
\description{
\code{is_vector_s3()} returns \code{TRUE} if an object represents an S3 vector.
The default implementation forwards to \code{vctrs::vec_is()}.
}
\section{Lifecycle}{

\code{is_vector_s3()} is soft-deprecated, because \code{\link[vctrs:vec_assert]{vctrs::vec_is()}}
provides the same functionality based on a robust framework.
The generic will remain available for a while but is no longer used.
Users and implementers are encouraged to ensure that \code{vctrs::vec_is()}
is \code{TRUE} for their class.
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type.R
\name{format_type_sum}
\alias{format_type_sum}
\alias{format_type_sum.default}
\alias{format_type_sum.AsIs}
\title{Format a type summary}
\usage{
format_type_sum(x, width, ...)

\method{format_type_sum}{default}(x, width, ...)

\method{format_type_sum}{AsIs}(x, width, ...)
}
\arguments{
\item{x}{A return value from \code{type_sum()}}

\item{width}{The desired total width. If the returned string still is
wider, it will be trimmed. Can be \code{NULL}.}

\item{...}{Arguments passed to methods.}
}
\description{
Called on values returned from \code{\link[=type_sum]{type_sum()}} for defining the description
in the capital.
}
\details{
Two methods are implemented by default for this generic: the default method,
and the method for the \code{"AsIs"} class.
Return \code{I("type")} from your \code{\link[=type_sum]{type_sum()}} implementation to format the type
without angle brackets.
For even more control over the formatting, implement your own method.
}
\examples{
# Default method: show the type with angle brackets
writeLines(format_type_sum("dbl", width = NULL))
pillar(1)

# AsIs method: show the type without angle brackets
type_sum.accel <- function(x) {
  I("kg m/s^2")
}

# Typically done through NAMESPACE
# (perhaps with an @export directive in roxygen2)
registerS3method("type_sum", "accel", type_sum.accel)

accel <- structure(9.81, class = "accel")
pillar(accel)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type.R
\name{format_type_sum}
\alias{format_type_sum}
\alias{format_type_sum.default}
\alias{format_type_sum.AsIs}
\title{Format a type summary}
\usage{
format_type_sum(x, width, ...)

\method{format_type_sum}{default}(x, width, ...)

\method{format_type_sum}{AsIs}(x, width, ...)
}
\arguments{
\item{x}{A return value from \code{type_sum()}}

\item{width}{The desired total width. If the returned string still is
wider, it will be trimmed. Can be \code{NULL}.}

\item{...}{Arguments passed to methods.}
}
\description{
Called on values returned from \code{\link[=type_sum]{type_sum()}} for defining the description
in the capital.
}
\details{
Two methods are implemented by default for this generic: the default method,
and the method for the \code{"AsIs"} class.
Return \code{I("type")} from your \code{\link[=type_sum]{type_sum()}} implementation to format the type
without angle brackets.
For even more control over the formatting, implement your own method.
}
\examples{
# Default method: show the type with angle brackets
format_type_sum(1, NULL)
pillar(1)

# AsIs method: show the type without angle brackets
type_sum.accel <- function(x) {
  I("kg m/s^2")
}

accel <- structure(9.81, class = "accel")
pillar(accel)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glimpse.R
\name{format_glimpse}
\alias{format_glimpse}
\title{Format a vector for horizontal printing}
\usage{
format_glimpse(x, ...)
}
\arguments{
\item{x}{A vector.}

\item{...}{Arguments passed to methods.}
}
\value{
A character vector of the same length as \code{x}.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

This generic provides the logic for printing vectors in \code{\link[=glimpse]{glimpse()}}.

The output strives to be as unambiguous as possible,
without compromising on readability.
In a list, to distinguish between vectors and nested lists,
the latter are surrounded by \verb{[]} brackets.
Empty lists are shown as \verb{[]}.
Vectors inside lists, of length not equal to one,
are surrounded by \verb{<>} angle brackets.
Empty vectors are shown as \verb{<>}.
}
\examples{
format_glimpse(1:3)

# Lists use [], vectors inside lists use <>
format_glimpse(list(1:3))
format_glimpse(list(1, 2:3))
format_glimpse(list(list(1), list(2:3)))
format_glimpse(list(as.list(1), as.list(2:3)))
format_glimpse(list(character()))
format_glimpse(list(NULL))

# Character strings are always quoted
writeLines(format_glimpse(letters[1:3]))
writeLines(format_glimpse(c("A", "B, C")))

# Factors are quoted only when needed
writeLines(format_glimpse(factor(letters[1:3])))
writeLines(format_glimpse(factor(c("A", "B, C"))))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tbl-format-header.R
\name{tbl_format_header}
\alias{tbl_format_header}
\title{Format the header of a tibble}
\usage{
tbl_format_header(x, setup, ...)
}
\arguments{
\item{x}{A tibble-like object.}

\item{setup}{A setup object returned from \code{\link[=tbl_format_setup]{tbl_format_setup()}}.}

\item{...}{These dots are for future extensions and must be empty.}
}
\value{
A character vector.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

For easier customization, the formatting of a tibble is split
into three components: header, body, and footer.
The \code{tbl_format_header()} method is responsible for formatting the header
of a tibble.

Override this method if you need to change the appearance
of the entire header.
If you only need to change or extend the components shown in the header,
override or extend \code{\link[=tbl_sum]{tbl_sum()}} for your class which is called by the
default method.
}
\examples{
\dontshow{if (rlang::is_installed("palmerpenguins")) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
setup <- tbl_format_setup(palmerpenguins::penguins)
tbl_format_header(palmerpenguins::penguins, setup)

# Shortcut for debugging
tbl_format_header(setup)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tbl-format-header.R
\name{tbl_format_header}
\alias{tbl_format_header}
\title{Format the header of a tibble}
\usage{
tbl_format_header(x, setup, ...)
}
\arguments{
\item{x}{A tibble-like object.}

\item{setup}{A setup object returned from \code{\link[=tbl_format_setup]{tbl_format_setup()}}.}

\item{...}{These dots are for future extensions and must be empty.}
}
\value{
A character vector.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

For easier customization, the formatting of a tibble is split
into three components: header, body, and footer.
The \code{tbl_format_header()} method is responsible for formatting the header
of a tibble.

Override this method if you need to change the appearance
of the entire header.
If you only need to change or extend the components shown in the header,
override or extend \code{\link[=tbl_sum]{tbl_sum()}} for your class which is called by the
default method.
}
\examples{
\dontshow{if (rlang::is_installed(c("palmerpenguins", "tibble"))) withAutoprint(\{ # examplesIf}
setup <- tbl_format_setup(palmerpenguins::penguins)
tbl_format_header(palmerpenguins::penguins, setup)

# Shortcut for debugging
tbl_format_header(setup)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tbl-format-body.R
\name{tbl_format_body}
\alias{tbl_format_body}
\title{Format the body of a tibble}
\usage{
tbl_format_body(x, setup, ...)
}
\arguments{
\item{x}{A tibble-like object.}

\item{setup}{A setup object returned from \code{\link[=tbl_format_setup]{tbl_format_setup()}}.}

\item{...}{These dots are for future extensions and must be empty.}
}
\value{
A character vector.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

For easier customization, the formatting of a tibble is split
into three components: header, body, and footer.
The \code{tbl_format_body()} method is responsible for formatting the body
of a tibble.

Override this method if you need to change the appearance of all parts
of the body.
If you only need to change the appearance of a single data type,
override \code{\link[vctrs:vec_ptype_full]{vctrs::vec_ptype_abbr()}} and \code{\link[=pillar_shaft]{pillar_shaft()}} for this data type.
}
\examples{
\dontshow{if (rlang::is_installed("palmerpenguins")) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
setup <- tbl_format_setup(palmerpenguins::penguins)
tbl_format_body(palmerpenguins::penguins, setup)

# Shortcut for debugging
tbl_format_body(setup)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tbl-format-body.R
\name{tbl_format_body}
\alias{tbl_format_body}
\title{Format the body of a tibble}
\usage{
tbl_format_body(x, setup, ...)
}
\arguments{
\item{x}{A tibble-like object.}

\item{setup}{A setup object returned from \code{\link[=tbl_format_setup]{tbl_format_setup()}}.}

\item{...}{These dots are for future extensions and must be empty.}
}
\value{
A character vector.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

For easier customization, the formatting of a tibble is split
into three components: header, body, and footer.
The \code{tbl_format_body()} method is responsible for formatting the body
of a tibble.

Override this method if you need to change the appearance of all parts
of the body.
If you only need to change the appearance of a single data type,
override \code{\link[vctrs:vec_ptype_full]{vctrs::vec_ptype_abbr()}} and \code{\link[=pillar_shaft]{pillar_shaft()}} for this data type.
}
\examples{
\dontshow{if (rlang::is_installed(c("palmerpenguins", "tibble"))) withAutoprint(\{ # examplesIf}
setup <- tbl_format_setup(palmerpenguins::penguins)
tbl_format_body(palmerpenguins::penguins, setup)

# Shortcut for debugging
tbl_format_body(setup)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glimpse.R
\name{glimpse}
\alias{glimpse}
\title{Get a glimpse of your data}
\usage{
glimpse(x, width = NULL, ...)
}
\arguments{
\item{x}{An object to glimpse at.}

\item{width}{Width of output: defaults to the setting of the
\code{width} \link[=pillar_options]{option} (if finite)
or the width of the console.}

\item{...}{Unused, for extensibility.}
}
\value{
x original x is (invisibly) returned, allowing \code{glimpse()} to be
used within a data pipe line.
}
\description{
\code{glimpse()} is like a transposed version of \code{print()}:
columns run down the page, and data runs across.
This makes it possible to see every column in a data frame.
It's a little like \code{\link[=str]{str()}} applied to a data frame
but it tries to show you as much data as possible.
(And it always shows the underlying data, even when applied
to a remote data source.)

See \code{\link[=format_glimpse]{format_glimpse()}} for details on the formatting.
}
\section{S3 methods}{

\code{glimpse} is an S3 generic with a customised method for \code{tbl}s and
\code{data.frames}, and a default method that calls \code{\link[=str]{str()}}.
}

\examples{
glimpse(mtcars)

\dontshow{if (rlang::is_installed("nycflights13")) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
glimpse(nycflights13::flights)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glimpse.R
\name{glimpse}
\alias{glimpse}
\title{Get a glimpse of your data}
\usage{
glimpse(x, width = NULL, ...)
}
\arguments{
\item{x}{An object to glimpse at.}

\item{width}{Width of output: defaults to the setting of the
\code{width} \link[=pillar_options]{option} (if finite)
or the width of the console.}

\item{...}{Unused, for extensibility.}
}
\value{
x original x is (invisibly) returned, allowing \code{glimpse()} to be
used within a data pipe line.
}
\description{
\code{glimpse()} is like a transposed version of \code{print()}:
columns run down the page, and data runs across.
This makes it possible to see every column in a data frame.
It's a little like \code{\link[=str]{str()}} applied to a data frame
but it tries to show you as much data as possible.
(And it always shows the underlying data, even when applied
to a remote data source.)

See \code{\link[=format_glimpse]{format_glimpse()}} for details on the formatting.
}
\section{S3 methods}{

\code{glimpse} is an S3 generic with a customised method for \code{tbl}s and
\code{data.frames}, and a default method that calls \code{\link[=str]{str()}}.
}

\examples{
glimpse(mtcars)

\dontshow{if (rlang::is_installed("nycflights13")) withAutoprint(\{ # examplesIf}
glimpse(nycflights13::flights)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pillar-package.R
\docType{package}
\name{pillar-package}
\alias{pillar-package}
\title{pillar: Coloured Formatting for Columns}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#stable}{\figure{lifecycle-stable.svg}{options: alt='[Stable]'}}}{\strong{[Stable]}}

Formats tabular data in columns or rows
using the full range of colours provided by modern terminals.
Provides various generics for making every aspect of the display customizable.
}
\examples{
pillar(1:3)
pillar(c(1, 2, 3))
pillar(factor(letters[1:3]), title = "letters")
tbl_format_setup(tibble::as_tibble(mtcars), width = 60)
}
\seealso{
\itemize{
\item \code{\link[=pillar]{pillar()}} for formatting a single column,
\item \code{\link[=print.tbl]{print.tbl()}} for formatting data-frame-like objects,
\item \link{pillar_options} for a list of package options.
}
}
\author{
\strong{Maintainer}: Kirill Müller \email{krlmlr+r@mailbox.org}

Authors:
\itemize{
  \item Hadley Wickham
}

Other contributors:
\itemize{
  \item RStudio [copyright holder]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pillar-package.R
\docType{package}
\name{pillar-package}
\alias{pillar-package}
\title{pillar: Coloured Formatting for Columns}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#stable}{\figure{lifecycle-stable.svg}{options: alt='[Stable]'}}}{\strong{[Stable]}}

Formats tabular data in columns or rows
using the full range of colours provided by modern terminals.
Provides various generics for making every aspect of the display customizable.
}
\examples{
\dontshow{if (rlang::is_installed("tibble")) withAutoprint(\{ # examplesIf}
pillar(1:3)
pillar(c(1, 2, 3))
pillar(factor(letters[1:3]), title = "letters")
tbl_format_setup(tibble::as_tibble(mtcars), width = 60)
\dontshow{\}) # examplesIf}
}
\seealso{
\itemize{
\item \code{\link[=pillar]{pillar()}} for formatting a single column,
\item \code{\link[=print.tbl]{print.tbl()}} for formatting data-frame-like objects,
\item \link{pillar_options} for a list of package options.
}
}
\author{
\strong{Maintainer}: Kirill Müller \email{kirill@cynkra.com} (\href{https://orcid.org/0000-0002-1416-3412}{ORCID})

Authors:
\itemize{
  \item Hadley Wickham
}

Other contributors:
\itemize{
  \item RStudio [copyright holder]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tbl-format-setup.R
\name{new_tbl_format_setup}
\alias{new_tbl_format_setup}
\title{Construct a setup object for formatting}
\usage{
new_tbl_format_setup(
  x,
  df,
  width,
  tbl_sum,
  body,
  rows_missing,
  rows_total,
  extra_cols,
  extra_cols_total,
  max_footer_lines,
  abbrev_cols
)
}
\arguments{
\item{x}{The input object unchanged.}

\item{df}{A data frame representation of the intended output,
trimmed to the desired number of rows.}

\item{width}{The \code{width} argument unchanged.}

\item{tbl_sum}{A named character vector, as returned from \code{\link[=tbl_sum]{tbl_sum()}}.}

\item{body}{A character vector with the formatted body,
one element per line,}

\item{rows_missing}{The number of rows not shown from the body,
\code{NA} if unknown.}

\item{rows_total}{The total number of rows in the data,
\code{NA} if unknown.}

\item{extra_cols}{Columns that did not fit into the body,
as a character vector of formatted column names and types.}

\item{extra_cols_total}{The total number of columns, may be larger than
\code{length(extra_cols)}.}

\item{max_footer_lines}{The maximum number of lines in the footer.}

\item{abbrev_cols}{Formatted names of the columns that are shown abbreviated
in the body.}
}
\description{
The object returned from the default method of \code{\link[=tbl_format_setup]{tbl_format_setup()}}
is an object with a \code{"class"} attribute and the elements described in the
"Parameters" section.

Named elements can be added to such objects without affecting the behavior.
Do not modify existing elements.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tbl-format-setup.R
\name{tbl_nrow}
\alias{tbl_nrow}
\title{Number of rows in a tbl object}
\usage{
tbl_nrow(x, ...)
}
\arguments{
\item{x}{A tbl object.}

\item{...}{These dots are for future extensions and must be empty.}
}
\description{
This generic will be called by \code{\link[=tbl_format_setup]{tbl_format_setup()}} to determine the number
of rows in a tbl object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shaft-.R
\name{pillar_shaft}
\alias{pillar_shaft}
\alias{print.pillar_shaft}
\alias{format.pillar_shaft}
\alias{pillar_shaft.logical}
\alias{pillar_shaft.numeric}
\alias{pillar_shaft.Date}
\alias{pillar_shaft.POSIXt}
\alias{pillar_shaft.character}
\alias{pillar_shaft.glue}
\alias{pillar_shaft.list}
\alias{pillar_shaft.factor}
\alias{pillar_shaft.AsIs}
\alias{pillar_shaft.default}
\title{Column data}
\usage{
pillar_shaft(x, ...)

\method{print}{pillar_shaft}(x, width = NULL, ...)

\method{format}{pillar_shaft}(x, width, ...)

\method{pillar_shaft}{logical}(x, ...)

\method{pillar_shaft}{numeric}(x, ..., sigfig = NULL)

\method{pillar_shaft}{Date}(x, ...)

\method{pillar_shaft}{POSIXt}(x, ...)

\method{pillar_shaft}{character}(x, ..., min_width = NULL)

\method{pillar_shaft}{glue}(x, ..., min_width = NULL, na_indent = 0L, shorten = NULL)

\method{pillar_shaft}{list}(x, ...)

\method{pillar_shaft}{factor}(x, ...)

\method{pillar_shaft}{AsIs}(x, ...)

\method{pillar_shaft}{default}(x, ...)
}
\arguments{
\item{x}{A vector to format}

\item{...}{Arguments passed to methods.}

\item{width}{Width for printing and formatting.}

\item{sigfig}{Deprecated, use \code{\link[=num]{num()}} or \code{\link[=set_num_opts]{set_num_opts()}} on the data instead.}

\item{min_width}{Deprecated, use \code{\link[=char]{char()}} or \code{\link[=set_char_opts]{set_char_opts()}} on the data instead.}

\item{na_indent}{Indentation of \code{NA} values.}

\item{shorten}{How to abbreviate the data if necessary:
\itemize{
\item \code{"back"} (default): add an ellipsis at the end
\item \code{"front"}: add an ellipsis at the front
\item \code{"mid"}: add an ellipsis in the middle
\item \code{"abbreviate"}: use \code{\link[=abbreviate]{abbreviate()}}
}}
}
\description{
Internal class for formatting the data for a column.
\code{pillar_shaft()} is a coercion method that must be implemented
for your data type to display it in a tibble.

This class comes with a default method for \code{\link[=print]{print()}} that calls \code{\link[=format]{format()}}.
If \code{print()} is called without \code{width} argument, the natural width will be
used when calling \code{format()}.
Usually there's no need to implement this method for your subclass.

Your subclass must implement \code{format()}, the default implementation just
raises an error.
Your \code{format()} method can assume a valid value for the \code{width} argument.
}
\details{
The default method will currently format via \code{\link[=format]{format()}},
but you should not rely on this behavior.
}
\examples{
pillar_shaft(1:3)
pillar_shaft(1.5:3.5)
pillar_shaft(NA)
pillar_shaft(c(1:3, NA))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tbl-format-setup.R
\name{new_tbl_format_setup}
\alias{new_tbl_format_setup}
\title{Construct a setup object for formatting}
\usage{
new_tbl_format_setup(
  width,
  tbl_sum,
  x = NULL,
  df = NULL,
  body = NULL,
  rows_missing = NULL,
  rows_total = NULL,
  extra_cols = NULL,
  extra_cols_total = NULL,
  max_footer_lines = NULL,
  abbrev_cols = NULL
)
}
\arguments{
\item{width}{The \code{width} argument unchanged.}

\item{tbl_sum}{A named character vector, as returned from \code{\link[=tbl_sum]{tbl_sum()}}.}

\item{x}{The input object unchanged.}

\item{df}{A data frame representation of the intended output,
trimmed to the desired number of rows.}

\item{body}{A character vector with the formatted body,
one element per line,}

\item{rows_missing}{The number of rows not shown from the body,
\code{NA} if unknown.}

\item{rows_total}{The total number of rows in the data,
\code{NA} if unknown.}

\item{extra_cols}{Columns that did not fit into the body,
as a character vector of formatted column names and types.}

\item{extra_cols_total}{The total number of columns, may be larger than
\code{length(extra_cols)}.}

\item{max_footer_lines}{The maximum number of lines in the footer.}

\item{abbrev_cols}{Formatted names of the columns that are shown abbreviated
in the body.}
}
\description{
The object returned from the default method of \code{\link[=tbl_format_setup]{tbl_format_setup()}}
is an object with a \code{"class"} attribute and the elements described in the
"Parameters" section.

Named elements can be added to such objects without affecting the behavior.
Do not modify existing elements.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dim.R
\name{dim_desc}
\alias{dim_desc}
\title{Format dimensions}
\usage{
dim_desc(x)
}
\arguments{
\item{x}{The object to format the dimensions for}
}
\description{
Multi-dimensional objects are formatted as \verb{a x b x ...}, for vectors the
length is returned.
}
\examples{
dim_desc(1:10)
dim_desc(Titanic)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/title.R
\name{new_pillar_title}
\alias{new_pillar_title}
\title{Prepare a column title for formatting}
\usage{
new_pillar_title(x, ...)
}
\arguments{
\item{x}{A character vector of column titles.}

\item{...}{These dots are for future extensions and must be empty.}
}
\description{
Call \code{\link[=format]{format()}} on the result to render column titles.
}
\examples{
format(new_pillar_title(names(trees)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ctl_new_pillar_list.R
\name{ctl_new_pillar_list}
\alias{ctl_new_pillar_list}
\title{Customize the appearance of compound pillars in your tibble subclass}
\usage{
ctl_new_pillar_list(
  controller,
  x,
  width,
  ...,
  title = NULL,
  first_pillar = NULL
)
}
\arguments{
\item{controller}{The object of class \code{"tbl"} currently printed.}

\item{x}{A vector, can also be a data frame, matrix, or array.}

\item{width}{The available width, can be a vector for multiple tiers.
If \code{NULL}, only the first pillar is instantiated.}

\item{...}{These dots are for future extensions and must be empty.}

\item{title}{The title, derived from the name of the column in the data.}

\item{first_pillar}{Can be passed to this method if the first pillar
for a compound pillar (or the pillar itself for a simple pillar)
has been constructed already.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Gain full control over the appearance of the pillars of your tibble subclass
in its body.
This method is intended for implementers of subclasses of the \code{"tbl"} class.
Users will rarely need them, and we also expect the default implementation
to be sufficient for the vast majority of cases.
}
\details{
\code{ctl_new_pillar_list()} is called to construct a list of pillars.
If \code{x} is a regular (one-dimensional) vector, the list contains one pillar
constructed by \code{\link[=ctl_new_pillar]{ctl_new_pillar()}}.
This method also works for compound columns: columns that are data frames,
matrices or arrays, with the following behavior:
\itemize{
\item If \code{width} is \code{NULL}, the method always returns a list of length one
containing one pillar object that represents the first sub-column in this
compound column.
\item Otherwise, the returned list contains one pillar object for all sub-columns
that can be fit in the available horizontal space.
These pillar objects are obtained by calling \code{ctl_new_pillar_list()}
with \code{width = NULL} on each sub-column until the available width is
exhausted.
}

This method is called to initiate the construction of all pillars
in the tibble to be printed.
To ensure that all packed columns that fit the available space are printed,
\code{ctl_new_pillar_list()} may be called twice on the same input:
once with \code{width = NULL}, and
once with \code{width} corresponding to the then known available space
and with \code{first_pillar} set to the pillar object constructed in the
first call.
}
\examples{
\dontshow{if (rlang::is_installed("palmerpenguins") && requireNamespace("tibble")) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
# Simple column
ctl_new_pillar_list(
  tibble::tibble(),
  palmerpenguins::penguins$weight[1:3],
  width = 10
)

# Packed data frame: unknown width
ctl_new_pillar_list(
  tibble::tibble(),
  palmerpenguins::penguins[1:3, ],
  width = NULL
)

# Packed data frame: known width
ctl_new_pillar_list(
  tibble::tibble(),
  palmerpenguins::penguins,
  width = 60
)

# Deeply packed data frame with known width:
# showing only the first sub-column even if the width is sufficient
ctl_new_pillar_list(
  tibble::tibble(),
  tibble::tibble(x = tibble::tibble(b = 1, c = 2), y = 3),
  width = 60
)

# Packed matrix: unknown width
ctl_new_pillar_list(tibble::tibble(), matrix(1:6, ncol = 2), width = NULL)

# Packed matrix: known width
ctl_new_pillar_list(tibble::tibble(), matrix(1:6, ncol = 2), width = 60)

# Packed array
ctl_new_pillar_list(tibble::tibble(), Titanic, width = 60)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ctl_new_pillar_list.R
\name{ctl_new_pillar_list}
\alias{ctl_new_pillar_list}
\title{Customize the appearance of compound pillars in your tibble subclass}
\usage{
ctl_new_pillar_list(
  controller,
  x,
  width,
  ...,
  title = NULL,
  first_pillar = NULL
)
}
\arguments{
\item{controller}{The object of class \code{"tbl"} currently printed.}

\item{x}{A vector, can also be a data frame, matrix, or array.}

\item{width}{The available width, can be a vector for multiple tiers.
If \code{NULL}, only the first pillar is instantiated.}

\item{...}{These dots are for future extensions and must be empty.}

\item{title}{The title, derived from the name of the column in the data.}

\item{first_pillar}{Can be passed to this method if the first pillar
for a compound pillar (or the pillar itself for a simple pillar)
has been constructed already.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Gain full control over the appearance of the pillars of your tibble subclass
in its body.
This method is intended for implementers of subclasses of the \code{"tbl"} class.
Users will rarely need them, and we also expect the default implementation
to be sufficient for the vast majority of cases.
}
\details{
\code{ctl_new_pillar_list()} is called to construct a list of pillars.
If \code{x} is a regular (one-dimensional) vector, the list contains one pillar
constructed by \code{\link[=ctl_new_pillar]{ctl_new_pillar()}}.
This method also works for compound columns: columns that are data frames,
matrices or arrays, with the following behavior:
\itemize{
\item If \code{width} is \code{NULL}, the method always returns a list of length one
containing one pillar object that represents the first sub-column in this
compound column.
\item Otherwise, the returned list contains one pillar object for all sub-columns
that can be fit in the available horizontal space.
These pillar objects are obtained by calling \code{ctl_new_pillar_list()}
with \code{width = NULL} on each sub-column until the available width is
exhausted.
}

This method is called to initiate the construction of all pillars
in the tibble to be printed.
To ensure that all packed columns that fit the available space are printed,
\code{ctl_new_pillar_list()} may be called twice on the same input:
once with \code{width = NULL}, and
once with \code{width} corresponding to the then known available space
and with \code{first_pillar} set to the pillar object constructed in the
first call.
}
\examples{
\dontshow{if (rlang::is_installed(c("palmerpenguins", "tibble")) && requireNamespace("tibble")) withAutoprint(\{ # examplesIf}
# Simple column
ctl_new_pillar_list(
  tibble::tibble(),
  palmerpenguins::penguins$weight[1:3],
  width = 10
)

# Packed data frame: unknown width
ctl_new_pillar_list(
  tibble::tibble(),
  palmerpenguins::penguins[1:3, ],
  width = NULL
)

# Packed data frame: known width
ctl_new_pillar_list(
  tibble::tibble(),
  palmerpenguins::penguins,
  width = 60
)

# Deeply packed data frame with known width:
# showing only the first sub-column even if the width is sufficient
ctl_new_pillar_list(
  tibble::tibble(),
  tibble::tibble(x = tibble::tibble(b = 1, c = 2), y = 3),
  width = 60
)

# Packed matrix: unknown width
ctl_new_pillar_list(tibble::tibble(), matrix(1:6, ncol = 2), width = NULL)

# Packed matrix: known width
ctl_new_pillar_list(tibble::tibble(), matrix(1:6, ncol = 2), width = 60)

# Packed array
ctl_new_pillar_list(tibble::tibble(), Titanic, width = 60)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extent.R
\name{get_extent}
\alias{get_extent}
\alias{get_max_extent}
\title{Calculate display width}
\usage{
get_extent(x)

get_max_extent(x)
}
\arguments{
\item{x}{A character vector.}
}
\description{
\code{get_extent()} calculates the display width for each string in a character
vector.

\code{get_max_extent()} calculates the maximum display width of all strings in a
character vector, zero for empty vectors.
}
\examples{
get_extent(c("abc", "de"))
get_extent("\u904b\u6c23")
get_max_extent(c("abc", "de"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggplot2.R
\name{scale_x_num}
\alias{scale_x_num}
\alias{scale_y_num}
\title{Scale that supports formatted numbers}
\usage{
scale_x_num(
  ...,
  position = "bottom",
  guide = ggplot2::waiver(),
  rescaler = NULL,
  super = NULL
)

scale_y_num(..., guide = ggplot2::waiver(), rescaler = NULL, super = NULL)
}
\arguments{
\item{...}{
  Arguments passed on to \code{\link[ggplot2:continuous_scale]{ggplot2::continuous_scale}}
  \describe{
    \item{\code{aesthetics}}{The names of the aesthetics that this scale works with.}
    \item{\code{scale_name}}{The name of the scale that should be used for error messages
associated with this scale.}
    \item{\code{palette}}{A palette function that when called with a numeric vector with
values between 0 and 1 returns the corresponding output values
(e.g., \code{\link[scales:area_pal]{scales::area_pal()}}).}
    \item{\code{name}}{The name of the scale. Used as the axis or legend title. If
\code{waiver()}, the default, the name of the scale is taken from the first
mapping used for that aesthetic. If \code{NULL}, the legend title will be
omitted.}
    \item{\code{breaks}}{One of:
\itemize{
\item \code{NULL} for no breaks
\item \code{waiver()} for the default breaks computed by the
\link[scales:trans_new]{transformation object}
\item A numeric vector of positions
\item A function that takes the limits as input and returns breaks
as output (e.g., a function returned by \code{\link[scales:breaks_extended]{scales::extended_breaks()}}).
Also accepts rlang \link[rlang:as_function]{lambda} function notation.
}}
    \item{\code{minor_breaks}}{One of:
\itemize{
\item \code{NULL} for no minor breaks
\item \code{waiver()} for the default breaks (one minor break between
each major break)
\item A numeric vector of positions
\item A function that given the limits returns a vector of minor breaks. Also
accepts rlang \link[rlang:as_function]{lambda} function notation.
}}
    \item{\code{n.breaks}}{An integer guiding the number of major breaks. The algorithm
may choose a slightly different number to ensure nice break labels. Will
only have an effect if \code{breaks = waiver()}. Use \code{NULL} to use the default
number of breaks given by the transformation.}
    \item{\code{labels}}{One of:
\itemize{
\item \code{NULL} for no labels
\item \code{waiver()} for the default labels computed by the
transformation object
\item A character vector giving labels (must be same length as \code{breaks})
\item A function that takes the breaks as input and returns labels
as output. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
}}
    \item{\code{limits}}{One of:
\itemize{
\item \code{NULL} to use the default scale range
\item A numeric vector of length two providing limits of the scale.
Use \code{NA} to refer to the existing minimum or maximum
\item A function that accepts the existing (automatic) limits and returns
new limits. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
Note that setting limits on positional scales will \strong{remove} data outside of the limits.
If the purpose is to zoom, use the limit argument in the coordinate system
(see \code{\link[ggplot2:coord_cartesian]{coord_cartesian()}}).
}}
    \item{\code{oob}}{One of:
\itemize{
\item Function that handles limits outside of the scale limits
(out of bounds). Also accepts rlang \link[rlang:as_function]{lambda}
function notation.
\item The default (\code{\link[scales:oob]{scales::censor()}}) replaces out of
bounds values with \code{NA}.
\item \code{\link[scales:oob]{scales::squish()}} for squishing out of bounds values into range.
\item \code{\link[scales:oob]{scales::squish_infinite()}} for squishing infinite values into range.
}}
    \item{\code{expand}}{For position scales, a vector of range expansion constants used to add some
padding around the data to ensure that they are placed some distance
away from the axes. Use the convenience function \code{\link[ggplot2:expansion]{expansion()}}
to generate the values for the \code{expand} argument. The defaults are to
expand the scale by 5\% on each side for continuous variables, and by
0.6 units on each side for discrete variables.}
    \item{\code{na.value}}{Missing values will be replaced with this value.}
    \item{\code{trans}}{For continuous scales, the name of a transformation object
or the object itself. Built-in transformations include "asn", "atanh",
"boxcox", "date", "exp", "hms", "identity", "log", "log10", "log1p", "log2",
"logit", "modulus", "probability", "probit", "pseudo_log", "reciprocal",
"reverse", "sqrt" and "time".

A transformation object bundles together a transform, its inverse,
and methods for generating breaks and labels. Transformation objects
are defined in the scales package, and are called \verb{<name>_trans} (e.g.,
\code{\link[scales:boxcox_trans]{scales::boxcox_trans()}}). You can create your own
transformation with \code{\link[scales:trans_new]{scales::trans_new()}}.}
  }}

\item{guide, position}{Passed on to \code{\link[ggplot2:continuous_scale]{ggplot2::continuous_scale()}}}

\item{rescaler, super}{Must remain \code{NULL}.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

This scale is used by default in ggplot2 with columns created with \code{\link[=num]{num()}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tbl-format.R
\name{format_tbl}
\alias{format_tbl}
\alias{print.tbl}
\alias{format.tbl}
\title{Formatting of tbl objects}
\usage{
\method{print}{tbl}(
  x,
  width = NULL,
  ...,
  n = NULL,
  max_extra_cols = NULL,
  max_footer_lines = NULL
)

\method{format}{tbl}(
  x,
  width = NULL,
  ...,
  n = NULL,
  max_extra_cols = NULL,
  max_footer_lines = NULL
)
}
\description{
See \link[tibble:formatting]{tibble::formatting} for details.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggplot2.R
\name{scale_x_num}
\alias{scale_x_num}
\alias{scale_y_num}
\title{Scale that supports formatted numbers}
\usage{
scale_x_num(
  ...,
  position = "bottom",
  guide = ggplot2::waiver(),
  rescaler = NULL,
  super = NULL
)

scale_y_num(..., guide = ggplot2::waiver(), rescaler = NULL, super = NULL)
}
\arguments{
\item{...}{
  Arguments passed on to \code{\link[ggplot2:continuous_scale]{ggplot2::continuous_scale}}
  \describe{
    \item{\code{aesthetics}}{The names of the aesthetics that this scale works with.}
    \item{\code{palette}}{A palette function that when called with a numeric vector with
values between 0 and 1 returns the corresponding output values
(e.g., \code{\link[scales:pal_area]{scales::pal_area()}}).}
    \item{\code{name}}{The name of the scale. Used as the axis or legend title. If
\code{waiver()}, the default, the name of the scale is taken from the first
mapping used for that aesthetic. If \code{NULL}, the legend title will be
omitted.}
    \item{\code{breaks}}{One of:
\itemize{
\item \code{NULL} for no breaks
\item \code{waiver()} for the default breaks computed by the
\link[scales:new_transform]{transformation object}
\item A numeric vector of positions
\item A function that takes the limits as input and returns breaks
as output (e.g., a function returned by \code{\link[scales:breaks_extended]{scales::extended_breaks()}}).
Note that for position scales, limits are provided after scale expansion.
Also accepts rlang \link[rlang:as_function]{lambda} function notation.
}}
    \item{\code{minor_breaks}}{One of:
\itemize{
\item \code{NULL} for no minor breaks
\item \code{waiver()} for the default breaks (one minor break between
each major break)
\item A numeric vector of positions
\item A function that given the limits returns a vector of minor breaks. Also
accepts rlang \link[rlang:as_function]{lambda} function notation. When
the function has two arguments, it will be given the limits and major
breaks.
}}
    \item{\code{n.breaks}}{An integer guiding the number of major breaks. The algorithm
may choose a slightly different number to ensure nice break labels. Will
only have an effect if \code{breaks = waiver()}. Use \code{NULL} to use the default
number of breaks given by the transformation.}
    \item{\code{labels}}{One of:
\itemize{
\item \code{NULL} for no labels
\item \code{waiver()} for the default labels computed by the
transformation object
\item A character vector giving labels (must be same length as \code{breaks})
\item An expression vector (must be the same length as breaks). See ?plotmath for details.
\item A function that takes the breaks as input and returns labels
as output. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
}}
    \item{\code{limits}}{One of:
\itemize{
\item \code{NULL} to use the default scale range
\item A numeric vector of length two providing limits of the scale.
Use \code{NA} to refer to the existing minimum or maximum
\item A function that accepts the existing (automatic) limits and returns
new limits. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
Note that setting limits on positional scales will \strong{remove} data outside of the limits.
If the purpose is to zoom, use the limit argument in the coordinate system
(see \code{\link[ggplot2:coord_cartesian]{coord_cartesian()}}).
}}
    \item{\code{oob}}{One of:
\itemize{
\item Function that handles limits outside of the scale limits
(out of bounds). Also accepts rlang \link[rlang:as_function]{lambda}
function notation.
\item The default (\code{\link[scales:oob]{scales::censor()}}) replaces out of
bounds values with \code{NA}.
\item \code{\link[scales:oob]{scales::squish()}} for squishing out of bounds values into range.
\item \code{\link[scales:oob]{scales::squish_infinite()}} for squishing infinite values into range.
}}
    \item{\code{expand}}{For position scales, a vector of range expansion constants used to add some
padding around the data to ensure that they are placed some distance
away from the axes. Use the convenience function \code{\link[ggplot2:expansion]{expansion()}}
to generate the values for the \code{expand} argument. The defaults are to
expand the scale by 5\% on each side for continuous variables, and by
0.6 units on each side for discrete variables.}
    \item{\code{na.value}}{Missing values will be replaced with this value.}
    \item{\code{transform}}{For continuous scales, the name of a transformation object
or the object itself. Built-in transformations include "asn", "atanh",
"boxcox", "date", "exp", "hms", "identity", "log", "log10", "log1p", "log2",
"logit", "modulus", "probability", "probit", "pseudo_log", "reciprocal",
"reverse", "sqrt" and "time".

A transformation object bundles together a transform, its inverse,
and methods for generating breaks and labels. Transformation objects
are defined in the scales package, and are called \verb{transform_<name>}. If
transformations require arguments, you can call them from the scales
package, e.g. \code{\link[scales:transform_boxcox]{scales::transform_boxcox(p = 2)}}.
You can create your own transformation with \code{\link[scales:new_transform]{scales::new_transform()}}.}
    \item{\code{trans}}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} Deprecated in favour of
\code{transform}.}
    \item{\code{call}}{The \code{call} used to construct the scale for reporting messages.}
  }}

\item{guide, position}{Passed on to \code{\link[ggplot2:continuous_scale]{ggplot2::continuous_scale()}}}

\item{rescaler, super}{Must remain \code{NULL}.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

This scale is used by default in ggplot2 with columns created with \code{\link[=num]{num()}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ctl_pillar.R
\name{pillar}
\alias{pillar}
\title{Object for formatting a vector suitable for tabular display}
\usage{
pillar(x, title = NULL, width = NULL, ...)
}
\arguments{
\item{x}{A vector to format.}

\item{title}{An optional title for the column. The title will be
used "as is", no quoting will be applied.}

\item{width}{Default width, optional.}

\item{...}{Passed on to \code{\link[=pillar_shaft]{pillar_shaft()}}.}
}
\description{
\code{pillar()} creates an object that formats a vector.
The output uses one row for a title (if given), one row for the type,
and \code{vec_size(x)} rows for the data.
}
\details{
A pillar consists of arbitrary components.
The \code{pillar()} constructor uses \code{title}, \code{type}, and \code{data}.
\itemize{
\item \code{title} via \code{\link[=new_pillar_title]{new_pillar_title()}}
\item \code{type} via \code{\link[=new_pillar_type]{new_pillar_type()}}, which calls \code{\link[=type_sum]{type_sum()}}
internally
\item \code{data} via \code{\link[=pillar_shaft]{pillar_shaft()}}
}

All components are formatted via \code{\link[=format]{format()}} when displaying the pillar.
A \code{width} argument is passed to each \code{format()} call.

As of pillar 1.5.0, \code{pillar()} returns \code{NULL} if the width is insufficient
to display the data.
}
\examples{
x <- 123456789 * (10^c(-1, -3, -5, NA, -8, -10))
pillar(x)
pillar(-x)
pillar(runif(10))
pillar(rcauchy(20))

# Special values are highlighted
pillar(c(runif(5), NA, NaN, Inf, -Inf))

# Very wide ranges will be displayed in scientific format
pillar(c(1e10, 1e-10), width = 20)
pillar(c(1e10, 1e-10))

x <- c(FALSE, NA, FALSE, FALSE, TRUE, FALSE, FALSE, TRUE, FALSE, TRUE)
pillar(x)

x <- c("This is string is rather long", NA, "?", "Short")
pillar(x)
pillar(x, width = 30)
pillar(x, width = 5)

date <- as.Date("2017-05-15")
pillar(date + c(1, NA, 3:5))
pillar(as.POSIXct(date) + c(30, NA, 600, 3600, 86400))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type.R
\name{new_pillar_type}
\alias{new_pillar_type}
\title{Prepare a column type for formatting}
\usage{
new_pillar_type(x, ...)
}
\arguments{
\item{x}{A vector for which the type is to be retrieved.}

\item{...}{These dots are for future extensions and must be empty.}
}
\description{
Calls \code{\link[=type_sum]{type_sum()}} to format the type.
Call \code{\link[=format]{format()}} on the result to render column types.
}
\examples{
format(new_pillar_type("a"))
format(new_pillar_type(factor("a")))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R, R/testthat.R
\name{deprecated}
\alias{deprecated}
\alias{is_vector_s3}
\alias{expect_known_display}
\title{Deprecated functions}
\usage{
is_vector_s3(x)

expect_known_display(object, file, ..., width = 80L, crayon = TRUE)
}
\arguments{
\item{object}{An object to check.}

\item{file}{File path where known value/output will be stored.}

\item{...}{Unused.}

\item{width}{The width of the output.}

\item{crayon}{Color the output?}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

Use \code{\link[vctrs:vec_assert]{vctrs::vec_is()}} instead of \code{is_vector_s3()}.

Use \code{\link[testthat:expect_snapshot]{testthat::expect_snapshot()}} instead of \code{expect_known_display()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\docType{data}
\name{pillar_options}
\alias{pillar_options}
\title{Package options}
\usage{
pillar_options
}
\description{
Options that affect display of tibble-like output.
}
\details{
These options can be set via \code{\link[=options]{options()}} and queried via \code{\link[=getOption]{getOption()}}.
For this, add a \code{pillar.} prefix (the package name and a dot) to the option name.
Example: for an option \code{foo}, use \code{options(pillar.foo = value)} to set it
and \code{getOption("pillar.foo")} to retrieve the current value.
An option value of \code{NULL} means that the default is used.
}
\section{Options for the pillar package}{

\itemize{
\item \code{print_max}: Maximum number of rows printed, default: \code{20}.
Set to \code{Inf} to always print all rows.
For compatibility reasons, \code{getOption("tibble.print_max")} and
\code{getOption("dplyr.print_max")} are also consulted,
this will be soft-deprecated in pillar v2.0.0.
\item \code{print_min}: Number of rows printed if the table has more than
\code{print_max} rows, default: \code{10}.
For compatibility reasons, \code{getOption("tibble.print_min")} and
\code{getOption("dplyr.print_min")} are also consulted,
this will be soft-deprecated in pillar v2.0.0.
\item \code{width}: Output width. Default: \code{NULL}
(use \code{getOption("width")}).
This can be larger than \code{getOption("width")}, in this case the output
of the table's body is distributed over multiple tiers for wide tibbles.
For compatibility reasons, \code{getOption("tibble.width")} and
\code{getOption("dplyr.width")} are also consulted,
this will be soft-deprecated in pillar v2.0.0.
\item \code{max_footer_lines}: The maximum number of lines in the footer,
default: \code{7}. Set to \code{Inf} to turn off truncation of footer lines.
The \code{max_extra_cols} option still limits
the number of columns printed.
\item \code{max_extra_cols}: The maximum number of columns printed in the footer,
default: \code{100}. Set to \code{Inf} to show all columns.
Set the more predictable \code{max_footer_lines} to control the number
of footer lines instead.
\item \code{bold}: Use bold font, e.g. for column headers? This currently
defaults to \code{FALSE}, because many terminal fonts have poor support for
bold fonts.
\item \code{subtle}: Use subtle style, e.g. for row numbers and data types?
Default: \code{TRUE}.
\item \code{subtle_num}: Use subtle style for insignificant digits? Default:
\code{FALSE}, is also affected by the \code{subtle} option.
\item \code{neg}: Highlight negative numbers? Default: \code{TRUE}.
\item \code{sigfig}: The number of significant digits that will be printed and
highlighted, default: \code{3}. Set the \code{subtle} option to \code{FALSE} to
turn off highlighting of significant digits.
\item \code{min_title_chars}: The minimum number of characters for the column
title, default: \code{3}.  Column titles may be truncated up to that width to
save horizontal space. Set to \code{Inf} to turn off truncation of column
titles.
\item \code{min_chars}: The minimum number of characters wide to
display character columns, default: \code{5}.  Character columns may be
truncated up to that width to save horizontal space. Set to \code{Inf} to
turn off truncation of character columns.
\item \code{max_dec_width}: The maximum allowed width for decimal notation,
default: \code{13}.
\item \code{bidi}: Set to \code{TRUE} for experimental support for bidirectional scripts.
Default: \code{FALSE}. When this option is set, "left right override"
and "first strong isolate"
\href{https://www.w3.org/International/questions/qa-bidi-unicode-controls}{Unicode controls}
are inserted to ensure that text appears in its intended direction
and that the column headings correspond to the correct columns.
\item \code{superdigit_sep}: The string inserted between superscript digits
and column names in the footnote. Defaults to a \code{"\\u200b"}, a zero-width
space, on UTF-8 platforms, and to \code{": "} on non-UTF-8 platforms.
}
}

\examples{
# Default setting:
getOption("pillar.sigfig")
pillar(1.234567)

# Change for the duration of the session:
old <- options(pillar.sigfig = 6)
pillar(1.234567)

# Change back to the original value:
options(old)
pillar(1.234567)

# Local scope:
local({
  rlang::local_options(pillar.sigfig = 6)
  pillar(1.234567)
})
pillar(1.234567)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\docType{data}
\name{pillar_options}
\alias{pillar_options}
\title{Package options}
\description{
Options that affect display of tibble-like output.
}
\details{
These options can be set via \code{\link[=options]{options()}} and queried via \code{\link[=getOption]{getOption()}}.
}
\section{Options for the pillar package}{

\itemize{
\item \code{width}: The width option controls the output width.
Setting \code{options(pillar.width = )} to a larger value
will lead to printing in multiple tiers (stacks).
\item \code{pillar.print_max}: Maximum number of rows printed, default: \code{20}.
Set to \code{Inf} to always print all rows.
For compatibility reasons, \code{getOption("tibble.print_max")} and
\code{getOption("dplyr.print_max")} are also consulted,
this will be soft-deprecated in pillar v2.0.0.
\item \code{pillar.print_min}: Number of rows printed if the table has more than
\code{print_max} rows, default: \code{10}.
For compatibility reasons, \code{getOption("tibble.print_min")} and
\code{getOption("dplyr.print_min")} are also consulted,
this will be soft-deprecated in pillar v2.0.0.
\item \code{pillar.width}: Output width. Default: \code{NULL}
(use \code{getOption("width")}).
This can be larger than \code{getOption("width")}, in this case the output
of the table's body is distributed over multiple tiers for wide tibbles.
For compatibility reasons, \code{getOption("tibble.width")} and
\code{getOption("dplyr.width")} are also consulted,
this will be soft-deprecated in pillar v2.0.0.
\item \code{pillar.max_footer_lines}: The maximum number of lines in the footer,
default: \code{7}. Set to \code{Inf} to turn off truncation of footer lines.
The \code{max_extra_cols} option still limits
the number of columns printed.
\item \code{pillar.max_extra_cols}: The maximum number of columns printed in the footer,
default: \code{100}. Set to \code{Inf} to show all columns.
Set the more predictable \code{max_footer_lines} to control the number
of footer lines instead.
\item \code{pillar.bold}: Use bold font, e.g. for column headers? This currently
defaults to \code{FALSE}, because many terminal fonts have poor support for
bold fonts.
\item \code{pillar.subtle}: Use subtle style, e.g. for row numbers and data types?
Default: \code{TRUE}.
\item \code{pillar.subtle_num}: Use subtle style for insignificant digits? Default:
\code{FALSE}, is also affected by the \code{subtle} option.
\item \code{pillar.neg}: Highlight negative numbers? Default: \code{TRUE}.
\item \code{pillar.sigfig}: The number of significant digits that will be printed and
highlighted, default: \code{3}. Set the \code{subtle} option to \code{FALSE} to
turn off highlighting of significant digits.
\item \code{pillar.min_title_chars}: The minimum number of characters for the column
title, default: \code{20}.  Column titles may be truncated up to that width to
save horizontal space. Set to \code{Inf} to turn off truncation of column
titles.
\item \code{pillar.min_chars}: The minimum number of characters wide to
display character columns, default: \code{3}.  Character columns may be
truncated up to that width to save horizontal space. Set to \code{Inf} to
turn off truncation of character columns.
\item \code{pillar.max_dec_width}: The maximum allowed width for decimal notation,
default: \code{13}.
\item \code{pillar.bidi}: Set to \code{TRUE} for experimental support for bidirectional scripts.
Default: \code{FALSE}. When this option is set, "left right override"
and "first strong isolate"
\href{https://www.w3.org/International/questions/qa-bidi-unicode-controls}{Unicode controls}
are inserted to ensure that text appears in its intended direction
and that the column headings correspond to the correct columns.
\item \code{pillar.superdigit_sep}: The string inserted between superscript digits
and column names in the footnote. Defaults to a \code{"\\u200b"}, a zero-width
space, on UTF-8 platforms, and to \code{": "} on non-UTF-8 platforms.
\item \code{pillar.advice}: Should advice be displayed in the footer when columns or rows
are missing from the output? Defaults to \code{TRUE} for interactive sessions,
and to \code{FALSE} otherwise.
}
}

\examples{
\dontshow{if (rlang::is_installed("tibble")) withAutoprint(\{ # examplesIf}
df <- tibble::tibble(x = c(1.234567, NA, 5:10))
df

# Change for the duration of the session:
old <- options(
  pillar.sigfig = 6,
  pillar.print_max = 5,
  pillar.print_min = 5,
  pillar.advice = FALSE
)
df

# Change back to the original value:
options(old)
df
\dontshow{\}) # examplesIf}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tbl-sum.R
\name{tbl_sum}
\alias{tbl_sum}
\title{Provide a succinct summary of an object}
\usage{
tbl_sum(x)
}
\arguments{
\item{x}{Object to summarise.}
}
\value{
A named character vector, describing the dimensions in the first element
and the data source in the name of the first element.
}
\description{
\code{tbl_sum()} gives a brief textual description of a table-like object,
which should include the dimensions and the data source in the first element,
and additional information in the other elements (such as grouping for \pkg{dplyr}).
The default implementation forwards to \code{\link[=obj_sum]{obj_sum()}}.
}
\examples{
tbl_sum(1:10)
tbl_sum(matrix(1:10))
tbl_sum(data.frame(a = 1))
tbl_sum(Sys.Date())
tbl_sum(Sys.time())
tbl_sum(mean)

}
\seealso{
\code{\link[=type_sum]{type_sum()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ctl_new_pillar.R
\name{ctl_new_pillar}
\alias{ctl_new_pillar}
\alias{ctl_new_rowid_pillar}
\title{Customize the appearance of simple pillars in your tibble subclass}
\usage{
ctl_new_pillar(controller, x, width, ..., title = NULL)

ctl_new_rowid_pillar(controller, x, width, ..., title = NULL, type = NULL)
}
\arguments{
\item{controller}{The object of class \code{"tbl"} currently printed.}

\item{x}{A simple (one-dimensional) vector.}

\item{width}{The available width, can be a vector for multiple tiers.}

\item{...}{These dots are for future extensions and must be empty.}

\item{title}{The title, derived from the name of the column in the data.}

\item{type}{String for specifying a row ID type. Current values in use are
\code{NULL} and \code{"*"}.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Gain full control over the appearance of the pillars of your tibble subclass
in its body.
This method is intended for implementers of subclasses of the \code{"tbl"} class.
Users will rarely need them.
}
\details{
\code{ctl_new_pillar()} is called to construct pillars for regular (one-dimensional)
vectors.
The default implementation returns an object constructed with \code{\link[=pillar]{pillar()}}.
Extend this method to modify the pillar components returned from the default
implementation.
Override this method to completely change the appearance of the pillars.
Components are created with \code{\link[=new_pillar_component]{new_pillar_component()}} or \code{\link[=pillar_component]{pillar_component()}}.
In order to customize printing of row IDs, a method can be supplied for the
\code{ctl_new_rowid_pillar()} generic.

All components must be of the same height.
This restriction may be levied in the future.

Implementations should return \code{NULL} if none of the data
fits the available width.
}
\examples{
\dontshow{if (rlang::is_installed("palmerpenguins") && requireNamespace("tibble")) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
# Create pillar objects
ctl_new_pillar(
  palmerpenguins::penguins,
  palmerpenguins::penguins$species[1:3],
  width = 60
)

ctl_new_pillar(
  palmerpenguins::penguins,
  palmerpenguins::penguins$bill_length_mm[1:3],
  width = 60
)
\dontshow{\}) # examplesIf}

# Customize output
lines <- function(char = "-") {
  stopifnot(nchar(char) == 1)
  structure(char, class = "lines")
}

format.lines <- function(x, width, ...) {
  paste(rep(x, width), collapse = "")
}

ctl_new_pillar.line_tbl <- function(controller, x, width, ...) {
  out <- NextMethod()
  new_pillar(list(
    title = out$title,
    type = out$type,
    lines = new_pillar_component(list(lines("=")), width = 1),
    data = out$data
  ))
}

ctl_new_rowid_pillar.line_tbl <- function(controller, x, width, ...) {
  out <- NextMethod()
  new_pillar(
    list(
      title = out$title,
      type = out$type,
      lines = new_pillar_component(list(lines("=")), width = 1),
      data = out$data
    ),
    width = as.integer(floor(log10(max(nrow(x), 1))) + 1)
  )
}

vctrs::new_data_frame(
  list(a = 1:3, b = letters[1:3]),
  class = c("line_tbl", "tbl")
)

ctl_new_rowid_pillar.roman_tbl <- function(controller, x, width, ...) {
  out <- NextMethod()
  rowid <- utils::as.roman(seq_len(nrow(x)))
  width <- max(nchar(as.character(rowid)))
  new_pillar(
    list(
      title = out$title,
      type = out$type,
      data = pillar_component(
        new_pillar_shaft(list(row_ids = rowid),
          width = width,
          class = "pillar_rif_shaft"
        )
      )
    ),
    width = width
  )
}

vctrs::new_data_frame(
  list(a = 1:3, b = letters[1:3]),
  class = c("roman_tbl", "tbl")
)

}
\seealso{
See \code{\link[=ctl_new_pillar_list]{ctl_new_pillar_list()}} for creating pillar objects for compound columns:
packed data frames, matrices, or arrays.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ctl_new_pillar.R
\name{ctl_new_pillar}
\alias{ctl_new_pillar}
\alias{ctl_new_rowid_pillar}
\title{Customize the appearance of simple pillars in your tibble subclass}
\usage{
ctl_new_pillar(controller, x, width, ..., title = NULL)

ctl_new_rowid_pillar(controller, x, width, ..., title = NULL, type = NULL)
}
\arguments{
\item{controller}{The object of class \code{"tbl"} currently printed.}

\item{x}{A simple (one-dimensional) vector.}

\item{width}{The available width, can be a vector for multiple tiers.}

\item{...}{These dots are for future extensions and must be empty.}

\item{title}{The title, derived from the name of the column in the data.}

\item{type}{String for specifying a row ID type. Current values in use are
\code{NULL} and \code{"*"}.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Gain full control over the appearance of the pillars of your tibble subclass
in its body.
This method is intended for implementers of subclasses of the \code{"tbl"} class.
Users will rarely need them.
}
\details{
\code{ctl_new_pillar()} is called to construct pillars for regular (one-dimensional)
vectors.
The default implementation returns an object constructed with \code{\link[=pillar]{pillar()}}.
Extend this method to modify the pillar components returned from the default
implementation.
Override this method to completely change the appearance of the pillars.
Components are created with \code{\link[=new_pillar_component]{new_pillar_component()}} or \code{\link[=pillar_component]{pillar_component()}}.
In order to customize printing of row IDs, a method can be supplied for the
\code{ctl_new_rowid_pillar()} generic.

All components must be of the same height.
This restriction may be levied in the future.

Implementations should return \code{NULL} if none of the data
fits the available width.
}
\examples{
\dontshow{if (rlang::is_installed(c("palmerpenguins", "tibble"))) withAutoprint(\{ # examplesIf}
# Create pillar objects
ctl_new_pillar(
  palmerpenguins::penguins,
  palmerpenguins::penguins$species[1:3],
  width = 60
)

ctl_new_pillar(
  palmerpenguins::penguins,
  palmerpenguins::penguins$bill_length_mm[1:3],
  width = 60
)
\dontshow{\}) # examplesIf}

# Customize output
lines <- function(char = "-") {
  stopifnot(nchar(char) == 1)
  structure(char, class = "lines")
}

format.lines <- function(x, width, ...) {
  paste(rep(x, width), collapse = "")
}

ctl_new_pillar.line_tbl <- function(controller, x, width, ...) {
  out <- NextMethod()
  new_pillar(list(
    title = out$title,
    type = out$type,
    lines = new_pillar_component(list(lines("=")), width = 1),
    data = out$data
  ))
}

ctl_new_rowid_pillar.line_tbl <- function(controller, x, width, ...) {
  out <- NextMethod()
  new_pillar(
    list(
      title = out$title,
      type = out$type,
      lines = new_pillar_component(list(lines("=")), width = 1),
      data = out$data
    ),
    width = as.integer(floor(log10(max(nrow(x), 1))) + 1)
  )
}

vctrs::new_data_frame(
  list(a = 1:3, b = letters[1:3]),
  class = c("line_tbl", "tbl")
)

ctl_new_rowid_pillar.roman_tbl <- function(controller, x, width, ...) {
  out <- NextMethod()
  rowid <- utils::as.roman(seq_len(nrow(x)))
  width <- max(nchar(as.character(rowid)))
  new_pillar(
    list(
      title = out$title,
      type = out$type,
      data = pillar_component(
        new_pillar_shaft(list(row_ids = rowid),
          width = width,
          class = "pillar_rif_shaft"
        )
      )
    ),
    width = width
  )
}

vctrs::new_data_frame(
  list(a = 1:3, b = letters[1:3]),
  class = c("roman_tbl", "tbl")
)

}
\seealso{
See \code{\link[=ctl_new_pillar_list]{ctl_new_pillar_list()}} for creating pillar objects for compound columns:
packed data frames, matrices, or arrays.
}
